@@ -1,4 +1,4 @@
-/* $Id: cpl_bivector.c,v 1.36 2010/12/15 14:50:11 llundin Exp $
+/* $Id: cpl_bivector.c,v 1.42 2012/02/07 12:48:32 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/12/15 14:50:11 $
- * $Revision: 1.36 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2012/02/07 12:48:32 $
+ * $Revision: 1.42 $
+ * $Name: cpl-6_0 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -41,7 +41,7 @@
 #include "cpl_bivector.h"
 #include "cpl_tools.h"
 #include "cpl_error_impl.h"
-#include "cpl_type_impl.h"
+#include "cpl_type.h"
 
 /*-----------------------------------------------------------------------------
                                    Define
@@ -97,7 +97,7 @@ struct _cpl_bivector_ {
   - CPL_ERROR_ILLEGAL_INPUT if n is < 1.
  */
 /*----------------------------------------------------------------------------*/
-cpl_bivector * cpl_bivector_new(int n)
+cpl_bivector * cpl_bivector_new(cpl_size n)
 {
     cpl_bivector * f;
 
@@ -277,7 +277,7 @@ void cpl_bivector_dump(const cpl_bivector * f, FILE * stream)
     fprintf(stream, "#--- Bi-vector ---\n");
     fprintf(stream, "#       X      Y  \n");
 
-    for (i=0 ; i < cpl_bivector_get_size(f) ; i++)
+    for (i = 0; i < cpl_bivector_get_size(f); i++)
         fprintf(stream, "%g\t%g\n", data_x[i], data_y[i]);
 
     fprintf(stream, "#-----------------------\n");
@@ -347,7 +347,8 @@ cpl_bivector * cpl_bivector_read(const char * filename)
         fclose(in);
         cpl_vector_delete(v1);
         cpl_vector_delete(v2);
-        cpl_ensure(0, CPL_ERROR_FILE_IO, NULL);
+        (void)cpl_error_set_(CPL_ERROR_FILE_IO);
+        return NULL;
     }
 
     fclose(in);
@@ -356,7 +357,8 @@ cpl_bivector * cpl_bivector_read(const char * filename)
     if (np == 0 || cpl_vector_set_size(v1, np) || cpl_vector_set_size(v2, np)) {
         cpl_vector_delete(v1);
         cpl_vector_delete(v2);
-        cpl_ensure(0, CPL_ERROR_BAD_FILE_FORMAT, NULL);
+        (void)cpl_error_set_(CPL_ERROR_BAD_FILE_FORMAT);
+        return NULL;
     }
 
     /* Cannot fail at this point */
@@ -376,7 +378,7 @@ cpl_bivector * cpl_bivector_read(const char * filename)
     different sizes
  */
 /*----------------------------------------------------------------------------*/
-int cpl_bivector_get_size(const cpl_bivector * in)
+cpl_size cpl_bivector_get_size(const cpl_bivector * in)
 {
 
     cpl_size     size;
@@ -598,7 +600,8 @@ cpl_error_code cpl_bivector_interpolate_linear(cpl_bivector * fout,
     double       * xout = cpl_bivector_get_x_data(fout);
     double       * yout = cpl_bivector_get_y_data(fout);
 
-    double grad, y_0;
+    double grad = DBL_MAX; /* Avoid (false) uninit warning */
+    double y_0  = DBL_MAX; /* Avoid (false) uninit warning */
     cpl_size ibelow, iabove;
     cpl_size i;
 
@@ -653,8 +656,7 @@ cpl_error_code cpl_bivector_interpolate_linear(cpl_bivector * fout,
         }
     }
 
-    return i == n ? CPL_ERROR_NONE
-        : cpl_error_set(cpl_func, CPL_ERROR_ILLEGAL_INPUT);
+    return i == n ? CPL_ERROR_NONE : cpl_error_set_(CPL_ERROR_ILLEGAL_INPUT);
 }
 
 /*----------------------------------------------------------------------------*/
@@ -690,11 +692,11 @@ cpl_error_code cpl_bivector_sort(cpl_bivector * self,
                                  cpl_sort_mode mode)
 {
 
-    const int      n = cpl_bivector_get_size(self);
-    int          * perm;
+    const cpl_size n = cpl_bivector_get_size(self);
+    cpl_size     * perm;
     const double * data;
     int            reverse;
-    int            i;
+    cpl_size       i;
     cpl_error_code error = CPL_ERROR_NONE;
 
 
@@ -730,7 +732,7 @@ cpl_error_code cpl_bivector_sort(cpl_bivector * self,
         return cpl_error_set_(CPL_ERROR_UNSUPPORTED_MODE);
     }
 
-    perm = cpl_malloc(n * sizeof(*perm));
+    perm = (cpl_size*)cpl_malloc((size_t)n * sizeof(*perm));
 
     for (i = 0; i < n; i++) {
         perm[i] = i;
