@@ -1,4 +1,4 @@
-/* $Id: cpl_bivector.h,v 1.14 2010/12/14 17:25:08 llundin Exp $
+/* $Id: cpl_bivector.h,v 1.15 2011/02/28 15:45:20 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/12/14 17:25:08 $
- * $Revision: 1.14 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2011/02/28 15:45:20 $
+ * $Revision: 1.15 $
+ * $Name: cpl-6_0 $
  */
 
 #ifndef CPL_BIVECTOR_H
@@ -55,7 +55,7 @@ typedef enum {
  -----------------------------------------------------------------------------*/
 
 /* Constructors and destructors */
-cpl_bivector * cpl_bivector_new(int) CPL_ATTR_ALLOC;
+cpl_bivector * cpl_bivector_new(cpl_size) CPL_ATTR_ALLOC;
 cpl_bivector * cpl_bivector_wrap_vectors(cpl_vector *,
                                          cpl_vector *) CPL_ATTR_ALLOC;
 cpl_bivector * cpl_bivector_duplicate(const cpl_bivector *) CPL_ATTR_ALLOC;
@@ -66,7 +66,7 @@ cpl_bivector * cpl_bivector_read(const char *) CPL_ATTR_ALLOC;
 cpl_error_code cpl_bivector_copy(cpl_bivector *, const cpl_bivector *);
 
 /* Accessors functions */
-int cpl_bivector_get_size(const cpl_bivector *);
+cpl_size cpl_bivector_get_size(const cpl_bivector *);
       cpl_vector * cpl_bivector_get_x(cpl_bivector *);
       cpl_vector * cpl_bivector_get_y(cpl_bivector *);
 const cpl_vector * cpl_bivector_get_x_const(const cpl_bivector *);
