@@ -1,4 +1,4 @@
-/* $Id: cpl_errorstate.c,v 1.18 2010/11/11 09:23:18 llundin Exp $
+/* $Id: cpl_errorstate.c,v 1.19 2011/11/30 15:59:22 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/11/11 09:23:18 $
- * $Revision: 1.18 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2011/11/30 15:59:22 $
+ * $Revision: 1.19 $
+ * $Name: cpl-6_0 $
  */
 
 /*-----------------------------------------------------------------------------
@@ -153,6 +153,14 @@ static struct {
 #pragma omp threadprivate(error_history, estate)
 #endif
 
+/*-----------------------------------------------------------------------------
+                              Private Function Declarations
+ -----------------------------------------------------------------------------*/
+static void cpl_errorstate_dump_one_level(void (*)(const char *,
+                                                   const char *, ...)
+                                          CPL_ATTR_PRINTF(2, 3),
+                                          unsigned, unsigned, unsigned);
+
 /*-----------------------------------------------------------------------------
                                    Functions code
  -----------------------------------------------------------------------------*/
@@ -372,6 +380,8 @@ void cpl_errorstate_dump(cpl_errorstate self,
   This function will dump a single CPL error, using the CPL error accessor
   functions. The error is numbered with the value of self.
 
+  The actual output is produced by cpl_msg_error().
+
   first and last are provided in the API to allow for special messaging in the
   dump of the first and last errors.
 
@@ -425,42 +435,64 @@ void cpl_errorstate_dump(cpl_errorstate self,
 void cpl_errorstate_dump_one(unsigned self, unsigned first, unsigned last)
 {
 
-    const cpl_boolean is_reverse = first > last ? CPL_TRUE : CPL_FALSE;
-    const unsigned    newest     = is_reverse ? first : last;
-    const unsigned    oldest     = is_reverse ? last : first;
-    const char      * revmsg     = is_reverse ? " in reverse order" : "";
-
-
-    cx_assert( oldest <= self );
-    cx_assert( newest >= self );
-
-    if (newest == 0) {
-        cpl_msg_info(cpl_func, "No error(s) to dump");
-        cx_assert( oldest == 0);
-    } else {
-        cx_assert( oldest > 0);
-        cx_assert( newest >= oldest);
-        if (self == first) {
-            if (oldest == 1) {
-                cpl_msg_error(cpl_func, "Dumping all %u error(s)%s:", newest,
-                              revmsg);
-            } else {
-                cpl_msg_error(cpl_func, "Dumping the %u most recent error(s) "
-                              "out of a total of %u errors%s:",
-                              newest - oldest + 1, newest, revmsg);
-            }
-            cpl_msg_indent_more();
-        }
-
-        cpl_msg_error(cpl_func, "[%u/%u] '%s' (%u) at %s", self, newest,
-                      cpl_error_get_message(), cpl_error_get_code(),
-                      cpl_error_get_where());
-
-        if (self == last) cpl_msg_indent_less();
-    }
+    cpl_errorstate_dump_one_level(cpl_msg_error, self, first, last);
 }
 
 
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    Dump a single CPL error using cpl_msg_warning()
+  @param    self      The number of the current error to be dumped
+  @param    first     The number of the first error to be dumped
+  @param    last      The number of the last error to be dumped
+  @return   void
+  @see cpl_errorstate_dump_one, cpl_msg_warning()
+
+ */
+/*----------------------------------------------------------------------------*/
+void cpl_errorstate_dump_one_warning(unsigned self, unsigned first,
+                                     unsigned last)
+{
+
+    cpl_errorstate_dump_one_level(cpl_msg_warning, self, first, last);
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    Dump a single CPL error using cpl_msg_info()
+  @param    self      The number of the current error to be dumped
+  @param    first     The number of the first error to be dumped
+  @param    last      The number of the last error to be dumped
+  @return   void
+  @see cpl_errorstate_dump_one, cpl_msg_info()
+
+ */
+/*----------------------------------------------------------------------------*/
+void cpl_errorstate_dump_one_info(unsigned self, unsigned first, unsigned last)
+{
+
+    cpl_errorstate_dump_one_level(cpl_msg_info, self, first, last);
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    Dump a single CPL error using cpl_msg_debug()
+  @param    self      The number of the current error to be dumped
+  @param    first     The number of the first error to be dumped
+  @param    last      The number of the last error to be dumped
+  @return   void
+  @see cpl_errorstate_dump_one, cpl_msg_debug()
+
+ */
+/*----------------------------------------------------------------------------*/
+void cpl_errorstate_dump_one_debug(unsigned self, unsigned first, unsigned last)
+{
+
+    cpl_errorstate_dump_one_level(cpl_msg_debug, self, first, last);
+}
+
+/**@}*/
+
 /*----------------------------------------------------------------------------*/
 /**
   @internal
@@ -586,4 +618,60 @@ const cpl_error * cpl_errorstate_find(void)
 
 }
 
-/**@}*/
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    Dump a single CPL error
+  @param    Pointer to one of cpl_msg_info(), cpl_msg_warning(), ...
+  @param    self      The number of the current error to be dumped
+  @param    first     The number of the first error to be dumped
+  @param    last      The number of the last error to be dumped
+  @return   void
+  @see irplib_errorstate_dump_one
+
+ */
+/*----------------------------------------------------------------------------*/
+static
+void cpl_errorstate_dump_one_level(void (*messenger)(const char *,
+                                                        const char *, ...),
+                                      unsigned self, unsigned first,
+                                      unsigned last)
+{
+
+
+    const cpl_boolean is_reverse = first > last ? CPL_TRUE : CPL_FALSE;
+    const unsigned    newest     = is_reverse ? first : last;
+    const unsigned    oldest     = is_reverse ? last : first;
+    const char      * revmsg     = is_reverse ? " in reverse order" : "";
+
+
+    cx_assert( messenger != NULL );
+    cx_assert( oldest    <= self );
+    cx_assert( newest    >= self );
+
+    if (newest == 0) {
+        messenger(cpl_func, "No error(s) to dump");
+        cx_assert( oldest == 0);
+    } else {
+        cx_assert( oldest > 0);
+        cx_assert( newest >= oldest);
+        if (self == first) {
+            if (oldest == 1) {
+                messenger(cpl_func, "Dumping all %u error(s)%s:", newest,
+                          revmsg);
+            } else {
+                messenger(cpl_func, "Dumping the %u most recent error(s) "
+                              "out of a total of %u errors%s:",
+                              newest - oldest + 1, newest, revmsg);
+            }
+            cpl_msg_indent_more();
+        }
+
+        messenger(cpl_func, "[%u/%u] '%s' (%u) at %s", self, newest,
+                      cpl_error_get_message(), cpl_error_get_code(),
+                      cpl_error_get_where());
+
+        if (self == last) cpl_msg_indent_less();
+    }
+}
+
