@@ -1,4 +1,4 @@
-/* $Id: cpl_filter_median.c,v 1.11 2011/01/12 10:09:31 llundin Exp $
+/* $Id: cpl_filter_median.c,v 1.16 2011/09/13 09:36:31 cgarcia Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -19,10 +19,10 @@
  */
 
 /*
- * $Author: llundin $
- * $Date: 2011/01/12 10:09:31 $
- * $Revision: 1.11 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Author: cgarcia $
+ * $Date: 2011/09/13 09:36:31 $
+ * $Revision: 1.16 $
+ * $Name: cpl-6_0 $
  */
 
 #include "cpl_filter.h"
@@ -39,11 +39,6 @@
 */
 /**@{*/
 
-#ifndef GET_KTH
-#define GET_KTH get_kth
-#define CPL_FILTER_NEEDS_GET_KTH
-#endif
-
 #ifndef inline
 #define inline /* inline */
 #endif
@@ -81,15 +76,15 @@
 #endif
 
 /** @cond */
-typedef struct dheap {
+typedef struct ADDTYPE(dheap) {
     unsigned *heaps;
-    /* Double binary heap and median element. Values are pixel buffer indices in the
+    /* Double binary heap and ADDTYPE(median) element. Values are pixel buffer indices in the
        current filtering window.
        heaps[0] ... heaps[m-1]    : a maximum heap of pixels with values less than
-                                    median
-       heaps[m]                   : median
+                                    ADDTYPE(median)
+       heaps[m]                   : ADDTYPE(median)
        heaps[m+1] ... heaps[2m]   : a minimum heap of pixels with values greater 
-                                    than median
+                                    than ADDTYPE(median)
     */
     unsigned *B; /* Size Nx x Ny map from image positions to double heap positions.
                     This is the inverse map of the heaps[] array, 
@@ -117,7 +112,7 @@ typedef struct dheap {
     unsigned m;  //kernel window half size, must be positive
     unsigned Rx;  //2rx + 1
     unsigned Ry;  //2ry + 1
-} dheap;
+} ADDTYPE(dheap);
 /** @endcond */
 
 
@@ -148,7 +143,7 @@ do { \
   SCA is sorted in increasing order by image[SCA[i]]
  */
 static void
-qsort_int(unsigned *SCA, 
+ADDTYPE(qsort_int)(unsigned *SCA, 
           unsigned n, const PIX_TYPE *image)
 {
     int i, ir, j, k, l;
@@ -234,7 +229,7 @@ qsort_int(unsigned *SCA,
   @param  x      current column
 */
 static void
-SCA_init(dheap *dh, unsigned x)
+ADDTYPE(SCA_init)(ADDTYPE(dheap) *dh, unsigned x)
 {
     unsigned y;
     unsigned *SCAx = dh->SCA + (dh->Ry)*x;
@@ -243,7 +238,7 @@ SCA_init(dheap *dh, unsigned x)
         SCAx[y] = x + y*dh->Nx;
     }
 
-    qsort_int(SCAx,
+    ADDTYPE(qsort_int)(SCAx,
               dh->Ry,
               dh->image);
     
@@ -259,7 +254,7 @@ SCA_init(dheap *dh, unsigned x)
   @param  ynew   image row of new element
 */
 static void
-SCA_replace(dheap *dh, unsigned x, 
+ADDTYPE(SCA_replace)(ADDTYPE(dheap) *dh, unsigned x, 
             unsigned yold, unsigned ynew)
 {
     unsigned n = dh->Ry;
@@ -322,10 +317,10 @@ SCA_replace(dheap *dh, unsigned x,
   @param  heaps  heaps array
 */
 static void
-HEAP_SWAP(unsigned i, 
-          unsigned j,
-          unsigned int *B,
-          unsigned int *heaps)
+ADDTYPE(HEAP_SWAP)(unsigned i, 
+                   unsigned j,
+                   unsigned int *B,
+                   unsigned int *heaps)
 {
     unsigned ai = heaps[i];
     unsigned aj = heaps[j];
@@ -340,11 +335,11 @@ HEAP_SWAP(unsigned i,
 
 /**
   @internal
-  @brief  Partion double heaps array in median and upper/lower values
+  @brief  Partion double heaps array in ADDTYPE(median) and upper/lower values
   @param  dh     double heap with elements in no particular order
 */
 static void
-median(dheap *dh)
+ADDTYPE(median)(ADDTYPE(dheap) *dh)
 {
     unsigned n = 2 * dh->m + 1;
     
@@ -365,7 +360,7 @@ median(dheap *dh)
             while (dh->image[dh->heaps[i]] < pivot) i++;
             while (dh->image[dh->heaps[j]] > pivot) j--;
             if (i <= j) {
-                if (i != j) HEAP_SWAP(i, j, dh->B, dh->heaps);
+                if (i != j) ADDTYPE(HEAP_SWAP)(i, j, dh->B, dh->heaps);
                 i++;
                 j--;
             }
@@ -391,7 +386,7 @@ median(dheap *dh)
   @return final position
  */
 static unsigned
-bubble_down_gt(const PIX_TYPE *image,
+ADDTYPE(bubble_down_gt)(const PIX_TYPE *image,
                unsigned *B,
                unsigned *heaps,
                unsigned m,
@@ -411,7 +406,7 @@ bubble_down_gt(const PIX_TYPE *image,
         }
         if (image[heaps[m1 + root]] > 
             image[heaps[m1 + child]]) {
-            HEAP_SWAP(m1 + root, m1 + child, B, heaps);
+            ADDTYPE(HEAP_SWAP)(m1 + root, m1 + child, B, heaps);
             root = child;
         }
         else {
@@ -432,7 +427,7 @@ bubble_down_gt(const PIX_TYPE *image,
   @return final position
  */
 static unsigned
-bubble_up_gt(const PIX_TYPE *image,
+ADDTYPE(bubble_up_gt)(const PIX_TYPE *image,
              unsigned *B,
              unsigned *heaps, 
              unsigned m1,
@@ -445,7 +440,7 @@ bubble_up_gt(const PIX_TYPE *image,
 
         if (image[heaps[m1 + parent]] >
             image[heaps[m1 + child]]) {
-            HEAP_SWAP(m1 + parent, m1 + child, B, heaps);
+            ADDTYPE(HEAP_SWAP)(m1 + parent, m1 + child, B, heaps);
             child = parent;
         }
         else return child;
@@ -459,12 +454,12 @@ bubble_up_gt(const PIX_TYPE *image,
   @param  dh     double heap
  */
 static void
-heapify_gt(dheap *dh)
+ADDTYPE(heapify_gt)(ADDTYPE(dheap) *dh)
 {
     int start = dh->m/2-1;
 
     while (start >= 0) {
-        bubble_down_gt(dh->image,
+        ADDTYPE(bubble_down_gt)(dh->image,
                        dh->B,
                        dh->heaps,
                        dh->m,
@@ -485,7 +480,7 @@ heapify_gt(dheap *dh)
   @return final position
  */
 static unsigned
-bubble_down_lt(const PIX_TYPE *image,
+ADDTYPE(bubble_down_lt)(const PIX_TYPE *image,
                unsigned *B,
                unsigned *heaps,
                unsigned end, unsigned root)
@@ -502,7 +497,7 @@ bubble_down_lt(const PIX_TYPE *image,
         }
         if (image[heaps[root]] <
             image[heaps[child]]) {
-            HEAP_SWAP(root, child, B, heaps);
+            ADDTYPE(HEAP_SWAP)(root, child, B, heaps);
             root = child;
         }
         else {
@@ -522,7 +517,7 @@ bubble_down_lt(const PIX_TYPE *image,
   @return final position
  */
 static unsigned
-bubble_up_lt(const PIX_TYPE *image,
+ADDTYPE(bubble_up_lt)(const PIX_TYPE *image,
              unsigned *B,
              unsigned *heaps, unsigned child)
 {
@@ -533,7 +528,7 @@ bubble_up_lt(const PIX_TYPE *image,
 
         if (image[heaps[parent]] <
             image[heaps[child]]) {
-            HEAP_SWAP(parent, child, B, heaps);
+            ADDTYPE(HEAP_SWAP)(parent, child, B, heaps);
             child = parent;
         }
         else return child;
@@ -547,12 +542,12 @@ bubble_up_lt(const PIX_TYPE *image,
   @param  dh     double heap
  */
 static void
-heapify_lt(dheap *dh)
+ADDTYPE(heapify_lt)(ADDTYPE(dheap) *dh)
 {
     int start = dh->m/2-1;
 
     while (start >= 0) {
-        bubble_down_lt(dh->image,
+        ADDTYPE(bubble_down_lt)(dh->image,
                        dh->B,
                        dh->heaps,
                        dh->m-1, start);
@@ -567,12 +562,12 @@ heapify_lt(dheap *dh)
   @param  dh    double heap with elements in no particular order
  */
 static void
-dheap_establish(dheap *dh)
+ADDTYPE(dheap_establish)(ADDTYPE(dheap) *dh)
 {
-    median(dh);
+    ADDTYPE(median)(dh);
     
-    heapify_lt(dh);
-    heapify_gt(dh);
+    ADDTYPE(heapify_lt)(dh);
+    ADDTYPE(heapify_gt)(dh);
     
     return;
 }
@@ -585,15 +580,15 @@ dheap_establish(dheap *dh)
   @param  Ny          image height
   @param  rx          filtering half-size in x
   @param  ry          filtering half-size in y
-  @return newly allocated double heap which may be destroyed using dheap_delete() 
+  @return newly allocated double heap which may be destroyed using ADDTYPE(dheap_delete)() 
 
   The double heap, inverse map and SCA invariants are established
   using the first image window
 */
-static dheap *
-dheap_new(const PIX_TYPE *image, unsigned Nx, unsigned Ny, unsigned rx, unsigned ry)
+static ADDTYPE(dheap) *
+ADDTYPE(dheap_new)(const PIX_TYPE *image, unsigned Nx, unsigned Ny, unsigned rx, unsigned ry)
 {
-    dheap *dh = malloc(sizeof(*dh));
+    ADDTYPE(dheap) *dh = malloc(sizeof(*dh));
     assure( dh != NULL );
     assure( rx > 0 || ry > 0 );
 
@@ -626,13 +621,13 @@ dheap_new(const PIX_TYPE *image, unsigned Nx, unsigned Ny, unsigned rx, unsigned
             }
         }
     }
-    dheap_establish(dh);
+    ADDTYPE(dheap_establish)(dh);
 
     {
         unsigned x;
         for (x = 0; x < 2*dh->rx; x++)
         {
-            SCA_init(dh, x);
+            ADDTYPE(SCA_init)(dh, x);
         }
     }
 
@@ -645,7 +640,7 @@ dheap_new(const PIX_TYPE *image, unsigned Nx, unsigned Ny, unsigned rx, unsigned
   @param dh     double heap
  */
 static void
-dheap_delete(dheap *dh)
+ADDTYPE(dheap_delete)(ADDTYPE(dheap) *dh)
 {
     free(dh->SCA);
     free(dh->B);
@@ -657,12 +652,12 @@ dheap_delete(dheap *dh)
 
 /**
   @internal
-  @brief  get current median
+  @brief  get current ADDTYPE(median)
   @param  dh    double heap
-  @return the median element value
+  @return the ADDTYPE(median) element value
  */
 static PIX_TYPE
-dheap_median(const dheap *dh)
+ADDTYPE(dheap_median)(const ADDTYPE(dheap) *dh)
 {
     return dh->image[dh->heaps[dh->m]];
 }
@@ -696,7 +691,7 @@ dheap_median(const dheap *dh)
 
  */
 static void
-dheap_replace(const PIX_TYPE *image,
+ADDTYPE(dheap_replace)(const PIX_TYPE *image,
               unsigned *B,
               unsigned *heaps,
               mybool   *pprev_was_larger,
@@ -715,13 +710,13 @@ dheap_replace(const PIX_TYPE *image,
     B[anew] = ci;
 
     /* Move new element to a correct place. 
-       Swap heaps priority element with median as needed */
+       Swap heaps priority element with ADDTYPE(median) as needed */
     if (ci < m) {
         /* Lower heap */
         if (*pprev_was_smaller) {
-            pos = bubble_down_lt(image, B, heaps, m-1,
+            pos = ADDTYPE(bubble_down_lt)(image, B, heaps, m-1,
                                  ci);
-            if (pos == ci) pos = bubble_up_lt(image, B, heaps, ci);
+            if (pos == ci) pos = ADDTYPE(bubble_up_lt)(image, B, heaps, ci);
         }
         else 
             /* previous was larger or nothing is known */
@@ -730,18 +725,18 @@ dheap_replace(const PIX_TYPE *image,
                and is faster than bubble_down. Therefore, in the case where nothing is known
                about the previous element, we try to bubble_up before bubble_down.
             */
-            pos = bubble_up_lt(image, B, heaps, ci);
-            if (pos == ci) pos = bubble_down_lt(image, B, heaps, m-1, ci);
+            pos = ADDTYPE(bubble_up_lt)(image, B, heaps, ci);
+            if (pos == ci) pos = ADDTYPE(bubble_down_lt)(image, B, heaps, m-1, ci);
         }
         
         if (pos == 0 &&
             image[heaps[0]] > image[heaps[m]]) {
-            HEAP_SWAP(0, m, B, heaps);
+            ADDTYPE(HEAP_SWAP)(0, m, B, heaps);
             
             if (image[heaps[m1]] < 
                 image[heaps[m]]) {
-                HEAP_SWAP(m1, m, B, heaps);
-                bubble_down_gt(image, B, heaps, m, m1, 0);
+                ADDTYPE(HEAP_SWAP)(m1, m, B, heaps);
+                ADDTYPE(bubble_down_gt)(image, B, heaps, m, m1, 0);
             }
         }
 
@@ -758,23 +753,23 @@ dheap_replace(const PIX_TYPE *image,
     else if (ci >= m1) {
         /* Upper heap */
         if (*pprev_was_larger) {
-            pos = bubble_down_gt(image, B, heaps, m, m1, ci-(m1));
+            pos = ADDTYPE(bubble_down_gt)(image, B, heaps, m, m1, ci-(m1));
             if (pos == ci-(m1)) 
-                pos = bubble_up_gt(image, B, heaps, m1, ci-(m1));
+                pos = ADDTYPE(bubble_up_gt)(image, B, heaps, m1, ci-(m1));
         }
         else {
-            pos = bubble_up_gt(image, B, heaps, m1, ci-(m1));
+            pos = ADDTYPE(bubble_up_gt)(image, B, heaps, m1, ci-(m1));
             if (pos == ci-(m1)) 
-                pos = bubble_down_gt(image, B, heaps, m, m1, ci-(m1));
+                pos = ADDTYPE(bubble_down_gt)(image, B, heaps, m, m1, ci-(m1));
         }
 
         if (pos == 0 &&
             image[heaps[m1]] < image[heaps[m]]) {
-            HEAP_SWAP(m1, m, B, heaps);
+            ADDTYPE(HEAP_SWAP)(m1, m, B, heaps);
             
             if (image[heaps[0]] > image[heaps[m]]) {
-                HEAP_SWAP(0, m, B, heaps);
-                bubble_down_lt(image, B, heaps, m-1, 0);
+                ADDTYPE(HEAP_SWAP)(0, m, B, heaps);
+                ADDTYPE(bubble_down_lt)(image, B, heaps, m-1, 0);
             }
         }
 
@@ -783,19 +778,19 @@ dheap_replace(const PIX_TYPE *image,
         return;
     }
     else {
-        /* The median element was replaced */
+        /* The ADDTYPE(median) element was replaced */
         if (*pprev_was_smaller) {
             if (image[heaps[0]] > image[heaps[m]]) {
-                HEAP_SWAP(0, m, B, heaps);
-                bubble_down_lt(image, B, heaps, m-1, 0);
+                ADDTYPE(HEAP_SWAP)(0, m, B, heaps);
+                ADDTYPE(bubble_down_lt)(image, B, heaps, m-1, 0);
                 *pprev_was_smaller = mytrue;
                 *pprev_was_larger = myfalse;
                 return;
             } 
             else if (image[heaps[m1]] < 
                      image[heaps[m]]) {
-                HEAP_SWAP(m1, m, B, heaps);
-                bubble_down_gt(image, B, heaps, m, m1, 0);
+                ADDTYPE(HEAP_SWAP)(m1, m, B, heaps);
+                ADDTYPE(bubble_down_gt)(image, B, heaps, m, m1, 0);
                 *pprev_was_smaller = myfalse;
                 *pprev_was_larger = mytrue;
                 return;
@@ -807,14 +802,14 @@ dheap_replace(const PIX_TYPE *image,
         } else {
             if (image[heaps[m1]] < 
                 image[heaps[m]]) {
-                HEAP_SWAP(m1, m, B, heaps);
-                bubble_down_gt(image, B, heaps, m, m1, 0);
+                ADDTYPE(HEAP_SWAP)(m1, m, B, heaps);
+                ADDTYPE(bubble_down_gt)(image, B, heaps, m, m1, 0);
                 *pprev_was_smaller = myfalse;
                 *pprev_was_larger = mytrue;
                 return;
             } else if (image[heaps[0]] > image[heaps[m]]) {
-                HEAP_SWAP(0, m, B, heaps);
-                bubble_down_lt(image, B, heaps, m-1, 0);
+                ADDTYPE(HEAP_SWAP)(0, m, B, heaps);
+                ADDTYPE(bubble_down_lt)(image, B, heaps, m-1, 0);
                 *pprev_was_smaller = mytrue;
                 *pprev_was_larger = myfalse;
                 return;
@@ -827,36 +822,6 @@ dheap_replace(const PIX_TYPE *image,
     }
 }
 
-#ifdef CPL_FILTER_NEEDS_GET_KTH
-
-static
-PIX_TYPE get_kth(PIX_TYPE * a, int n, int k)
-{
-    PIX_TYPE x;
-    int     i, j, l, m;
-
-    l=0; m=n-1;
-    while (l<m) {
-        x=a[k];
-        i=l;
-        j=m;
-        do {
-            while (a[i]<x) i++;
-            while (x<a[j]) j--;
-            if (i<=j) {
-                const PIX_TYPE temp = a[i];
-                a[i] = a[j];
-                a[j] = temp;
-                i++; j--;
-            }
-        } while (i<=j);
-        if (j<k) l=i;
-        if (k<i) m=j;
-    }
-    return a[k];
-}
-#endif
-
 /**
   @internal
   @brief max of 3 elements
@@ -868,7 +833,7 @@ PIX_TYPE get_kth(PIX_TYPE * a, int n, int k)
   Cost: 2 comparisons
  */
 static inline PIX_TYPE
-max3(PIX_TYPE p0,
+ADDTYPE(max3)(PIX_TYPE p0,
      PIX_TYPE p1,
      PIX_TYPE p2)
 {
@@ -890,7 +855,7 @@ max3(PIX_TYPE p0,
   Cost: 2 comparisons
 */
 static inline PIX_TYPE
-min3(PIX_TYPE p0,
+ADDTYPE(min3)(PIX_TYPE p0,
      PIX_TYPE p1,
      PIX_TYPE p2)
 {
@@ -903,13 +868,13 @@ min3(PIX_TYPE p0,
 
 /**
   @internal
-  @brief median of 5
+  @brief ADDTYPE(median) of 5
   @param  p0  element
   @param  p1  element
   @param  p2  element
   @param  p3  element
   @param  p4  element
-  @return median
+  @return ADDTYPE(median)
 
   Precondition: p0 <= p1 <= p2
 
@@ -919,14 +884,14 @@ min3(PIX_TYPE p0,
   = 20/7 = 2.86 comparisons
  */
 static inline PIX_TYPE
-median5(PIX_TYPE p0,
+ADDTYPE(median5)(PIX_TYPE p0,
         PIX_TYPE p1,
         PIX_TYPE p2,
         PIX_TYPE p3,
         PIX_TYPE p4)
 {
     /*
-      This would be a one-off median filter (i.e. rank 4th, 5th
+      This would be a one-off ADDTYPE(median) filter (i.e. rank 4th, 5th
       or 6th of 9). Saves some 35% execution time.
 
       return p1;
@@ -934,14 +899,14 @@ median5(PIX_TYPE p0,
     
     return (p3 < p1)
         ?
-        ((p4 >= p1) ? p1 : max3(p3, p4, p0))
+        ((p4 >= p1) ? p1 : ADDTYPE(max3)(p3, p4, p0))
         :
-        ((p4 <= p1) ? p1 : min3(p3, p4, p2));
+        ((p4 <= p1) ? p1 : ADDTYPE(min3)(p3, p4, p2));
 }
 
 /**
   @internal
-  @brief median of 9
+  @brief ADDTYPE(median) of 9
   @param  p0  element
   @param  p1  element
   @param  p2  element
@@ -951,7 +916,7 @@ median5(PIX_TYPE p0,
   @param  p6  element
   @param  p7  element
   @param  p8  element
-  @return median
+  @return ADDTYPE(median)
 
   Precondition: 
   p0 <= p1 <= p2
@@ -965,7 +930,7 @@ median5(PIX_TYPE p0,
   = 95/21 = 4.52 comparisons
 */
 static inline PIX_TYPE
-median9_2(PIX_TYPE p0,
+ADDTYPE(median9_2)(PIX_TYPE p0,
           PIX_TYPE p1,
           PIX_TYPE p2,
           PIX_TYPE p3,
@@ -977,14 +942,14 @@ median9_2(PIX_TYPE p0,
 {
     return (p4 <= p7) 
         ? 
-        median5(p3, p4, p5, p2, p6)
+        ADDTYPE(median5)(p3, p4, p5, p2, p6)
         :
-        ((p1 < p7) ? median5(p6, p7, p8, p2, p3)
-         : median5(p0, p1, p2, p8, p3));
+        ((p1 < p7) ? ADDTYPE(median5)(p6, p7, p8, p2, p3)
+         : ADDTYPE(median5)(p0, p1, p2, p8, p3));
 }
 /**
   @internal
-  @brief median of 9
+  @brief ADDTYPE(median) of 9
   @param  p0  element
   @param  p1  element
   @param  p2  element
@@ -994,7 +959,7 @@ median9_2(PIX_TYPE p0,
   @param  p6  element
   @param  p7  element
   @param  p8  element
-  @return median
+  @return ADDTYPE(median)
 
   Precondition: 
   p0 <= p1 <= p2
@@ -1004,7 +969,7 @@ median9_2(PIX_TYPE p0,
   Cost: 1 + 4.52 = 116/21 = 5.52 comparisons
 */
 static inline PIX_TYPE
-median9_1(PIX_TYPE p0,
+ADDTYPE(median9_1)(PIX_TYPE p0,
           PIX_TYPE p1,
           PIX_TYPE p2,
           PIX_TYPE p3,
@@ -1014,8 +979,8 @@ median9_1(PIX_TYPE p0,
           PIX_TYPE p7,
           PIX_TYPE p8)
 {
-    return (p1 < p4) ? median9_2(p0, p1, p2, p3, p4, p5, p6, p7, p8)
-             : median9_2(p3, p4, p5, p0, p1, p2, p6, p7, p8);
+    return (p1 < p4) ? ADDTYPE(median9_2)(p0, p1, p2, p3, p4, p5, p6, p7, p8)
+             : ADDTYPE(median9_2)(p3, p4, p5, p0, p1, p2, p6, p7, p8);
 }
 
 /**
@@ -1060,7 +1025,7 @@ do {                                                          \
 
 /**
   @internal
-  @brief fast 3x3 image median filter
+  @brief fast 3x3 image ADDTYPE(median) filter
   @param in           image to filter
   @param out          pre-allocated output image,
                       must not overlap with the input image
@@ -1069,9 +1034,14 @@ do {                                                          \
   @param border_mode
 
   periodic boundaries
+  
+  WARNING: If this function is modified (only sections regarding 
+  CPL_BORDER_FILTER), also the fill_chess function has to be modified, for
+  consistency. Here, an alternate lower and upper median is taken, which has
+  the same effect as the +- inf chess-like pattern.
 */
 static void
-filter_median_1(const PIX_TYPE *in, PIX_TYPE *out,
+ADDTYPE(filter_median_1)(const PIX_TYPE *in, PIX_TYPE *out,
                 unsigned Nx, unsigned Ny, unsigned border_mode)
 {
     register PIX_TYPE p0, p1, p2, p3, p4, p5;
@@ -1082,7 +1052,7 @@ filter_median_1(const PIX_TYPE *in, PIX_TYPE *out,
     const PIX_TYPE * i1 = i0 + Nx;
     const PIX_TYPE * i2 = i1 + Nx;
     const PIX_TYPE * istop = in + Nx*Ny; /* First pixel not to be accessed */
-    unsigned m = 2;  /* lower median of 6 elements */
+    unsigned m = 2;  /* lower ADDTYPE(median) of 6 elements */
 
     assure( Nx >= 3 );
     assure( Ny >= 3 );
@@ -1096,19 +1066,20 @@ filter_median_1(const PIX_TYPE *in, PIX_TYPE *out,
             buf6[1] = i0[1];
             buf6[2] = i1[0];
             buf6[3] = i1[1];
-            out[j++] = GET_KTH(buf6, 4, 2); /* upper median */
+            /* upper median */
+            out[j++] = ADDTYPE(cpl_tools_get_kth)(buf6, 4, 2);
 
             for (; j < Nx-1; j++) {
                 memcpy(buf6,   i0+j-1, 3*sizeof(*buf6));
                 memcpy(buf6+3, i1+j-1, 3*sizeof(*buf6));
-                out[j] = GET_KTH(buf6, 6, m);
+                out[j] = ADDTYPE(cpl_tools_get_kth)(buf6, 6, m);
                 m = 5-m; /* alternate between lower/upper median */
             }
             buf6[0] = i0[Nx-2];
             buf6[1] = i0[Nx-1];
             buf6[2] = i1[Nx-2];
             buf6[3] = i1[Nx-1];
-            out[j] = GET_KTH(buf6, 4, m-1);
+            out[j] = ADDTYPE(cpl_tools_get_kth)(buf6, 4, m-1);
             out += Nx;
 
             break;
@@ -1125,7 +1096,7 @@ filter_median_1(const PIX_TYPE *in, PIX_TYPE *out,
             break;
     }
 
-    if (border_mode == CPL_BORDER_FILTER) m = 2; /* lower median */
+    if (border_mode == CPL_BORDER_FILTER) m = 2; /* lower ADDTYPE(median) */
 
     for (;i2 < istop; i0 += Nx, i1 += Nx, i2 += Nx, out += Nx) {
 
@@ -1144,7 +1115,7 @@ filter_median_1(const PIX_TYPE *in, PIX_TYPE *out,
                 buf6[3] = i0[1];
                 buf6[4] = i1[1];
                 buf6[5] = i2[1];
-                out[0] = GET_KTH(buf6, 6, m);
+                out[0] = ADDTYPE(cpl_tools_get_kth)(buf6, 6, m);
 
                 /* j must be incremented twice, but out[j] should
                    only be incremented once */
@@ -1188,7 +1159,7 @@ filter_median_1(const PIX_TYPE *in, PIX_TYPE *out,
             }
         
             out[j]
-                = (PIX_TYPE)median9_1(p0, p1, p2, p3, p4, p5, p6, p7, p8);
+                = (PIX_TYPE)ADDTYPE(median9_1)(p0, p1, p2, p3, p4, p5, p6, p7, p8);
 
             /* The cost per pixel was determined by considering the 9! possible
                permutations of 9 different numbers and subsequently confirmed by
@@ -1223,7 +1194,8 @@ filter_median_1(const PIX_TYPE *in, PIX_TYPE *out,
             buf6[4] = i1[j-1];
             buf6[5] = i2[j-1];
 
-            out[j] = GET_KTH(buf6, 6, ((Nx & 1) == 1) ? m : 5-m);
+            out[j] = ADDTYPE(cpl_tools_get_kth)(buf6, 6,
+                                                ((Nx & 1) == 1) ? m : 5-m);
             m = 5-m;
 
             /* out was decremented, increment it back */
@@ -1240,20 +1212,20 @@ filter_median_1(const PIX_TYPE *in, PIX_TYPE *out,
             buf6[1] = i0[1];
             buf6[2] = i1[0];
             buf6[3] = i1[1];
-            out[j++] = GET_KTH(buf6, 4, m-1);
+            out[j++] = ADDTYPE(cpl_tools_get_kth)(buf6, 4, m-1);
             m = 5-m;
 
             for (; j < Nx-1; j++) {
                 memcpy(buf6,   i0+j-1, 3*sizeof(*buf6));
                 memcpy(buf6+3, i1+j-1, 3*sizeof(*buf6));
-                out[j] = GET_KTH(buf6, 6, m);
+                out[j] = ADDTYPE(cpl_tools_get_kth)(buf6, 6, m);
                 m = 5-m;
             }
             buf6[0] = i0[Nx-2];
             buf6[1] = i0[Nx-1];
             buf6[2] = i1[Nx-2];
             buf6[3] = i1[Nx-1];
-            out[j] = GET_KTH(buf6, 4, m-1);
+            out[j] = ADDTYPE(cpl_tools_get_kth)(buf6, 4, m-1);
 
             break;
         }
@@ -1283,9 +1255,13 @@ filter_median_1(const PIX_TYPE *in, PIX_TYPE *out,
   @param val2      other value
 
   Every second pixel is filled with val1, other pixels with val2
+  
+  WARNING: This function is duplicated from the code found in 
+  tests/cpl_filter_body.h. Do not forget to keep both in sync.
+  
 */
 static void
-fill_row(PIX_TYPE *in, unsigned nx, unsigned y,
+ADDTYPE(fill_row)(PIX_TYPE *in, unsigned nx, unsigned y,
          unsigned xmin, unsigned xmax,
          PIX_TYPE val1, PIX_TYPE val2) 
 {
@@ -1313,12 +1289,22 @@ fill_row(PIX_TYPE *in, unsigned nx, unsigned y,
   @param Ny            in height
   @param rx            filter x-radius
   @param ry            filter y-radius
+  
+  WARNING: This function is duplicated from the code found in 
+  tests/cpl_filter_body.h. Do not forget to keep both in sync.
+  
+  WARNING: If this function is modified , also the filter_median_1 function 
+  has to be modified (only sections regarding CPL_BORDER_FILTER), for
+  consistency. Here, an +- inf chess-like pattern is used, which has
+  the same effect as the alternate lower and upper median used in 
+  filter_median_1.
+  
 */
-static void
-fill_chess(PIX_TYPE *in_larger, const PIX_TYPE *in,
-           unsigned Nx_larger, unsigned Ny_larger, 
-           unsigned Nx, unsigned Ny, 
-           unsigned rx, unsigned ry)
+void
+ADDTYPE(cpl_image_filter_fill_chess)(PIX_TYPE *in_larger, const PIX_TYPE *in,
+                                     unsigned Nx_larger, unsigned Ny_larger, 
+                                     unsigned Nx, unsigned Ny, 
+                                     unsigned rx, unsigned ry)
 {
     unsigned y;
     
@@ -1326,7 +1312,7 @@ fill_chess(PIX_TYPE *in_larger, const PIX_TYPE *in,
        Loop over images in cache-friendly order.  */
     for (y = 0; y < ry; y++) {
 
-        fill_row(in_larger, Nx_larger, y,
+        ADDTYPE(fill_row)(in_larger, Nx_larger, y,
                  0, Nx_larger,
                  (y & 1) == 0 ? PIX_MAX : PIX_MIN,
                  (y & 1) == 0 ? PIX_MIN : PIX_MAX);
@@ -1334,7 +1320,7 @@ fill_chess(PIX_TYPE *in_larger, const PIX_TYPE *in,
 
     for (y = ry; y < ry+Ny; y++) {
 
-        fill_row(in_larger, Nx_larger, y,
+        ADDTYPE(fill_row)(in_larger, Nx_larger, y,
                  0, rx,
                  (y & 1) == 0 ? PIX_MAX : PIX_MIN,
                  (y & 1) == 0 ? PIX_MIN : PIX_MAX);
@@ -1342,7 +1328,7 @@ fill_chess(PIX_TYPE *in_larger, const PIX_TYPE *in,
         (void)memcpy(in_larger + rx + y*Nx_larger,
                      in + (y-ry)*Nx, Nx*sizeof(*in));
             
-        fill_row(in_larger, Nx_larger, y,
+        ADDTYPE(fill_row)(in_larger, Nx_larger, y,
                  rx + Nx, Nx_larger,
                  ((y + rx + Nx) & 1) == 0 ? PIX_MAX : PIX_MIN,
                  ((y + rx + Nx) & 1) == 0 ? PIX_MIN : PIX_MAX);
@@ -1350,7 +1336,7 @@ fill_chess(PIX_TYPE *in_larger, const PIX_TYPE *in,
 
     for (y = ry + Ny; y < Ny_larger; y++) {
 
-        fill_row(in_larger, Nx_larger, y,
+        ADDTYPE(fill_row)(in_larger, Nx_larger, y,
                  0, Nx_larger,
                  (y & 1) == 0 ? PIX_MAX : PIX_MIN,
                  (y & 1) == 0 ? PIX_MIN : PIX_MAX);
@@ -1359,7 +1345,7 @@ fill_chess(PIX_TYPE *in_larger, const PIX_TYPE *in,
 
 /**
   @internal
-  @brief Fast, any bit-depth, image median filter
+  @brief Fast, any bit-depth, image ADDTYPE(median) filter
 
   @param in          image to filter
   @param out         pre-allocated output image, may not overlap with the input image,
@@ -1372,7 +1358,7 @@ fill_chess(PIX_TYPE *in_larger, const PIX_TYPE *in,
                     where part of the kernel would be outside the image)
 
   Supported border modes:
-      CPL_BORDER_FILTER: Compute the median of available values.
+      CPL_BORDER_FILTER: Compute the ADDTYPE(median) of available values.
       CPL_BORDER_NOP:    The border region in the output image is not
                                    touched.
       CPL_BORDER_CROP:   The border region is removed. Consequently the
@@ -1389,22 +1375,22 @@ fill_chess(PIX_TYPE *in_larger, const PIX_TYPE *in,
     Mean: As any, except the arithmetic mean of the two central values is used.
 
             
-  For each pixel in the image the median of a (2*Rx + 1)(2*Ry + 1) window is 
+  For each pixel in the image the ADDTYPE(median) of a (2*Rx + 1)(2*Ry + 1) window is 
   computed.
 
   The straightforward approach is to loop over the image and build and sort the 
   kernel array for every pixel. This leads to an O(RxRy log RxRy) algorithm, or 
-  O(RxRy) if the median is computed by repeated partitions.
+  O(RxRy) if the ADDTYPE(median) is computed by repeated partitions.
 
   The basic idea in this implementation is to maintain a double heap structure of
   the running window pixels:
 
   _____
-  \   / <-- minimum heap of elements greater than the median
+  \   / <-- minimum heap of elements greater than the ADDTYPE(median)
    \ /
-    .   <-- median
+    .   <-- ADDTYPE(median)
 
-   / \  <-- maximum heap of elements less than the median
+   / \  <-- maximum heap of elements less than the ADDTYPE(median)
   /   \
   -----
 
@@ -1450,13 +1436,13 @@ fill_chess(PIX_TYPE *in_larger, const PIX_TYPE *in,
   Rx = Ry = 1 is handled as a separate case.
  */
 static void
-filter_median(const PIX_TYPE *in, PIX_TYPE *out,
+ADDTYPE(filter_median)(const PIX_TYPE *in, PIX_TYPE *out,
               unsigned Nx, unsigned Ny,
               unsigned rx, unsigned ry,
               unsigned border_mode)
 {
     unsigned out_Nx;
-    dheap *dh;
+    ADDTYPE(dheap) *dh;
     unsigned y;
     unsigned x;
     int dx;
@@ -1469,7 +1455,7 @@ filter_median(const PIX_TYPE *in, PIX_TYPE *out,
     assure( 2*ry + 1 <= Ny );
 
     if (rx == 1 && ry == 1) {
-        filter_median_1(in, out, Nx, Ny, border_mode);
+        ADDTYPE(filter_median_1)(in, out, Nx, Ny, border_mode);
         return;
     }
 
@@ -1504,9 +1490,10 @@ filter_median(const PIX_TYPE *in, PIX_TYPE *out,
         PIX_TYPE *in_larger = malloc(Nx_larger*Ny_larger*sizeof(*in_larger));
         assure( in_larger != NULL );
 
-        fill_chess(in_larger, in, Nx_larger, Ny_larger, Nx, Ny, rx, ry);
+        ADDTYPE(cpl_image_filter_fill_chess)(in_larger, in, Nx_larger,
+                                             Ny_larger, Nx, Ny, rx, ry);
         
-        filter_median(in_larger, out,
+        ADDTYPE(filter_median)(in_larger, out,
                       Nx_larger, Ny_larger,
                       rx, ry,
                       CPL_BORDER_CROP);
@@ -1531,7 +1518,7 @@ filter_median(const PIX_TYPE *in, PIX_TYPE *out,
     }
     else out_Nx = Nx;
 
-    dh = dheap_new(in, Nx, Ny, rx, ry);
+    dh = ADDTYPE(dheap_new)(in, Nx, Ny, rx, ry);
     
     /* Loop over image like this
        -->->->-
@@ -1560,11 +1547,11 @@ filter_median(const PIX_TYPE *in, PIX_TYPE *out,
         for (x = xfirst; x != xlast + dx; x += dx) {
 
             if (y == dh->ry) {
-                SCA_init(dh, x + dx * dh->rx);
+                ADDTYPE(SCA_init)(dh, x + dx * dh->rx);
             }
             else {
                 /* Most frequent case. Update just 1 pixel in 1 sorted column array */
-                SCA_replace(dh, x + dx * dh->rx,
+                ADDTYPE(SCA_replace)(dh, x + dx * dh->rx,
                             y - dh->ry - 1,
                             y + dh->ry);
             }
@@ -1577,11 +1564,11 @@ filter_median(const PIX_TYPE *in, PIX_TYPE *out,
                     unsigned i;
                     for (i  = x - dx*dh->rx; 
                          i != x + dx*dh->rx; i += dx) {
-                        SCA_replace(dh, i,
+                        ADDTYPE(SCA_replace)(dh, i,
                                     y - dh->ry - 1,
                                     y + dh->ry);
                         
-                        dheap_replace(dh->image,
+                        ADDTYPE(dheap_replace)(dh->image,
                                       dh->B,
                                       dh->heaps,
                                       &prev_was_larger,
@@ -1592,7 +1579,7 @@ filter_median(const PIX_TYPE *in, PIX_TYPE *out,
                                       i + ((y - dh->ry - 1) * dh->Nx));
                     }
                     
-                    dheap_replace(dh->image,
+                    ADDTYPE(dheap_replace)(dh->image,
                                   dh->B,
                                   dh->heaps,
                                   &prev_was_larger,
@@ -1630,14 +1617,14 @@ filter_median(const PIX_TYPE *in, PIX_TYPE *out,
                     anew = SCAxnew[i];
                     aold = SCAxold[i];
                     
-                    dheap_replace(image, B, heaps,
+                    ADDTYPE(dheap_replace)(image, B, heaps,
                                   &prev_was_larger,
                                   &prev_was_smaller,
                                   m, m1, anew, aold);
                 }
             }
 
-            out[x + y*out_Nx] = (PIX_TYPE)dheap_median(dh);
+            out[x + y*out_Nx] = (PIX_TYPE)ADDTYPE(dheap_median)(dh);
 
         } /* for x */
         
@@ -1660,8 +1647,37 @@ filter_median(const PIX_TYPE *in, PIX_TYPE *out,
         (void)memcpy(out, in, (ry*Nx + rx)*sizeof(*out));
     }
 
-    dheap_delete(dh);
+    ADDTYPE(dheap_delete)(dh);
     return;
 }
 
+static
+void ADDTYPE(cpl_filter_median_fast)(void * out, const void * in,
+                                     cpl_size Nx, cpl_size Ny, cpl_size hsizex,
+                                     cpl_size hsizey, cpl_border_mode mode)
+{
+
+    const unsigned Nxu     = (const unsigned)Nx;
+    const unsigned Nyu     = (const unsigned)Ny;
+    const unsigned hsizexu = (const unsigned)hsizex;
+    const unsigned hsizeyu = (const unsigned)hsizey;
+
+    if ((cpl_size)Nxu == Nx && (cpl_size)Nyu == Ny &&
+        (cpl_size)hsizexu == hsizex && (cpl_size)hsizeyu == hsizey) {
+        ADDTYPE(filter_median)(in, out, (unsigned)Nx, (unsigned)Ny,
+                               (unsigned)hsizex, (unsigned)hsizey,
+                               (unsigned)mode);
+    } else {
+        cpl_error_set_(CPL_ERROR_UNSUPPORTED_MODE);
+    }
+
+    return;
+}
+
+
 /**@}*/
+
+#undef IN_TYPE
+#undef PIX_TYPE
+#undef PIX_MIN
+#undef PIX_MAX
