@@ -1,4 +1,4 @@
-/* $Id: cpl_fits.c,v 1.15 2010/12/20 15:30:19 llundin Exp $
+/* $Id: cpl_fits.c,v 1.20 2012/03/05 13:46:41 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/12/20 15:30:19 $
- * $Revision: 1.15 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2012/03/05 13:46:41 $
+ * $Revision: 1.20 $
+ * $Name: cpl-6_0 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -33,14 +33,15 @@
                                    Includes
  -----------------------------------------------------------------------------*/
 
-#include <string.h>
-#include <fitsio.h>
-
 #include "cpl_fits.h"
+#include "cpl_io_fits.h"
 #include "cpl_error_impl.h"
 #include "cpl_memory.h"
 #include "cpl_tools.h"
 
+#include <string.h>
+#include <fitsio.h>
+
 /*----------------------------------------------------------------------------*/
 /**
  * @defgroup cpl_fits   FITS related basic routines
@@ -71,15 +72,15 @@
   - CPL_ERROR_ILLEGAL_INPUT if the input file is not FITS
  */
 /*----------------------------------------------------------------------------*/
-int cpl_fits_count_extensions(const char * filename)
+cpl_size cpl_fits_count_extensions(const char * filename)
 {
     fitsfile * fptr;
     int        error = 0;
-    int        next  = 0;
+    int        next  = 0; /* CFITSIO supports only int */
 
     cpl_ensure(filename != NULL, CPL_ERROR_NULL_INPUT, -1);
 
-    if (fits_open_diskfile(&fptr, filename, READONLY, &error)) {
+    if (cpl_io_fits_open_diskfile(&fptr, filename, READONLY, &error)) {
         (void)cpl_error_set_fits(CPL_ERROR_FILE_IO, error, fits_open_diskfile,
                                  "filename='%s'", filename);
         return -1;
@@ -92,13 +93,13 @@ int cpl_fits_count_extensions(const char * filename)
         next = 0;
     }
 
-    if (fits_close_file(fptr, &error)) {
+    if (cpl_io_fits_close_file(fptr, &error)) {
         (void)cpl_error_set_fits(CPL_ERROR_FILE_IO, error, fits_close_file,
                                  "filename='%s'", filename);
     }
 
-    /* The nb of HDUs includes the primary one, which is not an extension */
-    return next - 1;
+    /* The number of HDUs includes the primary one which is not an extension */
+    return (cpl_size)next - 1;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -106,25 +107,26 @@ int cpl_fits_count_extensions(const char * filename)
   @brief    Get the place of a given extension in a FITS file
   @param    filename The file name
   @param    extname  The extension name
-  @return   the extension number, 0 if not found or -1 on error
+  @return   The extension number, 0 if not found or -1 on error
 
   Possible #_cpl_error_code_ set in this function:
   - CPL_ERROR_NULL_INPUT if an input pointer is NULL
   - CPL_ERROR_FILE_IO if the file is not FITS
  */
 /*----------------------------------------------------------------------------*/
-int cpl_fits_find_extension(const char * filename,
-                            const char * extname)
+cpl_size cpl_fits_find_extension(const char * filename,
+                                 const char * extname)
 {
-    fitsfile * fptr;
-    int        error = 0;
-    int        ext_num    = 0;
-    char       c_extname[FLEN_VALUE];
+    fitsfile  * fptr;
+    int         error = 0;
+    cpl_size    ext_num    = 0;
+    char        c_extname[FLEN_VALUE];
+    cpl_boolean has_card;
 
     cpl_ensure(filename != NULL, CPL_ERROR_NULL_INPUT, -1);
     cpl_ensure(extname  != NULL, CPL_ERROR_NULL_INPUT, -1);
 
-    if (fits_open_diskfile(&fptr, filename, READONLY, &error)) {
+    if (cpl_io_fits_open_diskfile(&fptr, filename, READONLY, &error)) {
         (void)cpl_error_set_fits(CPL_ERROR_FILE_IO, error, fits_open_diskfile,
                                  "filename='%s', extname='%s'",
                                  filename, extname);
@@ -132,23 +134,26 @@ int cpl_fits_find_extension(const char * filename,
     }
 
     do {
-        const char * keyword = "EXTNAME";
-
-        if (!fits_movrel_hdu(fptr, 1, NULL, &error) &&
-            /* FIXME: Cast string literal to avoid compiler warnings for CFTISIO
-               older than v. 3.18 */
-            !fits_read_key_str(fptr, (char*)keyword, c_extname, NULL, &error))
-
+        has_card = CPL_TRUE;
+        if (!fits_movabs_hdu(fptr, 2 + ext_num, NULL, &error) &&
+            fits_read_key_str(fptr, "EXTNAME", c_extname, NULL, &error) ==
+            KEY_NO_EXIST) {
+            /* EXTNAME card is absent, that is allowed */
+            has_card = CPL_FALSE;
+            error = 0; /* Reset so subsequent calls work */
+        }
+        if (!error) {
             ext_num++;
+        }
 
-    } while (!error && strncmp(extname, c_extname, FLEN_VALUE));
+    } while (!error && (!has_card || strncmp(extname, c_extname, FLEN_VALUE)));
 
     if (error) {
         ext_num = error == END_OF_FILE ? 0 : -1;
         error = 0; /* Reset so fits_close_file works */
     }
 
-    if (fits_close_file(fptr, &error)) {
+    if (cpl_io_fits_close_file(fptr, &error)) {
         (void)cpl_error_set_fits(CPL_ERROR_FILE_IO, error, fits_close_file,
                                  "filename='%s', extname='%s'",
                                  filename, extname);
@@ -171,7 +176,7 @@ int cpl_fits_find_extension(const char * filename,
 
 int cpl_fits_get_nb_extensions(const char * filename)
 {
-    return cpl_fits_count_extensions(filename);
+    return (int)cpl_fits_count_extensions(filename);
 }
 
 /*----------------------------------------------------------------------------*/
@@ -189,7 +194,7 @@ int cpl_fits_get_nb_extensions(const char * filename)
 int cpl_fits_get_extension_nb(const char  *   filename,
                               const char  *   extname)
 {
-    return cpl_fits_find_extension(filename, extname);
+    return (int)cpl_fits_find_extension(filename, extname);
 }
 
 /**@}*/
