@@ -1,4 +1,4 @@
-/* $Id: cpl_image_basic.h,v 1.47 2010/11/11 09:23:18 llundin Exp $
+/* $Id: cpl_image_basic.h,v 1.49 2011/07/20 08:49:07 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/11/11 09:23:18 $
- * $Revision: 1.47 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2011/07/20 08:49:07 $
+ * $Revision: 1.49 $
+ * $Name: cpl-6_0 $
  */
 
 #ifndef CPL_IMAGE_BASIC_H
@@ -67,10 +67,14 @@ typedef enum _cpl_norm_ cpl_norm;
  -----------------------------------------------------------------------------*/
 
 /* Basic operations */
-cpl_image * cpl_image_add_create(const cpl_image *, const cpl_image *);
-cpl_image * cpl_image_subtract_create(const cpl_image *, const cpl_image *);
-cpl_image * cpl_image_multiply_create(const cpl_image *, const cpl_image *);
-cpl_image * cpl_image_divide_create(const cpl_image *, const cpl_image *); 
+cpl_image * cpl_image_add_create(const cpl_image *, const cpl_image *)
+    CPL_ATTR_ALLOC;
+cpl_image * cpl_image_subtract_create(const cpl_image *, const cpl_image *)
+    CPL_ATTR_ALLOC;
+cpl_image * cpl_image_multiply_create(const cpl_image *, const cpl_image *)
+    CPL_ATTR_ALLOC;
+cpl_image * cpl_image_divide_create(const cpl_image *, const cpl_image *)
+    CPL_ATTR_ALLOC;
 cpl_error_code cpl_image_add(cpl_image *, const cpl_image *);
 cpl_error_code cpl_image_subtract(cpl_image *, const cpl_image *);
 cpl_error_code cpl_image_multiply(cpl_image *, const cpl_image *);
@@ -86,52 +90,71 @@ cpl_error_code cpl_image_logarithm(cpl_image *, double);
 cpl_error_code cpl_image_normalise(cpl_image *, cpl_norm);
 cpl_error_code cpl_image_abs(cpl_image *);
 
-cpl_image * cpl_image_add_scalar_create(const cpl_image *, double);
-cpl_image * cpl_image_subtract_scalar_create(const cpl_image *, double);
-cpl_image * cpl_image_multiply_scalar_create(const cpl_image *, double);
-cpl_image * cpl_image_divide_scalar_create(const cpl_image *, double);
-cpl_image * cpl_image_power_create(const cpl_image *, double);
-cpl_image * cpl_image_exponential_create(const cpl_image *, double);
-cpl_image * cpl_image_logarithm_create(const cpl_image *, double);
-cpl_image * cpl_image_normalise_create(const cpl_image *, cpl_norm);
-cpl_image * cpl_image_abs_create(const cpl_image *);
+cpl_image * cpl_image_add_scalar_create(const cpl_image *, double)
+    CPL_ATTR_ALLOC;
+cpl_image * cpl_image_subtract_scalar_create(const cpl_image *, double)
+    CPL_ATTR_ALLOC;
+cpl_image * cpl_image_multiply_scalar_create(const cpl_image *, double)
+    CPL_ATTR_ALLOC;
+cpl_image * cpl_image_divide_scalar_create(const cpl_image *, double)
+    CPL_ATTR_ALLOC;
+cpl_image * cpl_image_power_create(const cpl_image *, double)
+    CPL_ATTR_ALLOC;
+cpl_image * cpl_image_exponential_create(const cpl_image *, double)
+    CPL_ATTR_ALLOC;
+cpl_image * cpl_image_logarithm_create(const cpl_image *, double)
+    CPL_ATTR_ALLOC;
+cpl_image * cpl_image_normalise_create(const cpl_image *, cpl_norm)
+    CPL_ATTR_ALLOC;
+cpl_image * cpl_image_abs_create(const cpl_image *)
+    CPL_ATTR_ALLOC;
 
 cpl_error_code cpl_image_threshold(cpl_image *, double, double, double, double);
-cpl_image * cpl_image_average_create(const cpl_image *, const cpl_image *);
+cpl_image * cpl_image_average_create(const cpl_image *, const cpl_image *)
+    CPL_ATTR_ALLOC;
 
 /* Collapse functions */
-cpl_image * cpl_image_collapse_window_create(const cpl_image *, int, int, int,
-                                             int, int);
-cpl_image * cpl_image_collapse_create(const cpl_image *, int);
-cpl_image * cpl_image_collapse_median_create(const cpl_image *, int, int, int);
+cpl_image * cpl_image_collapse_window_create(const cpl_image *, cpl_size,
+                                             cpl_size, cpl_size,
+                                             cpl_size, int)
+    CPL_ATTR_ALLOC;
+cpl_image * cpl_image_collapse_create(const cpl_image *, int)
+    CPL_ATTR_ALLOC;
+cpl_image * cpl_image_collapse_median_create(const cpl_image *, int,
+                                             cpl_size, cpl_size)
+    CPL_ATTR_ALLOC;
 
 /* Extraction function */
-cpl_image * cpl_image_extract(const cpl_image *, int, int, int, int);
-cpl_vector * cpl_vector_new_from_image_row(const cpl_image *, int);
-cpl_vector * cpl_vector_new_from_image_column(const cpl_image *, int);
+cpl_image * cpl_image_extract(const cpl_image *, cpl_size, cpl_size,
+                              cpl_size, cpl_size) CPL_ATTR_ALLOC;
+cpl_vector * cpl_vector_new_from_image_row(const cpl_image *, cpl_size)
+    CPL_ATTR_ALLOC;
+cpl_vector * cpl_vector_new_from_image_column(const cpl_image *, cpl_size)
+    CPL_ATTR_ALLOC;
 
 /* Rotation and Shift */
 cpl_error_code cpl_image_turn(cpl_image *, int);
-cpl_error_code cpl_image_shift(cpl_image *, int, int);
+cpl_error_code cpl_image_shift(cpl_image *, cpl_size, cpl_size);
 
 /* Insert an image in an other one */
-cpl_error_code cpl_image_copy(cpl_image *, const cpl_image *, int, int);
+cpl_error_code cpl_image_copy(cpl_image *, const cpl_image *,
+                              cpl_size, cpl_size);
 
 /* Symmetry function */
 cpl_error_code cpl_image_flip(cpl_image *, int);
 
 /* Pixels re-organization */
-cpl_error_code cpl_image_move(cpl_image *, int, const int *);
+cpl_error_code cpl_image_move(cpl_image *, cpl_size, const cpl_size *);
 
 /* Gaussian fit of an image zone */
 cpl_error_code
-cpl_image_fit_gaussian(const cpl_image *, int, int, int,
+cpl_image_fit_gaussian(const cpl_image *, cpl_size, cpl_size, cpl_size,
                        double *, double *, double *, double *,
                        double *, double *, double *) CPL_ATTR_DEPRECATED;
 
 /* FWHM computation on a local maximum */
-cpl_error_code cpl_image_get_fwhm(const cpl_image *, int, int, double *, 
-                                  double *);
+cpl_error_code cpl_image_get_fwhm(const cpl_image *, cpl_size, cpl_size,
+                                  double *, double *);
 
 /* FFT computation */
 cpl_error_code cpl_image_fft(cpl_image *, cpl_image *, unsigned);
