@@ -1,4 +1,4 @@
-/* $Id: cpl_image_filter.c,v 1.122 2011/01/21 16:24:21 llundin Exp $
+/* $Id: cpl_image_filter.c,v 1.126 2011/08/18 13:03:37 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2011/01/21 16:24:21 $
- * $Revision: 1.122 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2011/08/18 13:03:37 $
+ * $Revision: 1.126 $
+ * $Name: cpl-6_0 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -33,6 +33,15 @@
                                    Includes
  -----------------------------------------------------------------------------*/
 
+#include "cpl_memory.h"
+#include "cpl_error_impl.h"
+#include "cpl_image_filter_impl.h"
+#include "cpl_image_bpm.h"
+#include "cpl_mask.h"
+#include "cpl_tools.h"
+
+#include "cpl_image_defs.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -41,15 +50,8 @@
 
 #include <assert.h>
 
-#include "cpl_memory.h"
-#include "cpl_error_impl.h"
-#include "cpl_image_filter.h"
-#include "cpl_filter_median_impl.h"
-#include "cpl_image_bpm.h"
-#include "cpl_mask.h"
-#include "cpl_tools.h"
-
-#include "cpl_image_defs.h"
+#include <float.h>
+#include <limits.h>
 
 /*-----------------------------------------------------------------------------
                                    Defines
@@ -70,6 +72,27 @@
 /* First includes involve no casting */
 #define IN_EQ_OUT
 
+#define IN_TYPE double
+#define PIX_TYPE double
+/* FIXME: Use DBL_MIN */
+#define PIX_MIN (-DBL_MAX)
+#define PIX_MAX DBL_MAX
+#include "cpl_filter_median.c"
+
+#define IN_TYPE float
+#define PIX_TYPE float
+/* FIXME: Use FLT_MIN */
+#define PIX_MIN (-FLT_MAX)
+#define PIX_MAX FLT_MAX
+#include "cpl_filter_median.c"
+
+#define IN_TYPE int
+#define PIX_TYPE int
+/* FIXME: Use INT_MIN */
+#define PIX_MIN (-INT_MAX)
+#define PIX_MAX INT_MAX
+#include "cpl_filter_median.c"
+
 #define IN_TYPE double
 #define OUT_TYPE double
 #define ACCU_TYPE double
@@ -141,17 +164,17 @@ static cpl_mask * cpl_mask_new_from_matrix(const cpl_matrix *, double, double);
   @param  self   Pre-allocated image to hold the filtered result
   @param  other  Image to filter
   @param  kernel Pixels to use, if set to CPL_BINARY_1
-  @param  filter CPL_FILTER_MEDIAN, CPL_FILTER_AVERAGE, CPL_FILTER_STDEV
-  @param  border CPL_BORDER_FILTER
+  @param  filter CPL_FILTER_MEDIAN, CPL_FILTER_AVERAGE and more, see below
+  @param  border CPL_BORDER_FILTER and more, see below
   @return CPL_ERROR_NONE or the relevant CPL error code
 
+  The kernel must have an odd number of rows and an odd number of columns.
+
   The two images must have equal dimensions, except for the border mode
   CPL_BORDER_CROP, where the input image must have 2 * hx columns more and
   2 * hy rows more than the output image, where the kernel has size
   (1 + 2 * hx, 1 + 2 * hy).
 
-  The kernel must have an odd number of rows and an odd number of columns.
-
   In standard deviation filtering the kernel must have at least two elements
   set to CPL_BINARY_1, for others at least one element must be set to
   CPL_BINARY_1.
@@ -172,22 +195,24 @@ static cpl_mask * cpl_mask_new_from_matrix(const cpl_matrix *, double, double);
 
   Supported modes:
   CPL_FILTER_MEDIAN:
-      1x1 kernel or no bad pixels & kernel all ones:
+      1x1 kernel or no bad pixels and kernel all ones:
          CPL_BORDER_FILTER, CPL_BORDER_COPY, CPL_BORDER_NOP, CPL_BORDER_CROP.
       Otherwise:
-         CPL_BORDER_FILTER
+         CPL_BORDER_FILTER, CPL_BORDER_COPY, CPL_BORDER_NOP
   CPL_FILTER_AVERAGE:
          CPL_BORDER_FILTER
   CPL_FILTER_AVERAGE_FAST:
          CPL_BORDER_FILTER
   CPL_FILTER_STDEV:
          CPL_BORDER_FILTER
+  CPL_FILTER_STDEV_FAST:
+         CPL_BORDER_FILTER
 
 
    @par Example:
    @code
      cpl_image_filter_mask(filtered, raw, kernel, CPL_FILTER_MEDIAN,
-                                                CPL_BORDER_FILTER);
+                           CPL_BORDER_FILTER);
    @endcode
 
   To shift an image 1 pixel up and 1 pixel right with the CPL_FILTER_MEDIAN
@@ -228,28 +253,31 @@ cpl_error_code cpl_image_filter_mask(cpl_image * self,
                                      cpl_border_mode border)
 {
 
-    const int nsx    = cpl_image_get_size_x(self);
-    const int nsy    = cpl_image_get_size_y(self);
-    const int nx     = cpl_image_get_size_x(other);
-    const int ny     = cpl_image_get_size_y(other);
-    const int mx = cpl_mask_get_size_x(kernel);
-    const int my = cpl_mask_get_size_y(kernel);
-    const int hsizex = mx >> 1;
-    const int hsizey = my >> 1;
+    const cpl_size nsx    = cpl_image_get_size_x(self);
+    const cpl_size nsy    = cpl_image_get_size_y(self);
+    const cpl_size nx     = cpl_image_get_size_x(other);
+    const cpl_size ny     = cpl_image_get_size_y(other);
+    const cpl_size mx = cpl_mask_get_size_x(kernel);
+    const cpl_size my = cpl_mask_get_size_y(kernel);
+    const cpl_size hsizex = mx >> 1;
+    const cpl_size hsizey = my >> 1;
     const cpl_binary * pmask  = cpl_mask_get_data_const(kernel);
     const void       * pother = cpl_image_get_data_const(other);
     const void       * pself  = cpl_image_get_data_const(self);
     /* assert( sizeof(char) == 1 ) */
     const void * polast = (const void*)((const char*)pother
-                                        + nx * ny * cpl_type_get_sizeof
+                                        + (size_t)nx * (size_t)ny
+                                        * cpl_type_get_sizeof
                                         (cpl_image_get_type(other)));
     const void * psrows = (const void*)((const char*)pself
-                                        + nsx * (1+hsizey) * cpl_type_get_sizeof
+                                        + (size_t)nsx * (size_t)(1+hsizey)
+                                        * cpl_type_get_sizeof
                                         (cpl_image_get_type(self)));
     /* pmask may not overlap pself at all */
-    const void * pmlast = (const void*)(pmask + mx * my);
+    const void * pmlast = (const void*)(pmask + (size_t)mx * (size_t)my);
     const void * pslast = (const void*)((const char*)pself
-                                        + nsx * nsy * cpl_type_get_sizeof
+                                        + (size_t)nsx * (size_t)nsy
+                                        * cpl_type_get_sizeof
                                         (cpl_image_get_type(self)));
     cpl_binary       * pbpmself;
     const cpl_binary * badmap;
@@ -262,19 +290,21 @@ cpl_error_code cpl_image_filter_mask(cpl_image * self,
 
     /* Input type 1 */
     void               (*filter_func)(void *, const void *, const cpl_binary *,
-                                      int, int, int, int, cpl_border_mode);
+                                      cpl_size, cpl_size, cpl_size, cpl_size,
+                                      cpl_border_mode);
     /* Input type 2 */
     void               (*filter_fast)(void *, const void *,
-                                      int, int, int, int, cpl_border_mode);
+                                      cpl_size, cpl_size, cpl_size, cpl_size,
+                                      cpl_border_mode);
     /* Input type 3 */
-    void               (*filter_bpm)(void *, cpl_binary **,
+    cpl_error_code     (*filter_bpm)(void *, cpl_binary **,
                                      const void *, const cpl_binary *,
-                                     const cpl_binary *, int, int, int, int,
-                                     cpl_border_mode);
+                                     const cpl_binary *, cpl_size, cpl_size,
+                                     cpl_size, cpl_size, cpl_border_mode);
     /* Input type 4 */
     void               (*filter_full)(void *, cpl_binary **,
                                       const void *, const cpl_binary *,
-                                      int, int, int, int,
+                                      cpl_size, cpl_size, cpl_size, cpl_size,
                                       cpl_border_mode);
     cpl_boolean  same_type = cpl_image_get_type(self)
         == cpl_image_get_type(other);
@@ -290,7 +320,7 @@ cpl_error_code cpl_image_filter_mask(cpl_image * self,
     /* ... unless they have different pixel types, in which case the
            two pixel buffers may not overlap at all */
     if (!(pself >= polast || pother >= (same_type ? psrows : pslast))) {
-        return cpl_error_set_message(cpl_func, CPL_ERROR_UNSUPPORTED_MODE,
+        return cpl_error_set_message_(CPL_ERROR_UNSUPPORTED_MODE,
                                      "pself=%p, polast=%p, pother=%p, "
                                      "psrows=%p, pslast=%p, same_type=%d",
                                      pself, polast, pother, psrows, pslast,
@@ -309,14 +339,15 @@ cpl_error_code cpl_image_filter_mask(cpl_image * self,
     if (filter == CPL_FILTER_STDEV || filter == CPL_FILTER_STDEV_FAST) {
         /* Mask must have at least two non-zero elements */
         const cpl_binary * pmask1 = memchr(pmask, CPL_BINARY_1,
-                                           (size_t)mx * my * sizeof(*pmask));
+                                           (size_t)mx * (size_t)my
+                                           * sizeof(*pmask));
         const cpl_binary * pmask2;
 
         cpl_ensure_code(pmask1 != NULL, CPL_ERROR_DATA_NOT_FOUND);
 
         /* Rely on memchr() to return NULL on zero-size */
         pmask2 = memchr(pmask1 + sizeof(*pmask), CPL_BINARY_1,
-                        ((size_t)mx * my - (1 + pmask1 - pmask))
+                        ((size_t)mx * (size_t)my - (1 + pmask1 - pmask))
                         * sizeof(*pmask));
 
         cpl_ensure_code(pmask2 != NULL, CPL_ERROR_DATA_NOT_FOUND);
@@ -342,14 +373,15 @@ cpl_error_code cpl_image_filter_mask(cpl_image * self,
 
     /* Get and reset all bad pixels, if any */
     pbpmself = self->bpm != NULL
-        ? memset(cpl_mask_get_data(self->bpm), CPL_BINARY_0, (size_t)nsx * nsy)
+        ? memset(cpl_mask_get_data(self->bpm), CPL_BINARY_0,
+                 (size_t)nsx * (size_t)nsy)
         : NULL;
 
     badmap = other->bpm == NULL || cpl_mask_is_empty(other->bpm)
         ? NULL : cpl_mask_get_data_const(other->bpm);
 
     is_full = memchr(pmask, CPL_BINARY_0,
-                     (size_t)mx * my * sizeof(*pmask)) == NULL
+                     (size_t)mx * (size_t)my * sizeof(*pmask)) == NULL
         ? CPL_TRUE : CPL_FALSE;
 
     filter_func = NULL;
@@ -835,12 +867,15 @@ cpl_error_code cpl_image_filter_mask(cpl_image * self,
                    nx, ny, hsizex, hsizey,
                    border);
     } else if (filter_bpm != NULL) {
-        filter_bpm(cpl_image_get_data(self), &pbpmself,
-                   cpl_image_get_data_const(other), badmap,
-                   pmask, nx, ny, hsizex, hsizey,
-                   border);
+
+        if (filter_bpm(cpl_image_get_data(self), &pbpmself,
+                       cpl_image_get_data_const(other), badmap,
+                       pmask, nx, ny, hsizex, hsizey,
+                       border)) {
+            return cpl_error_set_where_();
+        }
     } else {
-        return cpl_error_set(cpl_func, CPL_ERROR_UNSUPPORTED_MODE);
+        return cpl_error_set_(CPL_ERROR_UNSUPPORTED_MODE);
     }
 
     if (self->bpm == NULL && pbpmself != NULL) {
@@ -922,38 +957,41 @@ cpl_error_code cpl_image_filter(cpl_image        * self,
                                 cpl_border_mode    border)
 {
 
-    const int nsx    = cpl_image_get_size_x(self);
-    const int nsy    = cpl_image_get_size_y(self);
-    const int nx     = cpl_image_get_size_x(other);
-    const int ny     = cpl_image_get_size_y(other);
-    const int mx = cpl_matrix_get_ncol(kernel);
-    const int my = cpl_matrix_get_nrow(kernel);
-    const int hsizex = mx >> 1;
-    const int hsizey = my >> 1;
+    const cpl_size nsx    = cpl_image_get_size_x(self);
+    const cpl_size nsy    = cpl_image_get_size_y(self);
+    const cpl_size nx     = cpl_image_get_size_x(other);
+    const cpl_size ny     = cpl_image_get_size_y(other);
+    const cpl_size mx = cpl_matrix_get_ncol(kernel);
+    const cpl_size my = cpl_matrix_get_nrow(kernel);
+    const cpl_size hsizex = mx >> 1;
+    const cpl_size hsizey = my >> 1;
     const double * pkernel  = cpl_matrix_get_data_const(kernel);
     const void   * pother = cpl_image_get_data_const(other);
     const void   * pself  = cpl_image_get_data_const(self);
     /* assert( sizeof(char) == 1 ) */
     const void * polast = (const void*)((const char*)pother
-                                        + nx * ny * cpl_type_get_sizeof
+                                        + (size_t)nx * (size_t)ny
+                                        * cpl_type_get_sizeof
                                         (cpl_image_get_type(other)));
     const void * psrows = (const void*)((const char*)pself
-                                        + nsx * (1+hsizey) * cpl_type_get_sizeof
+                                        + (size_t)nsx * (size_t)(1+hsizey)
+                                        * cpl_type_get_sizeof
                                         (cpl_image_get_type(self)));
     /* pkernel may not overlap pself at all */
     const void * pmlast = (const void*)(pkernel + mx * my);
     const void * pslast = (const void*)((const char*)pself
-                                        + nsx * nsy * cpl_type_get_sizeof
+                                        + (size_t)nsx * (size_t)nsy
+                                        * cpl_type_get_sizeof
                                         (cpl_image_get_type(self)));
     cpl_binary       * pbpmself;
     const cpl_binary * badmap;
-    void               (*filter_func)(void *, cpl_binary **,
+    cpl_error_code     (*filter_func)(void *, cpl_binary **,
                                       const void *, const cpl_binary *,
-                                      const double *, int, int, int, int,
-                                      cpl_border_mode);
+                                      const double *, cpl_size, cpl_size,
+                                      cpl_size, cpl_size, cpl_border_mode);
     cpl_boolean  same_type = cpl_image_get_type(self)
         == cpl_image_get_type(other);
-    int i;
+    cpl_size i;
 
     cpl_ensure_code(self    != NULL, CPL_ERROR_NULL_INPUT);
     cpl_ensure_code(other   != NULL, CPL_ERROR_NULL_INPUT);
@@ -998,7 +1036,8 @@ cpl_error_code cpl_image_filter(cpl_image        * self,
 
     /* Get and reset all bad pixels, if any */
     pbpmself = self->bpm != NULL
-        ? memset(cpl_mask_get_data(self->bpm), CPL_BINARY_0, (size_t)nsx * nsy)
+        ? memset(cpl_mask_get_data(self->bpm), CPL_BINARY_0,
+                 (size_t)nsx * (size_t)nsy)
         : NULL;
 
     badmap = other->bpm == NULL || cpl_mask_is_empty(other->bpm)
@@ -1159,11 +1198,13 @@ cpl_error_code cpl_image_filter(cpl_image        * self,
 
     cpl_ensure_code(filter_func != NULL, CPL_ERROR_UNSUPPORTED_MODE);
 
-    filter_func(cpl_image_get_data(self), &pbpmself,
-                cpl_image_get_data_const(other), badmap,
-                pkernel, nx, ny,
-                hsizex, hsizey,
-                border);
+    if (filter_func(cpl_image_get_data(self), &pbpmself,
+                    cpl_image_get_data_const(other), badmap,
+                    pkernel, nx, ny,
+                    hsizex, hsizey,
+                    border)) {
+        return cpl_error_set_where_();
+    }
 
     if (self->bpm == NULL && pbpmself != NULL) {
         /* A new BPM was created for the output */
@@ -1201,7 +1242,7 @@ cpl_image * cpl_image_filter_linear(const cpl_image  * in,
                          CPL_BORDER_FILTER)) {
         cpl_image_delete(self);
         self = NULL;
-        (void)cpl_error_set(cpl_func, cpl_error_get_code());
+        (void)cpl_error_set_where_();
     }
 
     return self;
@@ -1233,7 +1274,7 @@ cpl_image * cpl_image_filter_morpho(const cpl_image  * in,
                          CPL_BORDER_FILTER)) {
         cpl_image_delete(self);
         self = NULL;
-        (void)cpl_error_set(cpl_func, cpl_error_get_code());
+        (void)cpl_error_set_where_();
     }
 
     return self;
@@ -1271,7 +1312,7 @@ cpl_image * cpl_image_filter_median(const cpl_image  * in,
                               CPL_BORDER_FILTER)) {
         cpl_image_delete(self);
         self = NULL;
-        (void)cpl_error_set(cpl_func, cpl_error_get_code());
+        (void)cpl_error_set_where_();
     }
 
     cpl_mask_delete(mask);
@@ -1312,7 +1353,7 @@ cpl_image * cpl_image_filter_stdev(const cpl_image  * in,
                               CPL_BORDER_FILTER)) {
         cpl_image_delete(self);
         self = NULL;
-        (void)cpl_error_set(cpl_func, cpl_error_get_code());
+        (void)cpl_error_set_where_();
     }
 
     cpl_mask_delete(mask);
@@ -1347,13 +1388,13 @@ static cpl_mask * cpl_mask_new_from_matrix(const cpl_matrix * kernel,
                                            double value, double tol)
 {
 
-    const int      nx   = cpl_matrix_get_ncol(kernel);
-    const int      ny   = cpl_matrix_get_nrow(kernel);
-    const int      n    = nx * ny;
+    const cpl_size nx   = cpl_matrix_get_ncol(kernel);
+    const cpl_size ny   = cpl_matrix_get_nrow(kernel);
+    const cpl_size n    = nx * ny;
     const double * data = cpl_matrix_get_data_const(kernel);
     cpl_mask     * self;
     cpl_binary   * pself;
-    int            i;
+    cpl_size       i;
 
     cpl_ensure(kernel != NULL, CPL_ERROR_NULL_INPUT, NULL);
 
