@@ -1,4 +1,4 @@
-/* $Id: cpl_image_gen.h,v 1.19 2010/11/11 15:28:46 llundin Exp $
+/* $Id: cpl_image_gen.h,v 1.20 2011/07/21 08:03:53 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/11/11 15:28:46 $
- * $Revision: 1.19 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2011/07/21 08:03:53 $
+ * $Revision: 1.20 $
+ * $Name: cpl-6_0 $
  */
 
 #ifndef CPL_IMAGE_GEN_H
@@ -43,10 +43,10 @@ CPL_BEGIN_DECLS
 
 cpl_error_code cpl_image_fill_noise_uniform(cpl_image *, double, double);
 cpl_error_code cpl_image_fill_gaussian(cpl_image *, double, double, double,
-        double, double);
-cpl_error_code cpl_image_fill_polynomial(cpl_image *, 
-        const cpl_polynomial *, double, double, double, double);
-cpl_image * cpl_image_fill_test_create(int, int) CPL_ATTR_ALLOC;
+                                       double, double);
+cpl_error_code cpl_image_fill_polynomial(cpl_image *,  const cpl_polynomial *,
+                                         double, double, double, double);
+cpl_image * cpl_image_fill_test_create(cpl_size, cpl_size) CPL_ATTR_ALLOC;
 
 CPL_END_DECLS
 
