@@ -1,4 +1,4 @@
-/* $Id: cpl_image_iqe.c,v 1.28 2011/01/21 10:11:43 kbanse Exp $
+/* $Id: cpl_image_iqe.c,v 1.32 2012/02/14 16:03:26 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -19,10 +19,10 @@
  */
 
 /*
- * $Author: kbanse $
- * $Date: 2011/01/21 10:11:43 $
- * $Revision: 1.28 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Author: llundin $
+ * $Date: 2012/02/14 16:03:26 $
+ * $Revision: 1.32 $
+ * $Name: cpl-6_0 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -33,25 +33,22 @@
                                    Includes
  -----------------------------------------------------------------------------*/
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <unistd.h>
-#include <math.h>
-#include <assert.h>
-
 #include "cpl_image_iqe.h"
+
+#include "cpl_error_impl.h"
 #include "cpl_bivector.h"
 #include "cpl_image_basic.h"
 #include "cpl_image_defs.h"
 #include "cpl_math_const.h"
 
+#include <math.h>
+
 /*-----------------------------------------------------------------------------
                             Private Function prototypes
                             - more are declared below
  -----------------------------------------------------------------------------*/
 
-static int cpl_iqe(float *, int, int, float *, float *) ;
+static int cpl_iqe(float *, int, int, float *, float *);
 
 /*-----------------------------------------------------------------------------
                             Function codes
@@ -62,15 +59,15 @@ static int cpl_iqe(float *, int, int, float *, float *) ;
   @ingroup cpl_image
   @brief    Compute an image quality estimation for an object
   @param    in      the input image
-  @param    llx     
+  @param    llx
   @param    lly     the zone to analyse ((1, 1) for the first pixel)
   @param    urx     The zone must be at least 4 by 4 pixels
-  @param    ury     
+  @param    ury
   @return   a newly allocated cpl_bivector containing the results or
               NULL in error case.
 
   This function makes internal use of the iqe() MIDAS function (called
-  here cpl_iqe()) written by P. Grosbol. Refer to the MIDAS documentation 
+  here cpl_iqe()) written by P. Grosbol. Refer to the MIDAS documentation
   for more details. This function has proven to give good results over
   the years when called from RTD. The goal is to provide the exact same
   functionality in CPL as the one provided in RTD. The code is simply
@@ -90,10 +87,10 @@ static int cpl_iqe(float *, int, int, float *, float *) ;
 
   The bad pixels map of the image is not taken into account.
   The input image must be of type float.
-  
+
   Possible #_cpl_error_code_ set in this function:
   - CPL_ERROR_NULL_INPUT if (one of) the input pointer(s) is NULL
-  - CPL_ERROR_ILLEGAL_INPUT if the specified zone is not valid or if the 
+  - CPL_ERROR_ILLEGAL_INPUT if the specified zone is not valid or if the
   computation fails on this zone
   - CPL_ERROR_INVALID_TYPE if the input image has the wrong type
 
@@ -104,60 +101,71 @@ static int cpl_iqe(float *, int, int, float *, float *) ;
   case of noiseless data with a box 7 times the sigma).
  */
 /*----------------------------------------------------------------------------*/
-cpl_bivector * cpl_image_iqe(
-        const cpl_image *   in,
-        int                 llx,
-        int                 lly,
-        int                 urx,
-        int                 ury)
+cpl_bivector * cpl_image_iqe(const cpl_image * in,
+                             cpl_size          llx,
+                             cpl_size          lly,
+                             cpl_size          urx,
+                             cpl_size          ury)
 {
-    cpl_image       *   extracted ;
+    cpl_image       *   extracted;
     float               parm[8],
                         sdev[8];
-    float           *   pextracted ;
-    cpl_bivector    *   res ;
-    double          *   res_x ;
-    double          *   res_y ;
+    float           *   pextracted;
+    cpl_bivector    *   res;
+    double          *   res_x;
+    double          *   res_y;
+    int                 iqe_error;
+    cpl_size            nx, ny;
+    int                 inx, iny;
 
     /* Check entries */
     cpl_ensure(in,                         CPL_ERROR_NULL_INPUT,    NULL);
     cpl_ensure(in->type == CPL_TYPE_FLOAT, CPL_ERROR_INVALID_TYPE,  NULL);
-    cpl_ensure(urx >= llx + 3, CPL_ERROR_ILLEGAL_INPUT,             NULL);
-    cpl_ensure(ury >= lly + 3, CPL_ERROR_ILLEGAL_INPUT,             NULL);
+    cpl_ensure(urx >= llx + 3,             CPL_ERROR_ILLEGAL_INPUT, NULL);
+    cpl_ensure(ury >= lly + 3,             CPL_ERROR_ILLEGAL_INPUT, NULL);
 
     /* Extract the image */
-    extracted = cpl_image_extract(in, llx, lly, urx, ury) ;
+    extracted = cpl_image_extract(in, llx, lly, urx, ury);
     cpl_ensure(extracted, CPL_ERROR_ILLEGAL_INPUT, NULL);
 
-    /* Get the pointer to the data */
-    pextracted = cpl_image_get_data_float(extracted) ;
-    
-    /* Call the cpl_iqe() function */
-    if (cpl_iqe(pextracted, extracted->nx, extracted->ny, parm, sdev) != 0) {
-        cpl_image_delete(extracted) ;
-        cpl_ensure(0, CPL_ERROR_ILLEGAL_INPUT, NULL);
+    nx = cpl_image_get_size_x(extracted);
+    ny = cpl_image_get_size_y(extracted);
+
+    inx = (int)nx;
+    iny = (int)ny;
+
+    if ((cpl_size)inx != nx || (cpl_size)iny != ny) {
+        cpl_image_delete(extracted);
+        (void)cpl_error_set_(CPL_ERROR_ILLEGAL_INPUT);
+        return NULL;
     }
-    cpl_image_delete(extracted) ;
+
+    /* Get the pointer to the data */
+    pextracted = cpl_image_get_data_float(extracted);
+
+    iqe_error = cpl_iqe(pextracted, inx, iny, parm, sdev);
+    cpl_image_delete(extracted);
+    cpl_ensure(!iqe_error, CPL_ERROR_ILLEGAL_INPUT, NULL);
 
     /* Create the result cpl_bivector */
-    res = cpl_bivector_new(7) ;
-    res_x = cpl_bivector_get_x_data(res) ;
-    res_y = cpl_bivector_get_y_data(res) ;
-    res_x[0] = llx + parm[0] ;
-    res_y[0] = sdev[0] ;
-    res_x[1] = lly + parm[2] ;
-    res_y[1] = sdev[2] ;
-    res_x[2] = parm[1] ;
-    res_y[2] = sdev[1] ;
-    res_x[3] = parm[3] ;
-    res_y[3] = sdev[3] ;
-    res_x[4] = parm[4] ;
-    res_y[4] = sdev[4] ;
-    res_x[5] = parm[5] ;
-    res_y[5] = sdev[5] ;
-    res_x[6] = parm[6] ;
-    res_y[6] = sdev[6] ;
-    return res ;
+    res = cpl_bivector_new(7);
+    res_x = cpl_bivector_get_x_data(res);
+    res_y = cpl_bivector_get_y_data(res);
+    res_x[0] = (double)llx + parm[0];
+    res_y[0] = sdev[0];
+    res_x[1] = (double)lly + parm[2];
+    res_y[1] = sdev[2];
+    res_x[2] = parm[1];
+    res_y[2] = sdev[1];
+    res_x[3] = parm[3];
+    res_y[3] = sdev[3];
+    res_x[4] = parm[4];
+    res_y[4] = sdev[4];
+    res_x[5] = parm[5];
+    res_y[5] = sdev[5];
+    res_x[6] = parm[6];
+    res_y[6] = sdev[6];
+    return res;
 }
 
 /* Declare local functions private to avoid namespace conflicts */
@@ -209,7 +217,7 @@ static     double    yi[9];
 #ifdef _OPENMP
 #pragma omp threadprivate(pval,pwght,mxx,mp,w,xi,yi)
 #endif
- 
+
 
 static int cpl_iqe(
 /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
@@ -260,27 +268,27 @@ if ((err=iqesec(pfm, mx, my, bgv, est, sec, winsize))) return -3;
 parm[4] = CPL_MATH_DEG_RAD*sec[5];  /* Radian to Degrees      */
 
 if ((err=iqefit(pfm, mx, my, bgv, sec, ap, cv, winsize))<0) return -4;
-parm[0] = ap[1]; 
+parm[0] = ap[1];
 sdev[0] = cv[1];
 parm[1] = CPL_MATH_FWHM_SIG*ap[2];  /* Sigma to FWHM */
 sdev[1] = CPL_MATH_FWHM_SIG*cv[2];  /* Sigma to FWHM */
-parm[2] = ap[3]; 
+parm[2] = ap[3];
 sdev[2] = cv[3];
 parm[3] = CPL_MATH_FWHM_SIG*ap[4];  /* Sigma to FWHM */
 sdev[3] = CPL_MATH_FWHM_SIG*cv[4];  /* Sigma to FWHM */
-parm[4] = fmod(CPL_MATH_DEG_RAD*ap[5]+180.0, 180.0); 
+parm[4] = fmod(CPL_MATH_DEG_RAD*ap[5]+180.0, 180.0);
 sdev[4] = CPL_MATH_DEG_RAD*cv[5];  /* Radian to Degrees      */
 if (sdev[4] > 180.) sdev[4] = 180.0;        /* max is: Pi */
-parm[5] = ap[0]; 
+parm[5] = ap[0];
 sdev[5] = cv[0];
 
 return 0;
 }
- 
+
 static int iqebgv(
 /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 .PURPOSE   Estimate background level for subimage
-.RETURN    status,  0: OK, -1:no buffer space, -2: no points left 
+.RETURN    status,  0: OK, -1:no buffer space, -2: no points left
 ------------------------------------------------------------------------*/
 float      *pfm,
 int        mx,
@@ -313,7 +321,7 @@ int        *nbg
 /* extrat edge of matrix from each corner  */
 
   nt = 0;
-  pf = pfb; pw = pwb;
+  pf = pfb;
   for (m=0; m<ms; m++) {
      pf0 = pfs0; pf1 = pfs1; pf2 = pfs2; pf3 = pfs3;
      for (n=0; n<ns; n++) {
@@ -332,8 +340,8 @@ int        *nbg
 
 /*  skip all elements with zero weight and sort clean array */
 
-  pf = pf0 = pfb; pw = pwb;
-  n = nt; mt = 0;
+  pf0 = pfb; pw = pwb;
+  n = nt;
      mt = nt;
      while (n--) *pw++ = 1.0;
   hsort(mt, pfb);
@@ -374,7 +382,7 @@ int        *nbg
 
   return 0;
 }
- 
+
 static int iqemnt(
 /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 .PURPOSE   Find center of object and do simple moment analysis
@@ -402,7 +410,7 @@ double   val, x, y, dv, xm, ym;
 double   am, ax, ay, axx, ayy, axy;
 
 /* Missing initialization fixed */
-ndx = ndy = 0 ;
+ndx = ndy = 0;
 av = 0.0;
 
 dv = 5.0*bgs;
@@ -419,9 +427,9 @@ while (nt--)
    {
    if (estm9p(pfm, mx, my, nxc, nyc, &av, &dx, &dy)) break;
 
-   if (n) 
+   if (n)
       n = 0;
-   else 
+   else
       {
       if (dx*ndx<0.0) nx = 0;
       if (dy*ndy<0.0) ny = 0;
@@ -438,7 +446,7 @@ while (nt--)
 
 y = 0.0;
 nt = 0; ny = my;
-pf = pfm; 
+pf = pfm;
 ax = ay = 0.0;
 while (ny--)
    {
@@ -447,7 +455,7 @@ while (ny--)
    while (nx--)
       {
       val = *pf++ - bgv;
-      if (dv<val) 
+      if (dv<val)
          {
          ax  += x;
          ay  += y;
@@ -487,9 +495,9 @@ while (n--)
    k = kn;
    if (!ki && ks==-1)
       {
-      if (nx) 
+      if (nx)
          nx = 0;
-      else 
+      else
          break;
       }
    ioff = (ki) ? ks : ks*mx;
@@ -503,7 +511,7 @@ while (n--)
       if ((psize < 0) || (psize > winsize)) break;
 
       val = *pf - bgv;
-      if (dv<val) 
+      if (dv<val)
          {
          am  += val;
          ax  += val*x;
@@ -533,11 +541,11 @@ amm[0] = pfm[nx+ny*mx] - bgv;
 
 return 0;
 }
- 
+
 static int estm9p(
 /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 .PURPOSE   Estimate parameters for 3x3 pixel region
-.RETURN    status, 0:OK, -1:out of range, 
+.RETURN    status, 0:OK, -1:out of range,
 ------------------------------------------------------------------------*/
 float      *pfm,
 int        mx,
@@ -561,12 +569,12 @@ if (nx<1 || mx<nx-2 || ny<1 || my<ny-2) return -1;
 /* extract region into local array and generate a rank index for it */
 
 iy = 3;
-pfb = fb; pwb = wb; 
+pfb = fb; pwb = wb;
 pfm += nx-1 + mx*(ny-1);
-   while(iy--) 
+   while(iy--)
       {
       ix = 3;
-      while (ix--) 
+      while (ix--)
          {
          *pfb++ = *pfm++;
          *pwb++ = 1.0;
@@ -593,7 +601,7 @@ a = am = 0.0; ix = iy = 0;
 for (n=0; n<9; n +=3) {
    if (0.0<wb[n]) a += fb[n], ix++;
    if (0.0<wb[n+2]) am +=fb[n+2], iy++;
-   } 
+   }
 *dx = 0.5*(am/iy - a/ix);
 
 a = am = 0.0; ix = iy = 0;
@@ -606,7 +614,7 @@ for (n=0; n<3; n++) {
 
 return 0;
 }
- 
+
 static int iqesec(
 /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 .PURPOSE   Perform a sector analysis of object. Estimates for center and
@@ -631,7 +639,7 @@ int      n, k, ki, ks, kn, nxc, nyc, ioff, idx;
 int      ns[8], psize;
 float    *pf, f;
 double   x, y, xm, ym, xc, yc, dx, dy;
-double    r, rl, rh, sb[8], a1r, a1i, a2r, a2i;
+double    r, rl, rh, sb[8], a2r, a2i;
 
 /* initiate basic variables    */
 
@@ -650,7 +658,7 @@ double    r, rl, rh, sb[8], a1r, a1i, a2r, a2i;
 
   nxc = floor(x+0.5); nyc = floor(y+0.5);
   ioff = nxc+mx*nyc;
-  pf = pfm + ioff; 
+  pf = pfm + ioff;
 
   ki = ks = kn = 1;
   while (n--) {
@@ -660,7 +668,7 @@ double    r, rl, rh, sb[8], a1r, a1i, a2r, a2i;
     if (ki) x += ks; else y += ks;
     if (x<0.0 || y<0.0 || xm<x || ym<y) break;
 
-    pf += ioff; 
+    pf += ioff;
         psize = pf - pfm;
         if ((psize < 0) || (psize > winsize)) break;
 
@@ -683,8 +691,6 @@ double    r, rl, rh, sb[8], a1r, a1i, a2r, a2i;
      sb[n] /= ns[n];
    }
 
-  a1r = sb[0]+hsq2*sb[1]-hsq2*sb[3]-sb[4]-hsq2*sb[5]+hsq2*sb[7];
-  a1i = hsq2*sb[1]+sb[2]+hsq2*sb[3]-hsq2*sb[5]-sb[6]-hsq2*sb[7];
   a2r = sb[0]-sb[2]+sb[4]-sb[6];
   a2i = sb[1]-sb[3]+sb[5]-sb[7];
 
@@ -752,7 +758,7 @@ if ((psize < 0) || (psize > winsize)) { free (pfb); return -99; }
 
 pf = pfb; pw = pwb;
 iy = ny;
-   while (iy--) 
+   while (iy--)
       {
       ix = nx;
       while (ix--)
@@ -849,7 +855,6 @@ float      *dyda
   int      n;
   double   ff, sum, ci, si;
   double   xc, yc, xx, yy, x, y;
-  double   xm5, xp5, ym5, yp5;
 
   if (idx<0 || mp<=idx) return -1;              /* check index          */
   if (pwght && pwght[idx]<0.0) return 1;        /* check if valid pixel */
@@ -858,11 +863,6 @@ float      *dyda
   xc = (double) (idx%mxx) - a[1];
   yc = (double) (idx/mxx) - a[3];
 
-  xm5 = (double) (idx%mxx) - a[1] - 0.5;
-  xp5 = xm5 + 1.0;
-  ym5 = (double) (idx/mxx) - a[3] - 0.5;
-  yp5 = ym5 + 1.0;
-
   *val = pval[idx];
   *psig = (pwght) ? pwght[idx] : 1.0;
   si = sin(a[5]);
@@ -957,7 +957,7 @@ static      double            ochisq;
 
 static int mrqmin(
 /*++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
-.PURPOSE   
+.PURPOSE
 .RETURN    status,  0: OK, -1: Bad permutation LISTA 1,
                    -2: Bad permutation LISTA 2, -3: too many variables,
                    -4: No points (chisq<=0), -5: error in matrix inversion
@@ -1091,9 +1091,9 @@ int        m
   int      i, icol, irow, j, k, l, ll;
   int      indxc[MMA], indxr[MMA], ipiv[MMA];
   double   big, dum, pivinv;
-  
+
   /* Missing initialization fixed */
-  icol = irow = 0 ;
+  icol = irow = 0;
 
   if (MMA<n) return -3;
   for (j=0; j<n; j++) ipiv[j]=0;
@@ -1140,7 +1140,7 @@ int        m
      if (indxr[l] != indxc[l])
        for (k=0; k<n; k++)
      SWAP(a[k+indxr[l]*n],a[k+indxc[l]*n]);
-   }    
+   }
 
   return 0;
 }
