@@ -1,4 +1,4 @@
-/* $Id: cpl_image_resample.c,v 1.52 2010/11/11 09:23:18 llundin Exp $
+/* $Id: cpl_image_resample.c,v 1.56 2012/03/13 15:46:03 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/11/11 09:23:18 $
- * $Revision: 1.52 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2012/03/13 15:46:03 $
+ * $Revision: 1.56 $
+ * $Name: cpl-6_0 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -33,19 +33,19 @@
                                    Includes
  -----------------------------------------------------------------------------*/
 
+#include "cpl_tools.h"
+#include "cpl_memory.h"
+#include "cpl_image_resample.h"
+#include "cpl_mask.h"
+#include "cpl_error_impl.h"
+
+#include "cpl_image_defs.h"
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <math.h>
 #include <float.h>
-#include <assert.h>
-
-#include "cpl_tools.h"
-#include "cpl_memory.h"
-#include "cpl_image_resample.h"
-#include "cpl_mask.h"
-
-#include "cpl_image_defs.h"
 
 /*-----------------------------------------------------------------------------
                                    Defines
@@ -66,7 +66,7 @@
 
 inline static double cpl_image_get_interpolated_double(const double *,
                                                        const cpl_binary *,
-                                                       int, int,
+                                                       cpl_size, cpl_size,
                                                        double, double,
                                                        double, double,
                                                        double *,
@@ -77,7 +77,7 @@ inline static double cpl_image_get_interpolated_double(const double *,
 
 inline static double cpl_image_get_interpolated_float(const float *,
                                                       const cpl_binary *,
-                                                      int, int,
+                                                      cpl_size, cpl_size,
                                                       double, double,
                                                       double, double,
                                                       double *,
@@ -88,7 +88,7 @@ inline static double cpl_image_get_interpolated_float(const float *,
 
 inline static double cpl_image_get_interpolated_int(const int *,
                                                     const cpl_binary *,
-                                                    int, int,
+                                                    cpl_size, cpl_size,
                                                     double, double,
                                                     double, double,
                                                     double *,
@@ -180,12 +180,12 @@ cpl_error_code cpl_image_warp(
     const double    *   pxprof;
     const double    *   pyprof;
     double          *   yweight;
-    int                 ixprolen;
-    int                 iyprolen;
-    int                 pos;
+    cpl_size            ixprolen;
+    cpl_size            iyprolen;
+    cpl_size            pos;
     const double    *   pdeltax;
     const double    *   pdeltay;
-    int                 i, j;
+    cpl_size            i, j;
 
     /* Check entries */
     cpl_ensure_code(out,    CPL_ERROR_NULL_INPUT); 
@@ -218,10 +218,10 @@ cpl_error_code cpl_image_warp(
     hasbad = 0;
     pxprof  = cpl_vector_get_data_const(xprofile);
     pyprof  = cpl_vector_get_data_const(yprofile);
-    sqyxratio = sqyradius/sqxradius;
-    xtabsperpix = (ixprolen-1)/xradius;
-    ytabsperpix = (iyprolen-1)/yradius;
-    yweight = cpl_malloc((1 + (int)(2* yradius)) * sizeof(double));
+    sqyxratio = sqyradius / sqxradius;
+    xtabsperpix = (double)(ixprolen-1)/xradius;
+    ytabsperpix = (double)(iyprolen-1)/yradius;
+    yweight = cpl_malloc((1 + (size_t)(2.0 * yradius)) * sizeof(double));
 
     /* Access the offsets */
     pdeltax = cpl_image_get_data_double_const(deltax);
@@ -242,7 +242,7 @@ cpl_error_code cpl_image_warp(
 #include "cpl_image_resample_body.h"
 #undef CPL_CLASS
     default:
-        cpl_error_set(cpl_func, CPL_ERROR_INVALID_TYPE);
+        cpl_error_set_(CPL_ERROR_INVALID_TYPE);
 
     }
     cpl_free(yweight);
@@ -314,25 +314,25 @@ cpl_error_code cpl_image_warp_polynomial(
 {
     double       x, y;
     cpl_vector * val = NULL;
-    double * pval;
+    double     * pval;
     cpl_mask   * bad = NULL;
     cpl_binary * pbad;
     double value, confidence;
-    int          i, j;
+    cpl_size     i, j;
     int hasbad = 0;
 
     const double sqxradius = xradius * xradius;
     const double sqyradius = yradius * yradius;
     double sqyxratio;
 
-    double       xtabsperpix;
-    double       ytabsperpix;
+    double         xtabsperpix;
+    double         ytabsperpix;
     const double * pxprof;
     const double * pyprof;
-    double * yweight;
+    double       * yweight;
 
-    int ixprolen;
-    int iyprolen;
+    cpl_size ixprolen;
+    cpl_size iyprolen;
 
 
     /* Check entries */
@@ -361,10 +361,10 @@ cpl_error_code cpl_image_warp_polynomial(
 
     sqyxratio = sqyradius/sqxradius;
 
-    xtabsperpix = (ixprolen-1)/xradius;
-    ytabsperpix = (iyprolen-1)/yradius;
+    xtabsperpix = (double)(ixprolen-1) / xradius;
+    ytabsperpix = (double)(iyprolen-1) / yradius;
 
-    yweight = cpl_malloc((1 + (int)(2* yradius)) * sizeof(double));
+    yweight = cpl_malloc((1 + (size_t)(2.0 * yradius)) * sizeof(double));
 
 
     val = cpl_vector_new(2);
@@ -384,7 +384,7 @@ cpl_error_code cpl_image_warp_polynomial(
 #include "cpl_image_resample_body.h"
 #undef CPL_CLASS
     default:
-        cpl_error_set(cpl_func, CPL_ERROR_INVALID_TYPE);
+        cpl_error_set_(CPL_ERROR_INVALID_TYPE);
 
     }
 
@@ -425,12 +425,12 @@ cpl_error_code cpl_image_warp_polynomial(
 */
 /*----------------------------------------------------------------------------*/
 cpl_image *cpl_image_extract_subsample(const cpl_image *image,
-                                       int              xstep,
-                                       int              ystep)
+                                       cpl_size         xstep,
+                                       cpl_size         ystep)
 {
     cpl_image *out_im;
-    int        new_nx, new_ny;
-    int        i, j;
+    cpl_size   new_nx, new_ny;
+    cpl_size   i, j;
 
 
     cpl_ensure(image != NULL, CPL_ERROR_NULL_INPUT, NULL);
@@ -451,7 +451,8 @@ cpl_image *cpl_image_extract_subsample(const cpl_image *image,
 #include "cpl_image_resample_body.h"
 #undef CPL_CLASS
         default:
-            cpl_ensure(0, CPL_ERROR_INVALID_TYPE, NULL);
+            (void)cpl_error_set_(CPL_ERROR_INVALID_TYPE);
+            return NULL;
     }
 
     /* Sub sample the bad pixel map too */
@@ -502,16 +503,16 @@ cpl_image *cpl_image_extract_subsample(const cpl_image *image,
 */
 /*----------------------------------------------------------------------------*/
 cpl_image *cpl_image_rebin(const cpl_image *image,
-                           int              xstart,
-                           int              ystart,
-                           int              xstep,
-                           int              ystep)
+                           cpl_size         xstart,
+                           cpl_size         ystart,
+                           cpl_size         xstep,
+                           cpl_size         ystep)
 {
-    cpl_image *out_im;
-    int        nx, ny;
-    int        new_nx, new_ny;
-    int        old_nx, old_ny;
-    int        i, j;
+    cpl_image     *out_im;
+    cpl_size       nx, ny;
+    cpl_size       new_nx, new_ny;
+    const cpl_size old_nx = cpl_image_get_size_x(image);
+    cpl_size       i, j;
 
 
     cpl_ensure(image != NULL, CPL_ERROR_NULL_INPUT, NULL);
@@ -529,8 +530,6 @@ cpl_image *cpl_image_rebin(const cpl_image *image,
     nx = new_nx * xstep + xstart - 1;
     ny = new_ny * ystep + ystart - 1;
 
-    old_nx = cpl_image_get_size_x(image);
-    old_ny = cpl_image_get_size_y(image);
 
     switch (image->type) {
 #define CPL_CLASS CPL_CLASS_DOUBLE
@@ -543,7 +542,8 @@ cpl_image *cpl_image_rebin(const cpl_image *image,
 #include "cpl_image_resample_body.h"
 #undef CPL_CLASS
         default:
-            cpl_ensure(0, CPL_ERROR_INVALID_TYPE, NULL);
+            (void)cpl_error_set_(CPL_ERROR_INVALID_TYPE);
+            return NULL;
     }
 
     /* Propagate the bad pixel map if present */
@@ -551,7 +551,7 @@ cpl_image *cpl_image_rebin(const cpl_image *image,
     if (image->bpm != NULL) {
         const cpl_binary *pin;
         cpl_binary       *pout;
-        int               pos;
+        cpl_size          pos;
 
         out_im->bpm = cpl_mask_new(new_nx, new_ny);
         pin = cpl_mask_get_data_const(image->bpm);
@@ -669,10 +669,14 @@ double cpl_image_get_interpolated(const cpl_image  * source,
 
     double xtabsperpix, ytabsperpix;
 
-    const int nx = cpl_image_get_size_x(source);
-    const int ny = cpl_image_get_size_y(source);
-    const int ixprolen = cpl_vector_get_size(xprofile);
-    const int iyprolen = cpl_vector_get_size(yprofile);
+    const cpl_size nx       = cpl_image_get_size_x(source);
+    const cpl_size ny       = cpl_image_get_size_y(source);
+    const cpl_size ixprolen = cpl_vector_get_size(xprofile);
+    const cpl_size iyprolen = cpl_vector_get_size(yprofile);
+    /* Use the stack except for ridiculously large radii that would use
+       too much. A negative Y radius is cast to a very large number */
+    const size_t    ywsize   = 1 + (size_t)(2.0 * yradius);
+    double syweight[ywsize < 200 ? ywsize : 1];
     double * yweight;
     double value;
 
@@ -687,12 +691,12 @@ double cpl_image_get_interpolated(const cpl_image  * source,
     cpl_ensure(xradius > 0,    CPL_ERROR_ILLEGAL_INPUT, -5);
     cpl_ensure(yradius > 0,    CPL_ERROR_ILLEGAL_INPUT, -6);
 
-    xtabsperpix = (ixprolen-1)/xradius;
-    ytabsperpix = (iyprolen-1)/yradius;
+    xtabsperpix = (double)(ixprolen-1) / xradius;
+    ytabsperpix = (double)(iyprolen-1) / yradius;
 
     sqyxratio = sqyradius/sqxradius;
 
-    yweight = cpl_malloc((1 + (int)(2* yradius)) * sizeof(double));
+    yweight = ywsize < 200 ? syweight : cpl_malloc(ywsize * sizeof(double));
 
     switch (source->type) {
     case CPL_TYPE_DOUBLE:
@@ -736,11 +740,12 @@ double cpl_image_get_interpolated(const cpl_image  * source,
 
     default:
         value = 0;
-        cpl_error_set(cpl_func, CPL_ERROR_INVALID_TYPE);
+        cpl_error_set_(CPL_ERROR_INVALID_TYPE);
 
     }
 
-    cpl_free(yweight);
+    if (yweight != syweight)
+        cpl_free(yweight);
 
     return value;
 
@@ -813,20 +818,21 @@ cpl_error_code cpl_image_fill_jacobian_polynomial(cpl_image *out,
     double         *ddata;
     float          *fdata;
 
-    int             nx, ny;
-    int             i, j;
+    cpl_size        nx, ny;
+    cpl_size        i, j;
+    cpl_error_code error = CPL_ERROR_NONE;
 
 
     if (out == NULL || poly_x == NULL || poly_y == NULL)
-        return cpl_error_set(cpl_func, CPL_ERROR_NULL_INPUT);
+        return cpl_error_set_(CPL_ERROR_NULL_INPUT);
 
     if (cpl_polynomial_get_dimension(poly_x) != 2 ||
         cpl_polynomial_get_dimension(poly_y) != 2)
-        return cpl_error_set(cpl_func, CPL_ERROR_ILLEGAL_INPUT);
+        return cpl_error_set_(CPL_ERROR_ILLEGAL_INPUT);
 
     if (cpl_image_get_type(out) != CPL_TYPE_FLOAT &&
         cpl_image_get_type(out) != CPL_TYPE_DOUBLE)
-        return cpl_error_set(cpl_func, CPL_ERROR_INVALID_TYPE);
+        return cpl_error_set_(CPL_ERROR_INVALID_TYPE);
 
 
     /*
@@ -859,22 +865,22 @@ cpl_error_code cpl_image_fill_jacobian_polynomial(cpl_image *out,
     case CPL_TYPE_FLOAT:
         fdata = cpl_image_get_data_float(out);
         for (j=0; j < ny; j++) {
-            pval[1] = j + 1;
+            pval[1] = (double)(j + 1);
             for (i=0; i < nx; i++) {
-                pval[0] = i + 1;
-                *fdata++ = cpl_polynomial_eval(dxdu, val)
-                         * cpl_polynomial_eval(dydv, val)
-                         - cpl_polynomial_eval(dxdv, val)
-                         * cpl_polynomial_eval(dydu, val);
+                pval[0] = (double)(i + 1);
+                *fdata++ = (float)(cpl_polynomial_eval(dxdu, val)
+                                   * cpl_polynomial_eval(dydv, val)
+                                   - cpl_polynomial_eval(dxdv, val)
+                                   * cpl_polynomial_eval(dydu, val));
             }
         }
         break;
     case CPL_TYPE_DOUBLE:
         ddata = cpl_image_get_data_double(out);
         for (j=0; j < ny; j++) {
-            pval[1] = j + 1;
+            pval[1] = (double)(j + 1);
             for (i=0; i < nx; i++) {
-                pval[0] = i + 1;
+                pval[0] = (double)(i + 1);
                 *ddata++ = cpl_polynomial_eval(dxdu, val)
                          * cpl_polynomial_eval(dydv, val)
                          - cpl_polynomial_eval(dxdv, val)
@@ -883,12 +889,8 @@ cpl_error_code cpl_image_fill_jacobian_polynomial(cpl_image *out,
         }
         break;
     default:
-
-        /*
-         * Impossible to reach this point. An assert(0) here
-         * would be in order, but what the heck...
-         */
-
+        /* It is an error in CPL to reach this point */
+        error = CPL_ERROR_UNSPECIFIED;
         break;
     }
 
@@ -903,10 +905,10 @@ cpl_error_code cpl_image_fill_jacobian_polynomial(cpl_image *out,
      * Ensure the scale factor is positive...
      */
 
-    cpl_image_abs(out);
-
-    return CPL_ERROR_NONE;
+    if (!error) error = cpl_image_abs(out);
 
+    /* Propagate error, if any */
+    return cpl_error_set_(error);
 }
 
 
@@ -996,20 +998,21 @@ cpl_error_code cpl_image_fill_jacobian(cpl_image *out,
     double       *ddata;
     float        *fdata;
 
-    int           nx, ny, npix;
-    int           i, j, pos;
+    cpl_size      nx, ny, npix;
+    cpl_size      i, j, pos;
+    cpl_error_code error = CPL_ERROR_NONE;
 
 
     if (out == NULL || deltax == NULL || deltay == NULL)
-        return cpl_error_set(cpl_func, CPL_ERROR_NULL_INPUT);
+        return cpl_error_set_(CPL_ERROR_NULL_INPUT);
 
     if (cpl_image_get_type(out) != CPL_TYPE_FLOAT &&
         cpl_image_get_type(out) != CPL_TYPE_DOUBLE)
-        return cpl_error_set(cpl_func, CPL_ERROR_INVALID_TYPE);
+        return cpl_error_set_(CPL_ERROR_INVALID_TYPE);
 
     if (cpl_image_get_type(deltax) != CPL_TYPE_DOUBLE ||
         cpl_image_get_type(deltay) != CPL_TYPE_DOUBLE)
-        return cpl_error_set(cpl_func, CPL_ERROR_ILLEGAL_INPUT);
+        return cpl_error_set_(CPL_ERROR_ILLEGAL_INPUT);
 
     nx = cpl_image_get_size_x(out);
     ny = cpl_image_get_size_y(out);
@@ -1018,7 +1021,7 @@ cpl_error_code cpl_image_fill_jacobian(cpl_image *out,
         nx != cpl_image_get_size_x(deltay) ||
         ny != cpl_image_get_size_y(deltax) ||
         ny != cpl_image_get_size_y(deltay)) {
-        return cpl_error_set(cpl_func, CPL_ERROR_ILLEGAL_INPUT);
+        return cpl_error_set_(CPL_ERROR_ILLEGAL_INPUT);
     }
 
 
@@ -1066,7 +1069,8 @@ cpl_error_code cpl_image_fill_jacobian(cpl_image *out,
     case CPL_TYPE_FLOAT:
         fdata = cpl_image_get_data_float(out);
         for (i = 0; i < npix; i++) {
-            *fdata++ = (*d_dxdu++) * (*d_dydv++) - (*d_dxdv++) * (*d_dydu++);
+            *fdata++ = (float)((*d_dxdu++) * (*d_dydv++)
+                               - (*d_dxdv++) * (*d_dydu++));
         }
         break;
     case CPL_TYPE_DOUBLE:
@@ -1077,11 +1081,8 @@ cpl_error_code cpl_image_fill_jacobian(cpl_image *out,
         break;
     default:
 
-        /*
-         * Impossible to reach this point. An assert(0) here
-         * would be in order, but what the heck...
-         */
-
+        /* It is an error in CPL to reach this point */
+        error = CPL_ERROR_UNSPECIFIED;
         break;
     }
 
@@ -1095,9 +1096,10 @@ cpl_error_code cpl_image_fill_jacobian(cpl_image *out,
      * Ensure the scale factor is positive...
      */
 
-    cpl_image_abs(out);
+    if (!error) error = cpl_image_abs(out);
 
-    return CPL_ERROR_NONE;
+    /* Propagate error, if any */
+    return cpl_error_set_(error);
 
 }
 
