@@ -1,4 +1,4 @@
-/* $Id: cpl_image_stats.c,v 1.88 2010/12/23 11:01:20 llundin Exp $
+/* $Id: cpl_image_stats.c,v 1.90 2011/08/17 16:39:00 cgarcia Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -19,10 +19,10 @@
  */
 
 /*
- * $Author: llundin $
- * $Date: 2010/12/23 11:01:20 $
- * $Revision: 1.88 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Author: cgarcia $
+ * $Date: 2011/08/17 16:39:00 $
+ * $Revision: 1.90 $
+ * $Name: cpl-6_0 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -125,10 +125,10 @@
  */ 
 /*----------------------------------------------------------------------------*/
 double cpl_image_get_min_window(const cpl_image * image,
-                                int               llx,
-                                int               lly,
-                                int               urx,
-                                int               ury)
+                                cpl_size          llx,
+                                cpl_size          lly,
+                                cpl_size          urx,
+                                cpl_size          ury)
 {
     CPL_IMAGE_GET_STATS_WINDOW_DOUBLE_BODY(min, MIN);
 }
@@ -168,10 +168,10 @@ double cpl_image_get_min(const cpl_image * image)
  */ 
 /*----------------------------------------------------------------------------*/
 double cpl_image_get_max_window(const cpl_image * image,
-                                int               llx,
-                                int               lly,
-                                int               urx,
-                                int               ury)
+                                cpl_size          llx,
+                                cpl_size          lly,
+                                cpl_size          urx,
+                                cpl_size          ury)
 {
     CPL_IMAGE_GET_STATS_WINDOW_DOUBLE_BODY(max, MAX);
 }
@@ -205,10 +205,10 @@ double cpl_image_get_max(const cpl_image * image)
 /*----------------------------------------------------------------------------*/
 double cpl_image_get_mean_window(
         const cpl_image   *   image,
-        int             llx,
-        int             lly,
-        int             urx,
-        int             ury)
+        cpl_size        llx,
+        cpl_size        lly,
+        cpl_size        urx,
+        cpl_size        ury)
 {
     CPL_IMAGE_GET_STATS_WINDOW_DOUBLE_BODY(mean, MEAN);
 }
@@ -247,9 +247,9 @@ double cpl_image_get_mean(const cpl_image * image)
   For a finite population or sample, the median is the middle value of an odd
   number of values (arranged in ascending order) or any value between the two
   middle values of an even number of values.
-  In case of an even number of values in the input array, the median is chosen
-  to be the lower of the two central values - this ensures that the computed
-  median is always among the values of the input arrays.
+  For an even number of elements in the array, the mean of the two central
+  values is returned. Note that in this case, the median might not belong
+  to the input array.
 
   Possible #_cpl_error_code_ set in this function:
   - CPL_ERROR_NULL_INPUT if an input pointer is NULL
@@ -259,10 +259,10 @@ double cpl_image_get_mean(const cpl_image * image)
  */ 
 /*----------------------------------------------------------------------------*/
 double cpl_image_get_median_window(const cpl_image * image,
-                                   int               llx,
-                                   int               lly,
-                                   int               urx,
-                                   int               ury)
+                                   cpl_size          llx,
+                                   cpl_size          lly,
+                                   cpl_size          urx,
+                                   cpl_size          ury)
 {
     CPL_IMAGE_GET_STATS_WINDOW_DOUBLE_BODY(median, MEDIAN);
 }
@@ -302,10 +302,10 @@ double cpl_image_get_median(const cpl_image * image)
  */ 
 /*----------------------------------------------------------------------------*/
 double cpl_image_get_stdev_window(const cpl_image * image,
-                                  int               llx,
-                                  int               lly,
-                                  int               urx,
-                                  int               ury)
+                                  cpl_size          llx,
+                                  cpl_size          lly,
+                                  cpl_size          urx,
+                                  cpl_size          ury)
 {
     CPL_IMAGE_GET_STATS_WINDOW_DOUBLE_BODY(stdev, STDEV);
 }
@@ -338,10 +338,10 @@ double cpl_image_get_stdev(const cpl_image * image)
  */ 
 /*----------------------------------------------------------------------------*/
 double cpl_image_get_flux_window(const cpl_image * image,
-                                 int               llx,
-                                 int               lly,
-                                 int               urx,
-                                 int               ury)
+                                 cpl_size          llx,
+                                 cpl_size          lly,
+                                 cpl_size          urx,
+                                 cpl_size          ury)
 {
     CPL_IMAGE_GET_STATS_WINDOW_DOUBLE_BODY(flux, FLUX);
 }
@@ -375,10 +375,10 @@ double cpl_image_get_flux(const cpl_image * image)
  */ 
 /*----------------------------------------------------------------------------*/
 double cpl_image_get_absflux_window(const cpl_image * image,
-                                    int               llx,
-                                    int               lly,
-                                    int               urx,
-                                    int               ury)
+                                    cpl_size          llx,
+                                    cpl_size          lly,
+                                    cpl_size          urx,
+                                    cpl_size          ury)
 {
     CPL_IMAGE_GET_STATS_WINDOW_DOUBLE_BODY(absflux, ABSFLUX);
 }
@@ -398,10 +398,10 @@ double cpl_image_get_absflux_window(const cpl_image * image,
 /*----------------------------------------------------------------------------*/
 double cpl_image_get_sqflux_window(
         const cpl_image   *   image,
-        int             llx,
-        int             lly,
-        int             urx,
-        int             ury)
+        cpl_size        llx,
+        cpl_size        lly,
+        cpl_size        urx,
+        cpl_size        ury)
 {
     CPL_IMAGE_GET_STATS_WINDOW_DOUBLE_BODY(sqflux, SQFLUX);
 }
@@ -462,10 +462,10 @@ double cpl_image_get_centroid_x(const cpl_image * image)
  */ 
 /*----------------------------------------------------------------------------*/
 double cpl_image_get_centroid_x_window(const cpl_image * image,
-                                       int               llx,
-                                       int               lly,
-                                       int               urx,
-                                       int               ury)
+                                       cpl_size          llx,
+                                       cpl_size          lly,
+                                       cpl_size          urx,
+                                       cpl_size          ury)
 {
     CPL_IMAGE_GET_STATS_WINDOW_DOUBLE_BODY(centroid_x, CENTROID);
 }
@@ -498,10 +498,10 @@ double cpl_image_get_centroid_y(const cpl_image * image)
  */ 
 /*----------------------------------------------------------------------------*/
 double cpl_image_get_centroid_y_window(const cpl_image * image,
-                                       int               llx,
-                                       int               lly,
-                                       int               urx,
-                                       int               ury)
+                                       cpl_size          llx,
+                                       cpl_size          lly,
+                                       cpl_size          urx,
+                                       cpl_size          ury)
 {
     CPL_IMAGE_GET_STATS_WINDOW_DOUBLE_BODY(centroid_y, CENTROID);
 }
@@ -529,12 +529,12 @@ double cpl_image_get_centroid_y_window(const cpl_image * image,
 */
 /*----------------------------------------------------------------------------*/
 cpl_error_code cpl_image_get_minpos_window(const cpl_image * image,
-                                           int               llx,
-                                           int               lly,
-                                           int               urx,
-                                           int               ury,
-                                           int             * px,
-                                           int             * py)
+                                           cpl_size          llx,
+                                           cpl_size          lly,
+                                           cpl_size          urx,
+                                           cpl_size          ury,
+                                           cpl_size        * px,
+                                           cpl_size        * py)
 {
     cpl_stats * stats;
 
@@ -573,8 +573,8 @@ cpl_error_code cpl_image_get_minpos_window(const cpl_image * image,
  */
 /*----------------------------------------------------------------------------*/
 cpl_error_code cpl_image_get_minpos(const cpl_image * image,
-                                    int             * px,
-                                    int             * py)
+                                    cpl_size        * px,
+                                    cpl_size        * py)
 {
 
     cpl_stats * stats;
@@ -613,12 +613,12 @@ cpl_error_code cpl_image_get_minpos(const cpl_image * image,
  */
 /*----------------------------------------------------------------------------*/
 cpl_error_code cpl_image_get_maxpos_window(const cpl_image * image,
-                                           int               llx,
-                                           int               lly,
-                                           int               urx,
-                                           int               ury,
-                                           int             * px,
-                                           int             * py)
+                                           cpl_size          llx,
+                                           cpl_size          lly,
+                                           cpl_size          urx,
+                                           cpl_size          ury,
+                                           cpl_size        * px,
+                                           cpl_size        * py)
 {
     cpl_stats * stats;
 
@@ -651,10 +651,9 @@ cpl_error_code cpl_image_get_maxpos_window(const cpl_image * image,
   @see      cpl_image_get_minpos()
  */
 /*----------------------------------------------------------------------------*/
-cpl_error_code cpl_image_get_maxpos(
-        const cpl_image *   image,
-        int             *   px,
-        int             *   py)
+cpl_error_code cpl_image_get_maxpos(const cpl_image * image,
+                                    cpl_size        * px,
+                                    cpl_size        * py)
 {
     cpl_stats * stats;
 
@@ -706,10 +705,10 @@ cpl_error_code cpl_image_get_maxpos(
  */
 /*----------------------------------------------------------------------------*/
 double cpl_image_get_median_dev_window(const cpl_image * image,
-                                       int               llx,
-                                       int               lly,
-                                       int               urx,
-                                       int               ury,
+                                       cpl_size          llx,
+                                       cpl_size          lly,
+                                       cpl_size          urx,
+                                       cpl_size          ury,
                                        double          * sigma)
 {
     cpl_stats * stats;
