@@ -1,4 +1,4 @@
-/* $Id: cpl_imagelist_basic.c,v 1.48 2010/11/11 09:23:18 llundin Exp $
+/* $Id: cpl_imagelist_basic.c,v 1.55 2011/07/20 14:32:39 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/11/11 09:23:18 $
- * $Revision: 1.48 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2011/07/20 14:32:39 $
+ * $Revision: 1.55 $
+ * $Name: cpl-6_0 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -49,6 +49,7 @@
 #include "cpl_image_io.h"
 #include "cpl_image_bpm.h"
 #include "cpl_stats.h"
+#include "cpl_error_impl.h"
 #include "cpl_tools.h"
 #include "cpl_mask.h"
 #include "cpl_imagelist_defs.h"
@@ -65,18 +66,6 @@
 #define CPL_IMLIST_BASIC_TIME_SIGCLIP       4
 #define CPL_IMLIST_BASIC_SWAP_AXIS          5
 
-/*----------------------------------------------------------------------------*/
-/**
- * @defgroup cpl_imagelist_basic    Images lists basic routines
- *
- * This module provides basic functions to handle @em cpl_imagelist.
- *
- * @par Synopsis:
- * @code
- *   #include "cpl_imagelist_basic.h"
- * @endcode
- */
-/*----------------------------------------------------------------------------*/
 /**@{*/
 
 /*-----------------------------------------------------------------------------
@@ -90,6 +79,7 @@
 #define CPL_OPERATION CPL_IMLIST_BASIC_OPER
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Add two image lists, the first one is replaced by the result.
   @param    in1        first input image list (modified)
   @param    in2        image list to add
@@ -115,6 +105,7 @@ cpl_error_code cpl_imagelist_add(
 
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Subtract two image lists, the first one is replaced by the result.
   @param    in1        first input image list (modified)
   @param    in2        image list to subtract
@@ -134,6 +125,7 @@ cpl_error_code cpl_imagelist_subtract(
 
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Multiply two image lists, the first one is replaced by the result.
   @param    in1        first input image list (modified)
   @param    in2        image list to multiply
@@ -153,6 +145,7 @@ cpl_error_code cpl_imagelist_multiply(
 
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Divide two image lists, the first one is replaced by the result.
   @param    in1        first input image list (modified)
   @param    in2        image list to divide
@@ -174,6 +167,7 @@ cpl_error_code cpl_imagelist_divide(
 #define CPL_OPERATION CPL_IMLIST_BASIC_IMAGE_LOCAL
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Add an image to an image list.
   @param    imlist  input image list (modified)
   @param    img     image to add
@@ -197,6 +191,7 @@ cpl_error_code cpl_imagelist_add_image(
 
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Subtract an image from an image list.
   @param    imlist    input image list (modified)
   @param    img        image to subtract
@@ -216,6 +211,7 @@ cpl_error_code cpl_imagelist_subtract_image(
 
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Multiply an image list by an image.
   @param    imlist    input image list (modified)
   @param    img     image to multiply
@@ -235,6 +231,7 @@ cpl_error_code cpl_imagelist_multiply_image(
 
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Divide an image list by an image.
   @param    imlist    input image list (modified)
   @param    img     image for division
@@ -255,6 +252,7 @@ cpl_error_code cpl_imagelist_divide_image(
 
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Elementwise addition of a scalar to each image in the imlist
   @param    imlist  Imagelist to be modified in place.
   @param    addend  Number to add
@@ -269,20 +267,21 @@ cpl_error_code cpl_imagelist_add_scalar(
         cpl_imagelist * imlist,
         double          addend)
 {
-    int          i ;
+    cpl_size     i;
 
     /* Check inputs */
     cpl_ensure_code(imlist != NULL, CPL_ERROR_NULL_INPUT);
         
-    for (i=0 ; i < imlist->ni ; i++)
+    for (i=0; i < imlist->ni; i++)
         cpl_ensure_code( !cpl_image_add_scalar(imlist->images[i], addend),
                          cpl_error_get_code());
 
-    return CPL_ERROR_NONE ;
+    return CPL_ERROR_NONE;
 }
 
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Elementwise subtraction of a scalar from each image in the imlist
   @param    imlist   Imagelist to be modified in place.
   @param    subtrahend  Number to subtract
@@ -294,20 +293,21 @@ cpl_error_code cpl_imagelist_subtract_scalar(
         cpl_imagelist * imlist,
         double          subtrahend)
 {
-    int          i ;
+    cpl_size     i;
 
     /* Check inputs */
     cpl_ensure_code(imlist != NULL, CPL_ERROR_NULL_INPUT);
         
-    for (i=0 ; i < imlist->ni ; i++)
+    for (i=0; i < imlist->ni; i++)
         cpl_ensure_code(!cpl_image_subtract_scalar(imlist->images[i],subtrahend),
                          cpl_error_get_code());
 
-    return CPL_ERROR_NONE ;
+    return CPL_ERROR_NONE;
 }
 
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Elementwise multiplication of the imlist with a scalar
   @param    imlist  Imagelist to be modified in place.
   @param    factor     Number to multiply with
@@ -319,20 +319,21 @@ cpl_error_code cpl_imagelist_multiply_scalar(
         cpl_imagelist * imlist,
         double          factor)
 {
-    int          i ;
+    cpl_size     i;
 
     /* Check inputs */
     cpl_ensure_code(imlist != NULL, CPL_ERROR_NULL_INPUT);
         
-    for (i=0 ; i < imlist->ni ; i++)
+    for (i=0; i < imlist->ni; i++)
         cpl_ensure_code( !cpl_image_multiply_scalar(imlist->images[i], factor),
                          cpl_error_get_code());
 
-    return CPL_ERROR_NONE ;
+    return CPL_ERROR_NONE;
 }
 
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Elementwise division of each image in the imlist with a scalar
   @param    imlist  Imagelist to be modified in place.
   @param    divisor    Non-zero number to divide with
@@ -347,20 +348,21 @@ cpl_error_code cpl_imagelist_divide_scalar(
         cpl_imagelist * imlist,
         double          divisor)
 {
-    int          i ;
+    cpl_size     i;
 
     /* Check inputs */
     cpl_ensure_code(imlist != NULL, CPL_ERROR_NULL_INPUT);
         
-    for (i=0 ; i < imlist->ni ; i++)
+    for (i=0; i < imlist->ni; i++)
         cpl_ensure_code( !cpl_image_divide_scalar(imlist->images[i], divisor),
                          cpl_error_get_code());
 
-    return CPL_ERROR_NONE ;
+    return CPL_ERROR_NONE;
 }
 
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Compute the elementwise logarithm of each image in the imlist
   @param    imlist  Imagelist to be modified in place.
   @param    base       Base of the logarithm.
@@ -375,20 +377,21 @@ cpl_error_code cpl_imagelist_logarithm(
         cpl_imagelist * imlist,
         double          base)
 {
-    int          i ;
+    cpl_size     i;
 
     /* Check inputs */
     cpl_ensure_code(imlist != NULL, CPL_ERROR_NULL_INPUT);
         
-    for (i=0 ; i < imlist->ni ; i++)
+    for (i=0; i < imlist->ni; i++)
         cpl_ensure_code( !cpl_image_logarithm(imlist->images[i], base),
                          cpl_error_get_code());
 
-    return CPL_ERROR_NONE ;
+    return CPL_ERROR_NONE;
 }
 
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Compute the elementwise exponential of each image in the imlist
   @param    imlist  Imagelist to be modified in place.
   @param    base       Base of the exponential.
@@ -403,20 +406,21 @@ cpl_error_code cpl_imagelist_exponential(
         cpl_imagelist * imlist,
         double          base)
 {
-    int          i ;
+    cpl_size     i;
 
     /* Check inputs */
     cpl_ensure_code(imlist != NULL, CPL_ERROR_NULL_INPUT);
         
-    for (i=0 ; i < imlist->ni ; i++)
+    for (i=0; i < imlist->ni; i++)
         cpl_ensure_code( !cpl_image_exponential(imlist->images[i], base),
                          cpl_error_get_code());
 
-    return CPL_ERROR_NONE ;
+    return CPL_ERROR_NONE;
 }
 
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Compute the elementwise power of each image in the imlist
   @param    imlist   Imagelist to be modified in place.
   @param    exponent Scalar exponent
@@ -431,20 +435,21 @@ cpl_error_code cpl_imagelist_power(
         cpl_imagelist * imlist,
         double          exponent)
 {
-    int          i ;
+    cpl_size     i;
 
     /* Check inputs */
     cpl_ensure_code(imlist != NULL, CPL_ERROR_NULL_INPUT);
         
-    for (i=0 ; i < imlist->ni ; i++)
+    for (i=0; i < imlist->ni; i++)
         cpl_ensure_code( !cpl_image_power(imlist->images[i], exponent),
                          cpl_error_get_code());
 
-    return CPL_ERROR_NONE ;
+    return CPL_ERROR_NONE;
 }
 
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Normalize each image in the list.
   @param    imlist  Imagelist to modify.
   @param    mode    Normalization mode.
@@ -461,20 +466,21 @@ cpl_error_code cpl_imagelist_normalise(
         cpl_imagelist *  imlist,
         cpl_norm         mode)
 {
-    int                 i ;
+    cpl_size            i;
 
     /* Check inputs */
     cpl_ensure_code(imlist != NULL, CPL_ERROR_NULL_INPUT);
     
-    for (i=0 ; i<imlist->ni ; i++)
+    for (i=0; i<imlist->ni; i++)
         cpl_ensure_code(!cpl_image_normalise(imlist->images[i], mode), 
                         cpl_error_get_code());
 
-    return CPL_ERROR_NONE ;
+    return CPL_ERROR_NONE;
 }
 
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Threshold all pixel values to an interval.
   @param    imlist               Image list to threshold.
   @param    lo_cut          Lower bound.
@@ -499,22 +505,23 @@ cpl_error_code cpl_imagelist_threshold(
         double          assign_lo_cut,
         double          assign_hi_cut)
 {
-    int                     i ;
+    cpl_size                i;
 
     /* Check inputs */
     cpl_ensure_code(imlist != NULL, CPL_ERROR_NULL_INPUT);
-    cpl_ensure_code(lo_cut < hi_cut, CPL_ERROR_INCOMPATIBLE_INPUT) ;
+    cpl_ensure_code(lo_cut < hi_cut, CPL_ERROR_INCOMPATIBLE_INPUT);
     
-    for (i=0 ; i<imlist->ni ; i++)
+    for (i=0; i<imlist->ni; i++)
         cpl_ensure_code(!cpl_image_threshold(imlist->images[i], lo_cut, hi_cut,
                                              assign_lo_cut, assign_hi_cut),
                         cpl_error_get_code());
 
-    return CPL_ERROR_NONE ;
+    return CPL_ERROR_NONE;
 }
 
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Create a contribution map from the bad pixel maps of the images.
   @param    imlist  The imagelist
   @return   The contributions map (a CPL_TYPE_INT cpl_image) or NULL on error
@@ -531,11 +538,11 @@ cpl_error_code cpl_imagelist_threshold(
 cpl_image * cpl_image_new_from_accepted(const cpl_imagelist * imlist)
 {
     const cpl_image * first = cpl_imagelist_get_const(imlist, 0);
-    const int         nx    = cpl_image_get_size_x(first);
-    const int         ny    = cpl_image_get_size_y(first);
-    const int         nz    = cpl_imagelist_get_size(imlist);
+    const cpl_size    nx    = cpl_image_get_size_x(first);
+    const cpl_size    ny    = cpl_image_get_size_y(first);
+    const cpl_size    nz    = cpl_imagelist_get_size(imlist);
     cpl_image       * goodsum;
-    int               i;
+    cpl_size          i;
     
 
     cpl_ensure(imlist != NULL, CPL_ERROR_NULL_INPUT, NULL);
@@ -543,7 +550,7 @@ cpl_image * cpl_image_new_from_accepted(const cpl_imagelist * imlist)
                NULL);
 
     goodsum = cpl_image_new(nx, ny, CPL_TYPE_INT);
-    (void)cpl_image_add_scalar(goodsum, nz); /* Assume all good */
+    (void)cpl_image_add_scalar(goodsum, (double)nz); /* Assume all good */
 
     /* Loop on the images */
     for (i = 0; i < nz; i++) {
@@ -566,6 +573,7 @@ cpl_image * cpl_image_new_from_accepted(const cpl_imagelist * imlist)
 
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Average an imagelist to a single image
   @param    imlist    the input images list
   @return   the average image or NULL on error case.
@@ -588,8 +596,7 @@ cpl_image * cpl_imagelist_collapse_create(const cpl_imagelist * imlist)
 {
     cpl_image    * avg;
     cpl_image    * contrib;
-    int            i;
-    cpl_error_code error;
+    cpl_size       i;
     
     /* Check inputs */
     cpl_ensure(imlist != NULL, CPL_ERROR_NULL_INPUT, NULL);
@@ -625,20 +632,22 @@ cpl_image * cpl_imagelist_collapse_create(const cpl_imagelist * imlist)
     /* Any zero in the contribution map will lead to a bad pixel in avg */
     /* - bad pixels in avg are zero-valued due to the initial
        cpl_image_fill_rejected() */
-    error = cpl_image_divide(avg, contrib);
-    cpl_image_delete(contrib);
 
-    if (error) {
+    if (cpl_image_divide(avg, contrib)) {
         cpl_image_delete(avg);
-        cpl_ensure(0, cpl_error_get_code(), NULL);
+        avg = NULL;
+        (void)cpl_error_set_where_();
     }
 
+    cpl_image_delete(contrib);
+
     return avg;    
 }
 
 #define CPL_OPERATION CPL_IMLIST_BASIC_TIME_MINMAX
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Average with rejection an imagelist to a single image
   @param    self  The image list to average
   @param    nlow   Number of low rejected values
@@ -664,15 +673,15 @@ cpl_image * cpl_imagelist_collapse_create(const cpl_imagelist * imlist)
  */
 /*----------------------------------------------------------------------------*/
 cpl_image * cpl_imagelist_collapse_minmax_create(const cpl_imagelist * self,
-                                                 int                   nlow,
-                                                 int                   nhigh)
+                                                 cpl_size              nlow,
+                                                 cpl_size              nhigh)
 {
     const cpl_image * imgself = cpl_imagelist_get_const(self, 0);
     const cpl_type    type    = cpl_image_get_type(imgself);
-    const int         ni      = cpl_imagelist_get_size(self);
-    const int         nx      = cpl_image_get_size_x(imgself);
-    const int         ny      = cpl_image_get_size_y(imgself);
-    const int         nuse    = ni - nlow - nhigh;
+    const cpl_size    ni      = cpl_imagelist_get_size(self);
+    const cpl_size    nx      = cpl_image_get_size_x(imgself);
+    const cpl_size    ny      = cpl_image_get_size_y(imgself);
+    const cpl_size    nuse    = ni - nlow - nhigh;
     cpl_image       * avg     = NULL;
 
     
@@ -697,7 +706,7 @@ cpl_image * cpl_imagelist_collapse_minmax_create(const cpl_imagelist * self,
 #include "cpl_imagelist_basic_body.h"
 #undef CPL_CLASS
         default:
-            (void)cpl_error_set(cpl_func, CPL_ERROR_INVALID_TYPE);
+            (void)cpl_error_set_(CPL_ERROR_INVALID_TYPE);
     }
 
     return avg;
@@ -707,101 +716,129 @@ cpl_image * cpl_imagelist_collapse_minmax_create(const cpl_imagelist * self,
 #define CPL_OPERATION CPL_IMLIST_BASIC_TIME_SIGCLIP
 /*----------------------------------------------------------------------------*/
 /**
-  @brief    Collapse an imagelist with kappa-sigma-clipping rejection
-  @param    imlist    the input imagelist
-  @param    kappalow  kappa-factor for lower boundary level
-  @param    kappahigh kappa-factor for upper boundary level
-  @param    keep      minimum percentage of values to keep (0.0 < keep < 1.0)
-  @param    mode      collapsing mode
-  @param    kept      image containing kept values per position
-  @return   the collapsed image or NULL on error case.
+  @ingroup cpl_imagelist
+  @brief  Collapse an imagelist with kappa-sigma-clipping rejection
+  @param  self       The input imagelist
+  @param  kappalow   kappa-factor for lower clipping threshold
+  @param  kappahigh  kappa-factor for upper clipping threshold
+  @param  keepfrac   The fraction of values to keep (0.0 < keepfrac <= 1.0)
+  @param  mode       Clipping mode, CPL_COLLAPSE_MEAN or CPL_COLLAPSE_MEDIAN
+  @param  contrib    Pre-allocated integer-image for contribution map or NULL
+  @return The collapsed image or NULL on error case.
+  @note The returned image has to be deallocated with cpl_image_delete().
 
-  The returned image has to be deallocated with cpl_image_delete().
+  The collapsing is an iterative process which will stop when it converges
+  (i.e. an iteration did not reject any values for a given pixel) or
+  when the next iteration would reduce the fraction of values to keep
+  to less than or equal to keepfrac.
 
-  The bad pixel maps of the input frames are not taken into account, and 
-  the one of the created image is empty.
+  A call with keepfrac == 1.0 will thus perform no clipping.
 
   Supported modes: 
       CPL_COLLAPSE_MEAN:
           The center value of the acceptance range will be the mean.
       CPL_COLLAPSE_MEDIAN:
           The center value of the acceptance range will be the median.
+      CPL_COLLAPSE_MEDIAN_MEAN:
+          The center value of the acceptance range will be the median in
+          the first iteration and in subsequent iterations it will be the
+          mean.
  
   For each pixel position the pixels whose value is higher than
   center + kappahigh * stdev or lower than center - kappalow * stdev 
-  are discarded for the average computation, where center is either mean or
-  median depending on the mode, and stdev is the standard deviation of the
-  pixels at that position.
+  are discarded for the subsequent center and stdev computation, where center
+  is defined according to the clipping mode, and stdev is the standard
+  deviation of the values at that pixel position. Since the acceptance
+  interval must be non-empty, the sum of kappalow and kappahigh must be
+  positive. A typical call has both kappalow and kappahigh positive.
 
-  The collapsing is an iterative process which will stop once either it reaches
-  convergence or the percentage of rejected values reaches 1.0 - keep.
+  The minimum number of values that the clipping can select is 2. This is
+  because the clipping criterion is based on the sample standard deviation,
+  which needs at least two values to be defined. This means that all calls
+  with (positive) values of keepfrac less than 2/n will behave the same. To
+  ensure that the values in (at least) i planes out of n are kept, keepfrac
+  can be set to (i - 0.5) / n, e.g. to keep at least 50 out of 100 values,
+  keepfrac can be set to 0.495.
 
-  The input parameter kept is optional. It must be either NULL or point to an
-  allocated image of type CPL_TYPE_INT and equal size of the images in the 
-  imagelist. It will contain in output the number of kept (non-rejected) values
-  after the iterating collapse process in every pixel position.
+  The output pixel is set to the mean of the non-clipped values, also in the
+  median mode. Regardless of the input pixel type, the mean is computed in
+  double precision. The result is then cast to the output-pixel type, which
+  is identical to the input pixel type.
+
+  The input parameter contrib is optional. It must be either NULL or point to a
+  pre-allocated image of type CPL_TYPE_INT and size equal to the images in the 
+  imagelist. On success, it will contain the contribution map, i.e.  the number
+  of kept (non-clipped) values after the iterative process on every pixel.
   
+  Bad pixels are ignored from the start. This means that with a sufficient
+  number of bad pixels, the fraction of good values will be less than keepfrac.
+  In this case no iteration is performed at all. If there is at least one
+  good value available, then the mean will be based on the good value(s). If
+  for a given pixel position there are no good values, then that pixel is
+  set to zero, rejected as bad and if available the value in the
+  contribution map is set to zero.
+
   The input imagelist can be of type CPL_TYPE_INT, CPL_TYPE_FLOAT and
   CPL_TYPE_DOUBLE.
 
   Possible #_cpl_error_code_ set in this function:
-  - CPL_ERROR_NULL_INPUT if (one of) the input pointer(s) is NULL
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
+  - CPL_ERROR_DATA_NOT_FOUND
+    if there are less than 2 images in the list
   - CPL_ERROR_ILLEGAL_INPUT 
-      if the input imagelist is not valid,
-      if any of kappalow and kappahigh is negative, 
-      if keep is not within the expected interval (0.0, 1.0),
-      if kept is non-NULL but the pointed image is not of type CPL_TYPE_INT or
-      of different size
-      if there are less than 2 images in the list
-  - CPL_ERROR_INVALID_TYPE if the passed imagelist type is not supported
-  - CPL_ERROR_UNSUPPORTED_MODE if the passed mode is none of the listed above
+    if the sum of kappalow and kappahigh is non-positive, 
+  - CPL_ERROR_ACCESS_OUT_OF_RANGE
+    if keepfrac is outside the required interval which is 0.0 < keepfrac <= 1.0
+  - CPL_ERROR_TYPE_MISMATCH
+    if contrib is non-NULL but not of type CPL_TYPE_INT
+  - CPL_ERROR_INCOMPATIBLE_INPUT if contrib is non-NULL but
+       of a size incompatible with the input imagelist
+  - CPL_ERROR_INVALID_TYPE if the type of the input imagelist is unsupported
+  - CPL_ERROR_UNSUPPORTED_MODE if the passed mode is none of the above listed
  */
 /*----------------------------------------------------------------------------*/
-cpl_image * cpl_imagelist_collapse_sigclip_create(
-        const cpl_imagelist *   imlist,
-        double                  kappalow,
-        double                  kappahigh,
-        double                  keep,
-        unsigned                mode,
-        cpl_image           *   kept)
+cpl_image *
+cpl_imagelist_collapse_sigclip_create(const cpl_imagelist * self,
+                                      double                kappalow,
+                                      double                kappahigh,
+                                      double                keepfrac,
+                                      cpl_collapse_mode     mode,
+                                      cpl_image           * contrib)
 {
-    const cpl_image *   cur_ima ;
-    int                 ni, nx, ny ;
-    cpl_type            type;
-    cpl_image       *   out_ima ;
-    
-    int             *    pkept = NULL;
+    const cpl_size    ni       = cpl_imagelist_get_size(self);
+    const cpl_image * img      = cpl_imagelist_get_const(self, 0);
+    const cpl_size    nx       = cpl_image_get_size_x(img);
+    const cpl_size    ny       = cpl_image_get_size_y(img);
+    cpl_image       * clipped  = NULL;
+    int             * pcontrib = NULL;
 
     /* Check inputs */
-    cpl_ensure(imlist != NULL, CPL_ERROR_NULL_INPUT, NULL);
-    cpl_ensure(cpl_imagelist_is_uniform(imlist) == 0, CPL_ERROR_ILLEGAL_INPUT,
+    cpl_ensure(self               != NULL, CPL_ERROR_NULL_INPUT,          NULL);
+    cpl_ensure(kappalow + kappahigh > 0.0, CPL_ERROR_ILLEGAL_INPUT,       NULL);
+    cpl_ensure(keepfrac             > 0.0, CPL_ERROR_ACCESS_OUT_OF_RANGE, NULL);
+    cpl_ensure(keepfrac            <= 1.0, CPL_ERROR_ACCESS_OUT_OF_RANGE, NULL);
+    cpl_ensure(ni                   > 1,   CPL_ERROR_DATA_NOT_FOUND,      NULL);
+
+    cpl_ensure(cpl_imagelist_is_uniform(self) == 0, CPL_ERROR_ILLEGAL_INPUT,
                NULL);
-    cpl_ensure(kappalow > 0.0 && kappahigh > 0.0,  CPL_ERROR_ILLEGAL_INPUT,
-           NULL);
-    cpl_ensure(keep > 0.0 && keep < 1.0, CPL_ERROR_ILLEGAL_INPUT, NULL);
-    cpl_ensure(mode & CPL_COLLAPSE_MEDIAN || mode & CPL_COLLAPSE_MEAN,
-           CPL_ERROR_UNSUPPORTED_MODE, NULL);
 
-    ni      = cpl_imagelist_get_size(imlist);
+    cpl_ensure(mode == CPL_COLLAPSE_MEDIAN_MEAN || mode == CPL_COLLAPSE_MEDIAN
+               || mode == CPL_COLLAPSE_MEAN,
+               CPL_ERROR_UNSUPPORTED_MODE, NULL);
 
-    cpl_ensure(ni > 1, CPL_ERROR_ILLEGAL_INPUT, NULL);
+    if (contrib) {
+        cpl_ensure(cpl_image_get_size_x(contrib) == nx,
+                   CPL_ERROR_ILLEGAL_INPUT, NULL);
+        cpl_ensure(cpl_image_get_size_y(contrib) == ny,
+                   CPL_ERROR_ILLEGAL_INPUT, NULL);
+        cpl_ensure(cpl_image_get_type(contrib)   == CPL_TYPE_INT,
+                   CPL_ERROR_ILLEGAL_INPUT, NULL);
 
-    cur_ima = cpl_imagelist_get_const(imlist, 0);
-    nx      = cpl_image_get_size_x(cur_ima);
-    ny      = cpl_image_get_size_y(cur_ima);
-    type    = cpl_image_get_type(cur_ima);
-
-    if (kept) {
-    cpl_ensure(cpl_image_get_size_x(kept) == nx &&
-           cpl_image_get_size_y(kept) == ny &&
-           cpl_image_get_type(kept)   == CPL_TYPE_INT,
-           CPL_ERROR_ILLEGAL_INPUT, NULL);
-
-    pkept = cpl_image_get_data_int(kept);
+        pcontrib = cpl_image_get_data_int(contrib);
     }
 
     /* Switch on the data type */
-    switch (cpl_image_get_type(cur_ima)) {
+    switch (cpl_image_get_type(img)) {
 #define CPL_CLASS CPL_CLASS_DOUBLE
 #include "cpl_imagelist_basic_body.h"
 #undef CPL_CLASS
@@ -813,18 +850,18 @@ cpl_image * cpl_imagelist_collapse_sigclip_create(
 #define CPL_CLASS CPL_CLASS_INT
 #include "cpl_imagelist_basic_body.h"
 #undef CPL_CLASS
-        default:
-            cpl_ensure(0, CPL_ERROR_INVALID_TYPE, NULL) ;
+    default:
+        (void)cpl_error_set_(CPL_ERROR_INVALID_TYPE);
     }
 
-    /* Return */
-    return out_ima ;
+    return clipped;
 }
 #undef CPL_OPERATION
 
 #define CPL_OPERATION CPL_IMLIST_BASIC_TIME_MEDIAN
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Compute the median of an image list to a single image
   @param    imlist    the input images list
   @return   the median image or NULL on error case.
@@ -846,10 +883,10 @@ cpl_image * cpl_imagelist_collapse_sigclip_create(
 /*----------------------------------------------------------------------------*/
 cpl_image * cpl_imagelist_collapse_median_create(const cpl_imagelist * imlist)
 {
-    cpl_image           *   median ;
-    int                     nx, ny ;
-    cpl_type                type ;
-    int                     i, j, k ;
+    cpl_image           *   median;
+    cpl_size                nx, ny;
+    cpl_type                type;
+    cpl_size                i, j, k;
     
     /* Check inputs */
     cpl_ensure(imlist != NULL, CPL_ERROR_NULL_INPUT, NULL);
@@ -860,7 +897,7 @@ cpl_image * cpl_imagelist_collapse_median_create(const cpl_imagelist * imlist)
     type = cpl_image_get_type(imlist->images[0]);
 
     /* Create the output image */
-    median = cpl_image_new(nx, ny, type) ;
+    median = cpl_image_new(nx, ny, type);
 
     cpl_ensure(median, cpl_error_get_code(), NULL);
 
@@ -879,16 +916,18 @@ cpl_image * cpl_imagelist_collapse_median_create(const cpl_imagelist * imlist)
 #undef CPL_CLASS
         default:
             cpl_image_delete(median);
-            cpl_ensure(0, CPL_ERROR_INVALID_TYPE, NULL) ;
+            median = NULL;
+            (void)cpl_error_set_(CPL_ERROR_INVALID_TYPE);
     }
 
-    return median ;    
+    return median;    
 }
 #undef CPL_OPERATION
 
 #define CPL_OPERATION CPL_IMLIST_BASIC_SWAP_AXIS
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Swap the axis of an image list
   @param    ilist   The image list to swap
   @param    mode    The swapping mode      
@@ -914,35 +953,35 @@ cpl_imagelist * cpl_imagelist_swap_axis_create(
         const cpl_imagelist *   ilist,
         cpl_swap_axis           mode)    
 {
-    cpl_imagelist       *   swapped ;
-    const cpl_image     *   image ;
-    cpl_image           *   cur_ima ;
-    const cpl_image     *   old_ima ;
-    int                     ni, nx, ny ;
-    cpl_type                type ;
-    int                     recompute_bpm, nbpm ;
-    cpl_mask            *   cur_mask ;
-    cpl_binary          *   pcur_mask ;
-    cpl_binary          *   pold_mask ;
-    int                     i, j, k ;
+    cpl_imagelist       *   swapped;
+    const cpl_image     *   image;
+    cpl_image           *   cur_ima;
+    const cpl_image     *   old_ima;
+    cpl_size                ni, nx, ny;
+    cpl_type                type;
+    int                     recompute_bpm, nbpm;
+    cpl_mask            *   cur_mask;
+    cpl_binary          *   pcur_mask;
+    cpl_binary          *   pold_mask;
+    cpl_size                i, j, k;
 
     /* Check entries */
     cpl_ensure(ilist != NULL, CPL_ERROR_NULL_INPUT, NULL);
     cpl_ensure(cpl_imagelist_is_uniform(ilist)==0, CPL_ERROR_ILLEGAL_INPUT,
                NULL);
     cpl_ensure(mode==CPL_SWAP_AXIS_XZ || mode==CPL_SWAP_AXIS_YZ, 
-            CPL_ERROR_ILLEGAL_INPUT, NULL) ;
+            CPL_ERROR_ILLEGAL_INPUT, NULL);
 
     /* Initialise */
-    recompute_bpm = 0 ;
+    recompute_bpm = 0;
     image = cpl_imagelist_get_const(ilist, 0);
     ni = cpl_imagelist_get_size(ilist);
     nx = cpl_image_get_size_x(image);
     ny = cpl_image_get_size_y(image);
-    type = cpl_image_get_type(image) ;
+    type = cpl_image_get_type(image);
 
     /* Create the new image list */
-    swapped = cpl_imagelist_new() ;
+    swapped = cpl_imagelist_new();
 
     /* Switch on image type */
     switch (type) {
@@ -956,16 +995,17 @@ cpl_imagelist * cpl_imagelist_swap_axis_create(
 #include "cpl_imagelist_basic_body.h"
 #undef CPL_CLASS
         default:
-            cpl_ensure(0, CPL_ERROR_INVALID_TYPE, NULL);
+            (void)cpl_error_set_(CPL_ERROR_INVALID_TYPE);
+            return NULL;
     }
 
     /* HANDLE THE BAD PIXELS MASKS */
     /* Check if there are bad pixels in the input image list */
-    for (i=0 ; i<ni ; i++) {
-        image = cpl_imagelist_get_const(ilist, i) ;
+    for (i=0; i<ni; i++) {
+        image = cpl_imagelist_get_const(ilist, i);
         if (image->bpm != NULL) {
-            recompute_bpm = 1 ;
-            break ;
+            recompute_bpm = 1;
+            break;
         }
     }
 
@@ -973,18 +1013,18 @@ cpl_imagelist * cpl_imagelist_swap_axis_create(
     if (recompute_bpm == 1) {
         /* SWAP X <-> Z */
         if (mode == CPL_SWAP_AXIS_XZ) {
-            for (i=0 ; i<nx ; i++) {
-                cur_mask = cpl_mask_new(ni, ny) ;
-                pcur_mask = cpl_mask_get_data(cur_mask) ;
-                nbpm = 0 ;
-                for (j=0 ; j<ni ; j++) {
-                    image = cpl_imagelist_get_const(ilist, j) ;
+            for (i=0; i<nx; i++) {
+                cur_mask = cpl_mask_new(ni, ny);
+                pcur_mask = cpl_mask_get_data(cur_mask);
+                nbpm = 0;
+                for (j=0; j<ni; j++) {
+                    image = cpl_imagelist_get_const(ilist, j);
                     if (image->bpm != NULL) {
-                        pold_mask = cpl_mask_get_data(image->bpm) ;
-                        for (k=0 ; k<ny ; k++) {
+                        pold_mask = cpl_mask_get_data(image->bpm);
+                        for (k=0; k<ny; k++) {
                             if (pold_mask[i+k*nx] == CPL_BINARY_1) {
-                                pcur_mask[j+k*ni]=pold_mask[i+k*nx] ;
-                                nbpm ++ ;
+                                pcur_mask[j+k*ni]=pold_mask[i+k*nx];
+                                nbpm = 1;
                             }
                         }
                     }
@@ -992,25 +1032,25 @@ cpl_imagelist * cpl_imagelist_swap_axis_create(
 
                 /* Put the new bpm in the new image */
                 if (nbpm > 0) {
-                    cpl_image * swap = cpl_imagelist_get(swapped, i) ;
-                    cpl_image_reject_from_mask(swap, cur_mask) ;
+                    cpl_image * swap = cpl_imagelist_get(swapped, i);
+                    cpl_image_reject_from_mask(swap, cur_mask);
                 }
-                cpl_mask_delete(cur_mask) ;
+                cpl_mask_delete(cur_mask);
             }
         } else {
         /* SWAP Y <-> Z */
-            for (i=0 ; i<ny ; i++) {
-                cur_mask = cpl_mask_new(nx, ni) ;
-                pcur_mask = cpl_mask_get_data(cur_mask) ;
-                nbpm = 0 ;
-                for (j=0 ; j<ni ; j++) {
-                    image = cpl_imagelist_get_const(ilist, j) ;
+            for (i=0; i<ny; i++) {
+                cur_mask = cpl_mask_new(nx, ni);
+                pcur_mask = cpl_mask_get_data(cur_mask);
+                nbpm = 0;
+                for (j=0; j<ni; j++) {
+                    image = cpl_imagelist_get_const(ilist, j);
                     if (image->bpm != NULL) {
-                        pold_mask = cpl_mask_get_data(image->bpm) ;
-                        for (k=0 ; k<nx ; k++) {
+                        pold_mask = cpl_mask_get_data(image->bpm);
+                        for (k=0; k<nx; k++) {
                             if (pold_mask[k+i*nx] == CPL_BINARY_1) {
-                                pcur_mask[k+j*ni]=pold_mask[k+i*nx] ;
-                                nbpm ++ ;
+                                pcur_mask[k+j*ni]=pold_mask[k+i*nx];
+                                nbpm = 1;
                             }
                         }
                     }
@@ -1018,14 +1058,14 @@ cpl_imagelist * cpl_imagelist_swap_axis_create(
 
                 /* Put the new bpm in the new image */
                 if (nbpm > 0) {
-                    cpl_image * swap = cpl_imagelist_get(swapped, i) ;
-                    cpl_image_reject_from_mask(swap, cur_mask) ;
+                    cpl_image * swap = cpl_imagelist_get(swapped, i);
+                    cpl_image_reject_from_mask(swap, cur_mask);
                 }
-                cpl_mask_delete(cur_mask) ;
+                cpl_mask_delete(cur_mask);
             }
         }
     }
-    return swapped ;
+    return swapped;
 }
 #undef CPL_OPERATION
 
