@@ -1,4 +1,4 @@
-/* $Id: cpl_imagelist_basic.h,v 1.22 2010/11/11 15:28:46 llundin Exp $
+/* $Id: cpl_imagelist_basic.h,v 1.25 2011/07/20 14:32:39 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/11/11 15:28:46 $
- * $Revision: 1.22 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2011/07/20 14:32:39 $
+ * $Revision: 1.25 $
+ * $Name: cpl-6_0 $
  */
 
 #ifndef CPL_IMAGELIST_BASIC_H
@@ -32,7 +32,6 @@
                                    Includes
  -----------------------------------------------------------------------------*/
 
-#include "cpl_image.h"
 #include "cpl_imagelist.h"
 
 CPL_BEGIN_DECLS
@@ -48,8 +47,11 @@ enum _cpl_swap_axis_ {
 
 typedef enum _cpl_swap_axis_ cpl_swap_axis;
 
-#define CPL_COLLAPSE_MEAN   ((unsigned)1 << 1)
-#define CPL_COLLAPSE_MEDIAN ((unsigned)1 << 2)
+typedef enum {
+    CPL_COLLAPSE_MEAN,
+    CPL_COLLAPSE_MEDIAN,
+    CPL_COLLAPSE_MEDIAN_MEAN
+} cpl_collapse_mode;
 
 /*-----------------------------------------------------------------------------
                             Function prototypes
@@ -86,9 +88,11 @@ cpl_image * cpl_imagelist_collapse_create(const cpl_imagelist *) CPL_ATTR_ALLOC;
 cpl_image * cpl_imagelist_collapse_median_create(const cpl_imagelist *)
     CPL_ATTR_ALLOC;
 cpl_image * cpl_imagelist_collapse_minmax_create(const cpl_imagelist *,
-                                                 int, int) CPL_ATTR_ALLOC;
-cpl_image * cpl_imagelist_collapse_sigclip_create(const cpl_imagelist *, double,
-                                                  double, double, unsigned,
+                                                 cpl_size, cpl_size)
+    CPL_ATTR_ALLOC;
+cpl_image * cpl_imagelist_collapse_sigclip_create(const cpl_imagelist *,
+                                                  double, double, double,
+                                                  cpl_collapse_mode,
                                                   cpl_image *) CPL_ATTR_ALLOC;
 
 CPL_END_DECLS
