@@ -1,4 +1,4 @@
-/* $Id: cpl_imagelist_io.c,v 1.84 2010/12/23 14:12:36 llundin Exp $
+/* $Id: cpl_imagelist_io.c,v 1.94 2011/12/16 15:39:57 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/12/23 14:12:36 $
- * $Revision: 1.84 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2011/12/16 15:39:57 $
+ * $Revision: 1.94 $
+ * $Name: cpl-6_0 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -33,6 +33,17 @@
                                 Includes
  -----------------------------------------------------------------------------*/
 
+#include "cpl_tools.h"
+#include "cpl_error_impl.h"
+#include "cpl_propertylist_impl.h"
+#include "cpl_memory.h"
+#include "cpl_imagelist_io.h"
+#include "cpl_image_io_impl.h"
+#include "cpl_io_fits.h"
+
+#include "cpl_imagelist_defs.h"
+
+/* Verify self-sufficiency of CPL header files by including system files last */
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -42,35 +53,15 @@
 
 #include <fitsio.h>
 
-#include "cpl_tools.h"
-#include "cpl_error_impl.h"
-#include "cpl_propertylist_impl.h"
-#include "cpl_memory.h"
-#include "cpl_imagelist_io_impl.h"
-#include "cpl_image_io_impl.h"
-
-#include "cpl_imagelist_defs.h"
-
 /*-----------------------------------------------------------------------------
                                 Defines
  -----------------------------------------------------------------------------*/
 
 static
-cpl_imagelist * cpl_imagelist_load_one(const char *, cpl_type, int, cpl_boolean,
-                                       int, int, int, int) CPL_ATTR_ALLOC;
+cpl_imagelist * cpl_imagelist_load_one(const char *, cpl_type, cpl_size,
+                                       cpl_boolean, cpl_size, cpl_size,
+                                       cpl_size, cpl_size) CPL_ATTR_ALLOC;
 
-/*----------------------------------------------------------------------------*/
-/**
- * @defgroup cpl_imagelist_io Image list IO routines
- *
- * This module provides IO functions to handle @em cpl_imagelist.
- *
- * @par Synopsis:
- * @code
- *   #include "cpl_imagelist_io.h"
- * @endcode
- */
-/*----------------------------------------------------------------------------*/
 /**@{*/
 
 /*-----------------------------------------------------------------------------
@@ -79,6 +70,7 @@ cpl_imagelist * cpl_imagelist_load_one(const char *, cpl_type, int, cpl_boolean,
 
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Create an empty imagelist
   @return   1 newly allocated cpl_imagelist
   @see      cpl_imagelist_set()
@@ -94,6 +86,7 @@ cpl_imagelist * cpl_imagelist_new(void)
 
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Load a FITS file extension into a list of images
   @param    filename   The FITS file name
   @param    im_type    Type of the images in the created image list
@@ -121,7 +114,7 @@ cpl_imagelist * cpl_imagelist_new(void)
 /*----------------------------------------------------------------------------*/
 cpl_imagelist * cpl_imagelist_load(const char * filename,
                                    cpl_type     im_type,
-                                   int          xtnum)
+                                   cpl_size     xtnum)
 {
     cpl_imagelist * self = cpl_imagelist_load_one(filename, im_type, xtnum,
                                                   CPL_FALSE, 0, 0, 0, 0);
@@ -133,6 +126,7 @@ cpl_imagelist * cpl_imagelist_load(const char * filename,
 
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief  Load images windows from a FITS file extension into an image list
   @param  filename    The FITS file name
   @param  im_type     Type of the images in the created image list
@@ -163,11 +157,11 @@ cpl_imagelist * cpl_imagelist_load(const char * filename,
 /*----------------------------------------------------------------------------*/
 cpl_imagelist * cpl_imagelist_load_window(const char * filename,
                                           cpl_type     im_type,
-                                          int          xtnum,
-                                          int          llx,
-                                          int          lly,
-                                          int          urx,
-                                          int          ury)
+                                          cpl_size     xtnum,
+                                          cpl_size     llx,
+                                          cpl_size     lly,
+                                          cpl_size     urx,
+                                          cpl_size     ury)
 
 {
     cpl_imagelist * self = cpl_imagelist_load_one(filename, im_type, xtnum,
@@ -180,6 +174,7 @@ cpl_imagelist * cpl_imagelist_load_window(const char * filename,
 
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Get the number of images in the imagelist
   @param    imlist    the list of image
   @return   The number of images or -1 on error
@@ -188,7 +183,7 @@ cpl_imagelist * cpl_imagelist_load_window(const char * filename,
   - CPL_ERROR_NULL_INPUT if an input pointer is NULL
  */
 /*----------------------------------------------------------------------------*/
-int cpl_imagelist_get_size(const cpl_imagelist * imlist)
+cpl_size cpl_imagelist_get_size(const cpl_imagelist * imlist)
 {
     cpl_ensure(imlist != NULL, CPL_ERROR_NULL_INPUT,
                -1);
@@ -200,6 +195,7 @@ int cpl_imagelist_get_size(const cpl_imagelist * imlist)
 
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Get an image from a list of images
   @param    imlist    the image list
   @param    inum    the image id (from 0 to number of images-1)
@@ -214,7 +210,7 @@ int cpl_imagelist_get_size(const cpl_imagelist * imlist)
  */
 /*----------------------------------------------------------------------------*/
 cpl_image * cpl_imagelist_get(cpl_imagelist * imlist,
-                              int             inum)
+                              cpl_size        inum)
 {
 
     cpl_ensure(imlist != NULL,    CPL_ERROR_NULL_INPUT,          NULL);
@@ -226,6 +222,7 @@ cpl_image * cpl_imagelist_get(cpl_imagelist * imlist,
 
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Get an image from a list of images
   @param    imlist    the image list
   @param    inum    the image id (from 0 to number of images-1)
@@ -234,7 +231,7 @@ cpl_image * cpl_imagelist_get(cpl_imagelist * imlist,
  */
 /*----------------------------------------------------------------------------*/
 const cpl_image * cpl_imagelist_get_const(const cpl_imagelist * imlist,
-                                          int                   inum)
+                                          cpl_size              inum)
 {
     cpl_ensure(imlist != NULL,    CPL_ERROR_NULL_INPUT,          NULL);
     cpl_ensure(inum >= 0,         CPL_ERROR_ILLEGAL_INPUT,       NULL);
@@ -245,6 +242,7 @@ const cpl_image * cpl_imagelist_get_const(const cpl_imagelist * imlist,
 
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Insert an image into an imagelist
   @param    imlist   The imagelist
   @param    im       The image to insert
@@ -279,7 +277,7 @@ const cpl_image * cpl_imagelist_get_const(const cpl_imagelist * imlist,
 cpl_error_code cpl_imagelist_set(
         cpl_imagelist * imlist,
         cpl_image     * im,
-        int             pos)
+        cpl_size        pos)
 {
 
     cpl_ensure_code(imlist,            CPL_ERROR_NULL_INPUT);
@@ -306,7 +304,7 @@ cpl_error_code cpl_imagelist_set(
     if (pos == imlist->ni) {
         imlist->ni++;
         imlist->images = cpl_realloc(imlist->images,
-                                 imlist->ni * sizeof(cpl_image*));
+                                     (size_t)imlist->ni * sizeof(cpl_image*));
     } else {
         /* Check if the image at the position to be overwritten
            is present in only one position */
@@ -329,6 +327,7 @@ cpl_error_code cpl_imagelist_set(
 
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Remove an image from an imagelist
   @param    self   The imagelist
   @param    pos      The list position (from 0 to number of images-1)
@@ -347,10 +346,10 @@ cpl_error_code cpl_imagelist_set(
  */
 /*----------------------------------------------------------------------------*/
 cpl_image * cpl_imagelist_unset(cpl_imagelist * self,
-                                int             pos)
+                                cpl_size        pos)
 {
     cpl_image * out;
-    int         i;
+    cpl_size    i;
 
     cpl_ensure(self, CPL_ERROR_NULL_INPUT, NULL);
     cpl_ensure(pos >= 0, CPL_ERROR_ILLEGAL_INPUT, NULL);
@@ -372,9 +371,16 @@ cpl_image * cpl_imagelist_unset(cpl_imagelist * self,
 
 /*----------------------------------------------------------------------------*/
 /**
-  @internal
+  @ingroup cpl_imagelist
   @brief    Empty an imagelist and deallocate all its images
-  @param    self  The image list
+  @param    self  The image list or NULL
+  @return   Nothing
+  @see cpl_imagelist_delete()
+  @note If @em self is @c NULL nothing is done and no error is set.
+
+  After the call the image list can be populated again. It must eventually
+  be deallocted with a call to cpl_imagelist_delete().
+
  */
 /*----------------------------------------------------------------------------*/
 void cpl_imagelist_empty(cpl_imagelist * self)
@@ -383,7 +389,7 @@ void cpl_imagelist_empty(cpl_imagelist * self)
     if (self != NULL) {
 
         while (self->ni > 0) { /* An iteration may unset more than 1 image */
-            int i = self->ni - 1;
+            cpl_size i = self->ni - 1;
             cpl_image * del = cpl_imagelist_unset(self, i);
 
             cpl_image_delete(del);
@@ -404,25 +410,26 @@ void cpl_imagelist_empty(cpl_imagelist * self)
 
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Free memory associated to a cpl_imagelist object.
-  @param    imlist    the image list
-
+  @param    self    The image list or NULL
   @return   Nothing
+  @see      cpl_imagelist_empty()
+  @note If @em self is @c NULL nothing is done and no error is set.
 
-   The function deallocates the memory used by the imagelist @em imlist.
-   If @em imlist is @c NULL, nothing is done, and no error is set.
+  The function deallocates the memory used by the imagelist @em self.
 
  */
 /*----------------------------------------------------------------------------*/
-void cpl_imagelist_delete(cpl_imagelist * imlist)
+void cpl_imagelist_delete(cpl_imagelist * self)
 {
 
-    if (imlist != NULL) {
+    if (self != NULL) {
 
-        cpl_imagelist_empty(imlist);
+        cpl_imagelist_empty(self);
 
-        cpl_free(imlist->images);
-        cpl_free(imlist);
+        cpl_free(self->images);
+        cpl_free(self);
     }
 
     return;
@@ -430,6 +437,7 @@ void cpl_imagelist_delete(cpl_imagelist * imlist)
 
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Copy an image list
   @param    imlist    Source image list.
   @return   1 newly allocated image list, or NULL on error.
@@ -444,7 +452,7 @@ void cpl_imagelist_delete(cpl_imagelist * imlist)
 cpl_imagelist * cpl_imagelist_duplicate(const cpl_imagelist * imlist)
 {
     cpl_imagelist *   out;
-    int               i;
+    cpl_size          i;
 
     cpl_ensure(imlist != NULL, CPL_ERROR_NULL_INPUT, NULL);
 
@@ -461,6 +469,7 @@ cpl_imagelist * cpl_imagelist_duplicate(const cpl_imagelist * imlist)
 
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Reject one or more images in a list according to an array of flags.
   @param    imlist  Non-empty imagelist to examine for image rejection.
   @param    valid   Vector of flags (>=-0.5: valid, <-0.5: invalid)
@@ -483,8 +492,8 @@ cpl_error_code cpl_imagelist_erase(
         cpl_imagelist    * imlist,
         const cpl_vector * valid)
 {
-    int          nkeep = 0;
-    int          i;
+    cpl_size     nkeep = 0;
+    cpl_size     i;
 
     /* Check entries */
     cpl_ensure_code(imlist, CPL_ERROR_NULL_INPUT);
@@ -511,10 +520,11 @@ cpl_error_code cpl_imagelist_erase(
 
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Save an imagelist to disk in FITS format.
   @param    self      Imagelist to save
   @param    filename  Name of the FITS file to write
-  @param    bpp       Bits per pixels
+  @param    type      The type used to represent the data in the file
   @param    pl        Property list for the output header or NULL
   @param    mode      The desired output options (combined with bitwise or)
   @return   the #_cpl_error_code_ or CPL_ERROR_NONE
@@ -524,16 +534,19 @@ cpl_error_code cpl_imagelist_erase(
   property list is provided, it is written to the named file before the
   pixels are written.
 
-  The requested pixel depth (bpp) follows the FITS convention. Possible values
-  are CPL_BPP_8_UNSIGNED (8), CPL_BPP_16_SIGNED (16), CPL_BPP_16_UNSIGNED (-16),
-  CPL_BPP_32_SIGNED (32), CPL_BPP_IEEE_FLOAT (-32), CPL_BPP_IEEE_DOUBLE (-64)
-
-  In the case of CPL_BPP_16_UNSIGNED, the BITPIX stays 16, the data are
-  stored as signed (32768 is subtracted to the images) and BZERO is set to
-  32768 as specified by the FITS standard.
-
   Supported image lists types are CPL_TYPE_DOUBLE, CPL_TYPE_FLOAT, CPL_TYPE_INT.
 
+  The type used in the file can be one of:
+     CPL_TYPE_UCHAR  (8 bit unsigned),
+     CPL_TYPE_SHORT  (16 bit signed),
+     CPL_TYPE_USHORT (16 bit unsigned),
+     CPL_TYPE_INT    (32 bit signed),
+     CPL_TYPE_FLOAT  (32 bit floating point), or
+     CPL_TYPE_DOUBLE (64 bit floating point). Additionally, the special value
+     CPL_TYPE_UNSPECIFIED is allowed. This value means that the type used
+  for saving is the pixel type of the input image. Using the image pixel type
+  as saving type ensures that the saving incurs no loss of information.
+
   Supported output modes are CPL_IO_CREATE (create a new file),
   CPL_IO_EXTEND (extend an existing file with a new extension) and
   CPL_IO_APPEND (append list of images to the last data unit, which must
@@ -546,39 +559,38 @@ cpl_error_code cpl_imagelist_erase(
 
   Possible #_cpl_error_code_ set in this function:
   - CPL_ERROR_NULL_INPUT if an input pointer is NULL
-  - CPL_ERROR_ILLEGAL_INPUT if the bpp value or the mode is not supported
+  - CPL_ERROR_ILLEGAL_INPUT if the type or the mode is not supported
   - CPL_ERROR_FILE_IO if the file cannot be written
   - CPL_ERROR_INVALID_TYPE if the passed image list type is not supported
  */
 /*----------------------------------------------------------------------------*/
 cpl_error_code cpl_imagelist_save(const cpl_imagelist    * self,
                                   const char             * filename,
-                                  cpl_type_bpp             bpp,
+                                  cpl_type                 type,
                                   const cpl_propertylist * pl,
                                   unsigned                 mode)
 {
     int               error=0;
     fitsfile       *  fptr;
-    int               i;
+    cpl_size          i;
     const char      * badkeys = mode & CPL_IO_EXTEND ?
     CPL_FITS_BADKEYS_EXT  "|" CPL_FITS_COMPRKEYS :
     CPL_FITS_BADKEYS_PRIM "|" CPL_FITS_COMPRKEYS;
     const cpl_image * first = cpl_imagelist_get_const(self, 0);
-    const int         nx = cpl_image_get_size_x(first);
-    const int         ny = cpl_image_get_size_y(first);
-    const cpl_type    type = cpl_image_get_type(first);
-    const int         cfitsiotype = type == CPL_TYPE_DOUBLE ? TDOUBLE
-    : (type == CPL_TYPE_FLOAT ? TFLOAT : TINT);
-    const long        naxes[3] = {nx, ny, cpl_imagelist_get_size(self)};
+    const cpl_size    nx = cpl_image_get_size_x(first);
+    const cpl_size    ny = cpl_image_get_size_y(first);
+    const cpl_type    ftype = cpl_image_get_type(first);
+    const int         cfitsiotype = ftype == CPL_TYPE_DOUBLE ? TDOUBLE
+    : (ftype == CPL_TYPE_FLOAT ? TFLOAT : TINT);
+    const CPL_FITSIO_TYPE naxes[3] = {nx, ny, cpl_imagelist_get_size(self)};
     long              plane1 = 0; /* First plane to write */
+    const int         ibpp = cpl_tools_get_bpp(type == CPL_TYPE_UNSPECIFIED ?
+                                               ftype : type);
 
     /* Test entries */
     cpl_ensure_code(filename != NULL, CPL_ERROR_NULL_INPUT);
     cpl_ensure_code(self     != NULL, CPL_ERROR_NULL_INPUT);
-    cpl_ensure_code(bpp==CPL_BPP_8_UNSIGNED  || bpp==CPL_BPP_16_SIGNED ||
-                    bpp==CPL_BPP_16_UNSIGNED || bpp==CPL_BPP_32_SIGNED ||
-                    bpp==CPL_BPP_IEEE_FLOAT  || bpp==CPL_BPP_IEEE_DOUBLE,
-                    CPL_ERROR_ILLEGAL_INPUT);
+    cpl_ensure_code(ibpp, CPL_ERROR_ILLEGAL_INPUT);
 
     cpl_ensure_code(((mode & CPL_IO_CREATE) != 0) +
                     ((mode & CPL_IO_EXTEND) != 0) +
@@ -589,26 +601,25 @@ cpl_error_code cpl_imagelist_save(const cpl_imagelist    * self,
     cpl_ensure_code(cpl_imagelist_is_uniform(self)==0,
                     CPL_ERROR_ILLEGAL_INPUT);
 
-    cpl_ensure_code(type == CPL_TYPE_DOUBLE ||
-            type == CPL_TYPE_FLOAT ||
-            type == CPL_TYPE_INT, CPL_ERROR_INVALID_TYPE);
+    cpl_ensure_code(ftype == CPL_TYPE_DOUBLE || ftype == CPL_TYPE_FLOAT ||
+                    ftype == CPL_TYPE_INT, CPL_ERROR_INVALID_TYPE);
 
     if (mode & (CPL_IO_EXTEND | CPL_IO_APPEND)) {
         /* Open the file */
-        if (fits_open_diskfile(&fptr, filename, READWRITE, &error)) {
+        if (cpl_io_fits_open_diskfile(&fptr, filename, READWRITE, &error)) {
             return cpl_error_set_fits(CPL_ERROR_FILE_IO, error,
                                       fits_open_diskfile, "filename='%s', "
-                                      "bpp=%d, mode=%d", filename, bpp, mode);
+                                      "type=%d, mode=%d", filename, type, mode);
         }
     } else {
         /* Create the file */
         char * sval = cpl_sprintf("!%s", filename);
-        fits_create_file(&fptr, sval, &error);
+        cpl_io_fits_create_file(&fptr, sval, &error);
         cpl_free(sval);
         if (error != 0) {
             return cpl_error_set_fits(CPL_ERROR_FILE_IO, error,
                                       fits_create_file, "filename='%s', "
-                                      "bpp=%d, mode=%d", filename, bpp, mode);
+                                      "type=%d, mode=%d", filename, type, mode);
         }
     }
 
@@ -628,69 +639,72 @@ cpl_error_code cpl_imagelist_save(const cpl_imagelist    * self,
             fits_get_img_dim(fptr, &naxis, &error);
 
             if (naxis == 2 || naxis == 3) {
-                long fnaxes[3] = {nx, ny, 0}; /* Last element read on naxis=2 */
+                /* Last element read on naxis=2 */
+                CPL_FITSIO_TYPE fnaxes[3] = {nx, ny, 0};
                 int bitpix;
 
                 fits_get_img_type(fptr, &bitpix, &error);
 
-                if (bitpix == bpp) {
+                if (bitpix == ibpp) {
 
-                    fits_get_img_size(fptr, naxis, fnaxes, &error);
+                    CPL_FITSIO_GET_SIZE(fptr, naxis, fnaxes, &error);
 
                     if (naxes[0] == fnaxes[0] && naxes[1] == fnaxes[1]) {
                         plane1 = fnaxes[2];
                         fnaxes[2] += naxes[2];
-                        fits_resize_img(fptr, bitpix, naxis, fnaxes, &error);
+                        CPL_FITSIO_RESIZE_IMG(fptr, bitpix, naxis, fnaxes,
+                                              &error);
                         if (!error) isok = CPL_TRUE;
                     } else {
                     int error2 = 0;
-                    fits_close_file(fptr, &error2);
+                    cpl_io_fits_close_file(fptr, &error2);
                     return cpl_error_set_message_
                         (CPL_ERROR_ILLEGAL_INPUT, "Data-unit %d in file %s has "
-                         "wrong NAXIS1/2 (%ldX%ld != %ldX%ld). bpp=%d, "
-                         "mode=%d)", next, filename, fnaxes[0], fnaxes[1],
-                         naxes[0], naxes[1], bpp, mode);
+                         "wrong NAXIS1/2 (%ldX%ld != %ldX%ld). type=%d, "
+                         "mode=%d)", next, filename, (long)fnaxes[0],
+                         (long)fnaxes[1], (long)naxes[0], (long)naxes[1],
+                         type, mode);
                     }
                 } else {
                     int error2 = 0;
-                    fits_close_file(fptr, &error2);
+                    cpl_io_fits_close_file(fptr, &error2);
                     return cpl_error_set_message_
                         (CPL_ERROR_ILLEGAL_INPUT, "Data-unit %d in file %s has "
-                         "wrong BITPIX (%d != %d). mode=%d)", next, filename,
-                         bitpix, bpp, mode);
+                         "wrong BITPIX (%d != %d). type=%d, mode=%d)", next,
+                         filename, bitpix, ibpp, type, mode);
                 }
             } else {
                 int error2 = 0;
-                fits_close_file(fptr, &error2);
+                cpl_io_fits_close_file(fptr, &error2);
                 return cpl_error_set_message_
                     (CPL_ERROR_ILLEGAL_INPUT, "Data-unit %d in file %s has "
-                     "wrong NAXIS (%d != 2/3). bpp=%d, mode=%d)", next,
-                     filename, naxis, bpp, mode);
+                     "wrong NAXIS (%d != 2/3). type=%d, mode=%d)", next,
+                     filename, naxis, type, mode);
             }
         } else {
             int error2 = 0;
-            fits_close_file(fptr, &error2);
+            cpl_io_fits_close_file(fptr, &error2);
             return cpl_error_set_message_
                 (CPL_ERROR_ILLEGAL_INPUT, "Data-unit %d in file %s is not "
-                 "image-type (%d != %d. bpp=%d, mode=%d)",
-                 next, filename, IMAGE_HDU, exttype, bpp, mode);
+                 "image-type (%d != %d. type=%d, mode=%d)",
+                 next, filename, exttype, IMAGE_HDU, type, mode);
         }
 
         if (error) {
             int error2 = 0;
-            fits_close_file(fptr, &error2);
+            cpl_io_fits_close_file(fptr, &error2);
             return cpl_error_set_fits(CPL_ERROR_FILE_IO, error, "CFITSIO",
-                                      "filename='%s', bpp=%d, mode=%d",
-                                      filename, bpp, mode);
+                                      "filename='%s', type=%d, mode=%d",
+                                      filename, type, mode);
         } else if (!isok) {
             /* Not supposed to enter here */
             int error2 = 0;
-            fits_close_file(fptr, &error2);
+            cpl_io_fits_close_file(fptr, &error2);
             return cpl_error_set_(CPL_ERROR_UNSPECIFIED);
         }
     } else {
         /* Create the imagelist in the file */
-        fits_create_img(fptr, bpp, 3, (long*)naxes, &error);
+        CPL_FITSIO_CREATE_IMG(fptr, ibpp, 3, (CPL_FITSIO_TYPE*)naxes, &error);
 
         if (mode & CPL_IO_CREATE)
             fits_write_date(fptr, &error); /* Add Date */
@@ -698,48 +712,49 @@ cpl_error_code cpl_imagelist_save(const cpl_imagelist    * self,
 
     if (error) {
         int error2 = 0;
-        fits_close_file(fptr, &error2);
+        cpl_io_fits_close_file(fptr, &error2);
         return cpl_error_set_fits(CPL_ERROR_FILE_IO, error, "CFITSIO",
-                                  "filename='%s', bpp=%d, mode=%d",
-                                  filename, bpp, mode);
+                                  "filename='%s', type=%d, mode=%d",
+                                  filename, type, mode);
     }
 
     /* Add the property list */
     if (cpl_fits_add_properties(fptr, pl, badkeys)) {
-        fits_close_file(fptr, &error);
+        cpl_io_fits_close_file(fptr, &error);
         return cpl_error_set_(CPL_ERROR_ILLEGAL_INPUT);
     }
 
     /* Loop on the images */
     for (i=0; i < self->ni; i++) {
-        const long        fpixel[3] = {1, 1, plane1 + i + 1};
-    const cpl_image * image = cpl_imagelist_get_const(self, i);
-    const void      * data  = cpl_image_get_data_const(image);
+    const CPL_FITSIO_TYPE fpixel[3] = {1, 1, plane1 + i + 1};
+    const cpl_image     * image = cpl_imagelist_get_const(self, i);
+    const void          * data  = cpl_image_get_data_const(image);
 
     /* Write the pixels */
-    if (fits_write_pix(fptr, cfitsiotype, (long*)fpixel, nx*ny,
-                           (void*)data, &error)) break;
+    if (CPL_FITSIO_WRITE_PIX(fptr, cfitsiotype, (CPL_FITSIO_TYPE*)fpixel,
+                             nx*ny, (void*)data, &error)) break;
     }
 
     /* Check  */
     if (error) {
         int error2 = 0;
-        fits_close_file(fptr, &error2);
+        cpl_io_fits_close_file(fptr, &error2);
         return cpl_error_set_fits(CPL_ERROR_FILE_IO, error,
                                   "CFITSIO", "filename='%s', "
-                                  "bpp=%d, mode=%d", filename, bpp, mode);
+                                  "type=%d, mode=%d", filename, type, mode);
     }
 
     /* Close (and write to disk) */
-    return fits_close_file(fptr, &error)
+    return cpl_io_fits_close_file(fptr, &error)
         ? cpl_error_set_fits(CPL_ERROR_FILE_IO, error,
                              fits_close_file, "filename='%s', "
-                             "bpp=%d, mode=%d", filename, bpp, mode)
+                             "type=%d, mode=%d", filename, type, mode)
         : CPL_ERROR_NONE;
 }
 
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Determine if an imagelist contains images of equal size and type
   @param    imlist    The imagelist to check
   @return   Zero if uniform, positive if non-uniform and negative on error.
@@ -753,8 +768,8 @@ cpl_error_code cpl_imagelist_save(const cpl_imagelist    * self,
 int cpl_imagelist_is_uniform(const cpl_imagelist * imlist)
 {
     cpl_type     type;
-    int          nx, ny;
-    int          i;
+    cpl_size     nx, ny;
+    cpl_size     i;
 
     cpl_ensure(imlist != NULL, CPL_ERROR_NULL_INPUT, -1);
     if (imlist->ni == 0) return 1;
@@ -765,15 +780,16 @@ int cpl_imagelist_is_uniform(const cpl_imagelist * imlist)
     type = cpl_image_get_type  (imlist->images[0]);
 
     for (i=1; i < imlist->ni; i++) {
-        if (cpl_image_get_size_x(imlist->images[i]) != nx)   return i+1;
-        if (cpl_image_get_size_y(imlist->images[i]) != ny)   return i+1;
-        if (cpl_image_get_type  (imlist->images[i]) != type) return i+1;
+        if (cpl_image_get_size_x(imlist->images[i]) != nx)   return (int)i+1;
+        if (cpl_image_get_size_y(imlist->images[i]) != ny)   return (int)i+1;
+        if (cpl_image_get_type  (imlist->images[i]) != type) return (int)i+1;
     }
     return 0;
 }
 
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Dump structural information of images in an imagelist
   @param    self    Imagelist to dump
   @param    stream  Output stream, accepts @c stdout or @c stderr
@@ -788,7 +804,7 @@ cpl_error_code cpl_imagelist_dump_structure(const cpl_imagelist * self,
                         FILE * stream)
 {
     const char * msg    = "Imagelist with %d image(s)\n";
-    const int    msgmin = strlen(msg) - 5;
+    const int    msgmin = (int)strlen(msg) - 5;
 
     int i;
 
@@ -801,7 +817,7 @@ cpl_error_code cpl_imagelist_dump_structure(const cpl_imagelist * self,
     for (i = 0; i < self -> ni; i++) {
     const cpl_image * image   = cpl_imagelist_get_const(self, i);
     const char      * imsg    = "Image nb %d of %d in imagelist\n";
-    const int         imsgmin = strlen(imsg) - 5;
+    const int         imsgmin = (int)strlen(imsg) - 5;
 
     cpl_ensure_code( fprintf(stream,  imsg, i, self->ni) >= imsgmin,
              CPL_ERROR_FILE_IO );
@@ -815,6 +831,7 @@ cpl_error_code cpl_imagelist_dump_structure(const cpl_imagelist * self,
 
 /*----------------------------------------------------------------------------*/
 /**
+  @ingroup cpl_imagelist
   @brief    Dump pixel values of images in a CPL imagelist
   @param    self    Imagelist to dump
   @param    llx     Specifies the window position
@@ -832,10 +849,11 @@ cpl_error_code cpl_imagelist_dump_structure(const cpl_imagelist * self,
  */
 /*----------------------------------------------------------------------------*/
 cpl_error_code cpl_imagelist_dump_window(const cpl_imagelist * self,
-                     int llx, int lly, int urx, int ury,
-                     FILE * stream)
+                                         cpl_size llx, cpl_size lly,
+                                         cpl_size urx, cpl_size ury,
+                                         FILE * stream)
 {
-    int i;
+    cpl_size i;
 
     cpl_ensure_code(self   != NULL, CPL_ERROR_NULL_INPUT);
     cpl_ensure_code(stream != NULL, CPL_ERROR_NULL_INPUT);
@@ -843,7 +861,7 @@ cpl_error_code cpl_imagelist_dump_window(const cpl_imagelist * self,
     for (i = 0; i < self -> ni; i++) {
     const cpl_image * image   = cpl_imagelist_get_const(self, i);
     const char      * imsg    = "Image nb %d of %d in imagelist\n";
-    const int         imsgmin = strlen(imsg) - 5;
+    const int         imsgmin = (int)strlen(imsg) - 5;
 
     cpl_ensure_code( fprintf(stream,  imsg, i, self->ni) >= imsgmin,
              CPL_ERROR_FILE_IO );
@@ -861,6 +879,7 @@ cpl_error_code cpl_imagelist_dump_window(const cpl_imagelist * self,
 /*----------------------------------------------------------------------------*/
 /**
   @internal
+  @ingroup cpl_imagelist
   @brief  Load an imagelist from a FITS file.
   @param  filename    Name of the file to load from.
   @param  im_type     Type of the created images
@@ -876,23 +895,23 @@ cpl_error_code cpl_imagelist_dump_window(const cpl_imagelist * self,
 /*----------------------------------------------------------------------------*/
 static cpl_imagelist * cpl_imagelist_load_one(const char * filename,
                                               cpl_type     im_type,
-                                              int          xtnum,
+                                              cpl_size     xtnum,
                                               cpl_boolean  do_window,
-                                              int          llx,
-                                              int          lly,
-                                              int          urx,
-                                              int          ury)
+                                              cpl_size     llx,
+                                              cpl_size     lly,
+                                              cpl_size     urx,
+                                              cpl_size     ury)
 {
 
     /* Count number of images read - use also to indicate failure */
-    int             selfsize;
+    cpl_size        selfsize;
     cpl_imagelist * self;
     cpl_image     * image;
     fitsfile      * fptr;
     int             error = 0;
     /* Initialize to indicate that they need to be read from the file */
     int             naxis = 0;
-    long int        naxes[3] ={0, 0, 0};
+    CPL_FITSIO_TYPE naxes[3] ={0, 0, 0};
     cpl_type        pix_type = im_type;
 
     /* FIXME: Version 3.2 of fits_open_diskfile() seg-faults on NULL.
@@ -900,10 +919,10 @@ static cpl_imagelist * cpl_imagelist_load_one(const char * filename,
     cpl_ensure(filename != NULL, CPL_ERROR_NULL_INPUT,    NULL);
     cpl_ensure(xtnum    >= 0,    CPL_ERROR_ILLEGAL_INPUT, NULL);
 
-    if (fits_open_diskfile(&fptr, filename, READONLY, &error)) {
+    if (cpl_io_fits_open_diskfile(&fptr, filename, READONLY, &error)) {
         (void)cpl_error_set_fits(CPL_ERROR_FILE_IO, error, fits_open_diskfile,
-                                 "filename='%s', im_type=%u, xtnum=%d",
-                                 filename, im_type, xtnum);
+                                 "filename='%s', im_type=%u, xtnum=%"
+                                 CPL_SIZE_FORMAT, filename, im_type, xtnum);
         return NULL;
     }
 
@@ -927,7 +946,7 @@ static cpl_imagelist * cpl_imagelist_load_one(const char * filename,
 
         for (iplane = 1; iplane < naxes[2]; iplane++) {
             image = cpl_image_load_(fptr, &naxis, naxes, &pix_type,
-                                    filename, iplane, 0,
+                                    filename, iplane, xtnum,
                                     do_window, llx, lly, urx, ury);
             if (image == NULL) break;
 
@@ -943,10 +962,10 @@ static cpl_imagelist * cpl_imagelist_load_one(const char * filename,
         }
     }
 
-    if (fits_close_file(fptr, &error)) {
+    if (cpl_io_fits_close_file(fptr, &error)) {
         (void)cpl_error_set_fits(CPL_ERROR_BAD_FILE_FORMAT, error,
                                  fits_close_file, "filename='%s', "
-                                 "im_type=%u, xtnum=%d",
+                                 "im_type=%u, xtnum=%" CPL_SIZE_FORMAT,
                                  filename, (unsigned)im_type, xtnum);
         selfsize = 0; /* Indicate failure */
     } else if (selfsize == 0) {
