@@ -1,4 +1,4 @@
-/* $Id: cpl_mask.h,v 1.29 2010/11/11 09:23:18 llundin Exp $
+/* $Id: cpl_mask.h,v 1.33 2011/08/11 11:08:33 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/11/11 09:23:18 $
- * $Revision: 1.29 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2011/08/11 11:08:33 $
+ * $Revision: 1.33 $
+ * $Name: cpl-6_0 $
  */
 
 #ifndef CPL_MASK_H
@@ -62,41 +62,49 @@ typedef unsigned char   cpl_binary;
  -----------------------------------------------------------------------------*/
 
 /* IO operations */
-cpl_mask * cpl_mask_new(int, int) CPL_ATTR_ALLOC;
-cpl_mask * cpl_mask_wrap(int, int, cpl_binary *) CPL_ATTR_ALLOC;
+cpl_mask * cpl_mask_new(cpl_size, cpl_size) CPL_ATTR_ALLOC;
+cpl_mask * cpl_mask_wrap(cpl_size, cpl_size, cpl_binary *) CPL_ATTR_ALLOC;
 cpl_mask * cpl_mask_duplicate(const cpl_mask *) CPL_ATTR_ALLOC;
 void cpl_mask_delete(cpl_mask *);
 void * cpl_mask_unwrap(cpl_mask *);
-cpl_error_code cpl_mask_dump_window(const cpl_mask *, int, int,
-                                    int, int, FILE *);
+cpl_error_code cpl_mask_dump_window(const cpl_mask *, cpl_size, cpl_size,
+                                    cpl_size, cpl_size, FILE *);
+
+cpl_mask * cpl_mask_load(const char *, cpl_size, cpl_size) CPL_ATTR_ALLOC;
+cpl_mask * cpl_mask_load_window(const char *, cpl_size, cpl_size, cpl_size,
+                                cpl_size, cpl_size, cpl_size) CPL_ATTR_ALLOC;
+cpl_error_code cpl_mask_save(const cpl_mask *, const char *,
+                             const cpl_propertylist *, unsigned);
 
 /* Accessor functions */
 cpl_binary * cpl_mask_get_data(cpl_mask *);
 const cpl_binary * cpl_mask_get_data_const(const cpl_mask *);
-cpl_binary cpl_mask_get(const cpl_mask *, int, int);
-cpl_error_code cpl_mask_set(cpl_mask *, int, int, cpl_binary);
-int cpl_mask_get_size_x(const cpl_mask *);
-int cpl_mask_get_size_y(const cpl_mask *);
+cpl_binary cpl_mask_get(const cpl_mask *, cpl_size, cpl_size);
+cpl_error_code cpl_mask_set(cpl_mask *, cpl_size, cpl_size, cpl_binary);
+cpl_size cpl_mask_get_size_x(const cpl_mask *);
+cpl_size cpl_mask_get_size_y(const cpl_mask *);
 
 /* Basic operations */
 cpl_boolean cpl_mask_is_empty(const cpl_mask *);
-cpl_boolean cpl_mask_is_empty_window(const cpl_mask *, int, int, int, int);
-int cpl_mask_count(const cpl_mask *);
-int cpl_mask_count_window(const cpl_mask *, int, int, int, int);
+cpl_boolean cpl_mask_is_empty_window(const cpl_mask *, cpl_size, cpl_size,
+                                     cpl_size, cpl_size);
+cpl_size cpl_mask_count(const cpl_mask *);
+cpl_size cpl_mask_count_window(const cpl_mask *, cpl_size, cpl_size,
+                               cpl_size, cpl_size);
 cpl_error_code cpl_mask_and(cpl_mask *, const cpl_mask *);
 cpl_error_code cpl_mask_or(cpl_mask *, const cpl_mask *);
 cpl_error_code cpl_mask_xor(cpl_mask *, const cpl_mask *);
 cpl_error_code cpl_mask_not(cpl_mask *);
 cpl_mask * cpl_mask_collapse_create(const cpl_mask *, int) CPL_ATTR_ALLOC;
-cpl_mask * cpl_mask_extract(const cpl_mask *, int, int,
-                            int, int) CPL_ATTR_ALLOC;
+cpl_mask * cpl_mask_extract(const cpl_mask *, cpl_size, cpl_size,
+                            cpl_size, cpl_size) CPL_ATTR_ALLOC;
 cpl_error_code cpl_mask_turn(cpl_mask *, int);
-cpl_error_code cpl_mask_shift(cpl_mask *, int, int);
-cpl_error_code cpl_mask_copy(cpl_mask *, const cpl_mask *, int, int);
+cpl_error_code cpl_mask_shift(cpl_mask *, cpl_size, cpl_size);
+cpl_error_code cpl_mask_copy(cpl_mask *, const cpl_mask *, cpl_size, cpl_size);
 cpl_error_code cpl_mask_flip(cpl_mask *, int);
-cpl_error_code cpl_mask_move(cpl_mask *, int, const int *);
+cpl_error_code cpl_mask_move(cpl_mask *, cpl_size, const cpl_size *);
 cpl_mask * cpl_mask_extract_subsample(const cpl_mask *,
-                                      int, int) CPL_ATTR_ALLOC;
+                                      cpl_size, cpl_size) CPL_ATTR_ALLOC;
 
 /* Morphological operations */
 cpl_error_code cpl_mask_filter(cpl_mask *, const cpl_mask *, const cpl_mask *,
@@ -111,6 +119,8 @@ cpl_error_code cpl_mask_dilation(cpl_mask *,
                                  const cpl_matrix *) CPL_ATTR_DEPRECATED;
 
 /* Zones selection */
+cpl_error_code cpl_mask_threshold_image(cpl_mask *, const cpl_image *,
+                                        double, double, cpl_binary);
 cpl_mask * cpl_mask_threshold_image_create(const cpl_image *,
                                            double, double) CPL_ATTR_ALLOC;
 
