@@ -1,4 +1,4 @@
-/* $Id: cpl_matrix.h,v 1.37 2010/11/23 11:02:06 llundin Exp $
+/* $Id: cpl_matrix.h,v 1.40 2011/07/29 12:40:08 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/11/23 11:02:06 $
- * $Revision: 1.37 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2011/07/29 12:40:08 $
+ * $Revision: 1.40 $
+ * $Name: cpl-6_0 $
  */
 
 #ifndef CPL_MATRIX_H
@@ -32,10 +32,12 @@
                                    Includes
  -----------------------------------------------------------------------------*/
 
-#include <stdio.h>
-
 #include <cpl_macros.h>
+#include <cpl_type.h>
 #include <cpl_error.h>
+#include <cpl_array.h>
+
+#include <stdio.h>
 
 CPL_BEGIN_DECLS
 
@@ -53,8 +55,8 @@ typedef struct _cpl_matrix_ cpl_matrix;
  * IO methods
  */
 
-cpl_matrix *cpl_matrix_new(int, int) CPL_ATTR_ALLOC;
-cpl_matrix *cpl_matrix_wrap(int, int, double *) CPL_ATTR_ALLOC;
+cpl_matrix *cpl_matrix_new(cpl_size, cpl_size) CPL_ATTR_ALLOC;
+cpl_matrix *cpl_matrix_wrap(cpl_size, cpl_size, double *) CPL_ATTR_ALLOC;
 void cpl_matrix_delete(cpl_matrix *);
 void *cpl_matrix_unwrap(cpl_matrix *);
 void cpl_matrix_dump(const cpl_matrix *, FILE *);
@@ -63,12 +65,12 @@ void cpl_matrix_dump(const cpl_matrix *, FILE *);
  * Accessors 
  */
 
-int cpl_matrix_get_nrow(const cpl_matrix *);
-int cpl_matrix_get_ncol(const cpl_matrix *);
+cpl_size cpl_matrix_get_nrow(const cpl_matrix *);
+cpl_size cpl_matrix_get_ncol(const cpl_matrix *);
       double *cpl_matrix_get_data(cpl_matrix *);
 const double *cpl_matrix_get_data_const(const cpl_matrix *);
-double cpl_matrix_get(const cpl_matrix *, int, int);
-cpl_error_code cpl_matrix_set(cpl_matrix *, int, int, double);
+double cpl_matrix_get(const cpl_matrix *, cpl_size, cpl_size);
+cpl_error_code cpl_matrix_set(cpl_matrix *, cpl_size, cpl_size, double);
 
 /*
  * Copying methods
@@ -76,22 +78,26 @@ cpl_error_code cpl_matrix_set(cpl_matrix *, int, int, double);
 
 cpl_matrix *cpl_matrix_duplicate(const cpl_matrix *) CPL_ATTR_ALLOC;
 cpl_matrix *cpl_matrix_extract(const cpl_matrix *, 
-                               int, int, int, int, int, int) CPL_ATTR_ALLOC;
-cpl_matrix *cpl_matrix_extract_row(const cpl_matrix *, int) CPL_ATTR_ALLOC;
-cpl_matrix *cpl_matrix_extract_column(const cpl_matrix *, int) CPL_ATTR_ALLOC;
-cpl_matrix *cpl_matrix_extract_diagonal(const cpl_matrix *, int) CPL_ATTR_ALLOC;
+                               cpl_size, cpl_size, cpl_size, cpl_size,
+                               cpl_size, cpl_size) CPL_ATTR_ALLOC;
+cpl_matrix *cpl_matrix_extract_row(const cpl_matrix *, cpl_size) CPL_ATTR_ALLOC;
+cpl_matrix *cpl_matrix_extract_column(const cpl_matrix *, cpl_size)
+    CPL_ATTR_ALLOC;
+cpl_matrix *cpl_matrix_extract_diagonal(const cpl_matrix *, cpl_size)
+    CPL_ATTR_ALLOC;
 
 /*
  * Writing methods
  */
 
-cpl_error_code cpl_matrix_copy(cpl_matrix *, const cpl_matrix *, int, int);
+cpl_error_code cpl_matrix_copy(cpl_matrix *, const cpl_matrix *,
+                               cpl_size, cpl_size);
 cpl_error_code cpl_matrix_fill(cpl_matrix *, double);
-cpl_error_code cpl_matrix_fill_row(cpl_matrix *, double, int);
-cpl_error_code cpl_matrix_fill_column(cpl_matrix *, double, int);
-cpl_error_code cpl_matrix_fill_diagonal(cpl_matrix *, double, int);
+cpl_error_code cpl_matrix_fill_row(cpl_matrix *, double, cpl_size);
+cpl_error_code cpl_matrix_fill_column(cpl_matrix *, double, cpl_size);
+cpl_error_code cpl_matrix_fill_diagonal(cpl_matrix *, double, cpl_size);
 cpl_error_code cpl_matrix_fill_window(cpl_matrix *, double, 
-                                         int, int, int, int);
+                                      cpl_size, cpl_size, cpl_size, cpl_size);
 
 /*
  * Test methods
@@ -113,22 +119,23 @@ cpl_error_code cpl_matrix_sort_columns(cpl_matrix *, int);
  */
 
 cpl_error_code cpl_matrix_threshold_small(cpl_matrix *, double);
-cpl_error_code cpl_matrix_swap_rows(cpl_matrix *, int, int);
-cpl_error_code cpl_matrix_swap_columns(cpl_matrix *, int, int);
-cpl_error_code cpl_matrix_swap_rowcolumn(cpl_matrix *, int);
+cpl_error_code cpl_matrix_swap_rows(cpl_matrix *, cpl_size, cpl_size);
+cpl_error_code cpl_matrix_swap_columns(cpl_matrix *, cpl_size, cpl_size);
+cpl_error_code cpl_matrix_swap_rowcolumn(cpl_matrix *, cpl_size);
 cpl_error_code cpl_matrix_flip_rows(cpl_matrix *);
 cpl_error_code cpl_matrix_flip_columns(cpl_matrix *);
-cpl_error_code cpl_matrix_erase_rows(cpl_matrix *, int, int);
-cpl_error_code cpl_matrix_erase_columns(cpl_matrix *, int, int);
-cpl_error_code cpl_matrix_set_size(cpl_matrix *, int, int);
-cpl_error_code cpl_matrix_resize(cpl_matrix *, int, int, int, int);
+cpl_error_code cpl_matrix_erase_rows(cpl_matrix *, cpl_size, cpl_size);
+cpl_error_code cpl_matrix_erase_columns(cpl_matrix *, cpl_size, cpl_size);
+cpl_error_code cpl_matrix_set_size(cpl_matrix *, cpl_size, cpl_size);
+cpl_error_code cpl_matrix_resize(cpl_matrix *, cpl_size, cpl_size,
+                                 cpl_size, cpl_size);
 cpl_error_code cpl_matrix_append(cpl_matrix *, const cpl_matrix *, int);   
 
 /*
  * Basic operations
  */
 
-cpl_error_code cpl_matrix_shift(cpl_matrix *, int, int);
+cpl_error_code cpl_matrix_shift(cpl_matrix *, cpl_size, cpl_size);
 cpl_error_code cpl_matrix_add(cpl_matrix *, const cpl_matrix *);
 cpl_error_code cpl_matrix_subtract(cpl_matrix *, const cpl_matrix *);
 cpl_error_code cpl_matrix_multiply(cpl_matrix *, const cpl_matrix *);
@@ -156,9 +163,9 @@ cpl_matrix *cpl_matrix_solve_normal(const cpl_matrix *,
                                     const cpl_matrix *) CPL_ATTR_ALLOC;
 cpl_matrix *cpl_matrix_invert_create(const cpl_matrix *) CPL_ATTR_ALLOC;
 
-cpl_error_code cpl_matrix_decomp_lu(cpl_matrix *, int *, int *);
+cpl_error_code cpl_matrix_decomp_lu(cpl_matrix *, cpl_array *, int *);
 cpl_error_code cpl_matrix_solve_lu(const cpl_matrix *, cpl_matrix *,
-                                   const int *);
+                                   const cpl_array *);
 cpl_error_code cpl_matrix_decomp_chol(cpl_matrix *);
 cpl_error_code cpl_matrix_solve_chol(const cpl_matrix *, cpl_matrix *);
 
@@ -171,8 +178,10 @@ double cpl_matrix_get_median(const cpl_matrix *);
 double cpl_matrix_get_stdev(const cpl_matrix *);
 double cpl_matrix_get_min(const cpl_matrix *);
 double cpl_matrix_get_max(const cpl_matrix *);
-cpl_error_code cpl_matrix_get_minpos(const cpl_matrix *, int *, int *);
-cpl_error_code cpl_matrix_get_maxpos(const cpl_matrix *, int *, int *);
+cpl_error_code cpl_matrix_get_minpos(const cpl_matrix *, cpl_size *,
+                                     cpl_size *);
+cpl_error_code cpl_matrix_get_maxpos(const cpl_matrix *, cpl_size *,
+                                     cpl_size *);
 
 CPL_END_DECLS
 
