@@ -1,4 +1,4 @@
-/* $Id: cpl_msg.c,v 1.47 2010/11/24 09:19:36 cizzo Exp $
+/* $Id: cpl_msg.c,v 1.54 2012/03/06 10:38:44 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -19,10 +19,10 @@
  */
 
 /*
- * $Author: cizzo $
- * $Date: 2010/11/24 09:19:36 $
- * $Revision: 1.47 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Author: llundin $
+ * $Date: 2012/03/06 10:38:44 $
+ * $Revision: 1.54 $
+ * $Name: cpl-6_0 $
  */
 
 
@@ -31,26 +31,34 @@
 #endif
 
 #include <stdlib.h>
+#include <stdarg.h>
 #include <stdio.h>
 #include <string.h>
 #include <unistd.h>
-#include <stdarg.h>
+#include <signal.h>
+#include <time.h>
 
-#ifdef HAVE_TERMIO_H
-#  include <termio.h>
+
+#ifdef _OPENMP
+#include <omp.h>
+#endif
+
+#ifdef HAVE_TERMIOS_H
+#  include <termios.h>
 #else
-#  ifdef HAVE_TERMIOS_H
-#    include <termios.h>
+#  ifdef HAVE_TERMIO_H
+#    include <termio.h>
 #  else
 #    error Neither termios.h nor termio.h found!
 #  endif
 #endif
 
-#include <signal.h>
-
 #ifdef HAVE_STROPTS_H
 #  include <stropts.h>
-#else
+#endif
+
+#if !defined(HAVE_STROPTS_H) || defined(HAVE_TERMIOS_H) || \
+    defined(GWINSZ_IN_SYS_IOCTL)
 #  ifdef HAVE_SYS_IOCTL_H
 #    include <sys/ioctl.h>
 #  else
@@ -74,14 +82,14 @@
 #endif
 #endif
 
-#include <time.h>
 #include <cxutils.h>
 #include <cxmessages.h>
 
-#include <cpl_error.h>
+#include <cpl_error_impl.h>
 #include <cpl_msg.h>
 #include <cpl_memory.h>
 
+
 /**
  * @defgroup cpl_msg Messages
  *
@@ -116,6 +124,9 @@
  *   ...
  * @endcode
  *
+ * The functions of these kind, are meant to configure the messaging
+ * system, defining its "style", once and for all. For this reason
+ * such functions are not supposed to be called from threads.
  * Three different tags may be attached to any message: @em time,
  * @em domain, and @em component. The @em time tag is the time
  * of printing of the message, and can optionally be turned
@@ -190,12 +201,6 @@ inline static
 void cpl_msg_out(cpl_msg_severity, const char *, int,
                  const char *, va_list) CPL_ATTR_PRINTF(4,0);
 
-/* FIXME:
- * The following private members might be organised in one (or more)
- * structures, in case a complete description of the messaging system
- * configuration is requested by the client.
- */
-
 static const char       default_component[]             = "<empty field>";
 static const char       default_format[]                = "<empty message>";
 
@@ -211,15 +216,13 @@ static char             logfile_name[CPL_MAX_LOGFILE_NAME]  = ".logfile";
 static FILE            *logfile                         = NULL;
 
 static int              page_width                      = DEFAULT_WIDTH;
-static int              log_width                       = DEFAULT_WIDTH;
+static const int        log_width                       = DEFAULT_WIDTH;
 static int              indent_step                     = 2;
 static int              indent_value                    = 0;
 static int              overwrite                       = 0;
 
 #ifdef _OPENMP
-/* FIXME: Privatize vars that are likely to change after cpl_msg_init() */
-#pragma omp threadprivate(page_width, log_width, indent_step, indent_value, \
-                          overwrite)
+#pragma omp threadprivate(indent_value, overwrite)
 #endif
 
 static FILE            *msg_stdout;
@@ -496,7 +499,7 @@ static const char *strsplit(char * split, const char *s, int blanks, int width)
  *
  * This private function is used to actually display/add the message
  * to terminal and/or log file. Messages with severity level equal to
- * "warning" or greater would be sent to stderr, the other messages
+ * "error" or greater would be sent to stderr, the other messages
  * would go to stdout.
  *
  * If the severity level is lower than the levels set by
@@ -516,7 +519,9 @@ inline static void cpl_msg_out(cpl_msg_severity severity,
     char   msg_term[CPL_MAX_MSG_LENGTH] = "";
     char   split[CPL_MAX_MSG_LENGTH];
 
-    char  *c;
+#ifdef _OPENMP
+    char  *tid;
+#endif
     int    start_log_line, start_term_line;
     int    copy_only;
     int    i;
@@ -587,6 +592,20 @@ inline static void cpl_msg_out(cpl_msg_severity severity,
     strcat(msg_log, component);
     strcat(msg_log, ": ");
 
+
+#ifdef _OPENMP
+
+   /*
+    * Thread ID
+    */
+
+   tid = cpl_sprintf("[tid=%03d] ", omp_get_thread_num());
+   strcat(msg_term, tid);
+   strcat(msg_log, tid);
+   cpl_free(tid);
+
+#endif
+
    /*
     *  Message indentation
     */
@@ -628,8 +647,8 @@ inline static void cpl_msg_out(cpl_msg_severity severity,
         }
         else
             if (caller) {
-                c = strrchr(msg_term, '\n');
-                c = '\0';
+                char *c = strrchr(msg_term, '\n');
+                if (c >= msg_term) *c = '\0';
                 cx_print("\r%s", msg_term);
             }
             else
@@ -697,15 +716,14 @@ inline static void cpl_msg_out(cpl_msg_severity severity,
 cpl_error_code cpl_msg_init(void)
 {
 
-    const char *fid = "cpl_msg_init";
 #ifdef CPL_HAVE_STREAM_DUPLICATION
     struct winsize win;
     static int err_stream;
 #endif
 
 
-    if (msg_init > 1)
-        return cpl_error_set(fid, CPL_ERROR_FILE_ALREADY_OPEN);
+    if (msg_init > 0)
+        return cpl_error_set_(CPL_ERROR_FILE_ALREADY_OPEN);
 
 
 #ifdef CPL_HAVE_STREAM_DUPLICATION
@@ -714,16 +732,16 @@ cpl_error_code cpl_msg_init(void)
      */
 
     if ((out_stream = dup(fileno(stdout))) < 0)
-        return cpl_error_set(fid, CPL_ERROR_DUPLICATING_STREAM);
+        return cpl_error_set_(CPL_ERROR_DUPLICATING_STREAM);
 
     if (!(err_stream = dup(fileno(stderr))))
-        return cpl_error_set(fid, CPL_ERROR_DUPLICATING_STREAM);
+        return cpl_error_set_(CPL_ERROR_DUPLICATING_STREAM);
 
     if (!(msg_stdout = fdopen(out_stream, "a")))
-        return cpl_error_set(fid, CPL_ERROR_ASSIGNING_STREAM);
+        return cpl_error_set_(CPL_ERROR_ASSIGNING_STREAM);
 
     if (!(msg_stderr = fdopen(err_stream, "a")))
-        return cpl_error_set(fid, CPL_ERROR_ASSIGNING_STREAM);
+        return cpl_error_set_(CPL_ERROR_ASSIGNING_STREAM);
 #else
 
     msg_stdout = stdout;
@@ -734,7 +752,7 @@ cpl_error_code cpl_msg_init(void)
     default_printer = cx_print_set_handler(_cpl_print_out);
     default_error = cx_printerr_set_handler(_cpl_print_err);
 
-    ++msg_init;
+    msg_init = 1;
 
 #ifdef CPL_HAVE_STREAM_DUPLICATION
 #ifdef CPL_HAVE_WINDOW_RESIZING
@@ -845,15 +863,17 @@ void cpl_msg_stop(void)
  * Typically this function is called at the beginning of a program.
  * Calling it while a log file is already open has no effect, but it
  * will return an error code.
+ *
+ * @note
+ *   This function is meant to configure once and for all the behaviour
+ *   and the "style" of the messaging system, and it is not supposed to
+ *   be called in threads.
  */
 
 cpl_error_code cpl_msg_set_log_level(cpl_msg_severity verbosity)
 {
 
-    const char *fid = "cpl_msg_set_log_level";
-
-
-    _cpl_msg_init(fid);
+    _cpl_msg_init(cpl_func);
 
     if (logfile) {
 
@@ -862,7 +882,7 @@ cpl_error_code cpl_msg_set_log_level(cpl_msg_severity verbosity)
          *  is returned.
          */
 
-        return cpl_error_set(fid, CPL_ERROR_FILE_ALREADY_OPEN);
+        return cpl_error_set_(CPL_ERROR_FILE_ALREADY_OPEN);
 
     }
 
@@ -871,7 +891,7 @@ cpl_error_code cpl_msg_set_log_level(cpl_msg_severity verbosity)
         char timeLabel[TIME_ISO8601_LENGTH];
 
         if ((logfile = fopen(logfile_name, "w")) == NULL)
-            return cpl_error_set(fid, CPL_ERROR_FILE_NOT_CREATED);
+            return cpl_error_set_(CPL_ERROR_FILE_NOT_CREATED);
 
         (void)setvbuf(logfile, (char *) NULL, _IOLBF, 0);
 
@@ -893,7 +913,7 @@ cpl_error_code cpl_msg_set_log_level(cpl_msg_severity verbosity)
             case CPL_MSG_INFO    : fprintf(logfile, INFO_STRING); break;
             case CPL_MSG_WARNING : fprintf(logfile, WARNING_STRING); break;
             case CPL_MSG_ERROR   : fprintf(logfile, ERROR_STRING); break;
-            default              : ;
+            default              : break;
         }
 
         fprintf(logfile, "\n\n");
@@ -927,6 +947,7 @@ cpl_error_code cpl_msg_stop_log(void)
 
         log_min_level = CPL_MSG_OFF;
         fclose(logfile);
+        logfile = NULL;
 
     }
 
@@ -990,24 +1011,30 @@ const char *cpl_msg_get_log_name(void)
  * If this function is not called, or the specified @em name is
  * longer than <tt>CPL_MAX_LOGFILE_NAME</tt> characters, the log
  * file name is left to its default, ".logfile".
+ *
+ * @note
+ *   This function is meant to configure once and for all the behaviour
+ *   and the "style" of the messaging system, and it is not supposed to
+ *   be called in threads.
  */
 
 cpl_error_code cpl_msg_set_log_name(const char *name)
 {
 
-    const char *fid = "cpl_msg_set_log_name";
+    _cpl_msg_init(cpl_func);
 
-
-    _cpl_msg_init(fid);
-
-    if (name == 0x0)
-        return cpl_error_set(fid, CPL_ERROR_NULL_INPUT);
+    if (name == NULL)
+        return cpl_error_set_(CPL_ERROR_NULL_INPUT);
 
     if (logfile)
-        return cpl_error_set(fid, CPL_ERROR_FILE_ALREADY_OPEN);
+        return cpl_error_set_message_(CPL_ERROR_FILE_ALREADY_OPEN, "%s: %p",
+                                      name, (const void*)logfile);
 
     if (strlen(name) > CPL_MAX_LOGFILE_NAME - 1)
-        return cpl_error_set(fid, CPL_ERROR_ILLEGAL_INPUT);
+        return cpl_error_set_message_(CPL_ERROR_ILLEGAL_INPUT, "%s: %u + 1 > "
+                                      CPL_STRINGIFY(CPL_MAX_LOGFILE_NAME) " = "
+                                      CPL_XSTRINGIFY(CPL_MAX_LOGFILE_NAME),
+                                      name, (unsigned)strlen(name));
 
     strcpy(logfile_name, name);
 
@@ -1026,6 +1053,9 @@ cpl_error_code cpl_msg_set_log_name(const char *name)
  * The @em verbosity specifies the lowest severity level that a message
  * should have for being displayed to terminal. If this function is not
  * called, the verbosity level defaults to @c CPL_MSG_INFO.
+ *
+ * @note
+ *   This function is not supposed to be called in threads.
  */
 
 void cpl_msg_set_level(cpl_msg_severity verbosity)
@@ -1123,6 +1153,11 @@ cpl_msg_severity cpl_msg_get_level(void)
  * to the log file. This function must be called to display the @em time
  * tag also in messages written to terminal. To turn the @em time tag
  * off the function @c cpl_msg_set_time_off() should be called.
+ *
+ * @note
+ *   This function is meant to configure once and for all the behaviour
+ *   and the "style" of the messaging system, and it is not supposed to
+ *   be called in threads.
  */
 
 void cpl_msg_set_time_on(void)
@@ -1142,6 +1177,11 @@ void cpl_msg_set_time_on(void)
  * The @em time tag is turned off in messages written to terminal.
  * The @em time tag cannot be turned off in messages written to the
  * log file.
+ *
+ * @note
+ *   This function is meant to configure once and for all the behaviour
+ *   and the "style" of the messaging system, and it is not supposed to
+ *   be called in threads.
  */
 
 void cpl_msg_set_time_off(void)
@@ -1165,6 +1205,11 @@ void cpl_msg_set_time_off(void)
  * (or something analogous) would be attached to all messages. To turn
  * the @em domain tag off the function @c cpl_msg_set_domain_off() must
  * be called.
+ *
+ * @note
+ *   This function is meant to configure once and for all the behaviour
+ *   and the "style" of the messaging system, and it is not supposed to
+ *   be called in threads.
  */
 
 void cpl_msg_set_domain_on(void)
@@ -1182,6 +1227,11 @@ void cpl_msg_set_domain_on(void)
  * @return Nothing.
  *
  * The @em domain tag is turned off in messages written to terminal.
+ *
+ * @note
+ *   This function is meant to configure once and for all the behaviour
+ *   and the "style" of the messaging system, and it is not supposed to
+ *   be called in threads.
  */
 
 void cpl_msg_set_domain_off(void)
@@ -1204,6 +1254,11 @@ void cpl_msg_set_domain_off(void)
  * off the function @c cpl_msg_set_component_off() should be called. However,
  * the @em component tag is always shown when the verbosity level is set
  * to @c CPL_MSG_DEBUG.
+ *
+ * @note
+ *   This function is meant to configure once and for all the behaviour
+ *   and the "style" of the messaging system, and it is not supposed to
+ *   be called in threads.
  */
 
 void cpl_msg_set_component_on(void)
@@ -1223,6 +1278,11 @@ void cpl_msg_set_component_on(void)
  * The @em component tag is turned off in messages written to terminal,
  * unless the verbosity level is set to @c CPL_MSG_DEBUG. The @em component
  * tag cannot be turned off in messages written to the log file.
+ *
+ * @note
+ *   This function is meant to configure once and for all the behaviour
+ *   and the "style" of the messaging system, and it is not supposed to
+ *   be called in threads.
  */
 
 void cpl_msg_set_component_off(void)
@@ -1251,13 +1311,18 @@ void cpl_msg_set_component_off(void)
  * to all messages. To turn the @em domain tag off the function
  * @c cpl_msg_set_domain_off() should be called. If @em name is a
  * @c NULL pointer, nothing is done, and no error is set.
+ *
+ * @note
+ *   This function is meant to configure once and for all the behaviour
+ *   and the "style" of the messaging system, and it is not supposed to
+ *   be called in threads.
  */
 
 void cpl_msg_set_domain(const char *name)
 {
     _cpl_msg_init("cpl_msg_set_domain");
 
-    if (name == 0x0)
+    if (name == NULL)
         return;
 
     if (strlen(name) >= CPL_MAX_DOMAIN_NAME) {
@@ -1332,6 +1397,11 @@ void cpl_msg_set_width(int width)
  * indentation step. Setting the indentation step to zero or
  * to a negative number would eliminate messages indentation.
  * If this function is not called, the indentation step defaults to 2.
+ *
+ * @note
+ *   This function is meant to configure once and for all the behaviour
+ *   and the "style" of the messaging system, and it is not supposed to
+ *   be called in threads.
  */
 
 void cpl_msg_set_indentation(int step)
@@ -1437,12 +1507,12 @@ void cpl_msg_error(const char *component, const char *format, ...)
 
     _cpl_msg_init("cpl_msg_error");
 
-    if (component == 0x0)
+    if (component == NULL)
         c = default_component;
     else
         c = component;
 
-    if (format == 0x0) {
+    if (format == NULL) {
         cpl_msg_error(c, default_format);
         return;
     }
@@ -1473,12 +1543,12 @@ void cpl_msg_warning(const char *component, const char *format, ...)
 
     _cpl_msg_init("cpl_msg_warning");
 
-    if (component == 0x0)
+    if (component == NULL)
         c = default_component;
     else
         c = component;
 
-    if (format == 0x0) {
+    if (format == NULL) {
         cpl_msg_warning(c, default_format);
         return;
     }
@@ -1509,12 +1579,12 @@ void cpl_msg_info(const char *component, const char *format, ...)
 
     _cpl_msg_init("cpl_msg_info");
 
-    if (component == 0x0)
+    if (component == NULL)
         c = default_component;
     else
         c = component;
 
-    if (format == 0x0) {
+    if (format == NULL) {
         cpl_msg_info(c, default_format);
         return;
     }
@@ -1545,7 +1615,7 @@ void cpl_msg_info(const char *component, const char *format, ...)
  * typically in loops, as in the following example:
  * @code
  *  iter = 1000;
- *  for (i = 0 ; i < iter; i++) {
+ *  for (i = 0; i < iter; i++) {
  *      cpl_msg_info_overwritable(cpl_func,
                        "Median computation... %d out of %d", i, iter);
  *      <median computation would take place here>
@@ -1567,12 +1637,12 @@ void cpl_msg_info_overwritable(const char *component, const char *format, ...)
 
     overwrite = 1;
 
-    if (component == 0x0)
+    if (component == NULL)
         c = default_component;
     else
         c = component;
 
-    if (format == 0x0) {
+    if (format == NULL) {
         cpl_msg_info(c, default_format);
         return;
     }
@@ -1603,12 +1673,12 @@ void cpl_msg_debug(const char *component, const char *format, ...)
 
     _cpl_msg_init("cpl_msg_debug");
 
-    if (component == 0x0)
+    if (component == NULL)
         c = default_component;
     else
         c = component;
 
-    if (format == 0x0) {
+    if (format == NULL) {
         cpl_msg_debug(c, default_format);
         return;
     }
@@ -1645,7 +1715,7 @@ void cpl_msg_progress(const char *component, int i, int iter,
     const  double tquiet = 10; /* Accept silence for this many seconds */
     static double tstart = 0;  /* Initialize to avoid false warnings */
     static double tend = 0;    /* Initialize to avoid false warnings */
-    double tspent, now;
+    double tspent;
     static int iprev = 0;      /* Used to detect some illegal calls */
     static int nprev = 0;      /* Used to detect some illegal calls */
     static int didmsg = 0;
@@ -1655,7 +1725,7 @@ void cpl_msg_progress(const char *component, int i, int iter,
 
     _cpl_msg_init("cpl_msg_progress");
 
-    if (component == 0x0)
+    if (component == NULL)
         c = default_component;
     else
         c = component;
@@ -1674,7 +1744,7 @@ void cpl_msg_progress(const char *component, int i, int iter,
         /* Find out how much CPU was spent at this point */
         tstart = cpl_tools_get_cputime(CPL_CLOCK_TIME);
         /* Start counting the amount of CPU time spent */
-        now = cpl_tools_get_cputime(CPL_CLOCK_START);
+        (void)cpl_tools_get_cputime(CPL_CLOCK_START);
         tend = 0;
         didmsg = 0;
         return;
@@ -1688,9 +1758,9 @@ void cpl_msg_progress(const char *component, int i, int iter,
     if (iter != nprev) return; /* cpl_ensure() ? */
 
     /* Compute the time spent in the loop so far */
-    now = cpl_tools_get_cputime(CPL_CLOCK_STOP);
+    (void)cpl_tools_get_cputime(CPL_CLOCK_STOP);
     tspent = cpl_tools_get_cputime(CPL_CLOCK_TIME) - tstart;
-    if (i < iter-1) now = cpl_tools_get_cputime(CPL_CLOCK_START);
+    if (i < iter-1) (void)cpl_tools_get_cputime(CPL_CLOCK_START);
 
     /* This fraction (rounded down) of iterations has been completed */
     percent = (i * 100) / iter;
@@ -1748,6 +1818,7 @@ void cpl_msg_progress(const char *component, int i, int iter,
   @brief    Timer handling, including benchmarking.
   @param    mode [CPL_CLOCK_START CPL_CLOCK_STOP CPL_CLOCK_TIME CPL_CLOCK_RESET]
   @return   An elapsed time in seconds as a double.
+  @deprecated Use cpl_test_get_cputime() instead.
 
   This routine is useful to benchmark the time spent in an image processing
   routine. It is used in three steps:
@@ -1784,17 +1855,17 @@ static double cpl_tools_get_cputime(cpl_clock mode)
     /* clock() may wrap around after as little as 36 minutes,
        so store the result in a double */
     static clock_t chrono = 0;
-    static double  total  = 0;
+    static double  total  = 0.0;
     clock_t        diff;
-    double         result ;
+    double         result;
 
     switch (mode) {
         case CPL_CLOCK_START:
             cpl_ensure(pmode != CPL_CLOCK_START, CPL_ERROR_ILLEGAL_INPUT, 0);
-            chrono = clock() ;
+            chrono = clock();
             result = chrono;
             pmode = mode;
-            break ;
+            break;
         case CPL_CLOCK_STOP:
             cpl_ensure(pmode == CPL_CLOCK_START, CPL_ERROR_ILLEGAL_INPUT, 0);
             /* At this point clock() may have wrapped around, causing it to be
@@ -1804,21 +1875,23 @@ static double cpl_tools_get_cputime(cpl_clock mode)
             result = diff;
             total += result;
             pmode = mode;
-            break ;
+            break;
         case CPL_CLOCK_TIME:
             cpl_ensure(pmode != CPL_CLOCK_START, CPL_ERROR_ILLEGAL_INPUT, 0);
-            result = total ;
-            break ;
+            result = total;
+            break;
         case CPL_CLOCK_RESET:
             /* Set the static variables to their initial values */
             pmode = CPL_CLOCK_STOP;
             chrono = 0;
             total = 0.0;
             result = 0.0; /* Need to return something */
-            break ;
+            break;
         default:
             /* Only reached if cpl_clock is extended in error */
-            cpl_ensure(0, CPL_ERROR_UNSUPPORTED_MODE, 0.0);
+            result = 0.0; /* Need to return something */
+            (void)cpl_error_set_(CPL_ERROR_UNSUPPORTED_MODE);
+            break;
     }
 
     return result / CLOCKS_PER_SEC;
