@@ -1,4 +1,4 @@
-/* $Id: cpl_plot.c,v 1.14 2011/01/21 16:24:21 llundin Exp $
+/* $Id: cpl_plot.c,v 1.19 2011/07/21 08:35:14 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2011/01/21 16:24:21 $
- * $Revision: 1.14 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2011/07/21 08:35:14 $
+ * $Revision: 1.19 $
+ * $Name: cpl-6_0 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -148,9 +148,9 @@ cpl_error_code cpl_plot_vector(const char          *   pre,
                                         const cpl_vector    *   vector)
 {
     const double * pvec  = cpl_vector_get_data_const(vector);
-    const int      n     = cpl_vector_get_size(vector);
+    const cpl_size n     = cpl_vector_get_size(vector);
     cpl_error_code error = CPL_ERROR_NONE;
-    int            i;
+    cpl_size       i;
     cpl_plot     * plot;
 
 
@@ -165,7 +165,7 @@ cpl_error_code cpl_plot_vector(const char          *   pre,
     if (options != NULL) {
         error |= cpl_mplot_puts(plot, options);
     } else {
-        char * myoptions = cpl_sprintf("t '%d-vector (%p)", n,
+        char * myoptions = cpl_sprintf("t '%" CPL_SIZE_FORMAT"-vector (%p)", n,
                                        (const void*)vector);
         assert(myoptions != NULL);
         error |= cpl_mplot_puts(plot, myoptions);
@@ -214,12 +214,12 @@ cpl_error_code cpl_plot_bivector(const char          *   pre,
                                           const char          *   post,
                                           const cpl_bivector  *   bivector)
 {
-    const int      n         = cpl_bivector_get_size(bivector);
+    const cpl_size n         = cpl_bivector_get_size(bivector);
     const double * pvecx     = cpl_bivector_get_x_data_const(bivector);
     const double * pvecy     = cpl_bivector_get_y_data_const(bivector);
     cpl_error_code error     = CPL_ERROR_NONE;
-    cpl_plot  * plot;
-    int            i;
+    cpl_plot     * plot;
+    cpl_size       i;
 
 
     if (n <= 0) return cpl_error_set_where_();
@@ -233,8 +233,8 @@ cpl_error_code cpl_plot_bivector(const char          *   pre,
     if (options != NULL) {
         error |= cpl_mplot_puts(plot, options);
     } else {
-        char * myoptions = cpl_sprintf("t '%d-bivector (%p)", n,
-                                       (const void*)bivector);
+        char * myoptions = cpl_sprintf("t '%" CPL_SIZE_FORMAT "-bivector (%p)",
+                                       n, (const void*)bivector);
         assert(myoptions != NULL);
         error |= cpl_mplot_puts(plot, myoptions);
         cpl_free(myoptions);
@@ -283,19 +283,19 @@ cpl_error_code cpl_plot_bivector(const char          *   pre,
  */
 /*----------------------------------------------------------------------------*/
 cpl_error_code cpl_plot_vectors(const char          *   pre, 
-                                         const char          *   options,
-                                         const char          *   post,
-                                         const cpl_vector    **  vectors,
-                                         int                     nvec)
+                                const char          *   options,
+                                const char          *   post,
+                                const cpl_vector    **  vectors,
+                                cpl_size                nvec)
 {
     const double    *   pvecx;
     const double    *   pvecy;
-    int                 vec_size = 0; /* Avoid uninit warning */
+    cpl_size            vec_size = 0; /* Avoid uninit warning */
     char            **  names;
     char            *   sval;
     FILE            *   tmpfd;
-    cpl_plot     *   plot;
-    int                 i, j;
+    cpl_plot        *   plot;
+    cpl_size            i, j;
     
     /* Check entries */
     cpl_ensure_code(vectors != NULL, CPL_ERROR_NULL_INPUT);
@@ -315,9 +315,9 @@ cpl_error_code cpl_plot_vectors(const char          *   pre,
     pvecx = vectors[0] ? cpl_vector_get_data_const(vectors[0]) : NULL;
     
     /* Hold the files names */
-    names = cpl_malloc((nvec-1)*sizeof(char*));
+    names = cpl_malloc((size_t)(nvec-1)*sizeof(char*));
     for (i=1; i<nvec; i++)
-        names[i-1] = cpl_sprintf("cpl_plot-%d", i);
+        names[i-1] = cpl_sprintf("cpl_plot-%" CPL_SIZE_FORMAT, i);
     
     /* Open the plot */
     plot = cpl_mplot_open(pre);
@@ -357,7 +357,8 @@ cpl_error_code cpl_plot_vectors(const char          *   pre,
             sval = cpl_sprintf("plot '%s' %s;\n", names[i-1],
                                                  options);
         } else {
-            sval = cpl_sprintf("replot '%s' t 'Vector %d' w lines;\n", names[i-1], i);
+            sval = cpl_sprintf("replot '%s' t 'Vector %" CPL_SIZE_FORMAT
+                               "' w lines;\n", names[i-1], i);
         }
         cpl_mplot_puts(plot, sval);
         cpl_free(sval);
@@ -397,40 +398,41 @@ cpl_error_code cpl_plot_vectors(const char          *   pre,
     run-time system.
  */
 /*----------------------------------------------------------------------------*/
-cpl_error_code cpl_plot_bivectors(const char          *   pre, 
-                       const char          **  options,
-                       const char          *   post,
-                       const cpl_bivector  **  bivectors,
-                       int                     nbvec)
+cpl_error_code cpl_plot_bivectors(const char         *  pre, 
+                                  const char         ** options,
+                                  const char         *  post,
+                                  const cpl_bivector ** bivectors,
+                                  cpl_size              nbvec)
 {
-    const double    *   pvecx;
-    const double    *   pvecy;
-    char            **  names;
-    char            *   sval;
-    FILE            *   tmpfd;
-    cpl_plot     *   plot;
-    int                 i, j;
+    const double * pvecx;
+    const double * pvecy;
+    char        ** names;
+    char         * sval;
+    FILE         * tmpfd;
+    cpl_plot     * plot;
+    cpl_size       i;
+    cpl_size       j;
     
     /* Check entries */
     cpl_ensure_code(bivectors != NULL, CPL_ERROR_NULL_INPUT);
-    cpl_ensure_code(options != NULL, CPL_ERROR_NULL_INPUT);
-    cpl_ensure_code(nbvec>=1, CPL_ERROR_DATA_NOT_FOUND);
+    cpl_ensure_code(options   != NULL, CPL_ERROR_NULL_INPUT);
+    cpl_ensure_code(nbvec     >= 1,    CPL_ERROR_DATA_NOT_FOUND);
     for (i=0; i<nbvec; i++) {
         cpl_ensure_code(bivectors[i] != NULL, CPL_ERROR_NULL_INPUT);
         cpl_ensure_code(  options[i] != NULL, CPL_ERROR_NULL_INPUT);
     }
 
     /* Hold the files names */
-    names = cpl_malloc((nbvec)*sizeof(char*));
+    names = cpl_malloc((size_t)nbvec * sizeof(char*));
     for (i=0; i<nbvec; i++)
-    names[i] = cpl_sprintf("cpl_plot-%d", i+1);
+        names[i] = cpl_sprintf("cpl_plot-%" CPL_SIZE_FORMAT, i + 1);
     
     /* Open the plot */
     plot = cpl_mplot_open(pre);
 
     /* Loop on the signals to plot */
     for (i=0; i<nbvec; i++) {
-        int vec_size;
+        cpl_size vec_size;
         
         /* Get current vectors */
         pvecx = cpl_bivector_get_x_data_const(bivectors[i]);
@@ -504,14 +506,14 @@ cpl_error_code cpl_plot_mask(const char     *   pre,
                              const char     *   post,
                              const cpl_mask *   mask)
 {
-    const int          nx = cpl_mask_get_size_x(mask);
-    const int          ny = cpl_mask_get_size_y(mask);
+    const cpl_size     nx = cpl_mask_get_size_x(mask);
+    const cpl_size     ny = cpl_mask_get_size_y(mask);
     cpl_plot         * plot;
     const cpl_binary * pmask = cpl_mask_get_data_const(mask);
     char             * dvi_options = (char *)options;
     char             * myoptions   = (char *)options;
     cpl_error_code     error = CPL_ERROR_NONE;
-    int                i, j;
+    cpl_size           i, j;
 
 
     if (nx <= 0) return cpl_error_set_where_();
@@ -521,7 +523,8 @@ cpl_error_code cpl_plot_mask(const char     *   pre,
     if (plot == NULL) return cpl_error_set_where_();
 
     if (options == NULL || strlen(options) < 1) {
-        dvi_options = cpl_sprintf("%dX%d-mask (%p)", nx, ny, (const void*)mask);
+        dvi_options = cpl_sprintf("%" CPL_SIZE_FORMAT "X%" CPL_SIZE_FORMAT
+                                  "-mask (%p)", nx, ny, (const void*)mask);
         assert( dvi_options != NULL);
     }
 
@@ -540,7 +543,7 @@ cpl_error_code cpl_plot_mask(const char     *   pre,
 
         for (j = 0; j < ny; j++) {
             for (i = 0; i < nx; i++) {
-                char * snumber = cpl_sprintf("%d ", pmask[j*nx + i]);
+                char * snumber = cpl_sprintf("%d ", pmask[i + j * nx]);
 
                 error |= cpl_mplot_puts(plot, snumber);
                 cpl_free(snumber);
@@ -556,7 +559,8 @@ cpl_error_code cpl_plot_mask(const char     *   pre,
         unsigned char * raster = (unsigned char *)cpl_malloc(bsize);
 
         /* Create a PGM P5 header - with a maxval of 1 */
-        myoptions = cpl_sprintf("P5\n%d %d\n", nx, ny);
+        myoptions = cpl_sprintf("P5\n%" CPL_SIZE_FORMAT " %" CPL_SIZE_FORMAT
+                                "\n", nx, ny);
         error |= cpl_mplot_puts(plot, myoptions);
         cpl_free(myoptions);
 
@@ -565,7 +569,7 @@ cpl_error_code cpl_plot_mask(const char     *   pre,
         error |= cpl_mplot_puts(plot, myoptions);
 
         for (j = 0; j < ny; j++) for (i = 0; i < nx; i++)
-            raster[(ny-j-1)*nx + i] = (unsigned char) pmask[j*nx + i];
+            raster[i + (ny-j-1) * nx] = (unsigned char) pmask[i + j * nx];
 
         error |= cpl_mplot_write(plot, (const void *) raster, bsize);
 
@@ -613,20 +617,20 @@ cpl_error_code cpl_plot_mask(const char     *   pre,
  */
 /*----------------------------------------------------------------------------*/
 cpl_error_code cpl_plot_image(const char      *   pre, 
-                                       const char      *   options,
-                                       const char      *   post,
-                                       const cpl_image *   image)
+                              const char      *   options,
+                              const char      *   post,
+                              const cpl_image *   image)
 {
-    const int       nx = cpl_image_get_size_x(image);
-    const int       ny = cpl_image_get_size_y(image);
-    cpl_plot   * plot;
+    const cpl_size  nx = cpl_image_get_size_x(image);
+    const cpl_size  ny = cpl_image_get_size_y(image);
+    cpl_plot      * plot;
     cpl_image     * temp;
     const double  * pimage;
     unsigned char * raster = NULL;
     char          * dvi_options = (char *)options;
     char          * myoptions   = (char *)options;
     cpl_error_code  error = CPL_ERROR_NONE;
-    int             i, j;
+    cpl_size        i, j;
 
 
     if (nx <= 0) return cpl_error_set_where_();
@@ -636,8 +640,10 @@ cpl_error_code cpl_plot_image(const char      *   pre,
     if (plot == NULL) return cpl_error_set_where_();
 
     if (options == NULL || strlen(options) < 1) {
-        dvi_options = cpl_sprintf("%dX%d-image-(%d) (%p)", nx, ny, 
-                (int) cpl_image_get_type(image), (const void*)image);
+        dvi_options = cpl_sprintf("%" CPL_SIZE_FORMAT "X%" CPL_SIZE_FORMAT
+                                  "-image-(%d) (%p)", nx, ny, 
+                                  (int)cpl_image_get_type(image),
+                                  (const void*)image);
         assert( dvi_options != NULL);
     }
 
@@ -700,7 +706,8 @@ cpl_error_code cpl_plot_image(const char      *   pre,
         assert(pimage != NULL);
 
         /* Create a PGM P5 header - with a maxval of 255 */
-        myoptions = cpl_sprintf("P5\n%d %d\n", nx, ny);
+        myoptions = cpl_sprintf("P5\n%" CPL_SIZE_FORMAT " %" CPL_SIZE_FORMAT
+                                "\n", nx, ny);
         error |= cpl_mplot_puts(plot, myoptions);
         cpl_free(myoptions);
 
@@ -762,14 +769,14 @@ cpl_error_code cpl_plot_image(const char      *   pre,
  */
 /*----------------------------------------------------------------------------*/
 cpl_error_code cpl_plot_image_row(const char      *   pre,
-                                           const char      *   options,
-                                           const char      *   post,
-                                           const cpl_image *   image,
-                                           int                 firstrow, 
-                                           int                 lastrow,
-                                           int                 rowstep)
+                                  const char      *   options,
+                                  const char      *   post,
+                                  const cpl_image *   image,
+                                  cpl_size            firstrow, 
+                                  cpl_size            lastrow,
+                                  cpl_size            rowstep)
 {
-    int nx, ny;
+    cpl_size nx, ny;
 
     cpl_ensure_code(image != NULL, CPL_ERROR_NULL_INPUT);
 
@@ -790,7 +797,7 @@ cpl_error_code cpl_plot_image_row(const char      *   pre,
         const double * pimage;
         cpl_image * temp = NULL;
         char * myoptions = NULL;
-        int i, j;
+        cpl_size i, j;
 
         cpl_plot * plot = cpl_mplot_open(pre);
 
@@ -807,12 +814,15 @@ cpl_error_code cpl_plot_image_row(const char      *   pre,
 
                     cpl_free(myoptions);
                     myoptions = (j == firstrow-1)
-                        ? cpl_sprintf("t 'Row %d %dX%d-image-(%d) (%p)'", 
-                                j, nx, ny, (int)cpl_image_get_type(image),
-                                (const void*)image)
-                        : cpl_sprintf("t 'Row %d of the same image'", j);
+                        ? cpl_sprintf("t 'Row %" CPL_SIZE_FORMAT " %"
+                                      CPL_SIZE_FORMAT "X%" CPL_SIZE_FORMAT
+                                      "-image-(%d) (%p)'",  j, nx, ny,
+                                      (int)cpl_image_get_type(image),
+                                      (const void*)image)
+                        : cpl_sprintf("t 'Row %" CPL_SIZE_FORMAT
+                                      " of the same image'", j);
 
-                        assert( myoptions != NULL);
+                    assert( myoptions != NULL);
                         useoptions = myoptions;
                 }
                 if (cpl_mplot_puts(plot, useoptions)) break;
@@ -884,11 +894,11 @@ cpl_error_code cpl_plot_image_col(const char      * pre,
                                   const char      * options,
                                   const char      * post,
                                   const cpl_image * image,
-                                  int               firstcol, 
-                                  int               lastcol,
-                                  int               colstep)
+                                  cpl_size          firstcol, 
+                                  cpl_size          lastcol,
+                                  cpl_size          colstep)
 {
-    int nx, ny;
+    cpl_size nx, ny;
     cpl_ensure_code(image != NULL, CPL_ERROR_NULL_INPUT);
 
     nx = cpl_image_get_size_x(image);
@@ -908,7 +918,7 @@ cpl_error_code cpl_plot_image_col(const char      * pre,
         const double * pimage;
         cpl_image * temp = NULL;
         char * myoptions = NULL;
-        int i, j;
+        cpl_size i, j;
 
         cpl_plot * plot = cpl_mplot_open(pre);
 
@@ -924,13 +934,13 @@ cpl_error_code cpl_plot_image_col(const char      * pre,
 
                     cpl_free(myoptions);
                     myoptions = (i == firstcol-1)
-                        ? cpl_sprintf("t 'Column %d of a "
-                                                 "%dX%d-image-(%d) (%p) ",
-                                                 i, nx, ny,
-                                                 (int)cpl_image_get_type(image),
-                                                 (const void*)image)
-                        : cpl_sprintf("t 'Column %d of the "
-                                                        "same image'", i);
+                        ? cpl_sprintf("t 'Column %" CPL_SIZE_FORMAT " of a "
+                                      "%" CPL_SIZE_FORMAT "X%" CPL_SIZE_FORMAT
+                                      "-image-(%d) (%p) ", i, nx, ny,
+                                      (int)cpl_image_get_type(image),
+                                      (const void*)image)
+                        : cpl_sprintf("t 'Column %" CPL_SIZE_FORMAT " of the "
+                                      "same image'", i);
                     assert( myoptions != NULL);
                     useoptions = myoptions;
 
@@ -1005,7 +1015,7 @@ cpl_error_code cpl_plot_column(
         const char      *   xlab, 
         const char      *   ylab)
 {
-    int                 n;
+    cpl_size            n;
     cpl_table       *   tempx;
     cpl_table       *   tempy;
     cpl_bivector    *   bivect;
@@ -1013,9 +1023,8 @@ cpl_error_code cpl_plot_column(
     cpl_vector      *   yvec;
     cpl_type            type_x;
     cpl_type            type_y;
-    int                 invalid_x;
-    int                 invalid_y;
-    int                 i;
+    cpl_size            invalid_x;
+    cpl_size            invalid_y;
 
     /* Check input */
     cpl_ensure_code(tab, CPL_ERROR_NULL_INPUT);
@@ -1090,6 +1099,7 @@ cpl_error_code cpl_plot_column(
         tempx = NULL;
         
         if (xlab == NULL) {
+            cpl_size i;
             xvec = cpl_vector_duplicate(yvec);
             for (i=0; i<cpl_vector_get_size(xvec); i++) {
                 cpl_vector_set(xvec, i, (double)i+1);
@@ -1139,15 +1149,14 @@ cpl_error_code cpl_plot_column(
     run-time system.
  */
 /*----------------------------------------------------------------------------*/
-cpl_error_code cpl_plot_columns(
-        const char      *   pre, 
-        const char      *   options,
-        const char      *   post,
-        const cpl_table *   tab,
-        const char      **  labels, 
-        int                 nlabels)
+cpl_error_code cpl_plot_columns(const char      *  pre, 
+                                const char      *  options,
+                                const char      *  post,
+                                const cpl_table *  tab,
+                                const char      ** labels, 
+                                cpl_size           nlabels)
 {
-    int                 n;
+    cpl_size            n;
     cpl_table       *   tempx;
     cpl_table       *   tempy;
     cpl_type            type_x;
@@ -1158,13 +1167,13 @@ cpl_error_code cpl_plot_columns(
     char            *   sval;
     FILE            *   tmpfd;
     cpl_plot        *   plot;
-    int                 i, j;
+    cpl_size            i, j;
 
     /* Check input */
-    cpl_ensure_code(tab, CPL_ERROR_NULL_INPUT);
-    cpl_ensure_code(labels != NULL, CPL_ERROR_NULL_INPUT);
-    cpl_ensure_code(nlabels>=3, CPL_ERROR_NULL_INPUT);
-    for (i=1; i<nlabels; i++) {
+    cpl_ensure_code(tab     != NULL, CPL_ERROR_NULL_INPUT);
+    cpl_ensure_code(labels  != NULL, CPL_ERROR_NULL_INPUT);
+    cpl_ensure_code(nlabels >= 3,    CPL_ERROR_DATA_NOT_FOUND);
+    for (i = 1; i < nlabels; i++) {
         cpl_ensure_code(labels[i] != NULL, CPL_ERROR_NULL_INPUT);
     }
 
@@ -1192,9 +1201,9 @@ cpl_error_code cpl_plot_columns(
     }
 
     /* Hold the files names */
-    names = cpl_malloc((nlabels-1)*sizeof(char*));
-    for (i=1; i<nlabels; i++)
-        names[i-1] = cpl_sprintf("cpl_plot-%d", i);
+    names = cpl_malloc((size_t)(nlabels-1) * sizeof(char*));
+    for (i=1; i < nlabels; i++)
+        names[i-1] = cpl_sprintf("cpl_plot-%" CPL_SIZE_FORMAT, i);
 
     /* Open the plot */
     plot = cpl_mplot_open(pre);
@@ -1258,8 +1267,8 @@ cpl_error_code cpl_plot_columns(
             sval = cpl_sprintf("plot '%s' %s;\n", names[i-1],
                                                  options);
         } else {
-            sval = cpl_sprintf("replot '%s' t 'Column %d' w lines;\n", 
-                    names[i-1], i);
+            sval = cpl_sprintf("replot '%s' t 'Column %" CPL_SIZE_FORMAT
+                               "' w lines;\n", names[i-1], i);
         }
 
         cpl_mplot_puts(plot, sval);
@@ -1379,7 +1388,8 @@ static cpl_plot * cpl_mplot_open(const char * options)
 
     if (cpl_mplot_puts(plot, options)) {
         (void)cpl_mplot_close(plot, "");
-        cpl_ensure(0, cpl_error_get_code(), NULL);
+        plot = NULL;
+        (void)cpl_error_set_where_();
     }
 
     return plot;
@@ -1437,7 +1447,8 @@ static cpl_plot * cpl_image_open(const char * options)
 
     if (cpl_mplot_puts(plot, options)) {
         (void)cpl_mplot_close(plot, "");
-        cpl_ensure(0, cpl_error_get_code(), NULL);
+        plot = NULL;
+        (void)cpl_error_set_where_();
     }
 
     return plot;
