@@ -1,4 +1,4 @@
-/* $Id: cpl_property.c,v 1.31 2010/12/22 13:38:45 llundin Exp $
+/* $Id: cpl_property.c,v 1.34 2011/11/02 10:31:01 rpalsa Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2005 European Southern Observatory
@@ -19,10 +19,10 @@
  */
 
 /*
- * $Author: llundin $
- * $Date: 2010/12/22 13:38:45 $
- * $Revision: 1.31 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Author: rpalsa $
+ * $Date: 2011/11/02 10:31:01 $
+ * $Revision: 1.34 $
+ * $Name: cpl-6_0 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -169,7 +169,7 @@ _cpl_property_value_resize(cpl_property_value *value, cpl_type type,
 
 inline static int
 _cpl_property_value_set(cpl_property_value *value, cpl_type type,
-                        cxsize nelements, const void *data)
+                        cxsize nelements, cxcptr data)
 {
 
     cxsize sz = 0;
@@ -200,7 +200,7 @@ _cpl_property_value_set(cpl_property_value *value, cpl_type type,
 
 inline static cxptr
 _cpl_property_value_get(cpl_property_value *value, cpl_type type,
-                        const char *name)
+                        const cxchar *name)
 {
 
     cx_assert(value != NULL);
@@ -264,8 +264,6 @@ cpl_property *
 cpl_property_new(const char *name, cpl_type type)
 {
 
-
-
     cxsize sz;
 
     cpl_property *property;
@@ -332,11 +330,9 @@ cpl_property_new(const char *name, cpl_type type)
  */
 
 cpl_property *
-cpl_property_new_array(const char *name, cpl_type type, int size)
+cpl_property_new_array(const char *name, cpl_type type, cpl_size size)
 {
 
-
-
     cxsize sz;
 
     cpl_property *property;
@@ -396,7 +392,6 @@ cpl_property *
 cpl_property_duplicate(const cpl_property *other)
 {
 
-
     cpl_property *copy;
 
 
@@ -494,17 +489,16 @@ cpl_property_delete(cpl_property *self)
  * given by the @b strlen() function plus 1.
  */
 
-long
+cpl_size
 cpl_property_get_size(const cpl_property *self)
 {
 
-
     if (self == NULL) {
         cpl_error_set_(CPL_ERROR_NULL_INPUT);
         return -1;
     }
 
-    return (long)self->value->size;
+    return (cpl_size)self->value->size;
 
 }
 
@@ -539,8 +533,6 @@ cpl_type
 cpl_property_get_type(const cpl_property *self)
 {
 
-
-
     if (self == NULL) {
         cpl_error_set_(CPL_ERROR_NULL_INPUT);
         return CPL_TYPE_INVALID;
@@ -584,8 +576,6 @@ cpl_error_code
 cpl_property_set_name(cpl_property *self, const char *name)
 {
 
-
-
     if (self == NULL) {
         return cpl_error_set_(CPL_ERROR_NULL_INPUT);
     }
@@ -638,8 +628,6 @@ cpl_error_code
 cpl_property_set_comment(cpl_property *self, const char *comment)
 {
 
-
-
     if (self == NULL) {
         return cpl_error_set_(CPL_ERROR_NULL_INPUT);
     }
@@ -697,7 +685,6 @@ cpl_error_code
 cpl_property_set_char(cpl_property *self, char value)
 {
 
-
     cxint status = 0;
 
 
@@ -708,7 +695,7 @@ cpl_property_set_char(cpl_property *self, char value)
     cx_assert(self->value != NULL);
 
     status = _cpl_property_value_set(self->value, CPL_TYPE_CHAR,
-                                     1, (const void*)&value);
+                                     1, (cxcptr)&value);
 
     if (status) {
         return cpl_error_set_(CPL_ERROR_TYPE_MISMATCH);
@@ -757,7 +744,6 @@ cpl_error_code
 cpl_property_set_bool(cpl_property *self, int value)
 {
 
-
     cxint status = 0;
 
 
@@ -770,7 +756,7 @@ cpl_property_set_bool(cpl_property *self, int value)
     value = value ? TRUE : FALSE;
 
     status = _cpl_property_value_set(self->value, CPL_TYPE_BOOL,
-                                     1, (const void*)&value);
+                                     1, (cxcptr)&value);
 
     if (status) {
         return cpl_error_set_(CPL_ERROR_TYPE_MISMATCH);
@@ -818,7 +804,6 @@ cpl_error_code
 cpl_property_set_int(cpl_property *self, int value)
 {
 
-
     cxint status = 0;
 
 
@@ -829,7 +814,7 @@ cpl_property_set_int(cpl_property *self, int value)
     cx_assert(self->value != NULL);
 
     status = _cpl_property_value_set(self->value, CPL_TYPE_INT,
-                                     1, (const void*)&value);
+                                     1, (cxcptr)&value);
 
     if (status) {
         return cpl_error_set_(CPL_ERROR_TYPE_MISMATCH);
@@ -877,7 +862,6 @@ cpl_error_code
 cpl_property_set_long(cpl_property *self, long value)
 {
 
-
     cxint status = 0;
 
 
@@ -888,7 +872,65 @@ cpl_property_set_long(cpl_property *self, long value)
     cx_assert(self->value != NULL);
 
     status = _cpl_property_value_set(self->value, CPL_TYPE_LONG,
-                                     1, (const void*)&value);
+                                     1, (cxcptr)&value);
+
+    if (status) {
+        return cpl_error_set_(CPL_ERROR_TYPE_MISMATCH);
+    }
+
+    return CPL_ERROR_NONE;
+
+}
+
+
+/**
+ * @brief
+ *   Set the value of a long long property.
+ *
+ * @param self   The property.
+ * @param value  New value.
+ *
+ * @return
+ *   The function returns @c CPL_ERROR_NONE on success and an appropriate
+ *   error code if an error occurred. In the latter case the error code is
+ *   also set in the error handling system.
+ *
+ * @error
+ *   <table class="ec" align="center">
+ *     <tr>
+ *       <td class="ecl">CPL_ERROR_NULL_INPUT</td>
+ *       <td class="ecr">
+ *         The parameter <i>self</i> is a <tt>NULL</tt> pointer.
+ *       </td>
+ *     </tr>
+ *     <tr>
+ *       <td class="ecl">CPL_ERROR_TYPE_MISMATCH</td>
+ *       <td class="ecr">
+ *         The property <i>self</i> is not of type <tt>CPL_TYPE_LONG</tt>.
+ *       </td>
+ *     </tr>
+ *   </table>
+ * @enderror
+ *
+ * The function replaces the current long long value of @em self with a
+ * copy of the value @em value.
+ */
+
+cpl_error_code
+cpl_property_set_long_long(cpl_property *self, long long value)
+{
+
+    cxint status = 0;
+
+
+    if (self == NULL) {
+        return cpl_error_set_(CPL_ERROR_NULL_INPUT);
+    }
+
+    cx_assert(self->value != NULL);
+
+    status = _cpl_property_value_set(self->value, CPL_TYPE_LONG_LONG,
+                                     1, (cxcptr)&value);
 
     if (status) {
         return cpl_error_set_(CPL_ERROR_TYPE_MISMATCH);
@@ -936,7 +978,6 @@ cpl_error_code
 cpl_property_set_float(cpl_property *self, float value)
 {
 
-
     cxint status = 0;
 
 
@@ -947,7 +988,7 @@ cpl_property_set_float(cpl_property *self, float value)
     cx_assert(self->value != NULL);
 
     status = _cpl_property_value_set(self->value, CPL_TYPE_FLOAT,
-                                     1, (const void*)&value);
+                                     1, (cxcptr)&value);
 
     if (status) {
         return cpl_error_set_(CPL_ERROR_TYPE_MISMATCH);
@@ -1006,7 +1047,7 @@ cpl_property_set_double(cpl_property *self, double value)
     cx_assert(self->value != NULL);
 
     status = _cpl_property_value_set(self->value, CPL_TYPE_DOUBLE,
-                                     1, (const void*)&value);
+                                     1, (cxcptr)&value);
 
     if (status) {
         return cpl_error_set_(CPL_ERROR_TYPE_MISMATCH);
@@ -1080,7 +1121,7 @@ cpl_property_set_string(cpl_property *self, const char *value)
     len = (cxsize) strlen(value) + 1;
 
     status = _cpl_property_value_set(self->value, CPL_TYPE_STRING,
-                                     len, (const void*)value);
+                                     len, (cxcptr)value);
 
     if (status) {
         return cpl_error_set_(CPL_ERROR_TYPE_MISMATCH);
@@ -1129,15 +1170,23 @@ cpl_error_code
 cpl_property_set_float_complex(cpl_property *self, float complex value)
 {
 
+    cxint status = 0;
+
+
     if (self == NULL) {
         return cpl_error_set_(CPL_ERROR_NULL_INPUT);
     }
 
     cx_assert(self->value != NULL);
 
-    return _cpl_property_value_set(self->value, CPL_TYPE_FLOAT_COMPLEX,
-                                   1, (const void*)&value)
-        ? cpl_error_set_(CPL_ERROR_TYPE_MISMATCH) : CPL_ERROR_NONE;
+    status = _cpl_property_value_set(self->value, CPL_TYPE_FLOAT_COMPLEX,
+                                     1, (cxcptr)&value);
+
+    if (status) {
+        return cpl_error_set_(CPL_ERROR_TYPE_MISMATCH);
+    }
+
+    return CPL_ERROR_NONE;
 
 }
 
@@ -1180,6 +1229,8 @@ cpl_error_code
 cpl_property_set_double_complex(cpl_property *self, double complex value)
 {
 
+    cxint status = 0;
+
 
     if (self == NULL) {
         return cpl_error_set_(CPL_ERROR_NULL_INPUT);
@@ -1187,9 +1238,16 @@ cpl_property_set_double_complex(cpl_property *self, double complex value)
 
     cx_assert(self->value != NULL);
 
-    return _cpl_property_value_set(self->value, CPL_TYPE_DOUBLE_COMPLEX,
-                                   1, (const void*)&value)
-        ? cpl_error_set_(CPL_ERROR_TYPE_MISMATCH) : CPL_ERROR_NONE;
+    status = _cpl_property_value_set(self->value, CPL_TYPE_DOUBLE_COMPLEX,
+                                     1, (cxcptr)&value);
+
+    if (status)
+    {
+        return cpl_error_set_(CPL_ERROR_TYPE_MISMATCH);
+    }
+
+    return CPL_ERROR_NONE;
+
 }
 
 
@@ -1221,8 +1279,6 @@ const char *
 cpl_property_get_name(const cpl_property *self)
 {
 
-
-
     if (self == NULL) {
         cpl_error_set_(CPL_ERROR_NULL_INPUT);
         return NULL;
@@ -1261,8 +1317,6 @@ const char *
 cpl_property_get_comment(const cpl_property *self)
 {
 
-
-
     if (self == NULL) {
         cpl_error_set_(CPL_ERROR_NULL_INPUT);
         return NULL;
@@ -1312,8 +1366,6 @@ char
 cpl_property_get_char(const cpl_property *self)
 {
 
-
-
     cxptr result;
 
 
@@ -1474,38 +1526,166 @@ cpl_property_get_int(const cpl_property *self)
  *     <tr>
  *       <td class="ecl">CPL_ERROR_TYPE_MISMATCH</td>
  *       <td class="ecr">
- *         The property <i>self</i> is not of type <tt>CPL_TYPE_LONG</tt>.
+ *         The property <i>self</i> is not of type <tt>CPL_TYPE_LONG</tt>
+ *         or <tt>CPL_TYPE_INT</tt>.
  *       </td>
  *     </tr>
  *   </table>
  * @enderror
  *
- * The function retrieves the long value currently stored in the
- * property @em self.
+ * The function retrieves the value currently stored in the property @em self
+ * as a long integer. The function accepts properties of integer type
+ * with a rank less or equal than the function's return type.
  */
 
 long
 cpl_property_get_long(const cpl_property *self)
 {
 
-
-    cxptr result;
+    cxlong value = 0;
 
 
     if (self == NULL) {
         cpl_error_set_(CPL_ERROR_NULL_INPUT);
-        return 0;
+        return value;
     }
 
     cx_assert(self->value != NULL);
 
-    result = _cpl_property_value_get(self->value, CPL_TYPE_LONG, cpl_func);
 
-    if (!result) {
-        return 0;
+    /*
+     * If the property self is of the same kind, but of a smaller rank,
+     * promote the property's value to the target type.
+     */
+
+    switch (self->value->type) {
+
+        case CPL_TYPE_INT:
+        {
+            cxint *result = _cpl_property_value_get(self->value,
+                                                    CPL_TYPE_INT, cpl_func);
+
+            value = *result;
+            break;
+        }
+
+        case CPL_TYPE_LONG:
+        {
+            cxlong *result = _cpl_property_value_get(self->value,
+                                                     CPL_TYPE_LONG, cpl_func);
+
+            value = *result;
+            break;
+        }
+
+        default:
+        {
+            cpl_error_set_(CPL_ERROR_TYPE_MISMATCH);
+            break;
+        }
+
     }
 
-    return *((long *)result);
+    return value;
+
+}
+
+
+/**
+ * @brief
+ *   Get the value of a long long property.
+ *
+ * @param self  The property.
+ *
+ * @return
+ *   The current property value. If an error occurs the function returns
+ *   0 and an appropriate error code is set.
+ *
+ * @error
+ *   <table class="ec" align="center">
+ *     <tr>
+ *       <td class="ecl">CPL_ERROR_NULL_INPUT</td>
+ *       <td class="ecr">
+ *         The parameter <i>self</i> is a <tt>NULL</tt> pointer.
+ *       </td>
+ *     </tr>
+ *     <tr>
+ *       <td class="ecl">CPL_ERROR_TYPE_MISMATCH</td>
+ *       <td class="ecr">
+ *         The property <i>self</i> is not of type <tt>CPL_TYPE_LONG_LONG</tt>,
+ *         <tt>CPL_TYPE_LONG</tt>, or <tt>CPL_TYPE_INT</tt>.
+ *       </td>
+ *     </tr>
+ *   </table>
+ * @enderror
+ *
+ * The function retrieves the value currently stored in the property @em self
+ * as a long long integer. The function accepts properties of integer type
+ * with a rank less or equal than the function's return type.
+ */
+
+long long
+cpl_property_get_long_long(const cpl_property *self)
+{
+
+
+    long long value = 0;
+
+
+    if (self == NULL) {
+        cpl_error_set_(CPL_ERROR_NULL_INPUT);
+        return value;
+    }
+
+    cx_assert(self->value != NULL);
+
+
+    /*
+     * If the property self is of the same kind, but of a smaller rank,
+     * promote the property's value to the target type.
+     */
+
+    switch (self->value->type) {
+
+        case CPL_TYPE_INT:
+        {
+            cxint *result = _cpl_property_value_get(self->value,
+                                                    CPL_TYPE_INT,
+                                                    cpl_func);
+
+            value = *result;
+            break;
+        }
+
+        case CPL_TYPE_LONG:
+        {
+            cxlong *result = _cpl_property_value_get(self->value,
+                                                     CPL_TYPE_LONG,
+                                                     cpl_func);
+
+            value = *result;
+            break;
+        }
+
+        case CPL_TYPE_LONG_LONG:
+        {
+            long long *result = _cpl_property_value_get(self->value,
+                                                        CPL_TYPE_LONG_LONG,
+                                                        cpl_func);
+
+            value = *result;
+            break;
+        }
+
+        default:
+        {
+            cpl_error_set_(CPL_ERROR_TYPE_MISMATCH);
+            break;
+        }
+
+    }
+
+    return value;
 
 }
 
@@ -1518,7 +1698,7 @@ cpl_property_get_long(const cpl_property *self)
  *
  * @return
  *   The current property value. If an error occurs the function returns
- *   0.0 and an appropriate error code is set.
+ *   0. and an appropriate error code is set.
  *
  * @error
  *   <table class="ec" align="center">
@@ -1531,44 +1711,73 @@ cpl_property_get_long(const cpl_property *self)
  *     <tr>
  *       <td class="ecl">CPL_ERROR_TYPE_MISMATCH</td>
  *       <td class="ecr">
- *         The property <i>self</i> is not of type <tt>CPL_TYPE_FLOAT</tt> or <tt>CPL_TYPE_DOUBLE</tt>.
+ *         The property <i>self</i> is not of type <tt>CPL_TYPE_FLOAT</tt>
+ *         or <tt>CPL_TYPE_DOUBLE</tt>.
  *       </td>
  *     </tr>
  *   </table>
  * @enderror
  *
- * The function retrieves the float value currently stored in the
- * property @em self. If the value is of type double, the function
- * casts it to float before returning it.
+ * The function retrieves the value currently stored in the property @em self
+ * as a float. The function also accepts properties of type double and returns
+ * the property value as float.
+ *
+ * @note
+ *  Calling the function for a property of type double may lead to
+ *  truncation errors!
  */
 
 float
 cpl_property_get_float(const cpl_property *self)
 {
 
-
-    cxptr result;
+    cxfloat value = 0.;
 
 
     if (self == NULL) {
         cpl_error_set_(CPL_ERROR_NULL_INPUT);
-        return 0.;
+        return value;
     }
 
     cx_assert(self->value != NULL);
 
-    /* If the type is not a float, check if it is a double */
-    if (self->value->type == CPL_TYPE_FLOAT){
-        result = _cpl_property_value_get(self->value, CPL_TYPE_FLOAT, cpl_func);
-        return *((float *)result);
-    }
-    else if (self->value->type == CPL_TYPE_DOUBLE){
-        result = _cpl_property_value_get(self->value, CPL_TYPE_DOUBLE, cpl_func);
-        return *((double *)result);
+
+    /*
+     * If the property self is of type double cast it to float.
+     * This may lead to truncation errors!
+     */
+
+    switch (self->value->type) {
+
+        case CPL_TYPE_FLOAT:
+        {
+            cxfloat *result = _cpl_property_value_get(self->value,
+                                                      CPL_TYPE_FLOAT,
+                                                      cpl_func);
+
+            value = *result;
+            break;
+        }
+
+        case CPL_TYPE_DOUBLE:
+        {
+            cxdouble *result = _cpl_property_value_get(self->value,
+                                                       CPL_TYPE_DOUBLE,
+                                                       cpl_func);
+
+            value = (cxfloat)(*result);
+            break;
+        }
+
+        default:
+        {
+            cpl_error_set_(CPL_ERROR_TYPE_MISMATCH);
+            break;
+        }
+
     }
 
-    (void)cpl_error_set_(CPL_ERROR_TYPE_MISMATCH);
-    return 0.;
+    return value;
 
 }
 
@@ -1581,7 +1790,7 @@ cpl_property_get_float(const cpl_property *self)
  *
  * @return
  *   The current property value. If an error occurs the function returns
- *   0.0 and an appropriate error code is set.
+ *   0. and an appropriate error code is set.
  *
  * @error
  *   <table class="ec" align="center">
@@ -1594,43 +1803,68 @@ cpl_property_get_float(const cpl_property *self)
  *     <tr>
  *       <td class="ecl">CPL_ERROR_TYPE_MISMATCH</td>
  *       <td class="ecr">
- *         The property <i>self</i> is not of type <tt>CPL_TYPE_DOUBLE</tt> or <tt>CPL_TYPE_FLOAT</tt>.
+ *         The property <i>self</i> is not of type <tt>CPL_TYPE_DOUBLE</tt>
+ *         or <tt>CPL_TYPE_FLOAT</tt>.
  *       </td>
  *     </tr>
  *   </table>
  * @enderror
  *
- * The function retrieves the double value currently stored in the
- * property @em self. If the value is of type float, the function
- * casts it to double before returning it.
+ * The function retrieves the value currently stored in the property @em self
+ * as a double. The function accepts properties of a floating-point type
+ * with a rank less or equal than the function's return type.
  */
 
 double
 cpl_property_get_double(const cpl_property *self)
 {
 
-
-    cxptr result;
+    cxdouble value = 0.;
 
 
     if (self == NULL) {
         cpl_error_set_(CPL_ERROR_NULL_INPUT);
-        return 0.;
+        return value;
     }
 
     cx_assert(self->value != NULL);
 
-    if(self->value->type == CPL_TYPE_DOUBLE){
-        result = _cpl_property_value_get(self->value, CPL_TYPE_DOUBLE, cpl_func);
-        return *((double *)result);
-    }
-    else if(self->value->type == CPL_TYPE_FLOAT){
-        result = _cpl_property_value_get(self->value, CPL_TYPE_FLOAT, cpl_func);
-        return *((float *)result);
+
+    /*
+     * If the property self is of the same kind, but of a smaller rank,
+     * promote the property's value to the target type.
+     */
+
+    switch (self->value->type) {
+
+        case CPL_TYPE_FLOAT:
+        {
+            cxfloat *result = _cpl_property_value_get(self->value,
+                                                      CPL_TYPE_FLOAT, cpl_func);
+
+            value = *result;
+            break;
+        }
+
+        case CPL_TYPE_DOUBLE:
+        {
+            cxdouble *result = _cpl_property_value_get(self->value,
+                                                       CPL_TYPE_DOUBLE,
+                                                       cpl_func);
+
+            value = *result;
+            break;
+        }
+
+        default:
+        {
+            cpl_error_set_(CPL_ERROR_TYPE_MISMATCH);
+            break;
+        }
+
     }
 
-    (void)cpl_error_set_(CPL_ERROR_TYPE_MISMATCH);
-    return 0.;
+    return value;
 
 }
 
@@ -1670,7 +1904,6 @@ const char *
 cpl_property_get_string(const cpl_property *self)
 {
 
-
     cxptr result;
 
 
@@ -1699,7 +1932,7 @@ cpl_property_get_string(const cpl_property *self)
  *
  * @return
  *   The current property value. If an error occurs the function returns
- *   0.0 and an appropriate error code is set.
+ *   0. and an appropriate error code is set.
  *
  * @error
  *   <table class="ec" align="center">
@@ -1720,39 +1953,68 @@ cpl_property_get_string(const cpl_property *self)
  *   </table>
  * @enderror
  *
- * The function retrieves the float complex value currently stored in the
- * property @em self. If the value is of type double complex, the function
- * casts it to float complex before returning it.
+ * The function retrieves the value currently stored in the property @em self
+ * as a float complex. The function also accepts properties of type
+ * double complex and returns the property value as float complex.
+ *
+ * @note
+ *  Calling the function for a property of type double complex may lead to
+ *  truncation errors!
  */
 
 float complex
 cpl_property_get_float_complex(const cpl_property *self)
 {
 
-    cxptr result;
+    float complex value = 0.;
 
 
     if (self == NULL) {
-        (void)cpl_error_set_(CPL_ERROR_NULL_INPUT);
-        return (float complex)0.0;
+        cpl_error_set_(CPL_ERROR_NULL_INPUT);
+        return value;
     }
 
     cx_assert(self->value != NULL);
 
-    /* If the type is not a float, check if it is a double */
-    if (self->value->type == CPL_TYPE_FLOAT_COMPLEX){
-        result = _cpl_property_value_get(self->value, CPL_TYPE_FLOAT_COMPLEX,
-                                         cpl_func);
-        return *((float complex *)result);
-    }
-    else if (self->value->type == CPL_TYPE_DOUBLE_COMPLEX){
-        result = _cpl_property_value_get(self->value, CPL_TYPE_DOUBLE_COMPLEX,
-                                         cpl_func);
-        return (float complex)(*((double complex *)result));
+
+    /*
+     * If the property self is of type double cast it to float.
+     * This may lead to truncation errors!
+     */
+
+    switch (self->value->type) {
+
+        case CPL_TYPE_FLOAT_COMPLEX:
+        {
+            float complex *result =
+                    _cpl_property_value_get(self->value,
+                                            CPL_TYPE_FLOAT_COMPLEX,
+                                            cpl_func);
+
+            value = *result;
+            break;
+        }
+
+        case CPL_TYPE_DOUBLE_COMPLEX:
+        {
+            double complex *result =
+                    _cpl_property_value_get(self->value,
+                                            CPL_TYPE_DOUBLE_COMPLEX,
+                                            cpl_func);
+
+            value = (float complex)(*result);
+            break;
+        }
+
+        default:
+        {
+            cpl_error_set_(CPL_ERROR_TYPE_MISMATCH);
+            break;
+        }
+
     }
 
-    (void)cpl_error_set_(CPL_ERROR_TYPE_MISMATCH);
-    return (float complex)0.0;
+    return value;
 
 }
 
@@ -1765,7 +2027,7 @@ cpl_property_get_float_complex(const cpl_property *self)
  *
  * @return
  *   The current property value. If an error occurs the function returns
- *   0.0 and an appropriate error code is set.
+ *   0. and an appropriate error code is set.
  *
  * @error
  *   <table class="ec" align="center">
@@ -1786,38 +2048,64 @@ cpl_property_get_float_complex(const cpl_property *self)
  *   </table>
  * @enderror
  *
- * The function retrieves the double complex value currently stored in the
- * property @em self. If the value is of type float, the function
- * casts it to double complex before returning it.
+ * The function retrieves the value currently stored in the property @em self
+ * as a double complex. The function accepts properties of a complex type
+ * with a rank less or equal than the function's return type.
  */
 
 double complex
 cpl_property_get_double_complex(const cpl_property *self)
 {
 
-    cxptr result;
+    double complex value = 0.;
 
 
     if (self == NULL) {
-        (void)cpl_error_set_(CPL_ERROR_NULL_INPUT);
-        return (double complex)0.0;
+        cpl_error_set_(CPL_ERROR_NULL_INPUT);
+        return value;
     }
 
     cx_assert(self->value != NULL);
 
-    if(self->value->type == CPL_TYPE_DOUBLE_COMPLEX){
-        result = _cpl_property_value_get(self->value, CPL_TYPE_DOUBLE_COMPLEX,
-                                         cpl_func);
-        return *((double complex *)result);
-    }
-    else if(self->value->type == CPL_TYPE_FLOAT){
-        result = _cpl_property_value_get(self->value, CPL_TYPE_FLOAT_COMPLEX,
-                                         cpl_func);
-        return (double complex)(*((float complex *)result));
+
+    /*
+     * If the property self is of the same kind, but of a smaller rank,
+     * promote the property's value to the target type.
+     */
+
+    switch (self->value->type) {
+
+        case CPL_TYPE_FLOAT_COMPLEX:
+        {
+            float complex *result =
+                    _cpl_property_value_get(self->value,
+                                            CPL_TYPE_FLOAT_COMPLEX,
+                                            cpl_func);
+
+            value = *result;
+            break;
+        }
+
+        case CPL_TYPE_DOUBLE_COMPLEX:
+        {
+            double complex *result =
+                    _cpl_property_value_get(self->value,
+                                            CPL_TYPE_DOUBLE_COMPLEX,
+                                            cpl_func);
+
+            value = *result;
+            break;
+        }
+
+        default:
+        {
+            cpl_error_set_(CPL_ERROR_TYPE_MISMATCH);
+            break;
+        }
+
     }
 
-    (void)cpl_error_set_(CPL_ERROR_TYPE_MISMATCH);
-    return (double complex)0.0;
+    return value;
 
 }
 /**@}*/
