@@ -1,4 +1,4 @@
-/* $Id: cpl_stats.c,v 1.49 2010/11/11 09:23:18 llundin Exp $
+/* $Id: cpl_stats.c,v 1.55 2012/02/15 16:20:35 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/11/11 09:23:18 $
- * $Revision: 1.49 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2012/02/15 16:20:35 $
+ * $Revision: 1.55 $
+ * $Name: cpl-6_0 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -46,6 +46,7 @@
 #include "cpl_image_bpm.h"
 #include "cpl_image_stats.h"
 #include "cpl_mask.h"
+#include "cpl_error_impl.h"
 #include "cpl_tools.h"
 
 #include "cpl_image_defs.h"
@@ -66,15 +67,15 @@
 #define CONCAT2X(a,b) CONCAT(a,b)
 
 #define CPL_STATS_DUMP_ONE(OPERATOR, MEMBER, LABEL, CONVERTER)          \
-    if (bitmask & OPERATOR) {                                           \
-        bitmask ^= OPERATOR; /* Reset bit. At the end bitmask must be zero */ \
+    if (mode & OPERATOR) {                                           \
+        mode ^= OPERATOR; /* Reset bit. At the end mode must be zero */ \
         cpl_ensure_code(fprintf(stream, "\t\t%-13s%" CONVERTER "\n", LABEL ":", \
                                 self->MEMBER) > 0, CPL_ERROR_FILE_IO);  \
     }
 
 #define CPL_STATS_DUMP_TWO(OPERATOR, MEMBER, LABEL, CONVERTER)          \
-    if (bitmask & OPERATOR) {                                           \
-        bitmask ^= OPERATOR; /* Reset bit. At the end bitmask must be zero */ \
+    if (mode & OPERATOR) {                                           \
+        mode ^= OPERATOR; /* Reset bit. At the end mode must be zero */ \
         cpl_ensure_code(fprintf(stream, "\t\t%-13s%" CONVERTER "\n",    \
                                 "X " LABEL ":", self->CONCAT2X(MEMBER, x)) \
                         > 0,  CPL_ERROR_FILE_IO);                       \
@@ -116,12 +117,12 @@ struct _cpl_stats_
     double      sqflux;
     double      centroid_x;
     double      centroid_y;
-    int         min_x;
-    int         min_y;
-    int         max_x;
-    int         max_y;
-    int         npix;
-    unsigned    bitmask;
+    cpl_size    min_x;
+    cpl_size    min_y;
+    cpl_size    max_x;
+    cpl_size    max_y;
+    cpl_size    npix;
+    cpl_stats_mode mode;
 };
 
 /*-----------------------------------------------------------------------------
@@ -141,14 +142,14 @@ struct _cpl_stats_
   is undefined.
 
   Possible #_cpl_error_code_ set in this function:
-  - CPL_ERROR_NULL_INPUT if (one of) the input pointer(s) is NULL
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
   - CPL_ERROR_ILLEGAL_INPUT if the requested stat has not been computed in in
  */
 /*----------------------------------------------------------------------------*/
 double cpl_stats_get_min(const cpl_stats * in)
 {
     cpl_ensure(in!=NULL, CPL_ERROR_NULL_INPUT, 0.0);
-    cpl_ensure(in->bitmask & (CPL_STATS_MIN | CPL_STATS_MINPOS),
+    cpl_ensure(in->mode & (CPL_STATS_MIN | CPL_STATS_MINPOS),
                CPL_ERROR_ILLEGAL_INPUT, 0);
     return in->min;
 }
@@ -164,7 +165,7 @@ double cpl_stats_get_min(const cpl_stats * in)
 double cpl_stats_get_max(const cpl_stats * in)
 {
     cpl_ensure(in!=NULL, CPL_ERROR_NULL_INPUT, 0.0);
-    cpl_ensure(in->bitmask & (CPL_STATS_MAX | CPL_STATS_MAXPOS),
+    cpl_ensure(in->mode & (CPL_STATS_MAX | CPL_STATS_MAXPOS),
                CPL_ERROR_ILLEGAL_INPUT, 0);
     return in->max;
 }
@@ -180,7 +181,7 @@ double cpl_stats_get_max(const cpl_stats * in)
 double cpl_stats_get_mean(const cpl_stats * in)
 {
     cpl_ensure(in!=NULL, CPL_ERROR_NULL_INPUT, 0.0);
-    cpl_ensure(in->bitmask & CPL_STATS_MEAN, CPL_ERROR_ILLEGAL_INPUT, 0);
+    cpl_ensure(in->mode & CPL_STATS_MEAN, CPL_ERROR_ILLEGAL_INPUT, 0);
     return in->mean;
 }
 
@@ -195,7 +196,7 @@ double cpl_stats_get_mean(const cpl_stats * in)
 double cpl_stats_get_median(const cpl_stats * in)
 {
     cpl_ensure(in!=NULL, CPL_ERROR_NULL_INPUT, 0.0);
-    cpl_ensure(in->bitmask & CPL_STATS_MEDIAN, CPL_ERROR_ILLEGAL_INPUT, 0);
+    cpl_ensure(in->mode & CPL_STATS_MEDIAN, CPL_ERROR_ILLEGAL_INPUT, 0);
     return in->med;
 }
 
@@ -210,7 +211,7 @@ double cpl_stats_get_median(const cpl_stats * in)
 double cpl_stats_get_median_dev(const cpl_stats * in)
 {
     cpl_ensure(in!=NULL, CPL_ERROR_NULL_INPUT, 0.0);
-    cpl_ensure(in->bitmask & CPL_STATS_MEDIAN_DEV, CPL_ERROR_ILLEGAL_INPUT, 0);
+    cpl_ensure(in->mode & CPL_STATS_MEDIAN_DEV, CPL_ERROR_ILLEGAL_INPUT, 0);
     return in->med_dev;
 }
 
@@ -225,7 +226,7 @@ double cpl_stats_get_median_dev(const cpl_stats * in)
 double cpl_stats_get_stdev(const cpl_stats * in)
 {
     cpl_ensure(in!=NULL, CPL_ERROR_NULL_INPUT, -1);
-    cpl_ensure(in->bitmask & CPL_STATS_STDEV, CPL_ERROR_ILLEGAL_INPUT, 0);
+    cpl_ensure(in->mode & CPL_STATS_STDEV, CPL_ERROR_ILLEGAL_INPUT, 0);
     return in->stdev;
 }
 
@@ -240,7 +241,7 @@ double cpl_stats_get_stdev(const cpl_stats * in)
 double cpl_stats_get_flux(const cpl_stats * in)
 {
     cpl_ensure(in!=NULL, CPL_ERROR_NULL_INPUT, 0.0);
-    cpl_ensure(in->bitmask & CPL_STATS_FLUX, CPL_ERROR_ILLEGAL_INPUT, 0);
+    cpl_ensure(in->mode & CPL_STATS_FLUX, CPL_ERROR_ILLEGAL_INPUT, 0);
     return in->flux;
 }
 
@@ -255,7 +256,7 @@ double cpl_stats_get_flux(const cpl_stats * in)
 double cpl_stats_get_absflux(const cpl_stats * in)
 {
     cpl_ensure(in!=NULL, CPL_ERROR_NULL_INPUT, -1);
-    cpl_ensure(in->bitmask & CPL_STATS_ABSFLUX, CPL_ERROR_ILLEGAL_INPUT, -2);
+    cpl_ensure(in->mode & CPL_STATS_ABSFLUX, CPL_ERROR_ILLEGAL_INPUT, -2);
     return in->absflux;
 }
 
@@ -270,7 +271,7 @@ double cpl_stats_get_absflux(const cpl_stats * in)
 double cpl_stats_get_sqflux(const cpl_stats * in)
 {
     cpl_ensure(in!=NULL, CPL_ERROR_NULL_INPUT, -1);
-    cpl_ensure(in->bitmask & CPL_STATS_SQFLUX, CPL_ERROR_ILLEGAL_INPUT, -2);
+    cpl_ensure(in->mode & CPL_STATS_SQFLUX, CPL_ERROR_ILLEGAL_INPUT, -2);
     return in->sqflux;
 }
 
@@ -285,7 +286,7 @@ double cpl_stats_get_sqflux(const cpl_stats * in)
 double cpl_stats_get_centroid_x(const cpl_stats * in)
 {
     cpl_ensure(in!=NULL, CPL_ERROR_NULL_INPUT, 0.0);
-    cpl_ensure(in->bitmask & CPL_STATS_CENTROID, CPL_ERROR_ILLEGAL_INPUT,
+    cpl_ensure(in->mode & CPL_STATS_CENTROID, CPL_ERROR_ILLEGAL_INPUT,
                0);
     return in->centroid_x;
 }
@@ -301,7 +302,7 @@ double cpl_stats_get_centroid_x(const cpl_stats * in)
 double cpl_stats_get_centroid_y(const cpl_stats * in)
 {
     cpl_ensure(in!=NULL, CPL_ERROR_NULL_INPUT, 0.0);
-    cpl_ensure(in->bitmask & CPL_STATS_CENTROID, CPL_ERROR_ILLEGAL_INPUT,
+    cpl_ensure(in->mode & CPL_STATS_CENTROID, CPL_ERROR_ILLEGAL_INPUT,
                0);
     return in->centroid_y;
 }
@@ -313,13 +314,13 @@ double cpl_stats_get_centroid_y(const cpl_stats * in)
   @return   the x position (1 for the first pixel), non-positive on error.
 
   Possible #_cpl_error_code_ set in this function:
-  - CPL_ERROR_NULL_INPUT if (one of) the input pointer(s) is NULL
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
  */
 /*----------------------------------------------------------------------------*/
-int cpl_stats_get_min_x(const cpl_stats * in)
+cpl_size cpl_stats_get_min_x(const cpl_stats * in)
 {
     cpl_ensure(in!=NULL, CPL_ERROR_NULL_INPUT, -1);
-    cpl_ensure(in->bitmask & (CPL_STATS_MIN | CPL_STATS_MINPOS),
+    cpl_ensure(in->mode & (CPL_STATS_MIN | CPL_STATS_MINPOS),
                CPL_ERROR_ILLEGAL_INPUT, 0);
     return in->min_x;
 }
@@ -331,13 +332,13 @@ int cpl_stats_get_min_x(const cpl_stats * in)
   @return   the y position (1 for the first pixel), non-positive on error.
 
   Possible #_cpl_error_code_ set in this function:
-  - CPL_ERROR_NULL_INPUT if (one of) the input pointer(s) is NULL
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
  */
 /*----------------------------------------------------------------------------*/
-int cpl_stats_get_min_y(const cpl_stats * in)
+cpl_size cpl_stats_get_min_y(const cpl_stats * in)
 {
     cpl_ensure(in!=NULL, CPL_ERROR_NULL_INPUT, -1);
-    cpl_ensure(in->bitmask & (CPL_STATS_MIN | CPL_STATS_MINPOS),
+    cpl_ensure(in->mode & (CPL_STATS_MIN | CPL_STATS_MINPOS),
                CPL_ERROR_ILLEGAL_INPUT, 0);
     return in->min_y;
 }
@@ -349,13 +350,13 @@ int cpl_stats_get_min_y(const cpl_stats * in)
   @return   the x position (1 for the first pixel), non-positive on error.
 
   Possible #_cpl_error_code_ set in this function:
-  - CPL_ERROR_NULL_INPUT if (one of) the input pointer(s) is NULL
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
  */
 /*----------------------------------------------------------------------------*/
-int cpl_stats_get_max_x(const cpl_stats * in)
+cpl_size cpl_stats_get_max_x(const cpl_stats * in)
 {
     cpl_ensure(in!=NULL, CPL_ERROR_NULL_INPUT, -1);
-    cpl_ensure(in->bitmask & (CPL_STATS_MAX | CPL_STATS_MAXPOS),
+    cpl_ensure(in->mode & (CPL_STATS_MAX | CPL_STATS_MAXPOS),
                CPL_ERROR_ILLEGAL_INPUT, 0);
     return in->max_x;
 }
@@ -367,13 +368,13 @@ int cpl_stats_get_max_x(const cpl_stats * in)
   @return   the y position (1 for the first pixel), non-positive on error.
 
   Possible #_cpl_error_code_ set in this function:
-  - CPL_ERROR_NULL_INPUT if (one of) the input pointer(s) is NULL
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
  */
 /*----------------------------------------------------------------------------*/
-int cpl_stats_get_max_y(const cpl_stats * in)
+cpl_size cpl_stats_get_max_y(const cpl_stats * in)
 {
     cpl_ensure(in!=NULL, CPL_ERROR_NULL_INPUT, -1);
-    cpl_ensure(in->bitmask & (CPL_STATS_MAX | CPL_STATS_MAXPOS),
+    cpl_ensure(in->mode & (CPL_STATS_MAX | CPL_STATS_MAXPOS),
                CPL_ERROR_ILLEGAL_INPUT, 0);
     return in->max_y;
 }
@@ -388,10 +389,10 @@ int cpl_stats_get_max_y(const cpl_stats * in)
   to be determined.
 
   Possible #_cpl_error_code_ set in this function:
-  - CPL_ERROR_NULL_INPUT if (one of) the input pointer(s) is NULL
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
  */
 /*----------------------------------------------------------------------------*/
-int cpl_stats_get_npix(const cpl_stats * in)
+cpl_size cpl_stats_get_npix(const cpl_stats * in)
 {
     cpl_ensure(in!=NULL, CPL_ERROR_NULL_INPUT, -1);
     return in->npix;
@@ -417,18 +418,19 @@ void cpl_stats_delete(cpl_stats * stats)
 /**
   @brief    Compute various statistics of an image sub-window.
   @param    image       Input image.
-  @param    bitmask     Defines which parameters have to be computed
+  @param    mode        Bit field specifying which statistics to compute
   @param    llx         Lower left x position (FITS convention)
   @param    lly         Lower left y position (FITS convention)
   @param    urx         Upper right x position (FITS convention)
   @param    ury         Upper right y position (FITS convention)
   @return   1 newly allocated cpl_stats structure or NULL in error case
 
-  Compute various images statistics.
+  Compute various image statistics.
 
   The specified bounds are included in the specified region.
-  The user specifies the statistics he wants to get with a bitmask.
-  Possible requested values are:
+
+  The statistics to compute is specified with a bit field,
+  that may be set to any of these values
   - CPL_STATS_MIN
   - CPL_STATS_MAX
   - CPL_STATS_MEAN
@@ -441,16 +443,22 @@ void cpl_stats_delete(cpl_stats * stats)
   - CPL_STATS_CENTROID
   - CPL_STATS_MINPOS
   - CPL_STATS_MAXPOS
-  - CPL_STATS_ALL
-  or any bitwise combination of these.
+  or any bitwise or (|) of these.
+  For convenience the special value CPL_STATS_ALL may also be used,
+  it is the conbination of all of the above values.
 
-  E.g. the bitmask would be CPL_STATS_MIN | CPL_STATS_MEDIAN in order to get
+  E.g. the mode CPL_STATS_MIN | CPL_STATS_MEDIAN specifies
   the minimum and the median of the image.
 
   In the case of CPL_STATS_MIN and CPL_STATS_MAX where more than one set of
   coordinates share the extremum it is undefined which of those coordinates
   will be returned.
 
+  On i386 platforms there can be significant differences in the round-off of the
+  computation of single statistics and statistics computed via CPL_STATS_ALL.
+  This is especially true for squared quantities such as the CPL_STATS_SQFLUX
+  and CPL_STATS_STDEV. 
+
   Images can be CPL_TYPE_DOUBLE, CPL_TYPE_FLOAT, CPL_TYPE_INT.
 
   For the CPL_STATS_CENTROID computation, if there are negative pixels,
@@ -460,24 +468,24 @@ void cpl_stats_delete(cpl_stats * stats)
   The returned object must be deallocated using cpl_stats_delete().
 
   Possible #_cpl_error_code_ set in this function:
-  - CPL_ERROR_NULL_INPUT if (one of) the input pointer(s) is NULL
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
   - CPL_ERROR_ACCESS_OUT_OF_RANGE if the defined window is not in the image
   - CPL_ERROR_ILLEGAL_INPUT if the window definition is wrong (e.g llx > urx)
   - CPL_ERROR_DATA_NOT_FOUND if all the pixels are bad
   - CPL_ERROR_INVALID_TYPE if the passed image type is not supported
-  - CPL_ERROR_INVALID_TYPE if bitmask is 1, e.g. due to a logical or (||)
+  - CPL_ERROR_INVALID_TYPE if mode is 1, e.g. due to a logical or (||)
       of the allowed options.
-  - CPL_ERROR_UNSUPPORTED_MODE if bitmask is otherwise different from the
+  - CPL_ERROR_UNSUPPORTED_MODE if mode is otherwise different from the
        allowed options.
  */
 /*----------------------------------------------------------------------------*/
 cpl_stats * cpl_stats_new_from_image_window(
         const cpl_image *   image,
-        unsigned            bitmask,
-        int                 llx,
-        int                 lly,
-        int                 urx,
-        int                 ury)
+        cpl_stats_mode      mode,
+        cpl_size            llx,
+        cpl_size            lly,
+        cpl_size            urx,
+        cpl_size            ury)
 {
     cpl_stats    * self;
     double         pix_sum = 0.0;
@@ -488,28 +496,28 @@ cpl_stats * cpl_stats_new_from_image_window(
     double         pix_var = 0.0; /* The accumulated variance sum */
     double         max_pix = DBL_MAX; /* Avoid (false) uninit warning */
     double         min_pix = DBL_MAX; /* Avoid (false) uninit warning */
-    int            max_pos = -1; /* Avoid (false) uninit warning */
-    int            min_pos = -1; /* Avoid (false) uninit warning */
+    cpl_size       max_pos = -1; /* Avoid (false) uninit warning */
+    cpl_size       min_pos = -1; /* Avoid (false) uninit warning */
     double         ipix    = 0.0; /* Counter of pixels used */
-    int            npix;
-    const int      mpix = (urx-llx+1) * (ury-lly+1);
-    int            pos;
+    cpl_size       npix;
+    const cpl_size mpix = (urx-llx+1) * (ury-lly+1);
+    cpl_size       pos;
     /* Two statistics computation categories defined here */
-    const unsigned minmax_cat = bitmask &
+    const cpl_stats_mode minmax_cat = mode &
         (CPL_STATS_MIN|CPL_STATS_MAX|CPL_STATS_MINPOS|
          CPL_STATS_MAXPOS|CPL_STATS_CENTROID);
-    const unsigned flux_cat = bitmask &
+    const cpl_stats_mode flux_cat = mode &
         (CPL_STATS_FLUX | CPL_STATS_ABSFLUX | CPL_STATS_SQFLUX);
-    const unsigned variance_cat = bitmask &
+    const cpl_stats_mode variance_cat = mode &
         (CPL_STATS_MEAN | CPL_STATS_STDEV);
 
     /* Index of 1st good pixel - used for initialization */
-    int                 firstgoodpos;
+    cpl_size            firstgoodpos;
     /* The number of bad pixels inside the subwindow */
-    int                 nbadpix;
+    cpl_size            nbadpix;
     /* A map of the the bad pixels in the input */
     const cpl_binary  * badmap;
-    int                 i, j;
+    cpl_size            i, j;
 
     /* Test inputs */
     cpl_ensure(image != NULL, CPL_ERROR_NULL_INPUT, NULL);
@@ -518,13 +526,13 @@ cpl_stats * cpl_stats_new_from_image_window(
     cpl_ensure(lly > 0 && lly <= image->ny && ury > 0 && ury <= image->ny,
            CPL_ERROR_ACCESS_OUT_OF_RANGE, NULL);
     cpl_ensure(urx >= llx && ury >= lly, CPL_ERROR_ILLEGAL_INPUT, NULL);
-    cpl_ensure(bitmask != 1, CPL_ERROR_INVALID_TYPE, NULL);
-    cpl_ensure(bitmask != 0, CPL_ERROR_UNSUPPORTED_MODE, NULL);
-    cpl_ensure((bitmask & ~CPL_STATS_ALL) == 0, CPL_ERROR_UNSUPPORTED_MODE,
+    cpl_ensure(mode != 1, CPL_ERROR_INVALID_TYPE, NULL);
+    cpl_ensure(mode != 0, CPL_ERROR_UNSUPPORTED_MODE, NULL);
+    cpl_ensure((mode & ~CPL_STATS_ALL) == 0, CPL_ERROR_UNSUPPORTED_MODE,
                NULL);
 
     /* Need the median for the median deviation */
-    if (bitmask & CPL_STATS_MEDIAN_DEV) bitmask |= CPL_STATS_MEDIAN;
+    if (mode & CPL_STATS_MEDIAN_DEV) mode |= CPL_STATS_MEDIAN;
 
     /* Get the bad pixels map */
     badmap = image->bpm == NULL ? NULL : cpl_mask_get_data_const(image->bpm);
@@ -532,14 +540,21 @@ cpl_stats * cpl_stats_new_from_image_window(
     /* Get the first good pixel and the number of bad pixels */
     nbadpix = 0;
     if (badmap != NULL) {
+        const cpl_binary * bpmj1 = badmap + (lly-1) * image->nx - 1;
+        size_t urxsz = (size_t)urx;
+        size_t urysz = (size_t)ury;
+        size_t isz, jsz;
+
+        cpl_ensure((cpl_size)urxsz == urx, CPL_ERROR_UNSUPPORTED_MODE, NULL);
+        cpl_ensure((cpl_size)urysz == ury, CPL_ERROR_UNSUPPORTED_MODE, NULL);
+
         firstgoodpos = -1;
-        for (j=lly; j<=ury; j++) {
-            for (i=llx; i<=urx; i++) {
-                pos = (i-1)+(j-1)*image->nx;
-                if (badmap[pos] == CPL_BINARY_1) {
+        for (jsz = (size_t)lly; jsz < 1 + urysz; jsz++, bpmj1 += image->nx) {
+            for (isz = (size_t)llx; isz < 1 + urxsz; isz++) {
+                if (bpmj1[isz] == CPL_BINARY_1) {
                     nbadpix++;
                 } else if (firstgoodpos < 0) {
-                    firstgoodpos = pos;
+                    firstgoodpos = (isz - 1) + (jsz - 1) * image->nx;
                 }
             }
         }
@@ -556,14 +571,14 @@ cpl_stats * cpl_stats_new_from_image_window(
     self = cpl_malloc(sizeof(cpl_stats));
 
     /* When a member of the struct is accessed check that it was initialized */
-    self->bitmask = bitmask;
+    self->mode = mode;
 
     self->npix = npix;
 
     /* Code duplication not avoidable for performance reasons */
     /* The tests should stay outside the loops */
 
-    if (bitmask & CPL_STATS_MEDIAN) {
+    if (mode & CPL_STATS_MEDIAN) {
         switch (image->type) {
 #define CPL_OPERATION CPL_IMAGE_STATS_MEDIAN
 #define CPL_CLASS CPL_CLASS_DOUBLE
@@ -579,11 +594,12 @@ cpl_stats * cpl_stats_new_from_image_window(
         default:
             /* See comment in previous switch() default: */
             cpl_stats_delete(self);
-            cpl_ensure(0, CPL_ERROR_INVALID_TYPE, NULL);
+            (void)cpl_error_set_(CPL_ERROR_INVALID_TYPE);
+            return NULL;
         }
     }
 
-    if (bitmask == CPL_STATS_ALL) {
+    if (mode == CPL_STATS_ALL) {
         /* Switch on image type */
         switch (image->type) {
 #define CPL_OPERATION CPL_IMAGE_STATS_ALL
@@ -608,7 +624,8 @@ cpl_stats * cpl_stats_new_from_image_window(
          * error code in this default section.
          */
             cpl_stats_delete(self);
-            cpl_ensure(0, CPL_ERROR_INVALID_TYPE, NULL);
+            (void)cpl_error_set_(CPL_ERROR_INVALID_TYPE);
+            return NULL;
         }
 
     } else {
@@ -627,9 +644,10 @@ cpl_stats * cpl_stats_new_from_image_window(
 #undef CPL_CLASS
 #undef CPL_OPERATION
             default:
-        /* See comment in previous switch() default: */
+                /* See comment in previous switch() default: */
                 cpl_stats_delete(self);
-                cpl_ensure(0, CPL_ERROR_INVALID_TYPE, NULL);
+                (void)cpl_error_set_(CPL_ERROR_INVALID_TYPE);
+                return NULL;
             }
 
         }
@@ -648,9 +666,10 @@ cpl_stats * cpl_stats_new_from_image_window(
 #undef CPL_CLASS
 #undef CPL_OPERATION
             default:
-        /* See comment in previous switch() default: */
+                /* See comment in previous switch() default: */
                 cpl_stats_delete(self);
-                cpl_ensure(0, CPL_ERROR_INVALID_TYPE, NULL);
+                (void)cpl_error_set_(CPL_ERROR_INVALID_TYPE);
+                return NULL;
             }
         }
         if (variance_cat) {
@@ -668,13 +687,14 @@ cpl_stats * cpl_stats_new_from_image_window(
 #undef CPL_CLASS
 #undef CPL_OPERATION
             default:
-        /* See comment in previous switch() default: */
+                /* See comment in previous switch() default: */
                 cpl_stats_delete(self);
-                cpl_ensure(0, CPL_ERROR_INVALID_TYPE, NULL);
+                (void)cpl_error_set_(CPL_ERROR_INVALID_TYPE);
+                return NULL;
             }
         }
 
-        if (bitmask & CPL_STATS_MEDIAN_DEV) {
+        if (mode & CPL_STATS_MEDIAN_DEV) {
             switch (image->type) {
 #define CPL_OPERATION CPL_IMAGE_STATS_MEDIAN_DEV
 #define CPL_CLASS CPL_CLASS_DOUBLE
@@ -690,22 +710,23 @@ cpl_stats * cpl_stats_new_from_image_window(
             default:
                 /* See comment in previous switch() default: */
                 cpl_stats_delete(self);
-                cpl_ensure(0, CPL_ERROR_INVALID_TYPE, NULL);
+                (void)cpl_error_set_(CPL_ERROR_INVALID_TYPE);
+                return NULL;
             }
         }
     }
 
-    if (bitmask & CPL_STATS_MIN)  self->min  = min_pix;
-    if (bitmask & CPL_STATS_MAX)  self->max  = max_pix;
-    if (bitmask & CPL_STATS_MEAN) self->mean = pix_mean;
+    if (mode & CPL_STATS_MIN)  self->min  = min_pix;
+    if (mode & CPL_STATS_MAX)  self->max  = max_pix;
+    if (mode & CPL_STATS_MEAN) self->mean = pix_mean;
 
-    if (bitmask&CPL_STATS_STDEV) {
+    if (mode &CPL_STATS_STDEV) {
 
         /* Compute the bias-corrected standard deviation. */
         self->stdev = npix < 2 ? 0.0 : sqrt(pix_var/(double)(npix-1));
     }
 
-    if (bitmask&CPL_STATS_CENTROID) {
+    if (mode &CPL_STATS_CENTROID) {
         double sum_xz  = 0.0;
         double sum_yz  = 0.0;
         double sum_z   = 0.0;
@@ -725,16 +746,17 @@ cpl_stats * cpl_stats_new_from_image_window(
 #undef CPL_CLASS
 #undef CPL_OPERATION
           default:
-            /* See comment in previous switch() default: */
-            cpl_stats_delete(self);
-            cpl_ensure(0, CPL_ERROR_INVALID_TYPE, NULL);
+              /* See comment in previous switch() default: */
+              cpl_stats_delete(self);
+              (void)cpl_error_set_(CPL_ERROR_INVALID_TYPE);
+              return NULL;
         }
         if (sum_z > 0.0) {
             self->centroid_x =  sum_xz / sum_z;
             self->centroid_y =  sum_yz / sum_z;
         } else {
-            self->centroid_x = sum_x / npix;
-            self->centroid_y = sum_y / npix;
+            self->centroid_x = sum_x / (double)npix;
+            self->centroid_y = sum_y / (double)npix;
         }
         /* The centroid has to be inside the provided sub-window */
         /* It can only fail to be that due to round-off, for example
@@ -743,34 +765,34 @@ cpl_stats * cpl_stats_new_from_image_window(
         /* The round-off is especially likely to happen on a 1D-image,
            e.g. when lly == ury */
 
-        if (self->centroid_x < llx) {
-            assert( llx - self->centroid_x < FLT_EPSILON );
-            self->centroid_x = llx;
-        } else if (self->centroid_x > urx) {
-            assert( self->centroid_x - urx < FLT_EPSILON );
-            self->centroid_x = urx;
+        if (self->centroid_x < (double)llx) {
+            assert( (double)llx - self->centroid_x < FLT_EPSILON );
+            self->centroid_x = (double)llx;
+        } else if (self->centroid_x > (double)urx) {
+            assert( self->centroid_x - (double)urx < FLT_EPSILON );
+            self->centroid_x = (double)urx;
         }
-        if (self->centroid_y < lly) {
-            assert( lly - self->centroid_y < FLT_EPSILON );
-            self->centroid_y = lly;
-        } else if (self->centroid_y > ury) {
-            assert( self->centroid_y - ury < FLT_EPSILON );
-            self->centroid_y = ury;
+        if (self->centroid_y < (double)lly) {
+            assert( (double)lly - self->centroid_y < FLT_EPSILON );
+            self->centroid_y = (double)lly;
+        } else if (self->centroid_y > (double)ury) {
+            assert( self->centroid_y - (double)ury < FLT_EPSILON );
+            self->centroid_y = (double)ury;
         }
     }
-    if (bitmask&CPL_STATS_FLUX) self->flux = pix_sum;
-    if (bitmask&CPL_STATS_ABSFLUX) self->absflux = abs_sum;
-    if (bitmask&CPL_STATS_SQFLUX) self->sqflux = sqr_sum;
-    if (bitmask&CPL_STATS_MINPOS) {
+    if (mode &CPL_STATS_FLUX) self->flux = pix_sum;
+    if (mode &CPL_STATS_ABSFLUX) self->absflux = abs_sum;
+    if (mode &CPL_STATS_SQFLUX) self->sqflux = sqr_sum;
+    if (mode &CPL_STATS_MINPOS) {
         self->min_x = 1 + min_pos % image->nx;
         self->min_y = 1 + min_pos / image->nx;
     }
-    if (bitmask&CPL_STATS_MAXPOS) {
+    if (mode &CPL_STATS_MAXPOS) {
         self->max_x = 1 + max_pos % image->nx;
         self->max_y = 1 + max_pos / image->nx;
     }
 
-    if (bitmask&CPL_STATS_MEDIAN_DEV) self->med_dev = dev_sum / (double) npix;
+    if (mode &CPL_STATS_MEDIAN_DEV) self->med_dev = dev_sum / (double) npix;
 
     return self;
 }
@@ -778,23 +800,23 @@ cpl_stats * cpl_stats_new_from_image_window(
 /*----------------------------------------------------------------------------*/
 /**
   @brief    Compute various statistics of an image.
-  @param    image       input image.
-  @param    bitmask     Defines which parameters have to be computed
+  @param    image    input image.
+  @param    mode     Bit field specifying which statistics to compute
   @return   1 newly allocated cpl_stats structure or NULL in error case
   @see      cpl_stats_new_from_image_window()
  */
 /*----------------------------------------------------------------------------*/
 cpl_stats * cpl_stats_new_from_image(const cpl_image * image,
-                                     unsigned          bitmask)
+                                     cpl_stats_mode    mode)
 {
     cpl_stats * self;
 
     cpl_ensure(image != NULL, CPL_ERROR_NULL_INPUT, NULL);
-    self = cpl_stats_new_from_image_window(image, bitmask, 1, 1, image->nx,
+    self = cpl_stats_new_from_image_window(image, mode, 1, 1, image->nx,
                                            image->ny);
 
     /* Propagate error, if any */
-    cpl_ensure(self != NULL, cpl_error_get_code(), NULL);
+    if (self == NULL) (void)cpl_error_set_where_();
 
     return self;
 }
@@ -803,7 +825,7 @@ cpl_stats * cpl_stats_new_from_image(const cpl_image * image,
 /**
   @brief    Dump a cpl_stats object
   @param    self     cpl_stats object to dump
-  @param    bitmask  Defines which parameters have to be dumped.
+  @param    mode     Bit field specifying which statistics to dump
   @param    stream   The output stream
   @return   CPL_ERROR_NONE or the relevant the #_cpl_error_code_
   @see      cpl_stats_new_from_image_window()
@@ -811,21 +833,21 @@ cpl_stats * cpl_stats_new_from_image(const cpl_image * image,
   It is an error to request parameters that have not been set.
 
   Possible #_cpl_error_code_ set in this function:
-  - CPL_ERROR_NULL_INPUT if (one of) the input pointer(s) is NULL
-  - CPL_ERROR_ILLEGAL_INPUT if bitmask specifies statistics that have not been
+  - CPL_ERROR_NULL_INPUT if an input pointer is NULL
+  - CPL_ERROR_ILLEGAL_INPUT if mode specifies statistics that have not been
     computed
   - CPL_ERROR_FILE_IO if the write fails
  */
 /*----------------------------------------------------------------------------*/
 cpl_error_code cpl_stats_dump(const cpl_stats * self,
-                              unsigned          bitmask,
+                              cpl_stats_mode mode,
                               FILE            * stream)
 {
 
     cpl_ensure_code(self   != NULL, CPL_ERROR_NULL_INPUT);
     cpl_ensure_code(stream != NULL, CPL_ERROR_NULL_INPUT);
 
-    cpl_ensure_code((bitmask & ~self->bitmask) == 0, CPL_ERROR_ILLEGAL_INPUT);
+    cpl_ensure_code((mode & ~self->mode) == 0, CPL_ERROR_ILLEGAL_INPUT);
 
     cpl_ensure_code( fprintf(stream, "\t\tPixel count: %d\n", self->npix) >0,
                      CPL_ERROR_FILE_IO);
@@ -845,9 +867,7 @@ cpl_error_code cpl_stats_dump(const cpl_stats * self,
     CPL_STATS_DUMP_TWO(CPL_STATS_MAXPOS,     max,      "max. pos.",  "d");
 
     /* Failure here means a new member has been added, without dump support */
-    cpl_ensure_code(!bitmask, CPL_ERROR_UNSUPPORTED_MODE);
-
-    return CPL_ERROR_NONE;
+    return mode ? cpl_error_set_(CPL_ERROR_UNSUPPORTED_MODE) : CPL_ERROR_NONE;
 }
 
 /**@}*/
