@@ -1,4 +1,4 @@
-/* $Id: cpl_table.h,v 1.65 2011/01/12 09:28:46 cizzo Exp $
+/* $Id: cpl_table.h,v 1.67 2011/03/01 13:15:30 cizzo Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: cizzo $
- * $Date: 2011/01/12 09:28:46 $
- * $Revision: 1.65 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2011/03/01 13:15:30 $
+ * $Revision: 1.67 $
+ * $Name: cpl-6_0 $
  */
 
 
@@ -59,11 +59,11 @@ typedef enum _cpl_table_select_operator_
  * Constructors and destructors:
  */
 
-cpl_table *cpl_table_new(int);
+cpl_table *cpl_table_new(cpl_size);
 
 cpl_error_code cpl_table_new_column(cpl_table *, const char *, cpl_type);
 cpl_error_code cpl_table_new_column_array(cpl_table *, const char *,
-                                          cpl_type, int);
+                                          cpl_type, cpl_size);
 
 cpl_error_code cpl_table_set_column_savetype(cpl_table *, 
                                              const char *name, cpl_type);
@@ -90,8 +90,8 @@ void cpl_table_delete(cpl_table *);
  * Methods:
  */
 
-int cpl_table_get_nrow(const cpl_table *);
-int cpl_table_get_ncol(const cpl_table *);
+cpl_size cpl_table_get_nrow(const cpl_table *);
+cpl_size cpl_table_get_ncol(const cpl_table *);
 
 cpl_type cpl_table_get_column_type(const cpl_table *, const char *name);
 
@@ -103,12 +103,13 @@ cpl_error_code cpl_table_set_column_format(cpl_table *, const char *name,
                                            const char *format);
 const char *cpl_table_get_column_format(const cpl_table *, const char *name);
 
-cpl_error_code cpl_table_set_column_depth(cpl_table *, const char *, int);
-int cpl_table_get_column_depth(const cpl_table *, const char *);
-int cpl_table_get_column_dimensions(const cpl_table *, const char *);
+cpl_error_code cpl_table_set_column_depth(cpl_table *, const char *, cpl_size);
+cpl_size cpl_table_get_column_depth(const cpl_table *, const char *);
+cpl_size cpl_table_get_column_dimensions(const cpl_table *, const char *);
 cpl_error_code cpl_table_set_column_dimensions(cpl_table *, const char *,
                                                const cpl_array *);
-int cpl_table_get_column_dimension(const cpl_table *, const char *, int);
+cpl_size cpl_table_get_column_dimension(const cpl_table *, 
+                                        const char *, cpl_size);
 
 int *cpl_table_get_data_int(cpl_table *, const char *name);
 const int *cpl_table_get_data_int_const(const cpl_table *, 
@@ -134,58 +135,60 @@ cpl_array **cpl_table_get_data_array(cpl_table *, const char *);
 const cpl_array **cpl_table_get_data_array_const(const cpl_table *, 
                                                         const char *);
 
-double cpl_table_get(const cpl_table *, const char *, int, int *null);
-int cpl_table_get_int(const cpl_table *, const char *, int, int *null);
-float cpl_table_get_float(const cpl_table *, const char *, int, int *null);
-double cpl_table_get_double(const cpl_table *, const char *, int, int *null);
+double cpl_table_get(const cpl_table *, const char *, cpl_size, int *null);
+int cpl_table_get_int(const cpl_table *, const char *, cpl_size, int *null);
+float cpl_table_get_float(const cpl_table *, const char *, cpl_size, int *null);
+double cpl_table_get_double(const cpl_table *, const char *, cpl_size, 
+                                                                int *null);
 #ifdef _Complex_I
 double complex cpl_table_get_complex(const cpl_table *,
-                                     const char *, int, int *);
+                                     const char *, cpl_size, int *);
 float complex cpl_table_get_float_complex(const cpl_table *, 
-                                          const char *, int, int *null);
+                                          const char *, cpl_size, int *null);
 double complex cpl_table_get_double_complex(const cpl_table *, 
-                                          const char *, int, int *null);
+                                          const char *, cpl_size, int *null);
 #endif
-const char *cpl_table_get_string(const cpl_table *, const char *, int);
-const cpl_array *cpl_table_get_array(const cpl_table *, const char *, int);
+const char *cpl_table_get_string(const cpl_table *, const char *, cpl_size);
+const cpl_array *cpl_table_get_array(const cpl_table *, const char *, cpl_size);
 
-cpl_error_code cpl_table_set(cpl_table *, const char *, int, double);
-cpl_error_code cpl_table_set_int(cpl_table *, const char *, int, int);
-cpl_error_code cpl_table_set_float(cpl_table *, const char *, int, float);
-cpl_error_code cpl_table_set_double(cpl_table *, const char *, int, double);
+cpl_error_code cpl_table_set(cpl_table *, const char *, cpl_size, double);
+cpl_error_code cpl_table_set_int(cpl_table *, const char *, cpl_size, int);
+cpl_error_code cpl_table_set_float(cpl_table *, const char *, cpl_size, float);
+cpl_error_code cpl_table_set_double(cpl_table *, const char *, cpl_size, 
+                                                                    double);
 #ifdef _Complex_I
 cpl_error_code cpl_table_set_complex(cpl_table *, 
-                                     const char *, int, double complex);
+                                     const char *, cpl_size, double complex);
 cpl_error_code cpl_table_set_float_complex(cpl_table *, 
-                                           const char *, int, float complex);
+                                     const char *, cpl_size, float complex);
 cpl_error_code cpl_table_set_double_complex(cpl_table *, 
-                                            const char *, int, double complex);
+                                     const char *, cpl_size, double complex);
 #endif
-cpl_error_code cpl_table_set_string(cpl_table *, const char *, int, 
+cpl_error_code cpl_table_set_string(cpl_table *, const char *, cpl_size, 
                                     const char *);
-cpl_error_code cpl_table_set_array(cpl_table *, const char *, int, 
+cpl_error_code cpl_table_set_array(cpl_table *, const char *, cpl_size, 
                                    const cpl_array *);
 
 cpl_error_code cpl_table_fill_column_window(cpl_table *, const char *, 
-                                     int, int, double);
+                                     cpl_size, cpl_size, double);
 cpl_error_code cpl_table_fill_column_window_int(cpl_table *, const char *, 
-                                         int, int, int);
+                                         cpl_size, cpl_size, int);
 cpl_error_code cpl_table_fill_column_window_float(cpl_table *, const char *, 
-                                           int, int, float);
+                                           cpl_size, cpl_size, float);
 cpl_error_code cpl_table_fill_column_window_double(cpl_table *, const char *, 
-                                            int, int, double);
+                                            cpl_size, cpl_size, double);
 #ifdef _Complex_I
 cpl_error_code cpl_table_fill_column_window_complex(cpl_table *, const char *, 
-                                     int, int, double complex);
+                                     cpl_size, cpl_size, double complex);
 cpl_error_code cpl_table_fill_column_window_float_complex(cpl_table *, 
-                                 const char *, int, int, float complex);
+                     const char *, cpl_size, cpl_size, float complex);
 cpl_error_code cpl_table_fill_column_window_double_complex(cpl_table *, 
-                                 const char *, int, int, double complex);
+                     const char *, cpl_size, cpl_size, double complex);
 #endif
 cpl_error_code cpl_table_fill_column_window_string(cpl_table *, const char *, 
-                                            int, int, const char *);
+                                      cpl_size, cpl_size, const char *);
 cpl_error_code cpl_table_fill_column_window_array(cpl_table *, const char *, 
-                                                  int, int, const cpl_array *);
+                                      cpl_size, cpl_size, const cpl_array *);
 
 cpl_error_code cpl_table_copy_data_int(cpl_table *, const char *, const int *);
 cpl_error_code cpl_table_copy_data_float(cpl_table *, const char *, 
@@ -201,15 +204,15 @@ cpl_error_code cpl_table_copy_data_double_complex(cpl_table *, const char *,
 cpl_error_code cpl_table_copy_data_string(cpl_table *, const char *, 
                                           const char **);
 
-cpl_error_code cpl_table_shift_column(cpl_table *, const char *, int);
+cpl_error_code cpl_table_shift_column(cpl_table *, const char *, cpl_size);
 
-cpl_error_code cpl_table_set_invalid(cpl_table *, const char *, int);
+cpl_error_code cpl_table_set_invalid(cpl_table *, const char *, cpl_size);
 
 cpl_error_code cpl_table_set_column_invalid(cpl_table *, 
-                                            const char *, int, int);
+                                            const char *, cpl_size, cpl_size);
 
-int cpl_table_is_valid(const cpl_table *, const char *, int);
-int cpl_table_count_invalid(const cpl_table *, const char *);
+int cpl_table_is_valid(const cpl_table *, const char *, cpl_size);
+cpl_size cpl_table_count_invalid(const cpl_table *, const char *);
 int cpl_table_has_invalid(const cpl_table *table, const char *name);
 int cpl_table_has_valid(const cpl_table *table, const char *name);
 
@@ -237,21 +240,21 @@ int cpl_table_has_column(const cpl_table *, const char *);
 const char *cpl_table_get_column_name(const cpl_table *);
 cpl_array *cpl_table_get_column_names(const cpl_table *table);
 
-cpl_error_code cpl_table_set_size(cpl_table *, int);
+cpl_error_code cpl_table_set_size(cpl_table *, cpl_size);
 
 cpl_table *cpl_table_duplicate(const cpl_table *);
 
-cpl_table *cpl_table_extract(const cpl_table *, int, int);
+cpl_table *cpl_table_extract(const cpl_table *, cpl_size, cpl_size);
 cpl_table *cpl_table_extract_selected(const cpl_table *);
 cpl_array *cpl_table_where_selected(const cpl_table *);
 
 cpl_error_code cpl_table_erase_selected(cpl_table *);
-cpl_error_code cpl_table_erase_window(cpl_table *, int, int);
-cpl_error_code cpl_table_insert_window(cpl_table *, int, int);
+cpl_error_code cpl_table_erase_window(cpl_table *, cpl_size, cpl_size);
+cpl_error_code cpl_table_insert_window(cpl_table *, cpl_size, cpl_size);
 
 int cpl_table_compare_structure(const cpl_table *, const cpl_table *);
 
-cpl_error_code cpl_table_insert(cpl_table *, const cpl_table *, int);
+cpl_error_code cpl_table_insert(cpl_table *, const cpl_table *, cpl_size);
 
 cpl_error_code cpl_table_cast_column(cpl_table *, 
                                      const char *, const char *, cpl_type);
@@ -294,9 +297,9 @@ cpl_error_code cpl_table_erase_invalid(cpl_table *);
 double cpl_table_get_column_max(const cpl_table *, const char *);
 double cpl_table_get_column_min(const cpl_table *, const char *);
 cpl_error_code cpl_table_get_column_maxpos(const cpl_table *, 
-                                           const char *, int *);
+                                           const char *, cpl_size *);
 cpl_error_code cpl_table_get_column_minpos(const cpl_table *, 
-                                           const char *, int *);
+                                           const char *, cpl_size *);
 
 #ifdef _Complex_I
 double complex cpl_table_get_column_mean_complex(const cpl_table *, 
@@ -308,62 +311,62 @@ double cpl_table_get_column_stdev(const cpl_table *, const char *);
 
 cpl_error_code cpl_table_sort(cpl_table *, const cpl_propertylist *);
 
-int cpl_table_and_selected_window(cpl_table *, int, int);
-int cpl_table_or_selected_window(cpl_table *, int, int);
+cpl_size cpl_table_and_selected_window(cpl_table *, cpl_size, cpl_size);
+cpl_size cpl_table_or_selected_window(cpl_table *, cpl_size, cpl_size);
 
-int cpl_table_not_selected(cpl_table *);
+cpl_size cpl_table_not_selected(cpl_table *);
 
-cpl_error_code cpl_table_select_row(cpl_table *, int);
-cpl_error_code cpl_table_unselect_row(cpl_table *, int);
+cpl_error_code cpl_table_select_row(cpl_table *, cpl_size);
+cpl_error_code cpl_table_unselect_row(cpl_table *, cpl_size);
 cpl_error_code cpl_table_select_all(cpl_table *);
 cpl_error_code cpl_table_unselect_all(cpl_table *);
 
-int cpl_table_and_selected_invalid(cpl_table *, const char *);
-int cpl_table_or_selected_invalid(cpl_table *, const char *);
+cpl_size cpl_table_and_selected_invalid(cpl_table *, const char *);
+cpl_size cpl_table_or_selected_invalid(cpl_table *, const char *);
 
-int cpl_table_and_selected_int(cpl_table *, const char *, 
+cpl_size cpl_table_and_selected_int(cpl_table *, const char *, 
                                   cpl_table_select_operator, int);
-int cpl_table_or_selected_int(cpl_table *, const char *, 
+cpl_size cpl_table_or_selected_int(cpl_table *, const char *, 
                                   cpl_table_select_operator, int);
 
-int cpl_table_and_selected_float(cpl_table *, const char *, 
+cpl_size cpl_table_and_selected_float(cpl_table *, const char *, 
                                     cpl_table_select_operator, float);
-int cpl_table_or_selected_float(cpl_table *, const char *, 
+cpl_size cpl_table_or_selected_float(cpl_table *, const char *, 
                                     cpl_table_select_operator, float);
-int cpl_table_and_selected_double(cpl_table *, const char *, 
+cpl_size cpl_table_and_selected_double(cpl_table *, const char *, 
                                      cpl_table_select_operator, double);
-int cpl_table_or_selected_double(cpl_table *, const char *, 
+cpl_size cpl_table_or_selected_double(cpl_table *, const char *, 
                                      cpl_table_select_operator, double);
 #ifdef _Complex_I
-int cpl_table_and_selected_float_complex(cpl_table *, const char *, 
+cpl_size cpl_table_and_selected_float_complex(cpl_table *, const char *, 
                                     cpl_table_select_operator, float complex);
-int cpl_table_or_selected_float_complex(cpl_table *, const char *, 
+cpl_size cpl_table_or_selected_float_complex(cpl_table *, const char *, 
                                     cpl_table_select_operator, float complex);
-int cpl_table_and_selected_double_complex(cpl_table *, const char *, 
+cpl_size cpl_table_and_selected_double_complex(cpl_table *, const char *, 
                                      cpl_table_select_operator, double complex);
-int cpl_table_or_selected_double_complex(cpl_table *, const char *, 
+cpl_size cpl_table_or_selected_double_complex(cpl_table *, const char *, 
                                      cpl_table_select_operator, double complex);
 #endif
 
-int cpl_table_and_selected_string(cpl_table *, const char *, 
+cpl_size cpl_table_and_selected_string(cpl_table *, const char *, 
                                      cpl_table_select_operator, const char *);
-int cpl_table_or_selected_string(cpl_table *, const char *, 
+cpl_size cpl_table_or_selected_string(cpl_table *, const char *, 
                                      cpl_table_select_operator, const char *);
 
-int cpl_table_and_selected(cpl_table *, const char *, 
+cpl_size cpl_table_and_selected(cpl_table *, const char *, 
                          cpl_table_select_operator, const char *);
-int cpl_table_or_selected(cpl_table *, const char *, 
+cpl_size cpl_table_or_selected(cpl_table *, const char *, 
                         cpl_table_select_operator, const char *);
-int cpl_table_is_selected(const cpl_table *, int);
+int cpl_table_is_selected(const cpl_table *, cpl_size);
 
-int cpl_table_count_selected(const cpl_table *);
+cpl_size cpl_table_count_selected(const cpl_table *);
 
 void cpl_table_dump_structure(const cpl_table *, FILE *);
-void cpl_table_dump(const cpl_table *, int, int, FILE *);
+void cpl_table_dump(const cpl_table *, cpl_size, cpl_size, FILE *);
 
 cpl_table *cpl_table_load(const char *, int, int);
 cpl_table *cpl_table_load_window(const char *, int, int, const cpl_array *,
-                                 int, int);
+                                 cpl_size, cpl_size);
 cpl_error_code cpl_table_save(const cpl_table *, const cpl_propertylist *, 
                               const cpl_propertylist *, const char *filename, 
                               unsigned mode);
