@@ -1,4 +1,4 @@
-/* $Id: cpl_test.h,v 1.58 2011/02/10 12:54:53 llundin Exp $
+/* $Id: cpl_test.h,v 1.60 2011/07/20 13:40:49 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2011/02/10 12:54:53 $
- * $Revision: 1.58 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2011/07/20 13:40:49 $
+ * $Revision: 1.60 $
+ * $Name: cpl-6_0 $
  */
 
 #ifndef CPL_TEST_H
@@ -129,7 +129,7 @@ CPL_BEGIN_DECLS
         const double cpl_test_time = cpl_test_get_walltime();           \
         cpl_errorstate cpl_test_state = cpl_errorstate_get();           \
         cpl_test_macro(cpl_test_errno, cpl_test_time, cpl_test_flops,   \
-                       cpl_test_state, (zero) ? 1 : 0, CPL_FALSE,       \
+                       cpl_test_state, (zero), CPL_FALSE,               \
                        #zero, cpl_func, __FILE__, __LINE__);            \
     } while (0)
 
@@ -970,9 +970,9 @@ typedef unsigned long long int cpl_flops;
 
 void cpl_test_init_macro(const char *, const char *,
                          cpl_msg_severity) CPL_ATTR_NONNULL;
-int cpl_test_end(int);
-void cpl_test_macro(int, double, cpl_flops, cpl_errorstate, int, cpl_boolean,
-                    const char *, const char *,
+int cpl_test_end(cpl_size);
+void cpl_test_macro(int, double, cpl_flops, cpl_errorstate, cpl_size,
+                    cpl_boolean, const char *, const char *,
                     const char *, unsigned) CPL_ATTR_NONNULL;
 
 void cpl_test_errorstate_macro(int, double, cpl_flops, cpl_errorstate,
@@ -1001,12 +1001,12 @@ void cpl_test_nonnull_macro(int, double, cpl_flops, cpl_errorstate,
     __attribute__((nonnull(6, 7, 8)))
 #endif
     ;
-void cpl_test_eq_macro(int, double, cpl_flops, cpl_errorstate, int,
-                       const char *, int, const char *, const char *,
+void cpl_test_eq_macro(int, double, cpl_flops, cpl_errorstate, cpl_size,
+                       const char *, cpl_size, const char *, const char *,
                        const char *, unsigned) CPL_ATTR_NONNULL;
 
-void cpl_test_noneq_macro(int, double, cpl_flops, cpl_errorstate, int,
-                          const char *, int, const char *, const char *,
+void cpl_test_noneq_macro(int, double, cpl_flops, cpl_errorstate, cpl_size,
+                          const char *, cpl_size, const char *, const char *,
                           const char *, unsigned) CPL_ATTR_NONNULL;
 
 void cpl_test_eq_ptr_macro(int, double, cpl_flops, cpl_errorstate, const void *,
