@@ -1,4 +1,4 @@
-/* $Id: cpl_tools.c,v 1.114 2010/11/11 09:23:18 llundin Exp $
+/* $Id: cpl_tools.c,v 1.124 2012/02/27 16:12:47 rpalsa Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -19,10 +19,10 @@
  */
 
 /*
- * $Author: llundin $
- * $Date: 2010/11/11 09:23:18 $
- * $Revision: 1.114 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Author: rpalsa $
+ * $Date: 2012/02/27 16:12:47 $
+ * $Revision: 1.124 $
+ * $Name: cpl-6_0 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -33,6 +33,16 @@
                                    Includes
  -----------------------------------------------------------------------------*/
 
+#include <cxmemory.h>
+#include <cxmessages.h>
+#include <cxutils.h>
+
+#include "cpl_propertylist_impl.h"
+#include "cpl_memory.h"
+#include "cpl_tools.h"
+
+#include "cpl_error_impl.h"
+
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -45,12 +55,6 @@
 /* Needed by strcmp(), strncmp(), strlen() */
 #include <string.h>
 
-#include <cxmemory.h>
-#include <cxmessages.h>
-#include <cxutils.h>
-
-#include "cpl_propertylist_impl.h"
-#include "cpl_memory.h"
 #include "cpl_tools.h"
 
 /*-----------------------------------------------------------------------------
@@ -67,7 +71,7 @@
 #endif
 
 /* Swap macros */
-#define CPL_INT_SWAP(a,b) { register const int t=(a); (a)=(b); (b)=t; }
+#define CPL_INT_SWAP(a,b) { register const cpl_size t=(a); (a)=(b); (b)=t; }
 
 #define CONCAT(a,b) a ## _ ## b
 #define CONCAT2X(a,b) CONCAT(a,b)
@@ -142,38 +146,6 @@ typedef enum _cpl_fits_keytype_ {
                         Private function prototypes
  -----------------------------------------------------------------------------*/
 
-static int compar_ascn_int(const void *, const void *) CPL_ATTR_NONNULL;
-static int compar_ascn_float(const void *, const void *) CPL_ATTR_NONNULL;
-static int compar_ascn_double(const void *, const void *) CPL_ATTR_NONNULL;
-
-static int compar_desc_int(const void *, const void *) CPL_ATTR_NONNULL;
-static int compar_desc_float(const void *, const void *) CPL_ATTR_NONNULL;
-static int compar_desc_double(const void *, const void *) CPL_ATTR_NONNULL;
-
-static void cpl_tools_get_median_3_double(double *) CPL_ATTR_NONNULL;
-static void cpl_tools_get_median_3_float(float *) CPL_ATTR_NONNULL;
-static void cpl_tools_get_median_3_int(int *) CPL_ATTR_NONNULL;
-
-static void cpl_tools_get_median_5_double(double *) CPL_ATTR_NONNULL;
-static void cpl_tools_get_median_5_float(float *) CPL_ATTR_NONNULL;
-static void cpl_tools_get_median_5_int(int *) CPL_ATTR_NONNULL;
-
-static void cpl_tools_get_median_6_double(double *) CPL_ATTR_NONNULL;
-static void cpl_tools_get_median_6_float(float *) CPL_ATTR_NONNULL;
-static void cpl_tools_get_median_6_int(int *) CPL_ATTR_NONNULL;
-
-static void cpl_tools_get_median_7_double(double *) CPL_ATTR_NONNULL;
-static void cpl_tools_get_median_7_float(float *) CPL_ATTR_NONNULL;
-static void cpl_tools_get_median_7_int(int *) CPL_ATTR_NONNULL;
-
-static void cpl_tools_get_median_9_double(double *) CPL_ATTR_NONNULL;
-static void cpl_tools_get_median_9_float(float *) CPL_ATTR_NONNULL;
-static void cpl_tools_get_median_9_int(int *) CPL_ATTR_NONNULL;
-
-static void cpl_tools_get_median_25_double(double *) CPL_ATTR_NONNULL;
-static void cpl_tools_get_median_25_float(float *) CPL_ATTR_NONNULL;
-static void cpl_tools_get_median_25_int(int *) CPL_ATTR_NONNULL;
-
 static int cpl_fits_property_comparison(const cpl_property *, 
                                         const cpl_property *);
 static cpl_fits_keytype cpl_fits_property_get_type(const char *) CPL_ATTR_PURE;
@@ -212,6 +184,32 @@ static cpl_flops flop_count = 0;
 #undef CPL_TYPE_NAME
 #undef CPL_TOOLS_SORT_LT
 
+#define CPL_TYPE_IS_NUM
+#define CPL_TYPE long
+#define CPL_TYPE_NAME long
+#define CPL_TOOLS_SORT_LT(x,y) ((x) < (y))
+#include "cpl_tools_body.h"
+#undef CPL_TYPE
+#undef CPL_TYPE_NAME
+#undef CPL_TOOLS_SORT_LT
+
+#define CPL_TYPE_IS_NUM
+#define CPL_TYPE long long
+#define CPL_TYPE_NAME long_long
+#define CPL_TOOLS_SORT_LT(x,y) ((x) < (y))
+#include "cpl_tools_body.h"
+#undef CPL_TYPE
+#undef CPL_TYPE_NAME
+#undef CPL_TOOLS_SORT_LT
+
+#define CPL_TYPE cpl_size
+#define CPL_TYPE_NAME cplsize
+#define CPL_TOOLS_SORT_LT(x,y) ((x) < (y))
+#include "cpl_tools_body.h"
+#undef CPL_TYPE
+#undef CPL_TYPE_NAME
+#undef CPL_TOOLS_SORT_LT
+
 #define CPL_TYPE float
 #define CPL_TYPE_NAME float
 #define CPL_TOOLS_SORT_LT(x,y) ((x) < (y))
@@ -240,6 +238,68 @@ static cpl_flops flop_count = 0;
 #undef CPL_TOOLS_SORT_LT
 
 
+/*----------------------------------------------------------------------------*/
+/**
+  @internal
+  @brief    Find the FITS BITPIX value corresponding to the given CPL type.
+  @param    type The CPL type.
+  @return   The corresponding FITS BITPIX value, or zero on error.
+  @note The input type may have its array/pointer bit set, which is ignored.
+
+  The supported types (and their corresponding BITPIX) are:
+  CPL_TYPE_UCHAR:  BYTE_IMG     (8)
+  CPL_TYPE_SHORT:  SHORT_IMG   (16)
+  CPL_TYPE_USHORT: USHORT_IMG  (20)
+  CPL_TYPE_INT:    LONG_IMG    (32)
+  CPL_TYPE_FLOAT:  FLOAT_IMG  (-32)
+  CPL_TYPE_DOUBLE: DOUBLE_IMG (-64)
+
+  Possible #_cpl_error_code_ set in this function:
+  - CPL_ERROR_INVALID_TYPE if no BITPIX value corresponds to the type
+
+ */
+/*----------------------------------------------------------------------------*/
+int cpl_tools_get_bpp(cpl_type type)
+{
+    int bpp;
+
+    /* switch on type without POINTER and ARRAY bits */
+    switch (type & ~CPL_TYPE_POINTER & ~CPL_TYPE_FLAG_ARRAY) {
+
+    case CPL_TYPE_UCHAR:
+        bpp = BYTE_IMG; /* 8 */
+        break;
+
+    case CPL_TYPE_SHORT:
+        bpp = SHORT_IMG; /* 16 */
+        break;
+
+    case CPL_TYPE_USHORT:
+        bpp = USHORT_IMG; /* 20 */
+        break;
+
+    case CPL_TYPE_INT:
+        bpp = LONG_IMG; /* 32 */
+        break;
+
+    case CPL_TYPE_FLOAT:
+        bpp = FLOAT_IMG; /* -32 */
+        break;
+
+    case CPL_TYPE_DOUBLE:
+        bpp = DOUBLE_IMG; /* -64 */
+        break;
+
+    default:
+        bpp = 0;
+        (void)cpl_error_set_message_(CPL_ERROR_INVALID_TYPE,
+                                     "type=0x%x", (int)type);
+    }
+
+    return bpp;
+}
+
+
 /*----------------------------------------------------------------------------*/
 /**
   @internal
@@ -248,10 +308,10 @@ static cpl_flops flop_count = 0;
   @return   The corresponding power of 2 or -1 if p is not a power of 2
  */
 /*----------------------------------------------------------------------------*/
-int cpl_tools_is_power_of_2(int p)
+cpl_size cpl_tools_is_power_of_2(cpl_size p)
 {
-    int ipow = 0;
-    int done;
+    cpl_size ipow = 0;
+    cpl_size done;
 
     if (p <= 0) return -1;
 
@@ -338,7 +398,7 @@ cpl_error_code cpl_fits_add_properties(fitsfile               * self,
             /* Copy all but the black-listed properties */
             out = cpl_propertylist_new();
             if (cpl_propertylist_copy_property_regexp(out, to_add, to_rm, 1)) {
-                error = cpl_error_set_where(cpl_func);
+                error = cpl_error_set_where_();
             }
         } else {
             out = cpl_propertylist_duplicate(to_add);
@@ -349,7 +409,7 @@ cpl_error_code cpl_fits_add_properties(fitsfile               * self,
             if (cpl_propertylist_sort(out, cpl_fits_property_comparison) ||
                 cpl_propertylist_to_fitsfile(self, out, NULL)) {
 
-                error = cpl_error_set_where(cpl_func);
+                error = cpl_error_set_where_();
 
             }
 
@@ -478,36 +538,6 @@ cpl_flops cpl_tools_get_flops(void)
   return flop_count;
 }
 
-/*----------------------------------------------------------------------------*/
-/**
-  @internal
-  @brief    Given p and u, modify the polynomial to p(x) := p(x+u)
-  @param    p  The polynomial coefficients to be modified in place
-  @param    n  The number of coefficients
-  @param    u  The shift
-  @return   void
-  @see      cpl_polynomial_shift_1d
-  @note     The function will cx_assert() on NULL input.
-
- */
-/*----------------------------------------------------------------------------*/
-void cpl_polynomial_shift_double(double * coeffs, int n, double u)
-{
-
-    int i, j;
-
-
-    cx_assert( coeffs != NULL );
-
-    cx_assert( n > 0 );
-
-    for (j = 0; j < n-1; j++)
-        for (i = 1; i < n - j; i++ )
-            coeffs[n-1-i] += coeffs[n-i] * u;
-
-    cpl_tools_add_flops( n * ( n - 1) );
-}
-
 /*----------------------------------------------------------------------------*/
 /**
   @internal
@@ -539,63 +569,88 @@ cpl_vector * cpl_vector_transform_mean(const cpl_vector * x, double * pm)
 /*----------------------------------------------------------------------------*/
 /**
   @internal
-  @brief   Ensure that the vector has the required number of distinct values
-  @param   self      The vector to check
-  @param   ndistinct The (positive) number of distinct values to require
+  @brief   Count the number of distinct values in the vector
+  @param   self       The vector to check, it will be sorted
+  @param   pndistinct The number of distinct values
   @return  CPL_ERROR_NONE iff the check is sucessful
 
  */
 /*----------------------------------------------------------------------------*/
-cpl_error_code cpl_vector_ensure_distinct(const cpl_vector * self,
-                                          int ndistinct)
+cpl_error_code cpl_vector_count_distinct(cpl_vector * self,
+                                         cpl_size * pndistinct)
 {
 
-    cpl_ensure_code(self, CPL_ERROR_NULL_INPUT);
+    if (pndistinct == NULL) {
+        return cpl_error_set_(CPL_ERROR_NULL_INPUT);
+    } else if (cpl_vector_sort(self, CPL_SORT_ASCENDING)) {
+        return cpl_error_set_where_();
+    } else {
 
-    if (ndistinct > 1) {
-        cpl_vector   * tmp;
-        const double * dx;
-        double         xmin;
-        const int      n = cpl_vector_get_size(self);
-        int            i, j;
+        const double * dx = cpl_vector_get_data_const(self);
+        double         xmin = dx[0];
+        const cpl_size n = cpl_vector_get_size(self);
+        cpl_size       i, j;
 
-
-        if (ndistinct > n) {
-            return cpl_error_set_message_macro(cpl_func,
-                                               CPL_ERROR_DATA_NOT_FOUND,
-                                               __FILE__, __LINE__,
-                                               "A %d-element vector cannot have"
-                                               " at least %d distinct values",
-                                               n, ndistinct);
-        }
-
-        assert( n > 1 );
-      
-        tmp = cpl_vector_duplicate(self);
-        (void)cpl_vector_sort(tmp, 1);
-        dx = cpl_vector_get_data(tmp);
-
-        xmin = dx[0];
-        i = j = 1;
-        do {
+        for (j = i = 1; i < n; i++) {
             if (dx[i] > xmin) {
                 xmin = dx[i];
                 j++;
             }
-        } while (j < ndistinct && ++i < n);
-
-        cpl_vector_delete(tmp);
-
-        if (j < ndistinct) {
-            return cpl_error_set_message_macro(cpl_func,
-                                               CPL_ERROR_DATA_NOT_FOUND,
-                                               __FILE__, __LINE__,
-                                               "%d-element vector must have "
-                                               "at least %d (not %d) distinct "
-                                               "values", n, ndistinct, j);
         }
-    } else {
-        cpl_ensure_code(ndistinct > 0, CPL_ERROR_ILLEGAL_INPUT);
+
+        *pndistinct = j;
+    }
+
+    return CPL_ERROR_NONE;
+}
+
+
+/*----------------------------------------------------------------------------*/
+/**
+  @internal
+  @brief   Ensure that the vector has the required number of distinct values
+  @param   self      The vector to check
+  @param   ndistinct The (positive) number of distinct values to require
+  @return  CPL_ERROR_NONE iff the check is sucessful
+  @see cpl_vector_count_distinct()
+
+ */
+/*----------------------------------------------------------------------------*/
+cpl_error_code cpl_vector_ensure_distinct(const cpl_vector * self,
+                                          cpl_size ndistinct)
+{
+
+    if (ndistinct > 1) {
+        const cpl_size n = cpl_vector_get_size(self);
+
+        if (n < 1) {
+            return cpl_error_set_where_();
+        } else if (ndistinct > n) {
+            return cpl_error_set_message_(CPL_ERROR_DATA_NOT_FOUND,
+                                          "A %" CPL_SIZE_FORMAT "-element "
+                                          "vector cannot have at least %"
+                                          CPL_SIZE_FORMAT " distinct values",
+                                          n, ndistinct);
+        } else {
+            cpl_vector * tmp = cpl_vector_duplicate(self);
+            cpl_size     idistinct = 0;
+
+            (void)cpl_vector_count_distinct(tmp, &idistinct);
+
+            cpl_vector_delete(tmp);
+
+            if (idistinct < ndistinct) {
+                return cpl_error_set_message_(CPL_ERROR_DATA_NOT_FOUND, "%"
+                                              CPL_SIZE_FORMAT "-element vector "
+                                              "must have at least %"
+                                              CPL_SIZE_FORMAT " (not %"
+                                              CPL_SIZE_FORMAT ") distinct "
+                                              " values", n, ndistinct,
+                                              idistinct);
+            }
+        }
+    } else if (ndistinct < 1) {
+        return cpl_error_set_(CPL_ERROR_ILLEGAL_INPUT);
     }
 
     return CPL_ERROR_NONE;
@@ -624,8 +679,9 @@ cpl_error_code cpl_vector_ensure_distinct(const cpl_vector * self,
  */
 /*----------------------------------------------------------------------------*/
 cpl_error_code cpl_tools_copy_window(void *self, const void *src, size_t size,
-                                     int nx, int ny, int llx, int lly,
-                                     int urx, int ury)
+                                     cpl_size nx,  cpl_size ny,
+                                     cpl_size llx, cpl_size lly,
+                                     cpl_size urx, cpl_size ury)
 {
 
     /* FIXME: Need to do pointer arithmetic */
@@ -660,7 +716,7 @@ cpl_error_code cpl_tools_copy_window(void *self, const void *src, size_t size,
         /* Byte just after last byte to write to self */
         const char * cstop   = cself + rowsize * (size_t)(ury-lly+1);
 
-        cpl_ensure_code(llx > 0,    CPL_ERROR_ILLEGAL_INPUT);
+        cpl_ensure_code(llx >  0,   CPL_ERROR_ILLEGAL_INPUT);
         cpl_ensure_code(urx <= nx,  CPL_ERROR_ILLEGAL_INPUT);
 
         /* Point to first byte to read */
@@ -697,14 +753,15 @@ cpl_error_code cpl_tools_copy_window(void *self, const void *src, size_t size,
     nx or ny non-positive
  */
 /*----------------------------------------------------------------------------*/
-cpl_error_code cpl_tools_shift_window(void * self, size_t size, int nx, int ny,
-                                      int null, int dx, int dy)
+cpl_error_code cpl_tools_shift_window(void * self, size_t size,
+                                      cpl_size nx, cpl_size ny, int null,
+                                      cpl_size dx, cpl_size dy)
 {
 
     /* Need to do pointer arithmetic */
     char * myself = (char*)self;
     /* Number of elements to be set to null */
-    const size_t sset  = (size_t)(abs(dy) * nx) * size;
+    const size_t sset  = (size_t)(labs((long int)dy) * nx) * size;
     /* Remainder of buffer will be moved */
     const size_t smove = (size_t)(ny * nx) * size - sset;
     char * pdest = dy < 0 ? myself : myself + sset;
@@ -727,7 +784,7 @@ cpl_error_code cpl_tools_shift_window(void * self, size_t size, int nx, int ny,
         /* Size of a row */
         const size_t rsize = (size_t)nx * size;
         /* Number of elements to be set to null */
-        const size_t srset = (size_t)abs(dx) * size;
+        const size_t srset = (size_t)labs((long int)dx) * size;
         /* Remainder of buffer will be moved */
         const size_t srmove = rsize - srset;
 
@@ -736,10 +793,10 @@ cpl_error_code cpl_tools_shift_window(void * self, size_t size, int nx, int ny,
         char * prset  = dx > 0 ? pdest : pdest + srmove;
         /* source and dest overlap when dy is zero */
         void * (*myshift)(void *, const void *, size_t) = dy ? memcpy : memmove;
-        int j;
+        cpl_size j;
 
         if (dy <= 0) {
-            for (j = abs(dy); j < ny; j++,
+            for (j = -dy; j < ny; j++,
                  prdest += rsize, prsrc += rsize, prset += rsize) {
                 (void)myshift(prdest, prsrc, srmove);
                 (void)memset(prset, null, srset);
@@ -751,7 +808,7 @@ cpl_error_code cpl_tools_shift_window(void * self, size_t size, int nx, int ny,
             prsrc  += smove - rsize;
             prset  += smove - rsize;
 
-            for (j = abs(dy); j < ny; j++,
+            for (j = dy; j < ny; j++,
                  prdest -= rsize, prsrc -= rsize, prset -= rsize) {
                 (void)memcpy(prdest, prsrc, srmove);
                 (void)memset(prset, null, srset);
