@@ -1,4 +1,4 @@
-/* $Id: cpl_tools_body.h,v 1.20 2010/12/15 14:49:28 llundin Exp $
+/* $Id: cpl_tools_body.h,v 1.27 2011/08/17 16:48:55 cgarcia Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -19,10 +19,10 @@
  */
 
 /*
- * $Author: llundin $
- * $Date: 2010/12/15 14:49:28 $
- * $Revision: 1.20 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Author: cgarcia $
+ * $Date: 2011/08/17 16:48:55 $
+ * $Revision: 1.27 $
+ * $Name: cpl-6_0 $
  */
 
 #undef ADDTYPE
@@ -30,6 +30,22 @@
 
 #ifdef CPL_TYPE_IS_NUM
 
+static int ADDTYPE(compar_ascn)(const void *, const void *) CPL_ATTR_NONNULL;
+
+static int ADDTYPE(compar_desc)(const void *, const void *) CPL_ATTR_NONNULL;
+
+static void ADDTYPE(cpl_tools_get_median_3)(CPL_TYPE *) CPL_ATTR_NONNULL;
+
+static void ADDTYPE(cpl_tools_get_median_5)(CPL_TYPE *) CPL_ATTR_NONNULL;
+
+static void ADDTYPE(cpl_tools_get_median_6)(CPL_TYPE *) CPL_ATTR_NONNULL;
+
+static void ADDTYPE(cpl_tools_get_median_7)(CPL_TYPE *) CPL_ATTR_NONNULL;
+
+static void ADDTYPE(cpl_tools_get_median_9)(CPL_TYPE *) CPL_ATTR_NONNULL;
+
+static void ADDTYPE(cpl_tools_get_median_25)(CPL_TYPE *) CPL_ATTR_NONNULL;
+
 /* Swap macro */
 #define CPL_TYPE_SWAP(a, b) { register const CPL_TYPE t=(a); (a)=(b); (b)=t; }
 
@@ -49,20 +65,24 @@
   For a finite population or sample, the median is the middle value of an odd
   number of values (arranged in ascending order) or any value between the two
   middle values of an even number of values.
-  The computed median should actually be a value of the input array, so, in
-  case of an even number of values in the input array, the median would be the
-  lowest of the two central values.
+  For an even number of elements in the array, the mean of the two central
+  values is returned. Note that in this case, the median might not be a value
+  of the input array. Also, note that in the case of integer data types, 
+  the result will be converted to an integer.
+  Consider to transform your int array to float if that is not the desired 
+  behavior.  
 
-  The function is optimized for 9 elements.
+  The function is optimized for 3, 5, 6, 9 and 25 elements.
 
   After a successful call, self is permuted so elements less than the median
-  have lower indices, while elements greather than the median have higher
+  have lower indices, while elements greater than the median have higher
   indices.
 
-  Example: the median of (1 2 3) is 2 and the median of (1 2 3 4) is 2.
+  Example: the median of (1 2 3) is 2 and the median of (1. 2. 3. 4.) is 2.5.
+  The median of integers (1 2 3 4) is 2 (since it is implicitly converted).
  */
 /*----------------------------------------------------------------------------*/
-CPL_TYPE ADDTYPE(cpl_tools_get_median)(CPL_TYPE * self, int n)
+CPL_TYPE ADDTYPE(cpl_tools_get_median)(CPL_TYPE * self, cpl_size n)
 {
 
     switch (n) {
@@ -91,10 +111,19 @@ CPL_TYPE ADDTYPE(cpl_tools_get_median)(CPL_TYPE * self, int n)
         break;
 
     default:
-        (void)ADDTYPE(cpl_tools_quickselection)(self, n);
+        if(n & 1)  //Odd number
+            (void)ADDTYPE(cpl_tools_quickselection)(self, n, (n-1)/2);
+        else      //Even number
+        {
+            (void)ADDTYPE(cpl_tools_quickselection)(self, n, n/2-1);
+            (void)ADDTYPE(cpl_tools_quickselection)(self + n/2, n - n/2-1, 0);
+        }
     }
 
-    return self[(n-1)/2];
+    if(n & 1)  //Odd number
+        return self[(n-1)/2];
+    else      //Even number
+        return (self[n/2-1]+self[n/2])/((CPL_TYPE)2);
 }
 
 
@@ -339,6 +368,39 @@ static void ADDTYPE(cpl_tools_get_median_25)(CPL_TYPE * self)
 }
 
 
+/*----------------------------------------------------------------------------*/
+/**
+  @internal
+  @brief    Compute the arithmetic mean of an array
+  @param    a     The array
+  @param    n     The (positive) array size
+  @return   The mean, S(n) = (1/n) sum(a_i) (i=1 -> n), or 0 on NULL input
+
+  Compute the arithmetic mean of a dataset using the recurrence relation
+     mean_(n) = mean(n-1) + (v[n] - mean(n-1))/(n+1)
+     - this has a measurable impact on the output of
+       cpl_polynomial_fit_{1,2}d_create()
+
+ */
+/*----------------------------------------------------------------------------*/
+double ADDTYPE(cpl_tools_get_mean)(const CPL_TYPE * a, cpl_size n)
+{
+    double   mean = 0.0;
+    cpl_size i;
+
+
+    cpl_ensure(a != NULL, CPL_ERROR_NULL_INPUT,    0.0);
+    cpl_ensure(n >  0,    CPL_ERROR_ILLEGAL_INPUT, 0.0);
+
+    for (i = 0; i < n; i++)
+        mean += ((double)a[i] - mean) / (double)(i + 1);
+
+    cpl_tools_add_flops(3 * n);
+
+    return mean;
+}
+
+
 /*----------------------------------------------------------------------------*/
 /**
   @internal
@@ -347,6 +409,7 @@ static void ADDTYPE(cpl_tools_get_median_25)(CPL_TYPE * self)
   @param    n     The (non-negative) array size
   @param    pmean Iff non-NULL, *pmean is the mean (at no extra cost)
   @return   The summed sample variance, S(n) = sum((a_i-mean)^2) (i=1 -> n)
+  @note Even with rounding errors the returned result is always non-negative
 
 Math explanation for ticket DFS05126, written by Lander de Bilbao.
 
@@ -424,11 +487,11 @@ $$\sum_{i=1}^{N+1} (x_i - \overline{x}_{n+1})2 = \sum_{i=1}^N (x_i - \overline{x
  */
 /*----------------------------------------------------------------------------*/
 double ADDTYPE(cpl_tools_get_variancesum)(const CPL_TYPE * a,
-                                          int n, double * pmean)
+                                          cpl_size n, double * pmean)
 {
-    double varsum = 0.0;
-    double mean = 0.0;
-    int i;
+    double   varsum = 0.0;
+    double   mean = 0.0;
+    cpl_size i;
 
     cpl_ensure(a != NULL, CPL_ERROR_NULL_INPUT, 0.0);
     cpl_ensure(n >= 0, CPL_ERROR_ILLEGAL_INPUT, 0.0);
@@ -436,8 +499,8 @@ double ADDTYPE(cpl_tools_get_variancesum)(const CPL_TYPE * a,
     for (i=0; i < n; i++) {
         const double delta = (double)a[i] - mean;
 
-        varsum += i * delta * delta / (i + 1.0);
-        mean   += delta / (i + 1.0);
+        varsum += (double)i * delta * delta / (double)(i + 1);
+        mean   += delta / (double)(i + 1);
     }
 
     cpl_tools_add_flops( 1 + 6 * n ); /* Assume expression reuse */
@@ -459,7 +522,7 @@ double ADDTYPE(cpl_tools_get_variancesum)(const CPL_TYPE * a,
  */
 /*----------------------------------------------------------------------------*/
 double ADDTYPE(cpl_tools_get_variance)(const CPL_TYPE * a,
-                                       int n, double * pmean)
+                                       cpl_size n, double * pmean)
 {
 
     const double varsum
@@ -467,7 +530,7 @@ double ADDTYPE(cpl_tools_get_variance)(const CPL_TYPE * a,
 
     cpl_ensure(n > 0, CPL_ERROR_ILLEGAL_INPUT, 0.0);
 
-    return varsum / n;
+    return varsum / (double)n;
 }
 
 
@@ -539,25 +602,26 @@ static int ADDTYPE(compar_desc)(const void * p1, const void * p2)
  */
 /*----------------------------------------------------------------------------*/
 CPL_TYPE ADDTYPE(cpl_tools_quickselection)(CPL_TYPE * self,
-                                           int        n)
+                                           cpl_size   n,
+                                           cpl_size   k)
 {
 
-    int       low  = 0;
-    int       high = n-1;
-    const int median = (low + high) / 2;
+    cpl_size       low  = 0;
+    cpl_size       high = n-1;
 
 #ifdef CPL_TOOLS_STRICT_ERROR_CHECKING
     cpl_ensure(self != NULL, CPL_ERROR_NULL_INPUT,    (CPL_TYPE)0);
     cpl_ensure(n >  0,       CPL_ERROR_ILLEGAL_INPUT, (CPL_TYPE)0);
+    cpl_ensure(n >  k -1,    CPL_ERROR_ILLEGAL_INPUT, (CPL_TYPE)0);
 #endif
 
     /* Control flow has been changed to a single return at the end */
 
     for (;low + 1 < high;) {
         /* Find median of low, middle and high items; swap into position low */
-        const int middle = (low + high) / 2;
-        int ll = low + 1;
-        int hh = high;
+        const cpl_size middle = (low + high) / 2;
+        cpl_size       ll = low + 1;
+        cpl_size       hh = high;
 
         CPL_TYPE_SORT(self[middle], self[high]);
         CPL_TYPE_SORT(self[low],    self[high]);
@@ -581,9 +645,9 @@ CPL_TYPE ADDTYPE(cpl_tools_quickselection)(CPL_TYPE * self,
         CPL_TYPE_SWAP(self[low], self[hh]);
 
         /* Re-set active partition */
-        if (hh <= median)
+        if (hh <= k)
             low = ll;
-        if (hh >= median)
+        if (hh >= k)
             high = hh - 1;
     }
 
@@ -591,7 +655,7 @@ CPL_TYPE ADDTYPE(cpl_tools_quickselection)(CPL_TYPE * self,
         CPL_TYPE_SORT(self[low], self[high]);
     }
 
-    return self[median];
+    return self[k];
 }
 
 /*----------------------------------------------------------------------------*/
@@ -621,13 +685,13 @@ CPL_TYPE ADDTYPE(cpl_tools_quickselection)(CPL_TYPE * self,
  */
 /*----------------------------------------------------------------------------*/
 CPL_TYPE ADDTYPE(cpl_tools_get_kth)(CPL_TYPE * self,
-                                    int        n,
-                                    int        k)
+                                    cpl_size   n,
+                                    cpl_size   k)
 {
-    register int l = 0;
-    register int m = n - 1;
-    register int i = l;
-    register int j = m;
+    register cpl_size l = 0;
+    register cpl_size m = n - 1;
+    register cpl_size i = l;
+    register cpl_size j = m;
 
 #ifdef CPL_TOOLS_STRICT_ERROR_CHECKING
     cpl_ensure(self != NULL, CPL_ERROR_NULL_INPUT,          (CPL_TYPE)0);
@@ -725,12 +789,12 @@ void ADDTYPE(cpl_tools_sort_desc)(CPL_TYPE * self, int n)
  */
 /*----------------------------------------------------------------------------*/
 cpl_error_code
-ADDTYPE(cpl_tools_sort)(CPL_TYPE * self, int n)
+ADDTYPE(cpl_tools_sort)(CPL_TYPE * self, cpl_size n)
 {
-    int         i, ir, j, k, l;
-    int         i_stack[CPL_PIX_STACK_SIZE];
-    int         j_stack;
-    CPL_TYPE      a;
+    cpl_size    i, ir, j, k, l;
+    cpl_size    i_stack[CPL_PIX_STACK_SIZE];
+    cpl_size    j_stack;
+    CPL_TYPE    a;
 
     ir = n;
     l = 1;
@@ -819,12 +883,12 @@ ADDTYPE(cpl_tools_sort)(CPL_TYPE * self, int n)
 /*----------------------------------------------------------------------------*/
 cpl_error_code
 ADDTYPE(cpl_tools_sort_stable_pattern)(CPL_TYPE const * self,
-                                       int n,
+                                       cpl_size n,
                                        int reverse,
                                        int stable,
-                                       int *sort_pattern)
+                                       cpl_size *sort_pattern)
 {
-    int i;  
+    cpl_size i;  
     
     /* Check entries */
     cpl_ensure_code(self, CPL_ERROR_NULL_INPUT);
@@ -841,12 +905,12 @@ ADDTYPE(cpl_tools_sort_stable_pattern)(CPL_TYPE const * self,
      * Heap sort
      */
     for (i = n / 2 - 1; i >= 0; i--) {
-        int done = 0;
-        int root = i;
-        int bottom = n - 1;
+        int      done = 0;
+        cpl_size root = i;
+        cpl_size bottom = n - 1;
         
         while ((root*2 + 1 <= bottom) && (!done)) {
-            int child = root*2 + 1;
+            cpl_size child = root*2 + 1;
 
             if (child+1 <= bottom) {
                 if ((!reverse && CPL_TOOLS_SORT_LT(
@@ -879,13 +943,13 @@ ADDTYPE(cpl_tools_sort_stable_pattern)(CPL_TYPE const * self,
     }
     
     for (i = n - 1; i >= 1; i--) {
-        int done = 0;
-        int root = 0;
-        int bottom = i - 1;
+        int      done = 0;
+        cpl_size root = 0;
+        cpl_size bottom = i - 1;
         CPL_INT_SWAP(sort_pattern[0], sort_pattern[i]);
         
         while ((root*2 + 1 <= bottom) && (!done)) {
-            int child = root*2 + 1;
+            cpl_size child = root*2 + 1;
 
             if (child+1 <= bottom) {
                 if ((!reverse && CPL_TOOLS_SORT_LT(
@@ -921,7 +985,7 @@ ADDTYPE(cpl_tools_sort_stable_pattern)(CPL_TYPE const * self,
      */
     if (stable) {
         for (i = 0; i < n; i++) {
-            int j;
+            cpl_size j;
             j = i + 1;
             while(j < n &&
                   !CPL_TOOLS_SORT_LT(self[sort_pattern[i]],
@@ -931,7 +995,7 @@ ADDTYPE(cpl_tools_sort_stable_pattern)(CPL_TYPE const * self,
                 j++;
             }
             if (j - i > 1) {
-                cpl_tools_sort_int(sort_pattern + i, j - i);
+                cpl_tools_sort_cplsize(sort_pattern + i, j - i);
             }
             i = j - 1; 
         }
@@ -970,12 +1034,12 @@ ADDTYPE(cpl_tools_sort_stable_pattern)(CPL_TYPE const * self,
  */
 /*----------------------------------------------------------------------------*/
 cpl_error_code
-ADDTYPE(cpl_tools_permute)(int * self, int n,
+ADDTYPE(cpl_tools_permute)(cpl_size * self, cpl_size n,
                            CPL_TYPE * awrite, CPL_TYPE const * aread,
                            CPL_TYPE * bwrite, CPL_TYPE const * bread)
 {
 
-    int ido = 0; /* First element in cycle to process */
+    cpl_size ido = 0; /* First element in cycle to process */
     const cpl_boolean dob = bwrite != NULL;
 
     cpl_ensure_code(self   != NULL, CPL_ERROR_NULL_INPUT);
@@ -993,13 +1057,13 @@ ADDTYPE(cpl_tools_permute)(int * self, int n,
         /* Save last pair in cycle */
         CPL_TYPE const aread0 = aread[ido];
         CPL_TYPE const bread0 = dob ? bread[ido] : 0;
-        int ifrom = ido;
+        cpl_size ifrom = ido;
 
         while (self[ifrom] != ido) { /* Test here to avoid uni-cycles */
             /* Copy the pair first to support cross-wrapped bivectors */
             CPL_TYPE const areadj = aread[self[ifrom]];
             CPL_TYPE const breadj = dob ? bread[self[ifrom]] : 0;
-            const int j = ifrom;
+            const cpl_size j = ifrom;
 
             ifrom = self[ifrom]; /* Point to next in cycle */
 
