@@ -1,4 +1,4 @@
-/* $Id: cpl_vector_fit_impl.h,v 1.7 2011/02/09 10:01:59 llundin Exp $
+/* $Id: cpl_vector_fit_impl.h,v 1.12 2011/07/29 12:05:21 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2011/02/09 10:01:59 $
- * $Revision: 1.7 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2011/07/29 12:05:21 $
+ * $Revision: 1.12 $
+ * $Name: cpl-6_0 $
  */
 
 #ifndef CPL_VECTOR_FIT_IMPL_H
@@ -48,7 +48,7 @@
 #include "cpl_vector.h"
 #include "cpl_matrix.h"
 #include "cpl_memory.h"
-#include "cpl_error.h"
+#include "cpl_error_impl.h"
 #include "cpl_errorstate.h"
 
 #include <assert.h>
@@ -90,7 +90,7 @@
 /*----------------------------------------------------------------------------*/
 inline static int
 get_candidate(const double *a, const int ia[],
-          int M, int N, int D,
+          cpl_size M, cpl_size N, cpl_size D,
           double lambda,
           int    (*f)(const double x[], const double a[], double *result),
           int (*dfda)(const double x[], const double a[], double result[]),
@@ -102,13 +102,14 @@ get_candidate(const double *a, const int ia[],
           cpl_matrix *beta,
           double *a_da)
 {
-    int Mfit = 0;         /* Number of non-constant fit parameters */
+    cpl_size Mfit = 0;    /* Number of non-constant fit parameters */
     cpl_matrix *da;       /* Solution of   alpha * da = beta */
     double *alpha_data;
     double *beta_data;
     double *da_data;
-    int i, imfit = 0;
-    int j, jmfit = 0;
+    cpl_size i, j;
+    int imfit = 0;
+    int jmfit = 0;
     int k = 0;
 
     /* For efficiency, don't check input in this static function */
@@ -265,19 +266,18 @@ get_candidate(const double *a, const int ia[],
 /*----------------------------------------------------------------------------*/
 
 inline static double
-get_chisq(int N, int D,
+get_chisq(cpl_size N, cpl_size D,
       int (*f)(const double x[], const double a[], double *result),
       const double *a,
       const double *x,
       const double *y,
       const double *sigma)
 {
-    double chi_sq;     /* Result */
-    int i = 0;
+    double chi_sq = 0.0;     /* Result */
+    cpl_size i;
 
     /* For efficiency, don't check input in this static function */
 
-    chi_sq = 0.0;
     for (i = 0; i < N; i++)
     {
         double fx_i;
@@ -329,8 +329,8 @@ get_chisq(int N, int D,
              constant (zero). At least one element must be non-zero.
              Alternatively, pass NULL to fit all parameters.
    @param   f        Function that evaluates the fit function
-                     at the position specified by the first argument (an array of
-             size D) using the fit parameters specified by the second
+                     at the position specified by the first argument (an array
+             of size D) using the fit parameters specified by the second
              argument (an array of size M). The result must be output
              using the third parameter, and the function must return zero
              iff the evaluation succeded.
@@ -352,9 +352,9 @@ get_chisq(int N, int D,
                      @em tolerance_count times in a row. Recommended default:
                      CPL_FIT_LVMQ_COUNT
    @param max_iterations
-                     If this number of iterations is reached without convergence,
-                     the algorithm diverges, by definition. Recommended default:
-                     CPL_FIT_LVMQ_MAXITER
+                     If this number of iterations is reached without
+                     convergence, the algorithm diverges, by definition.
+                     Recommended default: CPL_FIT_LVMQ_MAXITER
    @param mse        If non-NULL, the mean squared error of the best fit is
                      computed.
    @param red_chisq  If non-NULL, the reduced chi square of the best fit is
@@ -404,29 +404,29 @@ cpl_fit_lvmq_(const cpl_matrix *x, const cpl_matrix *sigma_x,
          double *red_chisq,
          cpl_matrix **covariance)
 {
-    const double *x_data     = NULL; /* Pointer to input data                  */
-    const double *y_data     = NULL; /* Pointer to input data                  */
-    const double *sigma_data = NULL; /* Pointer to input data                  */
-    int N    = 0;                    /* Number of data points                  */
-    int D    = 0;                    /* Dimension of x-points                  */
-    int M    = 0;                    /* Number of fit parameters               */
-    int Mfit = 0;                    /* Number of non-constant fit
-                        parameters                             */
+    const double *x_data     = NULL; /* Pointer to input data                 */
+    const double *y_data     = NULL; /* Pointer to input data                 */
+    const double *sigma_data = NULL; /* Pointer to input data                 */
+    cpl_size N    = 0;               /* Number of data points                 */
+    cpl_size D    = 0;               /* Dimension of x-points                 */
+    cpl_size M = 0;                  /* Number of fit parameters              */
+    cpl_size Mfit = 0;               /* Number of non-constant fit
+                                        parameters                            */
 
-    double lambda    = 0.0;          /* Lambda in L-M algorithm                */
+    double lambda    = 0.0;          /* Lambda in L-M algorithm               */
     double MAXLAMBDA = 10e40;        /* Parameter to control the graceful exit
                     if steepest descent unexpectedly fails */
-    double chi_sq    = 0.0;          /* Current  chi^2                         */
+    double chi_sq    = 0.0;          /* Current  chi^2                        */
     int count        = 0;            /* Number of successive small improvements
                     in chi^2 */
     int iterations   = 0;
 
-    cpl_matrix *alpha  = NULL;       /* The MxM ~curvature matrix used in L-M  */
-    cpl_matrix *beta   = NULL;       /* Mx1 matrix = -.5 grad(chi^2)           */
-    double *a_data     = NULL;       /* Parameters, a                          */
-    double *a_da       = NULL;       /* Candidate position a+da                */
-    double *part       = NULL;       /* The partial derivatives df/da          */
-    int *ia_local      = NULL;       /* non-NULL version of ia                 */
+    cpl_matrix *alpha  = NULL;       /* The MxM ~curvature matrix used in L-M */
+    cpl_matrix *beta   = NULL;       /* Mx1 matrix = -.5 grad(chi^2)          */
+    double *a_data     = NULL;       /* Parameters, a                         */
+    double *a_da       = NULL;       /* Candidate position a+da               */
+    double *part       = NULL;       /* The partial derivatives df/da         */
+    int *ia_local      = NULL;       /* non-NULL version of ia                */
 
     /* If covariance computation is requested, then either
      * return the covariance matrix or return NULL.
@@ -446,8 +446,9 @@ cpl_fit_lvmq_(const cpl_matrix *x, const cpl_matrix *sigma_x,
     cpl_ensure_code(max_iterations     > 0, CPL_ERROR_ILLEGAL_INPUT);
 
     /* Chi^2 and covariance computations require sigmas to be known */
-    cpl_ensure_code( sigma_y != NULL || (red_chisq == NULL && covariance == NULL),
-             CPL_ERROR_INCOMPATIBLE_INPUT);
+    cpl_ensure_code( sigma_y != NULL ||
+                     (red_chisq == NULL && covariance == NULL),
+                     CPL_ERROR_INCOMPATIBLE_INPUT);
 
     D = cpl_matrix_get_ncol(x);
     N = cpl_matrix_get_nrow(x);
@@ -471,13 +472,13 @@ cpl_fit_lvmq_(const cpl_matrix *x, const cpl_matrix *sigma_x,
         sigma_data = cpl_vector_get_data_const(sigma_y);
     }
 
-    ia_local = cpl_malloc(M * sizeof(int));
+    ia_local = cpl_malloc((size_t)M * sizeof(int));
     cpl_ensure_code(ia_local != NULL, CPL_ERROR_ILLEGAL_OUTPUT);
 
     /* Count non-constant fit parameters, copy ia */
     if (ia != NULL)
     {
-        int i;
+        cpl_size i;
 
         Mfit = 0;
         for (i = 0; i < M; i++)
@@ -493,14 +494,13 @@ cpl_fit_lvmq_(const cpl_matrix *x, const cpl_matrix *sigma_x,
         if (! (Mfit > 0))
         {
             cpl_free(ia_local);
-            cpl_ensure_code( CPL_FALSE,
-                     CPL_ERROR_ILLEGAL_INPUT);
+            return cpl_error_set_(CPL_ERROR_ILLEGAL_INPUT);
         }
     }
     else
     {
         /* All parameters participate */
-        int i;
+        cpl_size i;
 
         Mfit = M;
 
@@ -514,9 +514,7 @@ cpl_fit_lvmq_(const cpl_matrix *x, const cpl_matrix *sigma_x,
     if (! ( red_chisq == NULL || N > Mfit ) )
     {
         cpl_free(ia_local);
-        cpl_ensure_code(
-        CPL_FALSE,
-        CPL_ERROR_ILLEGAL_INPUT);
+        return cpl_error_set_(CPL_ERROR_ILLEGAL_INPUT);
     }
 
     /* Create alpha, beta, a_da, part  work space */
@@ -524,9 +522,7 @@ cpl_fit_lvmq_(const cpl_matrix *x, const cpl_matrix *sigma_x,
     if (alpha == NULL)
     {
         cpl_free(ia_local);
-        cpl_ensure_code(
-        CPL_FALSE,
-        CPL_ERROR_ILLEGAL_OUTPUT);
+        return cpl_error_set_(CPL_ERROR_ILLEGAL_OUTPUT);
     }
 
     beta = cpl_matrix_new(Mfit, 1);
@@ -534,32 +530,26 @@ cpl_fit_lvmq_(const cpl_matrix *x, const cpl_matrix *sigma_x,
     {
         cpl_free(ia_local);
         cpl_matrix_delete(alpha);
-        cpl_ensure_code(
-        CPL_FALSE,
-        CPL_ERROR_ILLEGAL_OUTPUT);
+        return cpl_error_set_(CPL_ERROR_ILLEGAL_OUTPUT);
     }
 
-    a_da = cpl_malloc(M * sizeof(double));
+    a_da = cpl_malloc((size_t)M * sizeof(double));
     if (a_da == NULL)
     {
         cpl_free(ia_local);
         cpl_matrix_delete(alpha);
         cpl_matrix_delete(beta);
-        cpl_ensure_code(
-        CPL_FALSE,
-        CPL_ERROR_ILLEGAL_OUTPUT);
+        return cpl_error_set_(CPL_ERROR_ILLEGAL_OUTPUT);
     }
 
-    part = cpl_malloc(M * sizeof(double));
+    part = cpl_malloc((size_t)M * sizeof(double));
     if (part == NULL)
     {
         cpl_free(ia_local);
         cpl_matrix_delete(alpha);
         cpl_matrix_delete(beta);
         cpl_free(a_da);
-        cpl_ensure_code(
-        CPL_FALSE,
-        CPL_ERROR_ILLEGAL_OUTPUT);
+        return cpl_error_set_(CPL_ERROR_ILLEGAL_OUTPUT);
     }
 
     /* Initialize loop variables */
@@ -573,9 +563,7 @@ cpl_fit_lvmq_(const cpl_matrix *x, const cpl_matrix *sigma_x,
         cpl_matrix_delete(beta);
         cpl_free(a_da);
         cpl_free(part);
-        cpl_ensure_code(
-        CPL_FALSE,
-        cpl_error_get_code());
+        return cpl_error_set_where_();
     }
 
     /* Iterate until chi^2 didn't improve significantly many
@@ -623,9 +611,7 @@ cpl_fit_lvmq_(const cpl_matrix *x, const cpl_matrix *sigma_x,
                 cpl_matrix_delete(beta);
                 cpl_free(a_da);
                 cpl_free(part);
-                cpl_ensure_code(
-                    CPL_FALSE,
-                    CPL_ERROR_CONTINUE);
+                return cpl_error_set_(CPL_ERROR_CONTINUE);
                 }
             }
         else if (!cpl_errorstate_is_equal(prevstate))
@@ -637,9 +623,7 @@ cpl_fit_lvmq_(const cpl_matrix *x, const cpl_matrix *sigma_x,
                 cpl_matrix_delete(beta);
                 cpl_free(a_da);
                 cpl_free(part);
-                cpl_ensure_code(
-                CPL_FALSE,
-                cpl_error_get_code());
+                return cpl_error_set_where_();
             }
             }
 
@@ -654,9 +638,7 @@ cpl_fit_lvmq_(const cpl_matrix *x, const cpl_matrix *sigma_x,
             cpl_matrix_delete(beta);
             cpl_free(a_da);
             cpl_free(part);
-            cpl_ensure_code(
-            CPL_FALSE,
-            cpl_error_get_code());
+            return cpl_error_set_where_();
         }
 
         if (chi_sq_candidate > chi_sq)
@@ -685,7 +667,7 @@ cpl_fit_lvmq_(const cpl_matrix *x, const cpl_matrix *sigma_x,
 
             /* chi^2 improved, update a and chi^2 */
             {
-            int i;
+            cpl_size i;
             for (i = 0; i < M; i++) a_data[i] = a_da[i];
             }
             chi_sq = chi_sq_candidate;
@@ -701,15 +683,13 @@ cpl_fit_lvmq_(const cpl_matrix *x, const cpl_matrix *sigma_x,
         cpl_matrix_delete(beta);
         cpl_free(a_da);
         cpl_free(part);
-        cpl_ensure_code(
-        CPL_FALSE,
-        CPL_ERROR_CONTINUE);
+        return cpl_error_set_(CPL_ERROR_CONTINUE);
     }
 
     /* Compute mse if requested */
     if (mse != NULL)
     {
-        int i;
+        cpl_size i;
 
         *mse = 0.0;
 
@@ -728,22 +708,20 @@ cpl_fit_lvmq_(const cpl_matrix *x, const cpl_matrix *sigma_x,
                 cpl_matrix_delete(beta);
                 cpl_free(a_da);
                 cpl_free(part);
-                cpl_ensure_code(
-                CPL_FALSE,
-                CPL_ERROR_ILLEGAL_INPUT);
+                return cpl_error_set_(CPL_ERROR_ILLEGAL_INPUT);
             }
 
             residual = y_data[i] - fx_i;
             *mse += residual * residual;
         }
-        *mse /= N;
+        *mse /= (double)N;
     }
 
     /* Compute reduced chi^2 if requested */
     if (red_chisq != NULL)
     {
         /* We already know the optimal chi^2 (and that N > Mfit)*/
-        *red_chisq = chi_sq / (N-Mfit);
+        *red_chisq = chi_sq / (double)(N-Mfit);
     }
 
     /* Compute covariance matrix if requested
@@ -764,9 +742,7 @@ cpl_fit_lvmq_(const cpl_matrix *x, const cpl_matrix *sigma_x,
             cpl_matrix_delete(beta);
             cpl_free(a_da);
             cpl_free(part);
-            cpl_ensure_code(
-            CPL_FALSE,
-            cpl_error_get_code());
+            return cpl_error_set_where_();
         }
 
         cov = cpl_matrix_invert_create(alpha);
@@ -777,14 +753,12 @@ cpl_fit_lvmq_(const cpl_matrix *x, const cpl_matrix *sigma_x,
             cpl_matrix_delete(beta);
             cpl_free(a_da);
             cpl_free(part);
-            cpl_ensure_code(
-            CPL_FALSE,
-            cpl_error_get_code());
+            return cpl_error_set_where_();
         }
 
         /* Make sure that variances are positive */
         {
-        int i;
+        cpl_size i;
         for (i = 0; i < Mfit; i++)
             {
             if ( !(cpl_matrix_get(cov, i, i) > 0) )
@@ -796,9 +770,7 @@ cpl_fit_lvmq_(const cpl_matrix *x, const cpl_matrix *sigma_x,
                 cpl_free(part);
                 cpl_matrix_delete(cov);
                 *covariance = NULL;
-                cpl_ensure_code(
-                    CPL_FALSE,
-                    CPL_ERROR_SINGULAR_MATRIX);
+                return cpl_error_set_(CPL_ERROR_SINGULAR_MATRIX);
                 }
             }
         }
@@ -816,18 +788,16 @@ cpl_fit_lvmq_(const cpl_matrix *x, const cpl_matrix *sigma_x,
             cpl_free(a_da);
             cpl_free(part);
             cpl_matrix_delete(cov);
-            cpl_ensure_code(
-            CPL_FALSE,
-            CPL_ERROR_ILLEGAL_OUTPUT);
+            return cpl_error_set_(CPL_ERROR_ILLEGAL_OUTPUT);
         }
 
         {
-        int j, jmfit;
+            cpl_size j, jmfit;
 
         for (j = 0, jmfit = 0; j < M; j++)
             if (ia_local[j] != 0)
             {
-                int i, imfit;
+                cpl_size i, imfit;
 
                 for (i = 0, imfit = 0; i < M; i++)
                 if (ia_local[i] != 0)
