@@ -1,4 +1,4 @@
-/* $Id: cpl_xmemory.c,v 1.37 2011/01/03 14:16:17 llundin Exp $
+/* $Id: cpl_xmemory.c,v 1.41 2011/08/18 07:58:32 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2011/01/03 14:16:17 $
- * $Revision: 1.37 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2011/08/18 07:58:32 $
+ * $Revision: 1.41 $
+ * $Name: cpl-6_0 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -46,6 +46,25 @@
 
 #include "cpl_xmemory.h"
 
+/* The below doxygen has been inactivated by removing the '**' comment. */
+
+/*----------------------------------------------------------------------------*/
+/*
+ * @defgroup    cpl_xmemory     POSIX-compatible extended memory handling
+ *
+ * cpl_xmemory is a small and efficient module offering memory extension
+ * capabitilies to ANSI C programs running on POSIX-compliant systems. It
+ * offers several useful features such as memory leak detection, protection for
+ * free on NULL or unallocated pointers.
+ * This module has been tested on a number of
+ * current Unix * flavours and is reported to work fine.
+ * The current limitation is the limited number of pointers it can handle at
+ * the same time - and its absence of thread-safety.
+ * 
+ */
+/*----------------------------------------------------------------------------*/
+/**@{*/
+
 /*-----------------------------------------------------------------------------
                                 Defines
  -----------------------------------------------------------------------------*/
@@ -86,10 +105,6 @@
 #define CPL_XMEMORY_RESIZE_FACTOR 2
 #endif
 
-/*-----------------------------------------------------------------------------
-                                Macros
- -----------------------------------------------------------------------------*/
-
 /* A very simple hash */
 #define PTR_HASH(ptr) (((size_t)ptr) % cpl_xmemory_table_size)
 
@@ -98,20 +113,6 @@
 #define CPL_XMEMORY_TYPE_CALLOC  2
 #define CPL_XMEMORY_TYPE_REALLOC 3
 
-/*-----------------------------------------------------------------------------
-                        Private data types
- -----------------------------------------------------------------------------*/
-
-enum _cpl_xmemory_count_type_
-{
-    count_down,
-    count_up
-
-};
-
-typedef enum _cpl_xmemory_count_type_ count_type;
-
-
 /*-----------------------------------------------------------------------------
                         Private variables
  -----------------------------------------------------------------------------*/
@@ -155,7 +156,6 @@ static size_t        * cpl_xmemory_p_size;
  -----------------------------------------------------------------------------*/
 
 static void cpl_xmemory_init(void);
-static void cpl_xmemory_count(count_type);
 static void cpl_xmemory_init_alloc(void);
 static void cpl_xmemory_end(void);
 static void cpl_xmemory_resize(void);
@@ -164,29 +164,13 @@ static void cpl_xmemory_remcell(size_t);
 static size_t cpl_xmemory_findcell(const void *) CPL_ATTR_PURE;
 static size_t cpl_xmemory_findfree(const void *);
 
-/*----------------------------------------------------------------------------*/
-/**
- * @defgroup    cpl_xmemory     POSIX-compatible extended memory handling
- *
- * cpl_xmemory is a small and efficient module offering memory extension
- * capabitilies to ANSI C programs running on POSIX-compliant systems. It
- * offers several useful features such as memory leak detection, protection for
- * free on NULL or unallocated pointers.
- * This module has been tested on a number of
- * current Unix * flavours and is reported to work fine.
- * The current limitation is the limited number of pointers it can handle at
- * the same time - and its absence of thread-safety.
- * 
- */
-/*----------------------------------------------------------------------------*/
-/**@{*/
-
 /*-----------------------------------------------------------------------------
                             Function codes
  -----------------------------------------------------------------------------*/
 
 /*----------------------------------------------------------------------------*/
 /**
+  @internal
   @brief    malloc() with failure check
   @param    size        Size (in bytes) to allocate.
   @return   1 newly allocated pointer (possibly NULL on zero bytes)
@@ -203,7 +187,15 @@ void * cpl_xmemory_malloc_count(size_t size)
 
     if (ptr != NULL) {
 
-        cpl_xmemory_count(count_up);
+#       ifdef _OPENMP
+#       pragma omp atomic
+#       endif
+        cpl_xmemory_ncells++;
+        /* Remember peak allocation */
+        /* FIXME: This non-thread-safe check is incorrect, but might still
+           produce a useful estimate */
+        if (cpl_xmemory_ncells > cpl_xmemory_max_cells)
+            cpl_xmemory_max_cells = cpl_xmemory_ncells;
 
     } else if (size != 0) {
 
@@ -223,6 +215,7 @@ void * cpl_xmemory_malloc_count(size_t size)
 
 /*----------------------------------------------------------------------------*/
 /**
+  @internal
   @brief    calloc() with failure check
   @param    nmemb       Number of elements to allocate.
   @param    size        Size (in bytes) to allocate.
@@ -240,7 +233,15 @@ void * cpl_xmemory_calloc_count(size_t nmemb, size_t size)
 
     if (ptr != NULL) {
 
-        cpl_xmemory_count(count_up);
+#       ifdef _OPENMP
+#       pragma omp atomic
+#       endif
+        cpl_xmemory_ncells++;
+        /* Remember peak allocation */
+        /* FIXME: This non-thread-safe check is incorrect, but might still
+           produce a useful estimate */
+        if (cpl_xmemory_ncells > cpl_xmemory_max_cells)
+            cpl_xmemory_max_cells = cpl_xmemory_ncells;
 
     } else if (size != 0 && nmemb != 0) {
 
@@ -261,6 +262,7 @@ void * cpl_xmemory_calloc_count(size_t nmemb, size_t size)
 
 /*----------------------------------------------------------------------------*/
 /**
+  @internal
   @brief    realloc() with failure check
   @param    oldptr      Pointer to reallocate.
   @param    size        Size (in bytes) to reallocate.
@@ -303,7 +305,10 @@ void * cpl_xmemory_realloc_count(void * oldptr, size_t size)
             }
 
             /* This is just another way to do a free() */
-            cpl_xmemory_count(count_down);
+#           ifdef _OPENMP
+#           pragma omp atomic
+#           endif
+            cpl_xmemory_ncells--;
         }
     }
 
@@ -312,6 +317,7 @@ void * cpl_xmemory_realloc_count(void * oldptr, size_t size)
 
 /*----------------------------------------------------------------------------*/
 /**
+  @internal
   @brief    Free memory.
   @param    ptr Pointer to free.
   @return   void
@@ -329,7 +335,10 @@ void cpl_xmemory_free_count(void * ptr)
         } else {
             free(ptr);
 
-            cpl_xmemory_count(count_down);
+#           ifdef _OPENMP
+#           pragma omp atomic
+#           endif
+            cpl_xmemory_ncells--;
 
         }
     }
@@ -339,6 +348,7 @@ void cpl_xmemory_free_count(void * ptr)
 
 /*----------------------------------------------------------------------------*/
 /**
+  @internal
   @brief    Allocate memory.
   @param    size        Size (in bytes) to allocate.
   @return   1 newly allocated pointer.
@@ -379,6 +389,7 @@ void * cpl_xmemory_malloc(size_t size)
 
 /*----------------------------------------------------------------------------*/
 /**
+  @internal
   @brief    Allocate memory.
   @param    nmemb       Number of elements to allocate.
   @param    size        Size (in bytes) of each element.
@@ -421,6 +432,7 @@ void * cpl_xmemory_calloc(size_t nmemb, size_t size)
 
 /*----------------------------------------------------------------------------*/
 /**
+  @internal
   @brief    Re-Allocate memory.
   @param    oldptr      Pointer to free.
   @param    size        Size (in bytes) to allocate.
@@ -500,6 +512,7 @@ void * cpl_xmemory_realloc(void * oldptr, size_t size)
 
 /*----------------------------------------------------------------------------*/
 /**
+  @internal
   @brief    Free memory.
   @param    ptr         Pointer to free.
   @return   void
@@ -541,7 +554,10 @@ void cpl_xmemory_free(void * ptr)
     free(ptr);
 
     /* Decrement number of allocated pointers */
-    cpl_xmemory_count(count_down);
+#   ifdef _OPENMP
+#   pragma omp atomic
+#   endif
+    cpl_xmemory_ncells--;
 
     if (cpl_xmemory_ncells > 0) {
         /* Remove cell from main table */
@@ -556,6 +572,7 @@ void cpl_xmemory_free(void * ptr)
 
 /*----------------------------------------------------------------------------*/
 /**
+  @internal
   @brief    Display memory status information.
   @param mode  Mode (0: system, 1: Xmemory-failure check, 2: Xmemory-full)
   @return   void
@@ -588,8 +605,12 @@ void cpl_xmemory_status(int mode)
                         (unsigned long)cpl_xmemory_table_size_max,
                         (unsigned long)(cpl_xmemory_table_size_max*rowsize));
             } else {
-                fprintf(stderr, "Maximum number of pointers: %u\n",
-                        cpl_xmemory_max_cells);
+                fprintf(stderr, "Maximum number of pointers"
+#               ifdef _OPENMP
+                        /* cpl_xmemory_max_cells is not updated thread-safely */
+                        " (approximate)"
+#               endif
+                        ": %u\n", cpl_xmemory_max_cells);
             }
 
             fprintf(stderr, "#----- Memory Currently Allocated -----\n");
@@ -641,6 +662,7 @@ void cpl_xmemory_status(int mode)
 
 /*----------------------------------------------------------------------------*/
 /**
+  @internal
   @brief    Tell if there is still some memory allocated
   @param mode  Mode (0: system, 1: Xmemory-failure check, 2: Xmemory-full)
   @return   1 if the memory table is empty, 0 if no,
@@ -968,35 +990,3 @@ static void cpl_xmemory_remcell(size_t pos)
 
     return;
 }
-
-/*----------------------------------------------------------------------------*/
-/**
-  @internal
-  @brief    Increment or decrement the allocation counter
-  @param    type   count_down for decrement, othwise (count_up) for increment
-  @return   void
-
- */
-/*----------------------------------------------------------------------------*/
-#ifndef _OPENMP
-/* Be sure not to inline (replicate) a critical section */
-inline
-#endif
-static void cpl_xmemory_count(count_type type)
-{
-
-#   ifdef _OPENMP
-#   pragma omp critical
-#   endif
-    {
-        if (type == count_down) {
-            /* assert( cpl_xmemory_ncells != 0 ); */
-            cpl_xmemory_ncells--;
-        } else {
-            cpl_xmemory_ncells++;
-            /* Remember peak allocation */
-            if (cpl_xmemory_ncells > cpl_xmemory_max_cells)
-                cpl_xmemory_max_cells = cpl_xmemory_ncells;
-        }
-    }
-}
