@@ -1,4 +1,4 @@
-/* $Id: cpl_filter-test.c,v 1.14 2010/11/11 09:23:18 llundin Exp $
+/* $Id: cpl_filter-test.c,v 1.19 2011/11/23 12:21:55 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/11/11 09:23:18 $
- * $Revision: 1.14 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2011/11/23 12:21:55 $
+ * $Revision: 1.19 $
+ * $Name: cpl-6_0 $
  */
  
 #ifdef HAVE_CONFIG_H
@@ -33,9 +33,11 @@
                                 Includes
  -----------------------------------------------------------------------------*/
 
+#include <cpl_image_filter_impl.h>
+
 #include <cpl_tools.h>
+#include <cpl_memory.h>
 #include <cpl_test.h>
-#include <cpl_image_filter.h>
 
 /* sqrt() */
 #include <math.h>
@@ -157,7 +159,6 @@ static cpl_error_code filter_average_bf(cpl_image *, const cpl_image *,
 /*----------------------------------------------------------------------------*/
 int main(void)
 {
-    /* Initialize CPL + CPL */
     cpl_test_init(PACKAGE_BUGREPORT, CPL_MSG_WARNING);
 
     test_filter();
@@ -192,7 +193,7 @@ static void test_filter(void)
 
         unsigned ibord;
         
-        for (ibord = 0 ; 
+        for (ibord = 0; 
              ibord < sizeof(border_mode)/sizeof(border_mode[0]); 
              ibord++) {
 
@@ -258,17 +259,14 @@ static void test_cpl_image_filter(int nx, int ny,
         cpl_test_nonnull(mask);
         
         error = cpl_image_filter_mask(NULL, in, mask, filter, border_mode);
-        cpl_test_error(CPL_ERROR_NULL_INPUT);
-        cpl_test_eq(error, CPL_ERROR_NULL_INPUT);
+        cpl_test_eq_error(error, CPL_ERROR_NULL_INPUT);
 
 
         error = cpl_image_filter_mask(in, NULL, mask, filter, border_mode);
-        cpl_test_error(CPL_ERROR_NULL_INPUT);
-        cpl_test_eq(error, CPL_ERROR_NULL_INPUT);
+        cpl_test_eq_error(error, CPL_ERROR_NULL_INPUT);
 
         error = cpl_image_filter_mask(in, out, NULL, filter, border_mode);
-        cpl_test_error(CPL_ERROR_NULL_INPUT);
-        cpl_test_eq(error, CPL_ERROR_NULL_INPUT);
+        cpl_test_eq_error(error, CPL_ERROR_NULL_INPUT);
         
         cpl_image_delete(in);
         cpl_image_delete(out);
@@ -279,6 +277,7 @@ static void test_cpl_image_filter(int nx, int ny,
     test_cpl_image_filter_double(nx, ny, rx, ry, filter, border_mode, 1, 1);
     test_cpl_image_filter_float(nx, ny, rx, ry, filter, border_mode, 1, 1);
     test_cpl_image_filter_int(nx, ny, rx, ry, filter, border_mode, 1, 1);
+
     return;
 }
 
@@ -394,7 +393,7 @@ static cpl_error_code filter_average_bf(cpl_image * self,
         }
         default:
             /* It is an error in CPL to reach this point */
-            assert( 0 );
+            (void)cpl_error_set_(CPL_ERROR_UNSPECIFIED);
     }
 
     cpl_image_delete(copy);
@@ -411,15 +410,27 @@ static cpl_error_code filter_average_bf(cpl_image * self,
 #define CONCAT2X(a,b) CONCAT(a,b)
 
 #define PIXEL_TYPE double
+#define PIXEL_MIN (-DBL_MAX)
+#define PIXEL_MAX DBL_MAX
 #include "cpl_filter_body.h"
 #undef PIXEL_TYPE
+#undef PIXEL_MIN
+#undef PIXEL_MAX
 
 #define PIXEL_TYPE float
+#define PIXEL_MIN (-FLT_MAX)
+#define PIXEL_MAX FLT_MAX
 #include "cpl_filter_body.h"
 #undef PIXEL_TYPE
+#undef PIXEL_MIN
+#undef PIXEL_MAX
 
 #define PIXEL_TYPE_IS_INT 1
 #define PIXEL_TYPE int
+#define PIXEL_MIN (-INT_MAX)
+#define PIXEL_MAX INT_MAX
 #include "cpl_filter_body.h"
 #undef PIXEL_TYPE
 #undef PIXEL_TYPE_IS_INT
+#undef PIXEL_MIN
+#undef PIXEL_MAX
