@@ -1,4 +1,4 @@
-/* $Id: cpl_image_basic-test.c,v 1.87 2010/11/15 09:55:45 llundin Exp $
+/* $Id: cpl_image_basic-test.c,v 1.93 2012/02/15 15:56:49 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/11/15 09:55:45 $
- * $Revision: 1.87 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2012/02/15 15:56:49 $
+ * $Revision: 1.93 $
+ * $Name: cpl-6_0 $
  */
 
 /*-----------------------------------------------------------------------------
@@ -63,6 +63,7 @@
                         Private function prototypes
  -----------------------------------------------------------------------------*/
 
+static void cpl_image_flip_turn_test(const cpl_image *);
 static void cpl_image_turn_flip_test(void);
 static void cpl_image_copy_test(void);
 static void cpl_image_collapse_median_test(void);
@@ -97,13 +98,14 @@ int main(void)
     cpl_vector      *   taylor;
     double          *   ddata;
     int                 nb_cut;
-    int             *   new_pos;
+    const cpl_size      new_pos[9] = {9, 8, 7, 6, 5, 4, 3, 2, 1};
     double              err;
     double              value;
     int                 is_rejected;
     cpl_mask        *   mask;
     int                 is_bad;
     int                 i, j, k;
+    cpl_error_code      error;
 
     /*
      * FIXME: Ideally, we should simply use the macro I 
@@ -323,9 +325,9 @@ int main(void)
     }
 
     /* Test with lo_cut equal to hi_cut */
-    cpl_test(cpl_image_threshold(imd1, (double)MAGIC_PIXEL, (double)MAGIC_PIXEL,
-                                 (double)MAGIC_PIXEL, (double)MAGIC_PIXEL)
-             == CPL_ERROR_NONE);
+    error = cpl_image_threshold(imd1, (double)MAGIC_PIXEL, (double)MAGIC_PIXEL,
+                                (double)MAGIC_PIXEL, (double)MAGIC_PIXEL);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
     cpl_test_leq( cpl_image_get_max(imd1), (double)MAGIC_PIXEL);
     cpl_test_leq( (double)MAGIC_PIXEL, cpl_image_get_min(imd1));
 
@@ -344,8 +346,8 @@ int main(void)
     imi1 = cpl_image_divide_create(imi, imd);
     cpl_image_delete(imi);
     cpl_image_delete(imd);
-    value = (double)cpl_image_get(imi1, 1, 1, &is_bad);
-    cpl_test_leq(fabs(value - 20.0), 0.0);
+    value = cpl_image_get(imi1, 1, 1, &is_bad);
+    cpl_test_abs(value, 20.0, 0.0);
     cpl_image_delete(imi1);
     
     imd = cpl_image_new(IMAGESZ, IMAGESZ, CPL_TYPE_DOUBLE);
@@ -364,7 +366,8 @@ int main(void)
         cpl_test_eq( j, IMAGESZ);
     }
 
-    cpl_test( imd1 = cpl_image_duplicate(imd) );
+    imd1 = cpl_image_duplicate(imd);
+    cpl_test_nonnull( imd1 );
 
     cpl_test_zero( cpl_image_fft(imd1, NULL, CPL_FFT_DEFAULT) );
 
@@ -390,7 +393,7 @@ int main(void)
     cpl_msg_info("", "FFT(%g) round-off [DBL_EPSILON]: %g < %d", value,
                   err/DBL_EPSILON, IMAGESZ*IMAGESZ);
 
-    cpl_test_leq( fabs(err), IMAGESZ*IMAGESZ * DBL_EPSILON/2.44863);
+    cpl_test_leq( fabs(err), IMAGESZ*IMAGESZ * DBL_EPSILON/1.5);
 
     cpl_image_delete(imd1);
 
@@ -402,8 +405,10 @@ int main(void)
         cpl_vector_set(taylor, i, cpl_vector_get(taylor, i-1)/i);
 
     /* Evaluate exp() using Horners scheme on the Taylor expansion */
-    cpl_test( imd2 = cpl_image_new(IMAGESZ, IMAGESZ, CPL_TYPE_DOUBLE) );
-    cpl_test_zero( cpl_image_add_scalar(imd2, cpl_vector_get(taylor, POLY_SIZE-1)) );
+    imd2 = cpl_image_new(IMAGESZ, IMAGESZ, CPL_TYPE_DOUBLE);
+    cpl_test_nonnull( imd2 );
+    cpl_test_zero( cpl_image_add_scalar(imd2, cpl_vector_get(taylor,
+                                                             POLY_SIZE-1)) );
     
     for (k=POLY_SIZE-1; k > 0; k--) {
         cpl_test_zero( cpl_image_multiply(imd2, imd) );
@@ -417,7 +422,8 @@ int main(void)
     }        
 
     /* Verify the result (against cpl_image_exponential() ) */
-    cpl_test( imd1 = cpl_image_duplicate(imd) );
+    imd1 = cpl_image_duplicate(imd);
+    cpl_test_nonnull( imd1 );
 
     cpl_test_zero( cpl_image_exponential(imd1, CPL_MATH_E) );
 
@@ -430,7 +436,8 @@ int main(void)
 
     cpl_image_delete(imd2);
     /* Evaluate exp() using cpl_image_pow() on the Taylor expansion */
-    cpl_test( imd2 = cpl_image_new(IMAGESZ, IMAGESZ, CPL_TYPE_DOUBLE) );
+    imd2 = cpl_image_new(IMAGESZ, IMAGESZ, CPL_TYPE_DOUBLE);
+    cpl_test_nonnull( imd2 );
     cpl_test_zero( cpl_image_add_scalar(imd2, cpl_vector_get(taylor, 0)) );
     
     /* POLY_SIZE > 10 on alphaev56:
@@ -462,8 +469,10 @@ int main(void)
 
     cpl_test_zero( cpl_image_fft(imd1, NULL, CPL_FFT_DEFAULT) );
     cpl_image_delete(imd2);
-    cpl_test( imd2 = cpl_image_duplicate(imd1) );
-    cpl_test( imtmp = cpl_image_duplicate(imd1) );
+    imd2 = cpl_image_duplicate(imd1);
+    cpl_test_nonnull( imd2 );
+    imtmp = cpl_image_duplicate(imd1);
+    cpl_test_nonnull( imtmp );
     cpl_test_zero( cpl_image_copy(imd1, imd2, 1, 1) );
 
     cpl_test_zero( cpl_image_add(imd1, imd2) );
@@ -507,15 +516,25 @@ int main(void)
     imf1 = cpl_image_cast(imd1, CPL_TYPE_FLOAT);
     imi1 = cpl_image_cast(imd1, CPL_TYPE_INT);
 
+    cpl_image_flip_turn_test(imd1);
+    cpl_image_flip_turn_test(imf1);
+    cpl_image_flip_turn_test(imi1);
+
     imd2 = cpl_image_fill_test_create(IMAGESZ, IMAGESZ);
-    cpl_image_save(imd2, "test.fits", CPL_BPP_IEEE_FLOAT, NULL,
+    cpl_image_save(imd2, "test.fits", CPL_TYPE_FLOAT, NULL,
                    CPL_IO_CREATE);
     remove("test.fits");
     imf2 = cpl_image_cast(imd2, CPL_TYPE_FLOAT);
     imi2 = cpl_image_cast(imd2, CPL_TYPE_INT);
 
-    cpl_test( imd1 != NULL && imf1 != NULL && imi1 != NULL);
-    cpl_test( imd2 != NULL && imf2 != NULL && imi2 != NULL);
+    cpl_test_nonnull( imd1 );
+    cpl_test_nonnull( imd2 );
+
+    cpl_test_nonnull( imf1 );
+    cpl_test_nonnull( imf2 );
+
+    cpl_test_nonnull( imi1 );
+    cpl_test_nonnull( imi2 );
   
     /* COMPUTE imi =(imi1-imi2)*imi1/(imi1+imi2) with local functions */
     cpl_msg_info("","Compute imi = (imi1-imi2)*imi1/(imi1+imi2) in INTEGER");
@@ -629,22 +648,18 @@ int main(void)
     cpl_msg_info("","Test the pixels moving function on imd");
     imd1 = cpl_image_extract(imd, 11, 11, 13, 16);
     nb_cut = 3;
-    new_pos = cpl_malloc(nb_cut*nb_cut*sizeof(int));
-    new_pos[0] = 9;
-    new_pos[1] = 8;
-    new_pos[2] = 7;
-    new_pos[3] = 6;
-    new_pos[4] = 5;
-    new_pos[5] = 4;
-    new_pos[6] = 3;
-    new_pos[7] = 2;
-    new_pos[8] = 1;
-    if (cpl_image_move(imd1, nb_cut, new_pos) != CPL_ERROR_NONE) {
-        cpl_free(new_pos);
-        cpl_image_delete(imd1);
-        return cpl_test_end(1);
-    }
-    cpl_free(new_pos);
+
+    error = cpl_image_move(imd1, nb_cut, new_pos);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    cpl_image_delete(imd1);
+
+    /* Test extraction w. a bad pixel */
+    cpl_test_zero(cpl_image_reject(imd, 12, 14));
+    cpl_test_eq(cpl_image_count_rejected(imd), 1);
+    imd1 = cpl_image_extract(imd, 11, 11, 13, 16);
+    cpl_test_eq(cpl_image_get_size_x(imd1), 3);
+    cpl_test_eq(cpl_image_get_size_y(imd1), 6);
+    cpl_test(cpl_image_is_rejected(imd1, 2, 4));
     cpl_image_delete(imd1);
 
     /* Threshold imi, imf and imd */
@@ -665,16 +680,19 @@ int main(void)
     /* Extract sub windows from imf and imd */
     cpl_msg_info("","Extract the central part of imi, imf and imd");
     imtmp = cpl_image_extract(imi, IMAGESZ/4, IMAGESZ/4, IMAGESZ/2, IMAGESZ/2);
+    cpl_test_nonnull( imtmp );
     cpl_image_delete(imi); 
-    cpl_test( imi = imtmp );
+    imi = imtmp;
     imtmp = NULL;
     imtmp = cpl_image_extract(imf, IMAGESZ/4, IMAGESZ/4, IMAGESZ/2, IMAGESZ/2);
+    cpl_test_nonnull( imtmp );
     cpl_image_delete(imf); 
-    cpl_test( imf = imtmp );
+    imf = imtmp;
     imtmp = NULL;
     imtmp = cpl_image_extract(imd, IMAGESZ/4, IMAGESZ/4, IMAGESZ/2, IMAGESZ/2);
+    cpl_test_nonnull( imtmp );
     cpl_image_delete(imd); 
-    cpl_test( imd = imtmp );
+    imd = imtmp;
     imtmp = NULL;
 
     /* Test exp operation */
@@ -730,12 +748,14 @@ int main(void)
     /* Normalize imf and imd */
     cpl_msg_info("","Normalize imf and imd");
     imtmp = cpl_image_normalise_create(imf, CPL_NORM_MEAN);
+    cpl_test_nonnull( imtmp );
     cpl_image_delete(imf); 
-    cpl_test( imf = imtmp );
+    imf = imtmp;
     imtmp = NULL;
     imtmp = cpl_image_normalise_create(imd, CPL_NORM_MEAN);
+    cpl_test_nonnull( imtmp );
     cpl_image_delete(imd); 
-    cpl_test( imd = imtmp );
+    imd = imtmp;
     imtmp = NULL;
   
     /* Test the insertion function */
@@ -973,23 +993,32 @@ static void cpl_image_turn_flip_test(void)
 
     const int buf9[] = {1, 2, 3, 4, 5, 6, 7, 8, 9};
 
-    /* Rotation 90 degrees clockwise */
-    const int rot9[] = {3, 6, 9, 2, 5, 8, 1, 4, 7};
+    /* Rotate this 90 degrees clockwise to get buf9 */
+    const int rot9[] = {7, 4, 1, 8, 5, 2, 9, 6, 3};
 
     /* This image is not modified while wrapping around this buffer */
-    cpl_image * ref = cpl_image_wrap_int(3, 3, (int*)rot9);
+    cpl_image * ref = cpl_image_new(3, 3, CPL_TYPE_INT);
     cpl_image * raw = cpl_image_new(3, 3, CPL_TYPE_INT);
     cpl_error_code error;
 
+    cpl_test_eq_ptr(memcpy(cpl_image_get_data_int(ref), buf9, 9*sizeof(int)),
+                    cpl_image_get_data_int_const(ref));
+
     /* Test 1: Verify direction of 90 degree rotation */
-    cpl_test_eq_ptr(memcpy(cpl_image_get_data_int(raw), buf9, 9*sizeof(int)),
+    cpl_test_eq_ptr(memcpy(cpl_image_get_data_int(raw), rot9, 9*sizeof(int)),
                     cpl_image_get_data_int_const(raw));
 
-    cpl_test_eq(cpl_image_turn(raw, 1), CPL_ERROR_NONE);
+    error = cpl_image_turn(raw, 1);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
     cpl_test_image_abs(raw, ref, 0.0);
 
     cpl_image_delete(raw);
-    (void)cpl_image_unwrap(ref);
+
+    /* Test 1a: Verify flip around x=y and x=-y */
+
+    cpl_image_flip_turn_test(ref);
+
+    cpl_image_delete(ref);
 
     /* Test 2: Check error handling */
     error = cpl_image_turn(NULL, 1);
@@ -1332,3 +1361,59 @@ static void cpl_image_collapse_median_test(void)
     return;
 
 }
+
+
+/*----------------------------------------------------------------------------*/
+/**
+  @internal
+  @brief  Test cpl_image_flip(), cpl_image_turn()
+  @param  self    The image to test
+  @return void
+  @see cpl_mask_flip_turn_test()
+
+   A: 
+       Rotate 90 counterclock-wise, then
+       flip around the vertical axis, then
+       flip around x=y, and
+       it should be back to its original.
+
+   B: 
+
+       Rotate 90 counterclock-wise, then
+       flip around the horizontal axis, then
+       flip around x=-y, and
+       it should be back to its original.
+
+
+ */
+/*----------------------------------------------------------------------------*/
+static void cpl_image_flip_turn_test(const cpl_image * self)
+{
+
+    cpl_error_code error;
+    cpl_image * tmp = cpl_image_duplicate(self);
+
+    error = cpl_image_turn(tmp, -1);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+    error = cpl_image_flip(tmp, 2);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+    error = cpl_image_flip(tmp, 1);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+    cpl_test_image_abs(self, tmp, 0.0);
+
+    error = cpl_image_turn(tmp, -1);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+    error = cpl_image_flip(tmp, 0);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+    error = cpl_image_flip(tmp, 3);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+    cpl_test_image_abs(self, tmp, 0.0);
+
+    cpl_image_delete(tmp);
+}
