@@ -1,4 +1,4 @@
-/* $Id: cpl_image_io-test.c,v 1.59 2010/12/23 15:08:13 llundin Exp $
+/* $Id: cpl_image_io-test.c,v 1.66 2012/03/13 15:06:03 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/12/23 15:08:13 $
- * $Revision: 1.59 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2012/03/13 15:06:03 $
+ * $Revision: 1.66 $
+ * $Name: cpl-6_0 $
  */
 
 /*-----------------------------------------------------------------------------
@@ -54,7 +54,7 @@
                                   Static functions
  -----------------------------------------------------------------------------*/
 
-static double cpl_image_get_precision(cpl_type, cpl_type_bpp, double);
+static double cpl_image_get_precision(cpl_type, cpl_type, double);
 static void cpl_image_fill_int_test(void);
 
 /*-----------------------------------------------------------------------------
@@ -65,9 +65,9 @@ int main(void)
     const cpl_type     imtypes[] = {CPL_TYPE_DOUBLE, CPL_TYPE_FLOAT,
                                     CPL_TYPE_INT, CPL_TYPE_DOUBLE_COMPLEX,
                     CPL_TYPE_FLOAT_COMPLEX};
-    const cpl_type_bpp bpps[] = {CPL_BPP_8_UNSIGNED, CPL_BPP_16_SIGNED,
-                                 CPL_BPP_16_UNSIGNED, CPL_BPP_32_SIGNED,
-                                 CPL_BPP_IEEE_FLOAT, CPL_BPP_IEEE_DOUBLE};
+    const cpl_type bpps[] = {CPL_TYPE_UCHAR, CPL_TYPE_SHORT,
+                             CPL_TYPE_USHORT, CPL_TYPE_INT,
+                             CPL_TYPE_FLOAT, CPL_TYPE_DOUBLE};
     const double minval[] = {0.0,   -32767.0,     0.0, -2147483647.0,
                              -65535.0, -2147483647.0};
     const double maxval[] = {255.0,  32767.0, 65535.0,  2147483647.0,
@@ -218,8 +218,8 @@ int main(void)
 
     for (ibpp = 0; ibpp < nbpp; ibpp++) {
 
-        cpl_msg_info(cpl_func, "Testing saving with BITPIX=%d and no pixels",
-                     bpps[ibpp]);
+        cpl_msg_info(cpl_func, "Testing saving with save type '%s' and no "
+                     "pixels", cpl_type_get_name(bpps[ibpp]));
 
         cpl_test_zero(cpl_image_save(NULL, FILENAME, bpps[ibpp],
                                      NULL, CPL_IO_CREATE));
@@ -256,7 +256,8 @@ int main(void)
 
         cpl_test_nonnull(img);
 
-        cpl_image_dump_structure(img, stream);
+        error = cpl_image_dump_structure(img, stream);
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
 
         cpl_test_eq(cpl_image_get_type(img),   imtype);
         cpl_test_eq(cpl_image_get_size_x(img), IMAGESZ);
@@ -479,10 +480,43 @@ int main(void)
         cpl_test_image_abs(real_img, real_copy, 0.0);
         cpl_test_image_abs(imag_img, imag_copy, 0.0);
 
+        /* Test cpl_image_conjugate() and cpl_image_fill_re_im() */
+
+        error = cpl_image_conjugate(img, img);
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+        error = cpl_image_multiply_scalar(imag_copy, -1.0);
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+        error = cpl_image_fill_re_im(real_img, imag_img, img);
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+        cpl_test_image_abs(real_img, real_copy, 0.0);
+        cpl_test_image_abs(imag_img, imag_copy, 0.0);
+
+        error = cpl_image_fill_re_im(NULL, imag_img, img);
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+        error = cpl_image_fill_re_im(real_img, NULL, img);
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+        cpl_test_image_abs(real_img, real_copy, 0.0);
+        cpl_test_image_abs(imag_img, imag_copy, 0.0);
+
+        /* Test in-place */
+        error = cpl_image_fill_re_im(copy, NULL, copy);
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+        cpl_test_image_abs(copy, real_img, 0.0);
+
+        error = cpl_image_fill_re_im(NULL, copy, copy);
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
+
         cpl_image_delete(real_img);
         cpl_image_delete(imag_img);
         cpl_image_delete(real_copy);
         cpl_image_delete(imag_copy);
+
     }
 
         cpl_test_nonnull(cpl_image_unwrap(copy));
@@ -492,230 +526,231 @@ int main(void)
          imtype != CPL_TYPE_DOUBLE_COMPLEX) {
         for (ibpp = 0; ibpp < nbpp; ibpp++) {
 
-        cpl_msg_info(cpl_func, "Testing saving with BITPIX=%d and pixels "
-                 "ranging: %g -> %g", bpps[ibpp], minval[ibpp],
-                 maxval[ibpp]);
+            cpl_msg_info(cpl_func, "Testing saving with BITPIX=%d and pixels "
+                         "ranging: %g -> %g", bpps[ibpp], minval[ibpp],
+                         maxval[ibpp]);
 
-        /* Use bpp-specific value */
-        cpl_test_zero(cpl_image_fill_noise_uniform(img, minval[ibpp],
-                               maxval[ibpp]));
+            /* Use bpp-specific value */
+            cpl_test_zero(cpl_image_fill_noise_uniform(img, minval[ibpp],
+                                                       maxval[ibpp]));
 
-        /* First, do some failure testing */
+            /* First, do some failure testing */
 
-        /* NULL filename - and no image */
-        error = cpl_image_save(NULL, NULL, bpps[ibpp], NULL,
-                       CPL_IO_CREATE);
-        cpl_test_eq_error(error, CPL_ERROR_NULL_INPUT);
+            /* NULL filename - and no image */
+            error = cpl_image_save(NULL, NULL, bpps[ibpp], NULL,
+                                   CPL_IO_CREATE);
+            cpl_test_eq_error(error, CPL_ERROR_NULL_INPUT);
 
-        /* Directory as filename - and no image */
-        error = cpl_image_save(NULL, ".", bpps[ibpp], NULL,
-                       CPL_IO_CREATE);
-        cpl_test_eq_error(error, CPL_ERROR_FILE_IO);
+            /* Directory as filename - and no image */
+            error = cpl_image_save(NULL, ".", bpps[ibpp], NULL,
+                                   CPL_IO_CREATE);
+            cpl_test_eq_error(error, CPL_ERROR_FILE_IO);
 
-        /* Invalid mode - and no image */
-        error = cpl_image_save(NULL, FILENAME, bpps[ibpp], NULL, -1);
-        cpl_test_eq_error(error, CPL_ERROR_ILLEGAL_INPUT);
+            /* Invalid mode - and no image */
+            error = cpl_image_save(NULL, FILENAME, bpps[ibpp], NULL, -1);
+            cpl_test_eq_error(error, CPL_ERROR_ILLEGAL_INPUT);
 
-        /* More failure tests, with an image */
-        /* NULL filename */
-        error = cpl_image_save(img, NULL, bpps[ibpp], NULL,
-                       CPL_IO_CREATE);
-        cpl_test_eq_error(error, CPL_ERROR_NULL_INPUT);
+            /* More failure tests, with an image */
+            /* NULL filename */
+            error = cpl_image_save(img, NULL, bpps[ibpp], NULL,
+                                   CPL_IO_CREATE);
+            cpl_test_eq_error(error, CPL_ERROR_NULL_INPUT);
 
-        /* Directory as filename */
-        error = cpl_image_save(img, ".", bpps[ibpp], NULL,
-                       CPL_IO_CREATE);
-        cpl_test_eq_error(error, CPL_ERROR_FILE_IO);
+            /* Directory as filename */
+            error = cpl_image_save(img, ".", bpps[ibpp], NULL,
+                                   CPL_IO_CREATE);
+            cpl_test_eq_error(error, CPL_ERROR_FILE_IO);
 
-        /* Invalid BPP */
-        error = cpl_image_save(img, FILENAME, 0, NULL,
-                       CPL_IO_CREATE);
-        cpl_test_eq_error(error, CPL_ERROR_ILLEGAL_INPUT);
+            /* Invalid BPP */
+            error = cpl_image_save(img, FILENAME, 0, NULL,
+                                   CPL_IO_CREATE);
+            cpl_test_eq_error(error, CPL_ERROR_ILLEGAL_INPUT);
 
-        /* Invalid mode */
-        error = cpl_image_save(img, FILENAME, bpps[ibpp], NULL, -1);
-        cpl_test_eq_error(error, CPL_ERROR_ILLEGAL_INPUT);
+            /* Invalid mode */
+            error = cpl_image_save(img, FILENAME, bpps[ibpp], NULL, -1);
+            cpl_test_eq_error(error, CPL_ERROR_ILLEGAL_INPUT);
 
-        /* Illegal mode combination */
-        error = cpl_image_save(img, FILENAME, bpps[ibpp], NULL,
-                       CPL_IO_CREATE | CPL_IO_EXTEND);
-        cpl_test_eq_error(error, CPL_ERROR_ILLEGAL_INPUT);
+            /* Illegal mode combination */
+            error = cpl_image_save(img, FILENAME, bpps[ibpp], NULL,
+                                   CPL_IO_CREATE | CPL_IO_EXTEND);
+            cpl_test_eq_error(error, CPL_ERROR_ILLEGAL_INPUT);
 
-        /* Illegal mode combination */
-        error = cpl_image_save(img, FILENAME, bpps[ibpp], NULL,
-                       CPL_IO_APPEND);
-        cpl_test_eq_error(error, CPL_ERROR_ILLEGAL_INPUT);
+            /* Illegal mode combination */
+            error = cpl_image_save(img, FILENAME, bpps[ibpp], NULL,
+                                   CPL_IO_APPEND);
+            cpl_test_eq_error(error, CPL_ERROR_ILLEGAL_INPUT);
 
-        /* Create/Append file */
-        error = cpl_image_save(img, FILENAME, bpps[ibpp], NULL,
-                   ibpp ? CPL_IO_EXTEND : CPL_IO_CREATE);
-        cpl_test_eq_error(error, CPL_ERROR_NONE);
-        cpl_test_fits(FILENAME);
+            /* Create/Append file */
+            error = cpl_image_save(img, FILENAME, bpps[ibpp], NULL,
+                                   ibpp ? CPL_IO_EXTEND : CPL_IO_CREATE);
+            cpl_test_eq_error(error, CPL_ERROR_NONE);
+            cpl_test_fits(FILENAME);
 
-        cpl_test_eq( cpl_fits_count_extensions(FILENAME), ibpp );
-
-        /* Tests for no-casting type */
-        copy = cpl_image_load(FILENAME, CPL_TYPE_UNSPECIFIED,
-                      0, ibpp);
-        cpl_test_nonnull(copy);
-        cpl_test_error(CPL_ERROR_NONE);
-
-        if (bpps[ibpp] == CPL_BPP_8_UNSIGNED  ||
-            bpps[ibpp] == CPL_BPP_16_SIGNED   ||
-            bpps[ibpp] == CPL_BPP_16_UNSIGNED ||
-            bpps[ibpp] == CPL_BPP_32_SIGNED     )
-            cpl_test_eq(cpl_image_get_type(copy), CPL_TYPE_INT);
-        else if (bpps[ibpp] == CPL_BPP_IEEE_FLOAT)
-            cpl_test_eq(cpl_image_get_type(copy), CPL_TYPE_FLOAT);
-        else
-            cpl_test_eq(cpl_image_get_type(copy), CPL_TYPE_DOUBLE);
-
-        cpl_image_delete(copy);
-
-                /* Check error handling in windowed image loading */
-                nullimg = cpl_image_load_window(FILENAME, CPL_TYPE_UNSPECIFIED,
-                                                0, ibpp,
-                                                0, IMAGESZ/4,
-                                                3*IMAGESZ/4, 3*IMAGESZ/4);
-                cpl_test_error(CPL_ERROR_ILLEGAL_INPUT);
-                cpl_test_null(nullimg);
-
-                nullimg = cpl_image_load_window(FILENAME, CPL_TYPE_UNSPECIFIED,
-                                                0, ibpp,
-                                                IMAGESZ/4, 0,
-                                                3*IMAGESZ/4, 3*IMAGESZ/4);
-                cpl_test_error(CPL_ERROR_ILLEGAL_INPUT);
-                cpl_test_null(nullimg);
-
-                nullimg = cpl_image_load_window(FILENAME, CPL_TYPE_UNSPECIFIED,
-                                                0, ibpp,
-                                                3*IMAGESZ/4, IMAGESZ/4,
-                                                IMAGESZ/4, 3*IMAGESZ/4);
-                cpl_test_error(CPL_ERROR_ILLEGAL_INPUT);
-                cpl_test_null(nullimg);
-
-                nullimg = cpl_image_load_window(FILENAME, CPL_TYPE_UNSPECIFIED,
-                                                0, ibpp,
-                                                IMAGESZ/4, 3*IMAGESZ/4,
-                                                3*IMAGESZ/4, IMAGESZ/4);
-                cpl_test_error(CPL_ERROR_ILLEGAL_INPUT);
-                cpl_test_null(nullimg);
-
-                nullimg = cpl_image_load_window(FILENAME, CPL_TYPE_UNSPECIFIED,
-                                                0, ibpp,
-                                                IMAGESZ/4, IMAGESZ/4,
-                                                IMAGESZ+1, 3*IMAGESZ/4);
-                cpl_test_error(CPL_ERROR_ILLEGAL_INPUT);
-                cpl_test_null(nullimg);
-
-                nullimg = cpl_image_load_window(FILENAME, CPL_TYPE_UNSPECIFIED,
-                                                0, ibpp,
-                                                IMAGESZ/4, IMAGESZ/4,
-                                                3*IMAGESZ/4, IMAGESZ+1);
-                cpl_test_error(CPL_ERROR_ILLEGAL_INPUT);
-                cpl_test_null(nullimg);
-
-
-        copy = cpl_image_load_window(FILENAME, CPL_TYPE_UNSPECIFIED,
-                         0, ibpp,
-                         IMAGESZ/4, IMAGESZ/4,
-                         3*IMAGESZ/4, 3*IMAGESZ/4);
-        cpl_test_nonnull(copy);
-        cpl_test_error(CPL_ERROR_NONE);
-
-        if (bpps[ibpp] == CPL_BPP_8_UNSIGNED  ||
-            bpps[ibpp] == CPL_BPP_16_SIGNED   ||
-            bpps[ibpp] == CPL_BPP_16_UNSIGNED ||
-            bpps[ibpp] == CPL_BPP_32_SIGNED     )
-            cpl_test_eq(cpl_image_get_type(copy), CPL_TYPE_INT);
-        else if (bpps[ibpp] == CPL_BPP_IEEE_FLOAT)
-            cpl_test_eq(cpl_image_get_type(copy), CPL_TYPE_FLOAT);
-        else
-            cpl_test_eq(cpl_image_get_type(copy), CPL_TYPE_DOUBLE);
-
-        cpl_image_delete(copy);
-
-        /* Load it again with casting */
-        for (ityp2 = 0;
-             ityp2 < (int)(sizeof(imtypes)/sizeof(imtypes[0]) - 2);
-             ityp2++) { /* The '- 2' is to avoid complex types */
-            const cpl_type imtype2 = imtypes[ityp2];
-
-            copy = cpl_image_load(FILENAME, imtype2, 0, ibpp);
+            cpl_test_eq( cpl_fits_count_extensions(FILENAME), ibpp );
 
+            /* Tests for no-casting type */
+            copy = cpl_image_load(FILENAME, CPL_TYPE_UNSPECIFIED,
+                                  0, ibpp);
+            cpl_test_error(CPL_ERROR_NONE);
             cpl_test_nonnull(copy);
-            cpl_test_eq(cpl_image_get_type(copy), imtype2);
 
-            if (imtype == imtype2) {
-            const double precision
-                = cpl_image_get_precision(imtype, bpps[ibpp],
-                              maxval[ibpp]);
-            cpl_test_image_abs(img, copy, precision);
-            }
+            if (bpps[ibpp] == CPL_TYPE_UCHAR  ||
+                bpps[ibpp] == CPL_TYPE_SHORT   ||
+                bpps[ibpp] == CPL_TYPE_USHORT ||
+                bpps[ibpp] == CPL_TYPE_INT     )
+                cpl_test_eq(cpl_image_get_type(copy), CPL_TYPE_INT);
+            else if (bpps[ibpp] == CPL_TYPE_FLOAT)
+                cpl_test_eq(cpl_image_get_type(copy), CPL_TYPE_FLOAT);
+            else
+                cpl_test_eq(cpl_image_get_type(copy), CPL_TYPE_DOUBLE);
 
             cpl_image_delete(copy);
 
-            copy = cpl_image_load_window(FILENAME, imtype2, 0, ibpp,
-                         1, 1,
-                         IMAGESZ, IMAGESZ);
-
-            cpl_test_nonnull(copy);
-            cpl_test_eq(cpl_image_get_type(copy), imtype2);
-
-            if (imtype == imtype2) {
-            const double precision
-                = cpl_image_get_precision(imtype, bpps[ibpp],
-                              maxval[ibpp]);
-            cpl_test_image_abs(img, copy, precision);
-            }
-
-            cpl_image_delete(copy);
-
-            copy = cpl_image_load_window(FILENAME, imtype2, 0, ibpp,
-                         IMAGESZ/4, IMAGESZ/4,
-                         3*IMAGESZ/4, 3*IMAGESZ/4);
-            cpl_test_nonnull(copy);
-            cpl_test_eq(cpl_image_get_type(copy), imtype2);
-
-
-            cpl_test_eq(cpl_image_get_size_x(copy), 1+IMAGESZ/2);
-            cpl_test_eq(cpl_image_get_size_y(copy), 1+IMAGESZ/2);
-
-            cpl_image_delete(copy);
-
-
-            copy = cpl_image_load_window(FILENAME, imtype2, 0, ibpp,
-                         IMAGESZ, IMAGESZ,
-                         IMAGESZ, IMAGESZ);
-
-            cpl_test_nonnull(copy);
-            cpl_test_eq(cpl_image_get_type(copy), imtype2);
-
-            cpl_test_eq(cpl_image_get_size_x(copy), 1);
-            cpl_test_eq(cpl_image_get_size_y(copy), 1);
-
-            cpl_image_delete(copy);
-
-
-            copy = cpl_image_load_window(FILENAME, imtype2, 0, ibpp,
-                         2, 2, 1, 1);
-
+            /* Check error handling in windowed image loading */
+            nullimg = cpl_image_load_window(FILENAME, CPL_TYPE_UNSPECIFIED,
+                                            0, ibpp,
+                                            0, IMAGESZ/4,
+                                            3*IMAGESZ/4, 3*IMAGESZ/4);
             cpl_test_error(CPL_ERROR_ILLEGAL_INPUT);
-            cpl_test_null(copy);
+            cpl_test_null(nullimg);
+
+            nullimg = cpl_image_load_window(FILENAME, CPL_TYPE_UNSPECIFIED,
+                                            0, ibpp,
+                                            IMAGESZ/4, 0,
+                                            3*IMAGESZ/4, 3*IMAGESZ/4);
+            cpl_test_error(CPL_ERROR_ILLEGAL_INPUT);
+            cpl_test_null(nullimg);
+
+            nullimg = cpl_image_load_window(FILENAME, CPL_TYPE_UNSPECIFIED,
+                                            0, ibpp,
+                                            3*IMAGESZ/4, IMAGESZ/4,
+                                            IMAGESZ/4, 3*IMAGESZ/4);
+            cpl_test_error(CPL_ERROR_ILLEGAL_INPUT);
+            cpl_test_null(nullimg);
+
+            nullimg = cpl_image_load_window(FILENAME, CPL_TYPE_UNSPECIFIED,
+                                            0, ibpp,
+                                            IMAGESZ/4, 3*IMAGESZ/4,
+                                            3*IMAGESZ/4, IMAGESZ/4);
+            cpl_test_error(CPL_ERROR_ILLEGAL_INPUT);
+            cpl_test_null(nullimg);
+
+            nullimg = cpl_image_load_window(FILENAME, CPL_TYPE_UNSPECIFIED,
+                                            0, ibpp,
+                                            IMAGESZ/4, IMAGESZ/4,
+                                            IMAGESZ+1, 3*IMAGESZ/4);
+            cpl_test_error(CPL_ERROR_ILLEGAL_INPUT);
+            cpl_test_null(nullimg);
+
+            nullimg = cpl_image_load_window(FILENAME, CPL_TYPE_UNSPECIFIED,
+                                            0, ibpp,
+                                            IMAGESZ/4, IMAGESZ/4,
+                                            3*IMAGESZ/4, IMAGESZ+1);
+            cpl_test_error(CPL_ERROR_ILLEGAL_INPUT);
+            cpl_test_null(nullimg);
+
+
+            copy = cpl_image_load_window(FILENAME, CPL_TYPE_UNSPECIFIED,
+                                         0, ibpp,
+                                         IMAGESZ/4, IMAGESZ/4,
+                                         3*IMAGESZ/4, 3*IMAGESZ/4);
+            cpl_test_nonnull(copy);
+            cpl_test_error(CPL_ERROR_NONE);
+
+            if (bpps[ibpp] == CPL_TYPE_UCHAR  ||
+                bpps[ibpp] == CPL_TYPE_SHORT   ||
+                bpps[ibpp] == CPL_TYPE_USHORT ||
+                bpps[ibpp] == CPL_TYPE_INT     )
+                cpl_test_eq(cpl_image_get_type(copy), CPL_TYPE_INT);
+            else if (bpps[ibpp] == CPL_TYPE_FLOAT)
+                cpl_test_eq(cpl_image_get_type(copy), CPL_TYPE_FLOAT);
+            else
+                cpl_test_eq(cpl_image_get_type(copy), CPL_TYPE_DOUBLE);
+
+            cpl_image_delete(copy);
+
+            /* Load it again with casting */
+            for (ityp2 = 0;
+                 ityp2 < (int)(sizeof(imtypes)/sizeof(imtypes[0]) - 2);
+                 ityp2++) { /* The '- 2' is to avoid complex types */
+                const cpl_type imtype2 = imtypes[ityp2];
+
+                copy = cpl_image_load(ibpp % 2 ? FILENAME : "./" FILENAME,
+                                      imtype2, 0, ibpp);
+
+                cpl_test_nonnull(copy);
+                cpl_test_eq(cpl_image_get_type(copy), imtype2);
+
+                if (imtype == imtype2) {
+                    const double precision
+                        = cpl_image_get_precision(imtype, bpps[ibpp],
+                                                  maxval[ibpp]);
+                    cpl_test_image_abs(img, copy, precision);
+                }
+
+                cpl_image_delete(copy);
+
+                copy = cpl_image_load_window(FILENAME, imtype2, 0, ibpp,
+                                             1, 1,
+                                             IMAGESZ, IMAGESZ);
+
+                cpl_test_nonnull(copy);
+                cpl_test_eq(cpl_image_get_type(copy), imtype2);
+
+                if (imtype == imtype2) {
+                    const double precision
+                        = cpl_image_get_precision(imtype, bpps[ibpp],
+                                                  maxval[ibpp]);
+                    cpl_test_image_abs(img, copy, precision);
+                }
+
+                cpl_image_delete(copy);
+
+                copy = cpl_image_load_window(FILENAME, imtype2, 0, ibpp,
+                                             IMAGESZ/4, IMAGESZ/4,
+                                             3*IMAGESZ/4, 3*IMAGESZ/4);
+                cpl_test_nonnull(copy);
+                cpl_test_eq(cpl_image_get_type(copy), imtype2);
+
+
+                cpl_test_eq(cpl_image_get_size_x(copy), 1+IMAGESZ/2);
+                cpl_test_eq(cpl_image_get_size_y(copy), 1+IMAGESZ/2);
+
+                cpl_image_delete(copy);
+
+
+                copy = cpl_image_load_window(FILENAME, imtype2, 0, ibpp,
+                                             IMAGESZ, IMAGESZ,
+                                             IMAGESZ, IMAGESZ);
+
+                cpl_test_nonnull(copy);
+                cpl_test_eq(cpl_image_get_type(copy), imtype2);
+
+                cpl_test_eq(cpl_image_get_size_x(copy), 1);
+                cpl_test_eq(cpl_image_get_size_y(copy), 1);
+
+                cpl_image_delete(copy);
+
+
+                copy = cpl_image_load_window(FILENAME, imtype2, 0, ibpp,
+                                             2, 2, 1, 1);
+
+                cpl_test_error(CPL_ERROR_ILLEGAL_INPUT);
+                cpl_test_null(copy);
 
         }
         }
 
         /* Append one data-less extension (DFS06130) */
 
-        cpl_test_zero(cpl_image_save(NULL, FILENAME, CPL_BPP_8_UNSIGNED,
+        cpl_test_zero(cpl_image_save(NULL, FILENAME, CPL_TYPE_UCHAR,
                      NULL, CPL_IO_EXTEND));
         cpl_test_fits(FILENAME);
 
         /* Create overflow with 8 BPP */
         cpl_image_threshold(img, 256.0, 256.0, 256.0, 256.0);
 
-        cpl_image_save(img, FILENAME, CPL_BPP_8_UNSIGNED, NULL,
+        cpl_image_save(img, FILENAME, CPL_TYPE_UCHAR, NULL,
                CPL_IO_EXTEND);
         cpl_test_error(CPL_ERROR_FILE_IO);
 
@@ -723,7 +758,7 @@ int main(void)
         /* Create underflow with 16 BPP signed */
         cpl_image_subtract_scalar(img, (double)(1<<15) + 256.0 + 1.0);
 
-        cpl_image_save(img, FILENAME, CPL_BPP_16_SIGNED, NULL,
+        cpl_image_save(img, FILENAME, CPL_TYPE_SHORT, NULL,
                CPL_IO_EXTEND);
         cpl_test_error(CPL_ERROR_FILE_IO);
 
@@ -731,7 +766,7 @@ int main(void)
         /* Create overflow with 16 BPP unsigned */
         cpl_image_add_scalar(img, (double)(3<<16));
 
-        cpl_image_save(img, FILENAME, CPL_BPP_16_UNSIGNED, NULL,
+        cpl_image_save(img, FILENAME, CPL_TYPE_USHORT, NULL,
                CPL_IO_EXTEND);
         cpl_test_error(CPL_ERROR_FILE_IO);
 
@@ -739,7 +774,7 @@ int main(void)
         /* Create overflow with 32 BPP signed */
         cpl_image_add_scalar(img, 4294967296.0);
 
-        cpl_image_save(img, FILENAME, CPL_BPP_32_SIGNED, NULL,
+        cpl_image_save(img, FILENAME, CPL_TYPE_INT, NULL,
                    CPL_IO_EXTEND);
         cpl_test_error(CPL_ERROR_FILE_IO);
 
@@ -753,7 +788,7 @@ int main(void)
             /* Create overflow with 32 BPP float */
             cpl_image_add_scalar(img, 1e40);
 
-            cpl_test_zero(cpl_image_save(img, FILENAME, CPL_BPP_IEEE_FLOAT,
+            cpl_test_zero(cpl_image_save(img, FILENAME, CPL_TYPE_FLOAT,
                                          NULL, CPL_IO_CREATE));
             cpl_test_error(CPL_ERROR_NONE);
             cpl_test_fits(FILENAME);
@@ -775,12 +810,12 @@ int main(void)
         for (i=0; i < nbad-1; i++) 
             cpl_test_zero(cpl_image_reject(img, bad_pos_x[i], bad_pos_y[i]));
 
-        cpl_image_dump_structure(img, stream);
-        cpl_test_error(CPL_ERROR_NONE);
+        error = cpl_image_dump_structure(img, stream);
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
 
-        cpl_image_dump_window(img, bad_pos_x[0]-1, bad_pos_y[0]-1,
-                              bad_pos_x[0]+1, bad_pos_y[0]+1, stream);
-        cpl_test_error(CPL_ERROR_NONE);
+        error = cpl_image_dump_window(img, bad_pos_x[0]-1, bad_pos_y[0]-1,
+                                      bad_pos_x[0]+1, bad_pos_y[0]+1, stream);
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
 
         cpl_test_eq( nbad-1, cpl_image_count_rejected(img) );
 
@@ -817,6 +852,29 @@ int main(void)
         cpl_test_image_abs(real_img, real_copy, 0.0);
         cpl_test_image_abs(imag_img, imag_copy, 0.0);
 
+        /* Test cpl_image_conjugate() and cpl_image_fill_re_im() */
+
+        error = cpl_image_conjugate(img, img);
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+        error = cpl_image_multiply_scalar(imag_copy, -1.0);
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+        error = cpl_image_fill_re_im(real_img, imag_img, img);
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+        cpl_test_image_abs(real_img, real_copy, 0.0);
+        cpl_test_image_abs(imag_img, imag_copy, 0.0);
+
+        /* Test in-place */
+        error = cpl_image_fill_re_im(copy, NULL, copy);
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+        cpl_test_image_abs(copy, real_img, 0.0);
+
+        error = cpl_image_fill_re_im(NULL, copy, copy);
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
+
         cpl_image_delete(real_img);
         cpl_image_delete(imag_img);
         cpl_image_delete(real_copy);
@@ -854,9 +912,38 @@ int main(void)
                     cpl_image * real_copy = cpl_image_extract_real(copy);
                     cpl_image * imag_copy = cpl_image_extract_imag(copy);
 
+                    /* Test cpl_image_conjugate() and cpl_image_fill_re_im() */
+
+                    error = cpl_image_conjugate(img, img);
+                    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+                    error = cpl_image_multiply_scalar(imag_copy, -1.0);
+                    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+                    error = cpl_image_fill_re_im(real_img, imag_img, img);
+                    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
                     cpl_test_image_abs(real_img, real_copy, 0.0);
                     cpl_test_image_abs(imag_img, imag_copy, 0.0);
 
+                    error = cpl_image_fill_re_im(NULL, imag_img, img);
+                    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+                    error = cpl_image_fill_re_im(real_img, NULL, img);
+                    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+                    cpl_test_image_abs(real_img, real_copy, 0.0);
+                    cpl_test_image_abs(imag_img, imag_copy, 0.0);
+
+                    /* Test in-place */
+                    error = cpl_image_fill_re_im(copy, NULL, copy);
+                    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+                    cpl_test_image_abs(copy, real_img, 0.0);
+
+                    error = cpl_image_fill_re_im(NULL, copy, copy);
+                    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
                     cpl_image_delete(real_img);
                     cpl_image_delete(imag_img);
                     cpl_image_delete(real_copy);
@@ -890,10 +977,10 @@ int main(void)
 /**
   @internal
   @brief  Get the precision obtainable for the combined types
-  @note   Ugly, incorrect kludge for CPL_BPP_IEEE_FLOAT + CPL_TYPE_DOUBLE
+  @note   Ugly, incorrect kludge for CPL_TYPE_FLOAT + CPL_TYPE_DOUBLE
  */
 /*----------------------------------------------------------------------------*/
-static double cpl_image_get_precision(cpl_type pixeltype, cpl_type_bpp bpptype,
+static double cpl_image_get_precision(cpl_type pixeltype, cpl_type bpptype,
                                       double rel2abs)
 {
 
@@ -901,9 +988,9 @@ static double cpl_image_get_precision(cpl_type pixeltype, cpl_type_bpp bpptype,
 
     if (pixeltype == CPL_TYPE_DOUBLE) {
         switch (bpptype) {
-        case CPL_BPP_IEEE_DOUBLE:
+        case CPL_TYPE_DOUBLE:
             break;
-        case CPL_BPP_IEEE_FLOAT:
+        case CPL_TYPE_FLOAT:
             precision = FLT_EPSILON * rel2abs;
             break;
         default:
@@ -911,8 +998,8 @@ static double cpl_image_get_precision(cpl_type pixeltype, cpl_type_bpp bpptype,
         }
     } else if (pixeltype == CPL_TYPE_FLOAT) {
         switch (bpptype) {
-        case CPL_BPP_IEEE_DOUBLE:
-        case CPL_BPP_IEEE_FLOAT:
+        case CPL_TYPE_DOUBLE:
+        case CPL_TYPE_FLOAT:
             break;
         default:
             precision = 1.0;
