@@ -1,4 +1,4 @@
-/* $Id: cpl_image_resample-test.c,v 1.45 2010/11/11 09:23:18 llundin Exp $
+/* $Id: cpl_image_resample-test.c,v 1.50 2012/02/02 08:59:57 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/11/11 09:23:18 $
- * $Revision: 1.45 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2012/02/02 08:59:57 $
+ * $Revision: 1.50 $
+ * $Name: cpl-6_0 $
  */
 
 /*-----------------------------------------------------------------------------
@@ -62,6 +62,9 @@ static void cpl_image_warp_polynomial_test_turn(int, int, cpl_vector *, double);
 static void cpl_image_warp_polynomial_test_shift(int, int, int, int,
                                                  cpl_vector *, double);
 
+static void cpl_image_extract_subsample_test(void);
+static void cpl_image_get_interpolated_test(void);
+
 /*-----------------------------------------------------------------------------
                                   Main
  -----------------------------------------------------------------------------*/
@@ -77,9 +80,9 @@ int main(void)
     cpl_polynomial * py;
     cpl_vector     * xyprofile;
     const double     xyradius = 2.2; /* 2.3 gives much higher rounding ? */
-    int              expo[2];
+    cpl_size         expo[2];
     FILE           * stream;
-    int              i, j;
+    cpl_size         i, j;
 
 
     cpl_test_init(PACKAGE_BUGREPORT, CPL_MSG_WARNING);
@@ -91,6 +94,10 @@ int main(void)
 
     cpl_test_nonnull( stream );
 
+    cpl_image_get_interpolated_test();
+
+    cpl_image_extract_subsample_test();
+
     xyprofile = cpl_vector_new(1 + xyradius * CPL_KERNEL_TABSPERPIX);
     cpl_test_nonnull( xyprofile);
 
@@ -194,71 +201,6 @@ int main(void)
     cpl_image_delete(imf);
     cpl_image_delete(imd);
 
-    /* 
-     * Adding some real tests on cpl_image_extract_subsample() and 
-     * cpl_image_rebin()
-     */
-
-    {
-        cpl_image *carlo;
-        cpl_image *subcarlo;
-        cpl_image *expected;
-        int data[] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
-                     10,11,12,13,14,15,16,17,18,19,
-                     20,21,22,23,24,25,26,27,28,29,
-                     30,31,32,33,34,35,36,37,38,39,
-                     40,41,42,43,44,45,46,47,48,49,
-                     50,51,52,53,54,55,56,57,58,59};
-
-        int exp_data[] = {0, 2, 4, 6, 8,
-                         20,22,24,26,28,
-                         40,42,44,46,48};
-
-        int exp_data2[] = {0, 3, 6, 9,
-                          40,43,46,49};
-
-        int exp_data3[] = {102,120,138,
-                           222,240,258};
-
-        int exp_data4[] = {264, 328};
-
-        carlo = cpl_image_wrap_int(10, 6, data);
-        cpl_image_reject(carlo, 3, 3);
-
-        subcarlo = cpl_image_extract_subsample(carlo, 2, 2);
-        expected = cpl_image_wrap_int(5, 3, exp_data);
-
-        cpl_test_image_abs(subcarlo, expected, 0);
-
-        cpl_image_delete(subcarlo);
-        cpl_image_unwrap(expected);
-
-        subcarlo = cpl_image_extract_subsample(carlo, 3, 4);
-        expected = cpl_image_wrap_int(4, 2, exp_data2);
-
-        cpl_test_image_abs(subcarlo, expected, 0);
-
-        cpl_image_delete(subcarlo);
-        cpl_image_unwrap(expected);
-
-        subcarlo = cpl_image_rebin(carlo, 2, 2, 3, 2);
-        expected = cpl_image_wrap_int(3, 2, exp_data3);
-
-        cpl_test_image_abs(subcarlo, expected, 0);
-
-        cpl_image_delete(subcarlo);
-        cpl_image_unwrap(expected);
-
-        subcarlo = cpl_image_rebin(carlo, 1, 1, 4, 4);
-        expected = cpl_image_wrap_int(2, 1, exp_data4);
-
-        cpl_test_image_abs(subcarlo, expected, 0);
-
-        cpl_image_delete(subcarlo);
-        cpl_image_unwrap(expected);
-        cpl_image_unwrap(carlo);
-    }
-
     if (stream != stdout) cpl_test_zero( fclose(stream) );
 
     return cpl_test_end(0);
@@ -293,6 +235,7 @@ static void check_kernel(cpl_image * temp,
     /* These kernels preserve the actual pixel-values */
     cpl_kernel kernels[] = {CPL_KERNEL_TANH, CPL_KERNEL_SINC, CPL_KERNEL_SINC2,
                             CPL_KERNEL_LANCZOS, CPL_KERNEL_HANN,
+                            CPL_KERNEL_NEAREST,
                             CPL_KERNEL_HAMMING};
     /* FIXME: Integer tolerence higher (1 for int) for CPL_KERNEL_TANH */
     const double imtol0 = cpl_image_get_type(ref) == CPL_TYPE_INT ? 1.0
@@ -345,7 +288,7 @@ cpl_image_warp_polynomial_test_shift(int nx, int ny, int dx, int dy,
     cpl_polynomial * px = cpl_polynomial_new(2);
     cpl_polynomial * py = cpl_polynomial_new(2);
     cpl_mask       * mask = NULL;
-    int              expo[2];
+    cpl_size         expo[2];
     int              ityp;
 
     /* The shift: (u,v) = (x-dx, y-dy) */
@@ -433,7 +376,7 @@ cpl_image_warp_polynomial_test_turn(int nx, int ny, cpl_vector * xyprofile,
     cpl_polynomial * px = cpl_polynomial_new(2);
     cpl_polynomial * py = cpl_polynomial_new(2);
     cpl_mask       * mask = NULL;
-    int              expo[2];
+    cpl_size         expo[2];
     int              ityp;
 
     /* The rotation: (u,v) = (y, N+1-x) */
@@ -493,3 +436,183 @@ cpl_image_warp_polynomial_test_turn(int nx, int ny, cpl_vector * xyprofile,
 
     return;
 }
+
+/*----------------------------------------------------------------------------*/
+/**
+  @internal
+  @brief    Test the CPL function
+  @return   void
+  @see cpl_image_extract_subsample(), cpl_image_rebin()
+  @note Adding some real tests
+
+*/
+/*----------------------------------------------------------------------------*/
+static void cpl_image_extract_subsample_test(void)
+{
+    cpl_image *carlo;
+    cpl_image *subcarlo;
+    cpl_image *expected;
+    int data[] = {0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
+                  10,11,12,13,14,15,16,17,18,19,
+                  20,21,22,23,24,25,26,27,28,29,
+                  30,31,32,33,34,35,36,37,38,39,
+                  40,41,42,43,44,45,46,47,48,49,
+                  50,51,52,53,54,55,56,57,58,59};
+
+    int exp_data[] = {0, 2, 4, 6, 8,
+                      20,22,24,26,28,
+                      40,42,44,46,48};
+
+    int exp_data2[] = {0, 3, 6, 9,
+                       40,43,46,49};
+
+    int exp_data3[] = {102,120,138,
+                       222,240,258};
+
+    int exp_data4[] = {264, 328};
+
+    cpl_error_code error;
+
+    carlo = cpl_image_wrap_int(10, 6, data);
+    cpl_test_error(CPL_ERROR_NONE);
+    cpl_test_nonnull(carlo);
+
+    error = cpl_image_reject(carlo, 3, 3);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+    subcarlo = cpl_image_extract_subsample(carlo, 2, 2);
+    cpl_test_error(CPL_ERROR_NONE);
+    cpl_test_nonnull(subcarlo);
+    expected = cpl_image_wrap_int(5, 3, exp_data);
+    cpl_test_error(CPL_ERROR_NONE);
+    cpl_test_nonnull(expected);
+
+    cpl_test_image_abs(subcarlo, expected, 0);
+
+    cpl_image_delete(subcarlo);
+    cpl_test_eq_ptr(cpl_image_unwrap(expected), exp_data);
+
+    subcarlo = cpl_image_extract_subsample(carlo, 3, 4);
+    cpl_test_error(CPL_ERROR_NONE);
+    cpl_test_nonnull(subcarlo);
+    expected = cpl_image_wrap_int(4, 2, exp_data2);
+    cpl_test_error(CPL_ERROR_NONE);
+    cpl_test_nonnull(expected);
+
+    cpl_test_image_abs(subcarlo, expected, 0);
+
+    cpl_image_delete(subcarlo);
+    cpl_test_eq_ptr(cpl_image_unwrap(expected), exp_data2);
+
+    subcarlo = cpl_image_rebin(carlo, 2, 2, 3, 2);
+    cpl_test_error(CPL_ERROR_NONE);
+    cpl_test_nonnull(subcarlo);
+    expected = cpl_image_wrap_int(3, 2, exp_data3);
+    cpl_test_error(CPL_ERROR_NONE);
+    cpl_test_nonnull(expected);
+
+    cpl_test_image_abs(subcarlo, expected, 0);
+
+    cpl_image_delete(subcarlo);
+    cpl_test_eq_ptr(cpl_image_unwrap(expected), exp_data3);
+
+    subcarlo = cpl_image_rebin(carlo, 1, 1, 4, 4);
+    cpl_test_error(CPL_ERROR_NONE);
+    cpl_test_nonnull(subcarlo);
+    expected = cpl_image_wrap_int(2, 1, exp_data4);
+    cpl_test_error(CPL_ERROR_NONE);
+    cpl_test_nonnull(expected);
+
+    cpl_test_image_abs(subcarlo, expected, 0);
+
+    cpl_image_delete(subcarlo);
+    cpl_test_eq_ptr(cpl_image_unwrap(expected), exp_data4);
+    cpl_test_eq_ptr(cpl_image_unwrap(carlo), data);
+
+}
+
+/*----------------------------------------------------------------------------*/
+/**
+  @internal
+  @brief    Test the CPL function
+  @return   void
+  @see cpl_image_get_interpolated()
+
+*/
+/*----------------------------------------------------------------------------*/
+static void cpl_image_get_interpolated_test(void)
+{
+
+    const cpl_type ttype[] = {CPL_TYPE_INT, CPL_TYPE_FLOAT, CPL_TYPE_DOUBLE};
+    const cpl_size tsize[] = {50, 500};
+    const double value1[] = {2, 3};
+    const double value2[] = {5, 7};
+    const double mean = (value1[0] + value1[1] + value2[0] + value2[1])/4.0;
+    const double txyrad[] = {-1.0, 0.0, CPL_KERNEL_DEF_WIDTH, 24.0, 100.0,
+                             240.0};
+    cpl_vector * xyprofile = cpl_vector_new(CPL_KERNEL_DEF_SAMPLES);
+    size_t isize;
+
+    for (isize = 0; isize < sizeof(tsize)/sizeof(tsize[0]); isize++) {
+        size_t itype;
+        const cpl_size imsize = tsize[isize];
+        const double   c      = 0.5 + (double)imsize / 2.0;
+        cpl_size i, j;
+        cpl_error_code error;
+        cpl_image * dimage = cpl_image_new(imsize, imsize, CPL_TYPE_DOUBLE);
+
+        cpl_test_nonnull(dimage);
+
+        for (j = 0; j < imsize; j++) {
+            const double * pvalue = 2 * j < imsize ? value1 : value2;
+            for (i = 0; i < imsize; i++) {
+                (void)cpl_image_set(dimage, i+1, j+1,
+                                      pvalue[2 * i < imsize ? 0 : 1]);
+            }
+        }
+
+        for (itype = 0; itype < sizeof(ttype)/sizeof(ttype[0]); itype++) {
+            size_t ixyrad;
+            const cpl_type imtype = ttype[itype];
+
+            cpl_image * image = imtype == CPL_TYPE_DOUBLE ? dimage :
+                cpl_image_cast(dimage, imtype);
+
+            for (ixyrad = 0; ixyrad < sizeof(txyrad)/sizeof(txyrad[0]);
+                 ixyrad++) {
+                const double xyradius = txyrad[ixyrad];
+                double value, confidence;
+
+                cpl_msg_info(cpl_func, "Interpolating from %d X %d %s-image, "
+                             "r=%g", (int)imsize, (int)imsize, 
+                             cpl_type_get_name(imtype), xyradius);
+ 
+                error = cpl_vector_fill_kernel_profile(xyprofile,
+                                                       CPL_KERNEL_DEFAULT,
+                                                       xyradius);
+                cpl_test_eq_error(error, xyradius > 0.0 ? CPL_ERROR_NONE
+                                  : CPL_ERROR_ILLEGAL_INPUT);
+
+                value = cpl_image_get_interpolated(image, c, c, xyprofile,
+                                                   xyradius, xyprofile,
+                                                   xyradius, &confidence);
+                if (xyradius > 0.0) {
+                    cpl_test_eq_error(error, CPL_ERROR_NONE);
+                    cpl_test_abs(value, mean, 4.0 * xyradius * DBL_EPSILON);
+                    if (xyradius < imsize) {
+                        cpl_test_abs(confidence, 1.0, 30.0 * DBL_EPSILON);
+                    } else {
+                        cpl_test_abs(confidence, 1.0, xyradius * DBL_EPSILON);
+                    }
+                } else {
+                    cpl_test_eq_error(error, CPL_ERROR_ILLEGAL_INPUT);
+                }
+            }
+            if (image != dimage)
+                cpl_image_delete(image);
+        }
+        cpl_image_delete(dimage);
+    }
+
+    cpl_vector_delete(xyprofile);
+}
