@@ -1,4 +1,4 @@
-/* $Id: cpl_imagelist_io-test.c,v 1.48 2011/01/21 14:17:10 llundin Exp $
+/* $Id: cpl_imagelist_io-test.c,v 1.51 2011/08/17 14:44:45 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,16 +20,16 @@
 
 /*
  * $Author: llundin $
- * $Date: 2011/01/21 14:17:10 $
- * $Revision: 1.48 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2011/08/17 14:44:45 $
+ * $Revision: 1.51 $
+ * $Name: cpl-6_0 $
  */
 
 /*-----------------------------------------------------------------------------
                                    Includes
  -----------------------------------------------------------------------------*/
 
-#include "cpl_imagelist_io_impl.h"
+#include "cpl_imagelist_io.h"
 #include "cpl_image_gen.h"
 #include "cpl_image_io.h"
 #include "cpl_vector.h"
@@ -57,10 +57,10 @@ int main(void)
     cpl_image     * nullimg;
     double          flags[NIMAGES];
     cpl_vector    * eraser;
-    int             i;
+    cpl_size        i;
     FILE          * stream;
-    int             boxsize = IMAGE_SZ > 127 ? 5 :
-    ( IMAGE_SZ < 25 ? 1 : (int)(IMAGE_SZ / 25) );
+    cpl_size        boxsize = IMAGE_SZ > 127 ? 5 :
+    ( IMAGE_SZ < 25 ? 1 : (IMAGE_SZ / 25) );
     cpl_error_code  error;
 
 
@@ -135,15 +135,15 @@ int main(void)
     cpl_test_error( CPL_ERROR_NULL_INPUT );
     cpl_test_null( nulllist );
 
-    error = cpl_imagelist_save(imlist, NULL,  CPL_BPP_IEEE_FLOAT, NULL,
+    error = cpl_imagelist_save(imlist, NULL,  CPL_TYPE_FLOAT, NULL,
                                CPL_IO_CREATE);
     cpl_test_eq_error( error, CPL_ERROR_NULL_INPUT );
 
-    error = cpl_imagelist_save(NULL,  file, CPL_BPP_IEEE_FLOAT, NULL,
+    error = cpl_imagelist_save(NULL,  file, CPL_TYPE_FLOAT, NULL,
                                CPL_IO_CREATE);
     cpl_test_eq_error( error, CPL_ERROR_NULL_INPUT );
 
-    error = cpl_imagelist_save(imlist, file,  CPL_BPP_IEEE_FLOAT, NULL,
+    error = cpl_imagelist_save(imlist, file,  CPL_TYPE_FLOAT, NULL,
                                CPL_IO_CREATE);
     cpl_test_eq_error( error, CPL_ERROR_ILLEGAL_INPUT );
 
@@ -207,11 +207,11 @@ int main(void)
     cpl_imagelist_dump_structure(imlist, stream);
 
     cpl_imagelist_dump_window(imlist, 
-                  (int) (IMAGE_SZ / 2) - boxsize,
-                  (int) (IMAGE_SZ / 2) - boxsize,
-                  (int) (IMAGE_SZ / 2) + boxsize,
-                  (int) (IMAGE_SZ / 2) + boxsize,
-                  stream);
+                              (IMAGE_SZ / 2) - boxsize,
+                              (IMAGE_SZ / 2) - boxsize,
+                              (IMAGE_SZ / 2) + boxsize,
+                              (IMAGE_SZ / 2) + boxsize,
+                              stream);
 
     cpl_msg_info("", "Duplicate the image list");
 
@@ -222,20 +222,20 @@ int main(void)
     cpl_test_eq( cpl_imagelist_get_size(imlist2), NIMAGES );
 
     /* Failure tests involving non-empty, valid list */
-    error = cpl_imagelist_save(imlist2, ".",  CPL_BPP_IEEE_FLOAT, NULL,
+    error = cpl_imagelist_save(imlist2, ".",  CPL_TYPE_FLOAT, NULL,
                                CPL_IO_CREATE);
     cpl_test_eq_error( error, CPL_ERROR_FILE_IO );
 
     remove(file);
-    error = cpl_imagelist_save(imlist2, file,  CPL_BPP_IEEE_FLOAT, NULL,
+    error = cpl_imagelist_save(imlist2, file,  CPL_TYPE_FLOAT, NULL,
                                CPL_IO_APPEND);
     cpl_test_eq_error( error, CPL_ERROR_FILE_IO );
 
-    error = cpl_imagelist_save(imlist2, file,  CPL_BPP_IEEE_FLOAT, NULL,
+    error = cpl_imagelist_save(imlist2, file,  CPL_TYPE_FLOAT, NULL,
                                CPL_IO_CREATE | CPL_IO_EXTEND);
     cpl_test_eq_error( error, CPL_ERROR_ILLEGAL_INPUT );
 
-    error = cpl_imagelist_save(imlist2, file,  CPL_BPP_IEEE_FLOAT, NULL,
+    error = cpl_imagelist_save(imlist2, file,  CPL_TYPE_FLOAT, NULL,
                                CPL_IO_CREATE | CPL_IO_EXTEND | CPL_IO_APPEND);
     cpl_test_eq_error( error, CPL_ERROR_ILLEGAL_INPUT );
 
@@ -297,9 +297,9 @@ int main(void)
 
     /* Test cpl_imagelist_save() */
     cpl_msg_info("", "Save the image list to %s", file);
-    cpl_test_zero( cpl_imagelist_save(imlist, file,  CPL_BPP_IEEE_FLOAT, NULL,
+    cpl_test_zero( cpl_imagelist_save(imlist, file,  CPL_TYPE_FLOAT, NULL,
                                           CPL_IO_CREATE) );
-    cpl_test_zero( cpl_imagelist_save(imlist, file,  CPL_BPP_IEEE_FLOAT, NULL,
+    cpl_test_zero( cpl_imagelist_save(imlist, file,  CPL_TYPE_FLOAT, NULL,
                                           CPL_IO_EXTEND) );
 
     /* cpl_image_load(): Test loading of NAXIS=3 data (DFS09929) */
@@ -316,14 +316,14 @@ int main(void)
     cpl_test_null( nulllist );
 
     /* Append error test */
-    error = cpl_imagelist_save(imlist, file, CPL_BPP_IEEE_DOUBLE, NULL,
+    error = cpl_imagelist_save(imlist, file, CPL_TYPE_DOUBLE, NULL,
                                CPL_IO_APPEND);
     cpl_test_eq_error(error, CPL_ERROR_ILLEGAL_INPUT);
 
     image = cpl_image_new(IMAGE_SZ+1,IMAGE_SZ+1, CPL_TYPE_FLOAT);
     imlist2 = cpl_imagelist_new();
     cpl_imagelist_set(imlist2, image, 0);
-    error = cpl_imagelist_save(imlist2, file, CPL_BPP_IEEE_FLOAT, NULL,
+    error = cpl_imagelist_save(imlist2, file, CPL_TYPE_FLOAT, NULL,
                                CPL_IO_APPEND);
     cpl_test_eq_error(error, CPL_ERROR_ILLEGAL_INPUT);
     cpl_imagelist_delete(imlist2);
@@ -336,7 +336,7 @@ int main(void)
     cpl_test_imagelist_abs(imlist, imlist2, 10.0 * FLT_EPSILON);
     cpl_imagelist_delete(imlist2);
 
-    cpl_test_zero( cpl_imagelist_save(imlist, file,  CPL_BPP_IEEE_FLOAT, NULL,
+    cpl_test_zero( cpl_imagelist_save(imlist, file,  CPL_TYPE_FLOAT, NULL,
                                       CPL_IO_APPEND) );
 
     imlist2 = cpl_imagelist_load(file, CPL_TYPE_UNSPECIFIED, 1);
