@@ -1,4 +1,4 @@
-/* $Id: cpl_mask-test.c,v 1.64 2010/11/17 12:49:35 llundin Exp $
+/* $Id: cpl_mask-test.c,v 1.71 2011/11/22 14:50:30 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,27 +20,24 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/11/17 12:49:35 $
- * $Revision: 1.64 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2011/11/22 14:50:30 $
+ * $Revision: 1.71 $
+ * $Name: cpl-6_0 $
  */
 
 /*-----------------------------------------------------------------------------
                                    Includes
  -----------------------------------------------------------------------------*/
 
-#include <string.h>
-#include <math.h>
-
 #include "cpl_mask.h"
 #include "cpl_tools.h"
-#include "cpl_image_gen.h"
-#include "cpl_stats.h"
-#include "cpl_image_io.h"
 #include "cpl_memory.h"
-#include "cpl_msg.h"
+#include "cpl_fits.h"
 #include "cpl_test.h"
 
+#include <string.h>
+#include <math.h>
+
 /*-----------------------------------------------------------------------------
                                   Defines
  -----------------------------------------------------------------------------*/
@@ -53,6 +50,8 @@
 #define MASKSZ 8
 #endif
 
+#define FILENAME "cpl_mask_test.fits"
+
 /*-----------------------------------------------------------------------------
                                   Private functions
  -----------------------------------------------------------------------------*/
@@ -60,6 +59,7 @@
 static cpl_error_code cpl_mask_shift_bf(cpl_mask *, int, int, cpl_binary);
 static cpl_error_code cpl_mask_shift_filter(cpl_mask *, int, int,
                                             cpl_filter_mode);
+static void cpl_mask_flip_turn_test(const cpl_mask *);
 static void cpl_mask_shift_test(int, int);
 static void cpl_mask_test(cpl_boolean);
 static void cpl_mask_filter_test(int);
@@ -109,9 +109,9 @@ int main(void)
     cpl_image       *   labels1,
                     *   labels2;
     int                 count;
-    int                 nb_objs1,
+    cpl_size            nb_objs1,
                         nb_objs2;
-    const int           new_pos[] = {1,2,3,4};
+    const cpl_size      new_pos[] = {1,2,3,4};
     const int           nsize = IMAGESZ;
     cpl_binary      *   bins;
     cpl_error_code      error;
@@ -177,6 +177,8 @@ int main(void)
 
     cpl_test_zero(cpl_mask_is_empty(bin1));
 
+    cpl_mask_flip_turn_test(bin1);
+
     /* Test wrap / unwrap */
     bins = cpl_calloc(100, sizeof(cpl_binary));
     bin2 = cpl_mask_wrap(10, 10, bins);
@@ -273,21 +275,46 @@ int main(void)
     cpl_test_eq(cpl_mask_is_empty(bin1_dup), CPL_TRUE);
     cpl_mask_delete(bin1_dup);
 
+    error = cpl_mask_save(bin1, FILENAME, NULL, CPL_IO_CREATE);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+    error = cpl_mask_save(bin1, FILENAME, NULL, CPL_IO_EXTEND);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+    cpl_test_fits(FILENAME);
+    cpl_test_eq(cpl_fits_count_extensions(FILENAME), 1);
+
+    bin1_dup = cpl_mask_load(FILENAME, 0, 1);
+    cpl_test_error(CPL_ERROR_NONE);
+
+    cpl_test_eq_mask(bin1, bin1_dup);
+    cpl_mask_delete(bin1_dup);
+
+    bin1_dup = cpl_mask_load_window(FILENAME, 0, 1, 1, 1, nsize, nsize );
+    cpl_test_error(CPL_ERROR_NONE);
+
+    cpl_test_eq_mask(bin1, bin1_dup);
+    cpl_mask_delete(bin1_dup);
+
+    cpl_test_zero( remove(FILENAME) );
+
     /* Labelize the binary maps */
     labels1 = cpl_image_labelise_mask_create(bin1, &nb_objs1);
     cpl_test_nonnull(labels1);
 
-    cpl_msg_info("","Labelize bin1 ---> %d objects found", nb_objs1);
+    cpl_msg_info("","Labelize bin1 ---> %" CPL_SIZE_FORMAT " objects found",
+                 nb_objs1);
     labels2 = cpl_image_labelise_mask_create(bin2, &nb_objs2);
     cpl_test_nonnull(labels2);
 
-    cpl_msg_info("","Labelize bin2 ---> %d objects found", nb_objs2);
+    cpl_msg_info("","Labelize bin2 ---> %" CPL_SIZE_FORMAT " objects found",
+                 nb_objs2);
    
     /* sprintf(outname, "labelized1.fits"); */
-    /* cpl_image_save(labels1, outname, CPL_BPP_IEEE_FLOAT, NULL,
+    /* cpl_image_save(labels1, outname, CPL_TYPE_FLOAT, NULL,
                       CPL_IO_CREATE); */
     /* sprintf(outname, "labelized2.fits"); */
-    /* cpl_image_save(labels2, outname, CPL_BPP_IEEE_FLOAT, NULL,
+    /* cpl_image_save(labels2, outname, CPL_TYPE_FLOAT, NULL,
                       CPL_IO_CREATE); */
     
     cpl_image_delete(labels1);
@@ -1298,6 +1325,8 @@ static void cpl_mask_filter_test_schalkoff(void)
     cpl_msg_info(cpl_func, "Schalkoff filter test on " CPL_STRINGIFY(nx) " x "
                  CPL_STRINGIFY(ny) " mask");
 
+    cpl_mask_flip_turn_test(maskA);
+
     /* Fill 3x3 kernel */
     cpl_mask_set(kernel, 2, 2, CPL_BINARY_1);
     cpl_mask_set(kernel, 2, 3, CPL_BINARY_1);
@@ -2468,3 +2497,58 @@ void cpl_mask_closing_bf(cpl_binary * self, const cpl_binary * other,
     cpl_mask_erosion_bf(self,   middle, kernel, nx, ny, hx, hy, border);
     cpl_free(middle);
 }
+
+
+/*----------------------------------------------------------------------------*/
+/**
+  @internal
+  @brief  Test cpl_mask_flip(), cpl_mask_turn()
+  @param  self    The mask to test
+  @return void
+
+   A: 
+       Rotate 90 counterclock-wise, then
+       flip around the vertical axis, then
+       flip around x=y, and
+       it should be back to its original.
+
+   B: 
+
+       Rotate 90 counterclock-wise, then
+       flip around the horizontal axis, then
+       flip around x=-y, and
+       it should be back to its original.
+
+
+ */
+/*----------------------------------------------------------------------------*/
+static void cpl_mask_flip_turn_test(const cpl_mask * self)
+{
+
+    cpl_error_code error;
+    cpl_mask * tmp = cpl_mask_duplicate(self);
+
+    error = cpl_mask_turn(tmp, -1);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+    error = cpl_mask_flip(tmp, 2);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+    error = cpl_mask_flip(tmp, 1);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+    cpl_test_eq_mask(self, tmp);
+
+    error = cpl_mask_turn(tmp, -1);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+    error = cpl_mask_flip(tmp, 0);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+    error = cpl_mask_flip(tmp, 3);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+    cpl_test_eq_mask(self, tmp);
+
+    cpl_mask_delete(tmp);
+}
