@@ -1,4 +1,4 @@
-/* $Id: cpl_matrix-test.c,v 1.52 2011/02/17 12:49:50 llundin Exp $
+/* $Id: cpl_matrix-test.c,v 1.55 2011/12/13 14:42:11 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2011/02/17 12:49:50 $
- * $Revision: 1.52 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2011/12/13 14:42:11 $
+ * $Revision: 1.55 $
+ * $Name: cpl-6_0 $
  */
 
 /*-----------------------------------------------------------------------------
@@ -122,16 +122,18 @@
                         Private function prototypes
  -----------------------------------------------------------------------------*/
 
+static void cpl_matrix_test_banded(int, int);
+
 static cpl_error_code cpl_matrix_fill_illcond(cpl_matrix *, int);
-static cpl_error_code cpl_matrix_fill_test(cpl_matrix *, int, int);
+static cpl_error_code cpl_matrix_fill_test(cpl_matrix *, cpl_size, cpl_size);
 static double cpl_matrix_get_2norm_1(const cpl_matrix *);
 
 static void cpl_matrix_product_transpose_bench(int, int);
 static void cpl_matrix_solve_normal_bench(int, int);
-static void cpl_matrix_product_bilinear_test(int, int, cpl_boolean);
+static void cpl_matrix_product_bilinear_test(cpl_size, cpl_size, cpl_boolean);
 static cpl_matrix * cpl_matrix_product_bilinear_brute(const cpl_matrix *,
                                                       const cpl_matrix *);
-static cpl_error_code cpl_matrix_shift_brute(cpl_matrix *, int, int);
+static cpl_error_code cpl_matrix_shift_brute(cpl_matrix *, cpl_size, cpl_size);
 
 /*-----------------------------------------------------------------------------
                               main 
@@ -185,8 +187,8 @@ int main(void)
     double     *dArray;
     double     *subdArray;
     double      value;
-    int         i, j, k, l;
-    int         r, c;
+    cpl_size    i, j, k, l;
+    cpl_size    r, c;
 
     cpl_matrix  *matrix;
     cpl_matrix  *copia;
@@ -214,6 +216,9 @@ int main(void)
 
     cpl_test_nonnull( stream );
 
+    cpl_matrix_test_banded(10, 1);
+    cpl_matrix_test_banded(10, 2);
+
     for (i = 1; i <= 11; i++) {
         for (j = 1; j <= 11; j++) {
             cpl_matrix_product_bilinear_test(i, j, CPL_FALSE);
@@ -354,7 +359,7 @@ int main(void)
 
   cpl_matrix_delete(matrix);
 
-  dArray = cpl_malloc(nelem * sizeof(double));
+  dArray = cpl_malloc((size_t)nelem * sizeof(double));
 
   value = 9.00;
   for (i = 0; i < nelem; i++) {
@@ -362,7 +367,7 @@ int main(void)
       value += .01;
   }
 
-  subdArray = cpl_malloc(subnelem * sizeof(double));
+  subdArray = cpl_malloc((size_t)subnelem * sizeof(double));
 
   value = 1.00; 
   for (i = 0; i < subnelem; i++) {
@@ -625,7 +630,7 @@ int main(void)
       test_fvalue(1.11, 0.00001, cpl_matrix_get(matrix, i, i), message);
   }
 
-  test(cpl_matrix_fill_diagonal(matrix, 1.10, 1), 
+  test(cpl_matrix_fill_diagonal(matrix, 1.10, -1),
                     "Write 1.11 to second diagonal...");
 
   for (i = 0; i < c; i++) {
@@ -905,7 +910,7 @@ int main(void)
   cpl_matrix_delete(copia);
   cpl_matrix_delete(matrix);
 
-  dArray = cpl_malloc(nelem * sizeof(double));
+  dArray = cpl_malloc((size_t)nelem * sizeof(double));
 
   value = 9.00;
   for (i = 0; i < nelem; i++) {
@@ -1058,7 +1063,7 @@ int main(void)
   cpl_matrix_delete(copia);
   cpl_matrix_delete(matrix);
 
-  dArray = cpl_malloc(nelem * sizeof(double));
+  dArray = cpl_malloc((size_t)nelem * sizeof(double));
 
   value = 9.00;
   for (i = 0; i < nelem; i++) {
@@ -1106,7 +1111,7 @@ int main(void)
   cpl_matrix_delete(copia);
   cpl_matrix_delete(matrix);
 
-  dArray = cpl_malloc(nelem * sizeof(double));
+  dArray = cpl_malloc((size_t)nelem * sizeof(double));
 
   value = 9.00;
   for (i = 0; i < nelem; i++) {
@@ -1639,17 +1644,19 @@ static cpl_error_code cpl_matrix_fill_illcond(cpl_matrix * self, int size)
     cpl_ensure_code(self,     CPL_ERROR_NULL_INPUT);
     cpl_ensure_code(size > 0, CPL_ERROR_ILLEGAL_INPUT);
 
-    cpl_ensure_code(!cpl_matrix_set_size(self, size, size), cpl_error_get_code());
+    cpl_ensure_code(!cpl_matrix_set_size(self, size, size),
+                    cpl_error_get_code());
 
     for (i = 0; i<size; i++) {
         for (j = 0; j<size; j++) {
             /*
-               The 'usual' definition of this increasingly ill-conditioned matrix is
+               The 'usual' definition of this increasingly ill-conditioned
+               matrix is
                A(i,j) = 1/(1+i+j)
                - but to expose direct solvers without pivoting, we use
                A(i,j) = 1/(2*size - (1+i+j))
             */
-            const double value = 1.0/(2*size-(i+j+1));
+            const double value = 1.0/(double)(2*size-(i+j+1));
             cpl_ensure_code(!cpl_matrix_set(self, i, j, value),
                             cpl_error_get_code());
         }
@@ -1675,10 +1682,11 @@ static cpl_error_code cpl_matrix_fill_illcond(cpl_matrix * self, int size)
   - CPL_ERROR_ILLEGAL_INPUT if size is negative or zero
  */
 /*----------------------------------------------------------------------------*/
-static cpl_error_code cpl_matrix_fill_test(cpl_matrix * self, int nr, int nc)
+static cpl_error_code cpl_matrix_fill_test(cpl_matrix * self, cpl_size nr,
+                                           cpl_size nc)
 {
 
-    int i,j;
+    cpl_size i,j;
 
     cpl_ensure_code(self,   CPL_ERROR_NULL_INPUT);
     cpl_ensure_code(nr > 0, CPL_ERROR_ILLEGAL_INPUT);
@@ -1688,7 +1696,7 @@ static cpl_error_code cpl_matrix_fill_test(cpl_matrix * self, int nr, int nc)
 
     for (i = 0; i < nr; i++) {
         for (j = 0; j < nc; j++) {
-            const double value = 1.0/(1+abs(i-j));
+            const double value = 1.0/(double)(1+labs(i-j));
             cpl_ensure_code(!cpl_matrix_set(self, i, j, value),
                             cpl_error_get_code());
         }
@@ -1780,7 +1788,7 @@ static void cpl_matrix_solve_normal_bench(int nsize, int n)
 
     cpl_msg_info("","Speed while solving with size %d X %d in %g secs "
                  "[Mflop/s]: %g (%g)", nsize, nsize, secs/(double)i,
-                 flops0/secs/1e6,(double)flops0);
+                 (double)flops0/secs/1e6, (double)flops0);
     if (secs > 0.0) {
         cpl_msg_info(cpl_func,"Processing rate [MB/s]: %g",
                      1e-6 * (double)bytes / secs);
@@ -1888,12 +1896,13 @@ static cpl_matrix * cpl_matrix_product_bilinear_brute(const cpl_matrix * A,
   
 */
 /*----------------------------------------------------------------------------*/
-static cpl_error_code cpl_matrix_shift_brute(cpl_matrix *matrix, int rshift,
-                                             int cshift)
+static cpl_error_code cpl_matrix_shift_brute(cpl_matrix *matrix,
+                                             cpl_size rshift,
+                                             cpl_size cshift)
 {
     cpl_error_code error = CPL_ERROR_NONE;
-    const int nr = cpl_matrix_get_nrow(matrix);
-    const int nc = cpl_matrix_get_ncol(matrix);
+    const cpl_size nr = cpl_matrix_get_nrow(matrix);
+    const cpl_size nc = cpl_matrix_get_ncol(matrix);
 
     if (matrix == NULL)
         return cpl_error_set_(CPL_ERROR_NULL_INPUT);
@@ -1945,7 +1954,8 @@ static cpl_error_code cpl_matrix_shift_brute(cpl_matrix *matrix, int rshift,
  */
 /*----------------------------------------------------------------------------*/
 static
-void cpl_matrix_product_bilinear_test(int nr, int nc, cpl_boolean do_bench)
+void cpl_matrix_product_bilinear_test(cpl_size nr, cpl_size nc,
+                                      cpl_boolean do_bench)
 {
 
     cpl_matrix   * self   = cpl_matrix_new(nr, nr);
@@ -1987,12 +1997,50 @@ void cpl_matrix_product_bilinear_test(int nr, int nc, cpl_boolean do_bench)
     cpl_matrix_delete(brute);
 
     if (do_bench) {
-        cpl_msg_info(cpl_func,"Time spent on B * A * B', B: %dx%d "
-                     "[s]: %g <=> %g (brute force)", nr, nc, tfunc, tbrute);
+        cpl_msg_info(cpl_func,"Time spent on B * A * B', B: %" CPL_SIZE_FORMAT
+                     "x%" CPL_SIZE_FORMAT " [s]: %g <=> %g (brute force)",
+                     nr, nc, tfunc, tbrute);
         if (tfunc > 0.0 && tbrute > 0.0) {
-            cpl_msg_info(cpl_func,"Speed while computing B * A * B', B: %dx%d "
-                         "[Mflop/s]: %g <=> %g (brute force)", nr, nc,
-                         ffunc/tfunc/1e6, fbrute/tbrute/1e6);
+            cpl_msg_info(cpl_func,"Speed while computing B * A * B', B: %"
+                         CPL_SIZE_FORMAT "x%" CPL_SIZE_FORMAT " [Mflop/s]: "
+                         "%g <=> %g (brute force)", nr, nc,
+                         (double)ffunc/tfunc/1e6, (double)fbrute/tbrute/1e6);
         }
     }
 }
+
+
+
+/*----------------------------------------------------------------------------*/
+/**
+  @internal
+  @brief    LU factor a banded matrix
+  @param nc       The number of matrix columns/rows
+  @param np       The bandedness
+  @return   void
+ */
+/*----------------------------------------------------------------------------*/
+static void cpl_matrix_test_banded(int nc, int np)
+{
+    cpl_matrix * A = cpl_matrix_new(nc, nc);
+    cpl_array  * p = cpl_array_new(nc, CPL_TYPE_INT);
+    int i;
+    cpl_error_code error;
+
+    for (i = -np; i <= np; i++) {
+        error = cpl_matrix_fill_diagonal(A, 1.0, i);
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
+    }
+
+    if(cpl_msg_get_level() <= CPL_MSG_INFO)
+        cpl_matrix_dump(A, stderr);
+
+    error = cpl_matrix_decomp_lu(A, p, &i);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+    if(cpl_msg_get_level() <= CPL_MSG_INFO)
+        cpl_array_dump(p, 0, nc, stderr);
+
+    cpl_matrix_delete(A);
+    cpl_array_delete(p);
+}
