@@ -1,4 +1,4 @@
-/* $Id: cpl_memory-test.c,v 1.26 2010/11/11 09:23:18 llundin Exp $
+/* $Id: cpl_memory-test.c,v 1.27 2011/12/21 13:15:31 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/11/11 09:23:18 $
- * $Revision: 1.26 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2011/12/21 13:15:31 $
+ * $Revision: 1.27 $
+ * $Name: cpl-6_0 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -32,10 +32,14 @@
 /*-----------------------------------------------------------------------------
                                 Includes
  -----------------------------------------------------------------------------*/
-#include <string.h>
 
 #include "cpl_memory.h"
+
 #include "cpl_test.h"
+#include "cpl_io_fits.h"
+#include "cpl_tools.h"
+
+#include <string.h>
 
 /*-----------------------------------------------------------------------------
                                 Defines
@@ -93,10 +97,21 @@ int main(void)
 
     cpl_test_init(PACKAGE_BUGREPORT, CPL_MSG_WARNING);
 
+    /* Need to disable the (unused) FITS caching so no memory is allocated */
+    (void)cpl_io_fits_end();
+
     do_bench = cpl_msg_get_level() <= CPL_MSG_INFO ? CPL_TRUE : CPL_FALSE;
 
     /* Insert tests below */
 
+#if defined CPL_XMEMORY_MODE
+    cpl_msg_info(cpl_func, CPL_XSTRINGIFY(CPL_XMEMORY_MODE) ": "
+                 CPL_STRINGIFY(CPL_XMEMORY_MODE) "\n");
+#else
+    cpl_msg_info(cpl_func, "CPL_XMEMORY_MODE is not defined\n");
+#endif
+
+
     /* Test cpl_malloc(), cpl_calloc, cpl_realloc(), cpl_free() */
     {
         const size_t size = 997;
@@ -212,7 +227,7 @@ int main(void)
     {
 
         const char * string = __FILE__; /* Some non-empty string */
-        const char * null; /* This string is supposed to always be NULL */
+        char * null; /* This string is supposed to always be NULL */
         const char * null2; /* This string is supposed to always be NULL */
         char * nonnull; /* String is supposed to never be NULL */
 
@@ -245,7 +260,7 @@ int main(void)
             cpl_test_error(CPL_ERROR_NONE);
             cpl_msg_warning(cpl_func, "The supposedly illegal format '%s' "
                             "produced the string: '%s'", nonnull, null);
-            cpl_free((char*)null);
+            cpl_free(null);
             null = NULL;
         } else {
             cpl_test_error(CPL_ERROR_ILLEGAL_INPUT);
