@@ -1,4 +1,4 @@
-/* $Id: cpl_polynomial-test.c,v 1.108 2010/11/11 09:23:18 llundin Exp $
+/* $Id: cpl_polynomial-test.c,v 1.112 2012/02/07 10:10:37 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/11/11 09:23:18 $
- * $Revision: 1.108 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2012/02/07 10:10:37 $
+ * $Revision: 1.112 $
+ * $Name: cpl-6_0 $
  */
 
 /*-----------------------------------------------------------------------------
@@ -40,7 +40,7 @@
 #include <math.h>
 
 #include "cpl_polynomial.h"
-#include "cpl_error.h"
+#include "cpl_error_impl.h"
 #include "cpl_test.h"
 #include "cpl_tools.h"
 #include "cpl_memory.h"
@@ -59,7 +59,7 @@
                                   Pricate functions
  -----------------------------------------------------------------------------*/
 
-static void cpl_polynomial_fit_bench_2d(int, int, int);
+static void cpl_polynomial_fit_bench_2d(cpl_size, cpl_size, cpl_size);
 
 static void cpl_polynomial_fit_test_1d(FILE *);
 static cpl_polynomial * cpl_polynomial_fit_test_2d(FILE *, cpl_boolean);
@@ -77,8 +77,8 @@ static cpl_error_code cpl_polynomial_fit_cmp(cpl_polynomial    *,
                                              const cpl_vector  *,
                                              const cpl_vector  *,
                                              cpl_boolean        ,
-                                             const int         *,
-                                             const int         *);
+                                             const cpl_size    *,
+                                             const cpl_size    *);
 
 static cpl_flops  mse_flops = 0;
 static double     mse_secs  = 0.0;
@@ -103,9 +103,9 @@ int main(void)
     double              xmax = 0; /* Maximum rounding error on x */
     double              ymax = 0; /* Maximum rounding error on y */
     double              eps;
-    int                 expo[POLY_DIM];
-    int                 i, j;
-    int                 k;
+    cpl_size            expo[POLY_DIM];
+    cpl_size            i, j;
+    cpl_size            k;
     cpl_error_code      error;
     FILE              * stream;
 
@@ -138,7 +138,11 @@ int main(void)
     poly1 = cpl_polynomial_new(1);
     cpl_polynomial_dump(poly1, stream);
 
-    cpl_test_eq( cpl_polynomial_get_degree(poly1), 0);
+    cpl_test_zero( cpl_polynomial_get_degree(poly1));
+
+    error = cpl_polynomial_multiply_scalar(poly1, poly1, 0.0);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    cpl_test_zero( cpl_polynomial_get_degree(poly1));
 
     /* Various tests regarding 0-degree polynomials */
 
@@ -152,71 +156,73 @@ int main(void)
 
     cpl_test_zero( cpl_polynomial_derivative(poly1, 0) );
 
-    cpl_test_eq( cpl_polynomial_eval_1d(poly1, 5, &x), 0);
+    cpl_test_abs( cpl_polynomial_eval_1d(poly1, 5, &x), 0.0, 0.0);
 
-    cpl_test_zero( x );
+    cpl_test_abs(x, 0.0, 0.0 );
 
-    cpl_test_zero( cpl_polynomial_solve_1d(poly1, 5, &x, 1) );
+    cpl_test_abs( cpl_polynomial_solve_1d(poly1, 5, &x, 1), 0.0, 0.0 );
 
-    cpl_test_zero( x );
+    cpl_test_abs(x, 0.0, 0.0 );
 
-    cpl_test_eq( cpl_polynomial_eval_1d(poly1, 5, &x), 0);
+    cpl_test_abs( cpl_polynomial_eval_1d(poly1, 5, &x), 0.0, 0.0);
 
-    cpl_test_zero( x );
+    cpl_test_abs(x, 0.0, 0.0 );
 
     /* Properly react to a 0-degree polynomial with no roots */
     i = 0;
     cpl_test_zero( cpl_polynomial_set_coeff(poly1, &i, 1) );
-    cpl_test_eq( cpl_polynomial_get_coeff(poly1, &i), 1 );
+    cpl_test_abs( cpl_polynomial_get_coeff(poly1, &i), 1.0, 0.0 );
 
     error = cpl_polynomial_solve_1d(poly1, 1, &x, 1);
 
     cpl_test_eq_error(error, CPL_ERROR_DIVISION_BY_ZERO);
 
     /* Equivalent to setting all coefficients to zero */
-    cpl_test_zero( cpl_polynomial_derivative(poly1, 0) );
 
-    cpl_test_eq( cpl_polynomial_eval_1d(poly1, 5, &x), 0);
+    error = cpl_polynomial_derivative(poly1, 0);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
 
-    cpl_test_zero( x );
+    cpl_test_abs( cpl_polynomial_eval_1d(poly1, 5, &x), 0.0, 0.0);
+
+    cpl_test_abs( x, 0.0, 0.0);
 
     i = VECTOR_SIZE;
     cpl_test_zero( cpl_polynomial_set_coeff(poly1, &i, 1) );
-    cpl_test_eq( cpl_polynomial_get_coeff(poly1, &i), 1 );
+    cpl_test_abs( cpl_polynomial_get_coeff(poly1, &i), 1.0, 0.0 );
 
     cpl_test_zero( cpl_polynomial_set_coeff(poly1, &i, 0) );
 
     cpl_test_eq( cpl_polynomial_get_dimension(poly1), 1);
-    cpl_test_eq( cpl_polynomial_get_degree(poly1), 0);
+    cpl_test_zero( cpl_polynomial_get_degree(poly1));
 
     cpl_test( poly2 = cpl_polynomial_duplicate(poly1) );
 
-    cpl_test_eq( cpl_polynomial_get_degree(poly2), 0);
+    cpl_test_zero( cpl_polynomial_get_degree(poly2));
 
     cpl_polynomial_delete(poly2);
 
     cpl_test( poly2 = cpl_polynomial_new(VECTOR_SIZE) );
 
     cpl_test_eq( cpl_polynomial_get_dimension(poly2), VECTOR_SIZE );
-    cpl_test_eq( cpl_polynomial_get_degree(poly2), 0);
+    cpl_test_zero( cpl_polynomial_get_degree(poly2));
 
     cpl_test_zero( cpl_polynomial_copy(poly2, poly1));
 
     cpl_test_eq( cpl_polynomial_get_dimension(poly2), 1 );
-    cpl_test_eq( cpl_polynomial_get_degree(poly2), 0);
+    cpl_test_zero( cpl_polynomial_get_degree(poly2));
 
     cpl_polynomial_delete(poly2);
 
     /* Set the coefficients : 1 + 2.x + 3.x^2 + 4.x^3 + 5.x^4 */
     for (i=0; i < POLY_COEF_NB ; i++) {
         if (i % (POLY_COEF_NB-2) != 0) continue;
-        cpl_test_zero( cpl_polynomial_set_coeff(poly1, &i, i+1) );
+        cpl_test_zero( cpl_polynomial_set_coeff(poly1, &i, (double)(i+1)) );
             cpl_polynomial_dump(poly1, stream);
     }
     for (i=0; i < POLY_COEF_NB ; i++) {
         if (i % (POLY_COEF_NB-2) == 0) continue;
             cpl_polynomial_dump(poly1, stream);
-        cpl_test_zero( cpl_polynomial_set_coeff(poly1, &i, i+1) );
+            cpl_test_zero( cpl_polynomial_set_coeff(poly1, &i, (double)(i+1)));
     }
     cpl_polynomial_dump(poly1, stream);
     /* Test cpl_polynomial_get_degree() */
@@ -224,7 +230,7 @@ int main(void)
 
     /* Test cpl_polynomial_get_coeff() */
     for (i=0; i < POLY_COEF_NB ; i++)
-        cpl_test_eq( cpl_polynomial_get_coeff(poly1, &i), i+1 );
+        cpl_test_abs( cpl_polynomial_get_coeff(poly1, &i), (double)(i+1), 0.0);
 
     /* Test cpl_polynomial_eval() */
     vec = cpl_vector_new(1);
@@ -244,7 +250,29 @@ int main(void)
     cpl_polynomial_copy(poly1, poly2);
 
     cpl_test_zero( cpl_polynomial_compare(poly1, poly2, 0) );
-    
+
+    i = 10 + cpl_polynomial_get_degree(poly1);
+    error = cpl_polynomial_set_coeff(poly1, &i, 42.0);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+    error = cpl_polynomial_multiply_scalar(poly1, poly2, 0.5);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    cpl_test_eq(cpl_polynomial_get_degree(poly1),
+                cpl_polynomial_get_degree(poly2));
+
+    error = cpl_polynomial_multiply_scalar(poly1, poly1, 2.0);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+    cpl_test_zero( cpl_polynomial_compare(poly1, poly2, 0) );
+
+    error = cpl_polynomial_multiply_scalar(poly1, poly2, 0.0);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    cpl_test_zero( cpl_polynomial_get_degree(poly1));
+
+    error = cpl_polynomial_multiply_scalar(poly2, poly2, 0.0);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    cpl_test_zero( cpl_polynomial_get_degree(poly2));
+
     /* Free */
     cpl_polynomial_delete(poly1);
     cpl_polynomial_delete(poly2);
@@ -279,7 +307,7 @@ int main(void)
     i = 0;
     vec_data[i] = 1;
     for (i=1 ; i < POLY_SIZE ; i++) {
-        vec_data[i] = vec_data[i-1]/i;
+        vec_data[i] = vec_data[i-1] / (double)i;
     }
 
     for (i=POLY_SIZE-1 ; i >= 0; i--) {
@@ -317,7 +345,7 @@ int main(void)
 
     cpl_test_nonnull( vec_data );
     for (i=0 ; i < VECTOR_SIZE ; i++) {
-        const double xapp = -x + i * z2; 
+        const double xapp = -x + (double)i * z2; 
         const double yapp = exp( xapp );
 
         /* cpl_vector_fill_polynomial() is just a wrapper */
@@ -433,8 +461,8 @@ int main(void)
     cpl_polynomial_solve_1d(poly1, 10*x1, &x, i);
 
     z = cpl_polynomial_eval_1d(poly1, x, &y);
-    cpl_msg_info("", "(X+1)^15 (%d): %g %g %g", i, (x-x1)/DBL_EPSILON,
-                 z/DBL_EPSILON, y/DBL_EPSILON);
+    cpl_msg_info("", "(X+1)^15 (%" CPL_SIZE_FORMAT "): %g %g %g", i,
+                 (x-x1)/DBL_EPSILON, z/DBL_EPSILON, y/DBL_EPSILON);
     cpl_test_rel( x, x1, DBL_EPSILON );
     cpl_test_abs( z, 0.0, DBL_EPSILON );
     cpl_test_abs( y, 0.0, DBL_EPSILON);
@@ -443,8 +471,8 @@ int main(void)
     i = 5;
     cpl_test_zero( cpl_polynomial_solve_1d(poly1, -10*x1, &x, i) );
     z = cpl_polynomial_eval_1d(poly1, x, &y);
-    cpl_msg_info("", "(X+1)^15 (%d) " CPL_LDBL_EPSILON_STR ": %g %g %g",
-                 i, x-x1,
+    cpl_msg_info("", "(X+1)^15 (%" CPL_SIZE_FORMAT ") " CPL_LDBL_EPSILON_STR
+                 ": %g %g %g", i, x-x1,
                  (double)(z/CPL_LDBL_EPSILON),
                  (double)(y/CPL_LDBL_EPSILON));
     cpl_test_rel( x, x1, 0.35 );  /* alphaev56 */
@@ -466,15 +494,15 @@ int main(void)
     cpl_polynomial_dump(poly2, stream);
 
     cpl_test_eq( cpl_polynomial_get_dimension(poly2), POLY_DIM);
-    cpl_test_eq( cpl_polynomial_get_degree(poly2), 0);
+    cpl_test_zero( cpl_polynomial_get_degree(poly2));
 
     /* Set and reset some (outragous) coefficient */
     for (j = 0; j <POLY_DIM;j++) expo[j] = j*VECTOR_SIZE + POLY_SIZE;
     cpl_test_zero( cpl_polynomial_set_coeff(poly2, expo, 1));
-    cpl_test_eq( cpl_polynomial_get_coeff(poly2, expo), 1);
+    cpl_test_abs( cpl_polynomial_get_coeff(poly2, expo), 1.0, 0.0);
     cpl_test_zero( cpl_polynomial_set_coeff(poly2, expo, 0));
 
-    cpl_test_eq( cpl_polynomial_get_degree(poly2), 0);
+    cpl_test_zero( cpl_polynomial_get_degree(poly2));
 
     for (i = 0; i <= 15 ; i++) {
         for (j = 0; j <POLY_DIM;j++) expo[j] = 0;
@@ -486,20 +514,21 @@ int main(void)
     /* Scale polynomial (with integer to avoid rounding)  */
     i = 15;
     cpl_polynomial_set_coeff(poly1, &i, 1);
-    for (i = 0; i <=15;i++)
+    for (i = 0; i <= 15;i++)
         cpl_test_zero( cpl_polynomial_set_coeff(poly1, &i,
-                 5 * cpl_polynomial_get_coeff(poly1, &i)) );
+                 5.0 * cpl_polynomial_get_coeff(poly1, &i)) );
 
     i = 15;
-    j = i -1;
+    j = i - 1;
     cpl_test_zero( cpl_polynomial_shift_1d(poly1, 0,
             -cpl_polynomial_get_coeff(poly1, &j)
-           /(cpl_polynomial_get_coeff(poly1, &i)*i)));
+           /(cpl_polynomial_get_coeff(poly1, &i)*(double)i)));
 
     i = 15;
-    cpl_test_eq( cpl_polynomial_get_coeff(poly1, &i), 5);
+    cpl_test_abs( cpl_polynomial_get_coeff(poly1, &i), 5.0, 0.0);
 
-    for (i = 14; i >=0;i--) cpl_test_zero( cpl_polynomial_get_coeff(poly1, &i) );
+    for (i = 14; i >=0; i--)
+        cpl_test_abs( cpl_polynomial_get_coeff(poly1, &i), 0.0, 0.0);
 
     cpl_test_zero( cpl_polynomial_copy(poly1, poly2) );
 
@@ -863,7 +892,7 @@ int main(void)
     x1 = x;
     for (i=POLY_SIZE-1; i > 0; i--) {
 
-        x1 *= i;
+        x1 *= (double)i;
         cpl_test_zero( cpl_polynomial_derivative(poly1, 0) );
 
     }
@@ -890,7 +919,7 @@ int main(void)
             cpl_test_zero( cpl_polynomial_set_coeff(poly1, expo, x));
 
         }
-        if (i > 0) x1 *= i;
+        if (i > 0) x1 *= (double)i;
     }
 
     i = 0;
@@ -902,7 +931,7 @@ int main(void)
 
         z1 = 0.0;
         for (i=0; i < POLY_SIZE; i++) {
-            z1 += pow(y, i);
+            z1 += pow(y, (double)i);
         }
 
         z1 *= x;
@@ -918,7 +947,7 @@ int main(void)
 
         for (i=1; i < POLY_SIZE-1; i++) {
             z1 = cpl_polynomial_get_coeff(poly2, &i);
-            cpl_test_abs( z * (i+1), z1, 11*POLY_SIZE*FLT_EPSILON);
+            cpl_test_abs( z * (double)(i+1), z1, 11.0*POLY_SIZE*FLT_EPSILON);
         }
 
         cpl_polynomial_delete(poly2);
@@ -982,7 +1011,7 @@ int main(void)
     if (mse_secs > 0.0)
         cpl_msg_info("","Speed of cpl_vector_fill_polynomial_fit_residual() "
                      "over %g secs [Mflop/s]: %g", mse_secs,
-                     mse_flops/mse_secs/1e6);
+                     (double)mse_flops/mse_secs/1e6);
 
     if (stream != stdout) cpl_test_zero( fclose(stream) );
 
@@ -1032,7 +1061,7 @@ static cpl_polynomial * cpl_polynomial_fit_test_2d(FILE * stream,
     cpl_matrix     * xy_pos;
     cpl_vector     * xpoint;
     const double     xy_offset = 100;
-    int              degree;
+    cpl_size         degree;
     cpl_polynomial * poly1 = cpl_polynomial_new(2);
     cpl_polynomial * poly2 = cpl_polynomial_new(2);
     cpl_vector     * vec;
@@ -1040,8 +1069,8 @@ static cpl_polynomial * cpl_polynomial_fit_test_2d(FILE * stream,
     double           ymax = 0.0; /* Maximum rounding error on y */
     double           eps;
     double           x, y;
-    const int        zerodeg2[] = {0, 0};
-    int              i, j, k;
+    const cpl_size   zerodeg2[] = {0, 0};
+    cpl_size         i, j, k;
     cpl_error_code   error;
 
 
@@ -1056,8 +1085,8 @@ static cpl_polynomial * cpl_polynomial_fit_test_2d(FILE * stream,
     cpl_vector_set(xpoint, 1, sqrt(CPL_MATH_E)-1+xy_offset);
 
     for (degree = 0; degree < POLY_SIZE; degree++) {
-        const int maxdeg[2] = {degree, degree};
-        const int nc = dimdeg ? (degree+1)*(degree+1)
+        const cpl_size maxdeg[2] = {degree, degree};
+        const cpl_size nc = dimdeg ? (degree+1)*(degree+1)
             : (degree+1)*(degree+2)/2;
 
         k = 0;
@@ -1065,11 +1094,13 @@ static cpl_polynomial * cpl_polynomial_fit_test_2d(FILE * stream,
         vec = cpl_vector_new(nc);
         xy_pos = cpl_matrix_new(2, nc);
 
-        for (i=0; i <= degree; i++) {
-            for (j=0; j <= (dimdeg ? degree : i); j++, k++) {
+        for (i=0; i < 1 + degree; i++) {
+            for (j=0; j < 1 + (dimdeg ? degree : i); j++, k++) {
 
-                cpl_test_zero( cpl_matrix_set(xy_pos, 0, k, i+xy_offset) );
-                cpl_test_zero( cpl_matrix_set(xy_pos, 1, k, j+xy_offset) );
+                cpl_test_zero( cpl_matrix_set(xy_pos, 0, k,
+                                              (double)i+xy_offset) );
+                cpl_test_zero( cpl_matrix_set(xy_pos, 1, k,
+                                              (double)j+xy_offset) );
                 cpl_test_zero( cpl_vector_set(vec,   k,
                                               sqrt((double)i)*
                                               log((double)(j+1))));
@@ -1096,10 +1127,11 @@ static cpl_polynomial * cpl_polynomial_fit_test_2d(FILE * stream,
         cpl_vector_delete(vec);
         cpl_matrix_delete(xy_pos);
 
-        cpl_msg_info(cpl_func,"2D-Polynomial with degree %d (dimdeg=%d) "
-                     "fit of a %d-point dataset has a mean square error "
-                     "ratio to a %d-degree fit: %g (%g > %g)", degree,
-                     (int)dimdeg, nc, degree-1, xmax > 0 ? eps/xmax : 0,
+        cpl_msg_info(cpl_func,"2D-Polynomial with degree %" CPL_SIZE_FORMAT
+                     " (dimdeg=%d) fit of a %" CPL_SIZE_FORMAT "-point "
+                     "dataset has a mean square error ratio to a %"
+                     CPL_SIZE_FORMAT "-degree fit: %g (%g > %g)", degree,
+                     dimdeg, nc, degree-1, xmax > 0.0 ? eps/xmax : 0.0,
                      eps, xmax);
         xmax = eps;
 
@@ -1111,10 +1143,11 @@ static cpl_polynomial * cpl_polynomial_fit_test_2d(FILE * stream,
         if (fabs(0.25-eps) >= fabs(0.25 - ymax) && degree > 0) {
             /* Should be able to fit at least a 5-degree polynomial
                with increased accuracy - and without error-margin */
-            cpl_msg_info(cpl_func,"2D-Polynomial with degree %d fit of a "
-                         "%d-point dataset has a greater residual than a "
-                         "%d-degree fit to a %d-point dataset: fabs(%g) > "
-                         "fabs(%g)", degree, nc, degree-1,
+            cpl_msg_info(cpl_func,"2D-Polynomial with degree %" CPL_SIZE_FORMAT
+                         " fit of a %" CPL_SIZE_FORMAT "-point dataset has a "
+                         "greater residual than a %" CPL_SIZE_FORMAT "-degree "
+                         "fit to a %" CPL_SIZE_FORMAT "-point dataset: "
+                         "fabs(%g) > fabs(%g)", degree, nc, degree-1,
                          degree*(degree+1)/2, eps-0.25, ymax-0.25);
             break;
         }
@@ -1144,8 +1177,8 @@ static cpl_polynomial * cpl_polynomial_fit_test_2d(FILE * stream,
             x = (double) i * 0.5;
             y = (double) j * 2.0;
 
-            cpl_test_zero( cpl_matrix_set(xy_pos, 0, k, i+xy_offset) );
-            cpl_test_zero( cpl_matrix_set(xy_pos, 1, k, j-xy_offset) );
+            cpl_test_zero( cpl_matrix_set(xy_pos, 0, k, (double)i+xy_offset) );
+            cpl_test_zero( cpl_matrix_set(xy_pos, 1, k, (double)j-xy_offset) );
 
             cpl_test_zero( cpl_vector_set(vec,   k, sqrt(x)*log(y+1.0)));
         }
@@ -1155,8 +1188,8 @@ static cpl_polynomial * cpl_polynomial_fit_test_2d(FILE * stream,
 
     ymax = 0;
     for (degree = 0; degree < POLY_SIZE; degree++) {
-        const int maxdeg[2] = {degree, degree};
-        const int nc = 2 * POLY_SIZE * 2 * POLY_SIZE;
+        const cpl_size maxdeg[2] = {degree, degree};
+        const cpl_size nc = 2 * POLY_SIZE * 2 * POLY_SIZE;
         double mse;
 
         error = cpl_polynomial_fit_cmp(poly2, xy_pos, NULL, vec, NULL,
@@ -1173,22 +1206,26 @@ static cpl_polynomial * cpl_polynomial_fit_test_2d(FILE * stream,
 
         eps = cpl_polynomial_eval(poly2, xpoint);
 
-        cpl_msg_info(cpl_func, "2D-Polynomial with degree %d fit of a %d-point "
-                     "dataset has a mean square error: %g (P0-error=%g)",
+        cpl_msg_info(cpl_func, "2D-Polynomial with degree %" CPL_SIZE_FORMAT
+                     " fit of a %" CPL_SIZE_FORMAT "-point dataset "
+                     "has a mean square error: %g (P0-error=%g)",
                      degree, nc, mse, 0.25-eps);
 
         /* mse must decrease */
         if (degree > 0) {
             if (fabs(eps-0.25) > fabs(ymax - 0.25))
-                cpl_msg_info(cpl_func, "2D-Polynomial with degree %d fit of a "
-                             "%d-point dataset has a larger error than a %d-"
-                             "degree fit: fabs(%g-0.25) > fabs(%g-0.25)",
+                cpl_msg_info(cpl_func, "2D-Polynomial with degree %"
+                             CPL_SIZE_FORMAT " fit of a %" CPL_SIZE_FORMAT
+                             "-point dataset has a larger error than a %"
+                             CPL_SIZE_FORMAT "-degree fit: fabs(%g-0.25) > "
+                             "fabs(%g-0.25)",
                              degree, nc, degree - 1, eps, ymax);
             if (mse > xmax) {
-                cpl_msg_info(cpl_func, "2D-Polynomial with degree %d fit of a "
-                             "%d-point dataset has a larger mean square error "
-                             "than a %d-degree fit: %g > %g", degree, nc,
-                             degree - 1, mse, xmax);
+                cpl_msg_info(cpl_func, "2D-Polynomial with degree %"
+                             CPL_SIZE_FORMAT " fit of a %" CPL_SIZE_FORMAT
+                             "-point dataset has a larger mean square error "
+                             "than a %" CPL_SIZE_FORMAT "-degree fit: %g > %g",
+                             degree, nc, degree - 1, mse, xmax);
                 break;
             }
         }
@@ -1238,16 +1275,16 @@ static void cpl_polynomial_fit_test_1d(FILE * stream)
     cpl_polynomial  * poly1b = cpl_polynomial_new(1);
     cpl_polynomial  * poly2 = cpl_polynomial_new(2);
     const cpl_boolean symsamp = CPL_TRUE;
-    const int         zerodeg = 0;
-    const int         sqdeg  = 2;
-    const int         maxdeg = 3;
-    int               mindeg, errdeg;
+    const cpl_size    zerodeg = 0;
+    const cpl_size    sqdeg  = 2;
+    const cpl_size    maxdeg = 3;
+    cpl_size          mindeg, errdeg;
     double            eps, rechisq;
     double            xmax; /* Maximum rounding error on x */
     double            ymax; /* Maximum rounding error on y */
     double            zmax;
     cpl_error_code    error;
-    int               i, j;
+    cpl_size          i, j;
 
 
     /* Test 1: NULL input */
@@ -1425,12 +1462,12 @@ static void cpl_polynomial_fit_test_1d(FILE * stream)
     ymax = -DBL_EPSILON;
     zmax = 0.0;
     for (j=1; j <= POLY_SIZE; j++) {
-        const int degree = j - 1;
+        const cpl_size degree = j - 1;
         cpl_test_zero( cpl_matrix_set_size(samppos, 1, j) );
         cpl_test_zero( cpl_vector_set_size(fitvals,    j) );
         for (i=0; i < j; i++) {
-            cpl_test_zero( cpl_matrix_set(samppos, 0, i, i) );
-            cpl_test_zero( cpl_vector_set(fitvals, i, sqrt(i)) );
+            cpl_test_zero( cpl_matrix_set(samppos, 0, i, (double)i) );
+            cpl_test_zero( cpl_vector_set(fitvals, i, sqrt((double)i)) );
         }
 
         error = cpl_polynomial_fit_cmp(poly1a, samppos, &symsamp, fitvals, NULL,
@@ -1438,9 +1475,10 @@ static void cpl_polynomial_fit_test_1d(FILE * stream)
 
         if (error == CPL_ERROR_SINGULAR_MATRIX) {
 
-            cpl_msg_info("FIXME","1D-Polynomial fit of a %d-point dataset with "
-                         "degree %d leads to a (near) singular system of "
-                         "equations", j, degree);
+            cpl_msg_info("FIXME","1D-Polynomial fit of a %" CPL_SIZE_FORMAT
+                         "-point dataset with degree %" CPL_SIZE_FORMAT
+                         " leads to a (near) singular system of equations",
+                         j, degree);
             cpl_test_eq_error(error, CPL_ERROR_SINGULAR_MATRIX);
             break;
         }
@@ -1461,14 +1499,16 @@ static void cpl_polynomial_fit_test_1d(FILE * stream)
 
         eps = cpl_polynomial_get_coeff(poly1a, &zerodeg);
         if (fabs(eps) > fabs(zmax)) {
-            cpl_msg_info(cpl_func,"1D-Polynomial with degree %d fit of a "
-                         "%d-point dataset has a greater error on P0 than "
-                         "a %d-degree fit to a %d-point dataset: fabs(%g) "
-                         "> fabs(%g)", degree, j, degree-1, j-1, eps, zmax);
+            cpl_msg_info(cpl_func,"1D-Polynomial with degree %" CPL_SIZE_FORMAT
+                         " fit of a %" CPL_SIZE_FORMAT "-point dataset has a "
+                         "greater error on P0 than a %" CPL_SIZE_FORMAT
+                         "-degree fit to a %" CPL_SIZE_FORMAT "-point dataset: "
+                         "fabs(%g) > fabs(%g)",
+                         degree, j, degree-1, j-1, eps, zmax);
         }
         /* Should loose at most one decimal digit per degree
            fabs(eps) < DBL_EPSILON * 10 ** (degree-2)  */
-        cpl_test_abs( eps, 0.0, DBL_EPSILON * pow(10, j-3) );
+        cpl_test_abs( eps, 0.0, DBL_EPSILON * pow(10, (double)(j-3)) );
         zmax = eps;
 
         /* Compute approximation to sqrt(0.25)
@@ -1481,9 +1521,11 @@ static void cpl_polynomial_fit_test_1d(FILE * stream)
         if (eps <= ymax) {
             /* Should be able to fit at least an 18-degree polynomial
                with increased accuracy - and without error-margin */
-            cpl_msg_info(cpl_func, "1D-Polynomial with degree %d fit of a "
-                         "%d-point dataset has a greater residual than a %d-"
-                         "degree fit to a %d-point dataset: %g > %g",
+            cpl_msg_info(cpl_func, "1D-Polynomial with degree %"
+                         CPL_SIZE_FORMAT " fit of a %" CPL_SIZE_FORMAT "-point "
+                         "dataset has a greater residual than a %"
+                         CPL_SIZE_FORMAT "-degree fit to a %" CPL_SIZE_FORMAT
+                         "-point dataset: %g > %g",
                          degree, j, degree-1, j-1, 0.5-eps, 0.5-ymax);
             break;
         }
@@ -1500,7 +1542,7 @@ static void cpl_polynomial_fit_test_1d(FILE * stream)
     cpl_test_zero( cpl_matrix_set_size(samppos, 1, VECTOR_SIZE) );
     cpl_test_zero( cpl_vector_set_size(fitvals, VECTOR_SIZE) );
     for (i=0; i < VECTOR_SIZE; i++) {
-        cpl_test_zero( cpl_matrix_set(samppos, 0, i, i + 1e4) );
+        cpl_test_zero( cpl_matrix_set(samppos, 0, i, (double)i + 1e4) );
         cpl_test_zero( cpl_vector_set(fitvals, i, sqrt((double)i)) );
     }
 
@@ -1522,10 +1564,11 @@ static void cpl_polynomial_fit_test_1d(FILE * stream)
                with no degradation - and without error-margin
                (i686 can manage one degree more) */
             if (i < 9) cpl_test_leq( eps, xmax);
-            cpl_msg_info(cpl_func, "1D-Polynomial with degree %d fit of a "
-                         "%d-point dataset has a higher mean square error "
-                         "than a %d-degree fit: %g > %g", i, VECTOR_SIZE,
-                         i-1, eps, xmax);
+            cpl_msg_info(cpl_func, "1D-Polynomial with degree %"
+                         CPL_SIZE_FORMAT " fit of a %d-point "
+                         "dataset has a higher mean square error than a %"
+                         CPL_SIZE_FORMAT "-degree fit: %g > %g",
+                         i, VECTOR_SIZE, i-1, eps, xmax);
             break;
         }
 
@@ -1589,7 +1632,7 @@ static void cpl_polynomial_fit_test_1d(FILE * stream)
 
  */
 /*----------------------------------------------------------------------------*/
-static void cpl_polynomial_fit_bench_2d(int nr, int nc, int nd)
+static void cpl_polynomial_fit_bench_2d(cpl_size nr, cpl_size nc, cpl_size nd)
 {
 
 
@@ -1599,7 +1642,7 @@ static void cpl_polynomial_fit_bench_2d(int nr, int nc, int nd)
     cpl_flops        flops  = 0;
     double           secs   = 0.0;
     double           xmax   = DBL_MAX; /* This valuie should not be read */
-    int              i, j, k, degree = 0;
+    cpl_size         i, j, k, degree = 0;
     cpl_error_code   error = CPL_ERROR_NONE;
 
 
@@ -1609,9 +1652,14 @@ static void cpl_polynomial_fit_bench_2d(int nr, int nc, int nd)
             const double x = (double) i * 0.5;
             const double y = (double) j * 2.0;
 
-            cpl_test_zero( cpl_matrix_set(xy_pos, 0, k, i) );
-            cpl_test_zero( cpl_matrix_set(xy_pos, 1, k, j) );
-            cpl_test_zero( cpl_vector_set(vec,   k, sqrt(x)*log(y+1.0)));
+            error = cpl_matrix_set(xy_pos, 0, k, (double)i);
+            cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+            error = cpl_matrix_set(xy_pos, 1, k, (double)j);
+            cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+            error = cpl_vector_set(vec,   k, sqrt(x)*log(y+1.0));
+            cpl_test_eq_error(error, CPL_ERROR_NONE);
         }
     }
 
@@ -1647,9 +1695,9 @@ static void cpl_polynomial_fit_bench_2d(int nr, int nc, int nd)
     cpl_matrix_delete(xy_pos);
     cpl_polynomial_delete(poly2);
 
-    cpl_msg_info("","Speed while fitting %d 2D-points with up to degree %d in "
-                 "%g secs [Mflop/s]: %g", nc*nc, degree, secs,
-                 flops/secs/1e6);
+    cpl_msg_info("","Speed while fitting %" CPL_SIZE_FORMAT " 2D-points with "
+                 "up to degree %" CPL_SIZE_FORMAT " in %g secs [Mflop/s]: %g",
+                 nc*nc, degree, secs, (double)flops/secs/1e6);
 
 }
 
@@ -1672,11 +1720,11 @@ static double cpl_vector_get_mse(const cpl_vector     * fitvals,
                                  double               * prechisq)
 {
 
-    const int    np = cpl_vector_get_size(fitvals);
-    cpl_vector * residual;
-    double       mse;
-    cpl_flops    flops  = 0;
-    double       secs   = 0.0;
+    const cpl_size np = cpl_vector_get_size(fitvals);
+    cpl_vector   * residual;
+    double         mse;
+    cpl_flops      flops  = 0;
+    double         secs   = 0.0;
 
     cpl_ensure(fitvals != NULL, CPL_ERROR_NULL_INPUT, -1.0);
     cpl_ensure(fit     != NULL, CPL_ERROR_NULL_INPUT, -2.0);
@@ -1693,7 +1741,7 @@ static double cpl_vector_get_mse(const cpl_vector     * fitvals,
     mse_secs += cpl_test_get_cputime() - secs;
     mse_flops += cpl_tools_get_flops() - flops;
 
-    mse = cpl_vector_product(residual, residual)/np;
+    mse = cpl_vector_product(residual, residual) / (double)np;
     cpl_vector_delete(residual);
 
     return mse;
@@ -1723,8 +1771,8 @@ cpl_error_code cpl_polynomial_fit_cmp(cpl_polynomial    * self,
                                       const cpl_vector  * fitvals,
                                       const cpl_vector  * fitsigm,
                                       cpl_boolean         dimdeg,
-                                      const int         * mindeg,
-                                      const int         * maxdeg)
+                                      const cpl_size    * mindeg,
+                                      const cpl_size    * maxdeg)
 {
 
     const cpl_error_code error = cpl_polynomial_fit(self, samppos, sampsym,
@@ -1734,10 +1782,10 @@ cpl_error_code cpl_polynomial_fit_cmp(cpl_polynomial    * self,
     cpl_test_error(error);
 
     if (self != NULL) {
-        const int mdim = cpl_polynomial_get_dimension(self);
-        const int ndim = 1 + mdim;
+        const cpl_size mdim = cpl_polynomial_get_dimension(self);
+        const cpl_size ndim = 1 + mdim;
         cpl_polynomial * self1p = cpl_polynomial_new(ndim);
-        int idim;
+        cpl_size idim;
 
         /* This is quaranteed to fail */
         const cpl_error_code error1e = cpl_polynomial_fit(self1p, samppos,
@@ -1749,18 +1797,23 @@ cpl_error_code cpl_polynomial_fit_cmp(cpl_polynomial    * self,
 
         if (samppos != NULL && (mdim == 1 || dimdeg == CPL_TRUE)) {
             cpl_polynomial * zeropol = cpl_polynomial_new(mdim);
-            const int np = cpl_matrix_get_ncol(samppos);
-            const int nc = cpl_matrix_get_nrow(samppos);
+            const cpl_size np = cpl_matrix_get_ncol(samppos);
+            const cpl_size nc = cpl_matrix_get_nrow(samppos);
             cpl_matrix * samppos1p = cpl_matrix_new(1 + nc, np);
 
             cpl_boolean * sampsym1p = sampsym
-                ? cpl_malloc(ndim * sizeof(*sampsym1p)) : NULL;
-            int * mindeg1p = mindeg ? cpl_malloc(ndim * sizeof(*mindeg1p)) : NULL;
-            int * maxdeg1p = maxdeg ? cpl_malloc(ndim * sizeof(*maxdeg1p)) : NULL;
+                ? (cpl_boolean *)cpl_malloc((size_t)ndim * sizeof(*sampsym1p))
+                : NULL;
+            cpl_size * mindeg1p = mindeg
+                ? (cpl_size*)cpl_malloc((size_t)ndim * sizeof(*mindeg1p))
+                : NULL;
+            cpl_size * maxdeg1p = maxdeg
+                ? (cpl_size*)cpl_malloc((size_t)ndim * sizeof(*maxdeg1p))
+                : NULL;
             cpl_error_code error1p;
 
             for (idim = 0; idim < ndim; idim++) {
-                int i, j;
+                cpl_size i, j;
 
                 /* First copy all rows to new matrix, leaving one with zeros */
                 for (j = 0; j < idim; j++) {
@@ -1804,7 +1857,7 @@ cpl_error_code cpl_polynomial_fit_cmp(cpl_polynomial    * self,
                 } else if (mindeg != NULL && mindeg[0] > 0) {
                     cpl_test_eq_error(error1p, CPL_ERROR_UNSUPPORTED_MODE);
                 } else {
-                    const int degree = cpl_polynomial_get_degree(self);
+                    const cpl_size degree = cpl_polynomial_get_degree(self);
                     if (!error && error1p == CPL_ERROR_SINGULAR_MATRIX &&
                         1 + degree == np && np >= 20) {
 
@@ -1814,20 +1867,22 @@ cpl_error_code cpl_polynomial_fit_cmp(cpl_polynomial    * self,
                            is not as accurate as the uni-variate,
                            for degree 20 the difference is a failure */
 
-                        cpl_msg_debug(cpl_func, "2D-fail(%d:%d:%d:%d): S: "
-                                        "%d x %d", error, idim, maxdeg1p[0],
-                                        maxdeg1p[1], 1 + nc, np);
+                        cpl_msg_debug(cpl_func, "2D-fail(%d:%" CPL_SIZE_FORMAT
+                                      ":%" CPL_SIZE_FORMAT ":%" CPL_SIZE_FORMAT
+                                      "): S: %" CPL_SIZE_FORMAT "x %"
+                                      CPL_SIZE_FORMAT, error, idim, maxdeg1p[0],
+                                      maxdeg1p[1], 1 + nc, np);
                         if (cpl_msg_get_level() <= CPL_MSG_DEBUG)
                             cpl_matrix_dump(samppos1p, stdout);
                     } else if (!error1p && !error) {
-                        const int i0 = 0;
+                        const cpl_size i0 = 0;
                         const double k0 = cpl_polynomial_get_coeff(self, &i0);
                         cpl_polynomial * self0p
                             = cpl_polynomial_extract(self1p, idim, zeropol);
 
                         /* FIXME: Need relative polynomial comparison */
                         cpl_test_polynomial_abs(self, self0p, CX_MAX(fabs(k0),1)
-                                                * pow(10.0, degree)
+                                                * pow(10.0, (double)degree)
                                                 * DBL_EPSILON);
 
                         cpl_polynomial_delete(self0p);
@@ -1848,5 +1903,5 @@ cpl_error_code cpl_polynomial_fit_cmp(cpl_polynomial    * self,
         cpl_polynomial_delete(self1p);
     }
 
-    return cpl_error_set(cpl_func, error); /* Tested and reset by caller */
+    return cpl_error_set_(error); /* Tested and reset by caller */
 }
