@@ -1,4 +1,4 @@
-/* $Id: cpl_property-test.c,v 1.17 2011/01/04 14:32:46 llundin Exp $
+/* $Id: cpl_property-test.c,v 1.18 2011/04/05 14:29:02 rpalsa Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2005 European Southern Observatory
@@ -19,10 +19,10 @@
  */
 
 /*
- * $Author: llundin $
- * $Date: 2011/01/04 14:32:46 $
- * $Revision: 1.17 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Author: rpalsa $
+ * $Date: 2011/04/05 14:29:02 $
+ * $Revision: 1.18 $
+ * $Name: cpl-6_0 $
  */
 
 #include <complex.h>
@@ -43,7 +43,7 @@ cpl_test_property_dump(cpl_property *property)
 
     char c;
 
-    long size = cpl_property_get_size(property);
+    cpl_size size = cpl_property_get_size(property);
 
     cpl_type type = cpl_property_get_type(property);
 
@@ -263,17 +263,17 @@ int main(void)
 
     cpl_property_set_string(plist[6], strings[0]);
     cpl_test_eq(cpl_property_get_size(plist[6]),
-                ((long)strlen(strings[0]) + 1));
+                ((cpl_size)strlen(strings[0]) + 1));
     cpl_test_eq_string(cpl_property_get_string(plist[6]), strings[0]);
 
     cpl_property_set_string(plist[6], strings[1]);
     cpl_test_eq(cpl_property_get_size(plist[6]),
-                ((long)strlen(strings[1]) + 1));
+                ((cpl_size)strlen(strings[1]) + 1));
     cpl_test_eq_string(cpl_property_get_string(plist[6]), strings[1]);
 
     cpl_property_set_string(plist[6], strings[2]);
     cpl_test_eq(cpl_property_get_size(plist[6]),
-                ((long)strlen(strings[2]) + 1));
+                ((cpl_size)strlen(strings[2]) + 1));
     cpl_test_eq_string(cpl_property_get_string(plist[6]), strings[2]);
 
     code = cpl_property_set_float_complex(plist[7], zf0);
