@@ -1,4 +1,4 @@
-/* $Id: cpl_propertylist-test.c,v 1.75.2.1 2011/02/21 09:30:46 rpalsa Exp $
+/* $Id: cpl_propertylist-test.c,v 1.79 2011/04/18 11:31:00 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2005 European Southern Observatory
@@ -19,10 +19,10 @@
  */
 
 /*
- * $Author: rpalsa $
- * $Date: 2011/02/21 09:30:46 $
- * $Revision: 1.75.2.1 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Author: llundin $
+ * $Date: 2011/04/18 11:31:00 $
+ * $Revision: 1.79 $
+ * $Name: cpl-6_0 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -1223,7 +1223,8 @@ int main(void)
     cpl_test_zero(cpl_propertylist_is_empty(_plist));
     cpl_test_eq(cpl_propertylist_get_size(_plist), 5);
 
-    cpl_propertylist_append(plist, _plist);
+    code = cpl_propertylist_append(plist, _plist);
+    cpl_test_eq_error(code, CPL_ERROR_NONE);
 
     cpl_test_zero(cpl_propertylist_is_empty(plist));
     cpl_test_eq(cpl_propertylist_get_size(plist), nprops);
@@ -1393,7 +1394,7 @@ int main(void)
         cpl_test_eq_error(code, CPL_ERROR_NONE);
 
 
-        code = cpl_image_save(NULL, BASE "_sorted.fits", CPL_BPP_8_UNSIGNED,
+        code = cpl_image_save(NULL, BASE "_sorted.fits", CPL_TYPE_UCHAR,
                               plist, CPL_IO_CREATE);
         cpl_test_eq_error(code, CPL_ERROR_NONE);
         cpl_test_fits(BASE "_sorted.fits");
@@ -1919,6 +1920,8 @@ int main(void)
     /* Test the values */
     cpl_test_abs(f0, f1, 0.0);
     cpl_test_abs(d0, d2, 0.0);
+    cpl_test_rel(d1, f1, FLT_EPSILON);
+    cpl_test_rel(d2, f2, FLT_EPSILON);
 
 
     /*
