@@ -1,4 +1,4 @@
-/* $Id: cpl_stats-test.c,v 1.39 2010/11/11 09:23:18 llundin Exp $
+/* $Id: cpl_stats-test.c,v 1.41 2012/02/15 16:20:35 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/11/11 09:23:18 $
- * $Revision: 1.39 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2012/02/15 16:20:35 $
+ * $Revision: 1.41 $
+ * $Name: cpl-6_0 $
  */
 
 /*-----------------------------------------------------------------------------
@@ -52,6 +52,12 @@
 #define IMAGE_SIZE_Y IMAGE_SIZE_X
 #endif
 
+#if defined SIZEOF_SIZE_T && SIZEOF_SIZE_T == 4
+#define TOL (800.0 * FLT_EPSILON)
+#else
+#define TOL (16.0 * DBL_EPSILON)
+#endif
+
 #define CPL_STAT_CMP(OP, FUNC)                                          \
     do {                                                                \
                                                                         \
@@ -63,7 +69,7 @@
         /* Compare result from one-stats object with all-stats object */ \
         cpl_test_abs( CPL_CONCAT2X(cpl_stats_get, FUNC)(statall),       \
                       CPL_CONCAT2X(cpl_stats_get, FUNC)(statone),       \
-                      16.0 * DBL_EPSILON);                              \
+                      TOL);                                             \
                                                                         \
         cpl_stats_delete(statone);                                      \
                                                                         \
@@ -105,25 +111,25 @@
         cpl_test_abs( CPL_CONCAT2X(cpl_stats_get, OP)(statall),         \
                       CPL_CONCAT2X(cpl_image_get, CPL_CONCAT2X(OP,window)) \
                           (img, 1, 1, IMAGE_SIZE_X, IMAGE_SIZE_Y),      \
-                          16.0 * DBL_EPSILON);                          \
+                      TOL);                                             \
                                                                         \
         /* Compare result from stats object with image accessor */      \
         cpl_test_abs( CPL_CONCAT2X(cpl_stats_get, OP)(statall),         \
                       CPL_CONCAT2X(cpl_image_get, OP)(img),             \
-                      16.0 * DBL_EPSILON);                              \
+                      TOL);                                             \
     } while (0)    
 
 
 #define CPL_STAT_CMP_VECTOR(OP, TOL)                                    \
     do {                                                                \
-        const cpl_vector * vtest                                        \
+        cpl_vector * vtest                                              \
             = cpl_vector_wrap(IMAGE_SIZE_X*IMAGE_SIZE_Y,                \
                               cpl_image_get_data_double(img));          \
         cpl_test_nonnull(vtest);                                        \
         cpl_test_abs(CPL_CONCAT2X(cpl_stats_get, OP)(statall),          \
                      CPL_CONCAT2X(cpl_vector_get, OP)(vtest),           \
                      TOL* DBL_EPSILON);                                 \
-        cpl_test_nonnull(cpl_vector_unwrap((cpl_vector*)vtest));        \
+        cpl_test_nonnull(cpl_vector_unwrap(vtest));                     \
     } while (0)    
 
 /*-----------------------------------------------------------------------------
@@ -140,12 +146,12 @@ static void cpl_stats_new_median_bench(const cpl_image *, int);
 int main(void)
 {
     const cpl_type img_types[] = {CPL_TYPE_DOUBLE, CPL_TYPE_FLOAT, CPL_TYPE_INT};
-    const unsigned istats[] = {CPL_STATS_MIN,  CPL_STATS_MAX, CPL_STATS_MEAN,
-                               CPL_STATS_MEDIAN, CPL_STATS_MEDIAN_DEV,
-                               CPL_STATS_STDEV, CPL_STATS_FLUX,
-                               CPL_STATS_ABSFLUX, CPL_STATS_SQFLUX,
-                               CPL_STATS_MINPOS, CPL_STATS_MAXPOS,
-                               CPL_STATS_CENTROID};
+    const cpl_stats_mode istats[] = {CPL_STATS_MIN,  CPL_STATS_MAX,
+                                     CPL_STATS_MEDIAN, CPL_STATS_MEDIAN_DEV,
+                                     CPL_STATS_STDEV, CPL_STATS_FLUX,
+                                     CPL_STATS_ABSFLUX, CPL_STATS_SQFLUX,
+                                     CPL_STATS_MINPOS, CPL_STATS_MAXPOS,
+                                     CPL_STATS_CENTROID, CPL_STATS_MEAN};
     FILE          * stream;
     unsigned        itype;
     cpl_boolean     do_bench;
@@ -355,13 +361,13 @@ static void cpl_stats_new_all_bench(const cpl_image * image, int n)
     secs  = cpl_test_get_cputime();
 
     for (i = 0; i < n; i++) {
-        const cpl_stats * self
+        cpl_stats * self
             = cpl_stats_new_from_image_window(image, CPL_STATS_ALL, 2, 2,
                                               IMAGE_SIZE_X-1, IMAGE_SIZE_Y-1);
 
         cpl_test_nonnull(self);
 
-        cpl_stats_delete((cpl_stats*)self);
+        cpl_stats_delete(self);
     }
 
     secs = cpl_test_get_cputime() - secs;
@@ -389,7 +395,7 @@ static void cpl_stats_new_std_bench(const cpl_image * image, int n)
     secs  = cpl_test_get_cputime();
 
     for (i = 0; i < n; i++) {
-        const cpl_stats * self
+        cpl_stats * self
             = cpl_stats_new_from_image_window(image,
                                               CPL_STATS_MEAN | CPL_STATS_STDEV,
                                               2, 2,
@@ -397,7 +403,7 @@ static void cpl_stats_new_std_bench(const cpl_image * image, int n)
 
         cpl_test_nonnull(self);
 
-        cpl_stats_delete((cpl_stats*)self);
+        cpl_stats_delete(self);
     }
 
     secs = cpl_test_get_cputime() - secs;
@@ -424,13 +430,13 @@ static void cpl_stats_new_median_bench(const cpl_image * image, int n)
     secs  = cpl_test_get_cputime();
 
     for (i = 0; i < n; i++) {
-        const cpl_stats * self
+        cpl_stats * self
             = cpl_stats_new_from_image_window(image, CPL_STATS_MEDIAN, 2, 2,
                                               IMAGE_SIZE_X-1, IMAGE_SIZE_Y-1);
 
         cpl_test_nonnull(self);
 
-        cpl_stats_delete((cpl_stats*)self);
+        cpl_stats_delete(self);
     }
 
     secs = cpl_test_get_cputime() - secs;
