@@ -1,4 +1,4 @@
-/* $Id: cpl_vector-test.c,v 1.83 2010/12/13 11:03:10 llundin Exp $
+/* $Id: cpl_vector-test.c,v 1.94 2012/01/11 14:32:15 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/12/13 11:03:10 $
- * $Revision: 1.83 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2012/01/11 14:32:15 $
+ * $Revision: 1.94 $
+ * $Name: cpl-6_0 $
  */
 
 /*-----------------------------------------------------------------------------
@@ -33,15 +33,18 @@
 #include <config.h>
 #endif
 
-#include <float.h>
-#include <assert.h>
-#include <math.h>
-
+#include "cpl_io_fits.h"
 #include "cpl_test.h"
 #include "cpl_tools.h"
 #include "cpl_math_const.h"
 #include "cpl_memory.h"
 
+#include <float.h>
+#include <assert.h>
+#include <math.h>
+
+
+/* Must have three digits, for perl-test to work */
 #define VECTOR_SIZE 256
 #define VECTOR_CUT 24
 
@@ -55,7 +58,7 @@
 static void cpl_vector_save_bench(int);
 static void cpl_vector_get_stdev_bench(int);
 static void cpl_vector_corr_bench(int);
-
+static void cpl_vector_fit_gaussian_test_one(FILE *);
 /*-----------------------------------------------------------------------------
                                   Main
  -----------------------------------------------------------------------------*/
@@ -74,17 +77,19 @@ int main(void)
     cpl_vector  * vxc1;
     cpl_vector  * vxc3;
     double      * data;
-    const int     vdif = VECTOR_SIZE - VECTOR_CUT > VECTOR_CUT
+    const double  five[] = {1,2,3,4,5};
+    const cpl_size vdif = VECTOR_SIZE - VECTOR_CUT > VECTOR_CUT
                        ? VECTOR_CUT : VECTOR_SIZE - VECTOR_CUT;
-    const int     vdif2 = (VECTOR_SIZE - VECTOR_CUT)/2 > VECTOR_CUT/2
+    const cpl_size vdif2 = (VECTOR_SIZE - VECTOR_CUT)/2 > VECTOR_CUT/2
                         ? VECTOR_CUT/2 : (VECTOR_SIZE - VECTOR_CUT)/2;
-    int           delta;
-    int           half_search;
-    int           i,k;
+    cpl_size      delta;
+    cpl_size      half_search;
+    cpl_size           i,k;
     cpl_boolean   do_bench;
     FILE        * stream;
     FILE        * f_out;
     char          filename[1024];
+    cpl_boolean did_test_large = CPL_FALSE;
     cpl_error_code error;
     const int omp_num_threads =
 #ifdef _OPENMP
@@ -105,6 +110,8 @@ int main(void)
 
     cpl_test_nonnull( stream );
 
+    cpl_vector_fit_gaussian_test_one(stream);
+
     null = cpl_vector_new(0);
 
     cpl_test_error(CPL_ERROR_ILLEGAL_INPUT);
@@ -272,19 +279,19 @@ int main(void)
     cpl_test_eq( cpl_vector_get_size(tmp_vec), cpl_vector_get_size(sinus) );
 
     /* Test cpl_vector_save() / cpl_vector_load() */
-    error = cpl_vector_save(tmp_vec, "cpl_vector_save.fits", CPL_BPP_IEEE_DOUBLE,
+    error = cpl_vector_save(tmp_vec, "cpl_vector_save.fits", CPL_TYPE_DOUBLE,
                             NULL, CPL_IO_CREATE | CPL_IO_EXTEND); 
     cpl_test_eq_error( error, CPL_ERROR_ILLEGAL_INPUT );
 
-    error = cpl_vector_save(tmp_vec, "cpl_vector_save.fits", CPL_BPP_IEEE_DOUBLE,
+    error = cpl_vector_save(tmp_vec, "cpl_vector_save.fits", CPL_TYPE_DOUBLE,
                             NULL, CPL_IO_APPEND); 
     cpl_test_eq_error( error, CPL_ERROR_ILLEGAL_INPUT );
 
-    error = cpl_vector_save(tmp_vec, "cpl_vector_save.fits", CPL_BPP_IEEE_DOUBLE,
+    error = cpl_vector_save(tmp_vec, "cpl_vector_save.fits", CPL_TYPE_DOUBLE,
                             NULL, CPL_IO_CREATE | CPL_IO_APPEND); 
     cpl_test_eq_error( error, CPL_ERROR_ILLEGAL_INPUT );
 
-    error = cpl_vector_save(tmp_vec, "cpl_vector_save.fits", CPL_BPP_IEEE_DOUBLE,
+    error = cpl_vector_save(tmp_vec, "cpl_vector_save.fits", CPL_TYPE_DOUBLE,
                             NULL, CPL_IO_CREATE); 
     cpl_test_eq_error( error, CPL_ERROR_NONE );
     cpl_test_fits("cpl_vector_save.fits");
@@ -301,12 +308,12 @@ int main(void)
 
     /* Repeat test cpl_vector_save() / cpl_vector_load() on APPEND mode*/
     error = cpl_vector_save(tmp_vec, "cpl_vector_save.fits",
-                            CPL_BPP_IEEE_DOUBLE, NULL, CPL_IO_CREATE); 
+                            CPL_TYPE_DOUBLE, NULL, CPL_IO_CREATE); 
     cpl_test_eq_error( error, CPL_ERROR_NONE );
     cpl_test_fits("cpl_vector_save.fits");
 
     error = cpl_vector_save(tmp_vec, "cpl_vector_save.fits",
-                            CPL_BPP_IEEE_DOUBLE, NULL, CPL_IO_EXTEND);
+                            CPL_TYPE_DOUBLE, NULL, CPL_IO_EXTEND);
     cpl_test_eq_error( error, CPL_ERROR_NONE );
     cpl_test_fits("cpl_vector_save.fits");
 
@@ -322,6 +329,68 @@ int main(void)
     cpl_test_vector_abs(tmp_vec, tmp_vec2, 0.0);
     cpl_vector_delete(tmp_vec2);
 
+    if (!cpl_io_fits_is_enabled()) {
+        /* Decrease the number of elements by one,
+           thus verifying that an external application may modify the file */
+        if (system("perl -pi -e 'BEGIN{sleep(1)};/NAXIS1/ and s/"
+                   CPL_STRINGIFY(VECTOR_SIZE) "/sprintf(\"%d\","
+                   CPL_STRINGIFY(VECTOR_SIZE-1)
+                   ")/e' cpl_vector_save.fits") == 0) {
+
+            tmp_vec2 = cpl_vector_load("cpl_vector_save.fits", 0);
+            cpl_test_error(CPL_ERROR_NONE);
+            cpl_test_nonnull( tmp_vec2 );
+            cpl_test_eq(cpl_vector_get_size(tmp_vec2), VECTOR_SIZE-1);
+            cpl_vector_delete(tmp_vec2);
+        }
+    }
+
+    if (sizeof(cpl_size) == 4) {
+#if !defined CPL_SIZE_BITS || CPL_SIZE_BITS != 32
+        if (!cpl_io_fits_is_enabled()) {
+            /* Cannot load a vector longer than 2**31 - 1 */
+
+            /* Increase the number of elements to more than 2**31 */
+            if (system("perl -pi -e '/NAXIS1/ and s/       "
+                       CPL_STRINGIFY(VECTOR_SIZE)
+                       "/2200000000/' cpl_vector_save.fits") == 0) {
+
+                tmp_vec2 = cpl_vector_load("cpl_vector_save.fits", 0);
+                cpl_test_error(CPL_ERROR_UNSUPPORTED_MODE);
+                cpl_test_null( tmp_vec2 );
+                if (tmp_vec2 != NULL) {
+                    /* The original size is VECTOR_SIZE */
+                    cpl_test_noneq(cpl_vector_get_size(tmp_vec2), VECTOR_SIZE);
+                    cpl_vector_delete(tmp_vec2);
+                    cpl_test_assert(0);
+                }
+                did_test_large = CPL_TRUE;
+            }
+        }
+#endif
+#ifdef CPL_TEST_LARGE
+    } else if (sizeof(cpl_size) == 8) {
+        cpl_vector * long_vec = cpl_vector_new(2200000000L);
+        error = cpl_vector_save(long_vec, "cpl_vector_save.fits",
+                                CPL_TYPE_DOUBLE, NULL, CPL_IO_CREATE);
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
+        cpl_test_fits("cpl_vector_save.fits");
+        cpl_vector_delete(long_vec);
+        long_vec = cpl_vector_load("cpl_vector_save.fits", 0);
+        cpl_test_error(CPL_ERROR_NONE);
+        cpl_test_nonnull(long_vec);
+        cpl_vector_delete(long_vec);
+        did_test_large = CPL_TRUE;
+
+#endif
+    }
+
+    if (!did_test_large) {
+        cpl_msg_info(cpl_func, "I/O-testing of large vectors inactive");
+    }
+
+    cpl_test_zero( remove("cpl_vector_save.fits") );
+
     /* Loss of precision in cpl_vector_dump() */
     cpl_test_vector_abs(tmp_vec, sinus, 10.0 * FLT_EPSILON);
 
@@ -339,14 +408,46 @@ int main(void)
 
     /* Test fill function */
     cpl_test_eq_error( cpl_vector_fill(tmp_vec, 1.0), CPL_ERROR_NONE );
-    cpl_test_leq( cpl_vector_get_mean(tmp_vec) - 1, DBL_EPSILON );
+    cpl_test_abs( cpl_vector_get_mean(tmp_vec), 1.0, DBL_EPSILON );
 
     /* Test extract function */
     tmp_vec2 = cpl_vector_extract(tmp_vec, 0, VECTOR_SIZE/2, 1);
     cpl_test_nonnull( tmp_vec2 );
 
     cpl_vector_delete(tmp_vec2);
+
+    null = cpl_vector_extract(NULL, 0, 1, 1);
+    cpl_test_error(CPL_ERROR_NULL_INPUT);
+    cpl_test_null( null );
     
+    null = cpl_vector_extract(tmp_vec, 2, 1, 1);
+    cpl_test_error(CPL_ERROR_ILLEGAL_INPUT);
+    cpl_test_null( null );
+
+    null = cpl_vector_extract(tmp_vec, 1, 2, 2);
+    cpl_test_error(CPL_ERROR_ILLEGAL_INPUT);
+    cpl_test_null( null );
+
+    null = cpl_vector_extract(tmp_vec, -1, 2, 1);
+    cpl_test_error(CPL_ERROR_ACCESS_OUT_OF_RANGE);
+    cpl_test_null( null );
+
+    null = cpl_vector_extract(tmp_vec, 0, VECTOR_SIZE + 2, 1);
+    cpl_test_error(CPL_ERROR_ACCESS_OUT_OF_RANGE);
+    cpl_test_null( null );
+
+    vxc = cpl_vector_wrap(5, (double*)five);
+    vxc1 = cpl_vector_extract(vxc, 1, 4, 1);
+    cpl_test_error(CPL_ERROR_NONE);
+
+    cpl_test_eq_ptr(five, cpl_vector_unwrap(vxc));
+
+    vxc = cpl_vector_wrap(4, (double*)five + 1);
+    cpl_test_vector_abs(vxc, vxc1, 0.0);
+
+    (void)cpl_vector_unwrap(vxc);
+    cpl_vector_delete(vxc1);
+
     /* Create the vector cosinus */
     cosinus = cpl_vector_new(VECTOR_SIZE);
     cpl_test_eq( cpl_vector_get_size(sinus), VECTOR_SIZE );
@@ -781,95 +882,6 @@ int main(void)
     cpl_vector_delete(vxc1);
     cpl_vector_delete(vxc3);
 
-    /* FIXME: Enable this test when DFS06126 is fixed. */
-#if 0
-    /* Special test for cpl_vector_fit_gaussian(), provided by H. Lorch */
-    {
-    cpl_vector      *yval = NULL,
-        *xval = NULL,
-        *ysig;
-
-    cpl_matrix      *cov = NULL;
-
-    const int       N = 50.0;
-    const double    in_sigma = 10.0,
-        in_centre = 25.0,
-        peak = 769.52;
-
-    int             n;
-    double          pos = 0.0,
-        centre,
-        offset,
-        sigma,
-        area,
-        mse,
-        chisq;
-    
-
-    yval = cpl_vector_new(N);
-    ysig = cpl_vector_new(N);
-    xval = cpl_vector_new(N);
-
-    for (n = 0; n < N; n++) {
-        double  d;
-
-        d = (double)pos - in_centre;
-
-        cpl_vector_set(xval, n, pos);
-        cpl_vector_set(yval, n, peak*exp(-d*d/(2.0*in_sigma*in_sigma)));
-            
-        /* the following line seems to make it fail.
-         * normally, it should have no influence at all since all sigmas
-         * are the same. strangely, using 1.0/sqrt(N-1) also fails,
-         * but modifying this value slightly (e.g. by adding 1e-6)
-         * lets the fitting succeed. is there a meaning in the failure
-         * for 1.0/sqrt(integer)? */
-            
-        cpl_vector_set(ysig, n, 1.0/sqrt(N));
-            
-        pos+=1.0;
-        /* create one missing value,
-         * this has no special meaning, just replicates the generation of
-         * the test data with which I found the problem
-         */
-        if (n == 34)
-        pos += 1.0;
-    }
-
-
-    if (cpl_error_get_code() != CPL_ERROR_NONE)
-        cpl_msg_error(__func__, cpl_error_get_message());
-    
-    cpl_vector_dump(xval, stream);
-    cpl_vector_dump(yval, stream);
-    cpl_vector_dump(ysig, stream);
-    
-    cpl_vector_fit_gaussian(    xval,
-                    NULL,
-                    yval,
-                    ysig,
-                    CPL_FIT_ALL,
-                    &centre,
-                    &sigma,
-                    &area,
-                    &offset,
-                    &mse,
-                    &chisq,
-                    &cov);
-                                
-    if (cpl_error_get_code() != CPL_ERROR_NONE)
-        cpl_msg_error(__func__, cpl_error_get_message());
-    else
-        cpl_msg_info(__func__, "Success.");
-    
-    cpl_vector_delete(yval);
-    cpl_vector_delete(ysig);
-    cpl_vector_delete(xval);
-    cpl_matrix_delete(cov);
-    
-    }
-#endif
-
     if (stream != stdout) cpl_test_zero( fclose(stream) );
 
     /* End of tests */
@@ -890,7 +902,7 @@ static void cpl_vector_corr_bench(int n)
 {
 
     double       secs;
-    const int    nsize   = 10*VECTOR_SIZE*VECTOR_SIZE;
+    const cpl_size nsize   = 10*VECTOR_SIZE*VECTOR_SIZE;
     cpl_vector * cosinus = cpl_vector_new(nsize);
     cpl_vector * vxc = cpl_vector_new(5*VECTOR_SIZE | 1);
     double     * data = cpl_vector_get_data(cosinus);
@@ -915,9 +927,9 @@ static void cpl_vector_corr_bench(int n)
     flops0 = cpl_tools_get_flops() - flops0;
 
     if (secs > 0.0) {
-        cpl_msg_info("","Speed during %d correlations of size %d in %g secs "
-                     "[Mflop/s]: %g (%g)", n, nsize, secs, flops0/secs/1e6,
-                     (double)flops0);
+        cpl_msg_info("","Speed during %d correlations of size %" CPL_SIZE_FORMAT
+                     " in %g secs [Mflop/s]: %g (%g)", n, nsize, secs,
+                     flops0/secs/1e6, (double)flops0);
         cpl_msg_info(cpl_func,"Processing rate [MB/s]: %g",
                      1e-6 * (double)bytes / secs);
     }
@@ -937,7 +949,7 @@ static void cpl_vector_get_stdev_bench(int n)
 {
 
     double       secs;
-    const int    nsize   = 10 * VECTOR_SIZE*VECTOR_SIZE;
+    const cpl_size nsize   = 10 * VECTOR_SIZE*VECTOR_SIZE;
     cpl_vector * cosinus = cpl_vector_new(nsize);
     double     * data = cpl_vector_get_data(cosinus);
     cpl_flops    flops0;
@@ -963,9 +975,9 @@ static void cpl_vector_get_stdev_bench(int n)
     flops0 = cpl_tools_get_flops() - flops0;
 
     if (secs > 0.0) {
-        cpl_msg_info("","Speed during %d standard devs of size %d in %g secs "
-                     "[Mflop/s]: %g (%g)", n, nsize, secs, flops0/secs/1e6,
-                     (double)flops0);
+        cpl_msg_info(cpl_func,"Speed during %d standard devs of size %"
+                     CPL_SIZE_FORMAT "in %g secs [Mflop/s]: %g (%g)", n, nsize,
+                     secs, flops0/secs/1e6, (double)flops0);
         cpl_msg_info(cpl_func,"Processing rate [MB/s]: %g",
                      1e-6 * (double)bytes / secs);
     }
@@ -1015,7 +1027,7 @@ static void cpl_vector_save_bench(int n)
 
     for (i = 0; i < n; i++) {
 
-       cpl_test_zero( cpl_vector_save(vec, filename, CPL_BPP_IEEE_DOUBLE,
+       cpl_test_zero( cpl_vector_save(vec, filename, CPL_TYPE_DOUBLE,
                                            qclist, CPL_IO_CREATE));
     }
 
@@ -1037,3 +1049,92 @@ static void cpl_vector_save_bench(int n)
     return;
 
 }
+
+
+/*----------------------------------------------------------------------------*/
+/**
+  @brief    Reproduce DFS06126, original version by H. Lorch
+  @param    stream Output dump stream
+  @return   void
+ */
+/*----------------------------------------------------------------------------*/
+static void cpl_vector_fit_gaussian_test_one(FILE * stream)
+{
+
+    const int       N = 50;
+    cpl_vector      *yval = cpl_vector_new(N);
+    cpl_vector      *xval = cpl_vector_new(N);
+    cpl_vector      *ysig = cpl_vector_new(N);
+
+    cpl_matrix      *cov = NULL;
+
+    const double    in_sigma = 10.0,
+        in_centre = 25.0,
+        peak = 769.52;
+
+    int             n;
+    double          pos = 0.0,
+        centre,
+        offset,
+        sigma,
+        area,
+        mse,
+        chisq;
+    cpl_error_code error;
+    
+
+    for (n = 0; n < N; n++) {
+        const double d = (double)pos - in_centre;
+
+        error = cpl_vector_set(xval, n, pos);
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
+        error = cpl_vector_set(yval, n, peak*exp(-d*d/(2.0*in_sigma*in_sigma)));
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
+            
+        /* the following line seems to make it fail.
+         * normally, it should have no influence at all since all sigmas
+         * are the same. strangely, using 1.0/sqrt(N-1) also fails,
+         * but modifying this value slightly (e.g. by adding 1e-6)
+         * lets the fitting succeed. is there a meaning in the failure
+         * for 1.0/sqrt(integer)? */
+            
+        error = cpl_vector_set(ysig, n, 1.0/sqrt(N));
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
+            
+        pos += 1.0;
+        /* create one missing value,
+         * this has no special meaning, just replicates the generation of
+         * the test data with which I found the problem
+         */
+        if (n == 34)
+            pos += 1.0;
+    }
+
+
+    if (cpl_msg_get_level() <= CPL_MSG_INFO) {
+        cpl_vector_dump(xval, stream);
+        cpl_vector_dump(yval, stream);
+        cpl_vector_dump(ysig, stream);
+    }
+
+    error = cpl_vector_fit_gaussian(xval,
+                                    NULL,
+                                    yval,
+                                    ysig,
+                                    CPL_FIT_ALL,
+                                    &centre,
+                                    &sigma,
+                                    &area,
+                                    &offset,
+                                    &mse,
+                                    &chisq,
+                                    &cov);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    cpl_test_nonnull(cov);
+
+    cpl_vector_delete(yval);
+    cpl_vector_delete(ysig);
+    cpl_vector_delete(xval);
+    cpl_matrix_delete(cov);
+    
+}
