@@ -1,4 +1,4 @@
-/* $Id: cpl_dfs.c,v 1.121 2011/01/25 12:12:29 llundin Exp $
+/* $Id: cpl_dfs.c,v 1.130 2012/02/14 14:38:17 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2011/01/25 12:12:29 $
- * $Revision: 1.121 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2012/02/14 14:38:17 $
+ * $Revision: 1.130 $
+ * $Name: cpl-6_0 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -34,6 +34,7 @@
 #include <cpl_memory.h>
 #include <cpl_tools.h>
 #include <cpl_error_impl.h>
+#include <cpl_io_fits.h>
 
 #include "cpl_dfs.h"
 
@@ -117,9 +118,7 @@
 static char * cpl_dfs_extract_printable(const char *) CPL_ATTR_ALLOC;
 #endif
 
-static
-cpl_error_code cpl_dfs_update_key_string(fitsfile *, const char *, const char *,
-                                         const char *);
+static cpl_error_code cpl_dfs_update_product_header_(cpl_frameset *);
 
 static char * cpl_dfs_find_md5sum(fitsfile *) CPL_ATTR_ALLOC;
 
@@ -138,8 +137,8 @@ cpl_error_code cpl_dfs_paf_dump_string(const char *, const char *,
 static cpl_error_code cpl_dfs_paf_dump_double(const char *, double,
                                               const char *, FILE *);
 
-static cpl_error_code cpl_dfs_paf_dump_int(const char *, int, const char *,
-                                          FILE *);
+static cpl_error_code cpl_dfs_paf_dump_int(const char *, cpl_size,
+                                          const char *, FILE *);
 
 static cpl_error_code cpl_dfs_product_save(cpl_frameset *,
                                            cpl_propertylist *,
@@ -148,7 +147,7 @@ static cpl_error_code cpl_dfs_product_save(cpl_frameset *,
                                            const cpl_frame *,
                                            const cpl_imagelist *,
                                            const cpl_image *,
-                                           cpl_type_bpp,
+                                           cpl_type,
                                            const cpl_table *,
                                            const cpl_propertylist *,
                                            const char *,
@@ -172,7 +171,7 @@ static cpl_error_code cpl_dfs_product_save(cpl_frameset *,
   @param  usedframes The list of raw/calibration frames used for this product
   @param  inherit    NULL or product frames inherit their header from this frame
   @param  image      The image to be saved
-  @param  bpp        Bits per pixel
+  @param  type       The type used to represent the data in the file
   @param  recipe     The recipe name
   @param  applist    Propertylist to append to primary header, w. PRO.CATG
   @param  remregexp  Optional regexp of properties not to put in main header
@@ -199,21 +198,17 @@ cpl_error_code cpl_dfs_save_image(cpl_frameset            * allframes,
                                   const cpl_frameset      * usedframes,
                                   const cpl_frame         * inherit,
                                   const cpl_image         * image,
-                                  cpl_type_bpp              bpp,
+                                  cpl_type                  type,
                                   const char              * recipe,
                                   const cpl_propertylist  * applist,
                                   const char              * remregexp,
                                   const char              * pipe_id,
                                   const char              * filename)
 {
-    const cpl_error_code error =
-        cpl_dfs_product_save(allframes, header, parlist, usedframes, inherit,
-                             NULL, image, bpp, NULL, NULL, recipe, applist,
-                             remregexp, pipe_id, filename);
-
-    cpl_ensure_code(!error, error);
-
-    return CPL_ERROR_NONE;
+    return cpl_dfs_product_save(allframes, header, parlist, usedframes, inherit,
+                                NULL, image, type, NULL, NULL, recipe, applist,
+                                remregexp, pipe_id, filename)
+        ? cpl_error_set_where_() : CPL_ERROR_NONE;
 
 }
 
@@ -227,7 +222,7 @@ cpl_error_code cpl_dfs_save_image(cpl_frameset            * allframes,
   @param  usedframes The list of raw/calibration frames used for this product
   @param  inherit    NULL or product frames inherit their header from this frame
   @param  recipe     The recipe name
-  @param  applist    Optional propertylist to append to primary header or NULL
+  @param  applist    Propertylist to append to primary header, w. PRO.CATG
   @param  remregexp  Optional regexp of properties not to put in main header
   @param  pipe_id    PACKAGE "/" PACKAGE_VERSION
   @param  filename   Filename of created product
@@ -254,15 +249,11 @@ cpl_error_code cpl_dfs_save_propertylist(cpl_frameset            * allframes,
                                          const char              * pipe_id,
                                          const char              * filename)
 {
-    const cpl_error_code error =
-        cpl_dfs_product_save(allframes, header, parlist, usedframes, inherit,
-                             NULL, NULL, CPL_BPP_IEEE_FLOAT, NULL, NULL, recipe,
-                             applist, remregexp, pipe_id, filename);
-
-    cpl_ensure_code(!error, error);
-
-    return CPL_ERROR_NONE;
-
+    /* Use CPL_TYPE_INVALID to ensure it is not referenced */
+    return cpl_dfs_product_save(allframes, header, parlist, usedframes, inherit,
+                                NULL, NULL, CPL_TYPE_INVALID, NULL, NULL,
+                                recipe, applist, remregexp, pipe_id, filename)
+        ? cpl_error_set_where_() : CPL_ERROR_NONE;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -274,9 +265,9 @@ cpl_error_code cpl_dfs_save_propertylist(cpl_frameset            * allframes,
   @param  usedframes The list of raw/calibration frames used for this product
   @param  inherit    NULL or product frames inherit their header from this frame
   @param  imagelist  The imagelist to be saved
-  @param  bpp        Bits per pixel
+  @param  type       The type used to represent the data in the file
   @param  recipe     The recipe name
-  @param  applist    Optional propertylist to append to primary header or NULL
+  @param  applist    Propertylist to append to primary header, w. PRO.CATG
   @param  remregexp  Optional regexp of properties not to put in main header
   @param  pipe_id    PACKAGE "/" PACKAGE_VERSION
   @param  filename   Filename of created product
@@ -292,25 +283,19 @@ cpl_error_code cpl_dfs_save_imagelist(cpl_frameset            * allframes,
                                       const cpl_frameset      * usedframes,
                                       const cpl_frame         * inherit,
                                       const cpl_imagelist     * imagelist,
-                                      cpl_type_bpp              bpp,
+                                      cpl_type                  type,
                                       const char              * recipe,
                                       const cpl_propertylist  * applist,
                                       const char              * remregexp,
                                       const char              * pipe_id,
                                       const char              * filename)
 {
-    cpl_error_code error;
-
     cpl_ensure_code(imagelist != NULL, CPL_ERROR_NULL_INPUT);
 
-    error = cpl_dfs_product_save(allframes, header, parlist, usedframes,
-                                 inherit, imagelist, NULL, bpp, NULL, NULL,
-                                 recipe, applist, remregexp, pipe_id, filename);
-
-    cpl_ensure_code(!error, error);
-
-    return CPL_ERROR_NONE;
-
+    return cpl_dfs_product_save(allframes, header, parlist, usedframes,
+                                inherit, imagelist, NULL, type, NULL, NULL,
+                                recipe, applist, remregexp, pipe_id, filename)
+        ? cpl_error_set_where_() : CPL_ERROR_NONE;
 }
 
 
@@ -325,7 +310,7 @@ cpl_error_code cpl_dfs_save_imagelist(cpl_frameset            * allframes,
   @param  table      The table to be saved
   @param  tablelist  Optional propertylist to use in table extension or NULL
   @param  recipe     The recipe name
-  @param  applist    Optional propertylist to append to primary header or NULL
+  @param  applist    Propertylist to append to primary header, w. PRO.CATG
   @param  remregexp  Optional regexp of properties not to put in main header
   @param  pipe_id    PACKAGE "/" PACKAGE_VERSION
   @param  filename   Filename of created product
@@ -348,18 +333,13 @@ cpl_error_code cpl_dfs_save_table(cpl_frameset            * allframes,
                                   const char              * filename)
 {
 
-    cpl_error_code error;
-
     cpl_ensure_code(table != NULL, CPL_ERROR_NULL_INPUT);
 
-    error = cpl_dfs_product_save(allframes, header, parlist, usedframes,
-                                 inherit, NULL, NULL, CPL_BPP_IEEE_FLOAT, table,
-                                 tablelist, recipe, applist, remregexp, pipe_id,
-                                 filename);
-
-    cpl_ensure_code(!error, error);
-
-    return CPL_ERROR_NONE;
+    /* Use CPL_TYPE_INVALID to ensure it is not referenced */
+    return cpl_dfs_product_save(allframes, header, parlist, usedframes, inherit,
+                                NULL, NULL, CPL_TYPE_INVALID, table, tablelist,
+                                recipe, applist, remregexp, pipe_id, filename)
+        ? cpl_error_set_where_() : CPL_ERROR_NONE;
 }
 
 
@@ -648,7 +628,7 @@ cpl_error_code cpl_dfs_setup_product_header(cpl_propertylist *header,
     }
 
     if (first_frame == NULL) {
-        return cpl_error_set_message(cpl_func, CPL_ERROR_DATA_NOT_FOUND,
+        return cpl_error_set_message_(CPL_ERROR_DATA_NOT_FOUND,
                                      "Empty set-of-frames");
     }
 
@@ -669,7 +649,7 @@ cpl_error_code cpl_dfs_setup_product_header(cpl_propertylist *header,
         if (found) {
             switch (cpl_is_fits(cpl_frame_get_filename(inherit_frame))) {
             case 0:
-                return cpl_error_set_message(cpl_func, CPL_ERROR_ILLEGAL_INPUT,
+                return cpl_error_set_message_(CPL_ERROR_ILLEGAL_INPUT,
                 "Cannot inherit information from non-FITS file");
             case 1:
                 break;
@@ -678,7 +658,7 @@ cpl_error_code cpl_dfs_setup_product_header(cpl_propertylist *header,
             }
         }
         else {
-            return cpl_error_set_message(cpl_func, CPL_ERROR_ILLEGAL_INPUT,
+            return cpl_error_set_message_(CPL_ERROR_ILLEGAL_INPUT,
             "The reference frame does not belong to the input set-of-frames");
         }
     }
@@ -793,7 +773,7 @@ cpl_error_code cpl_dfs_setup_product_header(cpl_propertylist *header,
 
     kname = cpl_frame_get_tag(product_frame);
     if (kname == NULL) {
-        return cpl_error_set_message(cpl_func, CPL_ERROR_ILLEGAL_INPUT,
+        return cpl_error_set_message_(CPL_ERROR_ILLEGAL_INPUT,
                                      "Product frame has no tag");
     }
     cpl_propertylist_update_string(header, PRO_CATG, kname);
@@ -1010,8 +990,8 @@ cpl_error_code cpl_dfs_setup_product_header(cpl_propertylist *header,
     param = cpl_parameterlist_get_first_const(parlist);
 
     while (param != NULL) {
-        const char * pval;
-        const char * dval; /* The default value for value comment */
+        char * pval;
+        char * dval; /* The default value for value comment */
 #if defined CFITSIO_MAJOR && (CFITSIO_MAJOR > 3 || CFITSIO_MINOR >= 26)
         char * help = NULL;
         const char * comment = cpl_parameter_get_help(param);
@@ -1127,9 +1107,32 @@ cpl_error_code cpl_dfs_setup_product_header(cpl_propertylist *header,
 cpl_error_code cpl_dfs_update_product_header(cpl_frameset * self)
 {
 
-    const char      * md5key  = DATAMD5;
+    const cpl_error_code error = cpl_dfs_update_product_header_(self);
+    int status = 0;
+
+    if (cpl_io_fits_close_all(&status)) {
+        return cpl_error_set_fits(CPL_ERROR_FILE_IO, status, fits_close_file,
+                                  "Could not close all FITS files");
+    }
+
+    return error ? cpl_error_set_where_() : CPL_ERROR_NONE;
+}
+
+/**@}*/
+
+
+/*----------------------------------------------------------------------------*/
+/**
+  @internal
+  @brief  Perform any DFS-compliancy required actions (DATAMD5/PIPEFILE update)
+  @param  self     The list of frames with FITS products created by the recipe
+  @return CPL_ERROR_NONE or the relevant CPL error code on error
+  @see cpl_dfs_update_product_header()
+ */
+/*----------------------------------------------------------------------------*/
+static cpl_error_code cpl_dfs_update_product_header_(cpl_frameset * self)
+{
     const cpl_frame * frame;
-    int               nupdate = 0;
 
 
     cpl_ensure_code(self != NULL, CPL_ERROR_NULL_INPUT);
@@ -1146,57 +1149,69 @@ cpl_error_code cpl_dfs_update_product_header(cpl_frameset * self)
 
             if (cpl_is_fits(filename)) {
 
-                fitsfile   * fproduct = NULL;
-                char         card[FLEN_CARD];
-                int error  = 0; /* CFITSIO requires this to be zero */
-                int error2 = 0; /* CFITSIO requires this to be zero */
+                fitsfile     * fproduct = NULL;
+                char           card[FLEN_CARD];
+                int            error = 0; /* CFITSIO requires this to be zero */
+                cpl_error_code code = CPL_ERROR_NONE;
 
 
-                if (fits_open_diskfile(&fproduct, filename, READWRITE, &error)) {
+                if (cpl_io_fits_open_diskfile(&fproduct, filename, READWRITE,
+                                              &error)) {
                     return cpl_error_set_fits(CPL_ERROR_BAD_FILE_FORMAT, error,
                                               fits_open_diskfile,
                                               "filename='%s'", filename);
                 }
 
-
-                /* fits_read_card() is missing const modifiers on the string
-                   arguments!
-                   fits_read_card() is trusted not to actually modify these
-                   strings
-                   - if that fails the behavior is undefined (e.g. segmentation
-                   violation).
-                   :-(((((((((((((((((((((((((((((((((((((((((((((( */
-                if (fits_read_card(fproduct, (char*)md5key, card, &error)) {
-                    (void)cpl_error_set_fits(CPL_ERROR_BAD_FILE_FORMAT, error,
-                                             fits_read_card, "filename='%s', "
-                                             "MD5=%s", filename, md5key);
+                if (fits_read_card(fproduct, DATAMD5, card, &error)) {
+                    code = cpl_error_set_fits(CPL_ERROR_BAD_FILE_FORMAT, error,
+                                              fits_read_card, "filename='%s', "
+                                              "key='" DATAMD5 "'", filename);
+                    error = 0;
                 } else {
 
-                    /* assert( !error ); */
-
                     /* The fits header has the MD5-card, update it */
                     char * md5sum = cpl_dfs_find_md5sum(fproduct);
 
-                    if (md5sum != NULL) {
-                        error = cpl_dfs_update_key_string(fproduct, md5key,
-                                                          md5sum, "MD5 checksum");
-                        if (!error) nupdate++;
+                    if (!md5sum) {
+                        code = cpl_error_set_where_();
+                    } else {
+                        /* fits_update_key_str() is missing const modifiers on
+                           the value and comment string arguments!
+                           fits_update_key_str() is trusted not to actually
+                           modify these strings
+                           - if that fails the behavior is undefined
+                           (e.g. segmentation violation).
+                           :-(((((((((((((((((((((((((((((((((((((((((((((( */
+
+                        if (fits_update_key_str(fproduct, DATAMD5, md5sum,
+                                                (char*)"MD5 checksum",
+                                                &error)) {
+                            code = cpl_error_set_fits(CPL_ERROR_FILE_IO, error,
+                                                      fits_update_key_str,
+                                                      "filename='%s', key='"
+                                                      DATAMD5 "', value='%s'",
+                                                      filename, md5sum);
+                            error = 0;
+                        } else if (fits_update_key_str(fproduct, PIPEFILE,
+                                                       (char*)filename,
+                                                       (char*)"Filename of data"
+                                                       " product", &error)) {
+                            code = cpl_error_set_fits(CPL_ERROR_FILE_IO, error,
+                                                      fits_update_key_str,
+                                                      "filename='%s', key='"
+                                                      PIPEFILE "'", filename);
+                            error = 0;
+                        }
                         cpl_free(md5sum);
                     }
-
-                    if (!error) error =
-                        cpl_dfs_update_key_string(fproduct, PIPEFILE,
-                                                  filename,
-                                                  "Filename of data "
-                                                  "product");
                 }
 
-                if (fits_close_file(fproduct, &error2)) {
-                    return cpl_error_set_fits(CPL_ERROR_FILE_IO, error2,
+                if (cpl_io_fits_close_file(fproduct, &error)) {
+                    return cpl_error_set_fits(CPL_ERROR_FILE_IO, error,
                                               fits_close_file,
                                               "filename='%s'", filename);
                 }
-                cpl_ensure_code(!error, error);
+                if (code) return code;
             }
         }
     }
@@ -1205,7 +1220,6 @@ cpl_error_code cpl_dfs_update_product_header(cpl_frameset * self)
 
 }
 
-/**@}*/
 
 /*----------------------------------------------------------------------------*/
 /**
@@ -1335,13 +1349,13 @@ static char * cpl_dfs_find_md5sum(fitsfile * fproduct)
         }
         next++;
 
-    } while (!fits_movrel_hdu(fproduct, 1, NULL, &error));
+    } while (!fits_movabs_hdu(fproduct, 1 + next, NULL, &error));
 
     if (error == END_OF_FILE) {
         error = 0; /* Reset CFITSIO error */
     } else {
         (void)cpl_error_set_fits(CPL_ERROR_BAD_FILE_FORMAT, error,
-                                 fits_movrel_hdu, "HDU#=%d", next);
+                                 fits_movabs_hdu, "HDU#=%d", next);
         return NULL;
     }
 
@@ -1376,41 +1390,6 @@ static char * cpl_dfs_find_md5sum(fitsfile * fproduct)
 
 }
 
-/*----------------------------------------------------------------------------*/
-/**
-  @internal
-  @brief  Update a FITS key in an CFITSIO open file
-  @param  fproduct The CFITSIO FITS file structure
-  @param  key      The key of the card to update
-  @param  value    The value to set
-  @param  value    The (optional) comment to set
-  @return CPL_ERROR_NONE or the relevant CPL error code on error
-
- */
-/*----------------------------------------------------------------------------*/
-static cpl_error_code cpl_dfs_update_key_string(fitsfile * fproduct,
-                                                const char * key,
-                                                const char * value,
-                                                const char * comment)
-{
-
-    int error = 0; /* CFITSIO requires this to be zero */
-
-    cpl_ensure_code(fproduct, CPL_ERROR_NULL_INPUT);
-    cpl_ensure_code(key,      CPL_ERROR_NULL_INPUT);
-    cpl_ensure_code(value,    CPL_ERROR_NULL_INPUT);
-
-    /* fits_update_key_str() is missing const modifiers on the string arguments!
-       fits_update_key_str() is trusted not to actually modify these strings
-       - if that fails the behavior is undefined (e.g. segmentation violation).
-       :-(((((((((((((((((((((((((((((((((((((((((((((( */
-    return fits_update_key_str(fproduct, (char*)key, (char*)value,
-                               (char*)comment, &error)
-        ? cpl_error_set_fits(CPL_ERROR_FILE_IO, error, fits_update_key_str,
-                             "key='%s', value='%s'", key, value)
-        : CPL_ERROR_NONE;
-}
-
 
 /*----------------------------------------------------------------------------*/
 /**
@@ -1430,7 +1409,7 @@ static int cpl_is_fits(const char * self)
 
     cpl_ensure(self != NULL, CPL_ERROR_NULL_INPUT, -1);
 
-    if (fits_open_diskfile(&fptr, self, READONLY, &error)) {
+    if (cpl_io_fits_open_diskfile(&fptr, self, READONLY, &error)) {
         is_fits = error == FILE_NOT_OPENED ? -2 : 0;
 
         if (is_fits < 0)
@@ -1447,7 +1426,7 @@ static int cpl_is_fits(const char * self)
         error = 0;
     }
 
-    if (fptr != NULL && fits_close_file(fptr, &error) != 0) {
+    if (fptr != NULL && cpl_io_fits_close_file(fptr, &error) != 0) {
         /* FIXME: Is this even possible ? */
         is_fits = -1;
         (void)cpl_error_set_fits(CPL_ERROR_BAD_FILE_FORMAT, error,
@@ -1573,10 +1552,9 @@ static FILE * cpl_dfs_paf_init(const char * instrume,
             (void)fclose(paf);
             paf = NULL;
         }
-        (void)cpl_error_set_message_macro(cpl_func, error, __FILE__, __LINE__,
-                                          "Could not write PAF, instrume=%s, "
-                                          "recipe=%s, filename=%s", instrume,
-                                          recipe, filename);
+        (void)cpl_error_set_message_(error, "Could not write PAF, instrume=%s, "
+                                     "recipe=%s, filename=%s", instrume,
+                                     recipe, filename);
     }
 
     return paf;
@@ -1599,9 +1577,9 @@ static FILE * cpl_dfs_paf_init(const char * instrume,
   will cause the printing of a line with QC.PART1.PART2.PART3 as key.
 
   Supported property-types:
-  CPL_TYPE_CHAR (cast to int)
-  CPL_TYPE_INT 
-  CPL_TYPE_LONG (only if sizeof(long) == sizeof(int), cast to int)
+  CPL_TYPE_CHAR  (cast to cpl_size)
+  CPL_TYPE_INT   (cast to cpl_size)
+  CPL_TYPE_LONG  (cast to cpl_size)
   CPL_TYPE_FLOAT (cast to double)
   CPL_TYPE_DOUBLE
   CPL_TYPE_STRING
@@ -1616,8 +1594,8 @@ static FILE * cpl_dfs_paf_init(const char * instrume,
 static
 cpl_error_code cpl_dfs_paf_dump(const cpl_propertylist * self, FILE * paf)
 {
-    const int nproperties = cpl_propertylist_get_size(self);
-    int i;
+    const cpl_size nproperties = cpl_propertylist_get_size(self);
+    cpl_size i;
 
     cpl_ensure_code(self, CPL_ERROR_NULL_INPUT);
     cpl_ensure_code(paf,  CPL_ERROR_NULL_INPUT);
@@ -1626,7 +1604,10 @@ cpl_error_code cpl_dfs_paf_dump(const cpl_propertylist * self, FILE * paf)
         const cpl_property * prop = cpl_propertylist_get_const(self, i);
         const char * name    = cpl_property_get_name(prop);
         const char * comment = cpl_property_get_comment(prop);
-        char * qckey;
+        const char * usekey;
+        const char * usecom;
+        char * qckey = NULL;
+        char * qccom = NULL;
         /* Initialization needed in absence of #if for CPL_TYPE_LONG */
         cpl_error_code err = CPL_ERROR_UNSUPPORTED_MODE;
 
@@ -1634,56 +1615,71 @@ cpl_error_code cpl_dfs_paf_dump(const cpl_propertylist * self, FILE * paf)
         if (strstr(name, "ESO ") == name) {
             /* Drop prefix "ESO " */
             qckey = cpl_sprintf("%s", name+4);
-        } else if (strchr(name, ' '))
+        } else if (strchr(name, ' ')) {
             qckey = cpl_strdup(name);
-        else
-            qckey = (char *)name; /* Don't need to modify the key */
+        }
 
-        if (qckey != name) {
+        if (qckey != NULL) {
             /* Replace <space> with . */
             char * p;
             for (p = qckey; *p != '\0'; p++)
                 if (*p == ' ') *p = '.';
+            usekey = qckey;
+        } else {
+            usekey = name;
         }
 
+        if (comment != NULL && strchr(comment, '\n')) {
+            char * p;
+            /* Replace line-feed with space */
+            qccom = cpl_strdup(comment);
+            for (p = qccom; *p != '\0'; p++)
+                if (*p == '\n') *p = ' ';
+            usecom = qccom;
+        } else {
+            usecom = comment;
+        }
+
+
         /* Print the property as PAF */
 
         switch (cpl_property_get_type(prop)) {
         case CPL_TYPE_CHAR:
-            err = cpl_dfs_paf_dump_int(qckey, cpl_property_get_char(prop),
-                                      comment, paf);
+            err = cpl_dfs_paf_dump_int(usekey, cpl_property_get_char(prop),
+                                      usecom, paf);
+            break;
         case CPL_TYPE_BOOL:
-            err = cpl_dfs_paf_dump_int(qckey, cpl_property_get_bool(prop),
-                                      comment, paf);
+            err = cpl_dfs_paf_dump_int(usekey, cpl_property_get_bool(prop),
+                                      usecom, paf);
             break;
         case CPL_TYPE_INT:
-            err = cpl_dfs_paf_dump_int(qckey, cpl_property_get_int(prop),
-                                      comment, paf);
+            err = cpl_dfs_paf_dump_int(usekey, cpl_property_get_int(prop),
+                                      usecom, paf);
             break;
         case CPL_TYPE_LONG:
-            if (sizeof(long) == sizeof(int)) /* FIXME: Change to #if */
-                err = cpl_dfs_paf_dump_int(qckey, cpl_property_get_long(prop),
-                                          comment, paf);
+            err = cpl_dfs_paf_dump_int(usekey, cpl_property_get_long(prop),
+                                       usecom, paf);
             break;
         case CPL_TYPE_FLOAT:
-            err = cpl_dfs_paf_dump_double(qckey, cpl_property_get_float(prop),
-                                         comment, paf);
+            err = cpl_dfs_paf_dump_double(usekey, cpl_property_get_float(prop),
+                                         usecom, paf);
             break;
         case CPL_TYPE_DOUBLE:
-            err = cpl_dfs_paf_dump_double(qckey, cpl_property_get_double(prop),
-                                         comment, paf);
+            err = cpl_dfs_paf_dump_double(usekey, cpl_property_get_double(prop),
+                                         usecom, paf);
             break;
         case CPL_TYPE_STRING:
-            err = cpl_dfs_paf_dump_string(qckey, cpl_property_get_string(prop),
-                                         comment, paf);
+            err = cpl_dfs_paf_dump_string(usekey, cpl_property_get_string(prop),
+                                         usecom, paf);
             break;
         default:
             err = CPL_ERROR_UNSUPPORTED_MODE;
         }
 
-        if (qckey != name) cpl_free(qckey);
+        cpl_free(qckey);
+        cpl_free(qccom);
 
-        cpl_ensure_code(!err, err);
+        if (err) return cpl_error_set_(err);
 
     }
 
@@ -1772,20 +1768,19 @@ static cpl_error_code cpl_dfs_paf_dump_double(const char * key,
 
  */
 /*----------------------------------------------------------------------------*/
-static cpl_error_code cpl_dfs_paf_dump_int(const char * key,
-                                          int value,
-                                          const char * comment, FILE * paf)
+static cpl_error_code cpl_dfs_paf_dump_int(const char * key, cpl_size value,
+                                           const char * comment, FILE * paf)
 {
     cpl_ensure_code(paf,   CPL_ERROR_NULL_INPUT);
     cpl_ensure_code(key,   CPL_ERROR_NULL_INPUT);
 
     if (comment == NULL)
-        cpl_ensure_code(fprintf(paf, PAF_KEY_FORMAT "%d\n",
+        cpl_ensure_code(fprintf(paf, PAF_KEY_FORMAT "%" CPL_SIZE_FORMAT "\n",
                                 key, value) > PAF_KEY_LEN,
                         CPL_ERROR_FILE_IO);
     else
-        cpl_ensure_code(fprintf(paf, PAF_KEY_FORMAT "%d ; # %s\n",
-                                key, value, comment) > PAF_KEY_LEN,
+        cpl_ensure_code(fprintf(paf, PAF_KEY_FORMAT "%" CPL_SIZE_FORMAT
+                                "; # %s\n", key, value, comment) > PAF_KEY_LEN,
                         CPL_ERROR_FILE_IO);
 
     return CPL_ERROR_NONE;
@@ -1797,6 +1792,7 @@ static cpl_error_code cpl_dfs_paf_dump_int(const char * key,
 
 /*----------------------------------------------------------------------------*/
 /**
+  @internal
   @brief  Save either an image or table as a pipeline product
   @param  allframes  The list of input frames for the recipe
   @param  header     NULL, or filled with properties written to product header
@@ -1805,11 +1801,11 @@ static cpl_error_code cpl_dfs_paf_dump_int(const char * key,
   @param  inherit    NULL, or frame from which header information is inherited
   @param  imagelist  The imagelist to be saved or NULL
   @param  image      The image to be saved or NULL
-  @param  bpp        Bits per pixel
+  @param  type       The type used to represent the data in the file
   @param  table      The table to be saved or NULL
   @param  tablelist  Optional propertylist to use in table extension or NULL
   @param  recipe     The recipe name
-  @param  applist    Optional propertylist to append to main header or NULL
+  @param  applist    Propertylist to append to primary header, w. PRO.CATG
   @param  remregexp  Optional regexp of properties not to put in main header
   @param  pipe_id    PACKAGE "/" PACKAGE_VERSION
   @param  filename   Filename of created product
@@ -1828,7 +1824,7 @@ static cpl_error_code cpl_dfs_product_save(cpl_frameset            * allframes,
                                            const cpl_frame         * inherit,
                                            const cpl_imagelist     * imagelist,
                                            const cpl_image         * image,
-                                           cpl_type_bpp              bpp,
+                                           cpl_type                  type,
                                            const cpl_table         * table,
                                            const cpl_propertylist  * tablelist,
                                            const char              * recipe,
@@ -1917,7 +1913,7 @@ static cpl_error_code cpl_dfs_product_save(cpl_frameset            * allframes,
     }
 
     /* Add any QC parameters here */
-    if (applist != NULL) error = cpl_propertylist_append(plist, applist);
+    error = cpl_propertylist_append(plist, applist);
 
     /* Add DataFlow keywords */
     if (!error)
@@ -1934,7 +1930,7 @@ static cpl_error_code cpl_dfs_product_save(cpl_frameset            * allframes,
     if (!error) {
         switch (pronum) {
             case 0:
-                error = cpl_imagelist_save(imagelist, filename, bpp, plist,
+                error = cpl_imagelist_save(imagelist, filename, type, plist,
                                            CPL_IO_CREATE);
                 break;
             case 1:
@@ -1942,7 +1938,7 @@ static cpl_error_code cpl_dfs_product_save(cpl_frameset            * allframes,
                                        CPL_IO_CREATE);
                 break;
             case 2:
-                error = cpl_image_save(image, filename, bpp, plist,
+                error = cpl_image_save(image, filename, type, plist,
                                        CPL_IO_CREATE);
                 break;
             default:
