@@ -1,4 +1,4 @@
-/* $Id: cpl_dfs-test.c,v 1.53 2011/01/24 14:13:20 llundin Exp $
+/* $Id: cpl_dfs-test.c,v 1.60 2012/02/14 14:38:17 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2011/01/24 14:13:20 $
- * $Revision: 1.53 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2012/02/14 14:38:17 $
+ * $Revision: 1.60 $
+ * $Name: cpl-6_0 $
  */
 
 /*-----------------------------------------------------------------------------
@@ -49,6 +49,8 @@
 #define IMAGE_NEXT   32
 #define CPL_DFS_RAW_ASCII "ascii.txt"
 
+#define CPL_DFS_FITSFILE "product.fits"
+
 /*-----------------------------------------------------------------------------
                             Private declarations
  -----------------------------------------------------------------------------*/
@@ -72,7 +74,7 @@ int main(void)
     cpl_dfs_save_txt(CPL_DFS_RAW_ASCII);
 
     cpl_image_save(NULL, "inherit.fits",
-           CPL_BPP_IEEE_FLOAT, NULL, CPL_IO_CREATE);
+           CPL_TYPE_FLOAT, NULL, CPL_IO_CREATE);
 
     /* Insert tests here */
 
@@ -90,7 +92,6 @@ int main(void)
 
 static void cpl_dfs_product_tests(const char * rawname)
 {
-    const char        * filename = "product.fits";
     cpl_frame         * rawframe;
     cpl_frame         * inhframe;
     cpl_frame         * proframe;
@@ -135,14 +136,17 @@ static void cpl_dfs_product_tests(const char * rawname)
 
     error = cpl_dfs_update_product_header(frameset);
 
-    cpl_test_zero(error);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
 
     /* Test 3: Check the error handling of a rawframe without a filename */
 
     rawframe = cpl_frame_new();
-    cpl_test_zero(cpl_frame_set_tag(rawframe, "TAG"));
-    cpl_test_zero(cpl_frame_set_group(rawframe, CPL_FRAME_GROUP_RAW));
-    cpl_test_zero(cpl_frameset_insert(frameset, rawframe));
+    error = cpl_frame_set_tag(rawframe, "TAG");
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    error = cpl_frame_set_group(rawframe, CPL_FRAME_GROUP_RAW);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    error = cpl_frameset_insert(frameset, rawframe);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
 
     error = cpl_dfs_setup_product_header(plist, proframe, frameset, parlist,
                                          "Recipe", "Pipeline", "PRO-1.15",
@@ -156,10 +160,14 @@ static void cpl_dfs_product_tests(const char * rawname)
 
     frameset = cpl_frameset_new();
     rawframe = cpl_frame_new();
-    cpl_test_zero(cpl_frame_set_tag(rawframe, "TAG"));
-    cpl_test_zero(cpl_frame_set_group(rawframe, CPL_FRAME_GROUP_RAW));
-    cpl_test_zero(cpl_frame_set_filename(rawframe, rawname));
-    cpl_test_zero(cpl_frameset_insert(frameset, rawframe));
+    error = cpl_frame_set_tag(rawframe, "TAG");
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    error = cpl_frame_set_group(rawframe, CPL_FRAME_GROUP_RAW);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    error = cpl_frame_set_filename(rawframe, rawname);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    error = cpl_frameset_insert(frameset, rawframe);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
 
 
     error = cpl_dfs_setup_product_header(plist, proframe, frameset, parlist,
@@ -173,7 +181,8 @@ static void cpl_dfs_product_tests(const char * rawname)
        filename as the only attribute (and a valid, non-fits rawframe)
        - should fail on missing tag */
 
-    cpl_test_zero(cpl_frame_set_filename(proframe, filename));
+    error = cpl_frame_set_filename(proframe, CPL_DFS_FITSFILE);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
 
     error = cpl_dfs_setup_product_header(plist, proframe, frameset, parlist,
                                          "Recipe", "Pipeline", "PRO-1.15",
@@ -184,16 +193,17 @@ static void cpl_dfs_product_tests(const char * rawname)
        and tagged  (and a valid, non-fits rawframe) - should fail on missing
        group */
 
-    cpl_test_zero(cpl_frame_set_tag(proframe, "PRODUCT"));
+    error = cpl_frame_set_tag(proframe, "PRODUCT");
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
 
     error = cpl_dfs_setup_product_header(plist, proframe, frameset, parlist,
                                          "Recipe", "Pipeline", "PRO-1.15",
                                          NULL);
-    cpl_test_eq_error( error, CPL_ERROR_ILLEGAL_INPUT);
+    cpl_test_eq_error(error, CPL_ERROR_ILLEGAL_INPUT);
 
     /* Set product group */
-    cpl_test_zero(cpl_frame_set_group(proframe,
-                                      CPL_FRAME_GROUP_PRODUCT));
+    error = cpl_frame_set_group(proframe, CPL_FRAME_GROUP_PRODUCT);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
 
     /* Test 5c: Check the error handling of an inherit-frame not present in
        the frameset */
@@ -206,26 +216,39 @@ static void cpl_dfs_product_tests(const char * rawname)
 
     /* Test 5: 2 simple successful calls */
 
-    cpl_test_zero(cpl_dfs_setup_product_header(plist, proframe, frameset,
-                                               parlist, "Recipe", "Pipeline",
-                                               "PRO-1.15", NULL));
+    error = cpl_dfs_setup_product_header(plist, proframe, frameset,
+                                         parlist, "Recipe", "Pipeline",
+                                         "PRO-1.15", NULL);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
 
-    cpl_msg_debug("","Size of product header: %ld",
+    cpl_msg_debug("","Size of product header: %" CPL_SIZE_FORMAT,
                   cpl_propertylist_get_size(plist));
 
     inhframe = cpl_frame_new();
-    cpl_test_zero(cpl_frame_set_tag(inhframe, "TAG"));
-    cpl_test_zero(cpl_frame_set_group(inhframe, CPL_FRAME_GROUP_RAW));
-    cpl_test_zero(cpl_frame_set_filename(inhframe, "inherit.fits"));
-    cpl_test_zero(cpl_frameset_insert(frameset, inhframe));
+    error = cpl_frame_set_tag(inhframe, "TAG");
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    error = cpl_frame_set_group(inhframe, CPL_FRAME_GROUP_RAW);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    error = cpl_frame_set_filename(inhframe, "inherit.fits");
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    error = cpl_frameset_insert(frameset, inhframe);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
 
-    cpl_test_zero(cpl_dfs_setup_product_header(plist, proframe, frameset,
-                                               parlist, "Recipe", "Pipeline",
-                                               "PRO-1.15", inhframe));
+    error = cpl_dfs_setup_product_header(plist, proframe, frameset,
+                                         parlist, "Recipe", "Pipeline",
+                                         "PRO-1.15", inhframe);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
 
-    cpl_msg_debug("","Size of product header: %ld",
+    cpl_msg_debug("","Size of product header: %" CPL_SIZE_FORMAT,
                   cpl_propertylist_get_size(plist));
 
+    cpl_propertylist_append_char(plist, "QC MYCHAR", 42);
+    cpl_propertylist_set_comment(plist, "QC MYCHAR", "42");
+
+    error = cpl_dfs_save_paf("IIINSTRUME", "RRRECIPE", plist,
+                             "cpl_dfs-test.paf");
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
 #if 0
     /*
      * FIXME: It fails if inherit_frame is an ASCII file, what is 
@@ -236,12 +259,13 @@ static void cpl_dfs_product_tests(const char * rawname)
                                                parlist, "Recipe", "Pipeline",
                                                "PRO-1.15", rawframe));
 
-    cpl_msg_debug("","Size of product header: %ld",
+    cpl_msg_debug("","Size of product header: %" CPL_SIZE_FORMAT,
                   cpl_propertylist_get_size(plist));
 #endif
     /* Test 5a: A failure due to a missing product file */
 
-    cpl_test_zero(cpl_frameset_insert(frameset, proframe));
+    error = cpl_frameset_insert(frameset, proframe);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
 
     error = cpl_dfs_update_product_header(frameset);
     cpl_test_eq_error( error, CPL_ERROR_BAD_FILE_FORMAT);
@@ -258,9 +282,10 @@ static void cpl_dfs_product_tests(const char * rawname)
     /* Test 6a: A failure on file format (missing file) */
 
     /* Make sure file does not exist (from a previously failed test) */
-    (void)remove(filename);
+    (void)remove(CPL_DFS_FITSFILE);
 
-    cpl_test_zero(cpl_frame_set_type(proframe, CPL_FRAME_TYPE_IMAGE));
+    error = cpl_frame_set_type(proframe, CPL_FRAME_TYPE_IMAGE);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
 
     error = cpl_dfs_update_product_header(frameset);
 
@@ -269,7 +294,7 @@ static void cpl_dfs_product_tests(const char * rawname)
 
     /* Test 6a: Also support non-fits */
 
-    cpl_dfs_save_txt(filename);
+    cpl_dfs_save_txt(CPL_DFS_FITSFILE);
 
     error = cpl_dfs_update_product_header(frameset);
 
@@ -279,16 +304,17 @@ static void cpl_dfs_product_tests(const char * rawname)
     /* Test 7a: A successful call of cpl_dfs_update_product_header()
        with an empty data-unit */
 
-    cpl_test_zero(cpl_image_save(NULL, filename, CPL_BPP_8_UNSIGNED,
+    cpl_test_zero(cpl_image_save(NULL, CPL_DFS_FITSFILE, CPL_TYPE_UCHAR,
                                       plist, CPL_IO_CREATE));
-    cpl_test_fits(filename);
+    cpl_test_fits(CPL_DFS_FITSFILE);
 
-    cpl_test_zero(cpl_dfs_update_product_header(frameset));
-    cpl_test_fits(filename);
+    error = cpl_dfs_update_product_header(frameset);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    cpl_test_fits(CPL_DFS_FITSFILE);
 
     cpl_propertylist_delete(plist);
 
-    plist = cpl_propertylist_load(filename, 0);
+    plist = cpl_propertylist_load(CPL_DFS_FITSFILE, 0);
 
     cpl_test_nonnull(plist);
 
@@ -306,16 +332,17 @@ static void cpl_dfs_product_tests(const char * rawname)
 
     image = cpl_image_new(2, 3, CPL_TYPE_INT);
 
-    cpl_test_zero(cpl_image_save(image, filename, CPL_BPP_8_UNSIGNED,
+    cpl_test_zero(cpl_image_save(image, CPL_DFS_FITSFILE, CPL_TYPE_UCHAR,
                                       plist, CPL_IO_CREATE));
-    cpl_test_fits(filename);
+    cpl_test_fits(CPL_DFS_FITSFILE);
 
-    cpl_test_zero(cpl_dfs_update_product_header(frameset));
-    cpl_test_fits(filename);
+    error = cpl_dfs_update_product_header(frameset);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    cpl_test_fits(CPL_DFS_FITSFILE);
 
     cpl_propertylist_delete(plist);
 
-    plist = cpl_propertylist_load(filename, 0);
+    plist = cpl_propertylist_load(CPL_DFS_FITSFILE, 0);
 
     cpl_test_nonnull(plist);
 
@@ -332,16 +359,17 @@ static void cpl_dfs_product_tests(const char * rawname)
     /* Test 7c: A successful call of cpl_dfs_update_product_header()
                 with a main HDU and one data-less extension */
 
-    cpl_test_zero(cpl_image_save(NULL, filename, CPL_BPP_8_UNSIGNED,
+    cpl_test_zero(cpl_image_save(NULL, CPL_DFS_FITSFILE, CPL_TYPE_UCHAR,
                                       plist, CPL_IO_EXTEND));
-    cpl_test_fits(filename);
+    cpl_test_fits(CPL_DFS_FITSFILE);
 
-    cpl_test_zero(cpl_dfs_update_product_header(frameset));
-    cpl_test_fits(filename);
+    error = cpl_dfs_update_product_header(frameset);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    cpl_test_fits(CPL_DFS_FITSFILE);
 
     cpl_propertylist_delete(plist);
 
-    plist = cpl_propertylist_load(filename, 0);
+    plist = cpl_propertylist_load(CPL_DFS_FITSFILE, 0);
 
     cpl_test_nonnull(plist);
 
@@ -360,16 +388,17 @@ static void cpl_dfs_product_tests(const char * rawname)
                 with a main HDU, one data-less extension and
                 one non-empty extension. */
 
-    cpl_test_zero(cpl_image_save(image, filename, CPL_BPP_8_UNSIGNED,
+    cpl_test_zero(cpl_image_save(image, CPL_DFS_FITSFILE, CPL_TYPE_UCHAR,
                                       plist, CPL_IO_EXTEND));
-    cpl_test_fits(filename);
+    cpl_test_fits(CPL_DFS_FITSFILE);
 
-    cpl_test_zero(cpl_dfs_update_product_header(frameset));
-    cpl_test_fits(filename);
+    error = cpl_dfs_update_product_header(frameset);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    cpl_test_fits(CPL_DFS_FITSFILE);
 
     cpl_propertylist_delete(plist);
 
-    plist = cpl_propertylist_load(filename, 0);
+    plist = cpl_propertylist_load(CPL_DFS_FITSFILE, 0);
 
     cpl_test_nonnull(plist);
 
@@ -393,20 +422,25 @@ static void cpl_dfs_product_tests(const char * rawname)
 
     for (i = 0; i < next; i++) {
 
-        cpl_test_zero(cpl_image_add_scalar(image, 1.0));
+        error = cpl_image_add_scalar(image, 1.0);
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
 
-        cpl_test_zero(cpl_image_save(image, filename, CPL_BPP_8_UNSIGNED,
-                                          plist, CPL_IO_EXTEND));
-        cpl_test_fits(filename);
+        error = cpl_image_save(image, i % 2 ? CPL_DFS_FITSFILE
+                               : "./" CPL_DFS_FITSFILE, i >= 255 ? CPL_TYPE_INT
+                               : CPL_TYPE_UCHAR, plist, CPL_IO_EXTEND);
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+        cpl_test_fits(CPL_DFS_FITSFILE);
 
     }
 
-    cpl_test_zero(cpl_dfs_update_product_header(frameset));
-    cpl_test_fits(filename);
+    error = cpl_dfs_update_product_header(frameset);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    cpl_test_fits(CPL_DFS_FITSFILE);
 
     cpl_propertylist_delete(plist);
 
-    plist = cpl_propertylist_load(filename, 0);
+    plist = cpl_propertylist_load(CPL_DFS_FITSFILE, 0);
 
     cpl_test_nonnull(plist);
 
@@ -422,7 +456,13 @@ static void cpl_dfs_product_tests(const char * rawname)
        -e 'grep(print(sprintf("%c",$_) x (1009*1031)."\0"x(1042560-1009*1031)),'
        -e '1..32)' | md5sum -
     */
+#if defined IMAGE_NEXT && IMAGE_NEXT == 32
+#if defined IMAGE_SIZE_X && IMAGE_SIZE_X == 1009
+#if defined IMAGE_SIZE_Y && IMAGE_SIZE_X == 1031
     cpl_test_eq_string(md5sum, "503bcbb6fd6606ac1d035e2a43f0229c");
+#endif
+#endif
+#endif
 
     cpl_image_delete(image);
 
@@ -459,17 +499,20 @@ static void cpl_dfs_product_tests(const char * rawname)
 
                     newframe = cpl_frame_duplicate(frame);
 
-                    cpl_test_zero(cpl_frame_set_filename(newframe, newname));
+                    error = cpl_frame_set_filename(newframe, newname);
+                    cpl_test_eq_error(error, CPL_ERROR_NONE);
                     cpl_free(newname);
 
-                    cpl_test_zero(cpl_frameset_insert(newframes, newframe));
+                    error = cpl_frameset_insert(newframes, newframe);
+                    cpl_test_eq_error(error, CPL_ERROR_NONE);
 
                 }
             }
         }
 
         cpl_msg_info("", "Renamed %d product(s)", i);
-        cpl_test_zero(cpl_dfs_update_product_header(newframes));
+        error = cpl_dfs_update_product_header(newframes);
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
         cpl_frameset_delete(newframes);
 
         /* Do not remove file(s) if there was a failure */
@@ -502,7 +545,6 @@ static void cpl_dfs_save_tests(const char * rawname)
     const cpl_boolean is_debug
         = cpl_msg_get_level() <= CPL_MSG_DEBUG? CPL_TRUE : CPL_FALSE;
     cpl_error_code      error;
-    const char        * filename = "product.fits";
     const char        * remregexp = "DROP";
     cpl_frameset      * frames   = NULL;
     cpl_propertylist  * tlist    = NULL;
@@ -517,25 +559,25 @@ static void cpl_dfs_save_tests(const char * rawname)
 
     /* Test 1: Check the error handling of NULL-pointer(s) */
     error = cpl_dfs_save_image(frames, NULL, parlist, frames, NULL, image,
-                               CPL_BPP_8_UNSIGNED,
+                               CPL_TYPE_UCHAR,
                                "recipe", qclist, "none", "pipe_id",
-                               filename);
+                               CPL_DFS_FITSFILE);
 
     cpl_test_eq(error, CPL_ERROR_NULL_INPUT);
 
     error = cpl_dfs_save_imagelist(frames, NULL, parlist, frames, NULL, imlist,
-                                   CPL_BPP_8_UNSIGNED, "recipe",
-                                   qclist, "none", "pipe_id", filename);
+                                   CPL_TYPE_UCHAR, "recipe",
+                                   qclist, "none", "pipe_id", CPL_DFS_FITSFILE);
 
     cpl_test_eq_error( error, CPL_ERROR_NULL_INPUT);
 
     error = cpl_dfs_save_table(frames, NULL, parlist, frames, NULL, table,
                                tlist, "recipe", qclist, "none", "pipe_id",
-                               filename);
+                               CPL_DFS_FITSFILE);
 
     cpl_test_eq_error( error, CPL_ERROR_NULL_INPUT);
 
-    error = cpl_dfs_save_paf("INSTRUME", "recipe", qclist, filename);
+    error = cpl_dfs_save_paf("INSTRUME", "recipe", qclist, CPL_DFS_FITSFILE);
 
     cpl_test_eq_error( error, CPL_ERROR_NULL_INPUT);
 
@@ -559,14 +601,14 @@ static void cpl_dfs_save_tests(const char * rawname)
 
     /* Test 2: Check handling of empty objects (frameset) */
     error = cpl_dfs_save_image(frames, NULL, parlist, frames, NULL, image,
-                               CPL_BPP_8_UNSIGNED,
+                               CPL_TYPE_UCHAR,
                                "recipe", qclist, "none", "pipe_id",
                                "image" CPL_DFS_FITS);
 
     cpl_test_eq_error( error, CPL_ERROR_DATA_NOT_FOUND);
 
     error = cpl_dfs_save_imagelist(frames, NULL, parlist, frames, NULL, imlist,
-                               CPL_BPP_8_UNSIGNED, "recipe",
+                               CPL_TYPE_UCHAR, "recipe",
                                qclist, "none", "pipe_id", "imlist" CPL_DFS_FITS);
 
     cpl_test_eq_error( error, CPL_ERROR_DATA_NOT_FOUND);
@@ -583,21 +625,25 @@ static void cpl_dfs_save_tests(const char * rawname)
     /* Make sure file does not exist */
     (void)remove("table" CPL_DFS_FITS);
 
-    cpl_test_zero(cpl_frame_set_tag(rawframe, "TAG"));
-    cpl_test_zero(cpl_frame_set_group(rawframe, CPL_FRAME_GROUP_RAW));
-    cpl_test_zero(cpl_frame_set_filename(rawframe, "table" CPL_DFS_FITS));
-    cpl_test_zero(cpl_frameset_insert(frames, rawframe));
+    error = cpl_frame_set_tag(rawframe, "TAG");
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    error = cpl_frame_set_group(rawframe, CPL_FRAME_GROUP_RAW);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    error = cpl_frame_set_filename(rawframe, "table" CPL_DFS_FITS);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    error = cpl_frameset_insert(frames, rawframe);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
 
     framesetsize = cpl_frameset_get_size(frames);
 
     error = cpl_dfs_save_image(frames, NULL, parlist, frames, NULL, image,
-                               CPL_BPP_8_UNSIGNED, "recipe", qclist, "none",
+                               CPL_TYPE_UCHAR, "recipe", qclist, "none",
                                "pipe_id", "image" CPL_DFS_FITS);
     cpl_test_eq_error( error, CPL_ERROR_FILE_NOT_FOUND);
     cpl_test_eq( cpl_frameset_get_size(frames), framesetsize);
 
     error = cpl_dfs_save_imagelist(frames, NULL, parlist, frames, NULL, imlist,
-                                   CPL_BPP_8_UNSIGNED, "recipe",
+                                   CPL_TYPE_UCHAR, "recipe",
                                    qclist, "none", "pipe_id",
                                    "imlist" CPL_DFS_FITS);
     cpl_test_eq_error( error, CPL_ERROR_FILE_NOT_FOUND);
@@ -609,17 +655,21 @@ static void cpl_dfs_save_tests(const char * rawname)
     frames = cpl_frameset_new();
     rawframe = cpl_frame_new();
 
-    cpl_test_zero(cpl_frame_set_tag(rawframe, "TAG"));
-    cpl_test_zero(cpl_frame_set_group(rawframe, CPL_FRAME_GROUP_RAW));
-    cpl_test_zero(cpl_frame_set_filename(rawframe, rawname));
-    cpl_test_zero(cpl_frameset_insert(frames, rawframe));
+    error = cpl_frame_set_tag(rawframe, "TAG");
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    error = cpl_frame_set_group(rawframe, CPL_FRAME_GROUP_RAW);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    error = cpl_frame_set_filename(rawframe, rawname);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    error = cpl_frameset_insert(frames, rawframe);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
 
     framesetsize = cpl_frameset_get_size(frames);
 
     /* Make sure file does not exist */
     (void)remove("imlist" CPL_DFS_FITS);
     error = cpl_dfs_save_imagelist(frames, NULL, parlist, frames, NULL, imlist,
-                                   CPL_BPP_8_UNSIGNED, "recipe",
+                                   CPL_TYPE_UCHAR, "recipe",
                                    qclist, "none", "pipe_id",
                                    "imlist" CPL_DFS_FITS);
     cpl_test_eq_error(error, CPL_ERROR_ILLEGAL_INPUT);
@@ -628,12 +678,13 @@ static void cpl_dfs_save_tests(const char * rawname)
     /* Test 3: Check handling of empty objects (except frameset + imagelist) */
 
     /* Insert an image into the imagelist */
-    cpl_test_zero(cpl_imagelist_set(imlist, cpl_image_new(3, 2, CPL_TYPE_INT), 0));
+    error = cpl_imagelist_set(imlist, cpl_image_new(3, 2, CPL_TYPE_INT), 0);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
 
     /* Make sure file does not exist */
     (void)remove("image" CPL_DFS_FITS);
     error = cpl_dfs_save_image(frames, NULL, parlist, frames, NULL, image,
-                               CPL_BPP_8_UNSIGNED,
+                               CPL_TYPE_UCHAR,
                                "recipe", qclist, "none", "pipe_id",
                                "image" CPL_DFS_FITS);
     cpl_test_eq_error(error, CPL_ERROR_NONE);
@@ -643,7 +694,7 @@ static void cpl_dfs_save_tests(const char * rawname)
     /* Make sure file does not exist */
     (void)remove("imlist" CPL_DFS_FITS);
     error = cpl_dfs_save_imagelist(frames, NULL, parlist, frames, NULL, imlist,
-                                   CPL_BPP_8_UNSIGNED, "recipe",
+                                   CPL_TYPE_UCHAR, "recipe",
                                    qclist, "none", "pipe_id",
                                    "imlist" CPL_DFS_FITS);
     cpl_test_eq_error(error, CPL_ERROR_NONE);
@@ -666,22 +717,27 @@ static void cpl_dfs_save_tests(const char * rawname)
 
     cpl_test_eq_error(error, CPL_ERROR_NONE);
 
-    cpl_test_zero(cpl_dfs_update_product_header(frames));
+    error = cpl_dfs_update_product_header(frames);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
 
     /* Test 4: Check handling of empty objects (except frameset and qclist) */
     /*         And remove one property */
     cpl_test_zero(cpl_propertylist_append_string(qclist, "ESO QC STRING",
                                                       "'Lorem ipsum'"));
-    cpl_test_zero(cpl_propertylist_append_int(qclist, "ESO QC INT", 42));
+    error = cpl_propertylist_append_int(qclist, "ESO QC INT", 42);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
     /* Use the value 1/3 to see the accuracy in the FITS header */
-    cpl_test_zero(cpl_propertylist_append_float(qclist, "ESO QC FLOAT", 1/3.0));
+    error = cpl_propertylist_append_float(qclist, "ESO QC FLOAT", 1.0F/3.0F);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
     cpl_test_zero(cpl_propertylist_append_double(qclist, "ESO QC DOUBLE",
-                                                      1/3.0));
-    cpl_test_zero(cpl_propertylist_append_string(qclist, "ESO QC DROP", "image"));
+                                                      1.0/3.0));
+    error = cpl_propertylist_append_string(qclist, "ESO QC DROP", "image");
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
 
     cpl_test_zero(cpl_propertylist_set_comment(qclist, "ESO QC STRING",
                                                     "string"));
-    cpl_test_zero(cpl_propertylist_set_comment(qclist, "ESO QC INT", "int"));
+    error = cpl_propertylist_set_comment(qclist, "ESO QC INT", "int");
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
     cpl_test_zero(cpl_propertylist_set_comment(qclist, "ESO QC FLOAT",
                                                     "float"));
     cpl_test_zero(cpl_propertylist_set_comment(qclist, "ESO QC DOUBLE",
@@ -691,12 +747,12 @@ static void cpl_dfs_save_tests(const char * rawname)
     (void)remove("image" CPL_DFS_FITS);
     image = cpl_image_new(1, 41, CPL_TYPE_FLOAT);
     error = cpl_dfs_save_image(frames, NULL, parlist, frames, NULL, image,
-                               CPL_BPP_8_UNSIGNED, "recipe",
+                               CPL_TYPE_UCHAR, "recipe",
                                qclist, remregexp, "pipe_id",
                                "image" CPL_DFS_FITS);
 
     cpl_test_eq_error(error, CPL_ERROR_NONE);
-    cpl_test( cpl_frameset_get_size(frames) == ++framesetsize);
+    cpl_test_eq( cpl_frameset_get_size(frames), ++framesetsize);
     cpl_test_fits("image" CPL_DFS_FITS);
 
 
@@ -704,10 +760,14 @@ static void cpl_dfs_save_tests(const char * rawname)
 
     rawframe = cpl_frame_new();
 
-    cpl_test_zero(cpl_frame_set_tag(rawframe, "IMAGECALIB"));
-    cpl_test_zero(cpl_frame_set_group(rawframe, CPL_FRAME_GROUP_CALIB));
-    cpl_test_zero(cpl_frame_set_filename(rawframe, "image" CPL_DFS_FITS));
-    cpl_test_zero(cpl_frameset_insert(frames, rawframe));
+    error = cpl_frame_set_tag(rawframe, "IMAGECALIB");
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    error = cpl_frame_set_group(rawframe, CPL_FRAME_GROUP_CALIB);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    error = cpl_frame_set_filename(rawframe, "image" CPL_DFS_FITS);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    error = cpl_frameset_insert(frames, rawframe);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
 
     framesetsize++;
 
@@ -724,7 +784,7 @@ static void cpl_dfs_save_tests(const char * rawname)
     /* Make sure file does not exist */
     (void)remove("imlist" CPL_DFS_FITS);
     error = cpl_dfs_save_imagelist(frames, NULL, parlist, frames, NULL, imlist,
-                                   CPL_BPP_8_UNSIGNED, "recipe",
+                                   CPL_TYPE_UCHAR, "recipe",
                                    qclist, "none", "pipe_id",
                                    "imlist" CPL_DFS_FITS);
     cpl_test_eq_error(error, CPL_ERROR_NONE);
@@ -734,7 +794,7 @@ static void cpl_dfs_save_tests(const char * rawname)
     /* Make sure file does not exist */
     (void)remove("nullimage" CPL_DFS_FITS);
     error = cpl_dfs_save_image(frames, NULL, parlist, frames, NULL, NULL,
-                               CPL_BPP_8_UNSIGNED,
+                               CPL_TYPE_UCHAR,
                                "recipe", qclist, remregexp, "pipe_id",
                                "nullimage" CPL_DFS_FITS);
     cpl_test_eq_error(error, CPL_ERROR_NONE);
@@ -758,14 +818,15 @@ static void cpl_dfs_save_tests(const char * rawname)
 
     /* Final test: Update the headers as well */
 
-    cpl_test_zero(cpl_dfs_update_product_header(frames));
+    error = cpl_dfs_update_product_header(frames);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
 
     cpl_msg_info(cpl_func, "New size of frameset: %d", framesetsize);
 
-    if (cpl_error_get_code() == CPL_ERROR_NONE && !is_debug) {
-        cpl_test(remove("image" CPL_DFS_FITS) == 0);
-        cpl_test(remove("table" CPL_DFS_FITS) == 0);
-        cpl_test(remove("recipe" CPL_DFS_PAF) == 0);
+    if (error == CPL_ERROR_NONE && !is_debug) {
+        cpl_test_zero(remove("image" CPL_DFS_FITS));
+        cpl_test_zero(remove("table" CPL_DFS_FITS));
+        cpl_test_zero(remove("recipe" CPL_DFS_PAF));
     }
 
     cpl_frameset_delete(frames);
@@ -787,10 +848,9 @@ static void cpl_dfs_save_txt(const char * self)
 
     cpl_test_nonnull( stream );
 
-    cpl_test( fprintf(stream, self,
-                      "SIMPLE ASCII file - not FITS\n") > 0);
+    cpl_test_leq(1, fprintf(stream, self, "SIMPLE ASCII file - not FITS\n"));
 
-    cpl_test( fclose(stream) == 0);
+    cpl_test_zero( fclose(stream));
 
 }
 
