@@ -1,4 +1,4 @@
-/* $Id: cpl_apertures.h,v 1.5 2010/11/11 10:32:10 llundin Exp $
+/* $Id: cpl_apertures.h,v 1.10 2011/12/20 09:11:42 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/11/11 10:32:10 $
- * $Revision: 1.5 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2011/12/20 09:11:42 $
+ * $Revision: 1.10 $
+ * $Name: cpl-6_0 $
  */
 
 #ifndef CPL_APERTURES_H
@@ -57,26 +57,34 @@ cpl_apertures * cpl_apertures_new_from_image(const cpl_image *,
                                              const cpl_image *) CPL_ATTR_ALLOC;
 
 /* Accessor functions */
-int cpl_apertures_get_size(const cpl_apertures *);
-double cpl_apertures_get_max_x(const cpl_apertures *, int);
-double cpl_apertures_get_max_y(const cpl_apertures *, int);
-double cpl_apertures_get_centroid_x(const cpl_apertures *, int);
-double cpl_apertures_get_centroid_y(const cpl_apertures *, int);
-int cpl_apertures_get_npix(const cpl_apertures *, int);
-int cpl_apertures_get_left(const cpl_apertures *, int);
-int cpl_apertures_get_left_y(const cpl_apertures *, int);
-int cpl_apertures_get_right(const cpl_apertures *, int);
-int cpl_apertures_get_right_y(const cpl_apertures *, int);
-int cpl_apertures_get_top_x(const cpl_apertures *, int);
-int cpl_apertures_get_top(const cpl_apertures *, int);
-int cpl_apertures_get_bottom_x(const cpl_apertures *, int);
-int cpl_apertures_get_bottom(const cpl_apertures *, int);
-double cpl_apertures_get_max(const cpl_apertures *, int);
-double cpl_apertures_get_min(const cpl_apertures *, int);
-double cpl_apertures_get_mean(const cpl_apertures *, int);
-double cpl_apertures_get_median(const cpl_apertures *, int);
-double cpl_apertures_get_stdev(const cpl_apertures *, int);
-double cpl_apertures_get_flux(const cpl_apertures *, int);
+cpl_size cpl_apertures_get_size(const cpl_apertures *);
+double cpl_apertures_get_pos_x(const cpl_apertures *, cpl_size);
+double cpl_apertures_get_pos_y(const cpl_apertures *, cpl_size);
+double cpl_apertures_get_max_x(const cpl_apertures *, cpl_size)
+    CPL_ATTR_DEPRECATED;
+double cpl_apertures_get_max_y(const cpl_apertures *, cpl_size)
+    CPL_ATTR_DEPRECATED;
+double cpl_apertures_get_centroid_x(const cpl_apertures *, cpl_size);
+double cpl_apertures_get_centroid_y(const cpl_apertures *, cpl_size);
+cpl_size cpl_apertures_get_maxpos_x(const cpl_apertures *, cpl_size);
+cpl_size cpl_apertures_get_maxpos_y(const cpl_apertures *, cpl_size);
+cpl_size cpl_apertures_get_minpos_x(const cpl_apertures *, cpl_size);
+cpl_size cpl_apertures_get_minpos_y(const cpl_apertures *, cpl_size);
+cpl_size cpl_apertures_get_npix(const cpl_apertures *, cpl_size);
+cpl_size cpl_apertures_get_left(const cpl_apertures *, cpl_size);
+cpl_size cpl_apertures_get_left_y(const cpl_apertures *, cpl_size);
+cpl_size cpl_apertures_get_right(const cpl_apertures *, cpl_size);
+cpl_size cpl_apertures_get_right_y(const cpl_apertures *, cpl_size);
+cpl_size cpl_apertures_get_top_x(const cpl_apertures *, cpl_size);
+cpl_size cpl_apertures_get_top(const cpl_apertures *, cpl_size);
+cpl_size cpl_apertures_get_bottom_x(const cpl_apertures *, cpl_size);
+cpl_size cpl_apertures_get_bottom(const cpl_apertures *, cpl_size);
+double cpl_apertures_get_max(const cpl_apertures *, cpl_size);
+double cpl_apertures_get_min(const cpl_apertures *, cpl_size);
+double cpl_apertures_get_mean(const cpl_apertures *, cpl_size);
+double cpl_apertures_get_median(const cpl_apertures *, cpl_size);
+double cpl_apertures_get_stdev(const cpl_apertures *, cpl_size);
+double cpl_apertures_get_flux(const cpl_apertures *, cpl_size);
 
 /* Sorting functions */
 cpl_error_code cpl_apertures_sort_by_npix(cpl_apertures *);
@@ -85,14 +93,19 @@ cpl_error_code cpl_apertures_sort_by_flux(cpl_apertures *);
 
 /* Detection functions */
 cpl_apertures * cpl_apertures_extract(const cpl_image *, const cpl_vector *,
-                                      int *) CPL_ATTR_ALLOC;
+                                      cpl_size *) CPL_ATTR_ALLOC;
 cpl_apertures * cpl_apertures_extract_window(const cpl_image *,
-                                             const cpl_vector *, int, int,
-                                             int, int, int *) CPL_ATTR_ALLOC;
+                                             const cpl_vector *, cpl_size,
+                                             cpl_size, cpl_size,
+                                             cpl_size, cpl_size *)
+    CPL_ATTR_ALLOC;
 
 cpl_apertures * cpl_apertures_extract_sigma(const cpl_image *,
                                             double) CPL_ATTR_ALLOC;
 
+cpl_bivector* cpl_apertures_get_fwhm(const cpl_image*,
+                                     const cpl_apertures*) CPL_ATTR_DEPRECATED;
+
 CPL_END_DECLS
 
 #endif 
