@@ -1,4 +1,4 @@
-/* $Id: cpl_apertures_img.h,v 1.3 2010/11/11 09:23:18 llundin Exp $
+/* $Id: cpl_apertures_img.h,v 1.4 2011/04/19 09:05:08 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,11 +20,13 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/11/11 09:23:18 $
- * $Revision: 1.3 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2011/04/19 09:05:08 $
+ * $Revision: 1.4 $
+ * $Name: cpl-6_0 $
  */
 
+/* Deprecated: Include cpl_apertures.h instead */
+
 #ifndef CPL_APERTURES_IMG_H
 #define CPL_APERTURES_IMG_H
 
@@ -32,20 +34,6 @@
                                    Includes
  -----------------------------------------------------------------------------*/
 
-#include <stdio.h>
-#include <cpl_image.h>
-#include <cpl_bivector.h>
 #include "cpl_apertures.h"
 
-CPL_BEGIN_DECLS
-
-/*-----------------------------------------------------------------------------
-                            Function prototypes
- -----------------------------------------------------------------------------*/
-
-cpl_bivector * cpl_apertures_get_fwhm(const cpl_image *,
-                                      const cpl_apertures *);
-
-CPL_END_DECLS
-
 #endif 
