@@ -1,4 +1,4 @@
-/* $Id: cpl_detector.c,v 1.26 2011/01/13 11:51:52 llundin Exp $
+/* $Id: cpl_detector.c,v 1.30 2011/07/29 12:03:46 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2011/01/13 11:51:52 $
- * $Revision: 1.26 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2011/07/29 12:03:46 $
+ * $Revision: 1.30 $
+ * $Name: cpl-6_0 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -42,12 +42,7 @@
                                    Includes
  -----------------------------------------------------------------------------*/
 
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <unistd.h>
-#include <math.h>
-#include <complex.h>
+#include "cpl_detector.h"
 
 #include <cpl_msg.h>
 #include <cpl_stats.h>
@@ -56,8 +51,14 @@
 #include <cpl_tools.h>
 #include <cpl_memory.h>
 #include <cpl_math_const.h>
+#include <cpl_error_impl.h>
 
-#include "cpl_detector.h"
+#include <stdio.h>
+#include <stdlib.h>
+#include <string.h>
+#include <unistd.h>
+#include <math.h>
+#include <complex.h>
 
 /*-----------------------------------------------------------------------------
                                    Define
@@ -116,15 +117,17 @@
 #undef CPL_DO_ROUND
 #undef CPL_SUM_TYPE
 
-static cpl_error_code cpl_flux_get_window(const cpl_image *, const int *,
-                                          int, int, double *, double *,
-                                          double *, double *);
+static cpl_error_code cpl_flux_get_window(const cpl_image *, const cpl_size *,
+                                          cpl_size, cpl_size, double *,
+                                          double *, double *, double *);
 
-static cpl_bivector * cpl_bivector_gen_rect_poisson(const int *, const int,
-                                                    const int)
+static cpl_bivector * cpl_bivector_gen_rect_poisson(const cpl_size *,
+                                                    const cpl_size,
+                                                    const cpl_size)
     CPL_ATTR_NONNULL CPL_ATTR_ALLOC;
-static cpl_bivector * cpl_bivector_gen_ring_poisson(const double *, const int,
-                                                    const int)
+static cpl_bivector * cpl_bivector_gen_ring_poisson(const double *,
+                                                    const cpl_size,
+                                                    const cpl_size)
     CPL_ATTR_NONNULL CPL_ATTR_ALLOC;
 
 /*----------------------------------------------------------------------------*/
@@ -183,22 +186,16 @@ static cpl_bivector * cpl_bivector_gen_ring_poisson(const double *, const int,
   - CPL_ERROR_ILLEGAL_INPUT if the specified window (zone_def) is invalid
  */
 /*----------------------------------------------------------------------------*/
-cpl_error_code cpl_flux_get_noise_window(
-        const cpl_image *   diff,
-        const int       *   zone_def,
-        int                 ron_hsize,
-        int                 ron_nsamp,
-        double          *   noise,
-        double          *   error)
+cpl_error_code cpl_flux_get_noise_window(const cpl_image * diff,
+                                         const cpl_size  * zone_def,
+                                         cpl_size          ron_hsize,
+                                         cpl_size          ron_nsamp,
+                                         double          * noise,
+                                         double          * error)
 {
-    const cpl_error_code errorcode =
-    cpl_flux_get_window(diff, zone_def, ron_hsize, ron_nsamp,
-                noise, error, NULL, NULL);
-
-    /* Propagate any error, if necessary */
-    cpl_ensure_code(!errorcode, errorcode);
-
-    return CPL_ERROR_NONE;
+    return cpl_flux_get_window(diff, zone_def, ron_hsize, ron_nsamp,
+                               noise, error, NULL, NULL)
+        ? cpl_error_set_where_() : CPL_ERROR_NONE;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -215,22 +212,16 @@ cpl_error_code cpl_flux_get_noise_window(
   @note     This function uses the pseudo-random number generator.
  */
 /*----------------------------------------------------------------------------*/
-cpl_error_code cpl_flux_get_bias_window(
-        const cpl_image *   diff,
-        const int       *   zone_def,
-        int                 ron_hsize,
-        int                 ron_nsamp,
-        double          *   bias,
-        double          *   error)
+cpl_error_code cpl_flux_get_bias_window(const cpl_image * diff,
+                                        const cpl_size  * zone_def,
+                                        cpl_size          ron_hsize,
+                                        cpl_size          ron_nsamp,
+                                        double          * bias,
+                                        double          * error)
 {
-    const cpl_error_code errorcode =
-    cpl_flux_get_window(diff, zone_def, ron_hsize, ron_nsamp,
-                NULL, NULL, bias, error);
-
-    /* Propagate any error, if necessary */
-    cpl_ensure_code(!errorcode, errorcode);
-
-    return CPL_ERROR_NONE;
+    return cpl_flux_get_window(diff, zone_def, ron_hsize, ron_nsamp,
+                               NULL, NULL, bias, error)
+        ? cpl_error_set_where_() : CPL_ERROR_NONE;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -240,55 +231,55 @@ cpl_error_code cpl_flux_get_bias_window(
 /*----------------------------------------------------------------------------*/
 static cpl_error_code cpl_flux_get_window(
         const cpl_image *   diff,
-        const int       *   zone_def,
-        int                 ron_hsize,
-        int                 ron_nsamp,
+        const cpl_size  *   zone_def,
+        cpl_size            ron_hsize,
+        cpl_size            ron_nsamp,
         double          *   noise,
         double          *   noise_error,
         double          *   bias,
         double          *   bias_error)
 {
-    const int hsize    = ron_hsize < 0 ? RECT_RON_HS      : ron_hsize;
-    const int nsamples = ron_nsamp < 0 ? RECT_RON_SAMPLES : ron_nsamp;
-    cpl_bivector    *   sample_reg ;
-    cpl_vector      *   rms_list = NULL;
-    cpl_vector      *   bias_list = NULL;
-    int                 rect[4] ;
-    int                 zone[4];
-    double          *   px;
-    double          *   py;
-    double          *   pr1 = NULL;
-    double          *   pr2 = NULL;
-    int                 i ;
+    const cpl_size hsize = ron_hsize < 0 ? RECT_RON_HS      : ron_hsize;
+    const cpl_size nsamples = ron_nsamp < 0 ? RECT_RON_SAMPLES : ron_nsamp;
+    cpl_bivector * sample_reg;
+    cpl_vector   * rms_list = NULL;
+    cpl_vector   * bias_list = NULL;
+    cpl_size       rect[4];
+    cpl_size       zone[4];
+    const double * px;
+    const double * py;
+    double       * pr1 = NULL;
+    double       * pr2 = NULL;
+    cpl_size       i;
 
     /* Test entries */
-    cpl_ensure_code(diff && (noise || bias), CPL_ERROR_NULL_INPUT);
-
-    cpl_ensure_code(nsamples > 0,            CPL_ERROR_NULL_INPUT);
+    cpl_ensure_code(diff  != NULL,                 CPL_ERROR_NULL_INPUT);
+    cpl_ensure_code(noise != NULL || bias != NULL, CPL_ERROR_NULL_INPUT);
+    cpl_ensure_code(nsamples > 0,                  CPL_ERROR_ILLEGAL_INPUT);
 
     /* Generate nsamples window centers in the image */
     if (zone_def != NULL) {
-        rect[0] = zone_def[0] + hsize + 1 ; /* xmin */
-        rect[1] = zone_def[1] - hsize - 1 ; /* xmax */
-        rect[2] = zone_def[2] + hsize + 1 ; /* ymin */
-        rect[3] = zone_def[3] - hsize - 1 ; /* ymax */
+        rect[0] = zone_def[0] + hsize + 1; /* xmin */
+        rect[1] = zone_def[1] - hsize - 1; /* xmax */
+        rect[2] = zone_def[2] + hsize + 1; /* ymin */
+        rect[3] = zone_def[3] - hsize - 1; /* ymax */
     } else {
-        rect[0] = hsize + 1 ;                /* xmin */
-        rect[1] = cpl_image_get_size_x(diff) - hsize - 1 ;    /* xmax */
-        rect[2] = hsize + 1 ;                /* ymin */
-        rect[3] = cpl_image_get_size_y(diff) - hsize - 1 ;    /* ymax */
+        rect[0] = hsize + 1;                /* xmin */
+        rect[1] = cpl_image_get_size_x(diff) - hsize - 1;    /* xmax */
+        rect[2] = hsize + 1;                /* ymin */
+        rect[3] = cpl_image_get_size_y(diff) - hsize - 1;    /* ymax */
     }
 
     cpl_ensure_code( rect[0] < rect[1] && rect[2] < rect[3],
-                CPL_ERROR_ILLEGAL_INPUT) ;
+                CPL_ERROR_ILLEGAL_INPUT);
 
     /* Generate n+1 regions, because the first region is always at (0,0) */
     /* and it would bias the measurement. */
     sample_reg = cpl_bivector_gen_rect_poisson(rect, nsamples+1, nsamples+1);
     cpl_ensure_code( sample_reg != NULL, CPL_ERROR_ILLEGAL_INPUT);
 
-    px = cpl_bivector_get_x_data(sample_reg);
-    py = cpl_bivector_get_y_data(sample_reg);
+    px = cpl_bivector_get_x_data_const(sample_reg);
+    py = cpl_bivector_get_y_data_const(sample_reg);
 
     /* Now, for each window center, extract a vignette and compute the */
     /* signal RMS in it. Store this rms into a table. */
@@ -298,17 +289,17 @@ static cpl_error_code cpl_flux_get_window(
     if (noise) pr1 = cpl_vector_get_data(rms_list);
     if (bias)  pr2 = cpl_vector_get_data(bias_list);
 
-    for (i=0 ; i<nsamples ; i++) {
-        zone[0] = (int)px[i+1] - hsize ;
-        zone[1] = (int)px[i+1] + hsize ;
-        zone[2] = (int)py[i+1] - hsize ;
-        zone[3] = (int)py[i+1] + hsize ;
+    for (i=0; i<nsamples; i++) {
+        zone[0] = (cpl_size)px[i+1] - hsize;
+        zone[1] = (cpl_size)px[i+1] + hsize;
+        zone[2] = (cpl_size)py[i+1] - hsize;
+        zone[3] = (cpl_size)py[i+1] + hsize;
         if (noise) 
         pr1[i] = cpl_image_get_stdev_window(diff, 
-        zone[0], zone[2], zone[1], zone[3]) ;
+        zone[0], zone[2], zone[1], zone[3]);
     if (bias) 
         pr2[i] = cpl_image_get_mean_window(diff, 
-                zone[0], zone[2], zone[1], zone[3]) ;
+                zone[0], zone[2], zone[1], zone[3]);
     }
     cpl_bivector_delete(sample_reg);
 
@@ -317,8 +308,8 @@ static cpl_error_code cpl_flux_get_window(
 
     /* The final computed RMS is the median of all values.  */
     /* This call will modify the rms_list */
-    if (noise) *noise = cpl_vector_get_median(rms_list) ;
-    if (bias)   *bias = cpl_vector_get_median(bias_list) ;
+    if (noise) *noise = cpl_vector_get_median(rms_list);
+    if (bias)   *bias = cpl_vector_get_median(bias_list);
 
     cpl_vector_delete(rms_list);
     cpl_vector_delete(bias_list);
@@ -359,24 +350,24 @@ static cpl_error_code cpl_flux_get_window(
 cpl_error_code cpl_flux_get_noise_ring(
         const cpl_image *   diff,
         const double    *   zone_def,
-        int                 ron_hsize,
-        int                 ron_nsamp,
+        cpl_size            ron_hsize,
+        cpl_size            ron_nsamp,
         double          *   noise,
         double          *   error)
 {
-    cpl_bivector    *   sample_reg ;
+    cpl_bivector    *   sample_reg;
     double          *   px;
     double          *   py;
-    int                 zone[4];
-    const int           hsize = ron_hsize < 0 ? RING_RON_HS      : ron_hsize;
-    const int           nsamp = ron_nsamp < 0 ? RING_RON_SAMPLES : ron_nsamp;
+    cpl_size            zone[4];
+    const cpl_size      hsize = ron_hsize < 0 ? RING_RON_HS      : ron_hsize;
+    const cpl_size      nsamp = ron_nsamp < 0 ? RING_RON_SAMPLES : ron_nsamp;
     cpl_vector      *   rms_list;
-    int                 nx, ny;
-    int                 i, rsize;
+    cpl_size            nx, ny;
+    cpl_size            i, rsize;
 
-    cpl_ensure_code(diff,     CPL_ERROR_NULL_INPUT) ;
-    cpl_ensure_code(noise,    CPL_ERROR_NULL_INPUT) ;
-    cpl_ensure_code(zone_def, CPL_ERROR_NULL_INPUT) ;
+    cpl_ensure_code(diff,     CPL_ERROR_NULL_INPUT);
+    cpl_ensure_code(noise,    CPL_ERROR_NULL_INPUT);
+    cpl_ensure_code(zone_def, CPL_ERROR_NULL_INPUT);
     cpl_ensure_code(zone_def[2] < zone_def[3], CPL_ERROR_ILLEGAL_INPUT);
 
     nx = cpl_image_get_size_x(diff);
@@ -395,22 +386,22 @@ cpl_error_code cpl_flux_get_noise_ring(
     /* Now, for each valid sample, extract a vignette and compute the RMS */
     rms_list = cpl_vector_new(nsamp + 1);
     cpl_ensure_code(rms_list, CPL_ERROR_NULL_INPUT);
-    rsize = 0 ;
-    for (i=0 ; i < nsamp + 1 ; i++) {
+    rsize = 0;
+    for (i=0; i < nsamp + 1; i++) {
         /* Add 0.5 to round off to nearest (positive) integer */
         const double dx = 0.5 + zone_def[0] + px[i] * cos(py[i]);
         const double dy = 0.5 + zone_def[1] + px[i] * sin(py[i]);
 
-        zone[0] = (int)(dx - hsize);
+        zone[0] = (cpl_size)dx - hsize;
         if (zone[0] < 1) continue;
 
-        zone[1] = (int)(dx + hsize); 
+        zone[1] = (cpl_size)dx + hsize;
         if (zone[1] > nx) continue;
 
-        zone[2] = (int)(dy - hsize);
+        zone[2] = (cpl_size)dy - hsize;
         if (zone[2] < 1) continue;
 
-        zone[3] = (int)(dy + hsize);
+        zone[3] = (cpl_size)dy + hsize;
         if (zone[3] > ny) continue;
 
         cpl_vector_set(rms_list, rsize, cpl_image_get_stdev_window(diff,
@@ -426,10 +417,10 @@ cpl_error_code cpl_flux_get_noise_ring(
 
     if (4 * rsize < nsamp) {
         cpl_vector_delete(rms_list);
-        return cpl_error_set_message_macro(cpl_func, CPL_ERROR_DATA_NOT_FOUND,
-                                           __FILE__, __LINE__,
-                                           "Need at least %d/4 (not %d) samples"
-                                           " to compute noise", nsamp, rsize);
+        return cpl_error_set_message_(CPL_ERROR_DATA_NOT_FOUND,
+                                      "Need at least %" CPL_SIZE_FORMAT "/4 "
+                                      "(not %" CPL_SIZE_FORMAT ") samples"
+                                      " to compute noise", nsamp, rsize);
     }
 
     /* Ignore all but the first rsize elements */
@@ -440,7 +431,7 @@ cpl_error_code cpl_flux_get_noise_ring(
     
     /* The final computed RMS is the median of all values.  */
     /* This call will modify the rms_list */
-    *noise = cpl_vector_get_median(rms_list) ;
+    *noise = cpl_vector_get_median(rms_list);
     
     cpl_vector_delete(rms_list);
 
@@ -481,11 +472,11 @@ cpl_error_code cpl_detector_interpolate_rejected(cpl_image * self)
     const cpl_mask * mask = cpl_image_get_bpm_const(self);
 
     if (mask != NULL) {
-        const int nx = cpl_image_get_size_x(self);
-        const int ny = cpl_image_get_size_y(self);
+        const cpl_size nx = cpl_image_get_size_x(self);
+        const cpl_size ny = cpl_image_get_size_y(self);
         const cpl_binary * bpm = cpl_mask_get_data_const(mask);
         cpl_binary * doit = memchr(bpm, CPL_BINARY_1,
-                                   (size_t)nx * ny * sizeof(*bpm));
+                                   (size_t)nx * (size_t)ny * sizeof(*bpm));
 
         if (doit) {
 
@@ -517,13 +508,13 @@ cpl_error_code cpl_detector_interpolate_rejected(cpl_image * self)
                                  bpm, nx, ny, doit), cpl_error_get_code());
                 break;
             default:
-                return cpl_error_set(cpl_func, CPL_ERROR_INVALID_TYPE);
+                return cpl_error_set_(CPL_ERROR_INVALID_TYPE);
             }
         }
 
         (void)cpl_image_accept_all(self); /* bpm could be empty */
     } else if (self == NULL) {
-        return cpl_error_set(cpl_func, CPL_ERROR_NULL_INPUT);
+        return cpl_error_set_(CPL_ERROR_NULL_INPUT);
     }
 
     return CPL_ERROR_NONE;
@@ -573,123 +564,123 @@ cpl_error_code cpl_detector_interpolate_rejected(cpl_image * self)
   - CPL_ERROR_ILLEGAL_INPUT if np is not strictly positive
  */
 /*----------------------------------------------------------------------------*/
-static cpl_bivector * cpl_bivector_gen_rect_poisson(
-        const int   *   r,
-        const int       np,
-        const int       homog)
+static cpl_bivector * cpl_bivector_gen_rect_poisson(const cpl_size * r,
+                                                    const cpl_size   np,
+                                                    const cpl_size   homog)
 {
-    double              min_dist ;
-    int                 i ;
-    int                 gnp ;
-    cpl_bivector    *   list ;
-    double              cand_x, cand_y ;
-    int                 ok ;
-    int                 start_ndx ;
-    int                 xmin, xmax, ymin, ymax ;
+    double              min_dist;
+    cpl_size            i;
+    cpl_size            gnp;
+    cpl_bivector    *   list;
+    double              cand_x, cand_y;
+    cpl_boolean         ok;
+    cpl_size            start_ndx;
+    cpl_size            xmin, xmax, ymin, ymax;
     /* Corrected Homogeneity factor */
-    const int           homogc = 0 < homog && homog < np ? homog : np;
-    double          *   px ;
-    double          *   py ;
+    const cpl_size      homogc = 0 < homog && homog < np ? homog : np;
+    double          *   px;
+    double          *   py;
 
     /* error handling: test arguments are correct */
-    cpl_ensure(r, CPL_ERROR_NULL_INPUT, NULL) ;
-    cpl_ensure( np > 0, CPL_ERROR_ILLEGAL_INPUT, NULL) ;
+    cpl_ensure(r, CPL_ERROR_NULL_INPUT, NULL);
+    cpl_ensure( np > 0, CPL_ERROR_ILLEGAL_INPUT, NULL);
 
     list = cpl_bivector_new(np);
-    cpl_ensure(list, CPL_ERROR_NULL_INPUT,NULL) ;
-    px = cpl_bivector_get_x_data(list) ;
-    py = cpl_bivector_get_y_data(list) ;
+    cpl_ensure(list, CPL_ERROR_NULL_INPUT,NULL);
+    px = cpl_bivector_get_x_data(list);
+    py = cpl_bivector_get_y_data(list);
 
-    xmin = r[0] ;
-    xmax = r[1] ;
-    ymin = r[2] ;
-    ymax = r[3] ;
+    xmin = r[0];
+    xmax = r[1];
+    ymin = r[2];
+    ymax = r[3];
 
-    min_dist   = CPL_MATH_SQRT1_2*((xmax-xmin)*(ymax-ymin) /(double)(homogc+1));
-    gnp        = 1 ;
-    px[0]      = 0 ;
-    py[0]      = 0 ;
+    min_dist   = CPL_MATH_SQRT1_2*(double)((xmax-xmin)*(ymax-ymin))
+        / (double)(homogc+1);
+    gnp        = 1;
+    px[0]      = 0;
+    py[0]      = 0;
 
     /* First: generate <homog> points */
     while (gnp < homogc) {
         /* Pick a random point within requested range */
-        cand_x = cpl_drand() * (xmax - xmin) + xmin ;
-        cand_y = cpl_drand() * (ymax - ymin) + ymin ;
+        cand_x = cpl_drand() * (double)(xmax - xmin) + (double)xmin;
+        cand_y = cpl_drand() * (double)(ymax - ymin) + (double)ymin;
 
         /* Check the candidate obeys the minimal Poisson distance */
-        ok = 1 ;
-        for (i=0 ; i<gnp ; i++) {
+        ok = CPL_TRUE;
+        for (i=0; i<gnp; i++) {
             if (pdist(cand_x, cand_y, px[i], py[i])<min_dist) {
                 /* does not check Poisson law: reject point */
-                ok = 0 ;
-                break ;
+                ok = CPL_FALSE;
+                break;
             }
         }
         if (ok) {
             /* obeys Poisson law: register the point as valid */
-            px[gnp] = cand_x ;
-            py[gnp] = cand_y ;
-            gnp ++ ;
+            px[gnp] = cand_x;
+            py[gnp] = cand_y;
+            gnp ++;
         }
     }
 
     /* Iterative process: */
     /* Pick points out of Poisson distance of the last <homogc-1> points. */
-    start_ndx=0 ;
+    start_ndx=0;
     while (gnp < np) {
         /* Pick a random point within requested range */
-        cand_x = cpl_drand() * (xmax - xmin) + xmin ;
-        cand_y = cpl_drand() * (ymax - ymin) + ymin ;
+        cand_x = cpl_drand() * (double)(xmax - xmin) + (double)xmin;
+        cand_y = cpl_drand() * (double)(ymax - ymin) + (double)ymin;
 
         /* Check the candidate obeys the minimal Poisson distance */
-        ok = 1 ;
-        for (i=0 ; i<homogc ; i++) {
+        ok = CPL_TRUE;
+        for (i=0; i<homogc; i++) {
             if (pdist(cand_x,
                       cand_y,
                       px[start_ndx+i],
                       py[start_ndx+i]) < min_dist) {
                 /* does not check Poisson law: reject point */
-                ok = 0 ;
-                break ;
+                ok = CPL_FALSE;
+                break;
             }
         }
         if (ok) {
             /* obeys Poisson law: register the point as valid */
-            px[gnp] = cand_x ;
-            py[gnp] = cand_y ;
-            gnp ++ ;
+            px[gnp] = cand_x;
+            py[gnp] = cand_y;
+            gnp ++;
         }
     }
 
     /* Iterative process: */
     /* Pick points out of Poisson distance of the last <homogc-1> points. */
-    start_ndx=0 ;
+    start_ndx=0;
     while (gnp < np) {
         /* Pick a random point within requested range */
-        cand_x = cpl_drand() * (xmax - xmin) + xmin ;
-        cand_y = cpl_drand() * (ymax - ymin) + ymin ;
+        cand_x = cpl_drand() * (double)(xmax - xmin) + (double)xmin;
+        cand_y = cpl_drand() * (double)(ymax - ymin) + (double)ymin;
 
         /* Check the candidate obeys the minimal Poisson distance */
-        ok = 1 ;
-        for (i=0 ; i<homogc ; i++) {
+        ok = CPL_TRUE;
+        for (i=0; i<homogc; i++) {
             if (pdist(cand_x,
                       cand_y,
                       px[start_ndx+i],
                       py[start_ndx+i]) < min_dist) {
                 /* does not check Poisson law: reject point */
-                ok = 0 ;
-                break ;
+                ok = CPL_FALSE;
+                break;
             }
         }
         if (ok) {
             /* obeys Poisson law: register the point as valid */
-            px[gnp] = cand_x ;
-            py[gnp] = cand_y ;
-            gnp ++ ;
-            start_ndx ++ ;
+            px[gnp] = cand_x;
+            py[gnp] = cand_y;
+            gnp ++;
+            start_ndx ++;
         }
     }
-    return list ;
+    return list;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -711,90 +702,90 @@ static cpl_bivector * cpl_bivector_gen_rect_poisson(
 /*----------------------------------------------------------------------------*/
 static cpl_bivector * cpl_bivector_gen_ring_poisson(
         const double    *   r,
-        const int           np,
-        const int           homog)
+        const cpl_size      np,
+        const cpl_size      homog)
 {
-    double              min_dist ;
-    int                 gnp ;
-    cpl_bivector    *   list ;
-    double              cand_r, cand_t ;
-    int                 ok ;
-    int                 start_ndx ;
-    int                 r1, r2 ;
-    int                 i ;
+    double              min_dist;
+    cpl_size            gnp;
+    cpl_bivector    *   list;
+    double              cand_r, cand_t;
+    cpl_boolean         ok;
+    cpl_size            start_ndx;
+    cpl_size            r1, r2;
+    cpl_size            i;
     /* Corrected Homogeneity factor */
-    const int           homogc = 0 < homog && homog < np ? homog : np;
-    double          *   px ;
-    double          *   py ;
+    const cpl_size      homogc = 0 < homog && homog < np ? homog : np;
+    double          *   px;
+    double          *   py;
 
     /* Check entries */
-    cpl_ensure(r, CPL_ERROR_NULL_INPUT, NULL) ;
-    cpl_ensure(np > 0, CPL_ERROR_ILLEGAL_INPUT, NULL) ;
+    cpl_ensure(r, CPL_ERROR_NULL_INPUT, NULL);
+    cpl_ensure(np > 0, CPL_ERROR_ILLEGAL_INPUT, NULL);
 
     list = cpl_bivector_new(np);
-    cpl_ensure(list, CPL_ERROR_NULL_INPUT,NULL) ;
-    px = cpl_bivector_get_x_data(list) ;
-    py = cpl_bivector_get_y_data(list) ;
+    cpl_ensure(list, CPL_ERROR_NULL_INPUT,NULL);
+    px = cpl_bivector_get_x_data(list);
+    py = cpl_bivector_get_y_data(list);
 
-    r1 = (int)(r[2]) ;
-    r2 = (int)(r[3]) ;
+    r1 = (cpl_size)r[2];
+    r2 = (cpl_size)r[3];
 
-    min_dist    = (CPL_MATH_PI_2/CPL_MATH_SQRT1_2)*((r2*r2)-(r1*r1)) /
-        (double)(homogc+1);
-    gnp         = 1 ;
-    px[0]       = r1 ;
-    py[0]       = 0 ;
+    min_dist    = (CPL_MATH_PI_2/CPL_MATH_SQRT1_2)*(double)((r2*r2)-(r1*r1))
+        / (double)(homogc+1);
+    gnp         = 1;
+    px[0]       = (double)r1;
+    py[0]       = 0.0;
 
     /* First: generate <homog> points */
     while (gnp < homogc) {
         /* Pick a random point within requested range */
-        cand_r = cpl_drand() * (r2 - r1) + r1 ;
-        cand_t = cpl_drand() * CPL_MATH_2PI ;
+        cand_r = cpl_drand() * (double)(r2 - r1) + (double)r1;
+        cand_t = cpl_drand() * CPL_MATH_2PI;
 
         /* Check the candidate obeys the minimal Poisson distance */
-        ok = 1 ;
-        for (i=0 ; i<gnp ; i++) {
-            if (qdist(cand_r, cand_t, px[i], py[i])<min_dist) {
+        ok = CPL_TRUE;
+        for (i=0; i<gnp; i++) {
+            if (qdist(cand_r, cand_t, px[i], py[i]) < min_dist) {
                 /* does not check Poisson law: reject point */
-                ok = 0 ;
-                break ;
+                ok = CPL_FALSE;
+                break;
             }
         }
         if (ok) {
             /* obeys Poisson law: register the point as valid */
-            px[gnp] = cand_r ;
-            py[gnp] = cand_t ;
-            gnp ++ ;
+            px[gnp] = cand_r;
+            py[gnp] = cand_t;
+            gnp ++;
         }
     }
 
     /* Iterative process: */
     /* Pick points out of Poisson distance of the last <homogc-1> points. */
-    start_ndx=0 ;
+    start_ndx = 0;
     while (gnp < np) {
         /* Pick a random point within requested range */
-        cand_r = cpl_drand() * (r2 - r1) + r1 ;
-        cand_t = cpl_drand() * CPL_MATH_2PI ;
+        cand_r = cpl_drand() * (double)(r2 - r1) + (double)r1;
+        cand_t = cpl_drand() * CPL_MATH_2PI;
 
         /* Check the candidate obeys the minimal Poisson distance */
-        ok = 1 ;
-        for (i=0 ; i<homogc ; i++) {
+        ok = CPL_TRUE;
+        for (i=0; i<homogc; i++) {
             if (qdist(cand_r,
                       cand_t,
                       px[start_ndx+i],
                       py[start_ndx+i]) < min_dist) {
                 /* does not check Poisson law: reject point */
-                ok = 0 ;
-                break ;
+                ok = CPL_FALSE;
+                break;
             }
         }
         if (ok) {
             /* obeys Poisson law: register the point as valid */
-            px[gnp] = cand_r ;
-            py[gnp] = cand_t ;
-            gnp ++ ;
-            start_ndx ++ ;
+            px[gnp] = cand_r;
+            py[gnp] = cand_t;
+            gnp ++;
+            start_ndx ++;
         }
     }
-    return list ;
+    return list;
 }
