@@ -1,4 +1,4 @@
-/* $Id: cpl_geom_img.c,v 1.40 2010/11/23 10:38:29 llundin Exp $
+/* $Id: cpl_geom_img.c,v 1.49 2011/12/20 09:11:42 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/11/23 10:38:29 $
- * $Revision: 1.40 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2011/12/20 09:11:42 $
+ * $Revision: 1.49 $
+ * $Name: cpl-6_0 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -47,12 +47,27 @@
 
 #include <cpl_tools.h>
 #include <cpl_vector.h>
-#include <cpl_error.h>
+#include <cpl_error_impl.h>
 #include <cpl_errorstate.h>
 
 #include "cpl_apertures.h"
 #include "cpl_geom_img.h"
 
+/*----------------------------------------------------------------------------*/
+/**
+ * @defgroup cpl_geom_img   High level functions for geometric transformations
+ *
+ * This module contains functions to compute the shift-and-add operation
+ * on an image list.
+ *
+ * @par Synopsis:
+ * @code
+ *   #include "cpl_geom_img.h"
+ * @endcode
+ */
+/*----------------------------------------------------------------------------*/
+/**@{*/
+
 /*-----------------------------------------------------------------------------
                             Defines
  -----------------------------------------------------------------------------*/
@@ -68,49 +83,57 @@
                             Static Function Prototypes
  -----------------------------------------------------------------------------*/
 
-static void cpl_geom_img_get_min_max_double(double *, int, int,
-                                            int) CPL_ATTR_NONNULL;
+static void cpl_geom_img_get_min_max_double(double *, cpl_size, cpl_size,
+                                            cpl_size) CPL_ATTR_NONNULL;
 
 static double cpl_geom_ima_offset_xcorr(const cpl_image *, const cpl_image *,
                                         const cpl_bivector *,
-                                        int, int, int, int,
+                                        cpl_size, cpl_size, cpl_size, cpl_size,
                                         double *, double *) CPL_ATTR_NONNULL;
 
 static double cpl_geom_ima_offset_xcorr_subw(double *, const cpl_image *,
                                              const cpl_image *,
-                                             int, int, int, int,
-                                             int, int, int, int, double *,
+                                             cpl_size, cpl_size,
+                                             cpl_size, cpl_size,
+                                             cpl_size, cpl_size,
+                                             cpl_size, cpl_size, double *,
                                              double *) CPL_ATTR_NONNULL;
 
 static
 void cpl_geom_ima_offset_xcorr_subw_double(double *,
                                            const cpl_image *,
                                            const cpl_image *,
-                                           int, int, int, int,
-                                           int, int, int, int,
-                                           int *, int *) CPL_ATTR_NONNULL;
+                                           cpl_size, cpl_size,
+                                           cpl_size, cpl_size,
+                                           cpl_size, cpl_size,
+                                           cpl_size, cpl_size,
+                                           cpl_size *, cpl_size *)
+    CPL_ATTR_NONNULL;
 
 static
 void cpl_geom_ima_offset_xcorr_subw_float(double *,
                                           const cpl_image *,
                                           const cpl_image *,
-                                          int, int, int, int,
-                                          int, int, int, int,
-                                          int *, int *) CPL_ATTR_NONNULL;
+                                          cpl_size, cpl_size,
+                                          cpl_size, cpl_size,
+                                          cpl_size, cpl_size,
+                                          cpl_size, cpl_size,
+                                          cpl_size *, cpl_size *)
+    CPL_ATTR_NONNULL;
 
 static void cpl_geom_img_offset_saa_double(cpl_image *, cpl_image *,
-                                           int, int, double, double,
-                                           const cpl_imagelist *, int,
+                                           cpl_size, cpl_size, double, double,
+                                           const cpl_imagelist *, cpl_size,
                                            const cpl_bivector *,
                                            const cpl_vector *,
-                                           int) CPL_ATTR_NONNULL;
+                                           cpl_size) CPL_ATTR_NONNULL;
 
 static void cpl_geom_img_offset_saa_float(cpl_image *, cpl_image *,
-                                          int, int, double, double,
-                                          const cpl_imagelist *, int,
+                                          cpl_size, cpl_size, double, double,
+                                          const cpl_imagelist *, cpl_size,
                                           const cpl_bivector *,
                                           const cpl_vector *,
-                                          int) CPL_ATTR_NONNULL;
+                                          cpl_size) CPL_ATTR_NONNULL;
 
 static void cpl_geom_img_offset_saa_all_double(cpl_image *,
                                                cpl_image *,
@@ -118,7 +141,7 @@ static void cpl_geom_img_offset_saa_all_double(cpl_image *,
                                                const cpl_imagelist *,
                                                const cpl_bivector *,
                                                const cpl_vector *,
-                                               int) CPL_ATTR_NONNULL;
+                                               cpl_size) CPL_ATTR_NONNULL;
 
 static void cpl_geom_img_offset_saa_all_float(cpl_image *,
                                               cpl_image *,
@@ -126,7 +149,7 @@ static void cpl_geom_img_offset_saa_all_float(cpl_image *,
                                               const cpl_imagelist *,
                                               const cpl_bivector *,
                                               const cpl_vector *,
-                                              int) CPL_ATTR_NONNULL;
+                                              cpl_size) CPL_ATTR_NONNULL;
 
 static cpl_imagelist * cpl_imagelist_wrap_all_but_first(cpl_imagelist *)
     CPL_ATTR_ALLOC;
@@ -137,21 +160,6 @@ static cpl_bivector * cpl_bivector_wrap_all_but_first(cpl_bivector *)
 static void cpl_bivector_unwrap(cpl_bivector *);
 
 
-/*----------------------------------------------------------------------------*/
-/**
- * @defgroup cpl_geom_img   High level functions for geometric transformations
- *
- * This module contains functions to compute the shift-and-add operation
- * on an image list.
- *
- * @par Synopsis:
- * @code
- *   #include "cpl_geom_img.h"
- * @endcode
- */
-/*----------------------------------------------------------------------------*/
-/**@{*/
-
 /*-----------------------------------------------------------------------------
                             Function codes
  -----------------------------------------------------------------------------*/
@@ -197,18 +205,18 @@ cpl_bivector * cpl_geom_img_offset_fine(
         const cpl_imagelist *   ilist,
         const cpl_bivector  *   estimates,
         const cpl_bivector  *   anchors,
-        int                     s_hx,
-        int                     s_hy,
-        int                     m_hx,
-        int                     m_hy,
+        cpl_size                s_hx,
+        cpl_size                s_hy,
+        cpl_size                m_hx,
+        cpl_size                m_hy,
         cpl_vector          *   correl)
 {
     cpl_errorstate    prevstate = cpl_errorstate_get();
-    const int         nima = cpl_imagelist_get_size(ilist);
+    const cpl_size    nima = cpl_imagelist_get_size(ilist);
     const cpl_image * img1 = cpl_imagelist_get_const(ilist, 0);
     const cpl_type    type = cpl_image_get_type(img1);
-    const int         nx = cpl_image_get_size_x(img1);
-    const int         ny = cpl_image_get_size_y(img1);
+    const cpl_size    nx = cpl_image_get_size_x(img1);
+    const cpl_size    ny = cpl_image_get_size_y(img1);
     cpl_image       * med1;
     cpl_image       * medi;
     cpl_mask        * kernel;
@@ -221,7 +229,7 @@ cpl_bivector * cpl_geom_img_offset_fine(
     double          * offsets_x_data;
     double          * offsets_y_data;
     double            offsx, offsy;
-    int               i;
+    cpl_size          i;
     
     /* Check entries */
     cpl_ensure(ilist,     CPL_ERROR_NULL_INPUT, NULL);
@@ -296,9 +304,11 @@ cpl_bivector * cpl_geom_img_offset_fine(
             }
         }
         if (correl_data[i] < 0.0) {
-            cpl_msg_debug(cpl_func, "Frame %d/%d does not correlate in window "
-                          " 1 + 2%d X 1 + 2%d sampling 1 + 2%d X 1 + 2%d",
-                          1+i, nima, s_hx, s_hy, m_hx, m_hy);
+            cpl_msg_debug(cpl_func, "Frame %" CPL_SIZE_FORMAT "/%"
+                          CPL_SIZE_FORMAT " does not correlate in window "
+                          " 1 + 2%" CPL_SIZE_FORMAT " X 1 + 2%" CPL_SIZE_FORMAT
+                          " sampling 1 + 2%" CPL_SIZE_FORMAT " X 1 + 2%"
+                          CPL_SIZE_FORMAT, 1 + i, nima, s_hx, s_hy, m_hx, m_hy);
         }
     }
     
@@ -367,23 +377,25 @@ cpl_bivector * cpl_geom_img_offset_fine(
   - CPL_ERROR_DATA_NOT_FOUND if the shift and add of the images fails
  */
 /*----------------------------------------------------------------------------*/
-cpl_image ** cpl_geom_img_offset_combine(cpl_imagelist      * self,
+cpl_image ** cpl_geom_img_offset_combine(const cpl_imagelist* self,
                                          const cpl_bivector * offs,
                                          int                  refine,
                                          const cpl_bivector * aperts,
                                          const cpl_vector   * sigmas,
-                                         int                * pisigma,
-                                         int                  s_hx,
-                                         int                  s_hy,
-                                         int                  m_hx,
-                                         int                  m_hy,
-                                         int                  min_rej,
-                                         int                  max_rej,
+                                         cpl_size           * pisigma,
+                                         cpl_size             s_hx,
+                                         cpl_size             s_hy,
+                                         cpl_size             m_hx,
+                                         cpl_size             m_hy,
+                                         cpl_size             min_rej,
+                                         cpl_size             max_rej,
                                          cpl_geom_combine     union_flag)
 {
+    const cpl_imagelist* uselist  = self;
+    cpl_imagelist      * modlist  = NULL;
     const cpl_bivector * use_offs = offs;
     cpl_bivector       * offs_fine = NULL;
-    const int            nima = cpl_imagelist_get_size(self);
+    const cpl_size       nima = cpl_imagelist_get_size(self);
     cpl_image         ** combined;
 
 
@@ -417,8 +429,8 @@ cpl_image ** cpl_geom_img_offset_combine(cpl_imagelist      * self,
                 /* ... of the largest object */
                 cpl_apertures_sort_by_npix(apertures);
 
-                anchor_x[0] = cpl_apertures_get_max_x(apertures, 1);
-                anchor_y[0] = cpl_apertures_get_max_y(apertures, 1);
+                anchor_x[0] = cpl_apertures_get_pos_x(apertures, 1);
+                anchor_y[0] = cpl_apertures_get_pos_y(apertures, 1);
                 cpl_apertures_delete(apertures);
 
                 use_aperts = anchor;
@@ -437,14 +449,21 @@ cpl_image ** cpl_geom_img_offset_combine(cpl_imagelist      * self,
                 const double * correl_data = cpl_vector_get_data_const(correl);
                 double       * offs_fine_x = cpl_bivector_get_x_data(offs_fine);
                 double       * offs_fine_y = cpl_bivector_get_y_data(offs_fine);
-                int            ngood = 0;
-                int            i;
+                cpl_size       ngood = 0;
+                cpl_size       i;
+
+                /* Create a modified list, with the correlating images */
+                modlist = cpl_imagelist_new();
 
                 /* Erase the non-correlating offsets and count the good ones */
                 for (i = 0; i < nima; i++) {
                     if (correl_data[i] > -0.5) {
                         offs_fine_x[ngood] = offs_fine_x[i];
                         offs_fine_y[ngood] = offs_fine_y[i];
+                        /* modlist will _not_ be modified */
+                        cpl_imagelist_set(modlist, (cpl_image*)
+                                          cpl_imagelist_get_const(self, i),
+                                          ngood);
                         ngood++;
                     }
                 }
@@ -456,13 +475,10 @@ cpl_image ** cpl_geom_img_offset_combine(cpl_imagelist      * self,
                         cpl_vector_set_size(cpl_bivector_get_y(offs_fine),
                                             ngood);
 
-                        /* Erase the non-correlating images */
-                        cpl_imagelist_erase(self, correl);
+                        uselist = modlist;
 
-                        /*
-                          assert( cpl_bivector_get_size(offs_fine) == ngood );
-                          assert( cpl_imagelist_get_size(self)    == ngood );
-                        */
+                        assert( cpl_bivector_get_size(offs_fine) == ngood );
+                        assert( cpl_imagelist_get_size(uselist)  == ngood );
                     }
 
                     use_offs = offs_fine;
@@ -473,14 +489,23 @@ cpl_image ** cpl_geom_img_offset_combine(cpl_imagelist      * self,
     }
 
     /* Shift & add */
-    combined = cpl_geom_img_offset_saa(self, use_offs, CPL_KERNEL_DEFAULT,
+    combined = cpl_geom_img_offset_saa(uselist, use_offs, CPL_KERNEL_DEFAULT,
                                        min_rej, max_rej, union_flag,
                                        NULL, NULL);
     cpl_bivector_delete(offs_fine);
+    if (modlist != NULL) {
+        cpl_size i = cpl_imagelist_get_size(modlist);
+
+        while (i--) {
+            (void)cpl_imagelist_unset(modlist, i);
+        }
+        cpl_imagelist_delete(modlist);
+    }
 
     if (combined == NULL) {
-        (void)cpl_error_set_message(cpl_func, CPL_ERROR_DATA_NOT_FOUND,
-                                    "Could not shift and add %d images", nima);
+        (void)cpl_error_set_message_(CPL_ERROR_DATA_NOT_FOUND, "Could not "
+                                     "shift and add %" CPL_SIZE_FORMAT
+                                     " images", nima);
     }
 
     return combined;
@@ -516,6 +541,7 @@ cpl_image ** cpl_geom_img_offset_combine(cpl_imagelist      * self,
   - CPL_KERNEL_LANCZOS: Lanczos2 kernel
   - CPL_KERNEL_HAMMING: Hamming kernel
   - CPL_KERNEL_HANN: Hann kernel
+  - CPL_KERNEL_NEAREST: Nearest neighbor kernel (1 when dist < 0.5, else 0)
 
   If the number of input images is lower or equal to 3, the rejection 
   parameters are ignored.
@@ -552,36 +578,36 @@ cpl_image ** cpl_geom_img_offset_combine(cpl_imagelist      * self,
 cpl_image ** cpl_geom_img_offset_saa(const cpl_imagelist * ilist,
                                      const cpl_bivector  * offs,
                                      cpl_kernel            kernel,
-                                     int                   rejmin,
-                                     int                   rejmax,
+                                     cpl_size              rejmin,
+                                     cpl_size              rejmax,
                                      cpl_geom_combine      union_flag,
                                      double              * ppos_x,
                                      double              * ppos_y)
 {
     cpl_imagelist    * copy = (cpl_imagelist *)ilist; /* Not modified */
     cpl_bivector     * offscopy = (cpl_bivector *)offs; /* Not modified */
-    const int          nima  = cpl_imagelist_get_size(ilist);
+    const cpl_size     nima  = cpl_imagelist_get_size(ilist);
     const cpl_image  * img1  = cpl_imagelist_get_const(ilist, 0);
-    const int          sizex = cpl_image_get_size_x(img1);
-    const int          sizey = cpl_image_get_size_y(img1);
+    const cpl_size     sizex = cpl_image_get_size_x(img1);
+    const cpl_size     sizey = cpl_image_get_size_y(img1);
     const cpl_type     type  = cpl_image_get_type(img1);
     const cpl_vector * voffsx;
     const cpl_vector * voffsy;
     /* Test rejection parameters */
     const cpl_boolean  do_rej = nima > 3 && nima > 2*(rejmin+rejmax)
         ? CPL_TRUE : CPL_FALSE;
-    const int          rmin = do_rej ? rejmin : 0;
-    const int          rmax = do_rej ? rejmax : 0;
-    const int          rtot = rmin + rmax;
-    const int          tabsperpix = CPL_KERNEL_TABSPERPIX;
+    const cpl_size     rmin = do_rej ? rejmin : 0;
+    const cpl_size     rmax = do_rej ? rejmax : 0;
+    const cpl_size     rtot = rmin + rmax;
+    const cpl_size     tabsperpix = CPL_KERNEL_TABSPERPIX;
     cpl_vector       * xyprofile;
-    int                nx, ny;
+    cpl_size           nx, ny;
     double             start_x, start_y;
     cpl_image        * final;
     cpl_image        * contrib;
     cpl_image       ** out_arr;
-    int                i_ima;
-    int                firstbpm;
+    cpl_size           i_ima;
+    cpl_size           firstbpm;
 
 
     cpl_ensure(ilist != NULL, CPL_ERROR_NULL_INPUT, NULL);
@@ -624,8 +650,8 @@ cpl_image ** cpl_geom_img_offset_saa(const cpl_imagelist * ilist,
         oy_max = oy_data[nima - rtot - 1];
 
         /* Round down */
-        nx = (int)(sizex + ox_max - start_x);
-        ny = (int)(sizey + oy_max - start_y);
+        nx = (cpl_size)((double)sizex + ox_max - start_x);
+        ny = (cpl_size)((double)sizey + oy_max - start_y);
 
         cpl_vector_delete(ox_tmp);
         cpl_vector_delete(oy_tmp);
@@ -641,8 +667,8 @@ cpl_image ** cpl_geom_img_offset_saa(const cpl_imagelist * ilist,
         if (ppos_y != NULL) *ppos_y = start_y;
 
         /* Round down */
-        nx = (int)(sizex - start_x + ox_min);
-        ny = (int)(sizey - start_y + oy_min);
+        nx = (cpl_size)((double)sizex - start_x + ox_min);
+        ny = (cpl_size)((double)sizey - start_y + oy_min);
 
         cpl_ensure(nx > 0, CPL_ERROR_ILLEGAL_OUTPUT, NULL);
         cpl_ensure(ny > 0, CPL_ERROR_ILLEGAL_OUTPUT, NULL);
@@ -668,17 +694,20 @@ cpl_image ** cpl_geom_img_offset_saa(const cpl_imagelist * ilist,
 
     } else {
         /* union_flag is not one of the three supported modes */
-        cpl_ensure(0, CPL_ERROR_UNSUPPORTED_MODE, NULL);
+        (void)cpl_error_set_(CPL_ERROR_UNSUPPORTED_MODE);
+        return NULL;
     }
 
     xyprofile = cpl_vector_new(CPL_KERNEL_DEF_SAMPLES);
 
-    if (cpl_vector_fill_kernel_profile(xyprofile, kernel,
-                                       CPL_KERNEL_DEF_WIDTH)) {
+    /* The resampling profile is for a resampling routine with a hard-coded
+       radius of 2.0 */
+    if (cpl_vector_fill_kernel_profile(xyprofile, kernel, 2.0)) {
         cpl_vector_delete(xyprofile);
         if (copy != ilist) cpl_imagelist_unwrap(copy);
         if (offscopy != offs) cpl_bivector_unwrap(offscopy);
-        cpl_ensure(0, cpl_error_get_code(), NULL);
+        (void)cpl_error_set_where_();
+        return NULL;
     }
 
     if (copy == ilist) {
@@ -700,7 +729,7 @@ cpl_image ** cpl_geom_img_offset_saa(const cpl_imagelist * ilist,
             cpl_image_accept_all(final);
         } else {
             /* Create an integer image filled with 1's */
-            int * dcontrib = cpl_malloc((size_t)nx * ny * sizeof(*dcontrib));
+            int * dcontrib = cpl_malloc((size_t)(nx * ny) * sizeof(*dcontrib));
 
             contrib = cpl_image_wrap_int(nx, ny, dcontrib);
             (void)cpl_image_fill_int(contrib, 1);
@@ -751,7 +780,8 @@ cpl_image ** cpl_geom_img_offset_saa(const cpl_imagelist * ilist,
             cpl_image_delete(contrib);
             if (copy != ilist) cpl_imagelist_unwrap(copy);
             if (offscopy != offs) cpl_bivector_unwrap(offscopy);
-            cpl_ensure(0, CPL_ERROR_INVALID_TYPE, NULL);
+            (void)cpl_error_set_(CPL_ERROR_INVALID_TYPE);
+            return NULL;
     }
 
     cpl_vector_delete(xyprofile);
@@ -813,16 +843,16 @@ cpl_image ** cpl_geom_img_offset_saa(const cpl_imagelist * ilist,
 static double cpl_geom_ima_offset_xcorr(const cpl_image    * im1,
                                         const cpl_image    * im2,
                                         const cpl_bivector * anchors,
-                                        int                  s_hx,
-                                        int                  s_hy,
-                                        int                  m_hx,
-                                        int                  m_hy,
+                                        cpl_size             s_hx,
+                                        cpl_size             s_hy,
+                                        cpl_size             m_hx,
+                                        cpl_size             m_hy,
                                         double             * offs_x,
                                         double             * offs_y)
 {
-    const int      nx = cpl_image_get_size_x(im1);
-    const int      ny = cpl_image_get_size_y(im1);
-    const int      nanchors = cpl_bivector_get_size(anchors);
+    const cpl_size nx = cpl_image_get_size_x(im1);
+    const cpl_size ny = cpl_image_get_size_y(im1);
+    const cpl_size nanchors = cpl_bivector_get_size(anchors);
     const double * anchor_x = cpl_bivector_get_x_data_const(anchors);
     const double * anchor_y = cpl_bivector_get_y_data_const(anchors);
     cpl_bivector * delta    = cpl_bivector_new(nanchors);
@@ -832,10 +862,10 @@ static double cpl_geom_ima_offset_xcorr(const cpl_image    * im1,
     double       * correl_data = cpl_vector_get_data(correl);
     /* Temporary work array */
     double       * corr_tmp
-        = cpl_malloc((2*s_hx+1) * (2*s_hy+1) * sizeof(double));
+        = cpl_malloc((size_t)(2*s_hx+1) * (size_t)(2*s_hy+1) * sizeof(double));
     double         best_xcorr = -1.0; /* Assume failure */
-    int            nvalid = 0;
-    int            i;
+    cpl_size       nvalid = 0;
+    cpl_size       i;
 
 
     /* Loop on all correlating points */
@@ -857,7 +887,7 @@ static double cpl_geom_ima_offset_xcorr(const cpl_image    * im1,
     cpl_free(corr_tmp);
 
     if (nvalid > 0) {  /* There are valid points */
-        int best_rank = 0;
+        cpl_size best_rank = 0;
 
         if (nvalid > 1) {
             double median_dx, median_dy;
@@ -951,22 +981,22 @@ static double cpl_geom_ima_offset_xcorr(const cpl_image    * im1,
 static double cpl_geom_ima_offset_xcorr_subw(double          * correl,
                                              const cpl_image * im1,
                                              const cpl_image * im2,
-                                             int               im1_posx,
-                                             int               im1_posy,
-                                             int               im2_posx,
-                                             int               im2_posy,
-                                             int               s_hx,
-                                             int               s_hy,
-                                             int               m_hx,
-                                             int               m_hy,
+                                             cpl_size          im1_posx,
+                                             cpl_size          im1_posy,
+                                             cpl_size          im2_posx,
+                                             cpl_size          im2_posy,
+                                             cpl_size          s_hx,
+                                             cpl_size          s_hy,
+                                             cpl_size          m_hx,
+                                             cpl_size          m_hy,
                                              double          * offs_x,
                                              double          * offs_y)
 {
     const cpl_type type = cpl_image_get_type(im1);
-    int            k_min = 0;
-    int            l_min = 0;
+    cpl_size       k_min = 0;
+    cpl_size       l_min = 0;
     double         inc_x, inc_y;
-    int            pos_min;
+    cpl_size       pos_min;
     double         best_correl;
     
     /* Switch on the data type */
@@ -997,7 +1027,8 @@ static double cpl_geom_ima_offset_xcorr_subw(double          * correl,
                                               m_hy,
                                               &k_min, &l_min);
     } else {
-            cpl_ensure(0, CPL_ERROR_INVALID_TYPE, -1.0);
+        (void)cpl_error_set_(CPL_ERROR_INVALID_TYPE);
+        return -1.0;
     }
 
     /* Store results (pixel precision) */
@@ -1039,14 +1070,14 @@ static double cpl_geom_ima_offset_xcorr_subw(double          * correl,
  */
 /*----------------------------------------------------------------------------*/
 inline static void cpl_geom_img_get_min_max_double(double * a,
-                                                   int      n,
-                                                   int      rmin,
-                                                   int      rmax)
+                                                   cpl_size n,
+                                                   cpl_size rmin,
+                                                   cpl_size rmax)
 {
 
     double tmp;
-    const int jeq = CX_MIN(rmin, rmax);
-    int i, j;
+    const cpl_size jeq = CX_MIN(rmin, rmax);
+    cpl_size i, j;
 
 
     /* Gain some locality */
@@ -1095,8 +1126,8 @@ static cpl_imagelist * cpl_imagelist_wrap_all_but_first(cpl_imagelist * other)
 {
 
     cpl_imagelist * self;
-    const int n = cpl_imagelist_get_size(other);
-    int i;
+    const cpl_size n = cpl_imagelist_get_size(other);
+    cpl_size i;
 
 
     cpl_ensure(other != NULL, CPL_ERROR_NULL_INPUT, NULL);
@@ -1123,7 +1154,7 @@ static cpl_imagelist * cpl_imagelist_wrap_all_but_first(cpl_imagelist * other)
 static void cpl_imagelist_unwrap(cpl_imagelist * self)
 {
     if (self != NULL) {
-        int n = cpl_imagelist_get_size(self);
+        cpl_size n = cpl_imagelist_get_size(self);
 
         for (; n > 0; n--) {
             (void)cpl_imagelist_unset(self, n-1);
@@ -1145,7 +1176,7 @@ static void cpl_imagelist_unwrap(cpl_imagelist * self)
 static cpl_bivector * cpl_bivector_wrap_all_but_first(cpl_bivector * self)
 {
 
-    const int      n = cpl_bivector_get_size(self);
+    const cpl_size n = cpl_bivector_get_size(self);
     cpl_vector   * xvec;
     cpl_vector   * yvec;
 
