@@ -1,4 +1,4 @@
-/* $Id: cpl_photom.h,v 1.3 2010/11/11 09:23:18 llundin Exp $
+/* $Id: cpl_photom.h,v 1.4 2011/07/21 09:45:24 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/11/11 09:23:18 $
- * $Revision: 1.3 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2011/07/21 09:45:24 $
+ * $Revision: 1.4 $
+ * $Name: cpl-6_0 $
  */
 
 #ifndef CPL_PHOTOM_H
@@ -32,9 +32,7 @@
                                    Includes
  -----------------------------------------------------------------------------*/
 
-#include <stdio.h>
 #include <cpl_macros.h>
-#include "cpl_error.h"
 
 #include "cpl_vector.h"
 
@@ -48,8 +46,7 @@ CPL_BEGIN_DECLS
  -----------------------------------------------------------------------------*/
 
 cpl_error_code cpl_photom_fill_blackbody(cpl_vector *, cpl_unit,
-                                  const cpl_vector *, cpl_unit,
-                                  double);
+                                         const cpl_vector *, cpl_unit, double);
 
 CPL_END_DECLS
 
