@@ -1,4 +1,4 @@
-/* $Id: cpl_ppm.c,v 1.31 2010/07/31 07:46:35 cizzo Exp $
+/* $Id: cpl_ppm.c,v 1.36 2011/10/06 10:07:48 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2002-2007 European Southern Observatory
@@ -19,10 +19,10 @@
  */
 
 /*
- * $Author: cizzo $
- * $Date: 2010/07/31 07:46:35 $
- * $Revision: 1.31 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Author: llundin $
+ * $Date: 2011/10/06 10:07:48 $
+ * $Revision: 1.36 $
+ * $Name: cpl-6_0 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -92,7 +92,7 @@
  * 
  * Typically, the arc lamp lines candidates @em peak will include 
  * light contaminations, hot pixels, and other unwanted signal, 
- * but only in extreme cases this prevents the pattern-recognition 
+ * but only in extreme cases does this prevent the pattern-recognition 
  * algorithm from identifying all the spectral lines. The pattern 
  * is detected even in the case @em peak contained more arc lamp 
  * lines than actually listed in the input line catalog.
@@ -131,9 +131,9 @@ cpl_bivector *cpl_ppm_match_positions(const cpl_vector *peaks,
 {
 
   int      i, j, k, l;
-  int      nlint, npint;
+  cpl_size nlint, npint;
   int      minpos;
-  float    min;
+  double   min;
   double   lratio, pratio;
   double   lo_start, lo_end, hi_start, hi_end, denom;
   double   disp = 0.0;
@@ -149,7 +149,7 @@ cpl_bivector *cpl_ppm_match_positions(const cpl_vector *peaks,
 
   const double  *peak;
   const double  *line;
-  int      npeaks, nlines;
+  cpl_size       npeaks, nlines;
 
   double  *xpos;
   double  *lambda;
@@ -179,21 +179,21 @@ cpl_bivector *cpl_ppm_match_positions(const cpl_vector *peaks,
   if (npeaks < 4)
       return NULL;
 
-  peak_lo     = cpl_malloc(npeaks * sizeof(int));
-  peak_hi     = cpl_malloc(npeaks * sizeof(int));
-  nident      = cpl_calloc(npeaks, sizeof(int));
-  lident      = cpl_calloc(nlines, sizeof(int));
-  xpos        = cpl_calloc(npeaks, sizeof(double));
-  lambda      = cpl_calloc(npeaks, sizeof(double));
-  ilambda     = cpl_calloc(npeaks, sizeof(int));
-  tmp_xpos    = cpl_calloc(npeaks, sizeof(double));
-  tmp_lambda  = cpl_calloc(npeaks, sizeof(double));
-  tmp_ilambda = cpl_calloc(npeaks, sizeof(int));
-  flag        = cpl_calloc(npeaks, sizeof(int));
-  seq_length  = cpl_calloc(npeaks, sizeof(int));
-  ident       = cpl_malloc(npeaks * sizeof(int *));
+  peak_lo     = cpl_malloc((size_t)npeaks * sizeof(int));
+  peak_hi     = cpl_malloc((size_t)npeaks * sizeof(int));
+  nident      = cpl_calloc((size_t)npeaks, sizeof(int));
+  lident      = cpl_calloc((size_t)nlines, sizeof(int));
+  xpos        = cpl_calloc((size_t)npeaks, sizeof(double));
+  lambda      = cpl_calloc((size_t)npeaks, sizeof(double));
+  ilambda     = cpl_calloc((size_t)npeaks, sizeof(int));
+  tmp_xpos    = cpl_calloc((size_t)npeaks, sizeof(double));
+  tmp_lambda  = cpl_calloc((size_t)npeaks, sizeof(double));
+  tmp_ilambda = cpl_calloc((size_t)npeaks, sizeof(int));
+  flag        = cpl_calloc((size_t)npeaks, sizeof(int));
+  seq_length  = cpl_calloc((size_t)npeaks, sizeof(int));
+  ident       = cpl_malloc((size_t)npeaks * sizeof(int *));
   for (i = 0; i < npeaks; i++)
-    ident[i]  = cpl_malloc(3 * npeaks * sizeof(int));
+      ident[i]  = cpl_malloc(3 * (size_t)npeaks * sizeof(int));
 
   /*
    * This is just the number of intervals - one less than the number
@@ -613,7 +613,7 @@ cpl_bivector *cpl_ppm_match_positions(const cpl_vector *peaks,
                 minpos = k;
             }
           }
-          if (min < 6 && fabs(peak[minpos] - xpos[n-1]) > 1.0) {
+          if (min < 6.0 && fabs(peak[minpos] - xpos[n-1]) > 1.0) {
             xpos[n] = peak[minpos];
             lambda[n] = line[ilambda[n-1] + 1];
             ilambda[n] = ilambda[n-1] + 1;
@@ -666,7 +666,7 @@ cpl_bivector *cpl_ppm_match_positions(const cpl_vector *peaks,
             minpos = k;
         }
       }
-      if (min < 6 && fabs(peak[minpos] - xpos[0]) > 1.0) {
+      if (min < 6.0 && fabs(peak[minpos] - xpos[0]) > 1.0) {
         for (j = n; j > 0; j--) {
           xpos[j] = xpos[j-1];
           lambda[j] = lambda[j-1];
@@ -898,7 +898,7 @@ static triangles *triangles_new(int n)
     triangles *t;
 
     t    = cpl_calloc(1, sizeof(triangles));
-    t->t = cpl_calloc(n, sizeof(triangle *));
+    t->t = cpl_calloc((size_t)n, sizeof(triangle *));
     t->n = n;
  
     return t;
@@ -929,13 +929,13 @@ static void triangles_delete_holder(triangles **t)
 static triangles *
 triangles_from_points(const cpl_matrix *points, int n, double err)
 {
-    triangles    *pattern;
-    const double *m = cpl_matrix_get_data_const(points);
-    double        x1, x2, x3, y1, y2, y3;
-    const int     nc = cpl_matrix_get_ncol(points);
-    int           nt = n*(n-1)*(n-2)/2;
-    int           count;
-    int           i, j, k;
+    triangles     *pattern;
+    const double  *m = cpl_matrix_get_data_const(points);
+    double         x1, x2, x3, y1, y2, y3;
+    const cpl_size nc = cpl_matrix_get_ncol(points);
+    int            nt = n*(n-1)*(n-2)/2;
+    int            count;
+    int            i, j, k;
 
 
     cpl_msg_debug(cpl_func, "Evaluate %d triangles...", nt);
@@ -1114,7 +1114,7 @@ find_matches(triangles *p_triangle, triangles *n_triangle, int np)
 
 static int
 find_transform(cpl_array *matches, const cpl_matrix *pattern, 
-               const cpl_matrix *data, int degree, 
+               const cpl_matrix *data, cpl_size degree, 
                cpl_polynomial *trans_x, cpl_polynomial *trans_y) 
 {
 
@@ -1127,15 +1127,13 @@ find_transform(cpl_array *matches, const cpl_matrix *pattern,
     double         *yd;
     double         *xp;
     double         *yp;
-    int             nvalid = cpl_array_count_invalid(matches);
-    int             np     = cpl_array_get_size(matches);
+    const cpl_size  np     = cpl_array_get_size(matches);
+    const cpl_size  nvalid = np - cpl_array_count_invalid(matches);
     int             null;
     int             count  = 0;
     int             i, j;
 
 
-    nvalid = np - nvalid;
-
     xdpos = cpl_vector_new(nvalid);
     ydpos = cpl_vector_new(nvalid);
     ppos  = cpl_matrix_new(2, nvalid);
@@ -1166,13 +1164,10 @@ find_transform(cpl_array *matches, const cpl_matrix *pattern,
          * Try a 1-D fit
          */
 
-        int degrees[2];
+        const cpl_size degrees[] = {degree, 0};
 
         cpl_errorstate_set(prev_state);
 
-        degrees[0] = degree;
-        degrees[1] = 0;
-
         error = cpl_polynomial_fit(trans_x, ppos, NULL, xdpos, NULL,
                                    CPL_TRUE, NULL, degrees);
 
@@ -1192,13 +1187,10 @@ find_transform(cpl_array *matches, const cpl_matrix *pattern,
              * Try a 1-D fit
              */
 
-            int degrees[2];
+            const cpl_size degrees[] = {0, degree};
 
             cpl_errorstate_set(prev_state);
     
-            degrees[0] = 0;
-            degrees[1] = degree;
-
             error = cpl_polynomial_fit(trans_y, ppos, NULL, ydpos, NULL,
                                        CPL_TRUE, NULL, degrees);
 
@@ -1226,10 +1218,10 @@ find_transform(cpl_array *matches, const cpl_matrix *pattern,
 static int
 nearest_point(double x, double y, const cpl_matrix *matrix)
 {
-    const int     nc     = cpl_matrix_get_ncol(matrix);
-    const double *data   = cpl_matrix_get_data_const(matrix);
-    int           minpos, i;
-    double        min, value;
+    const cpl_size nc     = cpl_matrix_get_ncol(matrix);
+    const double  *data   = cpl_matrix_get_data_const(matrix);
+    int            minpos, i;
+    double         min, value;
 
 
     minpos = 0;
@@ -1252,18 +1244,19 @@ find_all_matches(const cpl_matrix *pattern, const cpl_matrix *data,
                  const cpl_polynomial *trans_y,
                  cpl_matrix **mpattern, cpl_matrix **mdata)
 {
-    cpl_vector   *point   = cpl_vector_new(2);
-    double       *p       = cpl_vector_get_data(point);
-    const double *ddata   = cpl_matrix_get_data_const(data);
-    const int     dnc     = cpl_matrix_get_ncol(data);
-    const double *dpatt   = cpl_matrix_get_data_const(pattern);
-    const int     pnc     = cpl_matrix_get_ncol(pattern);
-    cpl_array    *matches = cpl_array_new(pnc, CPL_TYPE_INT);
-    double        exp_x, exp_y;
-    double       *md;
-    double       *mp;
-    int           null, nvalid;
-    int           i, j, count;
+    cpl_vector    *point   = cpl_vector_new(2);
+    double        *p       = cpl_vector_get_data(point);
+    const double  *ddata   = cpl_matrix_get_data_const(data);
+    const cpl_size dnc     = cpl_matrix_get_ncol(data);
+    const double  *dpatt   = cpl_matrix_get_data_const(pattern);
+    const cpl_size pnc     = cpl_matrix_get_ncol(pattern);
+    cpl_array     *matches = cpl_array_new(pnc, CPL_TYPE_INT);
+    double         exp_x, exp_y;
+    double        *md;
+    double        *mp;
+    cpl_size       nvalid, count;
+    int            null;
+    int            i, j;
 
 
     radius *= radius;
@@ -1455,19 +1448,20 @@ find_all_matches(const cpl_matrix *pattern, const cpl_matrix *data,
  * @em mdata are available):
  *
  * @code
+ *
  * int             degree  = 2;
  * int             npoints = cpl_matrix_get_ncol(mdata);
  * double         *dpoints = cpl_matrix_get_data(mdata);
- * double         *ppoints = cpl_matrix_get_data(mpattern);
  * cpl_vector     *data_x  = cpl_vector_wrap(npoints, dpoints);
  * cpl_vector     *data_y  = cpl_vector_wrap(npoints, dpoints + npoints);
- * cpl_vector     *patt_x  = cpl_vector_wrap(npoints, ppoints);
- * cpl_vector     *patt_y  = cpl_vector_wrap(npoints, ppoints + npoints);
- * cpl_bivector   *patt_xy = cpl_bivector_wrap_vectors(patt_x, patt_y);
- * cpl_polynomial *x_trans = cpl_polynomial_fit_2d_create(patt_xy, data_x, 
- *                                                        degree, NULL);
- * cpl_polynomial *y_trans = cpl_polynomial_fit_2d_create(patt_xy, data_y, 
- *                                                        degree, NULL);
+ * cpl_polynomial *x_trans = cpl_polynomial_new(degree);
+ * cpl_polynomial *y_trans = cpl_polynomial_new(degree);
+ *
+ * cpl_polynomial_fit(x_trans, mpattern, NULL, data_x, NULL, CPL_FALSE,
+ *                    NULL, degree);
+ * cpl_polynomial_fit(y_trans, mpattern, NULL, data_y, NULL, CPL_FALSE,
+ *                    NULL, degree);
+ *
  * @endcode
  *
  * @note
@@ -1537,14 +1531,14 @@ find_all_matches(const cpl_matrix *pattern, const cpl_matrix *data,
  */
 
 cpl_array *
-cpl_ppm_match_points(const cpl_matrix *data, int use_data, double err_data,
-                     const cpl_matrix *pattern, int use_pattern, double 
+cpl_ppm_match_points(const cpl_matrix *data, cpl_size use_data, double err_data,
+                     const cpl_matrix *pattern, cpl_size use_pattern, double
                      err_pattern, double tolerance, double radius, 
                      cpl_matrix **mdata, cpl_matrix **mpattern,
                      double *lin_scale, double *lin_angle)
 {
 
-    int             nd, np;       /* Number of points in data and pattern   */
+    cpl_size        nd, np;       /* Number of points in data and pattern   */
     triangles      *d_triangle;   /* Triangles from data points             */
     triangles      *p_triangle;   /* Triangles from pattern points          */
     triangles      *n_triangle;   /* Nearest d_triangles to each p_triangle */
@@ -1593,6 +1587,15 @@ cpl_ppm_match_points(const cpl_matrix *data, int use_data, double err_data,
         return NULL;
     }
 
+    // FIXME: Remove these checks! Currently only the interface is changed
+    //        to cpl_size. Once the module has been ported completely the
+    //        following tests must be removed.
+
+    if (((int)use_data != use_data) || ((int)use_pattern != use_pattern)) {
+        cpl_error_set(cpl_func, CPL_ERROR_ILLEGAL_INPUT);
+        return NULL;
+    }
+
 /*
     if (use_data < use_pattern) {
         cpl_error_set_message(cpl_func, CPL_ERROR_ILLEGAL_INPUT, 
@@ -1625,13 +1628,14 @@ cpl_ppm_match_points(const cpl_matrix *data, int use_data, double err_data,
      * association is found.
      */
 
+    // FIXME: Remove casts to int when the module is ported to cpl_size
     cpl_msg_debug(cpl_func, "Pattern:");
     cpl_msg_indent_more();
-    p_triangle = triangles_from_points(pattern, use_pattern, err_pattern);
+    p_triangle = triangles_from_points(pattern, (int)use_pattern, err_pattern);
     cpl_msg_indent_less();
     cpl_msg_debug(cpl_func, "Data:");
     cpl_msg_indent_more();
-    d_triangle = triangles_from_points(data, use_data, err_data);
+    d_triangle = triangles_from_points(data, (int)use_data, err_data);
     cpl_msg_indent_less();
     n_triangle = triangles_new(p_triangle->n);
 
@@ -1828,7 +1832,8 @@ cpl_ppm_match_points(const cpl_matrix *data, int use_data, double err_data,
      * column i, element i is left invalid.
      */
 
-    matches = find_matches(p_triangle, n_triangle, use_pattern);
+    // FIXME: Remove casts to int when the module is ported to cpl_size
+    matches = find_matches(p_triangle, n_triangle, (int)use_pattern);
 
     triangles_delete(&p_triangle);
     triangles_delete(&d_triangle);
