@@ -1,4 +1,4 @@
-/* $Id: cpl_wcs.c,v 1.46 2011/02/15 10:35:29 llundin Exp $
+/* $Id: cpl_wcs.c,v 1.50 2012/03/19 09:54:06 rpalsa Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -19,10 +19,10 @@
  */
 
 /*
- * $Author: llundin $
- * $Date: 2011/02/15 10:35:29 $
- * $Revision: 1.46 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Author: rpalsa $
+ * $Date: 2012/03/19 09:54:06 $
+ * $Revision: 1.50 $
+ * $Name: cpl-6_0 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -34,11 +34,6 @@
                                    Includes
  ----------------------------------------------------------------------------*/
 
-#include <math.h>
-#include <stdio.h>
-#include <string.h>
-#include <fitsio.h>
-
 #include <cpl_memory.h>
 #include <cpl_tools.h>
 #include <cpl_error_impl.h>
@@ -47,13 +42,15 @@
 #include <cpl_errorstate.h>
 #include "cpl_wcs.h"
 
+#include <math.h>
+#include <stdio.h>
+#include <string.h>
+#include <fitsio.h>
+
 #ifdef CPL_WCS_INSTALLED    /* If WCS is installed */
 #include <wcslib.h>
 #endif                      /* End If WCS is installed */
 
-/* Remove this when cfitsio is upgraded */
-#include <fitsio2.h>
-
 /*---------------------------------------------------------------------------*/
 /**
  * @defgroup cpl_wcs World Coordinate System
@@ -263,7 +260,7 @@ cpl_wcs *cpl_wcs_new_from_propertylist(const cpl_propertylist *plist) {
 
 	wcs->naxis = wcs->wcsptr->naxis;
 	if (wcs->naxis != 0) {
-	    wcs->dims = cpl_calloc(wcs->naxis,sizeof(int));
+	    wcs->dims = cpl_calloc((size_t)wcs->naxis, sizeof(int));
 	    wcs->imdims = cpl_array_wrap_int(wcs->dims, wcs->naxis);
 	}
         wcs->crval = cpl_array_wrap_double(wcs->wcsptr->crval, wcs->naxis);
@@ -307,7 +304,8 @@ cpl_wcs *cpl_wcs_new_from_propertylist(const cpl_propertylist *plist) {
 
 #else
     cpl_ensure(plist != NULL, CPL_ERROR_NULL_INPUT, NULL);
-    cpl_ensure(0, CPL_ERROR_NO_WCS, NULL);
+    (void)cpl_error_set_(CPL_ERROR_NO_WCS);
+    return NULL;
 #endif                      /* End If WCS is installed */
 }
 
@@ -447,8 +445,10 @@ cpl_error_code cpl_wcs_convert(const cpl_wcs *wcs, const cpl_matrix *from,
                                cpl_matrix **to, cpl_array **status,
                                cpl_wcs_trans_mode transform) {
 #ifdef CPL_WCS_INSTALLED    /* If WCS is installed */
-    const int nrows = cpl_matrix_get_nrow(from);
-    const int ncols = cpl_matrix_get_ncol(from);
+    const cpl_size mrows = cpl_matrix_get_nrow(from);
+    const cpl_size mcols = cpl_matrix_get_ncol(from);
+    const int nrows = (int)mrows;
+    const int ncols = (int)mcols;
     int *sdata,retval;
     cpl_matrix *x1;
     cpl_array *x2,*x3;
@@ -465,6 +465,10 @@ cpl_error_code cpl_wcs_convert(const cpl_wcs *wcs, const cpl_matrix *from,
     /* FIXME: Perhaps this should be an assertion ? */
     cpl_ensure_code(wcs->wcsptr != NULL, CPL_ERROR_NULL_INPUT);
 
+    cpl_ensure_code((cpl_size)nrows == mrows, CPL_ERROR_ILLEGAL_INPUT);
+    cpl_ensure_code((cpl_size)ncols == mcols, CPL_ERROR_ILLEGAL_INPUT);
+
+
     /* Initialise output */
 
     *to = NULL;
@@ -479,11 +483,11 @@ cpl_error_code cpl_wcs_convert(const cpl_wcs *wcs, const cpl_matrix *from,
 
     /* Get the output memory and some memory for wcslib to use */
 
-    *to = cpl_matrix_new(nrows,ncols);
-    *status = cpl_array_new(nrows,CPL_TYPE_INT);
-    x1 = cpl_matrix_new(nrows,ncols);
-    x2 = cpl_array_new(nrows,CPL_TYPE_DOUBLE);
-    x3 = cpl_array_new(nrows,CPL_TYPE_DOUBLE);
+    *to = cpl_matrix_new(mrows, mcols);
+    *status = cpl_array_new(mrows, CPL_TYPE_INT);
+    x1 = cpl_matrix_new(mrows, mcols);
+    x2 = cpl_array_new(mrows, CPL_TYPE_DOUBLE);
+    x3 = cpl_array_new(mrows, CPL_TYPE_DOUBLE);
 
     /* Now get the pointers for the data arrays */
 
@@ -638,6 +642,11 @@ cpl_error_code cpl_wcs_convert(const cpl_wcs *wcs, const cpl_matrix *from,
  *
  * The output property list contains WCS relevant information only.
  *
+ * The matrices @em cel, and @em xy have to be set up in the same way as it
+ * is required for @b cpl_wcs_convert(). See the documentation of
+ * @b cpl_wcs_convert() for details.
+ *
+ * @see cpl_wcs_convert()
  */
 
 cpl_error_code cpl_wcs_platesol(const cpl_propertylist *ilist,
@@ -647,7 +656,7 @@ cpl_error_code cpl_wcs_platesol(const cpl_propertylist *ilist,
                                 cpl_wcs_platesol_outmode outmode,
                                 cpl_propertylist **olist) {
 #ifdef CPL_WCS_INSTALLED    /* If WCS is installed */
-    const int npts = cpl_matrix_get_nrow(cel);
+    const cpl_size npts = cpl_matrix_get_nrow(cel);
     int iter,n,i;
     int * isbad;
     double xifit,etafit,mederr_xi,mederr_eta;
@@ -704,23 +713,23 @@ cpl_error_code cpl_wcs_platesol(const cpl_propertylist *ilist,
     }
     if (npts < 2) {
         cpl_wcs_delete(wcs);
-        return cpl_error_set_message_(CPL_ERROR_UNSPECIFIED,
-                                     "Insufficient points for a fit: "
-                                     "npoints=%d < 2", npts);
+        return cpl_error_set_message_(CPL_ERROR_UNSPECIFIED, "Insufficient "
+                                     "points for a fit: npoints=%"
+                                     CPL_SIZE_FORMAT " < 2", npts);
     }
 
     /* Convert the celestial coordinates to standard coordinates */
 
-    cpl_wcs_convert(wcs,cel,&std,&status,CPL_WCS_WORLD2STD);
+    cpl_wcs_convert(wcs, cel, &std, &status, CPL_WCS_WORLD2STD);
     cpl_array_delete(status);
     stddata = cpl_matrix_get_data_const(std);
 
     /* Get some workspace for rejection algorithm */
-    eta_work = (double*)cpl_malloc(npts * sizeof(double));
-    xi_work  = (double*)cpl_malloc(npts * sizeof(double));
+    eta_work = (double*)cpl_malloc((size_t)npts * sizeof(double));
+    xi_work  = (double*)cpl_malloc((size_t)npts * sizeof(double));
 
     /* Get an array to flag bad pairs */
-    isbad   = (int*)cpl_calloc(npts, sizeof(int));
+    isbad   = (int*)cpl_calloc((size_t)npts, sizeof(int));
     bad     = cpl_array_wrap_int(isbad, npts);
 
     /* Iterative loop */
@@ -761,8 +770,8 @@ cpl_error_code cpl_wcs_platesol(const cpl_propertylist *ilist,
             }
 
             /* Get the median of the array */
-            (void)memcpy(med_work,      xi_work, n * sizeof(double));
-            (void)memcpy(med_work + n, eta_work, n * sizeof(double));
+            (void)memcpy(med_work,      xi_work, (size_t)n * sizeof(double));
+            (void)memcpy(med_work + n, eta_work, (size_t)n * sizeof(double));
             mederr = 1.48 * cpl_tools_get_median_double(med_work, 2 * n);
 
             /* Now reject the bad ones... */
@@ -861,12 +870,13 @@ cpl_error_code cpl_wcs_platesol(const cpl_propertylist *ilist,
         cpl_errorstate    prestate = cpl_errorstate_get();
         cpl_propertylist *pvlist;
         cpl_property     *p;
-        char             *outstr = cpl_calloc((i+1)*80+1,1);
+        char             *outstr = cpl_calloc((size_t)(i+1) * 80 + 1, 1);
         /* FITS END card */
         const char *endcard = "END                            "
             "                                                 ";
+        cpl_size nprop, iprop;
 
-        strncpy(outstr,o,i*80);
+        strncpy(outstr, o, (size_t)i * 80);
         free(o);
         strncat(outstr,endcard,80);
         *olist = cpl_wcs_fitsstr2plist(outstr);
@@ -894,14 +904,14 @@ cpl_error_code cpl_wcs_platesol(const cpl_propertylist *ilist,
 
         pvlist = cpl_propertylist_new();
         cpl_propertylist_copy_property_regexp(pvlist, *olist, "PV", 0);
-        n = cpl_propertylist_get_size(pvlist);
-        for (i = 0; i < n; i++) {
-            p = cpl_propertylist_get(pvlist,i);
+        nprop = cpl_propertylist_get_size(pvlist);
+        for (iprop = 0; iprop < nprop; iprop++) {
+            p = cpl_propertylist_get(pvlist, iprop);
             if (cpl_property_get_type(p) == CPL_TYPE_INT) {
-                const double dval = (double)cpl_property_get_int(p);
+                const int ival = cpl_property_get_int(p);
                 cpl_propertylist_erase(*olist, cpl_property_get_name(p));
                 cpl_propertylist_append_double(*olist, cpl_property_get_name(p),
-                                               dval);
+                                               (double)ival);
             }
         }
         cpl_propertylist_delete(pvlist);
@@ -988,7 +998,8 @@ int cpl_wcs_is_table(const cpl_wcs *wcs) {
     /* Return the relevant value */
     return(wcs->istab);
 #else
-    cpl_ensure(0, CPL_ERROR_NO_WCS, -1);
+    (void)cpl_error_set_(CPL_ERROR_NO_WCS);
+    return -1;
 #endif                      /* End If WCS is installed */
 }
 #endif
@@ -1033,7 +1044,8 @@ int cpl_wcs_get_image_naxis(const cpl_wcs *wcs) {
     /* Return the relevant value */
     return(wcs->naxis);
 #else
-    cpl_ensure(0,CPL_ERROR_NO_WCS,0);
+    (void)cpl_error_set_(CPL_ERROR_NO_WCS);
+    return 0;
 #endif                      /* End If WCS is installed */
 }
 
@@ -1078,7 +1090,8 @@ const cpl_array *cpl_wcs_get_image_dims(const cpl_wcs *wcs) {
     return wcs->naxis == 0 ? NULL : wcs->imdims;
 
 #else
-    cpl_ensure(0,CPL_ERROR_NO_WCS,NULL);
+    (void)cpl_error_set_(CPL_ERROR_NO_WCS);
+    return NULL;
 #endif                      /* End If WCS is installed */
 }
 
@@ -1123,7 +1136,8 @@ const cpl_array *cpl_wcs_get_crval(const cpl_wcs *wcs) {
 
 #else
     cpl_ensure(wcs != NULL, CPL_ERROR_NULL_INPUT, NULL);
-    cpl_ensure(0,CPL_ERROR_NO_WCS,NULL);
+    (void)cpl_error_set_(CPL_ERROR_NO_WCS);
+    return NULL;
 #endif                      /* End If WCS is installed */
 }
 
@@ -1168,7 +1182,8 @@ const cpl_array *cpl_wcs_get_crpix(const cpl_wcs *wcs) {
 
 #else
     cpl_ensure(wcs != NULL, CPL_ERROR_NULL_INPUT, NULL);
-    cpl_ensure(0,CPL_ERROR_NO_WCS,NULL);
+    (void)cpl_error_set_(CPL_ERROR_NO_WCS);
+    return NULL;
 #endif                      /* End If WCS is installed */
 }
 
@@ -1211,7 +1226,8 @@ const cpl_matrix *cpl_wcs_get_cd(const cpl_wcs *wcs) {
 
     return wcs->wcsptr->naxis == 0 ? NULL : wcs->cd;
 #else
-    cpl_ensure(0, CPL_ERROR_NO_WCS,NULL);
+    (void)cpl_error_set_(CPL_ERROR_NO_WCS);
+    return NULL;
 #endif                      /* End If WCS is installed */
 }
 
@@ -1437,11 +1453,11 @@ static void cpl_wcs_platesol_6(const cpl_matrix *xy, const cpl_matrix *std,
     double sx1sq,sy1sq,sx1y1,sx1x2,sy1x2,*pc;
     double sy1y2,sx1y2,xposmean,yposmean,ximean,etamean,xx1,yy1,xx2,yy2;
     /* Get some convenience variables */
-    const double *xydata = cpl_matrix_get_data_const(xy);
-    const double *stddata = cpl_matrix_get_data_const(std);
-    const int    *isbad = cpl_array_get_data_int_const(bad);
-    const int     nstds = cpl_array_get_size(bad);
-    int           i,ngood;
+    const double  *xydata = cpl_matrix_get_data_const(xy);
+    const double  *stddata = cpl_matrix_get_data_const(std);
+    const int     *isbad = cpl_array_get_data_int_const(bad);
+    const cpl_size nstds = cpl_array_get_size(bad);
+    cpl_size       i,ngood;
 
     /* Initialise all the counters and summations */
 
@@ -1539,11 +1555,11 @@ static void cpl_wcs_platesol_4(const cpl_matrix *xy, const cpl_matrix *std,
     double ximean,etamean,xx1,yy1,xx2,yy2,det,num,denom,theta,mag;
     double stheta,ctheta;
     /* Get some convenience variables */
-    const double *xydata = cpl_matrix_get_data_const(xy);
-    const double *stddata = cpl_matrix_get_data_const(std);
-    const int    *isbad = cpl_array_get_data_int_const(bad);
-    const int     nstds = cpl_array_get_size(bad);
-    int           i,ngood;
+    const double  *xydata = cpl_matrix_get_data_const(xy);
+    const double  *stddata = cpl_matrix_get_data_const(std);
+    const int     *isbad = cpl_array_get_data_int_const(bad);
+    const cpl_size nstds = cpl_array_get_size(bad);
+    cpl_size       i,ngood;
 
 
     /* Initialise all the counters and summations */
