@@ -1,4 +1,4 @@
-/* $Id: cpl_detector-test.c,v 1.28 2010/12/10 13:41:37 llundin Exp $
+/* $Id: cpl_detector-test.c,v 1.29 2011/04/01 09:57:38 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/12/10 13:41:37 $
- * $Revision: 1.28 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2011/04/01 09:57:38 $
+ * $Revision: 1.29 $
+ * $Name: cpl-6_0 $
  */
 
 /*-----------------------------------------------------------------------------
@@ -141,7 +141,7 @@ int main(void)
                   cpl_detector_interpolate_rejected()
                   to enable a regression test of your changes.
                 */
-                cpl_image_save(im, "regression.fits", CPL_BPP_IEEE_DOUBLE,
+                cpl_image_save(im, "regression.fits", CPL_TYPE_DOUBLE,
                                NULL, CPL_IO_CREATE);
 #endif
             } else {
