@@ -1,4 +1,4 @@
-/* $Id: cpl_geom_img-test.c,v 1.41 2010/11/12 09:46:44 llundin Exp $
+/* $Id: cpl_geom_img-test.c,v 1.49 2011/12/20 09:11:42 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/11/12 09:46:44 $
- * $Revision: 1.41 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2011/12/20 09:11:42 $
+ * $Revision: 1.49 $
+ * $Name: cpl-6_0 $
  */
 
 /*-----------------------------------------------------------------------------
@@ -64,11 +64,13 @@
                                   Pricate functions
  -----------------------------------------------------------------------------*/
 
+static void cpl_geom_img_offset_saa_one(cpl_kernel);
+
 static
 void cpl_geom_img_offset_saa_bench(cpl_geom_combine, int, int, int, int, int);
 
-static void cpl_imagelist_fill_shifted(cpl_imagelist *, int, const double *, 
-                                       const double *);
+static void cpl_imagelist_fill_shifted(cpl_imagelist *, cpl_size,
+                                       const double *,  const double *);
 
 /**@{*/
 
@@ -77,6 +79,9 @@ static void cpl_imagelist_fill_shifted(cpl_imagelist *, int, const double *,
  -----------------------------------------------------------------------------*/
 int main(void)
 {
+    /* These kernels preserve the actual pixel-values */
+    cpl_kernel kernels[] = {CPL_KERNEL_DEFAULT,
+                            CPL_KERNEL_NEAREST};
     const cpl_geom_combine geoms[] = {CPL_GEOM_INTERSECT, CPL_GEOM_UNION,
                                       CPL_GEOM_FIRST};
     /* Shift by non-integer amount to evaluate resampling */
@@ -100,7 +105,7 @@ int main(void)
     cpl_vector      *   sigmas;
     cpl_image       **  combined;
     int                 i;
-    int                 pos;
+    cpl_size            pos;
 
 
     cpl_test_init(PACKAGE_BUGREPORT, CPL_MSG_WARNING);
@@ -114,6 +119,9 @@ int main(void)
         cpl_test_leq(fabs(off_y_init[i]), IMAGESZ);
     }
 
+    cpl_geom_img_offset_saa_one(CPL_KERNEL_DEFAULT);
+    cpl_geom_img_offset_saa_one(CPL_KERNEL_NEAREST);
+
     if (cpl_msg_get_level() <= CPL_MSG_INFO) {
         const double tprev = cpl_test_get_cputime();
         const cpl_flops fprev = cpl_tools_get_flops();
@@ -143,6 +151,9 @@ int main(void)
                      cputime,
                      cputime > 0.0 ? (double)nflops/cputime/1e6 : 0.0);
 
+    } else {
+        cpl_geom_img_offset_saa_bench(CPL_GEOM_FIRST, 1, 4, IMAGESZ/4,
+                                      IMAGESZ/4, 1);
     }
 
 
@@ -237,8 +248,9 @@ int main(void)
 
     cpl_test_leq(1, naperts);
 
-    cpl_msg_info("","Detected %d apertures at sigma %g (%d/%d)", naperts,
-                 psigmas[pos], 1+pos, NSIGMAS);
+    cpl_msg_info("","Detected %d apertures at sigma %g (%" CPL_SIZE_FORMAT "/%"
+                 CPL_SIZE_FORMAT ")", naperts, psigmas[pos], 1+pos,
+                 (cpl_size)NSIGMAS);
     if (cpl_msg_get_level() <= CPL_MSG_DEBUG)
         cpl_apertures_dump(aperts, stdout);
 
@@ -246,16 +258,16 @@ int main(void)
     aperts_pos_x = cpl_bivector_get_x(aperts_pos);
     aperts_pos_y = cpl_bivector_get_y(aperts_pos);
     for (i=0; i<naperts; i++) {
-        cpl_vector_set(aperts_pos_x, i, cpl_apertures_get_max_x(aperts, i+1));
-        cpl_vector_set(aperts_pos_y, i, cpl_apertures_get_max_y(aperts, i+1));
+        cpl_vector_set(aperts_pos_x, i, cpl_apertures_get_pos_x(aperts, i+1));
+        cpl_vector_set(aperts_pos_y, i, cpl_apertures_get_pos_y(aperts, i+1));
     }
     cpl_apertures_delete(aperts);
 
     cpl_test_error(CPL_ERROR_NONE);
    
     /* Refine the offsets with cpl_geom_img_offset_fine */
-    cpl_msg_info("","Refine the offsets for %d images using %d anchors",
-                 NFRAMES, cpl_bivector_get_size(aperts_pos));
+    cpl_msg_info("","Refine the offsets for %d images using %" CPL_SIZE_FORMAT
+                 " anchors", NFRAMES, cpl_bivector_get_size(aperts_pos));
     correl = cpl_vector_new(NFRAMES);
 
     offs_ref = cpl_geom_img_offset_fine(iset, offs_est, aperts_pos,
@@ -348,55 +360,62 @@ int main(void)
         cpl_image_accept_all(cpl_imagelist_get(iset, i));
     }
 
+
     for (i = 0; i < (int)(sizeof(geoms)/sizeof(geoms[0])); i++) {
-        const cpl_geom_combine geom = geoms[i];
-        const int rejmin = 1;
-        const int rejmax = 1;
-        const int maximg = NFRAMES - rejmin - rejmax;
+        int ityp;
+        for (ityp = 0; ityp < (int)(sizeof(kernels)/sizeof(kernels[0]));
+             ityp++) {
 
-        /* Shift and add */
-        cpl_msg_info("", "Shift and add with geom number %d", (int)geom);
-        combined = cpl_geom_img_offset_saa(iset, offs_ref,  CPL_KERNEL_DEFAULT,
-                                           rejmin, rejmax, geom, NULL, NULL);
+            const cpl_geom_combine geom = geoms[i];
+            const int rejmin = 1;
+            const int rejmax = 1;
+            const int maximg = NFRAMES - rejmin - rejmax;
 
-        cpl_test_nonnull(combined);
+            /* Shift and add */
+            cpl_msg_info("", "Shift and add with geom number %d and kernel "
+                         "type %d", (int)geom, (int)kernels[ityp]);
+            combined = cpl_geom_img_offset_saa(iset, offs_ref, kernels[ityp],
+                                               rejmin, rejmax, geom,
+                                               NULL, NULL);
+            cpl_test_error(CPL_ERROR_NONE);
+            cpl_test_nonnull(combined);
 
-        if (combined == NULL) continue;
+            if (combined == NULL) continue;
 
-        cpl_test_eq(cpl_image_get_type(combined[1]), CPL_TYPE_INT);
+            cpl_test_eq(cpl_image_get_type(combined[1]), CPL_TYPE_INT);
 
-        if (cpl_image_get_min(combined[1]) == 0) {
-            cpl_test(cpl_image_count_rejected(combined[0]));
-        } else {
-            cpl_test_leq(1, cpl_image_get_min(combined[1]));
-            cpl_test_zero(cpl_image_count_rejected(combined[0]));
+            if (cpl_image_get_min(combined[1]) == 0) {
+                cpl_test(cpl_image_count_rejected(combined[0]));
+            } else {
+                cpl_test_leq(1, cpl_image_get_min(combined[1]));
+                cpl_test_zero(cpl_image_count_rejected(combined[0]));
+            }
+
+            cpl_test_eq(cpl_image_get_size_x(combined[0]),
+                        cpl_image_get_size_x(combined[1]));
+
+            cpl_test_eq(cpl_image_get_size_y(combined[0]),
+                        cpl_image_get_size_y(combined[1]));
+
+            cpl_test_leq(cpl_image_get_max(combined[1]), maximg);
+
+            if (geom == CPL_GEOM_INTERSECT) {
+                cpl_test_eq(cpl_image_get_max(combined[1]), maximg);
+                cpl_test_leq(1, cpl_image_get_min(combined[1]));
+            } else if (geom == CPL_GEOM_FIRST) {
+                /* FIXME: Should at least be 1 */
+                cpl_test_leq(0, cpl_image_get_min(combined[1]));
+            } else if (geom == CPL_GEOM_UNION) {
+                cpl_test_leq(0, cpl_image_get_min(combined[1]));
+            }
+
+            cpl_msg_info("", "Minimum value in contribution map: %g",
+                         cpl_image_get_min(combined[1]));
+
+            cpl_image_delete(combined[0]);
+            cpl_image_delete(combined[1]);
+            cpl_free(combined);
         }
-
-        cpl_test_eq(cpl_image_get_size_x(combined[0]),
-                    cpl_image_get_size_x(combined[1]));
-
-        cpl_test_eq(cpl_image_get_size_y(combined[0]),
-                    cpl_image_get_size_y(combined[1]));
-
-        cpl_test_leq(cpl_image_get_max(combined[1]), maximg);
-
-        if (geom == CPL_GEOM_INTERSECT) {
-            cpl_test_eq(cpl_image_get_max(combined[1]), maximg);
-            cpl_test_leq(1, cpl_image_get_min(combined[1]));
-        } else if (geom == CPL_GEOM_FIRST) {
-            /* FIXME: Should at least be 1 */
-            cpl_test_leq(0, cpl_image_get_min(combined[1]));
-        } else if (geom == CPL_GEOM_UNION) {
-            cpl_test_leq(0, cpl_image_get_min(combined[1]));
-        }
-
-        cpl_msg_info("", "Minimum value in contribution map: %g",
-                     cpl_image_get_min(combined[1]));
-
-        cpl_image_delete(combined[0]);
-        cpl_image_delete(combined[1]);
-        cpl_free(combined);
-
     }
 
     cpl_bivector_delete(offs_ref);
@@ -421,148 +440,155 @@ int main(void)
         cpl_plot_image("","","", img);
 
     for (i = 0; i < (int)(sizeof(geoms)/sizeof(geoms[0])); i++) {
-        const cpl_geom_combine geom = geoms[i];
-        double pos_x, pos_y;
+        int ityp;
+        for (ityp = 0; ityp < (int)(sizeof(kernels)/sizeof(kernels[0]));
+             ityp++) {
 
-        cpl_msg_info("", "Shift and add with geom number %d", (int)geom);
+            const cpl_geom_combine geom = geoms[i];
+            double pos_x, pos_y;
 
-        combined = cpl_geom_img_offset_saa(iset, offs_ref,  CPL_KERNEL_DEFAULT,
-                                           0, 0, geom, &pos_x, &pos_y);
+            cpl_msg_info("", "Shift and add with geom number %d and kernel "
+                         "type %d", (int)geom, (int)kernels[ityp]);
 
-        cpl_test_nonnull(combined);
+            combined = cpl_geom_img_offset_saa(iset, offs_ref, kernels[ityp],
+                                               0, 0, geom, &pos_x, &pos_y);
 
-        if (combined == NULL) continue;
-
-        cpl_test_eq(cpl_image_get_size_x(combined[0]), IMAGESZ);
-        cpl_test_eq(cpl_image_get_size_x(combined[0]), IMAGESZ);
-
-        cpl_test_eq(cpl_image_get_type(combined[1]), CPL_TYPE_INT);
-
-        cpl_test_eq(cpl_image_get_size_x(combined[0]),
-                    cpl_image_get_size_x(combined[1]));
-
-        cpl_test_eq(cpl_image_get_size_y(combined[0]),
-                    cpl_image_get_size_y(combined[1]));
-
-        if (cpl_image_get_min(combined[1]) == 0) {
-            cpl_test(cpl_image_count_rejected(combined[0]));
-        } else {
-            cpl_test_leq(1, cpl_image_get_min(combined[1]));
-            cpl_test_zero(cpl_image_count_rejected(combined[0]));
-        }
-
-        cpl_test_eq(cpl_image_get_max(combined[1]), 2);
-
-        if (geom == CPL_GEOM_INTERSECT) {
-            cpl_test_eq(cpl_image_get_max(combined[1]), 2);
-            /* FIXME: Should at least be 1 */
-            cpl_test_leq(0, cpl_image_get_min(combined[1]));
-        } else if (geom == CPL_GEOM_FIRST) {
-            /* FIXME: Minimum value is zero */
-            cpl_test_leq(0, cpl_image_get_min(combined[1]));
-        } else if (geom == CPL_GEOM_UNION) {
-            cpl_test_leq(0, cpl_image_get_min(combined[1]));
-        }
-
-#ifdef TEST_RESAMPLING
-        /* Resampling introduces noise at the edge */
-        /* NB: Comparison works for all modes, due to zero offset ... */
-        cpl_test_image_abs(combined[0], img, MAX_SHIFT_ERROR2);
-#endif
-
-        if (cpl_msg_get_level() <= CPL_MSG_DEBUG) {
-            cpl_image_subtract(combined[0], img);
-            cpl_plot_image("","","", combined[0]);
-        }
-
-        cpl_image_delete(combined[0]);
-        cpl_image_delete(combined[1]);
-        cpl_free(combined);
-
-
-        /* Now try to combine two images, the second shifted along the X-axis */
-
-        cpl_image_shift(img, -MAX_SHIFT_ERROR1, 0);
-        cpl_image_accept_all(img);
-
-        cpl_vector_set(off_vec_x, 1, MAX_SHIFT_ERROR1);
-
-        combined = cpl_geom_img_offset_saa(iset, offs_ref, CPL_KERNEL_DEFAULT,
-                                           0, 0, geom, &pos_x, &pos_y);
-
-        cpl_test_nonnull(combined);
-
-        if (combined == NULL) continue;
-
-        cpl_test_eq(cpl_image_get_max(combined[1]), 2);
-
-        if (cpl_image_get_min(combined[1]) == 0) {
-            cpl_test(cpl_image_count_rejected(combined[0]));
-        } else {
-            cpl_test_zero(cpl_image_count_rejected(combined[0]));
-        }
-
-        cpl_test_eq(cpl_image_get_size_y(combined[0]), IMAGESZ);
-
-        if (geom == CPL_GEOM_INTERSECT) {
-
-#ifdef SAVE_COMBINED
-            cpl_image_save(combined[0], "PI.fits", CPL_BPP_IEEE_DOUBLE,
-                           NULL, CPL_IO_CREATE);
-            cpl_image_save(combined[1], "CI.fits", CPL_BPP_8_UNSIGNED,
-                           NULL, CPL_IO_CREATE);
-#endif
-
-            cpl_test_eq(cpl_image_get_size_x(combined[0]), IMAGESZ
-                        - MAX_SHIFT_ERROR1);
-
-        } else if (geom == CPL_GEOM_FIRST) {
-
-#ifdef SAVE_COMBINED
-            cpl_image_save(combined[0], "PF.fits", CPL_BPP_IEEE_DOUBLE,
-                           NULL, CPL_IO_CREATE);
-            cpl_image_save(combined[1], "CF.fits", CPL_BPP_8_UNSIGNED,
-                           NULL, CPL_IO_CREATE);
-#endif
+            cpl_test_nonnull(combined);
 
+            if (combined == NULL) continue;
 
+            cpl_test_eq(cpl_image_get_size_x(combined[0]), IMAGESZ);
             cpl_test_eq(cpl_image_get_size_x(combined[0]), IMAGESZ);
 
-        } else if (geom == CPL_GEOM_UNION) {
-            cpl_test_eq(cpl_image_get_size_x(combined[0]), IMAGESZ
-                        + MAX_SHIFT_ERROR1);
+            cpl_test_eq(cpl_image_get_type(combined[1]), CPL_TYPE_INT);
 
-#ifdef SAVE_COMBINED
-            cpl_image_save(combined[0], "PU.fits", CPL_BPP_IEEE_DOUBLE,
-                           NULL, CPL_IO_CREATE);
-            cpl_image_save(combined[1], "CU.fits", CPL_BPP_8_UNSIGNED,
-                           NULL, CPL_IO_CREATE);
+            cpl_test_eq(cpl_image_get_size_x(combined[0]),
+                        cpl_image_get_size_x(combined[1]));
+
+            cpl_test_eq(cpl_image_get_size_y(combined[0]),
+                        cpl_image_get_size_y(combined[1]));
+
+            if (cpl_image_get_min(combined[1]) == 0) {
+                cpl_test(cpl_image_count_rejected(combined[0]));
+            } else {
+                cpl_test_leq(1, cpl_image_get_min(combined[1]));
+                cpl_test_zero(cpl_image_count_rejected(combined[0]));
+            }
+
+            cpl_test_eq(cpl_image_get_max(combined[1]), 2);
+
+            if (geom == CPL_GEOM_INTERSECT) {
+                cpl_test_eq(cpl_image_get_max(combined[1]), 2);
+                /* FIXME: Should at least be 1 */
+                cpl_test_leq(0, cpl_image_get_min(combined[1]));
+            } else if (geom == CPL_GEOM_FIRST) {
+                /* FIXME: Minimum value is zero */
+                cpl_test_leq(0, cpl_image_get_min(combined[1]));
+            } else if (geom == CPL_GEOM_UNION) {
+                cpl_test_leq(0, cpl_image_get_min(combined[1]));
+            }
+
+#ifdef TEST_RESAMPLING
+            /* Resampling introduces noise at the edge */
+            /* NB: Comparison works for all modes, due to zero offset ... */
+            cpl_test_image_abs(combined[0], img, MAX_SHIFT_ERROR2);
 #endif
 
-            cpl_test_eq(cpl_image_get_min(combined[1]), 1);
-        }
-
-
-        img = cpl_imagelist_get(iset, 0);
-
-        if (cpl_msg_get_level() <= CPL_MSG_DEBUG) {
-            cpl_plot_image("","","", combined[0]);
-
-            if (geom == CPL_GEOM_FIRST) {
+            if (cpl_msg_get_level() <= CPL_MSG_DEBUG) {
                 cpl_image_subtract(combined[0], img);
                 cpl_plot_image("","","", combined[0]);
             }
+
+            cpl_image_delete(combined[0]);
+            cpl_image_delete(combined[1]);
+            cpl_free(combined);
+
+
+            /* Now try to combine two images, the second shifted along the X-axis */
+
+            cpl_image_shift(img, -MAX_SHIFT_ERROR1, 0);
+            cpl_image_accept_all(img);
+
+            cpl_vector_set(off_vec_x, 1, MAX_SHIFT_ERROR1);
+
+            combined = cpl_geom_img_offset_saa(iset, offs_ref, kernels[ityp],
+                                               0, 0, geom, &pos_x, &pos_y);
+
+            cpl_test_error(CPL_ERROR_NONE);
+            cpl_test_nonnull(combined);
+
+            if (combined == NULL) continue;
+
+            cpl_test_eq(cpl_image_get_max(combined[1]), 2);
+
+            if (cpl_image_get_min(combined[1]) == 0) {
+                cpl_test(cpl_image_count_rejected(combined[0]));
+            } else {
+                cpl_test_zero(cpl_image_count_rejected(combined[0]));
+            }
+
+            cpl_test_eq(cpl_image_get_size_y(combined[0]), IMAGESZ);
+
+            if (geom == CPL_GEOM_INTERSECT) {
+
+#ifdef SAVE_COMBINED
+                cpl_image_save(combined[0], "PI.fits", CPL_TYPE_DOUBLE,
+                               NULL, CPL_IO_CREATE);
+                cpl_image_save(combined[1], "CI.fits", CPL_TYPE_UCHAR,
+                               NULL, CPL_IO_CREATE);
+#endif
+
+                cpl_test_eq(cpl_image_get_size_x(combined[0]), IMAGESZ
+                            - MAX_SHIFT_ERROR1);
+
+            } else if (geom == CPL_GEOM_FIRST) {
+
+#ifdef SAVE_COMBINED
+                cpl_image_save(combined[0], "PF.fits", CPL_TYPE_DOUBLE,
+                               NULL, CPL_IO_CREATE);
+                cpl_image_save(combined[1], "CF.fits", CPL_TYPE_UCHAR,
+                               NULL, CPL_IO_CREATE);
+#endif
+
+
+                cpl_test_eq(cpl_image_get_size_x(combined[0]), IMAGESZ);
+
+            } else if (geom == CPL_GEOM_UNION) {
+                cpl_test_eq(cpl_image_get_size_x(combined[0]), IMAGESZ
+                            + MAX_SHIFT_ERROR1);
+
+#ifdef SAVE_COMBINED
+                cpl_image_save(combined[0], "PU.fits", CPL_TYPE_DOUBLE,
+                               NULL, CPL_IO_CREATE);
+                cpl_image_save(combined[1], "CU.fits", CPL_TYPE_UCHAR,
+                               NULL, CPL_IO_CREATE);
+#endif
+
+                cpl_test_eq(cpl_image_get_min(combined[1]), 1);
+            }
+
+
+            img = cpl_imagelist_get(iset, 0);
+
+            if (cpl_msg_get_level() <= CPL_MSG_DEBUG) {
+                cpl_plot_image("","","", combined[0]);
+
+                if (geom == CPL_GEOM_FIRST) {
+                    cpl_image_subtract(combined[0], img);
+                    cpl_plot_image("","","", combined[0]);
+                }
+            }
+
+            cpl_image_delete(combined[0]);
+            cpl_image_delete(combined[1]);
+            cpl_free(combined);
+
+            /* Reset offset and 2nd image */
+            cpl_vector_fill(off_vec_x, 0.0);
+            img = cpl_image_duplicate(img);
+            cpl_imagelist_set(iset, img, 1);
+
         }
-
-        cpl_image_delete(combined[0]);
-        cpl_image_delete(combined[1]);
-        cpl_free(combined);
-
-        /* Reset offset and 2nd image */
-        cpl_vector_fill(off_vec_x, 0.0);
-        img = cpl_image_duplicate(img);
-        cpl_imagelist_set(iset, img, 1);
-
     }
 
     cpl_imagelist_delete(iset);
@@ -640,12 +666,17 @@ void cpl_geom_img_offset_saa_bench(cpl_geom_combine mode, int nr, int nz,
 
         combined = cpl_geom_img_offset_saa(imglist, offset, CPL_KERNEL_DEFAULT,
                                            no, no, mode, NULL, NULL);
+        cpl_test_error(CPL_ERROR_NONE);
+        cpl_test_nonnull(combined);
 
         cputime += cpl_test_get_cputime() - secs;
         nflops += cpl_tools_get_flops() - flops;
 
         if (combined == NULL) continue;
 
+        cpl_test_nonnull(combined[0]);
+        cpl_test_nonnull(combined[1]);
+
         cpl_image_delete(combined[0]);
         cpl_image_delete(combined[1]);
         cpl_free(combined);
@@ -681,26 +712,26 @@ void cpl_geom_img_offset_saa_bench(cpl_geom_combine mode, int nr, int nz,
  */
 /*----------------------------------------------------------------------------*/
 static
-void cpl_imagelist_fill_shifted(cpl_imagelist * self, int napp,
+void cpl_imagelist_fill_shifted(cpl_imagelist * self, cpl_size napp,
                                 const double * dx, const double * dy)
 {
 
     const cpl_image * img         = cpl_imagelist_get_const(self, 0);
-    const int         type        = cpl_image_get_type  (img);
-    const int         nx          = cpl_image_get_size_x(img);
-    const int         ny          = cpl_image_get_size_x(img);
-    const int         ishift_0[2] = {0, 0};
-    const int         ishift_x[2] = {1, 0};
-    const int         ishift_y[2] = {0, 1};
+    const cpl_size    type        = cpl_image_get_type  (img);
+    const cpl_size    nx          = cpl_image_get_size_x(img);
+    const cpl_size    ny          = cpl_image_get_size_x(img);
+    const cpl_size    ishift_0[2] = {0, 0};
+    const cpl_size    ishift_x[2] = {1, 0};
+    const cpl_size    ishift_y[2] = {0, 1};
 
-    const double xyradius = CPL_KERNEL_DEF_WIDTH;
+    const double      xyradius = CPL_KERNEL_DEF_WIDTH;
   
-    cpl_vector * xyprofile = cpl_vector_new(CPL_KERNEL_DEF_SAMPLES);
+    cpl_vector      * xyprofile = cpl_vector_new(CPL_KERNEL_DEF_SAMPLES);
 
-    cpl_polynomial * shift_x = cpl_polynomial_new(2);
-    cpl_polynomial * shift_y = cpl_polynomial_new(2);
-    cpl_error_code   error;
-    int i;
+    cpl_polynomial  * shift_x = cpl_polynomial_new(2);
+    cpl_polynomial  * shift_y = cpl_polynomial_new(2);
+    cpl_error_code    error;
+    cpl_size          i;
 
 
     cpl_test_eq(cpl_imagelist_get_size(self), 1);
@@ -710,9 +741,10 @@ void cpl_imagelist_fill_shifted(cpl_imagelist * self, int napp,
 
 
     /* Identity transforms */
-    cpl_polynomial_set_coeff(shift_x, ishift_x, 1.0);
-    cpl_polynomial_set_coeff(shift_y, ishift_y, 1.0);
-    cpl_test_error(CPL_ERROR_NONE);
+    error = cpl_polynomial_set_coeff(shift_x, ishift_x, 1.0);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+    error = cpl_polynomial_set_coeff(shift_y, ishift_y, 1.0);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
 
     /* Resampling profile */
     error = cpl_vector_fill_kernel_profile(xyprofile, CPL_KERNEL_DEFAULT,
@@ -724,15 +756,20 @@ void cpl_imagelist_fill_shifted(cpl_imagelist * self, int napp,
         cpl_image * copy = cpl_image_new(nx, ny, type);
 
         /* Shift in X and Y */
-        cpl_polynomial_set_coeff(shift_x, ishift_0, dx[i]);
-        cpl_polynomial_set_coeff(shift_y, ishift_0, dy[i]);
+        error = cpl_polynomial_set_coeff(shift_x, ishift_0, dx[i]);
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
 
-        cpl_image_warp_polynomial(copy, img, shift_x, shift_y,
+        error = cpl_polynomial_set_coeff(shift_y, ishift_0, dy[i]);
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+        error = cpl_image_warp_polynomial(copy, img, shift_x, shift_y,
                                           xyprofile, xyradius,
                                           xyprofile, xyradius);
-        cpl_imagelist_set(self, copy, i);
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+        error = cpl_imagelist_set(self, copy, i);
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
 
-        cpl_test_error(CPL_ERROR_NONE);
     }
 
     cpl_polynomial_delete(shift_x);
@@ -742,3 +779,77 @@ void cpl_imagelist_fill_shifted(cpl_imagelist * self, int napp,
     cpl_test_eq(cpl_imagelist_get_size(self), napp+1);
 
 }
+
+
+/*----------------------------------------------------------------------------*/
+/**
+  @internal
+  @brief      Test the CPL function
+  @param     kernel Kernel type
+  @return    void
+  @note On return the number of images in self will be n+1
+
+ */
+/*----------------------------------------------------------------------------*/
+static
+void cpl_geom_img_offset_saa_one(cpl_kernel kernel)
+{
+
+    const int        nz = 2 + NFRAMES;
+    cpl_imagelist * imglist = cpl_imagelist_new();
+    cpl_image    ** combined;
+    cpl_bivector  * offset  = cpl_bivector_new(nz);
+    cpl_vector    * off_x   = cpl_bivector_get_x(offset);
+    cpl_vector    * off_y   = cpl_bivector_get_y(offset);
+    cpl_error_code  error;
+    cpl_size        iz;
+    cpl_image     * central0;
+    cpl_image     * central1;
+
+    for (iz = 0; iz < nz; iz++) {
+        cpl_image * img = cpl_image_new(IMAGESZ, IMAGESZ, CPL_TYPE_FLOAT);
+
+        cpl_test_nonnull(img);
+
+        /* Insert flat images with known sum of the non-rejected planes */
+        error = cpl_image_add_scalar(img, (double)(nz - iz - nz/5));
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+        error = cpl_imagelist_set(imglist, img, iz);
+        cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+        cpl_vector_set(off_x, iz, iz ? cpl_drand() : 0.0);
+        cpl_vector_set(off_y, iz, iz ? cpl_drand() : 0.0);
+    }
+
+    combined = cpl_geom_img_offset_saa(imglist, offset, kernel, nz/5, nz/4,
+                                       CPL_GEOM_INTERSECT, NULL, NULL);
+    cpl_test_error(CPL_ERROR_NONE);
+    cpl_test_nonnull(combined);
+
+    error = cpl_image_dump_structure(combined[1], stdout);
+    cpl_test_eq_error(error, CPL_ERROR_NONE);
+
+    cpl_test_eq(cpl_image_get_max(combined[0]),
+                (nz - nz/5 - nz/4 + 1) /2.0);
+    cpl_test_eq(cpl_image_get_max(combined[1]), nz - nz/5
+                - nz/4);
+
+    central0 = cpl_image_extract(combined[0], 3, 3, IMAGESZ - 2, IMAGESZ - 2);
+    central1 = cpl_image_extract(combined[1], 3, 3, IMAGESZ - 2, IMAGESZ - 2);
+
+    cpl_test_eq(cpl_image_get_min(central0),
+                (nz - nz/5 - nz/4 + 1) /2.0);
+    cpl_test_eq(cpl_image_get_min(central1), nz - nz/5 - nz/4);
+
+    cpl_image_delete(combined[0]);
+    cpl_image_delete(combined[1]);
+    cpl_free(combined);
+
+    cpl_image_delete(central0);
+    cpl_image_delete(central1);
+
+    cpl_imagelist_delete(imglist);
+    cpl_bivector_delete(offset);
+
+}
