@@ -1,4 +1,4 @@
-/* $Id: cpl_photom-test.c,v 1.24 2010/11/11 09:23:18 llundin Exp $
+/* $Id: cpl_photom-test.c,v 1.26 2011/07/29 11:27:26 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/11/11 09:23:18 $
- * $Revision: 1.24 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2011/07/29 11:27:26 $
+ * $Revision: 1.26 $
+ * $Name: cpl-6_0 $
  */
 
 /*-----------------------------------------------------------------------------
@@ -81,8 +81,8 @@ int main(void)
     cpl_bivector   * trans;
     FILE           * ftrans;
     const double     temp = TEMP_BB;
-    int              i;
-    const int        half_search = FUNCTION_SIZE/2;
+    cpl_size         i;
+    const cpl_size   half_search = FUNCTION_SIZE/2;
     cpl_vector     * vxc;
     cpl_error_code   error;
     double         * wlf;
@@ -369,7 +369,7 @@ static void cpl_photom_fill_blackbody_test(cpl_vector * spectrum,
                                            double temp, unsigned line)
 {
 
-    const int fsize = cpl_vector_get_size(spectrum);
+    const cpl_size       fsize = cpl_vector_get_size(spectrum);
     const cpl_error_code error = cpl_photom_fill_blackbody(spectrum, out_unit,
                                                            evalpoints, in_unit,
                                                            temp);
@@ -382,7 +382,7 @@ static void cpl_photom_fill_blackbody_test(cpl_vector * spectrum,
 
         const double * sp = cpl_vector_get_data_const(spectrum);
         const double * wl = cpl_vector_get_data_const(evalpoints);
-        int i;
+        cpl_size       i;
 
         if (in_unit == CPL_UNIT_LENGTH) {
             const double xtpt = CPL_PHYS_Wien/temp; /* wavelength */
