@@ -1,4 +1,4 @@
-/* $Id: cpl_ppm-test.c,v 1.12 2010/11/12 08:54:46 llundin Exp $
+/* $Id: cpl_ppm-test.c,v 1.13 2011/08/02 12:01:08 rpalsa Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -19,10 +19,10 @@
  */
 
 /*
- * $Author: llundin $
- * $Date: 2010/11/12 08:54:46 $
- * $Revision: 1.12 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Author: rpalsa $
+ * $Date: 2011/08/02 12:01:08 $
+ * $Revision: 1.13 $
+ * $Name: cpl-6_0 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -199,7 +199,8 @@ int main(void)
     cpl_test_eq(cpl_matrix_get_ncol(mpattern), 3);
     cpl_test_eq(cpl_array_get_size(matches), 3);
 
-    cpl_msg_info(cpl_func, "%d points matched", cpl_matrix_get_ncol(mpattern));
+    cpl_msg_info(cpl_func, "%" CPL_SIZE_FORMAT " points matched",
+                 cpl_matrix_get_ncol(mpattern));
 
     cpl_array_delete(matches);
     cpl_matrix_delete(mpattern);
@@ -216,7 +217,8 @@ int main(void)
     cpl_test_eq(cpl_matrix_get_ncol(mpattern), 3);
     cpl_test_eq(cpl_array_get_size(matches), 3);
 
-    cpl_msg_info(cpl_func, "%d points matched", cpl_matrix_get_ncol(mpattern));
+    cpl_msg_info(cpl_func, "%" CPL_SIZE_FORMAT " points matched",
+                 cpl_matrix_get_ncol(mpattern));
 
     cpl_array_delete(matches);
     cpl_matrix_delete(mpattern);
@@ -242,7 +244,8 @@ int main(void)
     cpl_test_eq(cpl_matrix_get_ncol(mpattern), 3);
     cpl_test_eq(cpl_array_get_size(matches), 3);
 
-    cpl_msg_info(cpl_func, "%d points matched", cpl_matrix_get_ncol(mpattern));
+    cpl_msg_info(cpl_func, "%" CPL_SIZE_FORMAT " points matched",
+                 cpl_matrix_get_ncol(mpattern));
 
     cpl_array_delete(matches);
     cpl_matrix_delete(mpattern);
@@ -259,7 +262,8 @@ int main(void)
     cpl_test_eq(cpl_matrix_get_ncol(mpattern), 3);
     cpl_test_eq(cpl_array_get_size(matches), 3);
 
-    cpl_msg_info(cpl_func, "%d points matched", cpl_matrix_get_ncol(mpattern));
+    cpl_msg_info(cpl_func, "%" CPL_SIZE_FORMAT " points matched",
+                 cpl_matrix_get_ncol(mpattern));
 
     cpl_array_delete(matches);
     cpl_matrix_delete(mpattern);
@@ -318,7 +322,8 @@ int main(void)
     cpl_test_eq(cpl_matrix_get_ncol(mpattern), 8);
     cpl_test_eq(cpl_array_get_size(matches), 8);
 
-    cpl_msg_info(cpl_func, "%d points matched", cpl_matrix_get_ncol(mpattern));
+    cpl_msg_info(cpl_func, "%" CPL_SIZE_FORMAT " points matched",
+                 cpl_matrix_get_ncol(mpattern));
 
     cpl_array_delete(matches);
     cpl_matrix_delete(mpattern);
@@ -337,7 +342,8 @@ int main(void)
     cpl_test_eq(cpl_array_get_size(matches), 8);
     cpl_test_eq(cpl_array_count_invalid(matches), 1);
 
-    cpl_msg_info(cpl_func, "%d points matched", cpl_matrix_get_ncol(mpattern));
+    cpl_msg_info(cpl_func, "%" CPL_SIZE_FORMAT " points matched",
+                 cpl_matrix_get_ncol(mpattern));
 
     cpl_array_delete(matches);
     cpl_matrix_delete(mpattern);
@@ -365,7 +371,8 @@ int main(void)
     cpl_test_eq(cpl_matrix_get_ncol(mpattern), 8);
     cpl_test_eq(cpl_array_get_size(matches), 8);
 
-    cpl_msg_info(cpl_func, "%d points matched", cpl_matrix_get_ncol(mpattern));
+    cpl_msg_info(cpl_func, "%" CPL_SIZE_FORMAT " points matched",
+                 cpl_matrix_get_ncol(mpattern));
 
     cpl_matrix_delete(rdata);
     cpl_array_delete(matches);
@@ -415,7 +422,8 @@ int main(void)
     cpl_test_eq(cpl_matrix_get_ncol(mpattern), 70);
     cpl_test_eq(cpl_array_get_size(matches), 70);
 
-    cpl_msg_info(cpl_func, "%d points matched", cpl_matrix_get_ncol(mpattern));
+    cpl_msg_info(cpl_func, "%" CPL_SIZE_FORMAT " points matched",
+                 cpl_matrix_get_ncol(mpattern));
 
     cpl_matrix_delete(rdata);
     cpl_array_delete(matches);
@@ -469,7 +477,8 @@ int main(void)
     cpl_test_eq(cpl_matrix_get_ncol(mpattern), npattern);
     cpl_test_eq(cpl_array_get_size(matches), npattern);
 
-    cpl_msg_info(cpl_func, "%d points matched", cpl_matrix_get_ncol(mpattern));
+    cpl_msg_info(cpl_func, "%" CPL_SIZE_FORMAT " points matched",
+                 cpl_matrix_get_ncol(mpattern));
 
     cpl_matrix_delete(rdata);
     cpl_array_delete(matches);
@@ -528,7 +537,8 @@ int main(void)
     cpl_test_eq(cpl_matrix_get_ncol(mpattern), ndata);
     cpl_test_eq(cpl_array_get_size(matches), npattern);
 
-    cpl_msg_info(cpl_func, "%d points matched", cpl_matrix_get_ncol(mpattern));
+    cpl_msg_info(cpl_func, "%" CPL_SIZE_FORMAT " points matched",
+                 cpl_matrix_get_ncol(mpattern));
 
     cpl_matrix_delete(rdata);
     cpl_array_delete(matches);
@@ -569,7 +579,8 @@ int main(void)
     cpl_test_eq(cpl_matrix_get_ncol(mpattern), npattern);
     cpl_test_eq(cpl_array_get_size(matches), npattern);
 
-    cpl_msg_info(cpl_func, "%d points matched", cpl_matrix_get_ncol(mpattern));
+    cpl_msg_info(cpl_func, "%" CPL_SIZE_FORMAT " points matched",
+                 cpl_matrix_get_ncol(mpattern));
 
     cpl_matrix_delete(mpattern);
     cpl_matrix_delete(mdata);
@@ -583,7 +594,8 @@ int main(void)
     cpl_test_eq(cpl_matrix_get_ncol(mpattern), ndata);
     cpl_test_eq(cpl_array_get_size(matches), ndata);
 
-    cpl_msg_info(cpl_func, "%d points matched", cpl_matrix_get_ncol(mpattern));
+    cpl_msg_info(cpl_func, "%" CPL_SIZE_FORMAT " points matched",
+                 cpl_matrix_get_ncol(mpattern));
 
     cpl_matrix_delete(mpattern);
     cpl_matrix_delete(mdata);
