@@ -1,4 +1,4 @@
-/* $Id: cpl_wcs-test.c,v 1.32 2011/02/15 10:48:37 llundin Exp $
+/* $Id: cpl_wcs-test.c,v 1.33 2011/04/12 06:20:52 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2011/02/15 10:48:37 $
- * $Revision: 1.32 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2011/04/12 06:20:52 $
+ * $Revision: 1.33 $
+ * $Name: cpl-6_0 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -293,7 +293,7 @@ int main (void) {
     double crval1, crval2, crpix1, crpix2, cd1, cd2, cd3, cd4;
     cpl_propertylist *pl,*opl;
     cpl_wcs *wcs;
-    const cpl_matrix *from,*xymat,*rdmat;
+    cpl_matrix      *from,*xymat,*rdmat;
     cpl_matrix      * to     = NULL;
     cpl_array       * status = NULL;
     const cpl_array * nullarray;
@@ -448,7 +448,7 @@ int main (void) {
     cpl_test_eq_error(error, CPL_ERROR_NONE);
     cpl_test_nonnull(to);
     cpl_test_nonnull(status);
-    (void)cpl_matrix_unwrap((cpl_matrix*)from);
+    (void)cpl_matrix_unwrap(from);
 
     /* Test the output values. The status should all be 0. The output matrix
        is compared to predefined world coordinate values */
@@ -492,7 +492,7 @@ int main (void) {
     cpl_test_eq_error(error, CPL_ERROR_NONE);
     cpl_test_nonnull(to);
     cpl_test_nonnull(status);
-    (void)cpl_matrix_unwrap((cpl_matrix*)from);
+    (void)cpl_matrix_unwrap(from);
 
     /* Test the output values again */
 
@@ -521,7 +521,7 @@ int main (void) {
     cpl_test_eq_error(error, CPL_ERROR_NONE);
     cpl_test_nonnull(to);
     cpl_test_nonnull(status);
-    (void)cpl_matrix_unwrap((cpl_matrix*)from);
+    (void)cpl_matrix_unwrap(from);
 
     /* Test the output values again */
 
@@ -546,11 +546,11 @@ int main (void) {
     cpl_msg_info("","Transform world -> standard");
     /* worldout is _not_ modified */
     from = cpl_matrix_wrap(1, 2, (double*)worldout);
-    error = cpl_wcs_convert(wcs,from,&to,&status,CPL_WCS_WORLD2STD);
+    error = cpl_wcs_convert(wcs, from, &to, &status, CPL_WCS_WORLD2STD);
     cpl_test_eq_error(error, CPL_ERROR_NONE);
     cpl_test_nonnull(to);
     cpl_test_nonnull(status);
-    (void)cpl_matrix_unwrap((cpl_matrix*)from);
+    (void)cpl_matrix_unwrap(from);
 
     /* Test the output values again */
 
@@ -699,8 +699,8 @@ int main (void) {
     /* Tidy a bit */
 
     cpl_propertylist_delete(pl);
-    (void)cpl_matrix_unwrap((cpl_matrix*)xymat);
-    (void)cpl_matrix_unwrap((cpl_matrix*)rdmat);
+    (void)cpl_matrix_unwrap(xymat);
+    (void)cpl_matrix_unwrap(rdmat);
 
     /* Test accessor functions */
 
