@@ -1,4 +1,4 @@
-/* $Id: cpl_frame.c,v 1.25 2010/11/11 09:23:18 llundin Exp $
+/* $Id: cpl_frame.c,v 1.29 2012/01/11 13:38:29 rpalsa Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2005 European Southern Observatory
@@ -19,21 +19,16 @@
  */
 
 /*
- * $Author: llundin $
- * $Date: 2010/11/11 09:23:18 $
- * $Revision: 1.25 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Author: rpalsa $
+ * $Date: 2012/01/11 13:38:29 $
+ * $Revision: 1.29 $
+ * $Name: cpl-6_0 $
  */
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
 #endif
 
-#include <stdio.h>
-#include <string.h>
-
-#include <fitsio.h>
-
 #include <cxmemory.h>
 #include <cxmessages.h>
 #include <cxstrutils.h>
@@ -42,6 +37,13 @@
 #include "cpl_frame.h"
 #include "cpl_fits.h"
 #include "cpl_frame_impl.h"
+#include "cpl_io_fits.h"
+#include "cpl_tools.h"
+
+#include <stdio.h>
+#include <string.h>
+
+#include <fitsio.h>
 
 
 /**
@@ -134,14 +136,13 @@ _cpl_frame_fileinfo_delete(cpl_frame_fileinfo *info)
  * is found, and -1 if an error occurred.
  */
 
-int
-cpl_frame_get_nplanes(const cpl_frame *self, int extnum)
+cpl_size
+cpl_frame_get_nplanes(const cpl_frame *self, cpl_size extnum)
 {
     const cxchar *const _id = "cpl_frame_get_nplanes";
-    cxint            next ;
-    cxint            nplanes;
-    cxint            naxis ;
-    cxlong          *naxes ;
+    cpl_size         next;
+    cpl_size         nplanes;
+    cxint            naxis;
     cxint status = 0;
     cxint hdu_type = ANY_HDU;
     cxint hdu_position = extnum + 1;
@@ -158,18 +159,18 @@ cpl_frame_get_nplanes(const cpl_frame *self, int extnum)
     }
     cx_assert(self->fileinfo->name != NULL);
 
-    next = cpl_frame_get_nextensions(self) ;
+    next = cpl_frame_get_nextensions(self);
     if (extnum > next) {
         cpl_error_set(_id, CPL_ERROR_ILLEGAL_INPUT);
-        return -1 ;
+        return -1;
     }
 
 
-    fits_open_diskfile(&file, self->fileinfo->name, READONLY, &status);
+    cpl_io_fits_open_diskfile(&file, self->fileinfo->name, READONLY, &status);
 
     if (status == FILE_NOT_OPENED) {
         status = 0;
-        fits_close_file(file, &status);
+        cpl_io_fits_close_file(file, &status);
         cpl_error_set(_id, CPL_ERROR_FILE_IO);
         return -1;
 
@@ -178,7 +179,7 @@ cpl_frame_get_nplanes(const cpl_frame *self, int extnum)
 
         if (status != 0) {
             status = 0;
-            fits_close_file(file, &status);
+            cpl_io_fits_close_file(file, &status);
             cpl_error_set(_id, CPL_ERROR_BAD_FILE_FORMAT);
             return -1;
 
@@ -189,7 +190,7 @@ cpl_frame_get_nplanes(const cpl_frame *self, int extnum)
 
     if ((status == BAD_HDU_NUM) || (status == END_OF_FILE)) {
         status = 0;
-        fits_close_file(file, &status);
+        cpl_io_fits_close_file(file, &status);
         cpl_error_set(_id, CPL_ERROR_DATA_NOT_FOUND);
 
         return -1;
@@ -197,34 +198,34 @@ cpl_frame_get_nplanes(const cpl_frame *self, int extnum)
     }
 
     /* Find the number of axes  */
-    naxis = 0 ;
+    naxis = 0;
     fits_get_img_dim(file, &naxis, &status);
 
     /* Check validity of naxis */
     if ((naxis < 2) || (naxis > 3)) {
-        fits_close_file(file, &status) ;
-        cpl_error_set(_id, CPL_ERROR_ILLEGAL_INPUT) ;
-        return -1 ;
+        cpl_io_fits_close_file(file, &status);
+        cpl_error_set(_id, CPL_ERROR_ILLEGAL_INPUT);
+        return -1;
     }
 
     status = 0;
 
     /* Two dimensions cube */
     if (naxis == 2) {
-        nplanes = 1 ;
+        nplanes = 1;
     }
     else {
         /* For 3D cubes, get the third dimension size   */
-        naxes = cx_malloc(naxis * sizeof(long)) ;
-        fits_get_img_size(file, naxis, naxes, &status);
-        nplanes = (cxint)naxes[2];
+        CPL_FITSIO_TYPE *naxes = cx_malloc(naxis * sizeof(CPL_FITSIO_TYPE));
+        CPL_FITSIO_GET_SIZE(file, naxis, naxes, &status);
+        nplanes = (cpl_size)naxes[2];
         if(naxes[2] < 1)
             nplanes = 0;
 
         cx_free(naxes);
     }
 
-    fits_close_file(file, &status);
+    cpl_io_fits_close_file(file, &status);
     cx_assert(status == 0);
 
     return nplanes;
@@ -861,10 +862,10 @@ cpl_frame_set_level(cpl_frame *self, cpl_frame_level level)
  * case of error.
  */
 
-int
+cpl_size
 cpl_frame_get_nextensions(const cpl_frame *self)
 {
-    cxint         next ;
+    cpl_size next;
 
     if (self == NULL) {
         cpl_error_set(cpl_func, CPL_ERROR_NULL_INPUT);
