@@ -1,4 +1,4 @@
-/* $Id: cpl_frame.h,v 1.18 2010/11/11 10:31:52 llundin Exp $
+/* $Id: cpl_frame.h,v 1.19 2012/01/11 13:38:29 rpalsa Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2005 European Southern Observatory
@@ -19,16 +19,18 @@
  */
 
 /*
- * $Author: llundin $
- * $Date: 2010/11/11 10:31:52 $
- * $Revision: 1.18 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Author: rpalsa $
+ * $Date: 2012/01/11 13:38:29 $
+ * $Revision: 1.19 $
+ * $Name: cpl-6_0 $
  */
 
 #ifndef CPL_FRAME_H
 #define CPL_FRAME_H
 
 #include <stdio.h>
+
+#include <cpl_type.h>
 #include <cpl_error.h>
 
 
@@ -273,7 +275,7 @@ cpl_error_code cpl_frame_set_level(cpl_frame *self, cpl_frame_level level);
  * Others
  */
 
-int cpl_frame_get_nextensions(const cpl_frame *self);
+cpl_size cpl_frame_get_nextensions(const cpl_frame *self);
 
 void cpl_frame_dump(const cpl_frame *frame, FILE *stream);
 
