@@ -1,4 +1,4 @@
-/* $Id: cpl_frame_impl.h,v 1.2 2007/05/16 15:41:53 scastro Exp $
+/* $Id: cpl_frame_impl.h,v 1.3 2012/01/11 13:38:29 rpalsa Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2006 European Southern Observatory
@@ -19,22 +19,22 @@
  */
 
 /*
- * $Author: scastro $
- * $Date: 2007/05/16 15:41:53 $
- * $Revision: 1.2 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Author: rpalsa $
+ * $Date: 2012/01/11 13:38:29 $
+ * $Revision: 1.3 $
+ * $Name: cpl-6_0 $
  */
 
 #ifndef CPL_FRAME_IMPL_H
 #define CPL_FRAME_IMPL_H
 
-/* Add includes here */
+#include "cpl_type.h"
 #include "cpl_frame.h"
 
 CPL_BEGIN_DECLS
 
 /* Private functions */
-int cpl_frame_get_nplanes(const cpl_frame *self, int extnum);
+cpl_size cpl_frame_get_nplanes(const cpl_frame *self, cpl_size extnum);
 
 CPL_END_DECLS
 
