@@ -1,4 +1,4 @@
-/* $Id: cpl_frameset.c,v 1.35 2010/11/11 09:23:18 llundin Exp $
+/* $Id: cpl_frameset.c,v 1.37 2012/01/11 13:38:29 rpalsa Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2005 European Southern Observatory
@@ -19,10 +19,10 @@
  */
 
 /*
- * $Author: llundin $
- * $Date: 2010/11/11 09:23:18 $
- * $Revision: 1.35 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Author: rpalsa $
+ * $Date: 2012/01/11 13:38:29 $
+ * $Revision: 1.37 $
+ * $Name: cpl-6_0 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -42,6 +42,7 @@
 #include <cpl_msg.h>
 #include "cpl_frameset.h"
 #include "cpl_errorstate.h"
+#include "cpl_error_impl.h"
 
 
 /**
@@ -420,7 +421,7 @@ cpl_frameset_delete(cpl_frameset *self)
  * @em self.
  */
 
-int
+cpl_size
 cpl_frameset_get_size(const cpl_frameset *self)
 {
 
@@ -1014,7 +1015,7 @@ cpl_frameset_insert(cpl_frameset *self, cpl_frame *frame)
  * has no effect.
  */
 
-int
+cpl_size
 cpl_frameset_erase(cpl_frameset *self, const char *tag)
 {
 
@@ -1229,19 +1230,19 @@ cpl_frameset_erase_frame(cpl_frameset *self, cpl_frame *frame)
  * The returned array must be deallocated with cpl_free().
  */
 
-int *
+cpl_size *
 cpl_frameset_labelise(const cpl_frameset *self,
                       int (*compare)(const cpl_frame*, const cpl_frame*),
-                      int *nb_labels)
+                      cpl_size *nb_labels)
 {
 
-    cxint *labels;
-    cxint *labelsinv;
-    cxint nframes;
-    cxint i;
-    cxint nlabels = 0;
-    cxint j = 0;
-    cxint ncomp = 0;
+    cpl_size *labels;
+    cpl_size *labelsinv;
+    cpl_size nframes;
+    cpl_size i;
+    cpl_size nlabels = 0;
+    cpl_size j = 0;
+    cpl_size ncomp = 0;
     const cpl_frame ** framelist;
     const cpl_frame * frame1;
 
@@ -1254,14 +1255,14 @@ cpl_frameset_labelise(const cpl_frameset *self,
 
     cpl_ensure(nframes >= 1, CPL_ERROR_ILLEGAL_INPUT, NULL);
 
-    labels    = cx_malloc(nframes * sizeof(cxint));
-    labelsinv = cx_malloc(nframes * sizeof(cxint));
+    labels    = cx_malloc(nframes * sizeof(cpl_size));
+    labelsinv = cx_malloc(nframes * sizeof(cpl_size));
     framelist = cx_malloc(nframes * sizeof(const cpl_frame *));
 
     for (frame1 = cpl_frameset_get_first_const(self), i = 0; frame1 != NULL;
          frame1 = cpl_frameset_get_next_const(self), i++) {
 
-        cxint jj;
+        cpl_size jj;
 
         for (jj = 0; jj < nlabels; jj++, j = j == nlabels-1 ? 0 : j+1) {
             const cpl_frame * frame2 = framelist[j];
@@ -1288,7 +1289,8 @@ cpl_frameset_labelise(const cpl_frameset *self,
                 cx_free(labelsinv);
 
                 /* Propagate error */
-                cpl_ensure(0, cpl_error_get_code(), NULL);
+                (void)cpl_error_set_where_();
+                return NULL;
             }
         }
 
@@ -1309,7 +1311,8 @@ cpl_frameset_labelise(const cpl_frameset *self,
     cx_free(framelist);
     cx_free(labelsinv);
 
-    cpl_msg_debug(cpl_func, "%d frames labelized with %d labels after %d "
+    cpl_msg_debug(cpl_func, "%" CPL_SIZE_FORMAT " frames labelized with %"
+                  CPL_SIZE_FORMAT " labels after %" CPL_SIZE_FORMAT
                   "comparisons", nframes, nlabels, ncomp);
 
     return labels;
@@ -1357,13 +1360,13 @@ cpl_frameset_labelise(const cpl_frameset *self,
  */
 
 const cpl_frame *
-cpl_frameset_get_frame_const(const cpl_frameset *set, int position)
+cpl_frameset_get_frame_const(const cpl_frameset *set, cpl_size position)
 {
 
     const cxchar *const _id = "cpl_frameset_get_frame_const";
 
-    cxint nbframes;
-    cxint i;
+    cpl_size nbframes;
+    cpl_size i;
 
     const cpl_frame *frame;
 
@@ -1438,7 +1441,7 @@ cpl_frameset_get_frame_const(const cpl_frameset *set, int position)
  */
 
 cpl_frame *
-cpl_frameset_get_frame(cpl_frameset *set, int position)
+cpl_frameset_get_frame(cpl_frameset *set, cpl_size position)
 {
 
     cpl_errorstate prestate = cpl_errorstate_get();
@@ -1479,13 +1482,13 @@ cpl_frameset_get_frame(cpl_frameset *set, int position)
  */
 
 cpl_frameset *
-cpl_frameset_extract(const cpl_frameset *self, const int *labels,
-                     int desired_label)
+cpl_frameset_extract(const cpl_frameset *self, const cpl_size *labels,
+                     cpl_size desired_label)
 {
 
     cpl_frameset    *selected = NULL;
     const cpl_frame *frame;
-    cxint i;
+    cpl_size i;
 
 
     cpl_ensure(self   != NULL, CPL_ERROR_NULL_INPUT, NULL);
