@@ -1,4 +1,4 @@
-/* $Id: cpl_frameset_io.h,v 1.5 2010/11/11 10:31:52 llundin Exp $
+/* $Id: cpl_frameset_io.h,v 1.6 2011/07/21 07:36:33 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2008 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2010/11/11 10:31:52 $
- * $Revision: 1.5 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2011/07/21 07:36:33 $
+ * $Revision: 1.6 $
+ * $Name: cpl-6_0 $
  */
 
 #ifndef CPL_FRAMESET_IO_H
@@ -35,9 +35,9 @@
 
 CPL_BEGIN_DECLS
 
-/* Convert a frame set to an image list */
+/* Load an image list from a frame set */
 cpl_imagelist * cpl_imagelist_load_frameset(const cpl_frameset *, cpl_type, 
-                                            int, int) CPL_ATTR_ALLOC;
+                                            cpl_size, cpl_size) CPL_ATTR_ALLOC;
 
 CPL_END_DECLS
 
