@@ -1,4 +1,4 @@
-/* $Id: cpl_parameterlist.c,v 1.14 2010/11/11 09:23:18 llundin Exp $
+/* $Id: cpl_parameterlist.c,v 1.15 2012/01/11 13:38:29 rpalsa Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2005 European Southern Observatory
@@ -19,10 +19,10 @@
  */
 
 /*
- * $Author: llundin $
- * $Date: 2010/11/11 09:23:18 $
- * $Revision: 1.14 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Author: rpalsa $
+ * $Date: 2012/01/11 13:38:29 $
+ * $Revision: 1.15 $
+ * $Name: cpl-6_0 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -152,7 +152,7 @@ cpl_parameterlist_delete(cpl_parameterlist *self)
  * parameter list @em self.
  */
 
-int
+cpl_size
 cpl_parameterlist_get_size(const cpl_parameterlist *self)
 {
 
@@ -164,7 +164,7 @@ cpl_parameterlist_get_size(const cpl_parameterlist *self)
         return 0;
     }
 
-    return (int)cx_slist_size(self->data);
+    return (cpl_size)cx_slist_size(self->data);
 
 }
 
