@@ -1,4 +1,4 @@
-/* $Id: cpl_recipeconfig.c,v 1.5 2010/11/11 09:23:18 llundin Exp $
+/* $Id: cpl_recipeconfig.c,v 1.6 2012/01/11 13:38:29 rpalsa Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2006 European Southern Observatory
@@ -19,10 +19,10 @@
  */
 
 /*
- * $Author: llundin $
- * $Date: 2010/11/11 09:23:18 $
- * $Revision: 1.5 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Author: rpalsa $
+ * $Date: 2012/01/11 13:38:29 $
+ * $Revision: 1.6 $
+ * $Name: cpl-6_0 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -557,7 +557,7 @@ cpl_recipeconfig_set_tags(cpl_recipeconfig* self, const cpl_framedata* data)
 
 int
 cpl_recipeconfig_set_tag(cpl_recipeconfig* self, const char* tag,
-                         int min_count, int max_count)
+                         cpl_size min_count, cpl_size max_count)
 {
 
     static const cxchar* const _id = "cpl_recipeconfig_set_tag";
@@ -878,8 +878,9 @@ cpl_recipeconfig_set_inputs(cpl_recipeconfig* self, const char* tag,
  */
 
 int
-cpl_recipeconfig_set_input(cpl_recipeconfig* self, const char* tag,
-                           const char* input, int min_count, int max_count)
+cpl_recipeconfig_set_input(cpl_recipeconfig* self,
+                           const char* tag, const char* input,
+                           cpl_size min_count, cpl_size max_count)
 {
 
     static const cxchar* const _id = "cpl_recipeconfig_set_input";
@@ -1308,7 +1309,7 @@ cpl_recipeconfig_set_output(cpl_recipeconfig* self, const char* tag,
  * the frame with tag @em tag are returned.
  */
 
-int
+cpl_size
 cpl_recipeconfig_get_min_count(const cpl_recipeconfig* self,
                                const char* tag, const char* input)
 {
@@ -1404,7 +1405,7 @@ cpl_recipeconfig_get_min_count(const cpl_recipeconfig* self,
  * the frame with tag @em tag are returned.
  */
 
-int
+cpl_size
 cpl_recipeconfig_get_max_count(const cpl_recipeconfig* self,
                                const char* tag, const char* input)
 {
