@@ -1,4 +1,4 @@
-/* $Id: cpl_recipeconfig.h,v 1.4 2010/11/11 10:31:52 llundin Exp $
+/* $Id: cpl_recipeconfig.h,v 1.5 2012/01/11 13:38:29 rpalsa Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2006 European Southern Observatory
@@ -19,16 +19,17 @@
  */
 
 /*
- * $Author: llundin $
- * $Date: 2010/11/11 10:31:52 $
- * $Revision: 1.4 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Author: rpalsa $
+ * $Date: 2012/01/11 13:38:29 $
+ * $Revision: 1.5 $
+ * $Name: cpl-6_0 $
  */
 
 #ifndef CPL_RECIPECONFIG_H
 #define CPL_RECIPECONFIG_H
 
 #include <cpl_macros.h>
+#include <cpl_type.h>
 #include <cpl_framedata.h>
 
 
@@ -62,10 +63,10 @@ char** cpl_recipeconfig_get_inputs(const cpl_recipeconfig* self,
 char** cpl_recipeconfig_get_outputs(const cpl_recipeconfig* self,
                                     const char* tag);
 
-int cpl_recipeconfig_get_min_count(const cpl_recipeconfig* self,
-                                   const char* tag, const char* input);
-int cpl_recipeconfig_get_max_count(const cpl_recipeconfig* self,
-                                   const char* tag, const char* input);
+cpl_size cpl_recipeconfig_get_min_count(const cpl_recipeconfig* self,
+                                        const char* tag, const char* input);
+cpl_size cpl_recipeconfig_get_max_count(const cpl_recipeconfig* self,
+                                        const char* tag, const char* input);
 int cpl_recipeconfig_is_required(const cpl_recipeconfig* self,
                                  const char* tag, const char* input);
 
@@ -74,10 +75,10 @@ int cpl_recipeconfig_is_required(const cpl_recipeconfig* self,
  */
 
 int cpl_recipeconfig_set_tag(cpl_recipeconfig* self, const char* tag,
-                             int min_count, int max_count);
+                             cpl_size min_count, cpl_size max_count);
 int cpl_recipeconfig_set_input(cpl_recipeconfig* self, const char* tag,
-                               const char* input, int min_count,
-                               int max_count);
+                               const char* input, cpl_size min_count,
+                               cpl_size max_count);
 int cpl_recipeconfig_set_output(cpl_recipeconfig* self, const char* tag,
                                 const char* output);
 
