@@ -1,4 +1,4 @@
-/* $Id: cpl_recipedefine.h,v 1.13 2011/01/03 15:39:21 llundin Exp $
+/* $Id: cpl_recipedefine.h,v 1.14 2011/08/19 19:57:07 llundin Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2006 European Southern Observatory
@@ -20,9 +20,9 @@
 
 /*
  * $Author: llundin $
- * $Date: 2011/01/03 15:39:21 $
- * $Revision: 1.13 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2011/08/19 19:57:07 $
+ * $Revision: 1.14 $
+ * $Name: cpl-6_0 $
  */
 
 #ifndef CPL_RECIPEDEFINE_H
@@ -105,9 +105,9 @@ CPL_BEGIN_DECLS
      static cpl_error_code <recipe>_fill_parameterlist(cpl_parameterlist *self);
 
    @endcode
-   A recipe that invokes cpl_recipedefine() must define this function.
+   A recipe that invokes cpl_recipe_define() must define this function.
 
-   The macro cpl_recipedefine() may be used by defining a macro, e.g. in
+   The macro cpl_recipe_define() may be used by defining a macro, e.g. in
    my_recipe.h:
 
    @code
