@@ -1,4 +1,4 @@
-/* $Id: cpl_frame-test.c,v 1.18 2010/11/11 09:23:18 llundin Exp $
+/* $Id: cpl_frame-test.c,v 1.19 2012/01/11 13:38:29 rpalsa Exp $
  *
  * This file is part of the ESO Common Pipeline Library
  * Copyright (C) 2001-2005 European Southern Observatory
@@ -19,10 +19,10 @@
  */
 
 /*
- * $Author: llundin $
- * $Date: 2010/11/11 09:23:18 $
- * $Revision: 1.18 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Author: rpalsa $
+ * $Date: 2012/01/11 13:38:29 $
+ * $Revision: 1.19 $
+ * $Name: cpl-6_0 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -49,7 +49,7 @@ int main(void)
 
     int i;
     int next;
-    int np;
+    cpl_size np;
     int status = 0;
     const char *filename  = "cplframetest1.fits";
     const char *filename1 = "cplframeplane.fits";
