@@ -1,7 +1,7 @@
-/* $Id: cxlist.c,v 1.6 2009/12/08 15:55:37 llundin Exp $
+/* $Id: cxlist.c,v 1.8 2011/12/22 14:33:32 rpalsa Exp $
  *
  * This file is part of the ESO C Extension Library
- * Copyright (C) 2001-2006 European Southern Observatory
+ * Copyright (C) 2001-2011 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,10 +19,10 @@
  */
 
 /*
- * $Author: llundin $
- * $Date: 2009/12/08 15:55:37 $
- * $Revision: 1.6 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Author: rpalsa $
+ * $Date: 2011/12/22 14:33:32 $
+ * $Revision: 1.8 $
+ * $Name: cpl-6_0 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -346,9 +346,9 @@ _cx_list_sorted(cx_list *list, cx_compare_func compare)
 {
 
     cx_lnode *node = list->head.next;
-    /* Avoid gcc uninit-warning */
     cx_lnode *next = node != NULL ? node->next : NULL;
 
+
     while (next != &list->head) {
         if (compare(_cx_lnode_get(node), _cx_lnode_get(next)) > 0)
             return 0;
@@ -383,7 +383,7 @@ _cx_list_next(const cx_list *list, cx_list_const_iterator position)
 
 
 /*
- * Get the previous element of a list, i.e. the element directly preceeding
+ * Get the previous element of a list, i.e. the element directly preceding
  * the current position in the list.
  *
  * Note that there are no checks whether the current list position actually
@@ -904,10 +904,13 @@ cx_list_assign(cx_list *list, cx_list_iterator position, cxcptr data)
  *
  * @param list  The list to query.
  *
- * @return Handle to the data object stored as the first list element.
+ * @return Handle to the data object stored in the first list element.
  *
  * The function returns a reference to the first data item in the list
  * @em list.
+ *
+ * Calling this function with an empty list is an invalid operation, and
+ * the result is undefined.
  */
 
 cxptr
@@ -928,10 +931,14 @@ cx_list_front(const cx_list *list)
  *
  * @param list  The list to query.
  *
- * @return Handle to the data object stored as the last list element.
+ * @return Handle to the data object stored in the last list element.
  *
  * The function returns a reference to the last data item in the list
  * @em list.
+ *
+ * Calling this function with an empty list is an invalid operation, and
+ * the result is undefined.
+ *
  */
 
 cxptr
@@ -1136,7 +1143,7 @@ cx_list_extract(cx_list *list, cx_list_iterator position)
  *
  * @param list  The list to update.
  *
- * @return Handle to the data object previously stored as the last
+ * @return Handle to the data object previously stored as the first
  *    list element.
  *
  * The function removes the first element from the list @em list returning
@@ -1146,12 +1153,17 @@ cx_list_extract(cx_list *list, cx_list_iterator position)
  * @code
  *   cx_list_extract(list, cx_list_begin(list));
  * @endcode
+ *
+ * Calling this function with an empty list is an invalid operation, and
+ * the result is undefined.
  */
 
 cxptr
 cx_list_pop_front(cx_list *list)
 {
 
+    cx_assert(!cx_list_empty(list));
+
     return _cx_lnode_delete(_cx_list_extract(cx_list_begin(list)));
 
 }
@@ -1173,13 +1185,21 @@ cx_list_pop_front(cx_list *list)
  * @code
  *   cx_list_extract(list, cx_list_previous(list, cx_list_end(list)));
  * @endcode
+ *
+ * Calling this function with an empty list is an invalid operation, and
+ * the result is undefined.
  */
 
 cxptr
 cx_list_pop_back(cx_list *list)
 {
 
-    cx_lnode *position = _cx_list_previous(list, cx_list_end(list));
+    cx_lnode *position = NULL;
+
+
+    cx_assert(!cx_list_empty(list));
+
+    position = _cx_list_previous(list, cx_list_end(list));
 
     return _cx_lnode_delete(_cx_list_extract(position));
 
