@@ -1,7 +1,7 @@
-/* $Id: cxmacros.h,v 1.6 2007/12/12 12:09:41 scastro Exp $
+/* $Id: cxmacros.h,v 1.7 2011/02/21 14:15:31 rpalsa Exp $
  *
  * This file is part of the ESO C Extension Library
- * Copyright (C) 2001-2006 European Southern Observatory
+ * Copyright (C) 2001-2011 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,10 +19,10 @@
  */
 
 /*
- * $Author: scastro $
- * $Date: 2007/12/12 12:09:41 $
- * $Revision: 1.6 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Author: rpalsa $
+ * $Date: 2011/02/21 14:15:31 $
+ * $Revision: 1.7 $
+ * $Name: cpl-6_0 $
  */
 
 
@@ -52,6 +52,37 @@
 #endif
 
 
+/*
+ * Macros for the GNU compiler function attributes
+ */
+
+#if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 96)
+#  define CX_GNUC_PURE   __attribute__((__pure__))
+#  define CX_GNUC_MALLOC __attribute__((__malloc__))
+#else
+#  define G_GNUC_PURE
+#  define G_GNUC_MALLOC
+#endif
+
+#if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ > 4)
+#  define CX_GNUC_PRINTF(fmt_idx, arg_idx) \
+          __attribute__((__format__ (__printf__, fmt_idx, arg_idx)))
+#  define CX_GNUC_SCANF(fmt_idx, arg_idx)  \
+          __attribute__((__format__ (__scanf__, fmt_idx, arg_idx)))
+#  define CX_GNUC_FORMAT(arg_idx) __attribute__((__format_arg__ (arg_idx)))
+#  define CX_GNUC_NORETURN        __attribute__((__noreturn__))
+#  define CX_GNUC_CONST           __attribute__((__const__))
+#  define CX_GNUC_UNUSED          __attribute__((__unused__))
+#else
+#  define CX_GNUC_PRINTF(fmt_idx, arg_idx)
+#  define CX_GNUC_SCANF(fmt_idx, arg_idx)
+#  define CX_GNUC_FORMAT(arg_idx)
+#  define CX_GNUC_NORETURN
+#  define CX_GNUC_CONST
+#  define CX_GNUC_UNUSED
+#endif
+
+
 /*
  * Wrap the gcc __PRETTY_FUNCTION__ and __FUNCTION__ variables with macros.
  */
@@ -79,6 +110,18 @@
 #endif
 
 
+/*
+ * Current function identifier
+ */
+#if defined (__GNUC__)
+#  define CX_FUNC_NAME  ((const char*) (__PRETTY_FUNCTION__))
+#elif defined (__STDC_VERSION__) && __STDC_VERSION__ >= 19901L
+#  define CX_FUNC_NAME  ((const char*) (__func__))
+#else
+#  define CX_FUNC_NAME  ((const char*) ("???"))
+#endif
+
+
 /*
  * C code guard
  */
@@ -119,26 +162,18 @@
 #ifndef CX_MIN
 # define CX_MIN(a, b)  ((a) < (b) ? (a) : (b))
 #endif
-/*#undef MIN
-#define MIN(a, b)  ((a) < (b) ? (a) : (b))*/
 
 #ifndef CX_MAX
 # define CX_MAX(a, b)  ((a) > (b) ? (a) : (b))
 #endif
-/*#undef MAX
-#define MAX(a, b)  ((a) > (b) ? (a) : (b))*/
 
 #ifndef CX_ABS
 # define CX_ABS(a)  ((a) < (0) ? -(a) : (a))
 #endif
-/*#undef ABS
-#define ABS(a)  ((a) < (0) ? -(a) : (a))*/
 
 #ifndef CX_CLAMP
 # define CX_CLAMP(a, low, high)  (((a) > (high)) ? (high) : (((a) < (low)) ? (low) : (a)))
 #endif
-/*#undef CLAMP
-#define CLAMP(a, low, high)  (((a) > (high)) ? (high) : (((a) < (low)) ? (low) : (a)))*/
 
 
 /*
