@@ -1,7 +1,7 @@
-/* $Id: cxmessages.c,v 1.7 2007/07/06 13:08:57 llundin Exp $
+/* $Id: cxmessages.c,v 1.9 2011/03/04 12:15:10 rpalsa Exp $
  *
  * This file is part of the ESO C Extension Library
- * Copyright (C) 2001-2006 European Southern Observatory
+ * Copyright (C) 2001-2011 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,10 +19,10 @@
  */
 
 /*
- * $Author: llundin $
- * $Date: 2007/07/06 13:08:57 $
- * $Revision: 1.7 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Author: rpalsa $
+ * $Date: 2011/03/04 12:15:10 $
+ * $Revision: 1.9 $
+ * $Name: cpl-6_0 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -35,19 +35,29 @@
 #include <string.h>
 
 #ifdef HAVE_UNISTD_H
-#include <unistd.h>
+#  include <unistd.h>
 #endif
 
 #ifdef HAVE_CHAR_BIT
-#include <limits.h>
+#  include <limits.h>
 #endif
 
+#include "cxthread.h"
 #include "cxmemory.h"
+#include "cxstring.h"
 #include "cxmessages.h"
 #include "cxstrutils.h"
 #include "cxutils.h"
 
 
+#define CX_MSG_FORMAT_UNSIGNED_BUFFER_SIZE  ((3 * SIZEOF_LONG) + 3)
+#define CX_MSG_PREFIX_BUFFER_SIZE           \
+    (CX_MSG_FORMAT_UNSIGNED_BUFFER_SIZE + 32)
+#define CX_MSG_ALERT_LEVELS                 \
+    (CX_LOG_LEVEL_ERROR | CX_LOG_LEVEL_CRITICAL | CX_LOG_LEVEL_WARNING)
+
+
+
 /**
  * @defgroup cxmessages Message Logging
  *
@@ -90,6 +100,14 @@ struct _cx_log_domain_ {
 };
 
 
+/*
+ * Messaging system mutexes
+ */
+
+CX_LOCK_DEFINE_STATIC(cx_messages_lock);
+CX_ONCE_DEFINE_STATIC(cx_messages_once);
+
+
 /*
  * Built in system defaults
  */
@@ -100,7 +118,7 @@ static cx_log_func cx_log_handler_func = cx_log_default_handler;
 static cx_print_func cx_print_message_printer = NULL;
 static cx_print_func cx_print_error_printer = NULL;
 
-static cxuint cx_log_depth = 0;
+CX_PRIVATE_DEFINE_STATIC(cx_log_depth);
 
 static cx_log_level_flags cx_log_prefix = (CX_LOG_LEVEL_ERROR    |
                                            CX_LOG_LEVEL_WARNING  |
@@ -120,7 +138,8 @@ struct _cx_debug_key_ {
 };
 
 typedef enum {
-    CX_DEBUG_FATAL_WARNINGS = 1 << 0
+    CX_DEBUG_FATAL_WARNINGS  = 1 << 0,
+    CX_DEBUG_FATAL_CRITICALS = 1 << 1
 } cx_debug_flag;
 
 static cxbool cx_debug_initialized = FALSE;
@@ -181,7 +200,8 @@ cx_debug_init(void)
     value = getenv("CX_DEBUG");
     if (value != NULL) {
         static const cx_debug_key keys[] = {
-            {"fata_warnings", CX_DEBUG_FATAL_WARNINGS}
+            {"fatal_warnings", CX_DEBUG_FATAL_WARNINGS},
+            {"fatal_criticals", CX_DEBUG_FATAL_CRITICALS}
         };
 
         cx_debug_flags = cx_debug_parse_string(value, keys,
@@ -199,73 +219,22 @@ cx_debug_init(void)
 
     }
 
+    if (cx_debug_flags & CX_DEBUG_FATAL_CRITICALS) {
+
+        cx_log_level_flags fatal_mask;
+
+        fatal_mask = cx_log_set_always_fatal(CX_LOG_FATAL_MASK);
+        fatal_mask |= CX_LOG_LEVEL_CRITICAL;
+
+        cx_log_set_always_fatal(fatal_mask);
+
+    }
+
     return;
 
 }
 
 
-/*
- * Write an unsigned integer value as formatted string to a file
- * descriptor. The format is determined by the given radix, which
- * has to be in the range 2 <= radix <= 36.
- */
-
-inline static void
-cx_msg_write_unsigned(cx_file_descriptor fd, cxulong value, cxuint radix)
-{
-
-    cxchar c, buffer[64];
-    cxint i, n;
-    cxulong tmp;
-
-
-    cx_assert(radix >= 2 && radix <= 36);
-
-    if (!value) {
-        write(fd, "0", 1);
-        return;
-    }
-
-
-    /* Hexadecimal or octal prefix for radix 16 or 8. */
-
-    if (radix == 16)
-        write(fd, "0x", 2);
-    else if (radix == 8)
-        write(fd, "0", 1);
-
-
-    /* Number of digits needed. */
-
-    n = 0;
-    tmp = value;
-
-    while (tmp) {
-        tmp /= radix;
-        n++;
-    }
-
-
-    /* Fill buffer with character representation */
-
-    i = n;
-    while (value) {
-        i--;
-        c = (value % radix);
-
-        if (c < 10)
-            buffer[i] = c + '0';
-        else
-            buffer[i] = c + 'a' - 10;
-
-        value /= radix;
-    }
-
-    write(fd, buffer, n);
-    return;
-}
-
-
 /*
  * Write a string to a file descriptor.
  */
@@ -278,6 +247,149 @@ cx_msg_write_string(cx_file_descriptor fd, const cxchar *string)
 }
 
 
+/*
+ * Write an unsigned integer value as formatted string to a file
+ * descriptor. The format is determined by the given radix, which
+ * has to be in the range 2 <= radix <= 36.
+ */
+
+inline static void
+cx_msg_format_unsigned(cxchar *buffer, cxulong value, cxuint radix)
+{
+
+    cxchar c;
+    cxint i, n;
+    cxulong tmp;
+
+
+    /* Do not use any cext library functions here! */
+
+    if ((radix != 8) && (radix != 10)  && (radix !=16)) {
+        *buffer = '\0';
+        return;
+    }
+
+    if (!value) {
+        *buffer++ = '0';
+        *buffer   = '\0';
+        return;
+    }
+
+
+    /* Hexadecimal or octal prefix for radix 16 or 8. */
+
+    if (radix == 16) {
+        *buffer++ = '0';
+        *buffer++ = 'x';
+    }
+    else if (radix == 8)
+        *buffer++ = '0';
+
+
+    /* Number of digits needed. */
+
+    n = 0;
+    tmp = value;
+
+    while (tmp) {
+        tmp /= radix;
+        ++n;
+    }
+
+
+    /* Fill buffer with character representation */
+
+    i = n;
+
+    if (n > CX_MSG_FORMAT_UNSIGNED_BUFFER_SIZE - 3) {
+        *buffer = '\0';
+        return;
+    }
+
+    while (value) {
+        --i;
+        c = (value % radix);
+
+        if (c < 10)
+            buffer[i] = c + '0';
+        else
+            buffer[i] = c + 'a' - 10;
+
+        value /= radix;
+    }
+
+    buffer[n] = '\0';
+
+    return;
+
+}
+
+
+/*
+ * Fill a buffer with the formatted log message prefix. The return value is
+ * the target file descriptor of the message.
+ */
+
+inline static cx_file_descriptor
+cx_msg_format_prefix(cxchar prefix[], cx_log_level_flags level)
+{
+
+    cxbool is_normal = TRUE;
+
+
+    /* Do not use any cext library functions here! */
+
+    switch (level & CX_LOG_LEVEL_MASK) {
+        case CX_LOG_LEVEL_ERROR:
+            strcpy(prefix, "ERROR");
+            is_normal = FALSE;
+            break;
+
+        case CX_LOG_LEVEL_CRITICAL:
+            strcpy(prefix, "CRITICAL");
+            is_normal = FALSE;
+            break;
+
+        case CX_LOG_LEVEL_WARNING:
+            strcpy(prefix, "WARNING");
+            is_normal = FALSE;
+            break;
+
+        case CX_LOG_LEVEL_MESSAGE:
+            strcpy(prefix, "Message");
+            is_normal = FALSE;
+            break;
+
+        case CX_LOG_LEVEL_INFO:
+            strcpy(prefix, "INFO");
+            break;
+
+        case CX_LOG_LEVEL_DEBUG:
+            strcpy(prefix, "DEBUG");
+            break;
+
+        default:
+            if (level) {
+                strcpy(prefix, "LOG-");
+                cx_msg_format_unsigned(prefix + 4, level & CX_LOG_LEVEL_MASK,
+                                       16);
+            }
+            else
+                strcpy(prefix, "LOG");
+            break;
+    }
+
+    if (level & CX_LOG_FLAG_RECURSION)
+        strcat(prefix, " (recursed)");
+
+    if (level & CX_MSG_ALERT_LEVELS)
+        strcat(prefix, " **");
+
+    return is_normal ? 1 : 2;
+
+}
+
+
 /*
  * Initialize the log message prefix
  */
@@ -287,11 +399,12 @@ cx_log_prefix_init(void)
 {
 
     static cxbool initialized = FALSE;
-    const cxchar *value;
 
 
     if (!initialized) {
 
+        const cxchar *value;
+
         initialized = TRUE;
         value = getenv("CX_MESSAGES_PREFIXED");
 
@@ -316,31 +429,55 @@ cx_log_prefix_init(void)
 
 
 /*
- * Write the log message prefix to a file descriptor
+ * Last resort print handler
  */
 
-inline static void
-cx_log_write_prefix(cx_file_descriptor fd, cx_log_level_flags mask)
+static void
+cx_log_fallback_handler(const cxchar *name, cx_log_level_flags level,
+                        const cxchar *message, cxptr data)
 {
 
-    cx_log_prefix_init();
+    cxchar prefix[CX_MSG_PREFIX_BUFFER_SIZE];
+    cxchar strpid[CX_MSG_FORMAT_UNSIGNED_BUFFER_SIZE];
 
-    if ((cx_log_prefix & mask) == mask) {
+    cxbool is_fatal = (level & CX_LOG_FLAG_FATAL) != 0;
 
-        const cxchar *program_name = cx_program_get_name();
+    cx_file_descriptor fd = cx_msg_format_prefix(prefix, level);
 
-        if (!program_name)
-            cx_msg_write_string(fd, "(process:");
-        else {
-            cx_msg_write_string(fd, program_name);
-            cx_msg_write_string(fd, " (pid:");
-        }
 
-        cx_msg_write_unsigned(fd, getpid(), 10);
-        cx_msg_write_string(fd, "): ");
+    /* Keep complier quiet */
+    data = NULL;
+
+    if (!message)
+        message = "(NULL) message";
+
+    cx_msg_format_unsigned(strpid, getpid(), 10);
+
+    if (name)
+        cx_msg_write_string(fd, "\n");
+    else
+        cx_msg_write_string(fd, "\n**");
+
+    cx_msg_write_string(fd, "(process:");
+    cx_msg_write_string(fd, strpid);
+    cx_msg_write_string(fd, "): ");
+
+    if (name) {
+        cx_msg_write_string(fd, name);
+        cx_msg_write_string(fd, "-");
     }
 
+    cx_msg_write_string(fd, prefix);
+    cx_msg_write_string(fd, ": ");
+    cx_msg_write_string(fd, message);
+
+    if (is_fatal)
+        cx_msg_write_string(fd, "\naborting...\n");
+    else
+        cx_msg_write_string(fd, "\n");
+
     return;
+
 }
 
 
@@ -455,6 +592,30 @@ cx_log_domain_get_handler(cx_log_domain *domain, cx_log_level_flags level,
 
 }
 
+/*
+ * Message system initialization function for threaded environments.
+ * For library internal use only!
+ *
+ * The current thread support does not need a initialization of the
+ * thread environment. If this should ever be needed this function should
+ * be called from the thread initialization function. This means that
+ * This function has to be become a global function. To restrict the
+ * visibility of the function to the library itself add the attribute
+ * macro CX_GNUC_INTERNAL to the function declaration.
+ */
+
+static void
+_cx_log_thread_init(void)
+{
+
+    cx_private_init(cx_log_depth, NULL);
+    cx_log_prefix_init();
+    cx_debug_init();
+
+    return;
+
+}
+
 
 /**
  * @brief
@@ -489,8 +650,10 @@ cx_log_set_always_fatal(cx_log_level_flags mask)
     mask |= CX_LOG_LEVEL_ERROR;
     mask &= ~CX_LOG_FLAG_FATAL;
 
+    CX_LOCK(cx_messages_lock);
     previous = cx_log_always_fatal;
     cx_log_always_fatal = mask;
+    CX_UNLOCK(cx_messages_lock);
 
     return previous;
 
@@ -596,6 +759,8 @@ cx_log_set_fatal_mask(const cxchar *name, cx_log_level_flags fatal_mask)
     fatal_mask |= CX_LOG_LEVEL_ERROR;
     fatal_mask &= ~CX_LOG_FLAG_FATAL;
 
+    CX_LOCK(cx_messages_lock);
+
     domain = cx_log_domain_find(name);
     if (!domain)
         domain = cx_log_domain_new(name);
@@ -604,6 +769,8 @@ cx_log_set_fatal_mask(const cxchar *name, cx_log_level_flags fatal_mask)
     domain->fatal_mask = fatal_mask;
     cx_log_domain_cleanup(domain);
 
+    CX_UNLOCK(cx_messages_lock);
+
     return previous;
 
 }
@@ -629,11 +796,16 @@ cx_log_func
 cx_log_set_default_handler(cx_log_func func)
 {
 
-    cx_log_func previous = cx_log_handler_func;
+    cx_log_func previous;
 
 
+    CX_LOCK(cx_messages_lock);
+
+    previous = cx_log_handler_func;
     cx_log_handler_func = func;
 
+    CX_UNLOCK(cx_messages_lock);
+
     return previous;
 
 }
@@ -675,6 +847,8 @@ cxuint cx_log_set_handler(const cxchar *name, cx_log_level_flags levels,
     if (!name)
         name = "";
 
+    CX_LOCK(cx_messages_lock);
+
     domain = cx_log_domain_find(name);
     if (!domain)
         domain = cx_log_domain_new(name);
@@ -687,6 +861,8 @@ cxuint cx_log_set_handler(const cxchar *name, cx_log_level_flags levels,
     handler->next = domain->handlers;
     domain->handlers = handler;
 
+    CX_UNLOCK(cx_messages_lock);
+
     return handler_id;
 
 }
@@ -717,6 +893,8 @@ cx_log_remove_handler(const cxchar *name, cxuint id)
     if (!name)
         name = "";
 
+    CX_LOCK(cx_messages_lock);
+
     domain = cx_log_domain_find(name);
     if (domain) {
 
@@ -732,8 +910,11 @@ cx_log_remove_handler(const cxchar *name, cxuint id)
                 else
                     domain->handlers = current->next;
 
-                cx_free(current);
                 cx_log_domain_cleanup(domain);
+                CX_UNLOCK(cx_messages_lock);
+
+                cx_free(current);
+
                 return;
             }
 
@@ -742,10 +923,12 @@ cx_log_remove_handler(const cxchar *name, cxuint id)
         }
     }
 
-   cx_warning("cx_log_remove_handler(): could not find handler with "
-              "id `%d' for domain \"%s\"", id, name);
+    CX_UNLOCK(cx_messages_lock);
 
-   return;
+    cx_warning("cx_log_remove_handler(): could not find handler with "
+               "id `%d' for domain \"%s\"", id, name);
+
+    return;
 
 }
 
@@ -773,7 +956,9 @@ cx_logv(const cxchar *name, cx_log_level_flags level, const cxchar *format,
         va_list args)
 {
 
-    cxchar buffer[1025];
+    cxbool is_fatal = (level & CX_LOG_FLAG_FATAL) != 0;
+    cxbool in_recursion = (level & CX_LOG_FLAG_RECURSION) != 0;
+
     register cxint i;
 
 
@@ -781,15 +966,13 @@ cx_logv(const cxchar *name, cx_log_level_flags level, const cxchar *format,
     if (!level)
         return;
 
+
     /*
-     * Use fixed size stack buffer, because we might get called
-     * recursively.
+     * Logging system initialization. Takes place only once.
      */
 
-    cx_vsnprintf(buffer, 1024, format, args);
+    cx_thread_once(cx_messages_once, _cx_log_thread_init, NULL);
 
-    if (!cx_debug_initialized)
-        cx_debug_init();
 
     for (i = cx_bits_find(level, -1); i >= 0; i = cx_bits_find(level, i)) {
 
@@ -800,33 +983,121 @@ cx_logv(const cxchar *name, cx_log_level_flags level, const cxchar *format,
 
         if (level & test_level) {
 
-            cxuint depth = cx_log_depth;
+            cxuint depth = CX_POINTER_TO_UINT(cx_private_get(cx_log_depth));
+
+            cx_log_level_flags fatal_mask;
             cx_log_domain *domain;
             cx_log_func log_func;
+
             cxptr data = NULL;
 
+
+            if (is_fatal)
+                test_level |= CX_LOG_FLAG_FATAL;
+
+            if (in_recursion)
+                test_level |= CX_LOG_FLAG_RECURSION;
+
+
+            /*
+             * Lookup handler
+             */
+
+            CX_LOCK(cx_messages_lock);
+
             domain = cx_log_domain_find(name ? name : "");
 
             if (depth)
                 test_level |= CX_LOG_FLAG_RECURSION;
 
-            depth++;
-            cx_log_depth = depth;
+            ++depth;
 
-            if ((((domain ? domain->fatal_mask : CX_LOG_FATAL_MASK) |
-                  cx_log_always_fatal) & test_level) != 0)
+            fatal_mask = domain ? domain->fatal_mask : CX_LOG_FATAL_MASK;
+
+            if (((fatal_mask | cx_log_always_fatal) & test_level) != 0)
                 test_level |= CX_LOG_FLAG_FATAL;
 
-            log_func = cx_log_domain_get_handler(domain, test_level, &data);
-            log_func(name, test_level, buffer, data);
+            if (test_level & CX_LOG_FLAG_RECURSION)
+                log_func = cx_log_fallback_handler;
+            else
+                log_func = cx_log_domain_get_handler(domain, test_level, &data);
 
-            /* *domain can be cluttered now */
+            domain = NULL;
+
+            CX_UNLOCK(cx_messages_lock);
+
+            cx_private_set(cx_log_depth, CX_UINT_TO_POINTER(depth));
+
+
+            /*
+             * Debug initialization
+             */
+
+            if (!(test_level & CX_LOG_FLAG_RECURSION) &&
+                    !cx_debug_initialized) {
+
+                cx_log_level_flags test_level_saved = test_level;
+
+
+                cx_debug_init();
+
+                if (((fatal_mask | cx_log_always_fatal) & test_level) != 0)
+                    test_level |= CX_LOG_FLAG_FATAL;
+
+                if (test_level != test_level_saved) {
+
+                    CX_LOCK(cx_messages_lock);
+
+                    domain = cx_log_domain_find(name ? name : "");
+                    log_func = cx_log_domain_get_handler(domain, test_level,
+                                                         &data);
+                    domain = NULL;
+
+                    CX_UNLOCK(cx_messages_lock);
+
+                }
+
+            }
+
+            if (test_level & CX_LOG_FLAG_RECURSION) {
+
+                /*
+                 * Use fixed size stack buffer, because we might be out of
+                 * memory.
+                 */
+
+                cxchar buffer[1025];
+                va_list args2;
+
+
+                CX_VA_COPY(args2, args);
+                cx_vsnprintf(buffer, 1024, format, args);
+                va_end(args2);
+
+                log_func(name, test_level, buffer, data);
+
+            }
+            else {
+
+                cxchar *buffer;
+                va_list args2;
+
+                CX_VA_COPY(args2, args);
+                buffer = cx_strvdupf(format, args2);
+                va_end(args2);
+
+                log_func(name, test_level, buffer, data);
+
+                cx_free(buffer);
+
+            }
 
             if (test_level & CX_LOG_FLAG_FATAL)
                 abort();
 
-            depth--;
-            cx_log_depth = depth;
+            --depth;
+            cx_private_set(cx_log_depth, CX_UINT_TO_POINTER(depth));
+
         }
     }
 
@@ -892,219 +1163,62 @@ cx_log_default_handler(const cxchar *name, cx_log_level_flags level,
                        const cxchar *message, cxptr data)
 {
 
+    cxchar prefix[CX_MSG_PREFIX_BUFFER_SIZE];
+    cxbool is_fatal = (level & CX_LOG_FLAG_FATAL) != 0;
+
+    cx_string* msg = NULL;
+
     cx_file_descriptor fd;
-    cxbool in_recursion;
-    cxbool is_fatal;
 
 
-    in_recursion = (level & CX_LOG_FLAG_RECURSION) != 0;
-    is_fatal = (level & CX_LOG_FLAG_FATAL) != 0;
-    level &= CX_LOG_LEVEL_MASK;
+    if (level & CX_LOG_FLAG_RECURSION) {
+        cx_log_fallback_handler(name, level, message, data);
+        return;
+    }
 
-    /* Keeps compiler quiet */
-    data = 0;
+    cx_log_prefix_init();
+    fd = cx_msg_format_prefix(prefix, level);
+
+    msg = cx_string_new();
+
+    if ((cx_log_prefix & level) == level) {
+
+        const cxchar *program = cx_program_get_name();
+
+        if (program)
+            cx_string_sprintf(msg, "(%s:%lu): ", program, (cxulong)getpid());
+        else
+            cx_string_sprintf(msg, "(process:%lu): ", (cxulong)getpid());
+
+    }
+
+    if (!name)
+        cx_string_prepend(msg, "** ");
+
+    if (level & CX_MSG_ALERT_LEVELS)
+        cx_string_prepend(msg, "\n");
+
+    if (name) {
+        cx_string_append(msg, name);
+        cx_string_append(msg, "-");
+    }
+
+    cx_string_append(msg, prefix);
+    cx_string_append(msg, ": ");
 
     if (!message)
-        message = "cx_log_default_handler(): (NULL) message";
+        cx_string_append(msg, "(NULL) message");
+    else
+        cx_string_append(msg, message);
 
-    fd = (level > CX_LOG_LEVEL_MESSAGE) ? 1 : 2;
+    if (is_fatal)
+        cx_string_append(msg, "\naborting...\n");
+    else
+        cx_string_append(msg, "\n");
 
-    switch (level) {
-        case CX_LOG_LEVEL_ERROR:
+    cx_msg_write_string(fd, cx_string_get(msg));
 
-            /*
-             * Use write(2) for output, in case we are out of memory
-             */
-
-            write(fd, "\n", 1);
-            cx_log_write_prefix(fd, level);
-
-            if (name) {
-                write(fd, name, strlen(name));
-                write(fd, "-", 1);
-            }
-            else
-                write(fd, "** ", 3);
-
-            if (in_recursion)
-                write(fd, "ERROR (recursed) **: ", 21);
-            else
-                write(fd, "ERROR **: ", 10);
-
-            write(fd, message, strlen(message));
-
-            if (is_fatal)
-                write(fd, "\naborting...\n", 13);
-            else
-                write(fd, "\n", 1);
-
-            break;
-
-        case CX_LOG_LEVEL_CRITICAL:
-            write(fd, "\n", 1);
-            cx_log_write_prefix(fd, level);
-
-            if (name) {
-                write(fd, name, strlen(name));
-                write(fd, "-", 1);
-            }
-            else
-                write(fd, "** ", 3);
-
-            if (in_recursion)
-                write(fd, "CRITICAL (recursed) **: ", 24);
-            else
-                write(fd, "CRITICAL **: ", 13);
-
-            write(fd, message, strlen(message));
-
-            if (is_fatal)
-                write(fd, "\naborting...\n", 13);
-            else
-                write(fd, "\n", 1);
-
-            break;
-
-        case CX_LOG_LEVEL_WARNING:
-            write(fd, "\n", 1);
-            cx_log_write_prefix(fd, level);
-
-            if (name) {
-                write(fd, name, strlen(name));
-                write(fd, "-", 1);
-            }
-            else
-                write(fd, "** ", 3);
-
-            if (in_recursion)
-                write(fd, "WARNING (recursed) **: ", 23);
-            else
-                write(fd, "WARNING **: ", 12);
-
-            write(fd, message, strlen(message));
-
-            if (is_fatal)
-                write(fd, "\naborting...\n", 13);
-            else
-                write(fd, "\n", 1);
-
-            break;
-
-        case CX_LOG_LEVEL_MESSAGE:
-            cx_log_write_prefix(fd, level);
-
-            if (name) {
-                write(fd, name, strlen(name));
-                write(fd, "-", 1);
-            }
-
-            if (in_recursion)
-                write(fd, "Message (recursed): ", 20);
-            else
-                write(fd, "Message: ", 9);
-
-            write(fd, message, strlen(message));
-
-            if (is_fatal)
-                write(fd, "\naborting...\n", 13);
-            else
-                write(fd, "\n", 1);
-
-            break;
-
-        case CX_LOG_LEVEL_INFO:
-            cx_log_write_prefix(fd, level);
-
-            if (name) {
-                write(fd, name, strlen(name));
-                write(fd, "-", 1);
-            }
-
-            if (in_recursion)
-                write(fd, "INFO (recursed): ", 17);
-            else
-                write(fd, "INFO: ", 6);
-
-            write(fd, message, strlen(message));
-
-            if (is_fatal)
-                write(fd, "\naborting...\n", 13);
-            else
-                write(fd, "\n", 1);
-
-            break;
-
-        case CX_LOG_LEVEL_DEBUG:
-            cx_log_write_prefix(fd, level);
-
-            if (name) {
-                write(fd, name, strlen(name));
-                write(fd, "-", 1);
-            }
-
-            if (in_recursion)
-                write(fd, "DEBUG (recursed): ", 18);
-            else
-                write(fd, "DEBUG: ", 7);
-
-            write(fd, message, strlen(message));
-
-            if (is_fatal)
-                write(fd, "\naborting...\n", 13);
-            else
-                write(fd, "\n", 1);
-
-            break;
-
-        default:
-
-            /*
-             * This catches any user defined log level. Try to make the
-             * best out of it.
-             */
-
-            cx_log_write_prefix(fd, level);
-
-            if (name) {
-                write(fd, name, strlen(name));
-
-                if (in_recursion)
-                    write(fd, "-LOG (recursed:", 15);
-                else
-                    write(fd, "-LOG (", 6);
-            }
-            else if (in_recursion)
-                write(fd, "LOG (recursed:", 14);
-            else
-                write(fd, "LOG (", 5);
-
-            if (level) {
-
-                const cxchar *string = "0x00): ";
-                cxchar *p = (cxchar*)string + 2;
-                cxuint i = cx_bits_find(level, -1);
-
-                *p++ = i >> 4;
-                *p = '0' + (i & 0xf);
-
-                if (*p > 9)
-                    *p += 'A' - '9' - 1;
-
-                write(fd, string, 7);
-
-            }
-            else
-                write(fd, "): ", 3);
-
-            write(fd, message, strlen(message));
-
-            if (is_fatal)
-                write(fd, "\naborting...\n", 13);
-            else
-                write(fd, "\n", 1);
-
-            break;
-    }
+    cx_string_delete(msg);
 
     return;
 
@@ -1132,9 +1246,14 @@ cx_print_set_handler(cx_print_func func)
 
     cx_print_func previous;
 
+
+    CX_LOCK(cx_messages_lock);
+
     previous = cx_print_message_printer;
     cx_print_message_printer = func;
 
+    CX_UNLOCK(cx_messages_lock);
+
     return previous;
 
 }
@@ -1179,7 +1298,9 @@ cx_print(const cxchar *format, ...)
     string = cx_strvdupf(format, args);
     va_end(args);
 
+    CX_LOCK(cx_messages_lock);
     printer = cx_print_message_printer;
+    CX_UNLOCK(cx_messages_lock);
 
     if (printer)
         printer(string);
@@ -1216,9 +1337,14 @@ cx_printerr_set_handler(cx_print_func func)
 
     cx_print_func previous;
 
+
+    CX_LOCK(cx_messages_lock);
+
     previous = cx_print_error_printer;
     cx_print_error_printer = func;
 
+    CX_UNLOCK(cx_messages_lock);
+
     return previous;
 
 }
@@ -1263,7 +1389,9 @@ cx_printerr(const cxchar *format, ...)
     string = cx_strvdupf(format, args);
     va_end(args);
 
+    CX_LOCK(cx_messages_lock);
     printer = cx_print_error_printer;
+    CX_UNLOCK(cx_messages_lock);
 
     if (printer)
         printer(string);
