@@ -1,7 +1,7 @@
-/* $Id: cxslist.c,v 1.8 2006/06/12 13:38:27 rpalsa Exp $
+/* $Id: cxslist.c,v 1.10 2012/03/06 12:33:15 rpalsa Exp $
  *
  * This file is part of the ESO C Extension Library
- * Copyright (C) 2001-2006 European Southern Observatory
+ * Copyright (C) 2001-2011 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,9 +20,9 @@
 
 /*
  * $Author: rpalsa $
- * $Date: 2006/06/12 13:38:27 $
- * $Revision: 1.8 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Date: 2012/03/06 12:33:15 $
+ * $Revision: 1.10 $
+ * $Name: cpl-6_0 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -404,7 +404,7 @@ _cx_slist_sorted(cx_slist *list, cx_compare_func compare)
 {
 
     cx_slnode *node = list->head.next;
-    cx_slnode *next;
+    cx_slnode *next = NULL;
 
     if (node != NULL)
         next = node->next;
