@@ -1,7 +1,7 @@
-/* $Id: cxstring.c,v 1.9 2009/12/07 15:48:15 llundin Exp $
+/* $Id: cxstring.c,v 1.10 2011/02/21 14:15:31 rpalsa Exp $
  *
  * This file is part of the ESO C Extension Library
- * Copyright (C) 2001-2006 European Southern Observatory
+ * Copyright (C) 2001-2011 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,10 +19,10 @@
  */
 
 /*
- * $Author: llundin $
- * $Date: 2009/12/07 15:48:15 $
- * $Revision: 1.9 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Author: rpalsa $
+ * $Date: 2011/02/21 14:15:31 $
+ * $Revision: 1.10 $
+ * $Name: cpl-6_0 $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -36,6 +36,7 @@
 #include "cxstring.h"
 
 
+
 /**
  * @defgroup cxstring Strings
  *
@@ -46,18 +47,16 @@
  * Together with the character data it also stores the length of the string.
  */
 
+/**@{*/
 
-/* Non-exported functions declared with gcc-attributes */
+/*
+ * Forward declaration. Needed because of attributes specification.
+ */
 
 inline static cxint
-_cx_string_vsprintf(cx_string *, const cxchar *, va_list)
-#ifdef __GNUC__
-    __attribute__((format (printf, 2, 0)))
-#endif
-    ;
+_cx_string_vsprintf(cx_string *, const cxchar *, va_list) CX_GNUC_PRINTF(2, 0);
 
 
-/**@{*/
 
 /*
  * Create a new, empty string
