@@ -1,7 +1,7 @@
-/* $Id: cxstring.h,v 1.7 2009/12/07 15:48:15 llundin Exp $
+/* $Id: cxstring.h,v 1.8 2011/02/21 14:15:31 rpalsa Exp $
  *
  * This file is part of the ESO C Extension Library
- * Copyright (C) 2001-2006 European Southern Observatory
+ * Copyright (C) 2001-2011 European Southern Observatory
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -19,10 +19,10 @@
  */
 
 /*
- * $Author: llundin $
- * $Date: 2009/12/07 15:48:15 $
- * $Revision: 1.7 $
- * $Name: cpl-5_3_0-BRANCH $
+ * $Author: rpalsa $
+ * $Date: 2011/02/21 14:15:31 $
+ * $Revision: 1.8 $
+ * $Name: cpl-6_0 $
  */
 
 #ifndef CX_STRING_H_
@@ -121,16 +121,10 @@ cxint cx_string_ncasecmp(const cx_string *, const cx_string *, cxsize);
  * I/O functions
  */
 
-cxint cx_string_sprintf(cx_string *, const cxchar *, ...)
-#ifdef __GNUC__
-    __attribute__((format (printf, 2, 3)))
-#endif
-    ;
-cxint cx_string_vsprintf(cx_string *, const cxchar *, va_list)
-#ifdef __GNUC__
-    __attribute__((format (printf, 2, 0)))
-#endif
-    ;
+cxint cx_string_sprintf(cx_string *,
+                        const cxchar *, ...) CX_GNUC_PRINTF(2, 3);
+cxint cx_string_vsprintf(cx_string *,
+                         const cxchar *, va_list) CX_GNUC_PRINTF(2, 0);
 
 /*
  * Debugging utilities
